Package: BayesRTMB
Type: Package
Title: Bayesian Inference Using 'RTMB'
Version: 0.1.1
Authors@R: person("Hiroshi", "Shimizu", email = "simizu706@gmail.com", role = c("aut", "cre"))
Description: Provides tools for Markov chain Monte Carlo (MCMC) and Maximum A Posteriori (MAP) estimation utilizing the 'RTMB' package. It supports various statistical models including generalized linear mixed models, factor analysis, item response theory, and multidimensional unfolding. The package allows users to easily transition between frequentist and Bayesian paradigms using a unified interface. Automatic differentiation and Laplace approximation follow Kristensen et al. (2016) <doi:10.18637/jss.v070.i05>, and MCMC sampling uses the No-U-Turn Sampler described by Hoffman and Gelman (2014) <https://jmlr.org/papers/v15/hoffman14a.html>.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
LazyData: true
Depends: R (>= 4.1.0), RTMB
Imports: R6, MASS
URL: https://github.com/norimune/BayesRTMB,
        https://norimune.github.io/BayesRTMB/
BugReports: https://github.com/norimune/BayesRTMB/issues
Suggests: knitr, rmarkdown, GPArotation, future, future.apply,
        progressr
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-28 07:39:58 UTC; norimune
Author: Hiroshi Shimizu [aut, cre]
Maintainer: Hiroshi Shimizu <simizu706@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-01 08:40:26 UTC

# Generated by roxygen2: do not edit by hand

S3method(AIC,Classic_Fit)
S3method(BIC,Classic_Fit)
S3method(anova,Classic_Fit)
S3method(conditional_effects,advi_fit)
S3method(conditional_effects,default)
S3method(conditional_effects,map_fit)
S3method(conditional_effects,mcmc_fit)
S3method(conditional_effects,vb_fit)
S3method(item_curve,RTMB_Fit_Base)
S3method(item_curve,advi_fit)
S3method(item_curve,map_fit)
S3method(item_curve,mcmc_fit)
S3method(item_info,RTMB_Fit_Base)
S3method(item_info,advi_fit)
S3method(item_info,map_fit)
S3method(item_info,mcmc_fit)
S3method(logLik,Classic_Fit)
S3method(lsmeans,Classic_Fit)
S3method(plot,ce_rtmb)
S3method(plot,rtmb_item_curve)
S3method(plot,rtmb_item_info)
S3method(plot,rtmb_lsmeans)
S3method(plot,rtmb_test_info)
S3method(print,Classic_Fit)
S3method(print,anova_classic_lm_f)
S3method(print,anova_classic_lrt)
S3method(print,anova_rtmb)
S3method(print,anova_rtmb_table)
S3method(print,bayes_factor)
S3method(print,bayes_factor_rtmb)
S3method(print,ce_rtmb)
S3method(print,ce_simple)
S3method(print,diagnose_BayesRTMB)
S3method(print,map_fit)
S3method(print,rtmb_estimate_matrix)
S3method(print,rtmb_item_curve)
S3method(print,rtmb_item_info)
S3method(print,rtmb_lrt_two_stage)
S3method(print,rtmb_lsmeans)
S3method(print,rtmb_lsmeans_grouped)
S3method(print,rtmb_test_info)
S3method(print,summary_BayesRTMB)
S3method(print,summary_Classic_Fit)
S3method(print,waic_BayesRTMB)
S3method(simple_effects,advi_fit)
S3method(simple_effects,map_fit)
S3method(simple_effects,mcmc_fit)
S3method(simple_effects,vb_fit)
S3method(summary,Classic_Fit)
S3method(summary,ce_rtmb)
S3method(summary,map_fit)
S3method(test_info,RTMB_Fit_Base)
S3method(test_info,advi_fit)
S3method(test_info,map_fit)
S3method(test_info,mcmc_fit)
export(Classic_Fit)
export(Dim)
export(MAP_Fit)
export(MCMC_Fit)
export(RTMB_Fit_Base)
export(RTMB_Model)
export(VB_Fit)
export(bayes_factor)
export(conditional_effects)
export(distance)
export(ess_basic)
export(ess_bulk)
export(ess_tail95)
export(fabs)
export(gaussian_process_lpdf)
export(inv_logit)
export(item_curve)
export(item_info)
export(log1m)
export(log1m_exp)
export(log1p_exp)
export(log_det_chol)
export(log_mix)
export(log_softmax)
export(log_sum_exp)
export(log_sum_exp_matrix)
export(logit)
export(lsmeans)
export(make_bw_from_ydif)
export(make_glmer_Z_matrix)
export(make_glmer_re_terms)
export(make_init_mdu)
export(make_ydif_from_bw)
export(map_est)
export(plot_acf)
export(plot_conditional_effects)
export(plot_dens)
export(plot_forest)
export(plot_item_curve)
export(plot_item_info)
export(plot_lsmeans)
export(plot_mdu)
export(plot_pairs)
export(plot_test_info)
export(plot_trace)
export(prior_flat)
export(prior_jzs)
export(prior_normal)
export(prior_rhs)
export(prior_ssp)
export(prior_uniform)
export(prior_weak)
export(quad_form_chol)
export(quad_form_diag)
export(quantile95)
export(r_hat)
export(read_mcmc_csv)
export(restore_bw_from_ydif)
export(rtmb_code)
export(rtmb_corr)
export(rtmb_fa)
export(rtmb_glm)
export(rtmb_glmer)
export(rtmb_irt)
export(rtmb_lm)
export(rtmb_lmer)
export(rtmb_loglinear)
export(rtmb_lrt)
export(rtmb_mdu)
export(rtmb_mediation)
export(rtmb_mixture)
export(rtmb_model)
export(rtmb_table)
export(rtmb_ttest)
export(safe_rtmb_model)
export(simple_effects)
export(softmax)
export(sort_loadings)
export(squared_distance)
export(stz_basis)
export(sum_to_zero)
export(test_info)
export(to_centered_matrix)
export(to_centered_tri)
export(to_long)
export(to_wide)
import(MASS)
import(RTMB)
importFrom(R6,R6Class)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,.getXlevels)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,Gamma)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,binomial)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,delete.response)
importFrom(stats,density)
importFrom(stats,dunif)
importFrom(stats,fisher.test)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,poisson)
importFrom(stats,prcomp)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rchisq)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,modifyList)
importFrom(utils,read.csv)
importFrom(utils,write.csv)

# BayesRTMB

**Language / 言語**: [English
introduction](https://norimune.github.io/BayesRTMB/articles/introduction.html)
\| [パッケージ紹介](https://norimune.github.io/BayesRTMB/articles/ja-introduction.html)

**BayesRTMB** is an R package for writing and fitting statistical models
with RTMB as the automatic differentiation engine.

You can start from wrapper functions such as `rtmb_lm()`,
`rtmb_glmer()`, `rtmb_corr()`, and `rtmb_ttest()`, or write your own
model with `rtmb_code()`. The same model object can then be used for
MCMC, MAP estimation, variational inference, and frequency-oriented
classical analyses where supported.

## Key Features

- **Model code in R**: Write models with `setup`, `parameters`,
  `transform`, `model`, and `generate` blocks.
- **Multiple estimation methods**: Use `$sample()`, `$optimize()`,
  `$variational()`, and `$classic()` from a common model object.
- **Wrapper functions**: Fit regression models, GLM/GLMMs, t tests,
  correlations, contingency tables, factor analysis, IRT, latent rank
  models, and multidimensional unfolding models.
- **Classical analyses from wrappers**: Use `$classic()` for
  frequency-oriented outputs, including `AIC()`, `BIC()`, and `anova()`
  for supported fits.
- **Random effects and Laplace approximation**: Declare parameters with
  `random = TRUE` and use RTMB’s Laplace machinery for latent variables
  and mixed models.
- **Diagnostics and visualization**: Plot posterior draws, convergence
  diagnostics, forest plots, conditional effects, and related summaries.

## Installation

You can install BayesRTMB from CRAN.

``` r
install.packages("BayesRTMB")
```

The development version can be installed from GitHub with either `pak` or
`remotes`.

``` r
pak::pak("norimune/BayesRTMB")
```

``` r
remotes::install_github("norimune/BayesRTMB")
```

### Windows Users

For ordinary use, Windows users can install the CRAN binary package
without Rtools. Rtools is only needed for source installation,
development, or compiling custom TMB C++ templates.

``` r
pkgbuild::check_build_tools(debug = TRUE)
```

If you install BayesRTMB from source and this check fails, install the
Rtools version that matches your R version from the [Rtools
page](https://cran.r-project.org/bin/windows/Rtools/), restart R, and
try again.

## Quick Example

For standard analyses, start with a wrapper function.

``` r
library(BayesRTMB)
data(debate)

mdl <- rtmb_lm(sat ~ talk * perf, data = debate)

fit_mcmc <- mdl$sample()
fit_map  <- mdl$optimize()
fit_lm   <- mdl$classic()
```

You can also write a model directly.

``` r
Y <- debate$sat
X <- debate[c("talk","perf")] |> as.matrix()

data_list <- list(Y = Y, X = X)

code <- rtmb_code(
  setup = {
    N <- length(Y)
    K <- ncol(X)
  },
  parameters = {
    Intercept <- Dim(1)
    b <- Dim(K)
    sigma = Dim(lower = 0)
  },
  model = {
    mu <- Intercept + X %*% b
    Y ~ normal(mu, sigma)
    Intercept ~ normal(0, 10)
    b ~ normal(0, 10)
    sigma ~ exponential(1)
  }
)

mdl_custom <- rtmb_model(data_list, code)
fit_custom <- mdl_custom$sample()
```

## Articles

- [Introduction](https://norimune.github.io/BayesRTMB/articles/introduction.html):
  overall concepts, entry points, and inference workflow.
- [Quick Start](https://norimune.github.io/BayesRTMB/articles/quick_start.html):
  installation, a minimal model, MCMC diagnostics, visualization, and t
  tests.
- [Wrapper Functions](https://norimune.github.io/BayesRTMB/articles/wrapper_functions.html):
  using wrappers for Bayesian and classical analyses.
- [Hierarchical Models and
  GLMMs](https://norimune.github.io/BayesRTMB/articles/rtmb_glmer.html):
  detailed use of `rtmb_glmer()` for mixed models, GLMMs, priors,
  residual correlation, and visualization.
- [Writing Model
  Codes](https://norimune.github.io/BayesRTMB/articles/writing_models.html):
  writing custom models with `rtmb_code()`.
- [RTMB Internals and Inference
  Algorithms](https://norimune.github.io/BayesRTMB/articles/rtmb_internals.html):
  internal model representation, Laplace approximation, and inference
  pipelines.
- [日本語:
  パッケージ紹介](https://norimune.github.io/BayesRTMB/articles/ja-introduction.html)
- [日本語:
  クイックスタート](https://norimune.github.io/BayesRTMB/articles/ja-quick_start.html)
- [日本語:
  ラッパー関数の使い方](https://norimune.github.io/BayesRTMB/articles/ja-wrapper_functions.html)
- [日本語:
  モデルコードの書き方](https://norimune.github.io/BayesRTMB/articles/ja-writing_models.html)

NUTS_method <- function(model,
                        sampling, warmup, delta = 0.8,
                        max_treedepth = 10, chain,
                        update_progress = NULL,
                        laplace, save_info = NULL) {

  iter <- sampling + warmup
  P_all <- length(model$env$last.par)

  nuts_core <- create_NUTS_core(model)
  fn_NUTS <- nuts_core$fn_NUTS
  gr_NUTS <- nuts_core$gr_NUTS
  calc_H <- nuts_core$calc_H
  safe_uturn <- nuts_core$safe_uturn
  BuildTree <- nuts_core$BuildTree
  FindReasonableEpsilon <- nuts_core$FindReasonableEpsilon

  q_fixed_init <- model$par
  P_fixed <- length(q_fixed_init)


  para_fixed <- array(NA, dim=c(iter, P_fixed))
  para_full  <- array(NA, dim=c(iter, P_all))

  para_fixed[1, ] <- q_fixed_init
  lp <- numeric(iter)
  lp[1] <- fn_NUTS(q_fixed_init)
  para_full[1, ] <- model$env$last.par

  treedepth_record <- numeric(iter)
  accept <- numeric(iter)
  accept[1] <- 1

  M_inv <- rep(1, P_fixed)
  eps <- FindReasonableEpsilon(q_fixed_init, M_inv)

  # Dual Averaging parameters
  mu_DA <- log(10 * eps)
  Hbar <- 0; gamma_DA <- 0.05; t0 <- 10; kappa <- 0.75
  eps_bar <- 1; log_eps_bar <- log(eps_bar)
  da_iter <- 1

  # --- Windowed Adaptation setting ---
  init_buffer <- 75
  term_buffer <- 50
  base_window <- 25

  if (warmup < 20) {
    init_buffer <- warmup
    term_buffer <- 0
    base_window <- 0
  } else if (warmup < init_buffer + term_buffer + base_window) {
    init_buffer <- floor(0.15 * warmup)
    term_buffer <- floor(0.10 * warmup)
    base_window <- warmup - init_buffer - term_buffer
  }

  next_window <- init_buffer + base_window
  window_size <- base_window

  w_mean <- rep(0, P_fixed)
  w_var  <- rep(0, P_fixed)
  w_n    <- 0

  msg_start <- paste0("chain ", chain, " started...")
  if (!is.null(update_progress)) {
    update_progress(msg = msg_start, amt = 1)
  } else {
    cat(msg_start, "\n")
  }

  for (i in 2:iter) {
    if (!is.null(model$rtmb_pd_error_state)) {
      model$rtmb_pd_error_state$post_warmup <- i > warmup
    }
    q_old <- para_fixed[i-1, ]
    p_old <- rnorm(P_fixed, mean = 0, sd = sqrt(1 / M_inv))
    gr_old <- gr_NUTS(q_old)
    H_old <- calc_H(q_old, p_old, M_inv)

    log_u <- log(runif(1)) + H_old
    q_minus <- q_old; q_plus <- q_old
    p_minus <- p_old; p_plus <- p_old
    gr_minus <- gr_old; gr_plus <- gr_old

    j <- 0; q_new <- q_old; n <- 1; s <- 1
    alpha_sum <- 0; n_alpha_sum <- 0

    while (s == 1 && j < max_treedepth) {
      v <- ifelse(runif(1) > 0.5, -1, 1)
      if (v == -1) {
        out <- BuildTree(p_minus, q_minus, gr_minus, log_u, v, j, eps, H_old, M_inv)
        p_minus <- out$p_minus; q_minus <- out$q_minus; gr_minus <- out$gr_minus
      } else {
        out <- BuildTree(p_plus, q_plus, gr_plus, log_u, v, j, eps, H_old, M_inv)
        p_plus <- out$p_plus; q_plus <- out$q_plus; gr_plus <- out$gr_plus
      }
      if (out$s == 1) {
        if (runif(1) < out$n / n) q_new <- out$q
      }
      n <- n + out$n
      s <- out$s * safe_uturn(q_plus, q_minus, p_minus, M_inv) * safe_uturn(q_plus, q_minus, p_plus, M_inv)
      alpha_sum <- alpha_sum + out$alpha; n_alpha_sum <- n_alpha_sum + out$n_alpha
      j <- j + 1
    }

    para_fixed[i, ] <- q_new
    lp[i] <- fn_NUTS(q_new)
    para_full[i, ] <- model$env$last.par

    treedepth_record[i] <- j
    accept_stat <- alpha_sum / n_alpha_sum
    if (is.na(accept_stat)) accept_stat <- 0
    accept[i] <- accept_stat

    if (i <= warmup) {
      Hbar <- (1 - 1/(da_iter+t0))*Hbar + (1/(da_iter+t0))*(delta - accept_stat)
      log_eps <- mu_DA - sqrt(da_iter)/gamma_DA * Hbar
      log_eps_bar <- da_iter^-kappa*log_eps + (1 - da_iter^-kappa)*log_eps_bar
      eps <- exp(log_eps)
      eps_bar <- exp(log_eps_bar)
      da_iter <- da_iter + 1

      if (base_window > 0 && i >= init_buffer && i <= warmup - term_buffer) {
        w_n <- w_n + 1
        delta_q <- q_new - w_mean
        w_mean <- w_mean + delta_q / w_n
        w_var <- w_var + delta_q * (q_new - w_mean)

        if (i == next_window || i == warmup - term_buffer) {
          new_var <- w_var / (w_n - 1)
          new_var[!is.finite(new_var)] <- 1
          new_var <- pmax(new_var, 1e-8)
          M_inv <- new_var

          w_mean <- rep(0, P_fixed); w_var <- rep(0, P_fixed); w_n <- 0
          window_size <- window_size * 2
          next_window <- i + window_size
          if (next_window > warmup - term_buffer) next_window <- warmup - term_buffer

          eps <- FindReasonableEpsilon(q_new, M_inv)
          mu_DA <- log(10 * eps); Hbar <- 0; eps_bar <- 1; log_eps_bar <- log(eps_bar); da_iter <- 1
        }
      }
    } else {
      eps <- eps_bar
    }
    if (i %% 200 == 0) {
      msg <- paste0("chain ", chain, ": iter ", i, ifelse(i <= warmup, " warmup", " sampling"))
      if (!is.null(update_progress)) {
        update_progress(amt = 1, msg = msg)
      } else {
        cat(msg, "\n")
      }
    }
  }

  if (exists("para_fixed")) para_fixed <- Re(para_fixed)
  if (exists("para_full") && !is.null(para_full)) para_full <- Re(para_full)

  return(list(
    para_fixed = para_fixed, para_full = para_full,
    lp = Re(lp), accept = accept, treedepth = treedepth_record, eps = eps,
    pd_error_count = if (!is.null(model$rtmb_pd_error_state)) model$rtmb_pd_error_state$count else 0L
  ))
}

create_NUTS_core <- function(ad_obj) {
  fn_NUTS <- function(q) -ad_obj$fn(q)
  gr_NUTS <- function(q) -ad_obj$gr(q)

  calc_H <- function(q, p, M_inv) {
    fn_NUTS(q) - sum((p^2) * M_inv) / 2
  }

  safe_uturn <- function(q_plus, q_minus, p, M_inv) {
    dq <- q_plus - q_minus
    dot_prod <- sum(dq * (p * M_inv))
    if (!is.finite(dot_prod)) return(0L)
    if (dot_prod >= 0) 1L else 0L
  }

  BuildTree <- function(p, q, gr, log_u, v, j, eps, H0, M_inv) {
    if (j == 0L) {
      eps_v <- v * eps
      p_prime <- p + (eps_v / 2) * gr
      q_prime <- q + eps_v * M_inv * p_prime
      gr_prime <- gr_NUTS(q_prime)
      p_prime <- p_prime + (eps_v / 2) * gr_prime

      H_prime <- calc_H(q_prime, p_prime, M_inv)

      if (!is.finite(H_prime)) {
        n_prime <- 0L; s_prime <- 0L; alpha_prime <- 0
      } else {
        n_prime <- if (isTRUE(log_u <= H_prime)) 1L else 0L
        s_prime <- if (isTRUE(log_u - 1000 < H_prime)) 1L else 0L
        alpha_raw <- exp(H_prime - H0)
        alpha_prime <- if (is.na(alpha_raw)) 0 else min(1, alpha_raw)
      }

      return(list(
        p_minus = p_prime, q_minus = q_prime, gr_minus = gr_prime,
        p_plus  = p_prime, q_plus  = q_prime, gr_plus  = gr_prime,
        q       = q_prime,
        n       = n_prime,
        s       = s_prime,
        alpha   = alpha_prime,
        n_alpha = 1L
      ))
    }

    out1 <- BuildTree(p, q, gr, log_u, v, j - 1L, eps, H0, M_inv)
    if (isTRUE(out1$s == 0L)) return(out1)

    if (v == -1L) {
      out2 <- BuildTree(out1$p_minus, out1$q_minus, out1$gr_minus, log_u, v, j - 1L, eps, H0, M_inv)
      out1$p_minus <- out2$p_minus; out1$q_minus <- out2$q_minus; out1$gr_minus <- out2$gr_minus
    } else {
      out2 <- BuildTree(out1$p_plus, out1$q_plus, out1$gr_plus, log_u, v, j - 1L, eps, H0, M_inv)
      out1$p_plus <- out2$p_plus; out1$q_plus <- out2$q_plus; out1$gr_plus <- out2$gr_plus
    }

    total_n <- out1$n + out2$n
    if (total_n > 0L) {
      prob <- out2$n / total_n
      if (isTRUE(runif(1) < prob)) out1$q <- out2$q
    }

    dq <- out1$q_plus - out1$q_minus
    sum1 <- sum(dq * (out1$p_minus * M_inv))
    sum2 <- sum(dq * (out1$p_plus * M_inv))

    s_uturn <- isTRUE(sum1 >= 0) && isTRUE(sum2 >= 0)

    out1$n <- total_n

    out1$s <- if (isTRUE(out1$s == 1L) && isTRUE(out2$s == 1L) && s_uturn) 1L else 0L

    out1$alpha <- out1$alpha + out2$alpha
    out1$n_alpha <- out1$n_alpha + out2$n_alpha

    return(out1)
  }

  FindReasonableEpsilon <- function(q, M_inv) {
    eps_try <- 1
    p <- rnorm(length(q), mean = 0, sd = sqrt(1 / M_inv))
    gr <- gr_NUTS(q)
    H_old <- calc_H(q, p, M_inv)

    p_new <- p + (eps_try / 2) * gr
    q_new <- q + eps_try * M_inv * p_new
    p_new <- p_new + (eps_try / 2) * gr_NUTS(q_new)
    H_new <- calc_H(q_new, p_new, M_inv)

    ratio <- if (!is.finite(H_new) || !is.finite(H_old)) 0 else exp(H_new - H_old)
    a <- if (isTRUE(ratio > 0.5)) 1 else -1

    while (isTRUE(ratio^a > 2^(-a))) {
      eps_try <- (2^a) * eps_try
      p_new <- p + (eps_try / 2) * gr
      q_new <- q + eps_try * M_inv * p_new
      p_new <- p_new + (eps_try / 2) * gr_NUTS(q_new)
      H_new <- calc_H(q_new, p_new, M_inv)
      ratio <- if (!is.finite(H_new) || !is.finite(H_old)) 0 else exp(H_new - H_old)
      if (eps_try > 1e7 || eps_try < 1e-7) break
    }
    eps_try
  }

  return(list(
    fn_NUTS = fn_NUTS, gr_NUTS = gr_NUTS, calc_H = calc_H,
    safe_uturn = safe_uturn, BuildTree = BuildTree,
    FindReasonableEpsilon = FindReasonableEpsilon
  ))
}

#' RTMB-based Mediation Analysis Wrapper
#'
#' @description
#' `rtmb_mediation` performs mediation analysis by simultaneously estimating multiple
#' GLM regression equations. It automatically identifies mediation paths and calculates
#' indirect, direct, and total effects.
#'
#' @param formula A list of formulas defining the regression paths (e.g., `list(M ~ X, Y ~ X + M)`).
#' @param data A data frame containing the variables.
#' @param family A single character string or a list of character strings specifying the error distribution
#'   for each equation (e.g., `family = list("gaussian", "binomial")`). Default is "gaussian".
#' @param prior An object of class "rtmb_prior" specifying the prior distribution.
#' @param y_range Theoretical minimum and maximum values of the response variable.
#' @param fixed A named list of parameter values to fix (optional).
#' @param view Character vector of parameter names to prioritize in summary.
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments passed to the model construction.
#'
#' @details
#' The function identifies mediation paths by looking for variables that are
#' responses in one equation and predictors in another. Indirect effects are
#' calculated as the product of coefficients along these paths (\eqn{a * b}).
#'
#' \strong{Uncertainty Estimation}:
#' When using `$optimize(ci_method = "sampling")`, the function provides asymmetric
#' confidence intervals for indirect effects based on the distribution of products,
#' which is more accurate than the standard Sobel test (Delta Method).
#'
#' @return An `RTMB_Model` object.
#' @example inst/examples/ex_mediation.R
#' @export
rtmb_mediation <- function(formula, data, family = "gaussian", prior = prior_flat(),
                           y_range = NULL, fixed = NULL, view = NULL,
                           WAIC = FALSE, ...) {


  if (!is.list(formula)) stop("formula must be a list of formulas (e.g., list(M ~ X, Y ~ X + M)).")
  n_eq <- length(formula)

  # Validate: No random effects allowed in this version
  for (i in 1:n_eq) {
    if (any(grepl("\\|", as.character(formula[[i]])))) {
      stop("Multilevel mediation (random effects) is not yet supported in 'rtmb_mediation'. Please use fixed-effects formulas.")
    }
  }

  # Validate: No duplicate response variables
  resp_check <- vapply(formula, function(f) as.character(f[[2]]), character(1))
  if (anyDuplicated(resp_check)) {
    dup_vars <- resp_check[duplicated(resp_check)]
    stop(
      sprintf("Duplicate response variable(s) detected: %s. Each equation must have a unique response.",
              paste(unique(dup_vars), collapse = ", ")),
      call. = FALSE
    )
  }

  if (is.null(prior)) prior <- prior_flat()

  # Automatically switch to prior_weak() if y_range is provided and prior is default flat
  if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }

  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), or prior_weak().",
      call. = FALSE
    )
  }

  # Prepare family list
  if (!is.list(family)) {
    family_list <- rep(list(family), n_eq)
  } else {
    if (length(family) != n_eq) stop("Length of family list must match length of formula list.")
    family_list <- family
  }
  valid_families <- c("gaussian", "bernoulli", "binomial", "poisson")
  bad_families <- unlist(family_list, use.names = FALSE)
  bad_families <- bad_families[
    !vapply(bad_families, function(x) is.character(x) && length(x) == 1L &&
              !is.na(x) && x %in% valid_families, logical(1))
  ]
  if (length(bad_families) > 0L) {
    stop(
      sprintf(
        "Invalid 'family' value in mediation model: %s. Valid options are: %s",
        paste(unique(bad_families), collapse = ", "),
        paste(valid_families, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  setup_vars <- unique(unlist(lapply(formula, all.vars), use.names = FALSE))
  missing_vars <- setdiff(setup_vars, names(data))
  if (length(missing_vars) > 0) {
    stop(
      "The following variables in formula are not found in data: ",
      paste(missing_vars, collapse = ", "),
      call. = FALSE
    )
  }
  setup_df <- stats::na.omit(as.data.frame(data)[, setup_vars, drop = FALSE])
  class(setup_df) <- c("rtmb_setup_df", class(setup_df))

  N <- nrow(setup_df)
  data_list <- list(df = setup_df)
  resp_names <- character(n_eq)
  X_list <- list()
  X_colnames <- list()

  # 1. Parse Formulas and Prepare Data
  for (i in 1:n_eq) {
    f <- formula[[i]]
    mf <- model.frame(f, data = data)
    y_name <- as.character(f[[2]])
    resp_names[i] <- y_name

    X_mat <- model.matrix(f, data = data)
    data_list[[paste0("formula_", i)]] <- f
    cols <- colnames(X_mat)
    cols[cols == "(Intercept)"] <- "Intercept"
    X_colnames[[i]] <- cols
    X_list[[i]] <- X_mat

    # Add range variables for weak priors dynamically
    if (inherits(prior, "rtmb_prior") && prior$type %in% c("weak", "rhs", "ssp")) {
       f_type <- family_list[[i]]
       if (!(f_type %in% c("bernoulli", "binomial", "poisson"))) {
         range_i <- if (is.list(y_range)) y_range[[y_name]] else y_range
         if (is.null(range_i)) {
           stop(paste0("y_range is required for response variable '", y_name, "' when using weakly informative priors. ",
                       "Please provide y_range as a vector or a named list (e.g., y_range = list(", y_name, " = c(1, 5)))."))
         }
         data_list[[paste0("half_d_y_", i)]] <- diff(range_i) / 2
         data_list[[paste0("mid_y_", i)]] <- mean(range_i)
       }
    }
  }

  prior_type <- prior$type
  if (prior_type == "normal") {
    if (is.null(prior$mu_sd) && !is.null(prior$Intercept_sd)) {
      prior$mu_sd <- prior$Intercept_sd
    }
  }
  if (prior_type == "weak") {
    if (is.null(prior$max_beta)) prior$max_beta <- 1.0
    if (is.null(prior$sd_ratio)) prior$sd_ratio <- 0.5
  }

  # 2. Setup AST Block
  setup_exprs <- list()
  setup_exprs[[1]] <- quote(N <- nrow(df))

  for (i in 1:n_eq) {
    mf_name <- as.name(paste0("mf_", i))
    formula_name <- as.name(paste0("formula_", i))
    Y_name <- as.name(paste0("Y_", i))
    X_name <- as.name(paste0("X_", i))

    setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(mf_name) <- model.frame(.(formula_name), df))
    setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(Y_name) <- as.numeric(model.response(.(mf_name))))
    setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_name) <- model.matrix(.(formula_name), .(mf_name)))
  }

  for (i in 1:n_eq) {
    f_type <- family_list[[i]]
    p_name <- paste0("b", i)
    X_name <- as.name(paste0("X_", i))

    has_b_prior <- prior_type %in% c("weak", "normal") || !is.null(prior$b_sd) || !is.null(prior$Intercept_sd)
    has_sigma_prior <- f_type == "gaussian" && (prior_type %in% c("weak", "normal") || !is.null(prior$sigma_rate))

    if (has_b_prior) {
      X_sd_name <- as.name(paste0("X_sd_", i))
      X_mean_name <- as.name(paste0("X_mean_", i))
      b_prior_sd_name <- as.name(paste0(p_name, "_prior_sd"))
      b_prior_mean_name <- as.name(paste0(p_name, "_prior_mean"))
      intercept_prior_sd_name <- as.name(paste0("intercept_prior_sd_", i))

      setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_sd_name) <- apply(.(X_name), 2, sd))
      setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_sd_name)[.(X_sd_name) == 0] <- 1)
      setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(b_prior_mean_name) <- rep(0, ncol(.(X_name))))

      if (prior_type == "weak") {
        X_c_name <- as.name(paste0("X_c_", i))
        mid_y_name <- as.name(paste0("mid_y_", i))

        setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_mean_name) <- apply(.(X_name), 2, mean))
        has_intercept <- "Intercept" %in% X_colnames[[i]]
        if (has_intercept) {
           idx <- which(X_colnames[[i]] == "Intercept")
           setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_mean_name)[.(idx)] <- 0)
        }
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_c_name) <- .(X_name) - rep(1, N) %*% t(.(X_mean_name)))

        base_scale_name <- as.name(paste0("base_scale_", i))
        if (f_type %in% c("bernoulli", "binomial")) {
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(base_scale_name) <- pi / sqrt(3))
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(intercept_prior_sd_name) <- .(base_scale_name) * .(prior$max_beta))
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(b_prior_sd_name) <- (.(prior$max_beta) * .(base_scale_name)) / .(X_sd_name))
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(mid_y_name) <- 0)
        } else if (f_type == "poisson") {
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(base_scale_name) <- 1.0)
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(intercept_prior_sd_name) <- 1.0)
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(b_prior_sd_name) <- 1.0 / .(X_sd_name))
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(mid_y_name) <- 0)
        } else {
          half_d_y_name <- as.name(paste0("half_d_y_", i))
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(base_scale_name) <- .(half_d_y_name) * .(prior$sd_ratio))
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(intercept_prior_sd_name) <- .(half_d_y_name))
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(b_prior_sd_name) <- (.(prior$max_beta) * .(base_scale_name)) / .(X_sd_name))
          if (has_sigma_prior) {
            sigma_rate_name <- as.name(paste0("sigma", i, "_rate"))
            setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(sigma_rate_name) <- 1.0 / .(base_scale_name))
          }
        }
        if (has_intercept) {
           idx <- which(X_colnames[[i]] == "Intercept")
           setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(b_prior_mean_name)[.(idx)] <- .(mid_y_name))
        }
      } else if (prior_type == "normal") {
        # Manual prior with explicit SDs
        b_sd_val <- if (!is.null(prior$b_sd)) prior$b_sd else 10
        int_sd_val <- if (!is.null(prior$mu_sd)) prior$mu_sd else 10
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(b_prior_sd_name) <- rep(.(b_sd_val), ncol(.(X_name))))
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(intercept_prior_sd_name) <- .(int_sd_val))
        
        X_mean_name <- as.name(paste0("X_mean_", i))
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_mean_name) <- apply(.(X_name), 2, mean))
        has_intercept <- "Intercept" %in% X_colnames[[i]]
        if (has_intercept) {
           idx <- which(X_colnames[[i]] == "Intercept")
           setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_mean_name)[.(idx)] <- 0)
        }
        X_c_name <- as.name(paste0("X_c_", i))
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(X_c_name) <- .(X_name) - rep(1, N) %*% t(.(X_mean_name)))
      } else {
        # flat: No prior added by setup_exprs (handled by prior_exprs check later if any)
        # However, in current logic, if has_b_prior is FALSE, we don't enter here.
        # If it's flat, has_b_prior will be FALSE.
        NULL
      }
      # Overwrite intercept SD if needed
      has_intercept <- "Intercept" %in% X_colnames[[i]]
      if (has_intercept) {
         idx <- which(X_colnames[[i]] == "Intercept")
         setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(b_prior_sd_name)[.(idx)] <- .(intercept_prior_sd_name))
      }
    }

    if (has_sigma_prior && prior_type == "normal") {
       sigma_rate_val <- if (!is.null(prior$sigma_rate)) prior$sigma_rate else 1
       sigma_rate_name <- as.name(paste0("sigma", i, "_rate"))
       setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(sigma_rate_name) <- .(sigma_rate_val))
    }
  }

  setup_ast <- as.call(c(list(as.name("{")), setup_exprs))

  tmp_env <- list2env(data_list)
  eval(setup_ast, tmp_env)

  # 3. Parameters, Transform and Model Block AST
  param_exprs <- list()
  tran_exprs <- list()
  model_exprs <- list()
  generate_exprs <- list()

  effect_names <- character(0)

  v_names <- list()
  init_list <- list()
  b_vars <- c()
  s_vars <- c()

  for (i in 1:n_eq) {
    y_name <- resp_names[i]
    p_name <- paste0("b", i)
    p_c_name <- paste0("b_c", i)
    s_name <- paste0("sigma", i)
    f_type <- family_list[[i]]
    P_dim <- ncol(X_list[[i]])

    has_intercept <- "Intercept" %in% X_colnames[[i]]
    is_centered <- (prior_type %in% c("weak", "normal") && has_intercept)
    target_p_name <- if (is_centered) p_c_name else p_name

    param_exprs[[length(param_exprs) + 1]] <- bquote(.(as.name(target_p_name)) <- Dim(.(P_dim)))

    if (is_centered) {
       v_names[[p_c_name]] <- X_colnames[[i]]
       v_names[[p_name]] <- X_colnames[[i]]
       init_list[[p_c_name]] <- rep(0, P_dim)

       idx <- which(X_colnames[[i]] == "Intercept")
       X_mean_name <- as.name(paste0("X_mean_", i))

       tran_exprs[[length(tran_exprs) + 1]] <- bquote(.(as.name(p_name)) <- .(as.name(p_c_name)))
       tran_exprs[[length(tran_exprs) + 1]] <- bquote(.(as.name(p_name))[.(idx)] <- .(as.name(p_c_name))[.(idx)] - sum(.(X_mean_name) * .(as.name(p_c_name))))

       lin_pred_expr <- bquote(.(as.name(paste0("X_c_", i))) %*% .(as.name(p_c_name)))
    } else {
       v_names[[p_name]] <- X_colnames[[i]]
       init_list[[p_name]] <- rep(0, P_dim)
       lin_pred_expr <- bquote(.(as.name(paste0("X_", i))) %*% .(as.name(p_name)))
    }

    if (f_type == "gaussian") {
      param_exprs[[length(param_exprs) + 1]] <- bquote(.(as.name(s_name)) <- Dim(lower = 0))
      init_list[[s_name]] <- 1.0
      model_exprs[[length(model_exprs) + 1]] <- bquote(.(as.name(paste0("Y_", i))) ~ normal(.(lin_pred_expr), .(as.name(s_name))))
      if (isTRUE(WAIC)) {
        generate_exprs[[length(generate_exprs) + 1]] <- as.call(list(
          as.name("<-"),
          as.name(paste0("log_lik_", i)),
          bquote(normal_lpdf(.(as.name(paste0("Y_", i))), .(lin_pred_expr), .(as.name(s_name)), sum = FALSE))
        ))
      }
    } else if (f_type %in% c("binomial", "bernoulli")) {
      model_exprs[[length(model_exprs) + 1]] <- bquote(.(as.name(paste0("Y_", i))) ~ bernoulli_logit(.(lin_pred_expr)))
      if (isTRUE(WAIC)) {
        generate_exprs[[length(generate_exprs) + 1]] <- as.call(list(
          as.name("<-"),
          as.name(paste0("log_lik_", i)),
          bquote(bernoulli_logit_lpmf(.(as.name(paste0("Y_", i))), .(lin_pred_expr), sum = FALSE))
        ))
      }
    } else if (f_type == "poisson") {
      model_exprs[[length(model_exprs) + 1]] <- bquote(.(as.name(paste0("Y_", i))) ~ poisson_log(.(lin_pred_expr)))
      if (isTRUE(WAIC)) {
        generate_exprs[[length(generate_exprs) + 1]] <- as.call(list(
          as.name("<-"),
          as.name(paste0("log_lik_", i)),
          bquote(poisson_lpmf(.(as.name(paste0("Y_", i))), exp(.(lin_pred_expr)), sum = FALSE))
        ))
      }
    }

    has_b_prior <- prior_type %in% c("weak", "normal")
    if (has_b_prior) {
      b_prior_sd_name <- as.name(paste0(p_name, "_prior_sd"))
      b_prior_mean_name <- as.name(paste0(p_name, "_prior_mean"))
      model_exprs[[length(model_exprs) + 1]] <- bquote(.(as.name(target_p_name)) ~ normal(.(b_prior_mean_name), .(b_prior_sd_name)))
    }

    if (f_type == "gaussian") {
      has_sigma_prior <- prior_type %in% c("weak", "normal")
      if (has_sigma_prior) {
        rate_name <- as.name(paste0(s_name, "_rate"))
        model_exprs[[length(model_exprs) + 1]] <- bquote(.(as.name(s_name)) ~ exponential(.(rate_name)))
      }
    }

    b_vars <- c(b_vars, p_name)
    if (f_type == "gaussian") s_vars <- c(s_vars, s_name)
  }

  # 4. Path Identification and Indirect Effects
  all_preds <- unique(unlist(X_colnames))
  all_resps <- unique(resp_names)
  mediators <- intersect(all_resps, all_preds)
  indeps <- setdiff(all_preds, c(all_resps, "Intercept"))
  if (length(indeps) == 0 && length(mediators) > 0) indeps <- setdiff(X_colnames[[1]], "Intercept")

  for (iv in indeps) {
    for (m in mediators) {
      idx_m_resp <- which(resp_names == m)
      idx_m_pred <- which(sapply(X_colnames, function(x) m %in% x))

      if (length(idx_m_resp) > 0 && length(idx_m_pred) > 0) {
        pos_iv <- which(X_colnames[[idx_m_resp]] == iv)
        if (length(pos_iv) > 0) {
          a_val <- bquote(.(as.name(paste0("b", idx_m_resp)))[.(pos_iv)])
          for (dv_idx in idx_m_pred) {
            if (dv_idx == idx_m_resp) next
            dv_name <- resp_names[dv_idx]
            pos_m <- which(X_colnames[[dv_idx]] == m)
            b_val <- bquote(.(as.name(paste0("b", dv_idx)))[.(pos_m)])
            ie_name <- paste0("IE_", iv, "_", m, "_", dv_name)
            tran_exprs[[ie_name]] <- bquote(.(as.name(ie_name)) <- .(a_val) * .(b_val))
            effect_names <- unique(c(effect_names, ie_name))
            pos_iv_direct <- which(X_colnames[[dv_idx]] == iv)
            if (length(pos_iv_direct) > 0) {
              de_name <- paste0("DE_", iv, "_", dv_name)
              de_val <- bquote(.(as.name(paste0("b", dv_idx)))[.(pos_iv_direct)])
              tran_exprs[[de_name]] <- bquote(.(as.name(de_name)) <- .(de_val))
              effect_names <- unique(c(effect_names, de_name))
              te_name <- paste0("TE_", iv, "_", m, "_", dv_name)
              tran_exprs[[te_name]] <- bquote(.(as.name(te_name)) <- .(as.name(ie_name)) + .(de_val))
              effect_names <- unique(c(effect_names, te_name))
            }
          }
        }
      }
    }
  }

  # Calculate specific degrees of freedom for each equation
  df_map <- list()
  for (i in 1:n_eq) {
    df_val <- N - ncol(X_list[[i]])
    df_map[[paste0("b", i)]] <- df_val
    df_map[[paste0("sigma", i)]] <- df_val
  }
  # For IE/DE/TE, use the DF of the outcome equation
  for (iv in indeps) {
    for (m in mediators) {
      idx_m_resp <- which(resp_names == m)
      idx_m_pred <- which(sapply(X_colnames, function(x) m %in% x))
      if (length(idx_m_resp) > 0 && length(idx_m_pred) > 0) {
        for (dv_idx in idx_m_pred) {
          if (dv_idx == idx_m_resp) next
          dv_name <- resp_names[dv_idx]
          df_dv <- N - ncol(X_list[[dv_idx]])
          df_map[[paste0("IE_", iv, "_", m, "_", dv_name)]] <- df_dv
          df_map[[paste0("DE_", iv, "_", dv_name)]] <- df_dv
          df_map[[paste0("TE_", iv, "_", m, "_", dv_name)]] <- df_dv
        }
      }
    }
  }

  mdl_code <- list(
    setup = setup_ast,
    parameters = as.call(c(list(as.name("{")), param_exprs))
  )
  if (length(tran_exprs) > 0) {
    mdl_code$transform <- as.call(c(list(as.name("{")), tran_exprs))
  }
  mdl_code$model <- as.call(c(list(as.name("{")), model_exprs))

  if (length(generate_exprs) > 0) {
    if (isTRUE(WAIC)) {
      log_lik_names <- lapply(seq_len(n_eq), function(i) as.name(paste0("log_lik_", i)))
      joint_log_lik <- if (length(log_lik_names) == 1L) {
        log_lik_names[[1L]]
      } else {
        Reduce(function(a, b) as.call(list(as.name("+"), a, b)), log_lik_names)
      }
      generate_exprs[[length(generate_exprs) + 1]] <- joint_log_lik
      generate_exprs[[length(generate_exprs)]] <- as.call(list(as.name("<-"), as.name("log_lik"), generate_exprs[[length(generate_exprs)]]))
    }
    gen_ast <- as.call(c(list(as.name("{")), generate_exprs))
    mdl_code$generate <- if (isTRUE(WAIC)) .rtmb_waic_generate_ast(NULL, gen_ast) else gen_ast
  }
  mdl_code$env <- tmp_env

  view_order <- c(b_vars, effect_names, s_vars)
  if (!is.null(view)) {
    view_order <- unique(c(view, view_order))
  }

  mdl <- rtmb_model(data = data_list, code = mdl_code, par_names = v_names, init = init_list, fixed = fixed, view = view_order, silent = FALSE)
  mdl$formula <- formula
  mdl$raw_data <- setup_df
  mdl$family <- family_list

  mdl$type <- "mediation"
  mdl$extra <- list(
    source = "wrapper",
    prior_type = prior$type,
    marginal = if (prior_type %in% c("weak", "normal")) paste0("b_c", 1:n_eq) else paste0("b", 1:n_eq),
    mediation = list(
      formula = formula,
      family = family_list,
      view = view,
      n_eq = n_eq,
      responses = resp_names
    )
  )

  mdl$extra$df_map <- df_map
  mdl$extra$effect_names <- effect_names
  mdl$extra$equation_df <- setNames(
    vapply(seq_len(n_eq), function(i) N - ncol(X_list[[i]]), numeric(1)),
    paste0("eq", seq_len(n_eq))
  )

  return(mdl)
}

#' RTMB-based Contingency Table Analysis (Chi-squared Test)
#'
#' @description
#' `rtmb_table` performs a chi-squared test of independence between two categorical variables.
#' It provides both classic (frequentist) Pearson chi-squared tests and Bayesian multinomial-style models.
#'
#' @param x Variable name, formula, table, or matrix.
#' @param y Variable name (optional if x is a formula).
#' @param data A data frame.
#' @param correct Logical; if TRUE, apply Yates' continuity correction for 2x2 classic analyses.
#' @param prior Prior specification (Bayesian mode). Default is `prior_flat()`.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments.
#'
#' @return An `RTMB_Model` object.
#'
#' @examples
#' \donttest{
#' # Classic chi-squared test
#' rtmb_table(skill, cond, data = debate)$classic()
#' rtmb_table(table(debate$skill, debate$cond))$classic()
#' }
#' @export
rtmb_table <- function(x, y = NULL, data = NULL, correct = TRUE, prior = prior_flat(),
                       fixed = NULL, WAIC = FALSE, ...) {

  x_expr <- substitute(x)
  y_expr <- substitute(y)
  if (is.null(prior)) prior <- prior_flat()
  if (!inherits(prior, "rtmb_prior")) {
    stop("prior must be an object of class 'rtmb_prior'.", call. = FALSE)
  }

  # 1. Extract table or variables
  if (!is.null(data)) {
    x_val <- if (is.name(x_expr)) data[[as.character(x_expr)]] else eval(x_expr, data)
  } else {
    x_val <- eval(x_expr, parent.frame())
  }

  if (inherits(x_val, "table") || (is.matrix(x_val) && is.numeric(x_val))) {
    tab <- as.table(x_val)
    if (length(dim(tab)) != 2L) {
      stop("`x` must be a two-way table or matrix when supplied as a table-like object.", call. = FALSE)
    }
    dimn <- dimnames(tab)
    if (is.null(dimn)) dimn <- vector("list", 2L)
    if (is.null(dimn[[1]])) dimn[[1]] <- paste0("Row", seq_len(nrow(tab)))
    if (is.null(dimn[[2]])) dimn[[2]] <- paste0("Col", seq_len(ncol(tab)))
    dimnames(tab) <- dimn
    table_name <- deparse(x_expr)
    dim_names <- names(dimn)
    v1_name <- if (!is.null(dim_names) && nzchar(dim_names[1])) dim_names[1] else paste0(table_name, "_row")
    v2_name <- if (!is.null(dim_names) && length(dim_names) >= 2L && nzchar(dim_names[2])) dim_names[2] else paste0(table_name, "_col")
  } else {
    if (missing(y) || identical(y_expr, quote(NULL))) {
      stop("`y` is required unless `x` is a two-way table or matrix.", call. = FALSE)
    }
    if (is.null(data)) {
      v1 <- x_val
      v2 <- eval(y_expr, parent.frame())
      v1_name <- deparse(x_expr)
      v2_name <- deparse(y_expr)
    } else {
      v1 <- x_val
      v2 <- if (is.name(y_expr)) data[[as.character(y_expr)]] else eval(y_expr, data)
      v1_name <- as.character(x_expr)
      v2_name <- as.character(y_expr)
    }

    v1 <- as.factor(v1)
    v2 <- as.factor(v2)
    tab <- table(v1, v2)
  }

  # Prepare row/column names for labels
  R_names <- rownames(tab)
  C_names <- colnames(tab)
  grid <- expand.grid(Row = R_names, Col = C_names)
  cell_labels <- paste0(v1_name, ":", grid$Row, ", ", v2_name, ":", grid$Col)

  # 2. Model (Multinomial Model)
  # Data for RTMB
  Y_vec <- as.vector(tab)
  R <- nrow(tab)
  C <- ncol(tab)
  N_total <- sum(Y_vec)
  if (R < 2L || C < 2L) {
    stop("rtmb_table() requires a table with at least two rows and two columns.", call. = FALSE)
  }
  if (any(is.na(Y_vec)) || any(!is.finite(Y_vec))) {
    stop("Table counts must be finite and non-missing.", call. = FALSE)
  }
  if (any(Y_vec < 0)) {
    stop("Table counts must be non-negative.", call. = FALSE)
  }
  if (any(abs(Y_vec - round(Y_vec)) > .Machine$double.eps^0.5)) {
    stop("Table counts must be whole numbers.", call. = FALSE)
  }
  if (N_total <= 0) {
    stop("Table counts must sum to a positive total.", call. = FALSE)
  }

  setup <- list(
    Y = Y_vec,
    R = R,
    C = C,
    N = N_total
  )

  # Pre-resolve prior settings (avoid runtime NSE dependency on prior object)
  dirichlet_alpha_val <- if (inherits(prior, "rtmb_prior") && prior$type == "normal" &&
                              !is.null(prior$dirichlet_alpha)) {
    prior$dirichlet_alpha
  } else {
    1
  }
  setup$dirichlet_alpha <- dirichlet_alpha_val

  gen_ast <- if (isTRUE(WAIC)) {
    .rtmb_waic_generate_ast(NULL, quote({
      log_lik <- multinomial_lpmf(Y, N, p)
    }))
  } else {
    quote({
      # mu and chisq_val are already reported in transform
    })
  }

  rtmb_model_code <- eval(substitute(
    rtmb_code(
      setup = {
        # No special setup needed for now, Y and N are provided
      },
      parameters = {
        # Simplex for probabilities (automatically constrained to sum to 1)
        # We give it meaningful names during rtmb_model() call
        p <- Dim(R * C, type = "simplex")
      },
      transform = {
        # Derived quantities for reporting
        mu <- p * N
        # Pearson Chi-squared statistic for the estimated probabilities
        # Sum (O - E)^2 / E
        chisq_val <- sum((Y - mu)^2 / mu)
      },
      model = {
        # Likelihood
        Y ~ multinomial(N, p)

        # Prior (alpha value is embedded as data at construction time)
        p ~ dirichlet(rep(dirichlet_alpha, R * C))
      },
      generate = G
    ),
    list(G = gen_ast)
  ))

  # Create the model object with explicit parameter names for 'p' and 'mu'
  res <- rtmb_model(
    code = rtmb_model_code,
    data = setup,
    par_names = list(p = cell_labels, mu = cell_labels),
    fixed = fixed
  )
  # Set metadata for special print/summary dispatch
  res$type <- "table"
  res$extra <- list(
    source = "wrapper",
    prior_type = if (inherits(prior, "rtmb_prior")) prior$type else "flat",
    marginal = "p",
    tab = tab, 
    correct = correct
  )
  
  class(res) <- c("rtmb_table", class(res))
  return(res)
}

#' Mixture Model Wrapper for RTMB
#'
#' @description
#' Provides a user-friendly interface for fitting Gaussian mixture models
#' with optional covariates on class membership probabilities and various
#' covariance structures.
#'
#' @param formula A formula specifying the response variable(s). For multivariate, use \code{cbind(y1, y2) ~ 1}.
#' @param k Number of mixture components.
#' @param data A data frame containing the variables in the model.
#' @param covariance Covariance structure: "diagonal" (default), "diagonal_equal", "full", "full_equal", or "full_equal_corr".
#' @param prior Prior configuration: `prior_flat()`, `prior_normal()`,
#'   `prior_weak()`, `prior_rhs()`, or `prior_ssp()`. Default is
#'   `prior_flat()`. If `y_range` is supplied with the default flat prior,
#'   the wrapper automatically switches to `prior_weak()`.
#' @param y_range Optional numeric vector or matrix defining the theoretical range (min, max) of response variables.
#'   Specifying this automatically enables weakly informative priors if `prior` is `prior_flat()`.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments passed to `rtmb_model`.
#' @return A \code{RTMB_Model} object.
#' @example inst/examples/ex_mixture.R
#' @export
rtmb_mixture <- function(formula, k = 2, data = NULL,
                         covariance = c("diagonal", "diagonal_equal", "full", "full_equal", "full_equal_corr"),
                         prior = prior_flat(), y_range = NULL, fixed = NULL,
                         WAIC = FALSE, ...) {

  if (!is.numeric(k) || length(k) != 1L || is.na(k) || k < 2 || k != as.integer(k)) {
    stop("'k' must be an integer greater than or equal to 2.", call. = FALSE)
  }
  k <- as.integer(k)

  if (is.null(prior)) {
    prior <- prior_flat()
  }

  # Automatically switch to prior_weak() if y_range is provided and prior is default flat
  if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }

  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), prior_weak(), prior_rhs(), or prior_ssp().",
      call. = FALSE
    )
  }

  # NSE for formula: handle case where formula is just a variable name in data
  formula_expr <- substitute(formula)
  formula_val <- try(formula, silent = TRUE)

  if (inherits(formula_val, "try-error") ||
      (!inherits(formula_val, "formula") && !is.matrix(formula_val) && !is.vector(formula_val))) {
    # Check if variables in formula_expr are in data
    if (!is.null(data) && all(all.vars(formula_expr) %in% names(data))) {
      formula <- as.formula(call("~", formula_expr, 1))
    } else if (!inherits(formula_val, "try-error")) {
      formula <- formula_val
    } else {
      stop(paste0("Object '", deparse(formula_expr), "' not found."))
    }
  } else {
    formula <- formula_val
  }

  covariance <- match.arg(covariance)
  K_mix <- k
  prior_type <- prior$type

  if (prior_type == "flat") {
    # no merge
  } else if (prior_type == "normal") {
    normal_defaults <- list(
      Intercept_sd = NULL,
      mu_sd = NULL,
      b_sd = NULL,
      sigma_rate = NULL,
      lkj_eta = NULL,
      dirichlet_alpha = NULL
    )
    prior <- .merge_prior(normal_defaults, prior)
  } else {
    default_prior <- list(Intercept_sd = 10, mu_sd = 10, b_sd = 10, sigma_rate = 1, lkj_eta = 1.0, dirichlet_alpha = 1.0)
    prior <- .merge_prior(default_prior, prior)
  }

  # Sync mu_sd and Intercept_sd
  if (prior_type == "normal") {
    if (is.null(prior$mu_sd) && !is.null(prior$Intercept_sd)) {
      prior$mu_sd <- prior$Intercept_sd
    }
  }

  regularization <- if (prior_type %in% c("rhs", "ssp")) prior_type else "none"
  use_weak_info <- prior_type %in% c("weak", "rhs", "ssp")

  if (use_weak_info) {
    if (is.null(prior$max_beta)) prior$max_beta <- 1.0
    if (is.null(prior$expected_vars)) prior$expected_vars <- 3
    if (is.null(prior$slab_scale)) prior$slab_scale <- 2.0
    if (is.null(prior$slab_df)) prior$slab_df <- 4.0
    if (is.null(prior$ssp_ratio)) prior$ssp_ratio <- 0.25
    if (prior_type == "weak" && is.null(y_range)) {
      warning(
        "prior_weak() is used without 'y_range'. ",
        "Default hyperparameters will be applied, which may not be well-scaled for your data. ",
        "Consider specifying 'y_range' for better prior calibration.",
        call. = FALSE
      )
    }
  }

  setup_from_formula <- inherits(formula, "formula") && !is.null(data)

  # Parse formulas and prepare data
  if (is.character(formula) && !is.null(data) && all(formula %in% names(data))) {
    Y_mat <- as.matrix(data[, formula, drop = FALSE])
    X_prob <- NULL
  } else if (is.matrix(formula) || (is.vector(formula) && !is.character(formula))) {
    Y_mat <- as.matrix(formula)
    X_prob <- NULL
  } else if (!inherits(formula, "formula")) {
    # Case where a single variable name is passed
    var_name <- as.character(formula)
    if (!is.null(data) && var_name %in% names(data)) {
      Y_mat <- as.matrix(data[[var_name]])
    } else {
      Y_mat <- as.matrix(formula)
    }
    X_prob <- NULL
  } else {
    mf_y <- model.frame(formula, data = data)
    Y_mat <- model.response(mf_y)
    if (is.null(Y_mat)) Y_mat <- as.matrix(mf_y)

    # Check if formula has RHS covariates for prob
    rhs <- formula[[3]]
    is_empty_rhs <- (is.numeric(rhs) && rhs == 1) || (is.symbol(rhs) && rhs == "1")

    if (!is_empty_rhs) {
      X_prob <- model.matrix(formula, data = data)
    } else {
      X_prob <- NULL
    }
  }

  if (setup_from_formula) {
    setup_vars <- all.vars(formula)
    missing_vars <- setdiff(setup_vars, names(data))
    if (length(missing_vars) > 0) {
      stop(
        "The following variables in formula are not found in data: ",
        paste(missing_vars, collapse = ", "),
        call. = FALSE
      )
    }
    setup_df <- as.data.frame(data)[, setup_vars, drop = FALSE]
    class(setup_df) <- c("rtmb_setup_df", class(setup_df))
  }

  Y_mat <- as.matrix(Y_mat)
  if (!is.numeric(Y_mat) && !is.logical(Y_mat)) {
    stop("The mixture response must be numeric.", call. = FALSE)
  }
  storage.mode(Y_mat) <- "double"
  if (anyNA(Y_mat)) {
    stop("rtmb_mixture() does not currently support missing response values.", call. = FALSE)
  }
  N_obs <- nrow(Y_mat)
  P_dim <- ncol(Y_mat)
  if (is.null(N_obs) || N_obs < 1L) {
    stop("The mixture response must contain at least one observation.", call. = FALSE)
  }
  if (is.null(P_dim) || P_dim < 1L) {
    stop("The mixture response must contain at least one variable.", call. = FALSE)
  }

  if (is.null(colnames(Y_mat))) {
    colnames(Y_mat) <- paste0("Y", 1:P_dim)
  }

  multivariate <- P_dim > 1
  has_cov_prob <- !is.null(X_prob)
  K_prob <- if (has_cov_prob) ncol(X_prob) else 0
  is_sigma_equal <- covariance %in% c("diagonal_equal", "full_equal", "full_equal_corr")

  # --- 1. Dynamic AST Construction: setup ---
  if (setup_from_formula) {
    setup_exprs <- list(
      as.name("{"),
      quote(mf <- model.frame(formula, df)),
      quote(Y <- model.response(mf)),
      quote(if (is.null(Y)) Y <- as.matrix(mf)),
      quote(Y <- as.matrix(Y)),
      quote(N <- nrow(Y)),
      quote(P <- ncol(Y)),
      quote(K <- K)
    )
  } else {
    setup_exprs <- list(
      as.name("{"),
      quote(N <- N),
      quote(K <- K),
      quote(P <- P)
    )
  }
  if (has_cov_prob) {
    if (setup_from_formula) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_prob <- model.matrix(formula, mf))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(K_prob <- ncol(X_prob))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_means <- matrix(colMeans(X_prob), 1, K_prob))
    } else {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(K_prob <- K_prob)
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_means <- matrix(X_means, 1, K_prob))
    }
    if (regularization == "rhs") {
      # RHS setup (cap expected_vars to prevent zero-division)
      p0_rhs <- min(prior$expected_vars, K_prob - 1)
      if (p0_rhs < 1) p0_rhs <- 1
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) {
        bquote(tau0 <- .(p0_rhs) / (K_prob - .(p0_rhs)) / sqrt(N))
      } else {
        quote(tau0 <- tau0)
      }
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) bquote(half_slab_df <- .(prior$slab_df) / 2) else quote(half_slab_df <- half_slab_df)
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) bquote(half_slab_scale2 <- .(prior$slab_scale)^2 / 2) else quote(half_slab_scale2 <- half_slab_scale2)
    } else if (regularization == "ssp") {
      # SSP setup
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) bquote(tau_scale <- .(prior$max_beta) / 1.96) else quote(tau_scale <- tau_scale)
    }
  }
  setup_ast <- as.call(setup_exprs)

  # --- 2. Dynamic AST Construction: parameters ---
  param_exprs <- list(as.name("{"))
  if (has_cov_prob) {
      if (regularization == "rhs") {
        param_exprs[[length(param_exprs) + 1]] <- bquote(z_b <- Dim(c(.(K_prob), .(K_mix - 1))))
        param_exprs[[length(param_exprs) + 1]] <- bquote(w_lambda_b <- Dim(c(.(K_prob), .(K_mix - 1)), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(w_tau_b <- Dim(.(K_mix - 1), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(c2_b <- Dim(.(K_mix - 1), lower = 0))
      } else if (regularization == "ssp") {
        param_exprs[[length(param_exprs) + 1]] <- bquote(b_raw <- Dim(c(.(K_prob), .(K_mix - 1))))
        param_exprs[[length(param_exprs) + 1]] <- bquote(r_b <- Dim(c(.(K_prob), .(K_mix - 1)), lower = 0, upper = 1))
        param_exprs[[length(param_exprs) + 1]] <- bquote(tau_b <- Dim(.(K_mix - 1), lower = 0))
      } else {
        param_exprs[[length(param_exprs) + 1]] <- bquote(b <- Dim(c(.(K_prob), .(K_mix - 1))))
      }
  } else {
    param_exprs[[length(param_exprs) + 1]] <- bquote(theta <- Dim(.(K_mix), type = "simplex"))
  }

  if (multivariate) {
    param_exprs[[length(param_exprs) + 1]] <- bquote(mu <- Dim(c(K, P)))
    s_dim <- if (is_sigma_equal && covariance != "diagonal_equal") P_dim else c(K_mix, P_dim)
    if (covariance == "diagonal_equal") s_dim <- P_dim
    param_exprs[[length(param_exprs) + 1]] <- bquote(sigma <- Dim(.(s_dim), lower = 0))
  } else {
    param_exprs[[length(param_exprs) + 1]] <- bquote(mu <- Dim(K))
    s_dim <- if (is_sigma_equal) NULL else K_mix
    param_exprs[[length(param_exprs) + 1]] <- if (is.null(s_dim)) quote(sigma <- Dim(lower = 0)) else bquote(sigma <- Dim(.(s_dim), lower = 0))
  }

  if (multivariate && covariance %in% c("full", "full_equal", "full_equal_corr")) {
    if (covariance == "full") {
      param_exprs[[length(param_exprs) + 1]] <- quote(L_corr <- Dim(c(K, P, P), type = "CF_corr"))
    } else {
      param_exprs[[length(param_exprs) + 1]] <- quote(L_corr <- Dim(c(P, P), type = "CF_corr"))
    }
  }
  param_ast <- as.call(param_exprs)

  # --- 3. Dynamic AST Construction: model ---
  model_exprs <- list(as.name("{"))

  # Regularization transformations
  if (has_cov_prob) {
    if (regularization == "rhs") {
      model_exprs[[length(model_exprs) + 1]] <- quote(lambda_b <- 1 / sqrt(w_lambda_b))
      model_exprs[[length(model_exprs) + 1]] <- quote(tau_hs_b <- tau0 / sqrt(w_tau_b))
      model_exprs[[length(model_exprs) + 1]] <- quote(b <- matrix(0, K_prob, K-1)) 
      model_exprs[[length(model_exprs) + 1]] <- quote(for (k in 1:(K-1)) {
        lambda_tilde_k <- sqrt(c2_b[k] * lambda_b[, k]^2 / (c2_b[k] + tau_hs_b[k]^2 * lambda_b[, k]^2))
        b[, k] <- z_b[, k] * tau_hs_b[k] * lambda_tilde_k
      })
    } else if (regularization == "ssp") {
      model_exprs[[length(model_exprs) + 1]] <- quote(b <- b_raw * r_b * tau_b)
    }
  }

  model_exprs[[length(model_exprs) + 1]] <- quote(lp <- mu[1] * 0)
  model_exprs[[length(model_exprs) + 1]] <- quote(log_dens_mat <- matrix(mu[1] * 0, N, K))

  is_diag <- covariance %in% c("diagonal", "diagonal_equal")
  s_k_expr <- if (is_sigma_equal) quote(sigma) else quote(sigma[k, ])
  if (!multivariate && !is_sigma_equal) s_k_expr <- quote(sigma[k])

  dist_body <- if (multivariate) {
    if (is_diag) {
      bquote({
        ld <- Y[1] * 0
        s_vec <- .(s_k_expr)
        m_vec <- mu[k, ]
        for (p in 1:P) {
          ld <- ld + normal_lpdf(Y[, p], m_vec[p], s_vec[p], sum = FALSE)
        }
        log_dens_mat[, k] <- ld
      })
    } else {
      L_corr_k_expr <- if (covariance == "full") quote(matrix(L_corr[k, , ], P, P)) else quote(L_corr)
      bquote({
        log_dens_mat[, k] <- multi_normal_CF_lpdf(Y, mean = mu[k, ], sd = .(s_k_expr), CF_Omega = .(L_corr_k_expr), sum = FALSE)
      })
    }
  } else {
    bquote({
      log_dens_mat[, k] <- normal_lpdf(Y, mu[k], .(s_k_expr), sum = FALSE)
    })
  }

  model_exprs[[length(model_exprs) + 1]] <- as.call(list(as.name("for"), as.name("k"), quote(1:K), dist_body))

  if (has_cov_prob) {
    model_exprs[[length(model_exprs) + 1]] <- quote(eta_prob <- X_prob %*% b)
    model_exprs[[length(model_exprs) + 1]] <- quote(log_pi_mat <- matrix(eta_prob[1] * 0, N, K))
    model_exprs[[length(model_exprs) + 1]] <- quote(for (i in 1:N) {
      log_pi_mat[i, ] <- log_softmax(c(0, eta_prob[i, ]))
    })
    model_exprs[[length(model_exprs) + 1]] <- quote(lp <- lp + sum(log_sum_exp(log_pi_mat + log_dens_mat)))
  } else {
    model_exprs[[length(model_exprs) + 1]] <- quote(log_theta <- log(theta))
    model_exprs[[length(model_exprs) + 1]] <- quote(lp <- lp + sum(log_sum_exp(t(t(log_dens_mat) + log_theta))))
  }

  # Priors
  if (use_weak_info) {
    # Component priors
    model_exprs[[length(model_exprs) + 1]] <- bquote(mu ~ normal(0, .(prior$Intercept_sd)))
    model_exprs[[length(model_exprs) + 1]] <- bquote(sigma ~ exponential(.(prior$sigma_rate)))

    if (multivariate && covariance %in% c("full", "full_equal", "full_equal_corr")) {
      if (covariance == "full") {
        model_exprs[[length(model_exprs) + 1]] <- bquote(for (k in 1:K) matrix(L_corr[k, , ], P, P) ~ lkj_CF_corr(.(prior$lkj_eta)))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(L_corr ~ lkj_CF_corr(.(prior$lkj_eta)))
      }
    }

    if (has_cov_prob) {
      if (regularization == "rhs") {
        model_exprs[[length(model_exprs) + 1]] <- quote(z_b ~ normal(0, 1))
        model_exprs[[length(model_exprs) + 1]] <- quote(w_lambda_b ~ gamma(0.5, 0.5))
        model_exprs[[length(model_exprs) + 1]] <- quote(w_tau_b ~ gamma(0.5, 0.5))
        model_exprs[[length(model_exprs) + 1]] <- quote(c2_b ~ inverse_gamma(half_slab_df, half_slab_scale2))
      } else if (regularization == "ssp") {
        model_exprs[[length(model_exprs) + 1]] <- quote(b_raw ~ laplace(0, 0.5))
        model_exprs[[length(model_exprs) + 1]] <- bquote(mu_r <- log(.(prior$ssp_ratio) / (1 - .(prior$ssp_ratio))))
        model_exprs[[length(model_exprs) + 1]] <- quote(r_b ~ logit_normal(mu_r, 3))
        model_exprs[[length(model_exprs) + 1]] <- quote(tau_b ~ exponential(1 / tau_scale))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(b ~ normal(0, .(prior$b_sd)))
      }
    } else {
      model_exprs[[length(model_exprs) + 1]] <- bquote(theta ~ dirichlet(rep(.(prior$dirichlet_alpha), K)))
    }
    } else if (prior_type == "normal") {
      if (!is.null(prior$Intercept_sd) || !is.null(prior$mu_sd)) {
        mu_sd_val <- if (!is.null(prior$mu_sd)) prior$mu_sd else prior$Intercept_sd
        model_exprs[[length(model_exprs) + 1]] <- bquote(mu ~ normal(0, .(mu_sd_val)))
      }

      if (!is.null(prior$sigma_rate)) {
        model_exprs[[length(model_exprs) + 1]] <- bquote(sigma ~ exponential(.(prior$sigma_rate)))
      }

      if (has_cov_prob && !is.null(prior$b_sd)) {
        model_exprs[[length(model_exprs) + 1]] <- bquote(b ~ normal(0, .(prior$b_sd)))
      }

      if (!is.null(prior$lkj_eta) && multivariate && !is_diag) {
        if (covariance == "full") {
          model_exprs[[length(model_exprs) + 1]] <-
            bquote(for (k in 1:K) matrix(L_corr[k, , ], P, P) ~ lkj_CF_corr(.(prior$lkj_eta)))
        } else {
          model_exprs[[length(model_exprs) + 1]] <-
            bquote(L_corr ~ lkj_CF_corr(.(prior$lkj_eta)))
        }
      }
    }

  model_ast <- as.call(model_exprs)

  # --- 4. Dynamic AST Construction: generate ---
  generate_exprs <- list(as.name("{"))
  if (has_cov_prob) {
    if (regularization == "rhs") {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(lambda_b <- 1 / sqrt(w_lambda_b))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(tau_hs_b <- tau0 / sqrt(w_tau_b))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(b <- matrix(0, P, K-1))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(for (k in 1:(K-1)) {
        lambda_tilde_k <- sqrt(c2_b[k] * lambda_b[, k]^2 / (c2_b[k] + tau_hs_b[k]^2 * lambda_b[, k]^2))
        b[, k] <- z_b[, k] * tau_hs_b[k] * lambda_tilde_k
      })
    } else if (regularization == "ssp") {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(b <- b_raw * r_b * tau_b)
    }
    generate_exprs[[length(generate_exprs) + 1]] <- quote(prob_mean <- softmax(c(0, X_means %*% b)))
  } else {
    generate_exprs[[length(generate_exprs) + 1]] <- quote(prob_mean <- theta)
  }

  if (multivariate && !is_diag) {
    if (covariance == "full") {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(corr <- array(mu[1] * 0, dim = c(K, P, P)))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(for (k in 1:K) {
        corr[k, , ] <- matrix(L_corr[k, , ], P, P) %*% t(matrix(L_corr[k, , ], P, P))
      })
    } else {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(corr <- L_corr %*% t(L_corr))
    }
  }
  generate_base_exprs <- generate_exprs
  if (isTRUE(WAIC)) {
    waic_exprs <- list(as.name("{"))
    waic_exprs[[length(waic_exprs) + 1]] <- quote(log_dens_mat <- matrix(mu[1] * 0, N, K))
    waic_exprs[[length(waic_exprs) + 1]] <- as.call(list(as.name("for"), as.name("k"), quote(1:K), dist_body))
    if (has_cov_prob) {
      waic_exprs[[length(waic_exprs) + 1]] <- quote(eta_prob <- X_prob %*% b)
      waic_exprs[[length(waic_exprs) + 1]] <- quote(log_pi_mat <- matrix(eta_prob[1] * 0, N, K))
      waic_exprs[[length(waic_exprs) + 1]] <- quote(for (i in 1:N) {
        log_pi_mat[i, ] <- log_softmax(c(0, eta_prob[i, ]))
      })
      waic_exprs[[length(waic_exprs) + 1]] <- quote(log_lik <- log_sum_exp(log_pi_mat + log_dens_mat))
    } else {
      waic_exprs[[length(waic_exprs) + 1]] <- quote(log_lik <- log_sum_exp(t(t(log_dens_mat) + log(theta))))
    }
  }
  generate_ast <- if (isTRUE(WAIC)) {
    .rtmb_waic_generate_ast(as.call(generate_base_exprs), as.call(waic_exprs))
  } else {
    as.call(generate_base_exprs)
  }

  mdl_code <- list(
    setup = setup_ast,
    parameters = param_ast,
    model = model_ast,
    generate = generate_ast,
    env = parent.frame()
  )
  class(mdl_code) <- "rtmb_code"

  v_names <- list()
  if (multivariate) {
    v_names$mu <- list(paste0("C", 1:K_mix), colnames(Y_mat))
    v_names$sigma <- if (is_sigma_equal && covariance != "diagonal_equal") colnames(Y_mat) else list(paste0("C", 1:K_mix), colnames(Y_mat))
    if (covariance == "diagonal_equal") v_names$sigma <- colnames(Y_mat)
  } else {
    v_names$mu <- paste0("C", 1:K_mix)
    v_names$sigma <- if (is_sigma_equal) "sigma" else paste0("C", 1:K_mix)
  }

  if (has_cov_prob) {
    colnames(X_prob)[colnames(X_prob) == "(Intercept)"] <- "Intercept"
    v_names$b <- list(colnames(X_prob), paste0("C", 2:K_mix))
  }
  if (multivariate && !is_diag) {
    if (covariance == "full") {
      v_names$L_corr <- list(paste0("C", 1:K_mix), colnames(Y_mat), colnames(Y_mat))
      v_names$corr <- list(paste0("C", 1:K_mix), colnames(Y_mat), colnames(Y_mat))
    } else {
      v_names$L_corr <- list(colnames(Y_mat), colnames(Y_mat))
      v_names$corr <- list(colnames(Y_mat), colnames(Y_mat))
    }
  }

  data_list <- if (setup_from_formula) {
    list(df = setup_df, formula = formula, K = K_mix)
  } else {
    list(Y = Y_mat, N = N_obs, K = K_mix, P = P_dim)
  }
  if (has_cov_prob) {
    if (!setup_from_formula) {
      data_list$X_prob <- X_prob
      data_list$K_prob <- K_prob
      data_list$X_means <- as.vector(colMeans(X_prob))
    }
    if (regularization == "rhs") {
      p0 <- min(prior$expected_vars, K_prob - 1)
      if (p0 < 1) p0 <- 1
      if (!setup_from_formula) {
        data_list$tau0 <- p0 / (K_prob - p0) / sqrt(N_obs)
        data_list$half_slab_df <- prior$slab_df / 2
        data_list$half_slab_scale2 <- prior$slab_scale^2 / 2
      }
    } else if (regularization == "ssp") {
      if (!setup_from_formula) data_list$tau_scale <- prior$max_beta / 1.96
    }
  }

  init_list <- list()
  if (nrow(Y_mat) >= K_mix) {
    km <- try(kmeans(Y_mat, centers = K_mix, nstart = 5), silent = TRUE)
    if (!inherits(km, "try-error")) {
      if (multivariate) {
        init_list$mu <- km$centers
      } else {
        init_list$mu <- as.vector(km$centers)
      }
      props <- as.vector(table(factor(km$cluster, levels = 1:K_mix))) / nrow(Y_mat)
      init_list$theta <- props

      if (has_cov_prob) {
        b_init <- matrix(0, ncol(X_prob), K_mix - 1)
        for (k in 2:K_mix) {
          z_k <- as.numeric(km$cluster == k)
          fit_k <- try(suppressWarnings(glm(z_k ~ X_prob - 1 + offset(rep(qlogis(max(0.01, props[k])), length(z_k))), family = binomial)), silent = TRUE)
          if (!inherits(fit_k, "try-error")) {
            b_init[, k-1] <- coef(fit_k)
          }
        }
        if (regularization == "rhs") {
          init_list$z_b <- b_init
          init_list$w_lambda_b <- matrix(1, nrow(b_init), ncol(b_init))
          init_list$w_tau_b <- rep(1, ncol(b_init))
          init_list$c2_b <- rep(1, ncol(b_init))
        } else if (regularization == "ssp") {
          init_list$b_raw <- b_init
          init_list$r_b <- matrix(0.5, nrow(b_init), ncol(b_init))
          init_list$tau_b <- rep(1, ncol(b_init))
        } else {
          init_list$b <- b_init
        }
      }
    }
  }

  if (is.null(init_list$mu)) {
    if (multivariate) {
      mu_init <- matrix(0, K_mix, P_dim)
      for (p in 1:P_dim) {
        r_y <- range(Y_mat[, p])
        mu_init[, p] <- seq(r_y[1], r_y[2], length.out = K_mix + 2)[2:(K_mix + 1)]
      }
      init_list$mu <- mu_init
    } else {
      r_y <- range(Y_mat)
      init_list$mu <- seq(r_y[1], r_y[2], length.out = K_mix + 2)[2:(K_mix + 1)]
    }
  }

  if (!is.null(Y_mat)) {
     init_list$sigma <- if (is_sigma_equal) apply(Y_mat, 2, sd) else matrix(apply(Y_mat, 2, sd), K_mix, P_dim, byrow = TRUE)
  }

  view_order <- if (has_cov_prob) c("b", "prob_mean", "mu", "sigma") else c("prob_mean", "mu", "sigma")
  if (multivariate && !is_diag) view_order <- c(view_order, "corr")

  mdl <- rtmb_model(data_list, mdl_code, par_names = v_names, init = init_list, fixed = fixed, view = view_order)
  mdl$type <- "mixture"
  mdl$extra <- list(
    source = "wrapper",
    prior_type = prior$type,
    marginal = "mu"
  )
  return(mdl)
}

#' Convert Wide Data to Long Format
#'
#' @description
#' A highly intuitive wrapper around \code{stats::reshape} designed for 
#' psychological research. It converts data from wide format to long format
#' by identifying within-subjects factors.
#'
#' @param data A data frame in wide format.
#' @param within The columns to gather into long format. Can be:
#'   \itemize{
#'     \item A character vector of column names.
#'     \item A range string like \code{"time1:time4"}.
#'     \item A prefix string like \code{"time"} (matches all columns starting with "time").
#'   }
#' @param label The name for the new column that will contain the level names (e.g., "Time"). 
#'   If NULL, it defaults to the prefix used in \code{within} or "Condition".
#' @param value The name for the new column that will contain the measurement values. 
#'   Default is "Value".
#' @param id The identifier columns that should be repeated for each row. 
#'   If NULL (default), all columns NOT specified in \code{within} are treated as IDs.
#' @return A data frame in long format.
#' @export
to_long <- function(data, within = NULL, label = NULL, value = "Value", id = NULL) {
  if (is.null(within)) {
    stop("Argument 'within' is required. Specify columns to gather (e.g., 'time1:time4' or 'time').")
  }
  
  all_names <- names(data)
  target_cols <- NULL
  inferred_label <- "Condition"

  # 1. Handle 'within' specification
  if (is.character(within) && length(within) == 1) {
    if (grepl(":", within)) {
      # Range notation: "time1:time4"
      parts <- strsplit(within, ":")[[1]]
      start_idx <- which(all_names == parts[1])
      end_idx <- which(all_names == parts[2])
      if (length(start_idx) == 1 && length(end_idx) == 1) {
        target_cols <- all_names[start_idx:end_idx]
      } else {
        stop(sprintf("Range columns '%s' or '%s' not found.", parts[1], parts[2]))
      }
    } else if (within %in% all_names) {
      # Single column name
      target_cols <- within
      inferred_label <- within
    } else {
      # Prefix matching: "time" -> time1, time2, etc.
      target_cols <- all_names[grepl(paste0("^", within), all_names)]
      if (length(target_cols) > 0) {
        inferred_label <- within
      } else {
        stop(sprintf("No columns found starting with prefix '%s'.", within))
      }
    }
  } else {
    # Vector of names or indices
    if (is.numeric(within)) {
      target_cols <- all_names[within]
    } else {
      target_cols <- within
    }
  }
  
  if (is.null(target_cols) || length(target_cols) == 0) {
    stop("Could not identify columns to gather based on 'within' specification.")
  }

  # 2. Determine label column name
  if (is.null(label)) {
    label <- inferred_label
  }

  # 3. Determine ID columns (everything not in target_cols)
  if (is.null(id)) {
    id <- setdiff(all_names, target_cols)
  }

  # 4. Perform reshape
  data_res <- as.data.frame(data)
  
  # stats::reshape requires a list of varying column names
  res <- stats::reshape(data_res, 
                        direction = "long", 
                        varying = list(target_cols), 
                        v.names = value, 
                        timevar = label, 
                        times = target_cols, 
                        idvar = id)
  
  # Sort by ID and label for readability
  res <- res[do.call(order, res[c(id, label)]), ]
  rownames(res) <- NULL
  
  # Ensure the label column is a factor with levels in original column order
  res[[label]] <- factor(res[[label]], levels = target_cols)
  
  return(res)
}

#' Convert Long Data to Wide Format
#'
#' @description
#' A user-friendly wrapper around \code{stats::reshape} to convert data from
#' long format back to wide format.
#'
#' @param data A data frame in long format.
#' @param within The name of the column containing within-subjects factor levels (e.g., "Condition").
#' @param value The name of the column containing measurement values (e.g., "Value").
#' @param id The identifier columns (e.g., "Subject", "Group"). If NULL, 
#'   inferred as all columns except \code{within} and \code{value}.
#' @return A data frame in wide format.
#' @export
to_wide <- function(data, within = "Condition", value = "Value", id = NULL) {
  df <- as.data.frame(data)
  if (is.null(id)) {
    id <- setdiff(names(df), c(within, value))
  }

  res <- stats::reshape(df, 
                        direction = "wide", 
                        idvar = id, 
                        timevar = within, 
                        v.names = value)
  
  # Clean up column names (remove the 'value.' prefix added by reshape)
  new_names <- names(res)
  prefix <- paste0(value, ".")
  idx_to_change <- grepl(paste0("^", prefix), new_names)
  new_names[idx_to_change] <- gsub(paste0("^", prefix), "", new_names[idx_to_change])
  names(res) <- new_names
  
  rownames(res) <- NULL
  return(res)
}

#' Parameter Types and Constraints in RTMB Models
#'
#' @description
#' When declaring parameters in the \code{parameters} block using \code{Dim()},
#' you can specify a \code{type} to automatically apply structural constraints.
#' These constraints ensure that parameters remain within their valid mathematical
#' space during estimation.
#'
#' @details
#' \strong{Standard Types:}
#' \itemize{
#'   \item \code{"vector"}, \code{"matrix"}, \code{"array"}: Standard unconstrained containers.
#'   Use \code{lower} and \code{upper} for element-wise bounds.
#' }
#'
#' \strong{Ordering Constraints:}
#' \itemize{
#'   \item \code{"ordered"}: A vector where $x_1 < x_2 < \dots < x_K$.
#'   \item \code{"positive_ordered"}: A vector where $0 < x_1 < x_2 < \dots < x_K$.
#' }
#'
#' \strong{Constrained Vectors:}
#' \itemize{
#'   \item \code{"simplex"}: A vector where all elements are >= 0 and sum(x) = 1.
#'   \item \code{"centered"}: A vector of length K where sum(x) = 0. (Estimated using K-1 degrees of freedom).
#' }
#'
#' \strong{Correlation and Covariance Matrices:}
#' \itemize{
#'   \item \code{"corr_matrix"}: A symmetric, positive-definite correlation matrix (diagonal elements are 1).
#'   \item \code{"cov_matrix"}: A symmetric, positive-definite covariance matrix.
#'   \item \code{"CF_corr"}: The Cholesky factor of a correlation matrix.
#'   \item \code{"CF_cov"}: The Cholesky factor of a covariance matrix (diagonal elements are positive).
#' }
#'
#' \strong{Special Structural Matrices:}
#' \itemize{
#'   \item \code{"lower_tri"}: A lower-triangular matrix.
#'   \item \code{"positive_lower_tri"}: A lower-triangular matrix with positive diagonal elements.
#'   \item \code{"centered_matrix"}: A matrix where each column sums to zero.
#'   \item \code{"centered_tri"}: A triangular matrix with column-wise sum-to-zero constraints (often used for identification in factor analysis).
#' }
#'
#' @name parameter_types
#' @seealso \code{\link{rtmb_code}}, \code{\link{math_functions}}, \code{\link{distributions}}
NULL

#' Define parameter dimensions and types
#'
#' @param dim Dimensions of the parameter.
#' @param type Type of the parameter.
#' @param lower Lower bound.
#' @param upper Upper bound.
#' @param random Logical; whether it is a random effect.
#'
#' @export
Dim <- function(dim = 1, type = NULL, lower = NULL, upper = NULL, random = FALSE) {
  # 1. Basic Type Handling
  if (is.null(type)) {
    if (length(dim) == 1) type <- "vector"
    else if (length(dim) == 2) type <- "matrix"
    else type <- "array"
  }

  bounds_type <- "none"
  if (type %in% c("ordered", "positive_ordered", "simplex", "corr_matrix",
                  "cov_matrix", "CF_corr", "CF_cov", "centered",
                  "centered_matrix", "lower_tri", "lower_tri_stz",
                  "centered_tri", "positive_centered_tri",
                  "positive_lower_tri")) {
    bounds_type <- type
  } else if (!is.null(lower) && is.null(upper)) bounds_type <- "lower"
  else if (is.null(lower) && !is.null(upper)) bounds_type <- "upper"
  else if (!is.null(lower) && !is.null(upper)) bounds_type <- "interval"

  # 2. Multi-dimensional recursion (K x P x P convention)
  if (length(dim) > 2 && bounds_type %in% c("corr_matrix", "CF_corr", "cov_matrix", "CF_cov",
                                          "lower_tri", "lower_tri_stz", "positive_lower_tri",
                                          "centered_matrix", "centered_tri", "positive_centered_tri")) {
    K <- dim[1]
    sub_dim <- dim[-1]
    sub_p <- Dim(dim = sub_dim, type = type, lower = lower, upper = upper, random = random)

    return(list(
      dim        = dim,
      length     = prod(dim),
      unc_length = sub_p$unc_length * K,
      type       = type,
      bounds     = bounds_type,
      lower      = lower,
      upper      = upper,
      random     = random
    ))
  }

  # 3. Standard 1D/2D Calculation
  calc_length <- prod(dim)
  calc_unc_length <- calc_length

  if (bounds_type %in% c("corr_matrix", "CF_corr")) {
    R <- dim[1]
    C <- if (length(dim) > 1) dim[2] else dim[1]
    if (bounds_type == "corr_matrix") {
      if (R != C) stop(sprintf("[Parameter definition error] Variable specified as type = 'corr_matrix', but it does not have square matrix dimensions (Specified dimension: %s).", paste(dim, collapse = " x ")))
      calc_unc_length <- R * (R - 1) / 2
    } else {
      calc_unc_length <- 0
      if (R >= 2) {
        for (i in 2:R) {
          calc_unc_length <- calc_unc_length + min(i - 1, C - 1)
        }
      }
    }
  } else if (bounds_type %in% c("cov_matrix", "CF_cov")) {
    R <- dim[1]
    C <- if (length(dim) > 1) dim[2] else dim[1]
    if (bounds_type == "cov_matrix") {
      if (R != C) stop(sprintf("[Parameter definition error] Variable specified as type = 'cov_matrix', but it does not have square matrix dimensions (Specified dimension: %s).", paste(dim, collapse = " x ")))
      calc_unc_length <- R * (R + 1) / 2
    } else {
      calc_unc_length <- 0
      for (i in 1:R) {
        calc_unc_length <- calc_unc_length + min(i, C)
      }
    }
  } else if (bounds_type %in% c("simplex", "centered")) {
    calc_unc_length <- calc_length - 1
  } else if (bounds_type == "centered_matrix") {
    R <- dim[1]
    C <- dim[2]
    calc_unc_length <- (R - 1) * C
  } else if (bounds_type %in% c("centered_tri", "positive_centered_tri")) {
    R <- dim[1]
    C <- dim[2]
    calc_unc_length <- 0
    for (d in 1:C) {
      if (d <= R - 1) calc_unc_length <- calc_unc_length + (R - d)
    }
  } else if (bounds_type %in% c("lower_tri", "lower_tri_stz", "positive_lower_tri")) {
    R <- dim[1]
    C <- if (length(dim) > 1) dim[2] else dim[1]
    if (R >= C) {
      calc_unc_length <- C * (C + 1) / 2 + (R - C) * C
    } else {
      calc_unc_length <- R * (R + 1) / 2
    }
    if (bounds_type == "lower_tri_stz") calc_unc_length <- calc_unc_length - 1
  }

  list(
    dim        = dim,
    length     = calc_length,
    unc_length = calc_unc_length,
    type       = type,
    bounds     = bounds_type,
    lower      = lower,
    upper      = upper,
    random     = random
  )
}


generate_flat_names <- function(base_name, dims, names_def = NULL) {
  len <- prod(dims)
  if (is.null(dims)) dims <- len

  if (len == 1) {
    if (!is.null(names_def) && length(names_def) == 1) {
      return(paste0(base_name, "[", names_def, "]"))
    }
    return(base_name)
  }

  if (length(dims) > 1) {
    if (!is.null(names_def)) {
      if (is.list(names_def) && length(names_def) == length(dims)) {
        # When names are specified as a list per dimension
        grid <- do.call(expand.grid, c(names_def, list(stringsAsFactors = FALSE)))
        return(paste0(base_name, "[", apply(grid, 1, paste, collapse = ","), "]"))
      } else if (is.atomic(names_def) && length(names_def) == dims[1]) {
        # When names are specified as a vector
        if (length(dims) == 2 && dims[1] == dims[2]) {
          # Square matrix (e.g., CF_corr): Apply to both rows and columns
          grid <- expand.grid(names_def, names_def, stringsAsFactors = FALSE)
        } else {
          # Non-square matrix: Apply only to rows
          dim_list <- lapply(dims, seq_len)
          dim_list[[1]] <- names_def
          grid <- do.call(expand.grid, dim_list)
        }
        return(paste0(base_name, "[", apply(grid, 1, paste, collapse = ","), "]"))
      }
    }
    # When names are not specified, or sizes do not match
    grid <- do.call(expand.grid, lapply(dims, seq_len))
    return(paste0(base_name, "[", apply(grid, 1, paste, collapse = ","), "]"))
  } else {
    # For 1D vectors
    if (!is.null(names_def) && length(names_def) == len) {
      return(paste0(base_name, "[", names_def, "]"))
    }
    return(paste0(base_name, "[", seq_len(len), "]"))
  }
}



# Function to restore a flat vector in constrained space to a list
constrained_vector_to_list <- function(vec, par_list) {
  res <- list()
  idx <- 1
  for (name in names(par_list)) {
    p <- par_list[[name]]
    len <- p$length
    val <- vec[idx:(idx + len - 1)]

    if (length(p$dim) > 1 && !(p$bounds %in% c("corr_matrix", "CF_corr", "cov_matrix", "CF_cov", "lower_tri", "lower_tri_stz", "positive_lower_tri"))) {
      dim(val) <- p$dim
    } else if (p$bounds %in% c("corr_matrix", "CF_corr", "cov_matrix", "CF_cov", "lower_tri", "lower_tri_stz", "positive_lower_tri")) {
      dim(val) <- p$dim
    }

    res[[name]] <- val
    idx <- idx + len
  }
  return(res)
}

# Function to restore a flat vector in unconstrained space to a list
# Now handles parameter mapping (fixing elements) by skipping NA indices in map factors.
unconstrained_vector_to_list <- function(vec, par_list, map = NULL, original_list = NULL) {
  res <- if (!is.null(original_list)) original_list else list()
  idx <- 1
  
  for (name in names(par_list)) {
    p <- par_list[[name]]
    
    # If this parameter is not in original_list, initialize it
    if (is.null(res[[name]])) {
      res[[name]] <- if (p$unc_length > 0) rep(0, p$unc_length) else numeric(0)
    }

    # Handle mapping
    m <- if (!is.null(map[[name]])) map[[name]] else NULL
    
    if (is.null(m)) {
      # No mapping: take all elements
      len <- p$unc_length
      if (len > 0) {
        if (idx + len - 1 <= length(vec)) {
          res[[name]] <- vec[idx:(idx + len - 1)]
          idx <- idx + len
        }
      }
    } else {
      # Mapping exists: only take elements that are NOT NA in the factor
      m_vec <- as.integer(m)
      active_indices <- which(!is.na(m_vec))
      
      # Important: TMB maps multiple elements to the same parameter if indices match.
      # Here we assume 1:1 mapping or unique indices for simplicity in summary.
      unique_active <- unique(na.omit(m_vec))
      len <- length(unique_active)
      
      if (len > 0) {
        if (idx + len - 1 <= length(vec)) {
          active_vals <- vec[idx:(idx + len - 1)]
          # Map these back to the unconstrained parameter list
          for (i in seq_along(unique_active)) {
            target_indices <- which(m_vec == unique_active[i])
            res[[name]][target_indices] <- active_vals[i]
          }
          idx <- idx + len
        }
      }
    }
    
    # Restore matrix dimensions if applicable
    if (length(p$dim) > 1 && length(res[[name]]) == prod(p$dim)) {
      dim(res[[name]]) <- p$dim
    }
  }
  return(res)
}

# Convert to unconstrained space
to_unconstrained <- function(para_orig_list, par_list) {
  para_unc <- list()
  for (name in names(par_list)) {
    p <- par_list[[name]]
    val_orig <- para_orig_list[[name]]
    b_type <- p$bounds

    if (b_type == "lower") {
      para_unc[[name]] <- log(val_orig - p$lower)
    } else if (b_type == "upper") {
      para_unc[[name]] <- log(p$upper - val_orig)
    } else if (b_type == "interval") {
      prob <- (val_orig - p$lower) / (p$upper - p$lower)
      para_unc[[name]] <- log(prob / (1 - prob))
    } else if (b_type == "ordered") {
      if (is.matrix(val_orig)) {
        y <- val_orig
        y[, 1] <- val_orig[, 1]
        if (ncol(val_orig) > 1) {
          for (k in 2:ncol(val_orig)) {
            y[, k] <- log(val_orig[, k] - val_orig[, k - 1])
          }
        }
        para_unc[[name]] <- y
      } else {
        y <- numeric(p$length)
        y[1] <- val_orig[1]
        if (p$length > 1) {
          for (i in 2:p$length) {
            y[i] <- log(val_orig[i] - val_orig[i-1])
          }
        }
        para_unc[[name]] <- y
      }
    } else if (b_type == "positive_ordered") {
      if (is.matrix(val_orig)) {
        y <- val_orig
        y[, 1] <- log(val_orig[, 1])
        if (ncol(val_orig) > 1) {
          for (k in 2:ncol(val_orig)) {
            y[, k] <- log(val_orig[, k] - val_orig[, k - 1])
          }
        }
        para_unc[[name]] <- y
      } else {
        y <- numeric(p$length)
        y[1] <- log(val_orig[1])
        if (p$length > 1) {
          for (i in 2:p$length) {
            y[i] <- log(val_orig[i] - val_orig[i-1])
          }
        }
        para_unc[[name]] <- y
      }
    } else if (b_type == "simplex") {
      K <- p$length
      y <- numeric(K - 1)
      stick_len <- 1
      for (k in 1:(K - 1)) {
        z_k <- if (stick_len > 1e-10) val_orig[k] / stick_len else 0
        z_k <- min(max(z_k, 1e-10), 1 - 1e-10)
        y[k] <- log(z_k / (1 - z_k)) - log(1 / (K - k))
        stick_len <- stick_len - val_orig[k]
      }
      para_unc[[name]] <- y
    } else if (b_type == "centered") {
      K <- p$length
      Q <- stz_basis(K)
      para_unc[[name]] <- as.numeric(crossprod(Q, val_orig))

    } else if (b_type == "centered_matrix") {
      R <- p$dim[1]
      C <- p$dim[2]
      Q <- stz_basis(R)
      Y <- crossprod(Q, val_orig)
      para_unc[[name]] <- as.numeric(Y)

    } else if (b_type == "centered_tri") {
      R <- p$dim[1]
      C <- p$dim[2]
      y <- numeric(p$unc_length)
      idx <- 1
      for (d in 1:min(C, R - 1)) {
        x_d <- val_orig[d:R, d]
        n_d <- length(x_d)
        Q_d <- stz_basis(n_d)
        z_d <- as.numeric(crossprod(Q_d, x_d))
        y[idx:(idx + n_d - 2)] <- z_d
        idx <- idx + n_d - 1
      }
      para_unc[[name]] <- y

    } else if (b_type == "positive_centered_tri") {
      R <- p$dim[1]
      C <- p$dim[2]
      y <- numeric(p$unc_length)
      idx <- 1
      for (d in 1:min(C, R - 1)) {
        x_d <- val_orig[d:R, d]
        n_d <- length(x_d)
        u1 <- log(x_d[1])
        y[idx] <- u1
        if (n_d > 2) {
          v <- x_d[2:n_d] + x_d[1] / (n_d - 1)
          Q_d <- stz_basis(n_d - 1)
          z_d <- as.numeric(crossprod(Q_d, v))
          y[(idx + 1):(idx + n_d - 2)] <- z_d
        }
        idx <- idx + n_d - 1
      }
      para_unc[[name]] <- y

    } else if (b_type %in% c("corr_matrix", "CF_corr")) {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }
      z_all <- numeric(p$unc_length)
      idx <- 1
      for (k in 1:K) {
        # --- Logic for restoring parameter matrix dimensions ---
        # Restores dimensions using info saved during model construction, as RTMB treats them as vectors internally.
        val_slice <- if (K > 1) {
          if (length(p$dim) == 3) matrix(val_orig[k, , ], R, C) else val_orig
        } else {
          val_orig
        }
        L <- if (b_type == "corr_matrix") t(chol(val_slice + diag(1e-8, R))) else val_slice
        if (R >= 2) {
          for (i in 2:R) {
            prod_term <- 1
            max_j <- min(i - 1, C - 1)
            if (max_j > 0) {
              for (j in 1:max_j) {
                z_val <- L[i, j] / sqrt(prod_term)
                z_val <- min(max(z_val, -0.999999), 0.999999)
                z_all[idx] <- z_val
                prod_term <- prod_term * (1 - z_val^2)
                idx <- idx + 1
              }
            }
          }
        }
      }
      para_unc[[name]] <- atanh(z_all)

    } else if (b_type %in% c("cov_matrix", "CF_cov")) {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }
      y <- numeric(p$unc_length)
      idx <- 1
      for (k in 1:K) {
        val_slice <- if (K > 1) {
          if (length(p$dim) == 3) matrix(val_orig[k, , ], R, C) else val_orig
        } else {
          val_orig
        }
        L <- if (b_type == "cov_matrix") t(chol(val_slice + diag(1e-8, R))) else val_slice
        for (i in 1:R) {
          max_j <- min(i, C)
          for (j in 1:max_j) {
            if (i == j) {
              y[idx] <- log(L[i, j])
            } else {
              y[idx] <- L[i, j]
            }
            idx <- idx + 1
          }
        }
      }
      para_unc[[name]] <- y

    } else if (b_type %in% c("lower_tri", "positive_lower_tri")) {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }
      y <- numeric(p$unc_length)
      idx <- 1
      for (k in 1:K) {
        val_slice <- if (K > 1) {
          if (length(p$dim) == 3) matrix(val_orig[k, , ], R, C) else val_orig
        } else {
          val_orig
        }
        for (i in 1:R) {
          max_j <- min(i, C)
          for (j in 1:max_j) {
            y[idx] <- val_slice[i, j]
            idx <- idx + 1
          }
        }
      }
      para_unc[[name]] <- y

    } else if (b_type == "lower_tri_stz") {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }
      y <- numeric(p$unc_length)
      idx <- 1
      for (k in 1:K) {
        val_slice <- if (K > 1) {
          if (length(p$dim) == 3) matrix(val_orig[k, , ], R, C) else val_orig
        } else {
          val_orig
        }
        last_i <- R
        last_j <- min(R, C)
        for (i in 1:R) {
          max_j <- min(i, C)
          for (j in 1:max_j) {
            if (i == last_i && j == last_j) break
            y[idx] <- val_slice[i, j]
            idx <- idx + 1
          }
        }
      }
      para_unc[[name]] <- y
    } else {
      para_unc[[name]] <- as.numeric(val_orig)
    }
  }
  return(para_unc)
}

to_constrained <- function(para_unc_list, par_list) {
  para <- para_unc_list
  for (name in names(par_list)) {
    p <- par_list[[name]]
    val_unc <- para_unc_list[[name]]
    b_type <- p$bounds

    ad_zero <- val_unc[1] * 0

    if (b_type == "lower") {
      para[[name]] <- p$lower + exp(val_unc)
    } else if (b_type == "upper") {
      para[[name]] <- p$upper - exp(val_unc)
    } else if (b_type == "interval") {
      prob <- plogis(val_unc)
      para[[name]] <- p$lower + (p$upper - p$lower) * prob
    } else if (b_type == "ordered") {
      if (p$length > 1) {
        if (is.matrix(val_unc)) {
          res <- val_unc
          res[, 1] <- val_unc[, 1]
          if (ncol(val_unc) > 1) {
            for (k in 2:ncol(val_unc)) {
              res[, k] <- res[, k - 1] + exp(val_unc[, k])
            }
          }
          para[[name]] <- res
        } else {
          para[[name]] <- val_unc[1] + cumsum(c(ad_zero, exp(val_unc[-1])))
        }
      } else {
        para[[name]] <- val_unc
      }
    } else if (b_type == "positive_ordered") {
      if (is.matrix(val_unc)) {
        res <- val_unc
        res[, 1] <- exp(val_unc[, 1])
        if (ncol(val_unc) > 1) {
          for (k in 2:ncol(val_unc)) {
            res[, k] <- res[, k - 1] + exp(val_unc[, k])
          }
        }
        para[[name]] <- res
      } else {
        para[[name]] <- cumsum(exp(val_unc))
      }
    } else if (b_type == "simplex") {
      K <- p$length
      z <- 1 / (1 + exp(-(val_unc - log(1 / (K - seq_len(K - 1))))))
      ad_one <- ad_zero + 1
      rem_prev <- c(ad_one, cumprod(1 - z)[- (K - 1)])
      x_first <- rem_prev * z
      x_last <- cumprod(1 - z)[K - 1]
      para[[name]] <- c(x_first, x_last)

    } else if (b_type == "centered") {
      K <- p$length
      Q <- stz_basis(K)
      para[[name]] <- as.numeric(Q %*% val_unc)

    } else if (b_type == "centered_matrix") {
      R <- p$dim[1]; C <- p$dim[2]
      Q <- stz_basis(R)
      mat_unc <- matrix(val_unc, nrow = R - 1, ncol = C)
      para[[name]] <- Q %*% mat_unc

    } else if (b_type == "centered_tri") {
      R <- p$dim[1]; C <- p$dim[2]
      L <- matrix(ad_zero, nrow = R, ncol = C)
      idx <- 1
      for (d in 1:min(C, R - 1)) {
        n_d <- R - d + 1
        z_d <- val_unc[idx:(idx + n_d - 2)]
        Q_d <- stz_basis(n_d)
        x_d <- as.numeric(Q_d %*% z_d)
        L[d:R, d] <- x_d
        idx <- idx + n_d - 1
      }
      para[[name]] <- L

    } else if (b_type == "positive_centered_tri") {
      R <- p$dim[1]; C <- p$dim[2]
      L <- matrix(ad_zero, nrow = R, ncol = C)
      idx <- 1
      for (d in 1:min(C, R - 1)) {
        n_d <- R - d + 1
        x_d <- rep(ad_zero, n_d)
        u1 <- val_unc[idx]
        x1 <- exp(u1)
        x_d[1] <- x1
        if (n_d > 2) {
          z_d <- val_unc[(idx + 1):(idx + n_d - 2)]
          Q_d <- stz_basis(n_d - 1)
          v <- as.numeric(Q_d %*% z_d)
          x_d[2:n_d] <- v - x1 / (n_d - 1)
        } else if (n_d == 2) {
          x_d[2] <- -x1
        }
        L[d:R, d] <- x_d
        idx <- idx + n_d - 1
      }
      para[[name]] <- L

    } else if (b_type %in% c("corr_matrix", "CF_corr")) {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }
      z_all <- tanh(val_unc)

      if (K > 1) {
        res_list <- vector("list", K)
        unc_per_slice <- length(val_unc) / K
        for (k in 1:K) {
          z <- z_all[((k-1)*unc_per_slice + 1):(k*unc_per_slice)]
          L <- matrix(ad_zero, nrow = R, ncol = C)
          L[1, 1] <- 1
          idx <- 1
          if (R >= 2) {
            for (i in 2:R) {
              prod_term <- 1
              max_j <- min(i - 1, C - 1)
              if (max_j > 0) {
                for (j in 1:max_j) {
                  L[i, j] <- z[idx] * sqrt(prod_term)
                  prod_term <- prod_term * (1 - z[idx]^2)
                  idx <- idx + 1
                }
              }
              last_j <- min(i, C)
              if (last_j > max_j) {
                L[i, last_j] <- sqrt(abs(prod_term))
              }
            }
          }
          if (b_type == "corr_matrix") {
            res_list[[k]] <- as.vector(L %*% t(L))
          } else {
            res_list[[k]] <- as.vector(L)
          }
        }
        # Fix: Shuffle indices to match R's array memory layout and restore K x R x C
        res_array_flat <- do.call(c, res_list)
        idx_base <- array(1:(K * R * C), dim = c(R, C, K))
        idx_perm <- as.vector(aperm(idx_base, c(3, 1, 2)))

        res_array <- res_array_flat[idx_perm]
        dim(res_array) <- p$dim
        para[[name]] <- res_array
      } else {
        z <- z_all
        L <- matrix(ad_zero, nrow = R, ncol = C)
        L[1, 1] <- 1
        idx <- 1
        if (R >= 2) {
          for (i in 2:R) {
            prod_term <- 1
            max_j <- min(i - 1, C - 1)
            if (max_j > 0) {
              for (j in 1:max_j) {
                L[i, j] <- z[idx] * sqrt(prod_term)
                prod_term <- prod_term * (1 - z[idx]^2)
                idx <- idx + 1
              }
            }
            last_j <- min(i, C)
            if (last_j > max_j) {
              L[i, last_j] <- sqrt(abs(prod_term))
            }
          }
        }
        if (b_type == "corr_matrix") para[[name]] <- L %*% t(L)
        else para[[name]] <- L
      }

    } else if (b_type %in% c("cov_matrix", "CF_cov")) {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }

      if (K > 1) {
        res_list <- vector("list", K)
        unc_per_slice <- length(val_unc) / K
        for (k in 1:K) {
          v_u <- val_unc[((k-1)*unc_per_slice + 1):(k*unc_per_slice)]
          L <- matrix(ad_zero, nrow = R, ncol = C)
          idx <- 1
          for (i in 1:R) {
            max_j <- min(i, C)
            for (j in 1:max_j) {
              if (i == j) L[i, j] <- exp(v_u[idx])
              else L[i, j] <- v_u[idx]
              idx <- idx + 1
            }
          }
          if (b_type == "cov_matrix") {
            res_list[[k]] <- as.vector(L %*% t(L))
          } else {
            res_list[[k]] <- as.vector(L)
          }
        }
        res_array_flat <- do.call(c, res_list)
        idx_base <- array(1:(K * R * C), dim = c(R, C, K))
        idx_perm <- as.vector(aperm(idx_base, c(3, 1, 2)))

        res_array <- res_array_flat[idx_perm]
        dim(res_array) <- p$dim
        para[[name]] <- res_array
      } else {
        L <- matrix(ad_zero, nrow = R, ncol = C)
        idx <- 1
        for (i in 1:R) {
          max_j <- min(i, C)
          for (j in 1:max_j) {
            if (i == j) L[i, j] <- exp(val_unc[idx])
            else L[i, j] <- val_unc[idx]
            idx <- idx + 1
          }
        }
        if (b_type == "cov_matrix") para[[name]] <- L %*% t(L)
        else para[[name]] <- L
      }

    } else if (b_type %in% c("lower_tri", "positive_lower_tri")) {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }

      if (K > 1) {
        res_list <- vector("list", K)
        unc_per_slice <- length(val_unc) / K
        for (k in 1:K) {
          v_u <- val_unc[((k-1)*unc_per_slice + 1):(k*unc_per_slice)]
          L <- matrix(ad_zero, nrow = R, ncol = C)
          idx <- 1
          for (i in 1:R) {
            max_j <- min(i, C)
            for (j in 1:max_j) {
              L[i, j] <- v_u[idx]
              idx <- idx + 1
            }
          }
          res_list[[k]] <- as.vector(L)
        }
        res_array_flat <- do.call(c, res_list)
        idx_base <- array(1:(K * R * C), dim = c(R, C, K))
        idx_perm <- as.vector(aperm(idx_base, c(3, 1, 2)))

        res_array <- res_array_flat[idx_perm]
        dim(res_array) <- p$dim
        para[[name]] <- res_array
      } else {
        L <- matrix(ad_zero, nrow = R, ncol = C)
        idx <- 1
        for (i in 1:R) {
          max_j <- min(i, C)
          for (j in 1:max_j) {
            L[i, j] <- val_unc[idx]
            idx <- idx + 1
          }
        }
        para[[name]] <- L
      }

    } else if (b_type == "lower_tri_stz") {
      if (length(p$dim) == 3) {
        K <- p$dim[1]; R <- p$dim[2]; C <- p$dim[3]
      } else {
        R <- p$dim[1]; C <- if (length(p$dim) > 1) p$dim[2] else p$dim[1]; K <- 1
      }

      if (K > 1) {
        res_list <- vector("list", K)
        unc_per_slice <- length(val_unc) / K
        for (k in 1:K) {
          v_u <- val_unc[((k-1)*unc_per_slice + 1):(k*unc_per_slice)]
          L <- matrix(ad_zero, nrow = R, ncol = C)
          idx <- 1
          sum_val <- 0
          last_i <- R
          last_j <- min(R, C)
          for (i in 1:R) {
            max_j <- min(i, C)
            for (j in 1:max_j) {
              if (i == last_i && j == last_j) break
              L[i, j] <- v_u[idx]
              sum_val <- sum_val + v_u[idx]
              idx <- idx + 1
            }
          }
          L[last_i, last_j] <- -sum_val
          res_list[[k]] <- as.vector(L)
        }
        res_array_flat <- do.call(c, res_list)
        idx_base <- array(1:(K * R * C), dim = c(R, C, K))
        idx_perm <- as.vector(aperm(idx_base, c(3, 1, 2)))

        res_array <- res_array_flat[idx_perm]
        dim(res_array) <- p$dim
        para[[name]] <- res_array
      } else {
        L <- matrix(ad_zero, nrow = R, ncol = C)
        idx <- 1
        sum_val <- 0
        last_i <- R
        last_j <- min(R, C)
        for (i in 1:R) {
          max_j <- min(i, C)
          for (j in 1:max_j) {
            if (i == last_i && j == last_j) break
            L[i, j] <- val_unc[idx]
            sum_val <- sum_val + val_unc[idx]
            idx <- idx + 1
          }
        }
        L[last_i, last_j] <- -sum_val
        para[[name]] <- L
      }
    }

    if (!(b_type %in% c("corr_matrix", "CF_corr", "cov_matrix", "CF_cov", "lower_tri", "lower_tri_stz", "positive_lower_tri")) && length(p$dim) > 1) {
      dim(para[[name]]) <- p$dim
    }
  }
  return(para)
}

# calc_log_jacobian
calc_log_jacobian <- function(para_unc_list, par_list, only_random = FALSE) {
  lj <- 0
  for (name in names(par_list)) {
    p <- par_list[[name]]
    val_unc <- para_unc_list[[name]]
    b_type <- p$bounds

    if (only_random && !isTRUE(p$random)) next

    if (b_type == "lower" || b_type == "upper") {
      lj <- lj + sum(val_unc)
    } else if (b_type == "interval") {
      lj <- lj + sum(log(p$upper - p$lower) - val_unc - 2 * log(1 + exp(-val_unc)))
    } else if (b_type == "ordered") {
      if (p$length > 1) {
        if (is.matrix(val_unc)) {
          lj <- lj + sum(val_unc[, -1])
        } else {
          lj <- lj + sum(val_unc[2:p$length])
        }
      }
    } else if (b_type == "positive_ordered") {
      lj <- lj + sum(val_unc)
    } else if (b_type == "simplex") {
      K <- p$length
      z <- 1 / (1 + exp(-(val_unc - log(1 / (K - seq_len(K - 1))))))
      lj <- lj + sum(log(z) + log(1 - z))
      if (K > 2) {
        for (k in 1:(K - 2)) {
          lj <- lj + (K - k - 1) * log(1 - z[k])
        }
      }
    } else if (b_type %in% c("centered", "centered_matrix", "centered_tri")) {
      diff_dim <- p$length - p$unc_length
      lj <- lj + (diff_dim / 2) * log(2 * pi)
    } else if (b_type == "positive_centered_tri") {
      diff_dim <- p$length - p$unc_length
      lj <- lj + (diff_dim / 2) * log(2 * pi)
      R <- p$dim[1]
      C <- p$dim[2]
      idx <- 1
      for (d in 1:min(C, R - 1)) {
        n_d <- R - d + 1
        u1 <- val_unc[idx]
        lj <- lj + u1 + 0.5 * log(n_d / (n_d - 1))
        idx <- idx + n_d - 1
      }
    } else if (b_type %in% c("corr_matrix", "CF_corr")) {
      K_rep <- if (length(p$dim) > 2) p$dim[1] else 1
      P_mat <- if (length(p$dim) > 2) p$dim[2] else p$dim[1]

      unc_per_slice <- length(val_unc) / K_rep
      for (k in 1:K_rep) {
        v_u <- val_unc[((k-1)*unc_per_slice + 1):(k*unc_per_slice)]
        lj_term <- 0
        idx_u <- 1
        for (i in 2:P_mat) {
          for (j in 1:(i - 1)) {
            # Fix: use the correct Jacobian for CF_corr and corr_matrix
            if (b_type == "corr_matrix") {
              lj_term <- lj_term - 2 * (P_mat - i + 1) * log(cosh(v_u[idx_u]))
            } else { # CF_corr
              lj_term <- lj_term - (i - j + 1) * log(cosh(v_u[idx_u]))
            }
            idx_u <- idx_u + 1
          }
        }
        lj <- lj + lj_term
      }
    } else if (b_type %in% c("cov_matrix", "CF_cov")) {
      K_rep <- if (length(p$dim) > 2) p$dim[1] else 1
      P_mat <- if (length(p$dim) > 2) p$dim[2] else p$dim[1]

      unc_per_slice <- length(val_unc) / K_rep
      for (k in 1:K_rep) {
        v_u <- val_unc[((k-1)*unc_per_slice + 1):(k*unc_per_slice)]
        idx_u <- 1
        for (i in 1:P_mat) {
          for (j in 1:i) {
            if (i == j) {
              if (b_type == "cov_matrix") lj <- lj + (P_mat - i + 2) * v_u[idx_u]
              else lj <- lj + v_u[idx_u]
            }
            idx_u <- idx_u + 1
          }
        }
      }
    } else if (b_type %in% c("lower_tri", "positive_lower_tri")) {
      K_rep <- if (length(p$dim) > 2) p$dim[1] else 1
      R_mat <- if (length(p$dim) > 2) p$dim[2] else p$dim[1]
      C_mat <- if (length(p$dim) > 2) p$dim[3] else (if (length(p$dim) > 1) p$dim[2] else p$dim[1])

      unc_per_slice <- length(val_unc) / K_rep
      for (k in 1:K_rep) {
        v_u <- val_unc[((k-1)*unc_per_slice + 1):(k*unc_per_slice)]
        idx_u <- 1
        for (i in 1:R_mat) {
          max_j <- min(i, C_mat)
          for (j in 1:max_j) {
            if (i == j && b_type == "positive_lower_tri") {
              lj <- lj + v_u[idx_u]
            }
            idx_u <- idx_u + 1
          }
        }
      }
    }
  }
  return(lj)
}
#' Generate Random Initial Values
#'
#' @description Generates random initial values for model parameters by drawing from a uniform distribution
#' on the unconstrained scale and then transforming them to the constrained scale.
#'
#' @param pl_full A list containing the full parameter structure, including the `init` field which serves as a template.
#' @param par_list A list defining the structure and constraints of the parameters, including `unc_length`.
#' @param range Numeric; the range for the uniform distribution `[-range, range]` used for generating unconstrained values. Default is 2.
#' @return A numeric vector of initial values where `NA` elements are replaced with randomly generated constrained values.
generate_random_init <- function(pl_full, par_list, range = 2) {
  unc_list <- list()
  for(name in names(par_list)) {
    unc_list[[name]] <- runif(par_list[[name]]$unc_length, min = -range, max = range)
  }

  con_list <- to_constrained(unc_list, par_list)

  # Group-wise NA checking instead of element-wise NA checking to preserve structures
  init_list <- constrained_vector_to_list(pl_full$init, par_list)
  for (name in names(par_list)) {
    # If ANY element in the parameter block is NA, we replace the ENTIRE block with random values
    if (any(is.na(init_list[[name]]))) {
      init_list[[name]] <- con_list[[name]]
    }
  }

  return(unlist(init_list, use.names = FALSE))
}

parse_parameters <- function(par_list, par_names = NULL) {
  P <- if (length(par_list) > 0) sum(vapply(par_list, function(x) x$length, numeric(1))) else 0

  init_vec    <- numeric(P)
  lower_vec   <- rep(NA, P)
  upper_vec   <- rep(NA, P)
  bounds_type <- character(P)
  flat_names  <- character(P)

  idx <- 1
  for (name in names(par_list)) {
    p <- par_list[[name]]
    len <- p$length
    end_idx <- idx + len - 1
    names_def <- if (!is.null(par_names)) par_names[[name]] else NULL
    flat_names[idx:end_idx] <- generate_flat_names(name, p$dim, names_def)

    if (!is.null(p$lower)) lower_vec[idx:end_idx] <- p$lower
    if (!is.null(p$upper)) upper_vec[idx:end_idx] <- p$upper

    b_type <- "none"
    if (!is.null(p$lower) && is.null(p$upper)) b_type <- "lower"
    if (is.null(p$lower) && !is.null(p$upper)) b_type <- "upper"
    if (!is.null(p$lower) && !is.null(p$upper)) b_type <- "interval"
    bounds_type[idx:end_idx] <- b_type

    if (!is.null(p$init)) {
      init_vec[idx:end_idx] <- p$init
    } else {
      init_vec[idx:end_idx] <- NA
    }

    idx <- idx + len
  }

  return(list(
    names  = flat_names,
    init   = init_vec,
    lower  = lower_vec,
    upper  = upper_vec,
    bounds = bounds_type,
    idx    = list(
      lower = which(bounds_type == "lower"),
      upper = which(bounds_type == "upper"),
      intv  = which(bounds_type == "interval")
    )
  ))
}

unpack_parameters <- function(para, par_list) {
  res <- list()
  idx <- 1
  for (name in names(par_list)) {
    p <- par_list[[name]]
    len <- p$length
    val <- para[idx:(idx + len - 1)]
    if (length(p$dim) > 1) {
      dim(val) <- p$dim
    }
    res[[name]] <- val
    idx <- idx + len
  }
  return(res)
}

#' Select parameters from a list by name or index
#' @param all_list A named list of parameters.
#' @param pars A character or numeric vector.
#' @return A subset of the list.
#' @keywords internal
select_parameters <- function(all_list, pars) {
  if (is.null(pars)) return(all_list)
  if (identical(pars, "parameters") || identical(pars, "all")) return(all_list)
  
  # 1. Numeric indexing (positive or negative)
  if (is.numeric(pars)) {
    # We use a trick to handle both positive and negative indices correctly in one go
    # However, base R already handles this: list[pars]
    # But we want to ensure we don't return out-of-bounds NAs if possible
    res <- tryCatch(all_list[pars], error = function(e) NULL)
    if (is.null(res)) return(all_list[0]) # Return empty list if error
    return(res[names(res) != ""]) # Filter out any NA names that might appear from out-of-bounds positive indices
  }
  
  # 2. Character indexing
  if (is.character(pars)) {
    is_negative <- grepl("^-", pars)
    if (any(is_negative)) {
      if (!all(is_negative)) {
        stop("Cannot mix positive and negative parameter names in selection.")
      }
      exclude_names <- gsub("^-", "", pars)
      return(all_list[!(names(all_list) %in% exclude_names)])
    } else {
      # Standard positive selection
      return(all_list[names(all_list) %in% pars])
    }
  }
  
  return(all_list)
}

#' Mathematical and Matrix Utility Functions for RTMB Models
#'
#' @description
#' This page summarizes the utility functions provided to assist in writing
#' efficient and numerically stable model code within `rtmb_code`. These functions
#' are specifically designed to be compatible with RTMB's Automatic Differentiation (AD).
#'
#' @details
#' \strong{1. Link and Inverse Functions:}
#' \itemize{
#'   \item \code{logit(x)}: Computes the logit transformation \code{log(x/(1-x))}.
#'   \item \code{inv_logit(x)}: Computes the inverse logit (logistic) transformation \code{1/(1+exp(-x))}.
#' }
#'
#' \strong{2. Numerical Stability Utilities:}
#' These functions use specialized algorithms to prevent overflow/underflow in AD calculations.
#' \itemize{
#'   \item \code{log_sum_exp(x)}: Safely computes \code{log(sum(exp(x)))} using the log-sum-exp trick.
#'   \item \code{log1p_exp(x)}: Computes \code{log(1 + exp(x))} stably.
#'   \item \code{log1m_exp(x)}: Computes \code{log(1 - exp(x))} stably for \code{x < 0}.
#'   \item \code{log_softmax(x)}: Computes the log of the softmax function.
#'   \item \code{fabs(x)}: A smooth version of the absolute value function \code{abs(x)} to ensure differentiability at zero.
#' }
#'
#' \strong{3. Matrix and Vector Transformations:}
#' Used to convert unconstrained vectors into structured matrices (e.g., for factor analysis or identification constraints).
#' \itemize{
#'   \item \code{sum_to_zero(x)}: Transforms a vector of length K-1 into a vector of length K that sums to zero.
#'   \item \code{to_lower_tri(x, M, D)}: Fills a matrix of size M x D with elements from vector \code{x} in a lower-triangular fashion.
#'   \item \code{to_centered_matrix(x, R, C)}: Creates an R x C matrix where each column sums to zero.
#'   \item \code{to_centered_tri(x, R, C)}: Creates an R x C matrix with column-wise sum-to-zero constraints on the lower elements (useful for identification in factor analysis).
#' }
#'
#' \strong{4. Linear Algebra for AD:}
#' \itemize{
#'   \item \code{log_det_chol(L)}: Calculates the log-determinant of a covariance matrix from its Cholesky factor \code{L}.
#'   \item \code{quad_form_chol(x, L)}: Computes the quadratic form x^T Sigma^(-1) x using the Cholesky factor \code{L}.
#'   \item \code{distance(x, y)}: Computes the Euclidean distance between two vectors with a small epsilon for stability.
#' }
#' @name math_functions
#' @family utilities
#' @import RTMB
NULL

#' Inverse logit function
#'
#' @param x A numeric vector.
#' @return The inverse logit of x.
#' @export
inv_logit <- function(x) plogis(x)

#' Logit function
#'
#' @param x A numeric vector of probabilities.
#' @return The logit of x.
#' @export
logit <- function(x) qlogis(x)

#' Euclidean distance
#'
#' @param x A numeric vector.
#' @param y A numeric vector.
#' @param eps A small value added for numerical stability (default is 1e-8).
#' @return The Euclidean distance between x and y.
#' @export
distance <- function(x, y, eps = 1e-8) {
  sqrt(sum((x - y)^2) + eps)
}

#' Squared Euclidean distance
#'
#' @param x A numeric vector.
#' @param y A numeric vector.
#' @param eps A small value added for numerical stability (default is 1e-8).
#' @return The squared Euclidean distance between x and y.
#' @export
squared_distance <- function(x, y, eps = 1e-8) {
  sum((x - y)^2) + eps
}

#' Log-sum-exp function
#'
#' @param x A numeric vector or scalar.
#' @param y An optional numeric scalar.
#' @return The log of the sum of the exponentials.
#' @export
log_sum_exp <- function(x, y = NULL) {
  if (is.null(y)) {
    # Check if x is a matrix
    if (is.matrix(x)) {
      return(log_sum_exp_matrix(x))
    }
    max_val <- max(x)
    return(max_val + log(sum(exp(x - max_val))))
  } else {
    max_val <- (x + y + abs(x - y)) / 2
    return(max_val + log(exp(x - max_val) + exp(y - max_val)))
  }
}

#' Log-sum-exp function for matrices (row-wise)
#'
#' @param M A numeric matrix or advector matrix.
#' @return A numeric vector of row-wise log-sum-exp.
#' @export
log_sum_exp_matrix <- function(M) {
  # This function hides 'apply' from the BayesRTMB parser
  max_val <- apply(M, 1, max)
  return(max_val + log(rowSums(exp(M - max_val))))
}

#' Log of one minus x
#'
#' @param x A numeric vector.
#' @return The natural logarithm of 1 - x.
#' @export
log1m <- function(x) {
  log1p(-x)
}

#' Log of one plus exponential of x
#'
#' @param x A numeric vector.
#' @return The natural logarithm of 1 + exp(x).
#' @export
log1p_exp <- function(x) {
  max_val <- (x + sqrt(x^2 + 1e-10)) / 2
  return(max_val + log(exp(x - max_val) + exp(-max_val)))
}

#' Log of one minus exponential of x
#'
#' @param x A numeric vector.
#' @return The natural logarithm of 1 - exp(x).
#' @export
log1m_exp <- function(x) {
  # Stable implementation of log(1 - exp(x)) for x < 0
  ifelse(x > -0.6931472, log(-expm1(x)), log1p(-exp(x)))
}

#' Log mixture of two probabilities
#'
#' @param theta Mixing proportion (between 0 and 1).
#' @param lp1 Log-probability of the first component.
#' @param lp2 Log-probability of the second component.
#' @return The log-probability of the mixture.
#' @export
log_mix <- function(theta, lp1, lp2) {
  l1 <- log(theta) + lp1
  l2 <- log1p(-theta) + lp2
  return(log_sum_exp(l1, l2))
}

#' Softmax function
#'
#' @param x A numeric vector.
#' @return A numeric vector of softmax probabilities.
#' @export
softmax <- function(x) {
  max_x <- max(x)
  exp_x <- exp(x - max_x)
  return(exp_x / sum(exp_x))
}

#' Log-softmax function
#'
#' @param x A numeric vector.
#' @return A numeric vector of log-softmax probabilities.
#' @export
log_softmax <- function(x) {
  return(x - log_sum_exp(x))
}

#' Smooth absolute value function
#'
#' @param x A numeric vector.
#' @param eps A small value added for numerical stability (default is 1e-8).
#' @return The smooth absolute value of x.
#' @export
fabs <- function(x, eps = 1e-8) {
  sqrt(x^2 + eps)
}

#' Log determinant of a Cholesky factor
#'
#' @param L A lower triangular Cholesky factor matrix.
#' @return The log determinant of the corresponding covariance matrix.
#' @export
log_det_chol <- function(L) {
  return(2 * sum(log(diag(L))))
}

#' Quadratic form using a Cholesky factor
#'
#' @param x A numeric vector.
#' @param L A lower triangular Cholesky factor matrix.
#' @return The value of the quadratic form.
#' @export
quad_form_chol <- function(x, L) {
  z <- solve(L, x)
  return(sum(z^2))
}

#' Quadratic form with a diagonal matrix
#'
#' @param m A numeric matrix.
#' @param v A numeric vector representing the diagonal elements.
#' @return The value of the quadratic form.
#' @export
quad_form_diag <- function(m, v) {
  v_col <- matrix(as.vector(v), ncol = 1)
  return(m * (v_col %*% t(v_col)))
}
#' stz basis function
#'
#' @param K A numeric value
#' @return A transformation matrix
#' @export
stz_basis <- function(K) {
  Q <- matrix(0, nrow = K, ncol = K - 1)
  for (j in 1:(K - 1)) {
    Q[1:j, j] <- 1 / sqrt(j * (j + 1))
    Q[j + 1, j] <- -j / sqrt(j * (j + 1))
  }
  return(Q)
}

#' Sum-to-zero transformation
#'
#' Transforms a vector of length K-1 to a vector of length K that sums to zero
#' using an orthogonal basis matrix.
#'
#' @param x A numeric vector of length K-1.
#' @return A numeric vector of length K whose elements sum to zero.
#' @export
sum_to_zero <- function(x) {
  K <- length(x) + 1
  Q <- stz_basis(K)
  return(as.vector(Q %*% x))
}

#' Vector to lower triangular matrix (RTMB compatible)
#'
#' @param x A numeric or advector.
#' @param M Number of rows.
#' @param D Number of columns.
#' @return An M x D lower triangular matrix.
to_lower_tri <- function(x, M, D) {
  y <- matrix(x[1] * 0, nrow = M, ncol = D)
  y[lower.tri(y, diag = TRUE)] <- x
  return(y)
}
#' Vector to centered matrix (RTMB compatible)
#'
#' @param x A numeric vector of length (R-1) * C.
#' @param R Number of rows.
#' @param C Number of columns.
#' @return An R x C matrix where each column sums to zero.
#' @export
to_centered_matrix <- function(x, R, C) {
  mat_unc <- matrix(x[1] * 0, nrow = R - 1, ncol = C)
  mat_unc[] <- x
  Q <- stz_basis(R)
  return(Q %*% mat_unc)
}
#' Vector to centered triangular matrix (RTMB compatible)
#'
#' @param x A numeric vector of appropriate length.
#' @param R Number of rows.
#' @param C Number of columns.
#' @return An R x C matrix with column-wise sum-to-zero constraints on lower elements.
#' @export
to_centered_tri <- function(x, R, C) {
  y <- matrix(x[1] * 0, nrow = R, ncol = C)
  max_d <- min(C, R - 1)
  pos <- 1

  for (d in 1:max_d) {
    n_d <- R - d + 1
    n_params <- n_d - 1

    if (pos + n_params - 1 <= length(x)) {
      z_d <- x[pos:(pos + n_params - 1)]
      Q_d <- stz_basis(n_d)
      y[d:R, d] <- Q_d %*% z_d
      pos <- pos + n_params
    }
  }

  return(y)
}

#' Guidelines for Writing RTMB-Compatible Code
#'
#' @description
#' Models defined in \code{rtmb_code} rely on Automatic Differentiation (AD) via the
#' \code{RTMB} package. To ensure the model is differentiable and numerically stable,
#' specific coding practices must be followed.
#'
#' @details
#' \strong{1. Automatic Differentiation and \code{advector}:}
#' Parameters and intermediate calculations involving them are treated as \code{advector}
#' objects. RTMB "records" all mathematical operations to compute derivatives
#' automatically. If a function strips these attributes or is not differentiable, the
#' gradient calculation will fail.
#'
#' \strong{2. Avoid Discrete Branching:}
#' Standard R conditional statements like \code{if (x > 0)} or \code{ifelse()} based
#' on parameter values do not provide derivatives.
#' \itemize{
#'   \item \strong{Problem:} They create "jumps" in the likelihood surface.
#'   \item \strong{Solution:} Use smooth approximations. For example, use \code{\link{fabs}}
#'   instead of \code{abs}, or \code{\link{log_mix}} for mixture logic.
#' }
#'
#' \strong{3. Numerical Stability:}
#' Computations in log-space are preferred to prevent overflow or underflow.
#' Use the following stable utilities instead of raw algebraic expressions:
#' \itemize{
#'   \item \code{\link{log_sum_exp}}: For summing probabilities in log-space.
#'   \item \code{\link{log1p_exp}}: For \code{log(1 + exp(x))}.
#'   \item \code{\link{inv_logit}}: For mapping real numbers to probabilities.
#' }
#'
#' \strong{4. Vectorization vs. Loops:}
#' \itemize{
#'   \item \strong{Vectorization:} Highly recommended for performance. Standard R
#'   vectorized arithmetic (\code{+}, \code{-}, \code{*}, \code{/}, \code{log}, \code{exp})
#'   works seamlessly with AD.
#'   \item \strong{Loops:} Standard \code{for} loops are safe as long as the operations
#'   inside are differentiable.
#'   \item \strong{Avoid \code{apply}:} Functions like \code{apply}, \code{sapply},
#'   or \code{lapply} may sometimes strip AD attributes and should be replaced with
#'   vectorized operations or explicit loops.
#' }
#'
#' \strong{5. Matrix Operations:}
#' Use specialized functions for matrix algebra to maintain efficiency:
#' \itemize{
#'   \item \code{\link{quad_form_chol}}: For efficient quadratic forms.
#'   \item \code{\link{log_det_chol}}: For log-determinants via Cholesky factors.
#' }
#'
#' @name rtmb_syntax
#' @seealso \code{\link{math_functions}}, \code{\link{distributions}}, \code{\link{rtmb_code}}
NULL

#' RTMB model object
#'
#' @name RTMB_Model-class
#' @rdname RTMB_Model-class
#'
#' @description
#' An R6 class representing a Bayesian model built with RTMB.
#' This class stores model components and provides methods for
#' building the automatic differentiation object, optimizing the
#' posterior, and drawing posterior samples.
#'
#'
#' @field data Named list containing observation data.
#' @field par_list Evaluated list of parameter definitions.
#' @field log_prob Executable function for likelihood and prior calculations.
#' @field transform Function for calculating transformed parameters.
#' @field generate Function for calculating generated quantities.
#' @field par_names List of variable names corresponding to parameter dimensions.
#' @field pl_full Complete parameter list with expanded constraints and dimension information used internally.
#' @field formula Formula used for model generation (if created via a wrapper function).
#' @field requested_contrasts Character; user-requested contrast setting (e.g. "treatment").
#' @field contrasts Character; internal contrast setting used for fitting (e.g. "sum").
#' @field raw_data Data frame; the original data used for the model.
#' @field family Character string of the distribution family (if created via a wrapper function).
#' @field init List or numeric vector of initial values.
#' @field view Character vector of variable names to prioritize in summary.
#' @field code Original model AST generated by `rtmb_code()`.
#' @field map A list specifying parameter mappings for fixing values (e.g., via \code{fixed_model()}).
#' @field prior_correction Numeric value for legacy or low-level marginal-likelihood correction.
#' Fixed-parameter prior removal is handled during AD objective construction.
#' @field fixed_prior_specs Internal specification for removing fixed-parameter prior contributions during AD evaluation.
#' @field type Character; the type of the model (e.g. 'lmer', 'glm', 'table', 'ttest', 'corr', 'mediation').
#' @field extra List; used to store auxiliary data (like term assignments, factors, test results) needed by methods such as classic().
#'
#' @import RTMB
#' @export
RTMB_Model <- R6::R6Class(
  classname = "RTMB_Model",
  public = list(
    data = NULL,
    par_list = NULL,
    log_prob = NULL,
    transform = NULL,
    generate = NULL,
    par_names = NULL,
    pl_full = NULL,
    formula = NULL,
    raw_data = NULL,
    family = NULL,
    init = NULL,
    view = NULL,
    code = NULL,
    map = NULL,
    type = NULL,
    extra = NULL,
    contrasts = NULL,
    requested_contrasts = NULL,
    prior_correction = 0,
    fixed_prior_specs = NULL,

    # 1. Constructor
    #' @description Create a new `RTMB_Model` object.
    #' @param data Named list containing observation data.
    #' @param par_list Evaluated list of parameter definitions.
    #' @param log_prob Executable function for likelihood and prior calculations.
    #' @param transform Function for calculating transformed parameters (optional).
    #' @param generate Function for calculating generated quantities (optional).
    #' @param par_names List of variable names corresponding to parameter dimensions.
    #' @param init List or numeric vector of initial values.
    #' @param view Character vector of variable names to prioritize in summary.
    #' @param code Original model AST generated by `rtmb_code()`.
    initialize = function(data, par_list, log_prob,
                          transform = NULL, generate = NULL, par_names = NULL,
                          init = NULL, view = NULL, code = NULL) {
      self$data <- data
      self$par_names <- par_names
      self$init <- init
      self$view <- view
      self$code <- code
      self$par_list <- lapply(par_list, function(x) {
        if (typeof(x) %in% c("language", "symbol")) {
          eval(x, envir = self$data, enclos = parent.frame())
        } else {
          x
        }
      })
      self$log_prob <- log_prob
      self$transform <- transform
      self$generate <- generate

      self$pl_full <- parse_parameters(self$par_list, self$par_names)
      names(self$pl_full$init) <- self$pl_full$names

      # Changed to go through prepare_init
      init_vec <- self$prepare_init(self$init)
      self$init <- init_vec
      test_para <- constrained_vector_to_list(init_vec, self$par_list)

      test_val <- tryCatch(
        {
          if (!is.null(self$transform)) {
            test_tran <- self$transform(self$data, test_para)
            test_para <- c(test_para, test_tran)
          }
          self$log_prob(self$data, test_para)
        },
        error = function(e) {
          stop("An error occurred during the test execution of the R function (log_prob).\n[Error]: ",
            e$message,
            "\n  * Possible cause: Inappropriate initial parameter values may lead to uncomputable likelihoods (Inf or NaN).",
            "\n  * Solution: Set appropriate initial values within the domain (e.g., >0 for variance) via the 'init' argument, or reconsider the data scaling.",
            call. = FALSE
          )
        }
      )

      if (!is.numeric(test_val) || length(test_val) != 1) {
        stop("log_prob must return a single numeric value (scalar).")
      }

      if (!is.null(self$transform)) {
        test_tran <- tryCatch(
          {
            self$transform(self$data, test_para)
          },
          error = function(e) {
            stop("An error occurred during the test execution of the R function (transform).\n[Error]: ",
              e$message,
              call. = FALSE
            )
          }
        )

        if (!is.null(test_tran)) {
          if (!is.list(test_tran)) {
            stop("transform must return a list.", call. = FALSE)
          }
          if (length(test_tran) > 0 && (is.null(names(test_tran)) || any(names(test_tran) == ""))) {
            stop("transform must return a named list.", call. = FALSE)
          }
        }
      }

      if (!getOption("BayesRTMB.silent", FALSE)) cat("Checking RTMB setup...\n")
      test_ad <- self$build_ad_obj(init = init_vec, laplace = FALSE, jacobian_target = "all")
      test_gr <- tryCatch(test_ad$ad_obj$gr(test_ad$ad_obj$par), error = function(e) e)
      if (inherits(test_gr, "error")) {
        stop("An error occurred during gradient calculation in MakeADFun.\n[Error]: ",
          test_gr$message,
          call. = FALSE
        )
      }
    },

    # Helper method to format and complement initial values (uses self$init if no arguments are provided)
    #' @description Prepare and format initial values for the model parameters.
    #' @param init_arg Optional list or numeric vector of initial values. If NULL, defaults to `self$init` or random generation.
    #' @return A flat numeric vector of constrained initial values.
    prepare_init = function(init_arg) {
      target_init <- if (!is.null(init_arg)) init_arg else self$init

      if (is.null(target_init)) {
        return(generate_random_init(self$pl_full, self$par_list, range = 2))
      }

      # When a named list is provided (partial initial value specification)
      if (is.list(target_init)) {
        # 1. First, initialize everything with random values and convert to list format
        base_vec <- generate_random_init(self$pl_full, self$par_list, range = 2)
        base_list <- constrained_vector_to_list(base_vec, self$par_list)

        # 2. Overwrite only the parameters specified by the user
        for (name in names(target_init)) {
          if (name %in% names(base_list)) {
            base_list[[name]] <- as.numeric(target_init[[name]])
          } else {
            warning(sprintf("The initial value specified for '%s' does not exist in the model.", name), call. = FALSE)
          }
        }
        # 3. Return to a complete flat vector and return
        return(unlist(base_list, use.names = FALSE))
      }

      # When a vector is provided (full specification)
      if (is.numeric(target_init)) {
        return(target_init)
      }

      stop("init must be specified as a named list or a numeric vector.")
    },

    #' @description Get the current parameters as a named list.
    #' @param init Optional numeric vector or list of initial values. If NULL, uses the model's current initial values.
    #' @return A named list of constrained parameters.
    get_par_list = function(init = NULL) {
      init_vec <- self$prepare_init(init)
      return(constrained_vector_to_list(init_vec, self$par_list))
    },

    #' @description Get the RTMB automatic differentiation object.
    #' @param ... Arguments passed to \code{build_ad_obj}.
    #' @return A TMB objective object (ad_obj).
    get_ad_obj = function(...) {
      return(self$build_ad_obj(...)$ad_obj)
    },

    #' @description Convert a list of constrained parameters to unconstrained space.
    #' @param par_list_constrained A named list of parameters in their natural space.
    #' @return A named list of parameters in unconstrained space.
    to_unconstrained = function(par_list_constrained) {
      return(to_unconstrained(par_list_constrained, self$par_list))
    },

    #' @description Convert a list of unconstrained parameters to constrained (natural) space.
    #' @param par_list_unconstrained A named list of parameters in unconstrained space.
    #' @return A named list of parameters in constrained space.
    to_constrained = function(par_list_unconstrained) {
      return(to_constrained(par_list_unconstrained, self$par_list))
    },

    #' @description Convert a flat unconstrained vector to a named list.
    #' @param vec A numeric vector in unconstrained space.
    #' @return A named list of unconstrained parameters.
    unconstrained_vector_to_list = function(vec) {
      return(unconstrained_vector_to_list(vec, self$par_list))
    },

    #' @description Convert a flat constrained vector to a named list.
    #' @param vec A numeric vector in constrained space.
    #' @return A named list of constrained parameters.
    constrained_vector_to_list = function(vec) {
      return(constrained_vector_to_list(vec, self$par_list))
    },

    #' @description Calculate Satterthwaite degrees of freedom for parameters.
    #' @param ad_obj An RTMB objective object at the optimum.
    #' @param idx_fix_active Integer vector; indices of active fixed parameters in the full unconstrained vector.
    #' @param L_u_total Integer; total length of the full unconstrained parameter vector.
    #' @param opt_par Optional numeric vector of optimized parameters.
    #' @param max_df Numeric; maximum allowed degrees of freedom. Default is NULL.
    #' @param silent Logical; whether to suppress informational messages. Default is FALSE.
    #' @param return_sensitivities Logical; if TRUE, returns Hessian sensitivities and V for delta method.
    #' @return A numeric vector of estimated degrees of freedom (length = L_u_total). Inf for random effects.
    calculate_satterthwaite_df = function(ad_obj, idx_fix_active = NULL, L_u_total = NULL, opt_par = NULL, max_df = NULL, silent = FALSE, return_sensitivities = FALSE) {
      .calculate_satterthwaite_df_impl(self, private, ad_obj, idx_fix_active, L_u_total, opt_par, max_df, silent, return_sensitivities)
    },

    #' @description Calculate Satterthwaite degrees of freedom for integrated fixed effects (REML).
    #' @param ad_obj An RTMB objective object.
    #' @param opt_par Numeric vector of optimized variance components.
    #' @param beta_idx Integer vector; indices of fixed effects within the random effects vector.
    #' @param max_df Numeric; maximum allowed degrees of freedom. Default is NULL.
    #' @param silent Logical; whether to suppress informational messages. Default is FALSE.
    #' @param return_sensitivities Logical; if TRUE, returns Hessian sensitivities and delta-method components.
    #' @param ... Additional arguments.
    #' @return A numeric vector of estimated degrees of freedom for the fixed effects.
    calculate_reml_satterthwaite_df = function(ad_obj, opt_par, beta_idx, max_df = NULL, silent = FALSE, return_sensitivities = FALSE) {
      .calculate_reml_satterthwaite_df_impl(
        self = self,
        private = private,
        ad_obj = ad_obj,
        opt_par = opt_par,
        beta_idx = beta_idx,
        max_df = max_df,
        silent = silent,
        return_sensitivities = return_sensitivities
      )
    },

    #' @description Build the RTMB automatic differentiation object.
    #' @param init Optional numeric vector or list of initial values for the parameters. Default is NULL.
    #' @param laplace Logical; whether to use Laplace approximation to marginalize random effects. Default is FALSE.
    #' @param jacobian_target Character string specifying which parameters to apply Jacobian adjustments to:
    #' \itemize{
    #'   \item \code{"all"}: Apply to all parameters (standard for Bayesian MCMC).
    #'   \item \code{"random"}: Apply only to parameters with \code{random = TRUE} (standard for Laplace approximation/REML).
    #'   \item \code{"none"}: No Jacobian adjustment (standard for Maximum Likelihood estimation).
    #' }
    #' Default is "none".
    #' @param map Optional list specifying parameters to fix. Passed directly to MakeADFun. Default is NULL.
    #' @param .marginal_vars Internal use for profiling marginalized parameters.
    #' @return An RTMB objective object (ad_obj).
    build_ad_obj = function(init = NULL, laplace = FALSE, jacobian_target = "none", map = NULL, .marginal_vars = NULL) {
      target_map <- if (is.null(map)) {
        self$map
      } else {
        utils::modifyList(as.list(self$map), as.list(map))
      }

      random_effs <- if (!is.null(.marginal_vars)) {
        .marginal_vars
      } else {
        names(self$par_list)[sapply(self$par_list, function(x) isTRUE(x$random))]
      }

      # Exclude random components where all elements are NA in fixed/map
      if (!is.null(target_map)) {
        random_effs <- random_effs[vapply(random_effs, function(nm) {
          m <- target_map[[nm]]
          if (is.null(m)) return(self$par_list[[nm]]$unc_length > 0)
          any(!is.na(m))
        }, logical(1))]
        
        target_map <- lapply(target_map, private$.normalize_map_factor)
      }

      use_random <- if (laplace && length(random_effs) > 0) random_effs else NULL

      current_init <- self$prepare_init(init)
      current_init_list <- constrained_vector_to_list(current_init, self$par_list)
      init_unc_list <- to_unconstrained(current_init_list, self$par_list)

      pl_full_local <- self$pl_full
      par_list_local <- self$par_list
      log_prob_local <- self$log_prob
      data_local <- self$data

      # Pre-calculate shadow environment for fixed-prior removal
      # This identifies the full names of all parameters and transformations
      env_shadow <- list2env(self$data, parent = asNamespace("BayesRTMB"))
      idx <- 1
      for (nm in names(self$par_list)) {
        p <- self$par_list[[nm]]
        shadow_vec <- self$pl_full$names[idx:(idx + p$length - 1)]
        if (length(p$dim) > 1) dim(shadow_vec) <- p$dim
        env_shadow[[nm]] <- shadow_vec
        idx <- idx + p$length
      }
      if (!is.null(self$transform)) {
        par_only <- mget(names(self$par_list), envir = env_shadow, inherits = FALSE)
        tran_res_shadow <- tryCatch(suppressMessages(self$transform(self$data, par_only)), error = function(e) list())
        for (nm in names(tran_res_shadow)) env_shadow[[nm]] <- tran_res_shadow[[nm]]
      }
      shadow_list_local <- as.list(env_shadow)

      f_ad <- private$.build_f_ad(
        data_local = data_local,
        par_list_local = par_list_local,
        log_prob_local = log_prob_local,
        transform_local = self$transform,
        jacobian_target = jacobian_target,
        adreport = TRUE,
        fixed_prior_specs_local = self$fixed_prior_specs,
        code_model_local = self$code$model,
        shadow_list_local = shadow_list_local
      )

      ad_obj <- tryCatch(
        {
          RTMB::MakeADFun(
            func = f_ad,
            parameters = init_unc_list,
            random = use_random,
            map = target_map,
            silent = TRUE
          )
        },
        error = function(e) {
          stop("Failed to setup MakeADFun.\n[Error]: ", e$message, call. = FALSE)
        }
      )

      return(list(
        ad_obj = ad_obj,
        pl_full = pl_full_local,
        use_random = use_random
      ))
    },

    # 3. Maximum Likelihood / MAP Estimation Method
    #' @description Perform Maximum Likelihood or Maximum A Posteriori (MAP) estimation.
    #' @param laplace Logical; whether to use Laplace approximation for random effects. Default is TRUE.
    #' @param init Optional initial values for parameters (numeric vector or list).
    #' @param num_estimate Integer; Number of multi-start optimization runs. Default is 1.
    #' @param control A list of control settings passed to the optimizer.
    #' @param optimizer Character; The optimizer to use: "nlminb" (default) or "optim".
    #' @param method Character; The method for "optim" (e.g., "BFGS", "L-BFGS-B").
    #' @param map Optional list specifying parameters to fix (factors).
    #' @param fixed Optional list specifying parameter values to fix.
    #' @param se_method Character; method for uncertainty estimation.
    #' \itemize{
    #'   \item \code{"wald"}: Wald standard errors and Wald confidence intervals.
    #'   \item \code{"sampling"}: Simulation-based error propagation.
    #'   \item \code{"none"}: Do not compute standard errors, confidence intervals, or degrees of freedom.
    #' }
    #' @param num_samples Integer; number of samples to draw when se_method is "sampling". Default is 1000.
    #' @param seed Integer; random seed for sampling.
    #' @param marginal Character vector or "auto" or "none"; specifies which parameters to marginalize out via Laplace approximation.
    #' \itemize{
    #'   \item \code{"none"} (default): No additional marginalization.
    #'   \item \code{"auto"}: Uses variables specified by the model wrapper (e.g., fixed effects in lmer).
    #'   \item \code{"all"}: Marginalize all parameters (advanced use).
    #'   \item \code{character vector}: Specific parameter names to marginalize.
    #' }
    #' @param df_method Character or numeric; method for finite degrees-of-freedom approximation.
    #' Used only when `marginal` resolves to one or more parameters and `se_method != "none"`.
    #' \itemize{
    #'   \item \code{"auto"}: Uses Satterthwaite approximation when marginalization is active.
    #'   \item \code{"satterthwaite"}: Satterthwaite approximation.
    #'   \item \code{"inf"} or \code{"none"}: Use infinite degrees of freedom.
    #'   \item numeric: Use the supplied fixed degrees of freedom.
    #' }
    #' @param view Character vector of parameter names to prioritize in summary display.
    #' @param ... Additional arguments.
    #' @return A fitted `MAP_Fit` object.
    optimize = function(laplace = TRUE, init = NULL, num_estimate = 1, control = list(),
                        optimizer = "nlminb", method = "BFGS", map = NULL, fixed = NULL,
                        se_method = c("wald", "sampling", "none"), 
                        num_samples = 1000, seed = 123,
                        marginal = "none", df_method = "auto", view = NULL, ...) {
      
      se_method <- match.arg(se_method)
      se_flag <- !identical(se_method, "none")

      # --- Strict check for deprecated arguments ---
      dot_args <- list(...)
      reserved_old_args <- c(
        "empirical", "REML", "df", "df_pars",
        "target_vars", "is_classic", ".return_object",
        "ci_method", "se_sampling", "views", "se"
      )
      bad_args <- intersect(names(dot_args), reserved_old_args)
      if (length(bad_args) > 0L) {
        stop(
          "Unsupported argument(s) in optimize(): ",
          paste(bad_args, collapse = ", "), "\n",
          "Use `se_method = 'none'`, `se_method = 'wald'`, or `se_method = 'sampling'`.",
          call. = FALSE
        )
      }

      if (isTRUE(self$extra$two_stage) && identical(self$type, "lrt")) {
        return(.lrt_two_stage_optimize(
          self = self,
          laplace = laplace,
          init = init,
          num_estimate = num_estimate,
          control = control,
          optimizer = optimizer,
          method = method,
          map = map,
          fixed = fixed,
          se_method = se_method,
          num_samples = num_samples,
          seed = seed,
          marginal = marginal,
          df_method = df_method,
          view = view,
          ...
        ))
      }

      .inference_pipeline(
        self = self,
        private = private,
        laplace = laplace,
        init = init,
        num_estimate = num_estimate,
        control = control,
        optimizer = optimizer,
        method = method,
        map = map,
        fixed = fixed,
        se = se_flag,
        se_method = se_method,
        num_samples = num_samples,
        seed = seed,
        marginal = marginal,
        df_method = df_method,
        view = view,
        ...
      )
    },


    #' @description Perform frequentist inference (REML/ML) with model-appropriate degrees of freedom.
    #' @param df_method Character; Method for calculating degrees of freedom. Default is "auto".
    #' \itemize{
    #'   \item \code{"auto"}: Determined based on model type (e.g., Satterthwaite for mixed models).
    #'   \item \code{"residual"}: Residual degrees of freedom.
    #'   \item \code{"satterthwaite"}: Satterthwaite approximation (robust for mixed models).
    #'   \item \code{"inf"}: Use z-distribution (infinite degrees of freedom).
    #' }
    #' @param se_method Character; The method for CI estimation: "wald" (default) or "sampling".
    #' @param num_samples Integer; number of samples to draw when se_method is "sampling". Default is 1000.
    #' @param seed Integer; random seed for sampling.
    #' @param view Character vector of parameters to prioritize in the summary output.
    #' @param map Optional list specifying parameters to fix.
    #' @param fixed Optional list specifying parameter values to fix.
    #' @param ... Additional arguments.
    #' @return A `Classic_Fit` object.
    classic = function(df_method = "auto", se_method = c("wald", "sampling"),
                       num_samples = 1000, seed = 123,
                       view = NULL, map = NULL, fixed = NULL, ...) {
      
      # --- Check for unsupported/deprecated arguments ---
      dot_args <- list(...)
      reserved_classic_args <- c(
        "df", "REML", "marginal", "df_pars", "views",
        "empirical", "target_vars", "is_classic", ".return_object",
        "ci_method", "se_sampling",
        "robust", "robust_se",
        "bootstrap", "n_boot", "B",
        "cluster", "vcov", "se_type"
      )
      bad_args <- intersect(names(dot_args), reserved_classic_args)
      if (length(bad_args) > 0L) {
        stop(
          "Unsupported argument(s) in classic(): ",
          paste(bad_args, collapse = ", "), "\n",
          "Use fit$robust_se() or fit$bootstrap() after classic().",
          call. = FALSE
        )
      }

      private$.classic_pipeline(
        df_method = df_method,
        se_method = se_method,
        num_samples = num_samples,
        seed = seed,
        view = view,
        map = map,
        fixed = fixed,
        ...
      )
    },

    # 4. MCMC NUTS Sampling Method
    #' @description Draw posterior samples from the model.
    #' @param sampling Number of sampling iterations. Default is 1000.
    #' @param warmup Number of warmup iterations. Default is 1000.
    #' @param chains Number of MCMC chains. Default is 4.
    #' @param thin Thinning interval. Default is 1.
    #' @param seed Random seed.
    #' @param delta Target acceptance rate for HMC/NUTS. Default is 0.8.
    #' @param max_treedepth Maximum tree depth for HMC/NUTS. Default is 10.
    #' @param parallel Logical; whether to run chains in parallel. Default is FALSE.
    #' @param laplace Logical; whether to use Laplace approximation. Default is FALSE.
    #' @param init Optional initial values for parameters.
    #' @param init_jitter sd of randomize initial values for parameters.
    #' @param save_csv Optional list for saving MCMC results. e.g., list(name = "model", dir = "BayesRTMB_mcmc").
    #' @param map Optional list specifying parameters to fix (factors).
    #' @param fixed Optional list specifying parameter values to fix.
    #' @return A fitted `MCMC_Fit` object.
    sample = function(sampling = 1000, warmup = 1000, chains = 4,
                      thin = 1, seed = sample.int(1e6, 1),
                      delta = 0.8, max_treedepth = 10,
                      parallel = FALSE, laplace = FALSE,
                      init = NULL, init_jitter = 0.1, save_csv = NULL,
                      map = NULL, fixed = NULL) {
      if (isTRUE(self$extra$two_stage) && identical(self$type, "lrt")) {
        stop(
          "two_stage estimation for rtmb_lrt() is currently implemented only for optimize(). ",
          "MCMC sampling is not implemented for two_stage models yet.",
          call. = FALSE
        )
      }
      .sample_impl(self, private, sampling, warmup, chains, thin, seed, delta, 
                   max_treedepth, parallel, laplace, init, init_jitter, 
                   save_csv, map, fixed)
    },

    #' @description Run Automatic Differentiation Variational Inference (ADVI).
    #' @param iter Integer; fixed number of iterations for the optimization. Default is 3000.
    #' @param tol_rel_obj Numeric; relative tolerance for the ELBO change to determine convergence. Default is 0.005.
    #' @param window_size Integer; window size for median smoothing in the convergence check. Default is 100.
    #' @param num_samples Integer; number of posterior samples to generate from the fitted variational distribution. Default is 1000.
    #' @param num_estimate Integer; number of times to run the VB estimation (treated as chains). Default is 4.
    #' @param alpha Numeric; learning rate for the Adam optimizer. Default is 0.01.
    #' @param laplace Logical; whether to use Laplace approximation to marginalize random effects. Default is FALSE.
    #' @param print_freq Integer; iterations interval for progress output. Set to 0 to disable. Default is 1000.
    #' @param method Character; method of Variational Inference. Default is "meanfield".
    #' @param parallel Logical; whether to run estimations in parallel. Default is FALSE.
    #' @param seed Integer; random seed for reproducibility.
    #' @param init Optional numeric vector or list for initial parameter values. Default is NULL.
    #' @param save_csv Optional list for saving VB results. e.g., list(name = "model", dir = "BayesRTMB_vb").
    #' @param map Optional list specifying parameters to fix (factors).
    #' @param fixed Optional list specifying parameter values to fix.
    #' @return A fitted `VB_Fit` object containing posterior samples and diagnostic information.
    variational = function(iter = 3000,
                           tol_rel_obj = 0.005, window_size = 100,
                           num_samples = 1000, num_estimate = 4, alpha = 0.01,
                           laplace = FALSE, print_freq = 1000,
                           method = c("meanfield", "fullrank", "hybrid"), parallel = FALSE,
                           seed = sample.int(1e6, 1), init = NULL, save_csv = NULL,
                           map = NULL, fixed = NULL) {
      if (isTRUE(self$extra$two_stage) && identical(self$type, "lrt")) {
        stop(
          "two_stage estimation for rtmb_lrt() is currently implemented only for optimize(). ",
          "Variational inference is not implemented for two_stage models yet.",
          call. = FALSE
        )
      }
      .variational_impl(self, private, iter, tol_rel_obj, window_size, num_samples, 
                        num_estimate, alpha, laplace, print_freq, method, 
                        parallel, seed, init, save_csv, map, fixed)
    },


    # 5. Print model code method
    #' @description Print model code or model structure.
    #' @return The object itself, invisibly.
    print_code = function() {
      .print_code_impl(self, private)
    },

    #' @description Print the internal log-probability function.
    #' @return The object itself, invisibly.
    print_log_prob = function() {
      if (is.null(self$log_prob)) {
        cat("log_prob is not available.\n")
        return(invisible(self))
      }

      # Extract the internal function from the error-handling wrapper if it exists
      fn <- self$log_prob
      if (exists("comp_model", envir = environment(fn))) {
        fn <- get("comp_model", envir = environment(fn))
      }

      cat("=== RTMB Model: log_prob (Internal Function) ===\n")
      print(fn)

      if (!is.null(self$fixed_prior_specs) && length(self$fixed_prior_specs) > 0) {
        cat("\n=== Fixed-prior removal ===\n")
        cat("The following fixed parameters have their sampling-statement priors removed during AD evaluation:\n")
        cat(paste0("- ", names(self$fixed_prior_specs), collapse = "\n"), "\n")
      }

      return(invisible(self))
    },

    #' @description Print the internal transformation function.
    #' @return The object itself, invisibly.
    print_transform = function() {
      if (is.null(self$transform)) {
        cat("transform is not available.\n")
        return(invisible(self))
      }

      # Extract the internal function from the error-handling wrapper if it exists
      fn <- self$transform
      if (exists("comp_transform", envir = environment(fn))) {
        fn <- get("comp_transform", envir = environment(fn))
      }

      cat("=== RTMB Model: transform (Internal Function) ===\n")
      print(fn)
      return(invisible(self))
    },

    #' @description Print the internal generation function.
    #' @return The object itself, invisibly.
    print_generate = function() {
      if (is.null(self$generate)) {
        cat("generate is not available.\n")
        return(invisible(self))
      }

      # Extract the internal function from the error-handling wrapper if it exists
      fn <- self$generate
      if (exists("comp_generate", envir = environment(fn))) {
        fn <- get("comp_generate", envir = environment(fn))
      }

      cat("=== RTMB Model: generate (Internal Function) ===\n")
      print(fn)
      return(invisible(self))
    },

    #' @description Automatically detect and count the number of independent data points (observations).
    #' @return Integer; total number of observations, or NULL if model code is missing.
    get_n_obs = function() {
      .get_n_obs_impl(self, private)
    },

    #' @description Create a model with fixed parameters.
    #' @param fixed A named list of parameter values to fix.
    #' @param silent Logical; whether to suppress informational messages. Default is FALSE.
    #' @return A new RTMB_Model object.
    fixed_model = function(fixed = list(), silent = FALSE) {
      return(private$.apply_constraints(fixed, silent = silent))
    }
  ),

  private = list(
    # --- Classic pipeline implementation ---
    .validate_classic_model = function() {
      .validate_classic_model(self, private)
    },
    .resolve_classic_settings = function(df_method = "auto") {
      .resolve_classic_settings(self, private, df_method)
    },
    .classic_pipeline = function(...) {
      .classic_pipeline(self, private, ...)
    },
    .build_classic_components = function(...) {
      .build_classic_components(self, private, ...)
    },
    .get_unc_indices = function(name) {
      idx_curr <- 1L
      for (nm in names(self$par_list)) {
        L_u <- self$par_list[[nm]]$unc_length
        if (nm == name) {
          if (L_u == 0) return(integer(0))
          return(idx_curr:(idx_curr + L_u - 1L))
        }
        idx_curr <- idx_curr + L_u
      }
      integer(0)
    },

    # --- Internal helper: merge self$map and user supplied map consistently ---
    .merge_map_arg = function(map = NULL) {
      if (is.null(self$map) && is.null(map)) return(NULL)
      if (is.null(self$map)) return(as.list(map))
      if (is.null(map)) return(self$map)
      utils::modifyList(as.list(self$map), as.list(map))
    },

    # --- Internal helper: ensure map factor levels follow first-seen order ---
    .normalize_map_factor = function(m) {
      if (is.null(m)) return(NULL)
      vals <- as.character(m)
      factor(vals, levels = unique(vals[!is.na(vals)]))
    },

    # --- Internal helper: compute full unconstrained indices for active fixed and random parameters ---
    .compute_rtmb_index_map = function(ad_obj, target_map = NULL) {
      idx_ran <- ad_obj$env$random
      if (is.null(idx_ran)) idx_ran <- integer(0)

      idx_fix_active <- integer(0)
      idx_ran_full <- integer(0)
      factor_levels_seen <- list()

      opt_par_curr <- 1L
      idx_mapped_curr <- 1L
      idx_full_curr <- 1L

      for (name in names(self$par_list)) {
        L_u <- self$par_list[[name]]$unc_length
        if (L_u > 0) {
          map_f <- if (!is.null(target_map[[name]])) target_map[[name]] else NULL
          for (i in seq_len(L_u)) {
            pos_full <- idx_full_curr + i - 1L
            if (!is.null(map_f) && is.na(map_f[i])) next

            if (idx_mapped_curr %in% idx_ran) {
              idx_ran_full <- c(idx_ran_full, pos_full)
              idx_mapped_curr <- idx_mapped_curr + 1L
              next
            }

            if (!is.null(map_f)) {
              lvl <- as.character(map_f[i])
              if (!(lvl %in% names(factor_levels_seen))) {
                factor_levels_seen[[lvl]] <- opt_par_curr
                idx_fix_active <- c(idx_fix_active, pos_full)
                opt_par_curr <- opt_par_curr + 1L
              }
            } else {
              idx_fix_active <- c(idx_fix_active, pos_full)
              opt_par_curr <- opt_par_curr + 1L
            }
            idx_mapped_curr <- idx_mapped_curr + 1L
          }
          idx_full_curr <- idx_full_curr + L_u
        }
      }

      list(idx_fix_active = idx_fix_active, idx_ran_full = idx_ran_full, 
           idx_ran = idx_ran, factor_levels_seen = factor_levels_seen)
    },

    # --- Internal helper: copy SEs to duplicated mapped parameters ---
    .copy_mapped_se = function(unc_se_vec, se_fix, target_map, factor_levels_seen, idx_ran_full) {
      idx_curr <- 1L
      for (name in names(self$par_list)) {
        L_u <- self$par_list[[name]]$unc_length
        if (L_u > 0) {
          map_f <- if (!is.null(target_map[[name]])) target_map[[name]] else NULL
          for (i in seq_len(L_u)) {
            pos_full <- idx_curr + i - 1L
            if (pos_full %in% idx_ran_full) next
            if (!is.null(map_f) && !is.na(map_f[i])) {
              lvl <- as.character(map_f[i])
              mapped_opt_idx <- factor_levels_seen[[lvl]]
              if (!is.null(mapped_opt_idx) && mapped_opt_idx <= length(se_fix)) {
                unc_se_vec[pos_full] <- se_fix[mapped_opt_idx]
              }
            }
          }
          idx_curr <- idx_curr + L_u
        }
      }
      unc_se_vec
    },

    # --- Internal helper: resolve marginalization targets ---
    # --- Internal helper: resolve marginalization targets ---
    .resolve_marginal_vars = function(marginal = "none") {
      if (is.null(marginal)) marginal <- "none"
      if (!is.character(marginal)) stop("`marginal` must be 'none', 'auto', 'all', or a character vector of parameter names.", call. = FALSE)
      if (length(marginal) == 0L) return(character(0))
      if (length(marginal) == 1L && identical(marginal, "none")) return(character(0))

      # 1. Resolve keywords to parameter names
      if (length(marginal) == 1L && identical(marginal, "auto")) {
        target_vars <- self$extra$marginal
        if (is.null(target_vars) || length(target_vars) == 0L) return(character(0))
        if (!is.character(target_vars)) stop("Wrapper-defined marginal variables must be a character vector.", call. = FALSE)
        
        missing <- setdiff(target_vars, names(self$par_list))
        if (length(missing) > 0L) stop("Wrapper-defined marginal variables are not model parameters: ", paste(missing, collapse = ", "), call. = FALSE)
        target_vars <- unique(target_vars)
      } else if (length(marginal) == 1L && identical(marginal, "all")) {
        target_vars <- names(self$par_list)
      } else {
        reserved <- c("none", "auto", "all")
        if (any(marginal %in% reserved)) stop("`marginal` must be exactly one of 'none', 'auto', 'all', or a character vector of parameter names.", call. = FALSE)
        target_vars <- unique(marginal)
        missing <- setdiff(target_vars, names(self$par_list))
        if (length(missing) > 0L) stop("The following variables specified in `marginal` are not model parameters: ", paste(missing, collapse = ", "), call. = FALSE)
      }

      # 2. Prune fully fixed parameters
      if (!is.null(self$map) && length(target_vars) > 0L) {
        if (identical(marginal, "auto")) {
          # Silently remove fully fixed parameters in 'auto' mode
          target_vars <- target_vars[vapply(target_vars, function(nm) {
            m <- self$map[[nm]]
            is.null(m) || any(!is.na(m))
          }, logical(1))]
        } else if (!identical(marginal, "all")) {
          # Error if explicitly requested parameters are fully fixed
          fully_fixed_targets <- vapply(target_vars, function(nm) {
            m <- self$map[[nm]]
            !is.null(m) && all(is.na(m))
          }, logical(1))
          if (any(fully_fixed_targets)) {
            stop(sprintf("`marginal` contains parameters that are fixed and therefore cannot be marginalized: %s", 
                 paste(target_vars[fully_fixed_targets], collapse = ", ")), call. = FALSE)
          }
        }
      }

      return(target_vars)
    },

    # --- Internal helper: temporary marginal classification by changing random flags ---
    .with_marginal_par_list = function(target_vars = NULL, map = NULL) {
      original_par_list <- self$par_list
      modified_par_list <- self$par_list
      target_map <- private$.merge_map_arg(map)

      for (name in names(modified_par_list)) {
        is_mapped_out <- !is.null(target_map[[name]]) && all(is.na(target_map[[name]]))
        is_target <- !is.null(target_vars) && name %in% target_vars
        is_random_original <- isTRUE(modified_par_list[[name]]$random)

        if (!is_mapped_out && (is_target || is_random_original)) {
          modified_par_list[[name]]$random <- TRUE
        } else {
          modified_par_list[[name]]$random <- FALSE
        }
      }

      list(original = original_par_list, modified = modified_par_list)
    },

    # --- Low-level RTMB fitting engine shared by optimize() and classic() ---
    .fit_rtmb = function(laplace = TRUE, init = NULL, num_estimate = 1, control = list(),
                         optimizer = "nlminb", method = "BFGS", map = NULL, se = TRUE,
                         reml = FALSE, target_vars = NULL, jacobian_target = NULL,
                         apply_prior_correction = TRUE,
                         verbose = !getOption("BayesRTMB.silent", FALSE)) {

      if (is.null(control)) control <- list()
      optimizer <- match.arg(optimizer, c("nlminb", "optim"))
      if (!is.numeric(num_estimate) || length(num_estimate) != 1L || num_estimate < 1) stop("num_estimate must be a positive integer.", call. = FALSE)
      se_messages <- character(0)
      add_se_message <- function(msg) {
        if (!msg %in% se_messages) se_messages <<- c(se_messages, msg)
      }
      data_na_summary <- function(dat) {
        if (is.null(dat)) return(character(0))
        out <- character(0)
        for (nm in names(dat)) {
          x <- dat[[nm]]
          if (is.atomic(x) || is.matrix(x) || is.array(x) || is.data.frame(x)) {
            n_na <- sum(is.na(x))
            if (n_na > 0L) out <- c(out, sprintf("%s (%d NA)", nm, n_na))
          }
        }
        out
      }
      optimizer_error_messages <- character(0)
      format_optimizer_error <- function(msg) {
        na_vars <- data_na_summary(self$data)
        if (grepl("NA/NaN|NaN|NA", msg) && length(na_vars) > 0L) {
          paste0(
            "Optimization error: ", msg,
            ". The model data contain missing values in: ",
            paste(na_vars, collapse = ", "),
            ". If the model does not handle missing values explicitly, remove/impute them or use a wrapper/missing-data option that supports NA."
          )
        } else {
          paste0("Optimization error: ", msg)
        }
      }

      reml_state <- NULL
      if (isTRUE(reml)) {
        reml_state <- private$.with_marginal_par_list(target_vars = target_vars, map = map)
        self$par_list <- reml_state$modified
        on.exit({ self$par_list <- reml_state$original }, add = TRUE)
        laplace <- TRUE
      }

      # Decide jacobian target after potential REML -> laplace conversion
      if (is.null(jacobian_target)) jacobian_target <- if (isTRUE(laplace)) "random" else "none"
      jacobian_target <- match.arg(jacobian_target, c("all", "random", "none"))

      target_map <- private$.merge_map_arg(map)
      if (!is.null(target_map)) {
        target_map <- lapply(target_map, private$.normalize_map_factor)
      }

      if (verbose) cat("Starting RTMB optimization...\n")

      ad_setup <- if (isTRUE(self$silent) || getOption("BayesRTMB.silent", FALSE)) {
        suppressMessages(suppressWarnings(self$build_ad_obj(init = init, laplace = laplace, jacobian_target = jacobian_target, map = map)))
      } else {
        self$build_ad_obj(init = init, laplace = laplace, jacobian_target = jacobian_target, map = map)
      }
      base_ad_obj <- ad_setup$ad_obj

      opt_results <- vector("list", num_estimate); obj_vals <- rep(NA_real_, num_estimate)
      conv_codes <- rep(NA_integer_, num_estimate); messages <- rep(NA_character_, num_estimate)

      for (i in seq_len(num_estimate)) {
        if (num_estimate > 1L && verbose) cat(sprintf("Optimization run %d/%d...\r", i, num_estimate))
        res <- tryCatch({
          if (i > 1L && is.null(init) && length(base_ad_obj$par) > 0L) base_ad_obj$par <- base_ad_obj$par + rnorm(length(base_ad_obj$par), 0, 0.5)
          if (length(base_ad_obj$par) == 0L) {
            opt <- list(par = numeric(0), objective = base_ad_obj$fn(numeric(0)), convergence = 0L, message = "all parameters fixed")
          } else if (optimizer == "nlminb") {
            if (is.null(control$iter.max)) control$iter.max <- 5000
            opt <- suppressWarnings(nlminb(base_ad_obj$par, base_ad_obj$fn, base_ad_obj$gr, control = control))
          } else {
            if (is.null(control$maxit)) control$maxit <- 5000
            opt <- optim(base_ad_obj$par, base_ad_obj$fn, base_ad_obj$gr, method = method, control = control); opt$objective <- opt$value
          }
          list(opt = opt, ad_obj = base_ad_obj)
        }, error = function(e) {
          msg <- conditionMessage(e)
          optimizer_error_messages <<- c(optimizer_error_messages, msg)
          NULL
        })

        if (!is.null(res)) {
          opt_results[[i]] <- res; obj_vals[i] <- res$opt$objective
          conv_codes[i] <- res$opt$convergence; messages[i] <- if (!is.null(res$opt$message)) as.character(res$opt$message) else NA_character_
        }
      }
      if (verbose) cat("\n")
      valid_idx <- which(!is.na(obj_vals))
      if (length(valid_idx) == 0L) {
        na_vars <- data_na_summary(self$data)
        if (length(na_vars) > 0L && any(grepl("NA/NaN|NaN|NA", optimizer_error_messages))) {
          stop(
            "All optimization attempts failed. RTMB reported an NA/NaN-related optimization error, and the model data contain missing values in: ",
            paste(na_vars, collapse = ", "),
            ". If these NA values are not handled explicitly in the model code, remove/impute them or use a wrapper/missing-data option that supports NA.",
            call. = FALSE
          )
        }
        stop("All optimization attempts failed.", call. = FALSE)
      }
      if (length(optimizer_error_messages) > 0L) {
        warning(
          paste(unique(vapply(optimizer_error_messages, format_optimizer_error, character(1L))), collapse = "\n"),
          call. = FALSE
        )
      }
      converged_idx <- valid_idx[!is.na(conv_codes[valid_idx]) & conv_codes[valid_idx] == 0L]
      if (length(converged_idx) > 0L) {
        best_idx <- converged_idx[which.min(obj_vals[converged_idx])]
      } else {
        best_idx <- valid_idx[which.min(obj_vals[valid_idx])]
        best_code <- conv_codes[best_idx]
        best_message <- messages[best_idx]
        detail <- paste0(
          " convergence code = ", if (is.na(best_code)) "NA" else best_code,
          if (!is.na(best_message) && nzchar(best_message)) paste0("; message = ", best_message) else ""
        )
        if (num_estimate > 1L) {
          warning(
            "No optimization run converged; BEST is selected by the lowest objective among non-converged runs (",
            detail,
            ").",
            call. = FALSE
          )
        } else {
          warning(
            "Optimization did not converge (",
            detail,
            "). Estimates may be unreliable; consider increasing num_estimate, changing initial values, or adjusting optimizer control settings.",
            call. = FALSE
          )
        }
      }
      best_res <- opt_results[[best_idx]]

      if (isTRUE(apply_prior_correction) && !is.na(self$prior_correction) && self$prior_correction != 0) {
        for (i in seq_along(obj_vals)) if (!is.na(obj_vals[i])) obj_vals[i] <- obj_vals[i] + self$prior_correction
        best_res$opt$objective <- best_res$opt$objective + self$prior_correction
      }

      ad_obj <- best_res$ad_obj; opt <- best_res$opt; ad_obj$fn(opt$par)

      opt_history <- data.frame(
        estimate = seq_len(num_estimate),
        objective = obj_vals,
        code = conv_codes,
        message = messages,
        stringsAsFactors = FALSE
      )

      if (num_estimate > 1L && isTRUE(verbose)) {
        cat("\nConvergence Diagnostics per estimate:\n")
        for (i in seq_len(num_estimate)) {
          status <- if (!is.na(conv_codes[i]) && conv_codes[i] == 0L) "Converged" else "Not Converged"
          best_marker <- if (i == best_idx) "  <-- BEST" else ""
          cat(sprintf("  est%d: objective = %10.2f (%s)%s\n", i, obj_vals[i], status, best_marker))
        }
      }

      sd_rep <- if (isTRUE(se)) {
        tryCatch(
          withCallingHandlers(
            RTMB::sdreport(ad_obj, getJointPrecision = isTRUE(reml)),
            warning = function(w) {
              add_se_message(paste0("SE warning from sdreport(): ", conditionMessage(w)))
              invokeRestart("muffleWarning")
            }
          ),
          error = function(e) {
            add_se_message(paste0("SE calculation via sdreport() failed: ", conditionMessage(e)))
            NULL
          }
        )
      } else NULL

      unc_est_vec <- unlist(ad_obj$env$parList(), use.names = FALSE); L_u_total <- length(unc_est_vec)
      index_map <- private$.compute_rtmb_index_map(ad_obj, target_map)
      idx_fix_active <- index_map$idx_fix_active; idx_ran_full <- index_map$idx_ran_full
      if (length(opt$par) > 0L && length(idx_fix_active) == length(opt$par)) unc_est_vec[idx_fix_active] <- opt$par

      if (isTRUE(laplace) && !is.null(sd_rep) && length(idx_ran_full) > 0L) {
        smry_ran <- tryCatch(summary(sd_rep, select = "random"), error = function(e) NULL)
        if (!is.null(smry_ran) && nrow(smry_ran) == length(idx_ran_full)) unc_est_vec[idx_ran_full] <- smry_ran[, "Estimate"]
      }

      unc_se_vec <- rep(0, L_u_total); Cov_u <- matrix(0, L_u_total, L_u_total); fallback_needed <- FALSE
      if (isTRUE(se) && !is.null(sd_rep) && !is.null(sd_rep$cov.fixed)) {
        se_fix <- sqrt(pmax(diag(sd_rep$cov.fixed), 0))
        if (!isTRUE(sd_rep$pdHess) || any(!is.finite(se_fix)) || length(idx_fix_active) != length(se_fix)) {
          fallback_needed <- TRUE
          if (!isTRUE(sd_rep$pdHess)) {
            add_se_message("SE warning: sdreport() returned pdHess = FALSE; Hessian-based fallback will be attempted.")
          }
          if (any(!is.finite(se_fix))) {
            add_se_message("SE warning: sdreport() produced non-finite standard errors; Hessian-based fallback will be attempted.")
          }
          if (length(idx_fix_active) != length(se_fix)) {
            add_se_message("SE warning: sdreport() covariance size did not match the active parameter vector; Hessian-based fallback will be attempted.")
          }
        }
        else {
          unc_se_vec[idx_fix_active] <- se_fix; Cov_u[idx_fix_active, idx_fix_active] <- sd_rep$cov.fixed
          unc_se_vec <- private$.copy_mapped_se(unc_se_vec, se_fix, target_map, index_map$factor_levels_seen, idx_ran_full)
        }
      } else if (isTRUE(se)) {
        fallback_needed <- TRUE
        add_se_message("SE warning: sdreport() did not return cov.fixed; Hessian-based fallback will be attempted.")
      }

      if (isTRUE(se) && isTRUE(reml) && !is.null(sd_rep) && !is.null(sd_rep$jointPrecision) && length(idx_ran_full) > 0L) {
        V_joint <- tryCatch(
          solve(as.matrix(sd_rep$jointPrecision)),
          error = function(e) {
            add_se_message("SE warning: joint precision matrix was singular; using MASS::ginv() for random-effect covariance.")
            if (requireNamespace("MASS", quietly = TRUE)) MASS::ginv(as.matrix(sd_rep$jointPrecision)) else NULL
          }
        )
        if (!is.null(V_joint) && nrow(V_joint) == length(idx_ran_full) && ncol(V_joint) == length(idx_ran_full)) {
          Cov_u[idx_ran_full, idx_ran_full] <- as.matrix(V_joint)
          unc_se_vec[idx_ran_full] <- sqrt(pmax(diag(as.matrix(V_joint)), 0))
        }
      }

      if (isTRUE(se) && isTRUE(laplace) && !is.null(sd_rep) && length(idx_ran_full) > 0L) {
        smry_ran <- tryCatch(summary(sd_rep, select = "random"), error = function(e) NULL)
        if (!is.null(smry_ran) && nrow(smry_ran) == length(idx_ran_full) && "Std. Error" %in% colnames(smry_ran)) unc_se_vec[idx_ran_full] <- smry_ran[, "Std. Error"]
      }

      if (isTRUE(se) && isTRUE(fallback_needed) && length(opt$par) > 0L) {
        H <- tryCatch(ad_obj$he(opt$par), error = function(e) tryCatch(ad_obj$he(opt$par + 1e-6), error = function(e2) NULL))
        if (!is.null(H)) {
          Cov_pseudo <- tryCatch(
            solve(H),
            error = function(e) {
              add_se_message("SE warning: Hessian matrix was singular; using MASS::ginv() to approximate the covariance matrix.")
              if (requireNamespace("MASS", quietly = TRUE)) MASS::ginv(H) else NULL
            }
          )
          if (!is.null(Cov_pseudo) && length(idx_fix_active) == length(sqrt(pmax(diag(Cov_pseudo), 0)))) {
            se_pseudo <- sqrt(pmax(diag(Cov_pseudo), 0)); unc_se_vec[idx_fix_active] <- se_pseudo; Cov_u[idx_fix_active, idx_fix_active] <- Cov_pseudo
            unc_se_vec <- private$.copy_mapped_se(unc_se_vec, se_pseudo, target_map, index_map$factor_levels_seen, idx_ran_full)
          } else {
            add_se_message("SE warning: Hessian-based fallback failed; standard errors may be unavailable or unreliable.")
          }
        } else {
          add_se_message("SE warning: Hessian could not be evaluated; standard errors may be unavailable or unreliable.")
        }
      }
      if (isTRUE(se) && length(se_messages) > 0L && isTRUE(verbose) && !getOption("BayesRTMB.silent", FALSE)) {
        cat(paste0(se_messages, collapse = "\n"), "\n", sep = "")
      }

      diag(Cov_u) <- pmax(diag(Cov_u), unc_se_vec^2); Cov_u[is.na(Cov_u)] <- 0
      unc_est_list <- unconstrained_vector_to_list(unc_est_vec, self$par_list)
      unc_se_list <- unconstrained_vector_to_list(unc_se_vec, self$par_list)
      con_est_list <- to_constrained(unc_est_list, self$par_list)

      list(ad_setup = ad_setup, ad_obj = ad_obj, opt = opt, best_index = best_idx, opt_history = opt_history,
           objective = opt$objective, log_ml = -opt$objective, convergence = opt$convergence,
           sd_rep = sd_rep, par_unc = unc_est_vec, se_unc = unc_se_vec, vcov_unc = Cov_u,
           par_unc_list = unc_est_list, se_unc_list = unc_se_list, par = con_est_list,
           idx_fix_active = idx_fix_active, idx_ran_full = idx_ran_full, idx_ran = index_map$idx_ran,
           factor_levels_seen = index_map$factor_levels_seen, L_u_total = L_u_total,
           fallback_needed = fallback_needed,
           map = target_map, laplace = laplace, reml = reml, jacobian_target = jacobian_target)
    },

    .simple_jacobian = function(f, x, h = 1e-4) {
      p <- length(x)
      f0 <- f(x)
      res <- matrix(0, nrow = length(f0), ncol = p)
      for (i in 1:p) {
        h_i <- max(abs(x[i]), 1e-4) * h
        x_plus <- x
        x_minus <- x
        x_plus[i] <- x[i] + h_i
        x_minus[i] <- x[i] - h_i
        res[, i] <- (f(x_plus) - f(x_minus)) / (2 * h_i)
      }
      return(res)
    },
    # --- Common patterns for REML and parameter classification ---
    fix_patterns = c("Intercept", "Intercept_c", "b", "mean", "prob", "beta", "mu", "delta", "diff", "mean_diff"),
    var_patterns = c("sigma", "sd", "rho", "rho_resid", "L_resid", "corr", "CF_corr", "cov", "CF_cov", "cutpoints", "shape", "phi", "nu", "z", "lambda", "tau", "theta", "alpha", "omega", "kappa"),

    # --- Internal helper for index computation ---
    .compute_active_indices = function(target_map = NULL) {
      idx_fix_active <- integer(0)
      factor_levels_seen <- list()
      opt_par_curr <- 1
      
      map_to_use <- if (!is.null(target_map)) target_map else self$map
      
      for (name in names(self$par_list)) {
        p <- self$par_list[[name]]
        L_u <- p$unc_length
        m <- if (!is.null(map_to_use[[name]])) map_to_use[[name]] else factor(1:L_u)
        
        # Identify indices that are not NA (i.e., estimated)
        idx_active_local <- which(!is.na(m))
        if (length(idx_active_local) > 0) {
          # Check for shared levels (identical factors)
          lvl_names <- as.character(m[idx_active_local])
          for (lvl in unique(lvl_names)) {
            pos_in_active <- which(lvl_names == lvl)
            # The first occurrence of this level gets the next free optimization index
            if (!(lvl %in% names(factor_levels_seen))) {
              factor_levels_seen[[lvl]] <- opt_par_curr
              opt_par_curr <- opt_par_curr + 1
            }
            # Add to global unconstrained index list
            pos_last <- sum(self$pl_full$unc_length[seq_len(which(names(self$par_list) == name) - 1)]) + idx_active_local[pos_in_active]
            idx_fix_active <- c(idx_fix_active, pos_last)
          }
        }
      }
      return(list(idx_active = sort(unique(idx_fix_active)), n_opt = opt_par_curr - 1))
    },

    # --- Internal helper for derived parameter calculation (delta method) ---
    .build_derived_df_table = function(func, base_out, is_generate = FALSE, se_sampling = FALSE, num_samples = 1000) {
      if (is.null(func) || is.null(base_out)) return(NULL)
      
      # This replaces the 150-line local function in optimize() and classic()
      # Integration of both logic versions
      # ... (Implementation uses logic from build_derived_df)
      return(NULL) # Placeholder for now, will implement in next chunk
    },

    # --- Internal helper for fixed parameter recursion ---
    .dispatch_fixed = function(.method_to_call, ...) {
      args <- list(...)
      fixed <- args$fixed
      args$fixed <- NULL
      new_model <- self$fixed_model(fixed)
      # Ensure the method is retrieved as a bound function from the R6 object
      func <- new_model[[.method_to_call]]
      if (!is.function(func)) stop(sprintf("Method '%s' not found in RTMB_Model.", .method_to_call))
      return(do.call(func, args))
    },

    # --- Internal helper for AD function setup ---
    .build_f_ad = function(data_local, par_list_local, log_prob_local, transform_local = NULL, 
                           jacobian_target = "all", adreport = FALSE,
                           fixed_prior_specs_local = NULL, code_model_local = NULL, 
                           shadow_list_local = NULL) {
      function(y_unc_list) {
        para <- to_constrained(y_unc_list, par_list_local)

        if (!is.null(transform_local)) {
          tran_res <- transform_local(data_local, para)
          para <- c(para, tran_res)

          # transform ADREPORT (only if requested, e.g. for sdreport)
          if (adreport && length(tran_res) > 0) {
            for (n in names(tran_res)) {
              # Inject into parent environment so ADREPORT can find it by name
              assign(n, tran_res[[n]])
              eval(substitute(RTMB::ADREPORT(V), list(V = as.name(n))))
            }
          }
        }

        lp <- log_prob_local(data_local, para)

        # Apply dynamic fixed-prior removal if specs are present
        if (!is.null(fixed_prior_specs_local) && length(fixed_prior_specs_local) > 0) {
          lp <- lp - .eval_fixed_prior_contribution_impl(
            code_model = code_model_local,
            shadow_list = shadow_list_local,
            dat = data_local,
            para = para,
            fixed = fixed_prior_specs_local
          )
        }

        if (jacobian_target == "all") {
          lj <- calc_log_jacobian(y_unc_list, par_list_local, only_random = FALSE)
          return(-(lp + lj))
        } else if (jacobian_target == "random") {
          lj <- calc_log_jacobian(y_unc_list, par_list_local, only_random = TRUE)
          return(-(lp + lj))
        } else {
          return(-lp)
        }
      }
    },

    # --- Logic for applying parameter fixing (fixed argument) ---
    .has_multivariate_prior_on = function(expr, target_base) {
      if (is.null(expr)) return(FALSE)

      get_base_name <- function(x) {
        if (is.name(x)) return(as.character(x))
        if (is.call(x) && identical(x[[1]], as.name("["))) {
          return(get_base_name(x[[2]]))
        }
        NULL
      }

      is_multi_dist <- function(rhs) {
        if (!is.call(rhs)) return(FALSE)
        fn <- as.character(rhs[[1]])
        length(fn) == 1 && fn %in% c("multi_normal", "multi_student_t", "multi_cauchy")
      }

      walk <- function(x) {
        if (!is.call(x)) return(FALSE)

        if (identical(x[[1]], as.name("~"))) {
          lhs_base <- get_base_name(x[[2]])
          rhs <- x[[3]]
          if (identical(lhs_base, target_base) && is_multi_dist(rhs)) {
            return(TRUE)
          }
        }

        any(vapply(as.list(x)[-1], walk, logical(1)))
      }

      walk(expr)
    },

    .apply_constraints = function(fixed = list(), silent = FALSE) {
      new_model <- self$clone()
      if (length(fixed) == 0) return(new_model)

      new_map <- as.list(new_model$map)
      new_init <- new_model$init 

      if (is.null(names(new_init))) {
        names(new_init) <- self$pl_full$names
      }

      normalize_map <- function(m) {
        if (is.null(m)) return(NULL)
        factor(as.character(m))
      }

      for (name in names(fixed)) {
        val <- fixed[[name]]
        p_base <- gsub("\\[.*\\]$", "", name)
        has_index <- grepl("\\[", name)

        if (!(p_base %in% names(self$par_list))) stop("Parameter not found: ", p_base)

        if (has_index && !is.null(self$code$model)) {
          if (private$.has_multivariate_prior_on(self$code$model, p_base)) {
            stop(
              sprintf(
                "Cannot use fixed = list('%s' = ...) because '%s' has a multivariate prior. ",
                name, p_base
              ),
              "Partial fixing of a multivariate prior is not supported. ",
              "Use a reduced model instead, e.g. remove the corresponding predictor from the formula.",
              call. = FALSE
            )
          }
        }

        p_info <- self$par_list[[p_base]]
        L_u <- p_info$unc_length

        if (is.null(new_map[[p_base]])) {
          new_map[[p_base]] <- if (L_u > 0) factor(seq_len(L_u)) else factor(integer(0))
        }
        curr_map <- new_map[[p_base]]

        start_idx <- 1
        for (n in names(self$par_list)) {
          if (n == p_base) break
          start_idx <- start_idx + self$par_list[[n]]$length
        }

        if (!grepl("\\[", name)) {
          # Fix the entire parameter
          val_vec <- if (length(val) == 1) rep(val, p_info$length) else val
          new_init[start_idx:(start_idx + p_info$length - 1)] <- as.numeric(val_vec)

          if (L_u > 0) {
            # IMPORTANT: Recreate as a factor with no levels instead of curr_map[] <- NA
            curr_map <- factor(rep(NA_character_, L_u))
          }

        } else {
          # Fix specific elements
          # Guard: element-wise fixing is not supported for structurally constrained types
          # where constrained length != unconstrained length (simplex, corr_matrix, etc.)
          constrained_types <- c("simplex", "centered", "corr_matrix", "cov_matrix",
                                 "CF_corr", "CF_cov", "centered_matrix",
                                 "centered_tri", "positive_centered_tri")
          if (p_info$length != L_u && (p_info$type %in% constrained_types)) {
            stop(sprintf(
              "Element-wise fixing is not supported for parameter '%s' (type = '%s'). ",
              p_base, p_info$type
            ),
            "Fix the entire parameter instead with fixed = list('", p_base, "' = ...).",
            call. = FALSE)
          }

          idx_str <- gsub(".*\\[(.*)\\].*", "\\1", name)
          p_names <- self$par_names[[p_base]]
          idx <- suppressWarnings(as.integer(idx_str))
          if (is.na(idx) && !is.null(p_names)) idx <- which(p_names == idx_str)

          if (length(idx) > 0) {
            new_init[start_idx + idx - 1] <- as.numeric(val)

            if (L_u > 0) {
              # For types where length == unc_length, constrained index maps directly
              if (idx <= L_u) {
                curr_map[idx] <- NA
              }
              # Drop unused levels even in partial fixing
              curr_map <- normalize_map(curr_map)
            }
          }
        }

        new_map[[p_base]] <- curr_map
      }

      new_model$init <- new_init
      new_model$map <- new_map
      
      # Dynamic prior correction for Bayes Factor (log-density of fixed parameters)
      # Instead of wrapping log_prob (which is messy for printing), we store the specs
      # and handle removal during AD objective construction in .build_f_ad.
      new_model$prior_correction <- 0 # Explicitly reset scalar correction
      
      # Merge with existing fixed_prior_specs if any
      merged_specs <- self$fixed_prior_specs %||% list()
      merged_specs[names(fixed)] <- fixed
      new_model$fixed_prior_specs <- merged_specs

      # Validate to catch manual lp increments involving fixed parameters early
      # Align with runtime by including transformed parameters if they exist
      test_para <- constrained_vector_to_list(new_init, self$par_list)
      if (!is.null(self$transform)) {
        test_tran <- tryCatch(suppressMessages(self$transform(self$data, test_para)), error = function(e) list())
        if (length(test_tran) > 0) {
          missing <- setdiff(names(test_tran), names(test_para))
          # Exclude variables that are already in data
          missing <- setdiff(missing, names(self$data))
          if (length(missing) > 0) test_para <- c(test_para, test_tran[missing])
        }
      }
      
      # Perform validation using the internal implementation function
      # We need a temporary shadow list for validation
      env_shadow_tmp <- list2env(self$data, parent = asNamespace("BayesRTMB"))
      idx_tmp <- 1
      for (nm in names(self$par_list)) {
        p <- self$par_list[[nm]]
        shadow_vec <- self$pl_full$names[idx_tmp:(idx_tmp + p$length - 1)]
        if (length(p$dim) > 1) dim(shadow_vec) <- p$dim
        env_shadow_tmp[[nm]] <- shadow_vec
        idx_tmp <- idx_tmp + p$length
      }
      if (!is.null(self$transform)) {
        par_only <- mget(names(self$par_list), envir = env_shadow_tmp, inherits = FALSE)
        tran_res_shadow <- tryCatch(suppressMessages(self$transform(self$data, par_only)), error = function(e) list())
        for (nm in names(tran_res_shadow)) env_shadow_tmp[[nm]] <- tran_res_shadow[[nm]]
      }
      shadow_list_tmp <- as.list(env_shadow_tmp)

      suppressWarnings(.eval_fixed_prior_contribution_impl(
        code_model = self$code$model,
        shadow_list = shadow_list_tmp,
        dat = self$data,
        para = test_para,
        fixed = merged_specs
      ))

      # Prune fully fixed parameters from extra$marginal if they exist
      if (!is.null(new_model$extra$marginal)) {
        fully_fixed <- names(new_map)[vapply(new_map, function(m) !is.null(m) && all(is.na(m)), logical(1))]
        if (length(fully_fixed) > 0) {
          new_model$extra$marginal <- setdiff(new_model$extra$marginal, fully_fixed)
        }
      }

      return(new_model)
    }
  )
)

# --- Internal helper: calculate log-density of fixed parameters dynamically ---
# Note: Only priors specified as sampling statements (x ~ dist(...)) are automatically removed.
# Manual increments to 'lp' involving fixed parameters will trigger an error to avoid incorrect Bayes Factors.
.eval_fixed_prior_contribution_impl <- function(code_model, shadow_list, dat, para, fixed) {
  if (length(fixed) == 0 || is.null(code_model)) return(0)

  # 1. Prepare environment with CURRENT parameter values (including transformations)
  env_val <- list2env(dat, parent = asNamespace("BayesRTMB"))
  for (nm in names(para)) env_val[[nm]] <- para[[nm]]
  
  # 2. Shadow environment for identifying fixed components
  # Use the provided shadow_list (pre-calculated to avoid redundant transform calls)
  env_shadow <- list2env(shadow_list, parent = asNamespace("BayesRTMB"))
  fixed_keys <- names(fixed)
  fixed_bases <- gsub("\\[.*\\]$", "", fixed_keys)
  tainted_vars <- character(0)
  lp_corr <- 0
  
  evaluate_node <- function(x) {
    if (is.call(x)) {
      if (identical(x[[1]], as.name("~"))) {
        lhs_expr <- x[[2]]
        rhs_call <- x[[3]]
        dist_name <- as.character(rhs_call[[1]])
        
        # Evaluate LHS in shadow env to find which elements are fixed
        # We use tryCatch because transformed variables might not be representable as strings
        shadow_names <- tryCatch(eval(lhs_expr, envir = env_shadow), error = function(e) NULL)
        
        # Identify fixed elements
        fixed_mask <- if (!is.null(shadow_names)) {
          (shadow_names %in% fixed_keys) | (gsub("\\[.*\\]$", "", shadow_names) %in% fixed_keys)
        } else {
          FALSE
        }
        
        if (any(fixed_mask)) {
          val_lhs <- eval(lhs_expr, envir = env_val)
          val_args <- lapply(as.list(rhs_call[-1]), eval, envir = env_val)
          
          # Call distribution function
          lpdf_name <- paste0(dist_name, "_lpdf")
          lpmf_name <- paste0(dist_name, "_lpmf")
          
          # Find the function in BayesRTMB namespace
          pkg_ns <- asNamespace("BayesRTMB")
          dist_fn <- if (exists(lpdf_name, envir = pkg_ns)) get(lpdf_name, envir = pkg_ns)
                     else if (exists(lpmf_name, envir = pkg_ns)) get(lpmf_name, envir = pkg_ns)
                     else NULL
          
          if (!is.null(dist_fn)) {
            # If multivariate, we take the whole thing (we already checked partial fixing for multi)
            is_multi <- dist_name %in% c("multi_normal", "multi_student_t", "multi_cauchy")
            
            if (is_multi) {
              # Full parameter must be fixed
              lp_corr <<- lp_corr + do.call(dist_fn, c(list(val_lhs), val_args))
            } else {
              # Univariate - might be partial
              # Get element-wise density (force sum=FALSE if supported)
              densities <- tryCatch(
                do.call(dist_fn, c(list(val_lhs), val_args, list(sum = FALSE))),
                error = function(e) do.call(dist_fn, c(list(val_lhs), val_args))
              )
              
              # Check if element-wise was needed but only a scalar was returned
              if (length(densities) == 1L && length(fixed_mask) > 1L && !all(fixed_mask)) {
                stop(
                  sprintf("Cannot partially remove a vectorized prior for '%s' because element-wise log densities are unavailable.", dist_name),
                  call. = FALSE
                )
              }
              
              # Flatten to handle matrix/array accurately
              densities_v <- as.vector(densities)
              mask_v <- as.vector(fixed_mask)
              
              if (length(densities_v) == length(mask_v)) {
                lp_corr <<- lp_corr + sum(densities_v[mask_v])
              } else if (length(densities_v) == 1L) {
                # Scalar fallback (only if all elements are fixed, already checked above)
                lp_corr <<- lp_corr + densities_v
              }
            }
          } else {
            stop(
              sprintf("Cannot remove fixed prior contribution for distribution '%s': corresponding lpdf/lpmf function was not found.", dist_name),
              call. = FALSE
            )
          }
        }
      } else if (identical(x[[1]], as.name("for"))) {
        # Execute the loop
        var_name <- as.character(x[[2]])
        seq_val <- eval(x[[3]], envir = env_val)
        loop_body <- x[[4]]
        
        for (v in seq_val) {
          env_val[[var_name]] <- v
          env_shadow[[var_name]] <- v
          evaluate_node(loop_body)
        }
      } else if (identical(x[[1]], as.name("{"))) {
        lapply(as.list(x)[-1], evaluate_node)
      } else if (identical(x[[1]], as.name("if"))) {
        cond <- tryCatch(eval(x[[2]], envir = env_val), error = function(e) FALSE)
        if (isTRUE(cond)) evaluate_node(x[[3]]) else if (length(x) == 4) evaluate_node(x[[4]])
      } else if (identical(x[[1]], as.name("<-")) || identical(x[[1]], as.name("="))) {
        # Check for manual log-prob increments or assignments involving fixed parameters
        lhs_vars <- all.vars(x[[2]])
        lhs_name <- if (length(lhs_vars) > 0) lhs_vars[1] else as.character(x[[2]])[1]
        rhs_vars <- all.vars(x[[3]])
        
        # If the RHS involves fixed or already tainted variables
        if (any(setdiff(rhs_vars, "lp") %in% c(fixed_bases, tainted_vars))) {
          if (identical(lhs_name, "lp")) {
            stop("Manual log-probability increments (lp <- ...) involving fixed parameters or derived variables are not supported for automatic prior removal. Please use sampling statements (~) instead.", call. = FALSE)
          } else if (!is.na(lhs_name)) {
            # Taint the LHS variable so that subsequent use in lp <- ... can be detected
            tainted_vars <<- unique(c(tainted_vars, lhs_name))
          }
        }
      } else {
        # Recurse into all other calls
        lapply(as.list(x)[-1], evaluate_node)
      }
    }
  }

  evaluate_node(code_model)
  return(lp_corr)
}

test_that("negative binomial 2 log pmf matches stats parameterization", {
  x <- c(0, 1, 3, 8)
  mu <- c(0.5, 1.2, 4.0, 9.5)
  size <- 2.75

  expect_equal(
    neg_binomial_2_lpmf(x, mu, size, sum = FALSE),
    stats::dnbinom(x, size = size, mu = mu, log = TRUE),
    tolerance = 1e-12
  )
})

test_that("negative binomial GLM is AD compatible", {
  set.seed(1)
  dat <- data.frame(
    HR = rnbinom(20, mu = 10, size = 2),
    Steal = rnorm(20),
    Walk = rnorm(20),
    K = rnorm(20)
  )

  mdl <- rtmb_glm(
    HR ~ Steal + Walk + K,
    data = dat,
    family = "neg_binomial",
    prior = prior_normal()
  )
  obj <- mdl$get_ad_obj()

  expect_true(all(is.finite(obj$gr(obj$par))))
})

test_that("optimize() marginal MAP and Satterthwaite DF work correctly", {
  set.seed(1)
  Y1 <- rnorm(40, mean = 10, sd = 2)
  Y2 <- rnorm(40, mean = 11, sd = 2)

  mdl_tt <- rtmb_ttest(Y1, Y2, prior = prior_jzs(r = 0.707))

  fit_tt <- mdl_tt$optimize(
    marginal = "auto",
    df_method = "satterthwaite"
  )

  expect_s3_class(fit_tt, "map_fit")
  expect_true(all(c("total_mean", "delta") %in% fit_tt$marginal_vars))
  expect_true(all(c("total_mean", "delta") %in% fit_tt$laplace_random_vars))
  expect_true(fit_tt$show_df)

  tab_tt <- fit_tt$summary()
  expect_true("df" %in% colnames(tab_tt))

  # Verify DF for derived quantities
  for (nm in c("diff", "mean0", "mean1")) {
    df_val <- tab_tt[nm, "df"]
    expect_false(is.na(df_val))
    expect_true(df_val < 1e12)
  }
  
  # delta should be Inf or large finite due to low sensitivity in this balanced case
  expect_true(is.infinite(tab_tt["delta", "df"]))
})

test_that("JZS unequal-variance t-test uses an explicit delta parameter", {
  set.seed(2)
  Y1 <- rnorm(20, mean = 10, sd = 1)
  Y2 <- rnorm(25, mean = 11, sd = 2)

  mdl_tt <- rtmb_ttest(Y1, Y2, prior = prior_jzs(r = 0.707), var.equal = FALSE)

  expect_true(all(c("total_mean", "sd", "delta") %in% names(mdl_tt$par_list)))
  expect_false(any(c("mean0", "mean1") %in% names(mdl_tt$par_list)))
  expect_true(all(c("total_mean", "delta") %in% mdl_tt$extra$marginal))

  fit_null <- mdl_tt$optimize(fixed = list(delta = 0), se_method = "none")
  expect_s3_class(fit_null, "map_fit")
})

test_that("Mixed model metadata separation and profile() logic", {
  skip_if_not_installed("lme4")
  data("sleepstudy", package = "lme4")
  mdl_mm <- rtmb_lmer(Reaction ~ Days + (1 | Subject), data = sleepstudy)

  fit_mm <- mdl_mm$optimize(laplace = TRUE, marginal = "auto")

  expect_true(all(c("Intercept", "b") %in% fit_mm$marginal_vars))
  expect_true(all(c("Intercept", "b", "r_re") %in% fit_mm$laplace_random_vars))
  expect_gt(length(fit_mm$laplace_random_vars), length(fit_mm$marginal_vars))

  # Marginalized vars should be blocked from profiling
  expect_error(fit_mm$profile(pars = "Intercept"), "marginalized")

  # Non-marginalized fixed parameter should be profile-able in a non-marginalized fit
  fit_mm_none <- mdl_mm$optimize(marginal = "none")
  prof_mm <- fit_mm_none$profile(pars = "b[Days]", quiet = TRUE)
  expect_false(is.null(prof_mm))
})

test_that("se_method = 'none' and df_method = 'bw' rejection", {
  Y1 <- rnorm(20); Y2 <- rnorm(20)
  mdl_tt <- rtmb_ttest(Y1, Y2)

  fit_none <- mdl_tt$optimize(se_method = "none", df_method = "satterthwaite")
  tab_none <- fit_none$summary()
  
  expect_false(fit_none$show_df)
  expect_true(all(is.na(tab_none$`Std. Error`)))
  expect_false("df" %in% colnames(tab_none))

  expect_error(mdl_tt$optimize(df_method = "bw"), "not supported")
})

test_that("classic() regression and prior restriction", {
  mdl_lm <- rtmb_lm(mpg ~ wt, data = mtcars)
  fit_c <- mdl_lm$classic()
  expect_s3_class(fit_c, "Classic_Fit")

  mdl_p <- rtmb_lm(mpg ~ wt, data = mtcars, prior = prior_normal())
  expect_error(mdl_p$classic(), "prior_flat")
})

test_that("Wrappers optimize correctly", {
  
  # rtmb_lm
  fit_lm <- rtmb_lm(mpg ~ wt, data = mtcars)
  res_lm <- fit_lm$optimize()
  expect_true(inherits(res_lm, "map_fit"))
  
  # rtmb_ttest
  fit_t <- rtmb_ttest(mpg ~ am, data = mtcars)
  res_t <- fit_t$optimize()
  expect_true(inherits(res_t, "map_fit"))

  # rtmb_corr
  fit_c <- rtmb_corr(mtcars[, c("mpg", "wt")])
  res_c <- fit_c$optimize()
  expect_true(inherits(res_c, "map_fit"))
  
  # rtmb_fa
  # Added a simple fallback to handle mtcars subset
  fit_f <- rtmb_fa(scale(mtcars[, c("mpg", "disp", "hp")]), nfactors = 1)
  res_f <- fit_f$optimize()
  expect_true(inherits(res_f, "map_fit"))
})

test_that("Wrappers sample correctly (skip on CRAN)", {
  skip_on_cran()
  
  fit_lm <- rtmb_lm(mpg ~ wt, data = mtcars)
  # Extremely short sampling just to test code path
  res_lm <- fit_lm$sample(chains = 1, sampling = 10, warmup = 10)
  expect_true(inherits(res_lm, "mcmc_fit"))
})

library(testthat)
library(BayesRTMB)

test_that("classic ttest matches stats::t.test with equal variances", {
  set.seed(1)
  Y1 <- rnorm(100, mean = 4.1, sd = 1.0)
  Y2 <- rnorm(100, mean = 4.4, sd = 1.0)

  mdl <- rtmb_ttest(Y1, Y2, prior = prior_flat(), var.equal = TRUE)
  fit <- mdl$classic()

  # Use raw results for precise comparison (combine all components)
  all_dfs <- list()
  if (is.data.frame(fit$df_fixed)) all_dfs$fixed <- fit$df_fixed
  if (is.data.frame(fit$random_effects)) all_dfs$random <- fit$random_effects
  if (is.data.frame(fit$df_transform)) all_dfs$transform <- fit$df_transform
  tab <- do.call(rbind, unname(all_dfs))
  
  tt <- t.test(Y1, Y2, var.equal = TRUE)

  est <- unname(tt$estimate[1] - tt$estimate[2])
  se <- abs(est / unname(tt$statistic))

  expect_equal(tab["diff", "Estimate"], est, tolerance = 1e-6)
  expect_equal(tab["diff", "Std. Error"], se, tolerance = 1e-5)
  expect_equal(tab["diff", "Lower 95%"], unname(tt$conf.int[1]), tolerance = 1e-5)
  expect_equal(tab["diff", "Upper 95%"], unname(tt$conf.int[2]), tolerance = 1e-5)
  expect_equal(tab["diff", "df"], unname(tt$parameter), tolerance = 1e-8)
  
  t_val <- tab["diff", "Estimate"] / tab["diff", "Std. Error"]
  p_val <- 2 * pt(-abs(t_val), df = tab["diff", "df"])
  expect_equal(p_val, tt$p.value, tolerance = 1e-5)
  
  # Verify group means CI (derived quantities)
  n1 <- length(Y1); n2 <- length(Y2)
  s1 <- var(Y1); s2 <- var(Y2)
  sp <- sqrt(((n1-1)*s1 + (n2-1)*s2) / (n1+n2-2))
  se0 <- sp * sqrt(1/n1)
  se1 <- sp * sqrt(1/n2)
  df_t <- n1 + n2 - 2
  crit_t <- qt(0.975, df = df_t)
  
  expect_equal(tab["mean0", "Estimate"], mean(Y1), tolerance = 1e-6)
  expect_equal(tab["mean0", "Lower 95%"], mean(Y1) - crit_t * se0, tolerance = 1e-5)
  expect_equal(tab["mean1", "Estimate"], mean(Y2), tolerance = 1e-6)
  expect_equal(tab["mean1", "Lower 95%"], mean(Y2) - crit_t * se1, tolerance = 1e-5)

  coefs <- fit$summary(max_rows = 100)$coefficients
  expect_false("mean" %in% rownames(coefs))
  expect_true("total_mean" %in% rownames(coefs))
  expect_false(is.na(coefs["diff", "t value"]))
  expect_true(is.na(coefs["total_mean", "t value"]))
  expect_equal(coefs["total_mean", "Pr"], "")
  expect_equal(coefs["sd", "Pr"], "")
})

test_that("classic paired ttest matches stats::t.test paired", {
  set.seed(1)
  Y1 <- rnorm(30, mean = 1)
  Y2 <- Y1 + rnorm(30, mean = 0.2, sd = 1)

  mdl <- rtmb_ttest(Y1, Y2, prior = prior_flat(), paired = TRUE)
  fit <- mdl$classic()

  # Use raw results for precise comparison
  all_dfs <- list()
  if (is.data.frame(fit$df_fixed)) all_dfs$fixed <- fit$df_fixed
  if (is.data.frame(fit$random_effects)) all_dfs$random <- fit$random_effects
  if (is.data.frame(fit$df_transform)) all_dfs$transform <- fit$df_transform
  tab <- do.call(rbind, unname(all_dfs))
  
  tt <- t.test(Y1, Y2, paired = TRUE)

  est <- mean(Y1 - Y2)
  se <- abs(est / unname(tt$statistic))

  expect_equal(tab["diff", "Estimate"], est, tolerance = 1e-6)
  expect_equal(tab["diff", "Std. Error"], se, tolerance = 1e-5)
  expect_equal(tab["diff", "df"], unname(tt$parameter), tolerance = 1e-8)
})

test_that("classic auto df selects Satterthwaite for heteroscedastic t-test", {
  set.seed(1)
  Y1 <- rnorm(40, mean = 4.1, sd = 1.0)
  Y2 <- rnorm(70, mean = 4.4, sd = 2.0)

  mdl <- rtmb_ttest(Y1, Y2, prior = prior_flat(), var.equal = FALSE)
  fit <- mdl$classic()

  expect_equal(fit$df_method, "satterthwaite")
})

test_that("classic heteroscedastic t-test matches optimize with marginal Satterthwaite", {
  set.seed(1)
  Y1 <- rnorm(40, mean = 4.1, sd = 1.0)
  Y2 <- rnorm(70, mean = 4.4, sd = 2.0)

  mdl <- rtmb_ttest(Y1, Y2, prior = prior_flat(), var.equal = FALSE)

  fit_c <- mdl$classic()
  fit_o <- mdl$optimize(
    marginal = mdl$extra$marginal,
    df_method = "satterthwaite"
  )

  # Combine results for comparison
  df_c <- rbind(fit_c$df_fixed, fit_c$df_transform)
  df_o <- rbind(fit_o$df_fixed, fit_o$df_transform)

  expect_equal(df_c["diff", "Estimate"], df_o["diff", "Estimate"], tolerance = 1e-6)
  expect_equal(df_c["diff", "Std. Error"], df_o["diff", "Std. Error"], tolerance = 1e-5)
  expect_equal(df_c["diff", "Lower 95%"], df_o["diff", "Lower 95%"], tolerance = 1e-5)
  expect_equal(df_c["diff", "Upper 95%"], df_o["diff", "Upper 95%"], tolerance = 1e-5)
  expect_equal(df_c["diff", "df"], df_o["diff", "df"], tolerance = 1e-4)
})

test_that("classic heteroscedastic t-test is close to stats Welch t-test", {
  set.seed(1)
  Y1 <- rnorm(40, mean = 4.1, sd = 1.0)
  Y2 <- rnorm(70, mean = 4.4, sd = 2.0)

  mdl <- rtmb_ttest(Y1, Y2, prior = prior_flat(), var.equal = FALSE)
  fit <- mdl$classic()

  # Combine results
  df_c <- rbind(fit$df_fixed, fit$df_transform)
  tt <- t.test(Y1, Y2, var.equal = FALSE)

  expect_equal(df_c["diff", "Estimate"], unname(tt$estimate[1] - tt$estimate[2]), tolerance = 1e-6)
  # Welch and Satterthwaite are close but not identical
  expect_equal(df_c["diff", "Std. Error"], abs(unname(tt$estimate[1] - tt$estimate[2]) / unname(tt$statistic)), tolerance = 1e-3)
  expect_equal(df_c["diff", "df"], unname(tt$parameter), tolerance = 1.0)
})

  # Create a contingency table
  tab <- matrix(c(10, 20, 30, 40), nrow = 2)
  dimnames(tab) <- list(A = c("A1", "A2"), B = c("B1", "B2"))
  
  # Fit a log-linear model (independence model: ~ A + B)
  fit_log <- rtmb_loglinear(~ A + B, data = tab)
  
  # MAP estimation
  map_log <- fit_log$optimize()
  map_log$summary()

.validate_classic_model <- function(self, private) {
  if (is.null(self$extra) || !isTRUE(self$extra$source == "wrapper")) {
    stop(
      "classic() is available only for models created by wrapper functions.",
      call. = FALSE
    )
  }

  if (!identical(self$extra$prior_type, "flat")) {
    stop(
      "classic() is available only for models created with prior_flat(). ",
      "Use optimize() or sample() for models with informative priors.",
      call. = FALSE
    )
  }

  invisible(TRUE)
}

.resolve_classic_settings <- function(self, private, df_method = "auto") {
  type <- self$type %||% "generic"
  has_random_parameters <- any(vapply(self$par_list, function(p) isTRUE(p$random), logical(1)))

  if (is.numeric(df_method)) {
    use_reml <- type %in% c("lm", "lmer", "ttest", "corr", "mediation")
    return(list(
      use_reml = use_reml,
      use_laplace = use_reml || has_random_parameters,
      df_method = "numeric",
      df_value = as.numeric(df_method),
      target_vars = if (use_reml && !is.null(self$extra$marginal)) self$extra$marginal else character(0),
      show_df = TRUE
    ))
  }

  if (is.null(df_method)) df_method <- "auto"
  df_method <- tolower(df_method)

  if (df_method %in% c("infinite", "none")) {
    df_method <- "inf"
  }

  valid <- c("auto", "residual", "satterthwaite", "inf")
  if (!df_method %in% valid) {
    stop(
      "Invalid df_method in classic(). ",
      "Use 'auto', 'residual', 'satterthwaite', or 'inf'.",
      call. = FALSE
    )
  }

  use_reml <- type %in% c("lm", "lmer", "ttest", "corr", "mediation")

  if (identical(df_method, "auto")) {
    df_method <- if (type %in% c("lmer")) {
      "satterthwaite"
    } else if (type == "ttest") {
      # Satterthwaite for independent unequal-variance t-tests
      if (isFALSE(self$extra$paired) && isFALSE(self$extra$var_equal)) {
        "satterthwaite"
      } else {
        "residual"
      }
    } else if (type %in% c("lm", "corr", "mediation")) {
      "residual"
    } else {
      "inf"
    }
  }

  target_vars <- if (use_reml && !is.null(self$extra$marginal)) {
    self$extra$marginal
  } else {
    character(0)
  }

  list(
    use_reml = use_reml,
    use_laplace = use_reml || has_random_parameters,
    df_method = df_method,
    df_value = NULL,
    target_vars = target_vars,
    show_df = !identical(df_method, "inf")
  )
}

.classic_pipeline <- function(self, private,
                             df_method = "auto",
                             se_method = c("wald", "sampling"),
                             num_samples = 1000,
                             seed = 123,
                             view = NULL,
                             map = NULL,
                             fixed = NULL,
                             ...) {
  se_method <- match.arg(se_method)

  if (!is.null(fixed)) {
    return(private$.dispatch_fixed(
      .method_to_call = "classic",
      df_method = df_method,
      se_method = se_method,
      num_samples = num_samples,
      seed = seed,
      view = view,
      map = map,
      fixed = fixed,
      ...
    ))
  }

  private$.validate_classic_model()

  if (isTRUE(self$type == "corr") &&
      (self$extra$corr_method %||% "reml") %in% c("pearson", "spearman")) {
    return(.classic_corr_test_fit(self, method = self$extra$corr_method, view = view))
  }

  settings <- private$.resolve_classic_settings(df_method)
  has_random_parameters <- any(vapply(self$par_list, function(p) isTRUE(p$random), logical(1)))

  # For classic mode, we use .fit_rtmb with apply_prior_correction = FALSE.
  # Constraint targets (fixed parameters for REML) are handled via target_vars.
  model_to_fit <- self

  raw <- model_to_fit$.__enclos_env__$private$.fit_rtmb(
    laplace = settings$use_laplace,
    init = NULL,
    num_estimate = 1,
    control = list(),
    optimizer = "nlminb",
    method = "BFGS",
    map = map,
    se = TRUE,
    reml = settings$use_reml,
    target_vars = settings$target_vars,
    jacobian_target = NULL,
    apply_prior_correction = FALSE
  )

  info_log_lik <- NULL
  restricted_log_lik <- NULL
  if (isTRUE(settings$use_reml)) {
    restricted_log_lik <- -raw$opt$objective
    raw_ic <- model_to_fit$.__enclos_env__$private$.fit_rtmb(
      laplace = has_random_parameters,
      init = raw$par,
      num_estimate = 1,
      control = list(),
      optimizer = "nlminb",
      method = "BFGS",
      map = map,
      se = FALSE,
      reml = FALSE,
      target_vars = character(0),
      jacobian_target = NULL,
      apply_prior_correction = FALSE,
      verbose = FALSE
    )
    info_log_lik <- -raw_ic$opt$objective
  }

  comp <- private$.build_classic_components(
    raw = raw,
    settings = settings,
    se_method = se_method,
    num_samples = num_samples,
    seed = seed,
    info_log_lik = info_log_lik
  )

  fit <- Classic_Fit$new(
    model = self,
    par_vec = unlist(raw$par, use.names = FALSE),
    par = raw$par,
    objective = raw$opt$objective,
    log_lik = comp$log_lik,
    restricted_log_lik = restricted_log_lik,
    convergence = raw$opt$convergence,
    sd_rep = raw$sd_rep,
    df_fixed = comp$df_fixed,
    random_effects = comp$random_effects,
    df_transform = comp$df_transform,
    df_generate = comp$df_generate,
    opt_history = raw$opt_history,
    transform = comp$transform,
    generate = comp$generate,
    se_samples = comp$se_samples,
    par_unc = raw$par_unc,
    vcov_unc = raw$vcov_unc,
    ci_method = se_method,
    laplace = settings$use_laplace,
    map = raw$map,
    test_results = comp$test_results,
    view = view,
    vcov = comp$vcov,
    df_method = settings$df_method,
    idx_fix_active = raw$idx_fix_active,
    show_df = settings$show_df,
    rss = comp$rss,
    df_residual = comp$df_residual
  )

  class(fit) <- unique(c(
    paste0("rtmb_", self$type),
    "Classic_Fit",
    class(fit)
  ))

  fit
}

.classic_corr_test_fit <- function(self, method = c("pearson", "spearman"), view = NULL) {
  method <- match.arg(method)
  Y <- as.matrix(self$data$Y)
  P_y <- self$data$P_y %||% ncol(Y)
  P_x <- self$data$P_x %||% 0
  Y_target <- Y[, seq_len(P_y), drop = FALSE]
  var_names <- colnames(Y_target)
  if (is.null(var_names)) var_names <- paste0("V", seq_len(ncol(Y_target)))

  if (ncol(Y_target) < 2) {
    stop("rtmb_corr() requires at least two numeric variables for correlation tests.", call. = FALSE)
  }

  pairs <- utils::combn(seq_len(ncol(Y_target)), 2)

  if (P_x > 0) {
    missing_arg <- self$extra$missing %||% "listwise"
    if (missing_arg == "pairwise") {
      n_complete <- nrow(Y)
      Y_corr <- if (method == "spearman") {
        apply(Y, 2, rank, na.last = "keep", ties.method = "average")
      } else {
        Y
      }
      R <- stats::cor(Y_corr, use = "pairwise.complete.obs")
    } else {
      ok <- stats::complete.cases(Y)
      Y_complete <- Y[ok, , drop = FALSE]
      n_complete <- nrow(Y_complete)
      if (n_complete <= P_x + 2) {
        stop("Not enough complete observations to compute partial correlations.", call. = FALSE)
      }
      Y_corr <- if (method == "spearman") {
        apply(Y_complete, 2, rank, ties.method = "average")
      } else {
        Y_complete
      }
      R <- stats::cor(Y_corr, use = "complete.obs")
    }
    R_yy <- R[seq_len(P_y), seq_len(P_y), drop = FALSE]
    x_idx <- P_y + seq_len(P_x)
    R_yx <- R[seq_len(P_y), x_idx, drop = FALSE]
    R_xx <- R[x_idx, x_idx, drop = FALSE]
    P_cov <- R_yy - R_yx %*% MASS::ginv(R_xx) %*% t(R_yx)
    D <- diag(1 / sqrt(diag(P_cov)), nrow = P_y, ncol = P_y)
    P_corr <- D %*% P_cov %*% D

    rows <- vector("list", ncol(pairs))
    pcorr_values <- numeric(ncol(pairs))
    row_names <- character(ncol(pairs))
    df_val <- n_complete - P_x - 2

    for (k in seq_len(ncol(pairs))) {
      i <- pairs[1, k]
      j <- pairs[2, k]
      est <- P_corr[i, j]
      est <- pmax(pmin(est, 0.999999), -0.999999)
      pcorr_values[k] <- est

      t_val <- est * sqrt(df_val / pmax(1 - est^2, 1e-12))
      p_val <- 2 * stats::pt(-abs(t_val), df = df_val)
      z_se <- if (n_complete - P_x - 3 > 0) 1 / sqrt(n_complete - P_x - 3) else NA_real_
      if (is.na(z_se)) {
        lower <- upper <- NA_real_
      } else {
        z_val <- atanh(est)
        lower <- tanh(z_val - stats::qnorm(0.975) * z_se)
        upper <- tanh(z_val + stats::qnorm(0.975) * z_se)
      }

      rows[[k]] <- data.frame(
        Estimate = est,
        `Std. Error` = NA_real_,
        `Lower 95%` = lower,
        `Upper 95%` = upper,
        df = df_val,
        `t value` = t_val,
        Pr = p_val,
        check.names = FALSE
      )
      row_names[k] <- if (ncol(Y_target) == 2) {
        "pcorr[rho]"
      } else {
        paste0("pcorr[", var_names[i], ",", var_names[j], "]")
      }
    }

    df_fixed <- do.call(rbind, rows)
    rownames(df_fixed) <- row_names

    pcorr_out <- if (ncol(Y_target) == 2) {
      stats::setNames(pcorr_values, "rho")
    } else {
      out <- diag(1, ncol(Y_target))
      colnames(out) <- rownames(out) <- var_names
      for (k in seq_len(ncol(pairs))) {
        i <- pairs[1, k]
        j <- pairs[2, k]
        out[i, j] <- out[j, i] <- pcorr_values[k]
      }
      out
    }

    fit <- Classic_Fit$new(
      model = self,
      par = list(pcorr = pcorr_out),
      log_lik = NA_real_,
      df_fixed = df_fixed,
      transform = list(pcorr = pcorr_out),
      generate = NULL,
      ci_method = "wald",
      laplace = FALSE,
      test_results = list(cor_test_method = method, partial = TRUE),
      view = view,
      df_method = method,
      show_df = TRUE
    )

    class(fit) <- unique(c(
      "rtmb_corr",
      "Classic_Fit",
      class(fit)
    ))

    return(fit)
  }

  rows <- vector("list", ncol(pairs))
  corr_values <- numeric(ncol(pairs))
  row_names <- character(ncol(pairs))

  for (k in seq_len(ncol(pairs))) {
    i <- pairs[1, k]
    j <- pairs[2, k]
    ok <- stats::complete.cases(Y_target[, i], Y_target[, j])
    x <- Y_target[ok, i]
    y <- Y_target[ok, j]
    ct <- suppressWarnings(stats::cor.test(x, y, method = method))
    est <- unname(ct$estimate)
    corr_values[k] <- est

    lower <- upper <- NA_real_
    if (!is.null(ct$conf.int)) {
      lower <- unname(ct$conf.int[1])
      upper <- unname(ct$conf.int[2])
    }

    row <- data.frame(
      Estimate = est,
      `Std. Error` = NA_real_,
      `Lower 95%` = lower,
      `Upper 95%` = upper,
      check.names = FALSE
    )

    if (method == "pearson") {
      row$df <- unname(ct$parameter %||% (length(x) - 2))
      row$`t value` <- unname(ct$statistic)
    } else {
      row$`S value` <- unname(ct$statistic)
    }
    row$Pr <- ct$p.value

    row_names[k] <- if (ncol(Y_target) == 2) {
      "corr[rho]"
    } else {
      paste0("corr[", var_names[i], ",", var_names[j], "]")
    }
    rows[[k]] <- row
  }

  df_fixed <- do.call(rbind, rows)
  rownames(df_fixed) <- row_names

  corr_out <- if (ncol(Y_target) == 2) {
    stats::setNames(corr_values, "rho")
  } else {
    out <- diag(1, ncol(Y_target))
    colnames(out) <- rownames(out) <- var_names
    for (k in seq_len(ncol(pairs))) {
      i <- pairs[1, k]
      j <- pairs[2, k]
      out[i, j] <- out[j, i] <- corr_values[k]
    }
    out
  }

  fit <- Classic_Fit$new(
    model = self,
    par = list(corr = corr_out),
    log_lik = NA_real_,
    df_fixed = df_fixed,
    transform = NULL,
    generate = NULL,
    ci_method = "wald",
    laplace = FALSE,
    test_results = list(cor_test_method = method),
    view = view,
    df_method = method,
    show_df = method == "pearson"
  )

  class(fit) <- unique(c(
    "rtmb_corr",
    "Classic_Fit",
    class(fit)
  ))

  fit
}

.build_classic_components <- function(self, private, raw, settings, se_method, num_samples, seed, info_log_lik = NULL) {
  # Extract components from raw
  ad_obj         <- raw$ad_obj
  opt            <- raw$opt
  sd_rep         <- raw$sd_rep
  unc_est_vec    <- raw$par_unc
  unc_se_vec     <- raw$se_unc
  Cov_u          <- raw$vcov_unc
  con_est_list   <- raw$par
  target_map     <- raw$map
  idx_fix_active <- raw$idx_fix_active
  idx_ran_full   <- raw$idx_ran_full
  idx_ran        <- raw$idx_ran
  factor_levels_seen <- raw$factor_levels_seen
  unc_est_list   <- raw$par_unc_list
  unc_se_list    <- raw$se_unc_list
  L_u_total      <- raw$L_u_total
  
  se_sampling <- (se_method == "sampling")
  
  # Initialize sensitivity variables for Satterthwaite/delta-method
  dH_list <- NULL
  V_opt <- NULL
  active_idx <- NULL
  dH_theta <- NULL
  V_theta <- NULL
  dH_beta <- NULL
  V_beta <- NULL
  
  # 1. Degrees of Freedom
  est_dfs_all <- rep(Inf, L_u_total)
  
  if (identical(settings$df_method, "numeric")) {
    est_dfs_all <- rep(settings$df_value, L_u_total)
  } else if (identical(settings$df_method, "inf")) {
    est_dfs_all <- rep(Inf, L_u_total)
  } else if (identical(settings$df_method, "residual")) {
    n_obs <- self$get_n_obs()
    
    if (is.null(n_obs) || length(n_obs) == 0 || is.na(n_obs) || n_obs <= 0) {
      if (!is.null(self$extra$nobs)) {
        n_obs <- self$extra$nobs
      } else if (!is.null(self$raw_data)) {
        n_obs <- nrow(as.data.frame(self$raw_data))
      } else if (!is.null(self$data$Y)) {
        n_obs <- length(self$data$Y)
      } else {
        n_obs <- NA_integer_
      }
    }
    
    if (is.na(n_obs) || n_obs <= 0) {
      warning("Could not determine residual degrees of freedom; using infinite df.", call. = FALSE)
      df_res <- Inf
    } else {
      # Calculate effective parameters (fixed effects)
      p_eff <- NA_integer_
      if (!is.null(self$type) && self$type %in% c("lm", "lmer") &&
          !is.null(self$formula) && !is.null(self$raw_data)) {
        p_eff <- tryCatch({
          mf <- stats::model.frame(nobars(self$formula), data = as.data.frame(self$raw_data))
          qr(stats::model.matrix(nobars(self$formula), data = mf))$rank
        }, error = function(e) NA_integer_)
      }
      if (is.na(p_eff)) {
        p_eff <- length(settings$target_vars)
      }
      if (p_eff == 0 && !is.null(self$extra$marginal)) {
        p_eff <- sum(vapply(self$extra$marginal, function(nm) {
          if (nm %in% names(self$par_list)) self$par_list[[nm]]$unc_length else 0
        }, numeric(1)))
      }
      
      if (p_eff == 0) {
        # Fallback to sum of lengths of fixed parameters
        p_eff <- sum(vapply(names(self$par_list), function(nm) {
          p <- self$par_list[[nm]]
          if (!isTRUE(p$random)) p$unc_length else 0
        }, numeric(1)))
      }
      df_res <- pmax(n_obs - p_eff, 1)
    }
    
    # Apply to all non-random parameters
    for (name in names(self$par_list)) {
      if (!isTRUE(self$par_list[[name]]$random)) {
        u_indices <- private$.get_unc_indices(name)
        if (length(u_indices) > 0) est_dfs_all[u_indices] <- df_res
      }
    }
    
    # Apply df_map if present (e.g. for mediation)
    if (!is.null(self$extra$df_map)) {
      df_map <- self$extra$df_map
      df_list_unc <- unconstrained_vector_to_list(est_dfs_all, self$par_list)
      for (v in names(df_list_unc)) {
        v_display <- sub("^b_c", "b", v)
        if (v %in% names(df_map)) {
          df_list_unc[[v]] <- rep(as.numeric(df_map[[v]]), length(df_list_unc[[v]]))
        } else if (v_display %in% names(df_map)) {
          df_list_unc[[v]] <- rep(as.numeric(df_map[[v_display]]), length(df_list_unc[[v]]))
        }
      }
      est_dfs_all <- unlist(df_list_unc, use.names = FALSE)
    }

    if (isTRUE(self$type == "corr") && identical(self$extra$corr_method %||% "reml", "reml")) {
      n_corr <- if (!is.null(self$data$Y) && is.matrix(self$data$Y)) {
        nrow(self$data$Y)
      } else {
        n_obs
      }
      if (!is.na(n_corr) && n_corr > 2) {
        p_y <- self$data$P_y %||% 0
        p_x <- if (!is.null(self$data$Y) && is.matrix(self$data$Y) && p_y > 0) {
          max(ncol(self$data$Y) - p_y, 0)
        } else {
          self$data$P_x %||% 0
        }
        df_reml <- list(
          mean = n_corr - 1,
          sd = n_corr - 2,
          corr = n_corr - 2,
          CF_corr = n_corr - 2,
          pcorr = pmax(n_corr - p_x - 2, 1)
        )
        df_list_unc <- unconstrained_vector_to_list(est_dfs_all, self$par_list)
        for (v in intersect(names(df_list_unc), names(df_reml))) {
          df_list_unc[[v]] <- rep(df_reml[[v]], length(df_list_unc[[v]]))
        }
        est_dfs_all <- unlist(df_list_unc, use.names = FALSE)

        if (is.null(self$extra$df_map)) self$extra$df_map <- list()
        self$extra$df_map$mean <- df_reml$mean
        self$extra$df_map$sd <- df_reml$sd
        self$extra$df_map$corr <- df_reml$corr
        self$extra$df_map$CF_corr <- df_reml$CF_corr
        self$extra$df_map$pcorr <- df_reml$pcorr
      }
    }
  } else if (identical(settings$df_method, "satterthwaite")) {
    satt_res <- self$calculate_satterthwaite_df(
      ad_obj, idx_fix_active, L_u_total, opt$par, 
      silent = FALSE, return_sensitivities = TRUE
    )
    if (is.list(satt_res)) {
      est_dfs_all <- satt_res$df
      dH_list <- satt_res$sensitivities
      V_opt <- satt_res$V
    } else {
      est_dfs_all <- satt_res
    }
    dH_theta <- dH_list
    V_theta <- V_opt
    
    if (settings$use_reml && length(settings$target_vars) > 0 && length(idx_ran) > 0) {
      full_par_names_mapped <- names(ad_obj$env$last.par)[idx_ran]
      target_ran_idx <- which(full_par_names_mapped %in% settings$target_vars)
      if (length(target_ran_idx) > 0) {
        reml_res <- self$calculate_reml_satterthwaite_df(
          ad_obj, opt$par, target_ran_idx, 
          silent = FALSE, return_sensitivities = TRUE
        )
        if (is.list(reml_res)) {
          active_idx <- idx_ran_full[target_ran_idx]
          est_dfs_all[active_idx] <- reml_res$df
          dH_beta <- reml_res$sensitivities
          V_beta <- reml_res$V_beta
        }
      }
    }
    # Important: Reset ad_obj after sensitivity runs
    ad_obj$fn(opt$par)
  }

  # 2. Standard Errors and Confidence Intervals
  con_se_list <- list()
  con_lower_list <- list()
  con_upper_list <- list()
  samps_con <- list()
  samps_tran <- list()
  samps_gq <- list()

  if (se_sampling) {
    # Ported from .inference_pipeline (simplified for readability)
    set.seed(seed)
    # Ensure PD covariance for sampling
    Cov_u_valid <- Cov_u[idx_fix_active, idx_fix_active, drop = FALSE]
    mu_valid <- unc_est_vec[idx_fix_active]
    eig <- eigen(Cov_u_valid, symmetric = TRUE)
    eig$values <- pmax(eig$values, 1e-8)
    Cov_u_pd <- eig$vectors %*% diag(eig$values) %*% t(eig$vectors)
    
    raw_samples <- MASS::mvrnorm(num_samples, mu = mu_valid, Sigma = Cov_u_pd)
    active_dfs <- est_dfs_all[idx_fix_active]
    
    for (j in seq_len(ncol(raw_samples))) {
      df_j <- active_dfs[j]
      if (is.finite(df_j)) {
        sigma_j <- sqrt(Cov_u_pd[j, j])
        z <- (raw_samples[, j] - mu_valid[j]) / (sigma_j + 1e-12)
        w <- rchisq(num_samples, df = df_j)
        raw_samples[, j] <- mu_valid[j] + z * sigma_j * sqrt(df_j / pmax(w, 1e-6))
      }
    }
    
    u_samples_mat <- matrix(rep(unc_est_vec, each = num_samples), nrow = num_samples)
    u_samples_mat[, idx_fix_active] <- raw_samples
    
    # Handle mapped parameters in samples
    idx_curr <- 1
    for (name in names(self$par_list)) {
      L_u <- self$par_list[[name]]$unc_length
      if (L_u > 0) {
        map_f <- if (!is.null(target_map[[name]])) target_map[[name]] else NULL
        for (i in seq_len(L_u)) {
          pos_full <- idx_curr + i - 1
          if (pos_full %in% idx_ran_full) next
          if (!is.null(map_f) && !is.na(map_f[i])) {
            lvl <- as.character(map_f[i])
            mapped_opt_idx <- factor_levels_seen[[lvl]]
            if (!is.null(mapped_opt_idx)) u_samples_mat[, pos_full] <- raw_samples[, mapped_opt_idx]
          }
        }
        idx_curr <- idx_curr + L_u
      }
    }
    
    # Handle random effects in samples (Laplace)
    if (settings$use_laplace && length(idx_ran_full) > 0) {
      for (ridx in idx_ran_full) {
        u_samples_mat[, ridx] <- rnorm(num_samples, mean = unc_est_vec[ridx], sd = unc_se_vec[ridx])
      }
    }
    
    # Transform samples to constrained space and derived variables
    local_data <- self$data
    local_par_list <- self$par_list
    local_transform <- self$transform
    local_generate <- self$generate
    
    for (s in seq_len(num_samples)) {
      u_vec <- u_samples_mat[s, ]
      tmp_con_list <- self$to_constrained(unconstrained_vector_to_list(u_vec, local_par_list))
      
      if (s == 1) {
        for (name in names(tmp_con_list)) samps_con[[name]] <- matrix(NA_real_, num_samples, length(as.numeric(tmp_con_list[[name]])))
      }
      for (name in names(tmp_con_list)) samps_con[[name]][s, ] <- as.numeric(tmp_con_list[[name]])
      
      if (!is.null(local_transform)) {
        tmp_tran <- tryCatch(local_transform(local_data, tmp_con_list), error = function(e) NULL)
        if (!is.null(tmp_tran)) {
          if (s == 1) {
            for (name in names(tmp_tran)) samps_tran[[name]] <- matrix(NA_real_, num_samples, length(as.numeric(tmp_tran[[name]])))
          }
          for (name in names(tmp_tran)) samps_tran[[name]][s, ] <- as.numeric(tmp_tran[[name]])
          tmp_con_list <- c(tmp_con_list, tmp_tran)
        }
      }
      
      if (!is.null(local_generate)) {
        tmp_gq <- tryCatch(local_generate(local_data, tmp_con_list), error = function(e) NULL)
        if (!is.null(tmp_gq)) {
          if (s == 1) {
            for (name in names(tmp_gq)) samps_gq[[name]] <- matrix(NA_real_, num_samples, length(as.numeric(tmp_gq[[name]])))
          }
          for (name in names(tmp_gq)) samps_gq[[name]][s, ] <- as.numeric(tmp_gq[[name]])
        }
      }
    }
    
    # Aggregate sample statistics
    for (name in names(self$par_list)) {
      mat <- samps_con[[name]]
      p_info <- self$par_list[[name]]
      con_se_list[[name]] <- apply(mat, 2, sd, na.rm = TRUE)
      con_lower_list[[name]] <- apply(mat, 2, quantile, 0.025, na.rm = TRUE)
      con_upper_list[[name]] <- apply(mat, 2, quantile, 0.975, na.rm = TRUE)
      if (length(p_info$dim) > 1) {
        dim(con_se_list[[name]]) <- dim(con_lower_list[[name]]) <- dim(con_upper_list[[name]]) <- p_info$dim
      }
    }
  } else {
    # Wald CI
    u_idx_current <- 1
    for (name in names(self$par_list)) {
      p_info <- self$par_list[[name]]
      u_val <- unc_est_list[[name]]
      u_se <- unc_se_list[[name]]
      c_val <- con_est_list[[name]]
      L_u <- p_info$unc_length
      L_c <- p_info$length
      u_indices <- if (L_u > 0) u_idx_current:(u_idx_current + L_u - 1) else integer(0)
      u_idx_current <- u_idx_current + L_u
      
      # Delta method for SE
      transform_single <- function(u) {
        tmp_list <- unc_est_list
        tmp_list[[name]] <- u
        as.numeric(self$to_constrained(tmp_list)[[name]])
      }
      
      J <- matrix(0, L_c, L_u)
      for (i in seq_len(L_u)) {
        u_tmp <- u_val
        u_tmp[i] <- u_tmp[i] + 1e-5
        J[, i] <- (transform_single(u_tmp) - as.numeric(c_val)) / 1e-5
      }
      
      c_se <- numeric(L_c)
      if (L_u > 0) {
        Cov_sub <- Cov_u[u_indices, u_indices, drop = FALSE]
        for (j in seq_len(L_c)) {
          c_se[j] <- sqrt(sum((J[j, ] %*% Cov_sub) * J[j, ], na.rm = TRUE))
        }
      }
      con_se_list[[name]] <- if (length(p_info$dim) > 1) structure(c_se, dim = p_info$dim) else c_se
      
      # CI using t-distribution
      u_dfs <- est_dfs_all[u_indices]
      u_q_95 <- ifelse(is.na(u_dfs) | is.infinite(u_dfs), qnorm(0.975), qt(0.975, df = pmax(u_dfs, 2.1)))
      
      if (p_info$bounds == "corr_matrix" || p_info$type == "corr_matrix") {
        # Fisher's z transform for correlations
        rho_val <- as.numeric(c_val)
        rho_clipped <- pmax(pmin(rho_val, 0.9999), -0.9999)
        z_val <- atanh(rho_clipped)
        z_se <- c_se / (1 - rho_clipped^2 + 1e-10)
        u_q_95_m <- if (any(is.na(u_dfs) | is.infinite(u_dfs))) qnorm(0.975) else qt(0.975, df = pmax(mean(u_dfs, na.rm = TRUE), 2.1))
        c_low <- tanh(z_val - u_q_95_m * z_se)
        c_up <- tanh(z_val + u_q_95_m * z_se)
        is_fixed <- c_se < 1e-10 & abs(rho_val - 1) < 1e-8
        if (any(is_fixed)) { c_low[is_fixed] <- 1.0; c_up[is_fixed] <- 1.0 }
      } else {
        u_low <- u_val - u_q_95 * u_se
        u_up <- u_val + u_q_95 * u_se
        c_low_raw <- transform_single(u_low)
        c_up_raw <- transform_single(u_up)
        c_low <- pmin(c_low_raw, c_up_raw)
        c_up <- pmax(c_low_raw, c_up_raw)
      }
      con_lower_list[[name]] <- if (length(p_info$dim) > 1) structure(c_low, dim = p_info$dim) else c_low
      con_upper_list[[name]] <- if (length(p_info$dim) > 1) structure(c_up, dim = p_info$dim) else c_up
    }
  }

  # 3. Build Summary Tables
  df_list_con <- list()
  df_list_unc <- unconstrained_vector_to_list(est_dfs_all, self$par_list)
  for (name in names(self$par_list)) {
    p <- self$par_list[[name]]
    if (p$unc_length == p$length) {
      df_list_con[[name]] <- df_list_unc[[name]]
    } else {
      df_val <- rep(Inf, p$length)
      if (p$unc_length > 0) {
        m_df <- mean(df_list_unc[[name]], na.rm = TRUE)
        if (is.finite(m_df)) df_val[] <- m_df
      }
      df_list_con[[name]] <- df_val
    }
  }

  build_summary_df <- function(target_random = FALSE) {
    names_vec <- c(); est_vec <- c(); se_vec <- c(); low_vec <- c(); up_vec <- c(); df_vec <- c()
    for (name in names(self$par_list)) {
      p_info <- self$par_list[[name]]
      if (isTRUE(p_info$random) == target_random) {
        f_names <- generate_flat_names(name, p_info$dim, self$par_names[[name]])
        names_vec <- c(names_vec, f_names)
        est_vec <- c(est_vec, as.numeric(con_est_list[[name]]))
        se_vec <- c(se_vec, as.numeric(con_se_list[[name]]))
        low_vec <- c(low_vec, as.numeric(con_lower_list[[name]]))
        up_vec <- c(up_vec, as.numeric(con_upper_list[[name]]))
        df_vec <- c(df_vec, if (!is.null(df_list_con[[name]])) as.numeric(df_list_con[[name]]) else rep(Inf, p_info$length))
      }
    }
    if (length(names_vec) == 0) return(NULL)
    res_df <- data.frame(
      Estimate = est_vec, 
      `Std. Error` = se_vec, 
      `Lower 95%` = low_vec, 
      `Upper 95%` = up_vec, 
      check.names = FALSE
    )
    if (settings$show_df) res_df$df <- df_vec
    rownames(res_df) <- names_vec
    return(res_df)
  }
  
  df_fixed <- build_summary_df(FALSE)
  df_random <- if (settings$use_laplace) build_summary_df(TRUE) else NULL

  # 4. Transform and Generate Tables
  tran_list <- if (!is.null(self$transform)) tryCatch(self$transform(self$data, con_est_list), error = function(e) NULL) else NULL
  gq_list <- if (!is.null(self$generate)) {
    full_con <- if (!is.null(tran_list)) c(con_est_list, tran_list) else con_est_list
    tryCatch(self$generate(self$data, full_con), error = function(e) NULL)
  } else NULL

  # Helper for derived summaries (aligned with .inference_pipeline)
  build_derived_summary <- function(
    func, base_out, is_generate = FALSE, 
    dH_list_in = NULL, V_opt_in = NULL, is_reml_in = FALSE, 
    active_idx_in = NULL, dH_theta_in = NULL, V_theta_in = NULL, 
    theta_idx_in = NULL
  ) {
    if (is.null(func) || is.null(base_out) || length(base_out) == 0) return(NULL)
    flat_base <- unlist(base_out, use.names = FALSE)
    L_out <- length(flat_base)
    names_vec <- c()
    for (name in names(base_out)) {
      names_vec <- c(names_vec, generate_flat_names(name, if (is.null(dim(base_out[[name]]))) length(base_out[[name]]) else dim(base_out[[name]]), self$par_names[[name]]))
    }
    
    # J for Delta method
    J <- matrix(0, L_out, L_u_total)
    for (i in seq_len(L_u_total)) {
      u_tmp <- unc_est_vec
      u_tmp[i] <- u_tmp[i] + 1e-5
      tmp_con <- self$to_constrained(unconstrained_vector_to_list(u_tmp, self$par_list))
      if (is_generate && !is.null(self$transform)) {
        ut <- tryCatch(self$transform(self$data, tmp_con), error = function(e) NULL)
        if (!is.null(ut)) tmp_con <- c(tmp_con, ut)
      }
      tmp_out <- tryCatch(func(self$data, tmp_con), error = function(e) NULL)
      if (!is.null(tmp_out)) J[, i] <- (unlist(tmp_out, use.names = FALSE) - flat_base) / 1e-5
    }
    
    # 1. Degrees of Freedom for derived quantities
    derived_dfs <- rep(Inf, L_out)
    if (settings$show_df) {
      if (identical(settings$df_method, "satterthwaite") && !is.null(dH_theta_in) && !is.null(V_theta_in)) {
        # Use the Satterthwaite delta method helper
        derived_dfs <- .satterthwaite_df_delta(
          J_full = J,
          V_theta = V_theta_in,
          dH_theta = dH_theta_in,
          idx_theta_full = theta_idx_in %||% idx_fix_active,
          V_beta = if (is_reml_in) V_opt_in else NULL,
          dH_beta = if (is_reml_in) dH_list_in else NULL,
          idx_beta_full = if (is_reml_in) active_idx_in else NULL,
          max_df = NULL
        )
      } else if (identical(settings$df_method, "residual")) {
        for (j in seq_len(L_out)) {
          c_dfs <- est_dfs_all[abs(J[j, ]) > 1e-8]
          if (length(c_dfs) > 0) derived_dfs[j] <- min(c_dfs)
        }
      } else if (identical(settings$df_method, "numeric")) {
        derived_dfs[] <- settings$df_value
      }
    }

    if (se_sampling) {
      samps_list <- if (is_generate) samps_gq else samps_tran
      target_names <- names(base_out)
      samps_sub <- samps_list[target_names]
      samps_sub <- samps_sub[!vapply(samps_sub, is.null, logical(1))]

      if (length(samps_sub) == 0) {
        se_out <- low_out <- up_out <- rep(NA_real_, L_out)
      } else {
        mat_all <- do.call(cbind, unname(samps_sub))
        if (ncol(mat_all) != L_out) {
          se_out <- low_out <- up_out <- rep(NA_real_, L_out)
        } else {
          se_out <- apply(mat_all, 2, sd, na.rm = TRUE)
          low_out <- apply(mat_all, 2, quantile, 0.025, na.rm = TRUE)
          up_out <- apply(mat_all, 2, quantile, 0.975, na.rm = TRUE)
        }
      }
    } else {
      se_out <- sqrt(abs(rowSums((J %*% Cov_u) * J)))
      
      # Use t-critical if finite df is available
      crit <- ifelse(is.finite(derived_dfs), 
                     qt(0.975, df = pmax(derived_dfs, 2.1)), 
                     qnorm(0.975))
      low_out <- flat_base - crit * se_out
      up_out <- flat_base + crit * se_out
    }
    
    res_df <- data.frame(
      Estimate = flat_base, 
      `Std. Error` = se_out, 
      `Lower 95%` = low_out, 
      `Upper 95%` = up_out, 
      check.names = FALSE
    )
    if (settings$show_df) {
      res_df$df <- derived_dfs
    }
    rownames(res_df) <- names_vec
    return(res_df)
  }

  df_transform <- build_derived_summary(
    self$transform, tran_list, FALSE,
    if (settings$use_reml) dH_beta else dH_list,
    if (settings$use_reml) V_beta else V_opt,
    settings$use_reml,
    if (settings$use_reml) active_idx else NULL,
    if (settings$use_reml) dH_theta else NULL,
    if (settings$use_reml) V_theta else NULL,
    if (settings$use_reml) idx_fix_active else NULL
  )

  df_generate <- build_derived_summary(
    self$generate, gq_list, TRUE,
    if (settings$use_reml) dH_beta else dH_list,
    if (settings$use_reml) V_beta else V_opt,
    settings$use_reml,
    if (settings$use_reml) active_idx else NULL,
    if (settings$use_reml) dH_theta else NULL,
    if (settings$use_reml) V_theta else NULL,
    if (settings$use_reml) idx_fix_active else NULL
  )

  apply_df_map_to_summary <- function(df) {
    if (is.null(df) || is.null(self$extra$df_map) || !("df" %in% names(df))) {
      return(df)
    }
    rn <- rownames(df)
    if (is.null(rn)) return(df)
    base_names <- gsub("\\[.*\\]$", "", rn)
    for (i in seq_along(base_names)) {
      if (base_names[i] %in% names(self$extra$df_map)) {
        df$df[i] <- as.numeric(self$extra$df_map[[base_names[i]]])
      }
    }
    df
  }

  df_fixed <- apply_df_map_to_summary(df_fixed)
  df_random <- apply_df_map_to_summary(df_random)
  df_transform <- apply_df_map_to_summary(df_transform)
  df_generate <- apply_df_map_to_summary(df_generate)

  # 5. Log-likelihood and Vcov
  log_lik <- if (!is.null(info_log_lik)) info_log_lik else -opt$objective
  rss <- NULL
  df_residual <- NULL
  if (identical(self$type, "lm") && !is.null(con_est_list$sigma) && !is.null(self$data$Y)) {
    n_y <- length(as.numeric(self$data$Y))
    p_fixed <- if (is.null(df_fixed)) 0L else sum(gsub("\\[.*\\]$", "", rownames(df_fixed)) != "sigma")
    df_residual <- n_y - p_fixed
    if (is.finite(df_residual) && df_residual > 0) {
      rss <- as.numeric(con_est_list$sigma)^2 * df_residual
    }
  }


  # Build V_fixed_full (vcov)
  if (is.null(df_fixed) || nrow(df_fixed) == 0) {
    V_fixed_full <- matrix(0, 0, 0)
  } else {
    L_fixed <- nrow(df_fixed)
    J_fixed <- matrix(0, L_fixed, L_u_total)
    base_f <- df_fixed$Estimate
    for (i in seq_len(L_u_total)) {
      u_tmp <- unc_est_vec
      u_tmp[i] <- u_tmp[i] + 1e-5
      tmp_con <- self$to_constrained(unconstrained_vector_to_list(u_tmp, self$par_list))
      tmp_vec <- c()
      for (name in names(self$par_list)) {
        if (!isTRUE(self$par_list[[name]]$random)) {
          tmp_vec <- c(tmp_vec, as.numeric(tmp_con[[name]]))
        }
      }
      if (length(tmp_vec) == L_fixed) J_fixed[, i] <- (tmp_vec - base_f) / 1e-5
    }
    V_fixed_full <- J_fixed %*% Cov_u %*% t(J_fixed)
  }
  rownames(V_fixed_full) <- colnames(V_fixed_full) <- rownames(df_fixed)

  # 6. Additional test results (e.g. for table)
  test_results <- list()
  if (isTRUE(self$type == "table") && !is.null(self$extra$tab)) {
    tab <- self$extra$tab
    test_results$chisq <- stats::chisq.test(tab)
    test_results$fisher <- tryCatch(stats::fisher.test(tab), error = function(e) NULL)
  }

  list(
    df_fixed = df_fixed,
    random_effects = df_random,
    df_transform = df_transform,
    df_generate = df_generate,
    transform = tran_list,
    generate = gq_list,
    se_samples = if (se_sampling) list(con = samps_con, tran = samps_tran, gq = samps_gq) else NULL,
    vcov = V_fixed_full,
    log_lik = log_lik,
    rss = rss,
    df_residual = df_residual,
    test_results = test_results
  )
}

#' Create Initial Values for Multidimensional Unfolding
#'
#' @param Y Numeric matrix or data frame (N rows x M items).
#' @param D Number of unfolding dimensions.
#' @param distance Character; `"squared"` or `"euclidean"`.
#' @param alpha Character; `"random"` for item-specific alpha initial values or
#'   `"fix"` for a common alpha initial value.
#' @param distance_eps Small positive constant added to the distance.
#' @param min_sigma Minimum initial residual standard deviation.
#'
#' @return A named list of initial values.
#' @export
make_init_mdu <- function(Y, D, distance = c("squared", "euclidean"),
                          alpha = c("random", "fix"),
                          distance_eps = 1e-4, min_sigma = 0.1) {
  distance <- match.arg(distance)
  alpha <- match.arg(alpha)
  Y <- as.matrix(Y)
  storage.mode(Y) <- "double"

  N <- nrow(Y)
  M <- ncol(Y)
  if (!is.numeric(D) || length(D) != 1L || D < 1) {
    stop("'D' must be a positive integer.", call. = FALSE)
  }
  D <- as.integer(D)
  if (D >= M) {
    stop("'D' must be smaller than the number of items/columns.", call. = FALSE)
  }

  row_means <- rowMeans(Y, na.rm = TRUE)
  col_means <- colMeans(Y, na.rm = TRUE)
  grand_mean <- mean(Y, na.rm = TRUE)
  row_means[!is.finite(row_means)] <- grand_mean
  col_means[!is.finite(col_means)] <- grand_mean

  Y_centered <- Y -
    matrix(row_means, N, M) -
    matrix(col_means, N, M, byrow = TRUE) +
    grand_mean
  Y_centered[!is.finite(Y_centered)] <- 0

  svd_res <- svd(Y_centered, nu = D, nv = D)
  sv <- pmax(svd_res$d[seq_len(D)], 0)
  D_mat <- diag(sqrt(sv), nrow = D, ncol = D)

  init_theta <- svd_res$u[, seq_len(D), drop = FALSE] %*% D_mat
  init_delta <- svd_res$v[, seq_len(D), drop = FALSE] %*% D_mat

  all_coords <- rbind(init_theta, init_delta)
  scale_factor <- stats::sd(as.vector(all_coords), na.rm = TRUE)
  if (!is.finite(scale_factor) || scale_factor <= 1e-8) scale_factor <- 1

  init_theta <- init_theta / scale_factor
  init_delta <- init_delta / scale_factor

  init_theta <- sweep(init_theta, 2, colMeans(init_theta), "-")
  init_delta <- sweep(init_delta, 2, colMeans(init_delta), "-")

  qr_res <- qr(t(init_delta[seq_len(D), , drop = FALSE]))
  Q <- qr.Q(qr_res)
  R <- qr.R(qr_res)
  sign_diag <- sign(diag(R))
  sign_diag[sign_diag == 0] <- 1
  Q <- sweep(Q, 2, sign_diag, "*")
  init_delta <- init_delta %*% Q
  init_theta <- init_theta %*% Q

  # Match Dim(type = "centered_tri"): zero above the diagonal and center
  # the free lower part of each dimension.
  init_delta_tri <- matrix(0, nrow = M, ncol = D)
  for (d in seq_len(D)) {
    vals <- init_delta[d:M, d]
    vals <- vals - mean(vals)
    init_delta_tri[d:M, d] <- vals
  }
  init_delta <- init_delta_tri
  init_theta <- sweep(init_theta, 2, colMeans(init_theta), "-")

  init_alpha <- if (alpha == "random") {
    a <- colMeans(Y, na.rm = TRUE)
    a[!is.finite(a)] <- mean(Y, na.rm = TRUE)
    as.numeric(a)
  } else {
    mean(Y, na.rm = TRUE)
  }

  D_sq <- matrix(NA_real_, N, M)
  for (i in seq_len(N)) {
    for (j in seq_len(M)) {
      D_sq[i, j] <- sum((init_theta[i, ] - init_delta[j, ])^2)
    }
  }
  D_used <- if (distance == "squared") D_sq + distance_eps else sqrt(D_sq + distance_eps)

  alpha_for_mu <- if (length(init_alpha) == M) {
    matrix(init_alpha, N, M, byrow = TRUE)
  } else {
    matrix(init_alpha, N, M)
  }

  utility_init <- alpha_for_mu - D_used
  init_beta <- sum(Y * utility_init, na.rm = TRUE) / sum(utility_init^2, na.rm = TRUE)
  if (!is.finite(init_beta) || init_beta <= 0) init_beta <- 1
  init_beta <- max(init_beta, 1e-4)

  mu <- init_beta * utility_init
  init_sigma <- apply(Y - mu, 2, stats::sd, na.rm = TRUE)
  y_sd <- stats::sd(as.numeric(Y), na.rm = TRUE)
  if (!is.finite(y_sd) || y_sd <= 0) y_sd <- 1
  init_sigma[!is.finite(init_sigma)] <- y_sd
  init_sigma <- pmax(init_sigma, min_sigma)

  list(
    alpha = init_alpha,
    beta = init_beta,
    sigma = init_sigma,
    theta = init_theta,
    delta = init_delta
  )
}

.validate_mds_distance_matrix <- function(data) {
  Y <- as.matrix(data)
  storage.mode(Y) <- "double"

  if (!is.matrix(Y) || nrow(Y) != ncol(Y)) {
    stop("For method = 'MDS', 'data' must be a square distance matrix.", call. = FALSE)
  }
  if (anyNA(Y)) {
    stop("For method = 'MDS', 'data' must not contain missing values.", call. = FALSE)
  }
  if (any(!is.finite(Y))) {
    stop("For method = 'MDS', 'data' must contain only finite values.", call. = FALSE)
  }
  if (any(Y < 0)) {
    stop("For method = 'MDS', 'data' must be a non-negative distance matrix.", call. = FALSE)
  }
  if (!isTRUE(all.equal(Y, t(Y), tolerance = 1e-8, check.attributes = FALSE))) {
    stop("For method = 'MDS', 'data' must be a symmetric distance matrix.", call. = FALSE)
  }
  if (any(abs(diag(Y)) > 1e-8)) {
    stop("For method = 'MDS', the diagonal of the distance matrix must be zero.", call. = FALSE)
  }
  Y
}

.to_centered_tri_init <- function(coords, D) {
  coords <- as.matrix(coords)
  N <- nrow(coords)
  if (ncol(coords) < D) {
    coords <- cbind(coords, matrix(0, nrow = N, ncol = D - ncol(coords)))
  }
  coords <- coords[, seq_len(D), drop = FALSE]
  coords[!is.finite(coords)] <- 0
  coords <- sweep(coords, 2, colMeans(coords), "-")

  # Rotate the leading D x D block into a Cholesky-like lower-triangular
  # orientation so the constrained initial values match type = "centered_tri".
  if (N >= D) {
    qr_res <- qr(t(coords[seq_len(D), , drop = FALSE]))
    Q <- qr.Q(qr_res)
    R <- qr.R(qr_res)
    sign_diag <- sign(diag(R))
    sign_diag[sign_diag == 0] <- 1
    Q <- sweep(Q, 2, sign_diag, "*")
    coords <- coords %*% Q
  }

  coords_tri <- matrix(0, nrow = N, ncol = D)
  for (d in seq_len(D)) {
    vals <- coords[d:N, d]
    vals <- vals - mean(vals)
    coords_tri[d:N, d] <- vals
  }
  coords_tri
}

.make_init_mds <- function(Y, D, distance = c("euclidean", "squared"), min_sigma = 0.1) {
  distance <- match.arg(distance)
  coords <- tryCatch(
    stats::cmdscale(stats::as.dist(Y), k = D, eig = FALSE),
    error = function(e) NULL
  )
  if (is.null(coords)) {
    coords <- matrix(stats::rnorm(nrow(Y) * D, sd = 0.1), nrow = nrow(Y), ncol = D)
  }
  delta <- .to_centered_tri_init(coords, D)

  pred <- matrix(0, nrow = nrow(Y), ncol = ncol(Y))
  for (i in seq_len(nrow(Y) - 1L)) {
    for (j in (i + 1L):nrow(Y)) {
      d_sq <- sum((delta[i, ] - delta[j, ])^2)
      pred[i, j] <- if (distance == "squared") d_sq else sqrt(d_sq)
    }
  }
  obs <- Y[upper.tri(Y)]
  fitted <- pred[upper.tri(pred)]
  sigma <- stats::sd(obs - fitted, na.rm = TRUE)
  if (!is.finite(sigma) || sigma <= 0) sigma <- stats::sd(obs, na.rm = TRUE)
  if (!is.finite(sigma) || sigma <= 0) sigma <- 1
  tau <- stats::sd(as.numeric(delta), na.rm = TRUE)
  if (!is.finite(tau) || tau <= 0) tau <- 1

  list(delta = delta, tau = tau, sigma = max(sigma, min_sigma))
}

#' Make Best and Worst Responses from Best-Worst Pair Indices
#'
#' @description
#' Converts a matrix of Best-Worst pair indices (`Y_dif`) into separate `Best`
#' and `Worst` data frames. The returned values are positions within each row of
#' `sets`, not the item labels stored in `sets`.
#'
#' @param Y_dif Matrix or data frame of pair indices (N persons x P tasks).
#'   Each value must be an integer from 1 to `C * (C - 1)`, where `C` is the
#'   number of items per task.
#' @param sets Matrix or data frame of presented item sets (P tasks x C items).
#'
#' @return A list with two data frames: `Best` and `Worst`.
#' @export
make_bw_from_ydif <- function(Y_dif, sets) {
  Y_dif <- .as_mdu_integer_matrix(Y_dif, "Y_dif")
  sets <- .as_mdu_integer_matrix(sets, "sets")

  N <- nrow(Y_dif)
  P <- ncol(Y_dif)
  C <- ncol(sets)

  if (nrow(sets) != P) {
    stop("nrow(sets) must be equal to ncol(Y_dif).", call. = FALSE)
  }
  if (C < 2) {
    stop("'sets' must have at least two columns.", call. = FALSE)
  }
  if (anyNA(Y_dif)) {
    stop("'Y_dif' cannot contain missing values.", call. = FALSE)
  }

  idx_i <- rep(seq_len(C), each = C)
  idx_j <- rep(seq_len(C), times = C)
  mask <- idx_i != idx_j
  best_rel <- idx_i[mask]
  worst_rel <- idx_j[mask]

  Y_dif_int <- matrix(as.integer(Y_dif), nrow = N, ncol = P)
  max_pair <- C * (C - 1)
  if (any(Y_dif_int < 1 | Y_dif_int > max_pair)) {
    stop("'Y_dif' must contain values from 1 to C * (C - 1).", call. = FALSE)
  }

  Best <- matrix(NA, nrow = N, ncol = P)
  Worst <- matrix(NA, nrow = N, ncol = P)
  for (n in seq_len(N)) {
    for (p in seq_len(P)) {
      ans <- Y_dif_int[n, p]
      Best[n, p] <- best_rel[ans]
      Worst[n, p] <- worst_rel[ans]
    }
  }

  colnames(Best) <- colnames(Y_dif)
  colnames(Worst) <- colnames(Y_dif)
  rownames(Best) <- rownames(Y_dif)
  rownames(Worst) <- rownames(Y_dif)

  list(
    Best = as.data.frame(Best),
    Worst = as.data.frame(Worst)
  )
}

#' @rdname make_bw_from_ydif
#' @export
restore_bw_from_ydif <- make_bw_from_ydif

#' Make Best-Worst Pair Indices from Best and Worst Responses
#'
#' @description
#' Converts separate `Best` and `Worst` response matrices into `Y_dif` pair
#' indices for Best-Worst MDU models. The returned index is the position of the
#' ordered `(best, worst)` pair among all `C * (C - 1)` position pairs generated
#' from each row of `sets`.
#'
#' @param Best Matrix or data frame of best responses (N persons x P tasks).
#'   Values must be positions within the corresponding row of `sets`, from
#'   `1` to `ncol(sets)`.
#' @param Worst Matrix or data frame of worst responses (N persons x P tasks).
#'   Values must be positions within the corresponding row of `sets`, from
#'   `1` to `ncol(sets)`.
#' @param sets Matrix or data frame of presented item sets (P tasks x C items).
#'
#' @return An integer matrix of pair indices (`Y_dif`).
#' @export
make_ydif_from_bw <- function(Best, Worst, sets) {
  Best <- .as_mdu_integer_matrix(Best, "Best")
  Worst <- .as_mdu_integer_matrix(Worst, "Worst")
  sets <- .as_mdu_integer_matrix(sets, "sets")

  if (!identical(dim(Best), dim(Worst))) {
    stop("'Best' and 'Worst' must have the same dimensions.", call. = FALSE)
  }
  if (anyNA(Best) || anyNA(Worst)) {
    stop("'Best' and 'Worst' cannot contain missing values.", call. = FALSE)
  }

  N <- nrow(Best)
  P <- ncol(Best)
  C <- ncol(sets)
  if (nrow(sets) != P) {
    stop("nrow(sets) must be equal to ncol(Best).", call. = FALSE)
  }
  if (C < 2) {
    stop("'sets' must have at least two columns.", call. = FALSE)
  }

  best_dimnames <- dimnames(Best)
  Best <- matrix(as.integer(Best), nrow = N, ncol = P, dimnames = best_dimnames)
  Worst <- matrix(as.integer(Worst), nrow = N, ncol = P, dimnames = dimnames(Worst))
  if (any(Best < 1 | Best > C)) {
    stop("Each Best response must be a position from 1 to ncol(sets).", call. = FALSE)
  }
  if (any(Worst < 1 | Worst > C)) {
    stop("Each Worst response must be a position from 1 to ncol(sets).", call. = FALSE)
  }

  idx_i <- rep(seq_len(C), each = C)
  idx_j <- rep(seq_len(C), times = C)
  mask <- idx_i != idx_j
  best_rel <- idx_i[mask]
  worst_rel <- idx_j[mask]

  Y_dif <- matrix(NA_integer_, nrow = N, ncol = P)
  for (n in seq_len(N)) {
    for (p in seq_len(P)) {
      b_rel <- Best[n, p]
      w_rel <- Worst[n, p]
      if (b_rel == w_rel) {
        stop("Best and Worst responses cannot be the same item within a task.", call. = FALSE)
      }
      Y_dif[n, p] <- which(best_rel == b_rel & worst_rel == w_rel)
    }
  }

  colnames(Y_dif) <- colnames(Best)
  rownames(Y_dif) <- rownames(Best)
  Y_dif
}

.as_mdu_integer_matrix <- function(x, name) {
  if (is.data.frame(x)) x <- as.matrix(x)
  if (!is.matrix(x)) {
    stop("'", name, "' must be a matrix or data frame.", call. = FALSE)
  }
  x
}

.normalize_mdu_choice_data <- function(data, sets, method) {
  if (is.null(sets)) {
    stop("'sets' must be supplied when method is 'Best' or 'Best-Worst'.", call. = FALSE)
  }
  y_dif_input <- method == "Best-Worst" && !is.list(data)
  if (!is.list(data) && !y_dif_input) {
    stop("For choice MDU, 'data' must be a list containing 'Best' and optionally 'Worst'.", call. = FALSE)
  }
  if (!y_dif_input && is.null(data$Best)) {
    stop("'data$Best' is required when method is 'Best' or 'Best-Worst'.", call. = FALSE)
  }
  if (!y_dif_input && method == "Best-Worst" && is.null(data$Worst)) {
    stop("'data$Worst' is required when method is 'Best-Worst'.", call. = FALSE)
  }

  S_raw <- .as_mdu_integer_matrix(sets, "sets")
  if (anyNA(S_raw)) stop("'sets' cannot contain missing values.", call. = FALSE)
  P <- nrow(S_raw)
  C <- ncol(S_raw)
  if (P < 1 || C < 2) {
    stop("'sets' must have at least one row and at least two columns.", call. = FALSE)
  }

  set_values <- as.vector(S_raw)
  use_names <- is.character(S_raw) || is.factor(S_raw)
  if (use_names) {
    item_names <- unique(as.character(set_values))
    S <- matrix(match(as.character(S_raw), item_names), nrow = P, ncol = C)
  } else {
    S <- matrix(as.integer(S_raw), nrow = P, ncol = C)
    if (any(!is.finite(S)) || any(S < 1)) {
      stop("'sets' must contain positive item indices or item names.", call. = FALSE)
    }
    item_names <- paste0("Item", seq_len(max(S)))
  }
  if (any(apply(S, 1, function(z) anyDuplicated(z) > 0L))) {
    stop("Each row of 'sets' must contain distinct items.", call. = FALSE)
  }
  M <- length(item_names)

  idx_i <- rep(seq_len(C), each = C)
  idx_j <- rep(seq_len(C), times = C)
  mask <- idx_i != idx_j
  best_rel_all <- idx_i[mask]
  worst_rel_all <- idx_j[mask]

  if (y_dif_input) {
    Y_dif <- .as_mdu_integer_matrix(data, "data")
    Y_dif <- matrix(as.integer(Y_dif), nrow = nrow(Y_dif), ncol = ncol(Y_dif))
    if (ncol(Y_dif) != P) {
      stop("'data' must have the same number of columns as nrow(sets).", call. = FALSE)
    }
    if (anyNA(Y_dif)) {
      stop("'data' cannot contain missing values when supplied as Y_dif.", call. = FALSE)
    }
    max_pair <- C * (C - 1)
    if (any(Y_dif < 1 | Y_dif > max_pair)) {
      stop(
        "'data' as Y_dif must contain pair indices from 1 to C * (C - 1).",
        call. = FALSE
      )
    }

    N <- nrow(Y_dif)
    Y_best <- matrix(NA_integer_, N, P)
    score_mat <- matrix(0, N, M)
    for (n in seq_len(N)) {
      for (p in seq_len(P)) {
        ans <- Y_dif[n, p]
        b <- S[p, best_rel_all[ans]]
        w <- S[p, worst_rel_all[ans]]
        Y_best[n, p] <- best_rel_all[ans]
        score_mat[n, b] <- score_mat[n, b] + 1
        score_mat[n, w] <- score_mat[n, w] - 1
      }
    }

    return(list(
      Y_best = Y_best,
      Y_dif = Y_dif,
      S = S,
      Best = NULL,
      Worst = NULL,
      score_mat = score_mat,
      N = N,
      M = M,
      P = P,
      C = C,
      item_names = item_names,
      input = "Y_dif"
    ))
  }

  convert_response <- function(x, name, as_position = FALSE) {
    x <- .as_mdu_integer_matrix(x, paste0("data$", name))
    if (ncol(x) != P) {
      stop("'", name, "' must have the same number of columns as nrow(sets).", call. = FALSE)
    }
    if (as_position) {
      if (is.character(x) || is.factor(x)) {
        stop("'", name, "' must contain set positions from 1 to ncol(sets).", call. = FALSE)
      }
      res <- matrix(as.integer(x), nrow = nrow(x), ncol = ncol(x))
      if (any(!is.na(res) & (res < 1 | res > C))) {
        stop("'", name, "' must contain set positions from 1 to ncol(sets).", call. = FALSE)
      }
      return(res)
    }
    if (use_names || is.character(x) || is.factor(x)) {
      res <- matrix(match(as.character(x), item_names), nrow = nrow(x), ncol = ncol(x))
    } else {
      res <- matrix(as.integer(x), nrow = nrow(x), ncol = ncol(x))
    }
    if (any(!is.na(res) & (res < 1 | res > M))) {
      stop("'", name, "' contains item indices outside the item range in 'sets'.", call. = FALSE)
    }
    res
  }

  position_input <- method == "Best-Worst"
  Best <- convert_response(data$Best, "Best", as_position = position_input)
  if (anyNA(Best)) {
    stop("'data$Best' cannot contain missing values for choice MDU.", call. = FALSE)
  }
  N <- nrow(Best)
  Worst <- NULL
  if (method == "Best-Worst") {
    Worst <- convert_response(data$Worst, "Worst", as_position = TRUE)
    if (anyNA(Worst)) {
      stop("'data$Worst' cannot contain missing values for Best-Worst MDU.", call. = FALSE)
    }
    if (!identical(dim(Best), dim(Worst))) {
      stop("'data$Best' and 'data$Worst' must have the same dimensions.", call. = FALSE)
    }
  }

  Y_best <- matrix(NA_integer_, N, P)
  Y_dif <- if (method == "Best-Worst") make_ydif_from_bw(Best, Worst, S) else matrix(NA_integer_, N, P)
  score_mat <- matrix(0, N, M)

  for (n in seq_len(N)) {
    for (p in seq_len(P)) {
      b_rel <- Best[n, p]
      if (is.na(b_rel)) next
      Y_best[n, p] <- b_rel
      b <- S[p, b_rel]
      score_mat[n, b] <- score_mat[n, b] + 1

      if (method == "Best-Worst") {
        w <- S[p, Worst[n, p]]
        score_mat[n, w] <- score_mat[n, w] - 1
      }
    }
  }

  list(
    Y_best = Y_best,
    Y_dif = Y_dif,
    S = S,
    Best = Best,
    Worst = Worst,
    score_mat = score_mat,
    N = N,
    M = M,
    P = P,
    C = C,
    item_names = item_names,
    input = if (method == "Best-Worst") "Best-Worst" else "Best"
  )
}

.make_init_mdu_choice <- function(score_mat, D, min_lambda = 1e-4) {
  N <- nrow(score_mat)
  M <- ncol(score_mat)
  if (D >= M) {
    stop("'ndim' must be smaller than the number of items.", call. = FALSE)
  }

  alpha_score <- colSums(score_mat, na.rm = TRUE)
  alpha_init <- as.numeric(scale(alpha_score, center = TRUE, scale = FALSE)) * 0.1

  score_centered <- scale(score_mat, center = TRUE, scale = FALSE)
  score_centered <- t(scale(t(score_centered), center = TRUE, scale = FALSE))
  score_centered[!is.finite(score_centered)] <- 0

  svd_res <- svd(score_centered, nu = D, nv = D)
  sv <- pmax(svd_res$d[seq_len(D)], 0)
  theta_temp <- sweep(svd_res$u[, seq_len(D), drop = FALSE], 2, sqrt(sv), "*")
  delta_temp <- sweep(svd_res$v[, seq_len(D), drop = FALSE], 2, sqrt(sv), "*")

  qr_obj <- qr(t(delta_temp))
  Q_rot <- qr.Q(qr_obj)
  R_rot <- qr.R(qr_obj)
  sign_diag <- sign(diag(R_rot))
  sign_diag[sign_diag == 0] <- 1
  Q_rot <- sweep(Q_rot, 2, sign_diag, "*")
  delta_rot <- delta_temp %*% Q_rot
  theta_rot <- theta_temp %*% Q_rot

  delta_sd <- apply(delta_rot, 2, stats::sd, na.rm = TRUE)
  delta_sd[!is.finite(delta_sd) | delta_sd <= 1e-8] <- 1
  delta_scaled <- sweep(delta_rot, 2, delta_sd, "/")
  theta_scaled <- sweep(theta_rot, 2, delta_sd, "/")

  init_delta <- matrix(0, nrow = M, ncol = D)
  for (d in seq_len(D)) {
    vals <- delta_scaled[d:M, d]
    vals <- vals - mean(vals)
    init_delta[d:M, d] <- vals
  }
  init_theta <- sweep(theta_scaled, 2, colMeans(theta_scaled), "-")

  lambda_init <- 1
  if (!is.finite(lambda_init) || lambda_init < min_lambda) lambda_init <- min_lambda

  list(
    alpha_raw = alpha_init,
    delta = init_delta,
    theta = init_theta,
    sigma_alpha = 1,
    lambda = lambda_init
  )
}

#' RTMB-based Multidimensional Unfolding Wrapper
#'
#' @description
#' Fits a multidimensional unfolding model for preference/rating data. Rows are
#' persons or observations and columns are stimuli/items. The model represents
#' both row scores (`theta`) and item locations (`delta`) in a shared D-dimensional
#' space.
#'
#' @param data Numeric matrix or data frame (N rows x M items) for
#'   `method = "rating"`. For choice methods, a list containing `Best` and,
#'   for `method = "Best-Worst"`, `Worst`. For `method = "Best-Worst"`, a
#'   single matrix/data frame is treated as pre-coded `Y_dif` pair indices.
#' @param ndim Number of unfolding dimensions.
#' @param distance Character; `"squared"` uses squared Euclidean distance
#'   (default, often easier for optimization), while `"euclidean"` uses Euclidean
#'   distance.
#' @param alpha Character; `"random"` estimates item-specific alpha values as
#'   random effects (default), while `"fix"` estimates a single common alpha.
#' @param method Character; `"rating"` for continuous ratings, `"Best"` for
#'   best-only choice tasks, `"Best-Worst"` for best-worst choice tasks, or
#'   `"MDS"` for fitting a multidimensional scaling model to a distance matrix.
#' @param sets Matrix or data frame of presented item sets (P tasks x C items)
#'   for choice methods.
#' @param prior Prior configuration: `prior_flat()`, `prior_normal()`, or
#'   `prior_weak()`. `prior_flat()` creates a maximum-likelihood model suitable
#'   for `classic()`. The latent coordinates `delta` and `theta` are always
#'   treated as random effects with normal scale priors, similarly to IRT ability
#'   parameters.
#' @param y_range Optional response range for `method = "rating"`. If supplied
#'   with the default flat prior, `prior_weak()` is used. For choice methods
#'   (`"Best"` and `"Best-Worst"`), `prior_weak()` behaves like
#'   `prior_normal()` because there is no observed rating scale.
#' @param init Optional named list of initial values.
#' @param fixed Optional named list of parameter values to fix.
#' @param view Character vector of parameter names to prioritize in summaries.
#' @param distance_eps Small positive constant added to the distance.
#' @param missing Missing value handling strategy: "listwise" (default) or "fiml" (Full Information Maximum Likelihood).
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#'
#' @return An `RTMB_Model` object.
#' @example inst/examples/ex_mdu.R
#' @export
rtmb_mdu <- function(data, ndim = 2,
                     distance = c("squared", "euclidean"),
                     alpha = c("random", "fix"),
                     method = c("rating", "Best", "Best-Worst", "MDS"),
                     sets = NULL,
                     prior = prior_flat(), y_range = NULL,
                     init = NULL, fixed = NULL, view = NULL,
                     distance_eps = 1e-4,
                     missing = c("listwise", "fiml"), WAIC = FALSE) {

  missing_arg <- match.arg(missing)
  distance_missing <- missing(distance)
  method <- match.arg(method)
  if (method == "MDS" && distance_missing) {
    distance <- "euclidean"
    message("method = 'MDS' uses distance = 'euclidean' by default.")
  } else {
    distance <- match.arg(distance)
  }
  alpha_type <- match.arg(alpha)
  if (isTRUE(WAIC) && method == "rating" && missing_arg != "listwise") {
    stop("WAIC = TRUE is currently supported for rating MDU only with missing = 'listwise'.", call. = FALSE)
  }

  if (!is.numeric(ndim) || length(ndim) != 1L || ndim < 1) {
    stop("'ndim' must be a positive integer.", call. = FALSE)
  }
  D <- as.integer(ndim)

  if (method == "MDS") {
    Y <- .validate_mds_distance_matrix(data)
    N <- nrow(Y)
    M <- N
    if (D >= M) {
      stop("'ndim' must be smaller than the size of the distance matrix.", call. = FALSE)
    }
    item_names <- colnames(Y)
    if (is.null(item_names)) item_names <- rownames(Y)
    if (is.null(item_names)) item_names <- paste0("Item", seq_len(M))
    row_names <- rownames(Y)
    if (!is.null(row_names) && !identical(as.character(row_names), as.character(item_names))) {
      item_names <- row_names
    }
    person_names <- character(0)
  } else if (method == "rating") {
    if (is.data.frame(data)) {
      if (!all(sapply(data, is.numeric))) {
        stop("For rating MDU, all variables in the data must be numeric. Character or factor variables are not supported.", call. = FALSE)
      }
    } else if (!is.numeric(data) && !is.logical(data)) {
      stop("For rating MDU, the data matrix must be numeric.", call. = FALSE)
    }
    Y <- as.matrix(data)
    storage.mode(Y) <- "double"
    if (missing_arg == "listwise") {
      Y <- na.omit(Y)
    } else if (anyNA(Y) && missing_arg != "fiml") {
      stop("rtmb_mdu() requires complete data unless missing = 'fiml'.", call. = FALSE)
    }
    N <- nrow(Y)
    M <- ncol(Y)
    if (D >= M) {
      stop("'ndim' must be smaller than the number of items/columns.", call. = FALSE)
    }
    item_names <- colnames(Y)
    if (is.null(item_names)) item_names <- paste0("Item", seq_len(M))
    person_names <- rownames(Y)
    if (is.null(person_names)) person_names <- paste0("Person", seq_len(N))
  } else {
    choice_data <- .normalize_mdu_choice_data(data, sets, method)
    N <- choice_data$N
    M <- choice_data$M
    if (D >= M) {
      stop("'ndim' must be smaller than the number of items.", call. = FALSE)
    }
    item_names <- choice_data$item_names
    person_source <- if (is.list(data)) data$Best else data
    person_names <- rownames(.as_mdu_integer_matrix(person_source, "data"))
    if (is.null(person_names)) person_names <- paste0("Person", seq_len(N))
  }
  dim_names <- paste0("Dim", seq_len(D))

  if (is.null(prior)) prior <- prior_flat()
  if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }
  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), or prior_weak().",
      call. = FALSE
    )
  }

  prior_type <- prior$type
  if (prior_type == "weak" && method == "rating" && is.null(y_range)) {
    stop("When using prior_weak() with method = 'rating', please specify 'y_range' (e.g., y_range = c(1, 7)).", call. = FALSE)
  }

  mu_alpha_init <- if (method == "rating") mean(Y, na.rm = TRUE) else 0
  alpha_sd <- prior$alpha_sd %||% prior$mu_sd %||% 10
  sigma_alpha_rate <- prior$sigma_alpha_rate %||% (1 / 2.5)
  beta_rate <- prior$beta_rate %||% 0.1
  sigma_rate <- prior$sigma_rate %||% 0.1
  delta_sd <- prior$delta_sd %||% prior$tau_sd %||% 1
  theta_sd <- prior$theta_sd %||% 1
  lambda_rate <- prior$lambda_rate %||% (1 / 2.5)

  if (prior_type == "weak" && method == "rating") {
    y_range <- as.numeric(y_range)
    if (length(y_range) != 2L || any(!is.finite(y_range)) || y_range[1] >= y_range[2]) {
      stop("'y_range' must be an increasing numeric vector of length 2.", call. = FALSE)
    }
    sd_ratio <- prior$sd_ratio %||% 0.5
    max_beta <- prior$max_beta %||% 1
    if (!is.finite(sd_ratio) || sd_ratio <= 0) {
      stop("'prior$sd_ratio' must be a positive finite value.", call. = FALSE)
    }
    if (!is.finite(max_beta) || max_beta <= 0) {
      stop("'prior$max_beta' must be a positive finite value.", call. = FALSE)
    }
    delta_sd <- prior$delta_sd %||% 1
    theta_sd <- prior$theta_sd %||% 1
  }

  if (method == "MDS") {
    sigma_rate_mds <- prior$sigma_rate %||% 1
    tau_rate_mds <- prior$tau_rate %||% prior$lambda_rate %||% 1
    dat_mdu <- list(
      Y = Y,
      N = N,
      ndim = D,
      sigma_rate = sigma_rate_mds,
      tau_rate = tau_rate_mds
    )

    setup_ast <- quote({
      N <- nrow(Y)
      D <- ndim
    })

    param_ast <- if (prior_type == "flat") {
      quote({
        delta <- Dim(c(N, D), type = "centered_tri")
        sigma <- Dim(lower = 0)
      })
    } else {
      quote({
        delta <- Dim(c(N, D), type = "centered_tri", random = TRUE)
        tau <- Dim(lower = 0)
        sigma <- Dim(lower = 0)
      })
    }

    loop_ast <- if (distance == "squared") {
      quote(for (i in 1:(N - 1)) {
        for (j in (i + 1):N) {
          d_ij <- squared_distance(delta[i, ], delta[j, ])
          Y[i, j] ~ normal(d_ij, sigma)
        }
      })
    } else {
      quote(for (i in 1:(N - 1)) {
        for (j in (i + 1):N) {
          d_ij <- distance(delta[i, ], delta[j, ])
          Y[i, j] ~ normal(d_ij, sigma)
        }
      })
    }

    model_exprs <- list(loop_ast)
    if (prior_type != "flat") {
      model_exprs[[length(model_exprs) + 1]] <- quote(delta ~ centered_tri_multi_normal(tau))
      model_exprs[[length(model_exprs) + 1]] <- quote(tau ~ exponential(tau_rate))
      model_exprs[[length(model_exprs) + 1]] <- quote(sigma ~ exponential(sigma_rate))
    }
    model_ast <- as.call(c(list(as.name("{")), model_exprs))

    code_obj <- list(setup = setup_ast, parameters = param_ast, model = model_ast)
    if (isTRUE(WAIC)) {
      pair_ll_ast <- if (distance == "squared") {
        quote({
          log_lik <- numeric(N * (N - 1) / 2)
          idx_ll <- 1
          for (i in 1:(N - 1)) {
            for (j in (i + 1):N) {
              d_ij <- squared_distance(delta[i, ], delta[j, ])
              log_lik[idx_ll] <- normal_lpdf(Y[i, j], d_ij, sigma)
              idx_ll <- idx_ll + 1
            }
          }
        })
      } else {
        quote({
          log_lik <- numeric(N * (N - 1) / 2)
          idx_ll <- 1
          for (i in 1:(N - 1)) {
            for (j in (i + 1):N) {
              d_ij <- distance(delta[i, ], delta[j, ])
              log_lik[idx_ll] <- normal_lpdf(Y[i, j], d_ij, sigma)
              idx_ll <- idx_ll + 1
            }
          }
        })
      }
      code_obj$generate <- .rtmb_waic_generate_ast(NULL, pair_ll_ast)
    }
    code_obj$env <- parent.frame()

    if (is.null(init)) {
      init <- .make_init_mds(Y, D, distance = distance)
      if (prior_type == "flat") init$tau <- NULL
    }

    par_names_list <- list(
      delta = list(item_names, dim_names)
    )
    if (is.null(view)) view <- if (prior_type == "flat") c("delta", "sigma") else c("delta", "tau", "sigma")
  } else if (method == "rating") {
    dat_mdu <- list(
      Y = Y,
      ndim = D,
      distance_eps = distance_eps,
      mu_alpha_init = mu_alpha_init,
      alpha_sd = alpha_sd,
      beta_rate = beta_rate,
      sigma_rate = sigma_rate,
      delta_sd = delta_sd,
      theta_sd = theta_sd,
      sigma_alpha_rate = sigma_alpha_rate
    )
    if (prior_type == "weak") {
      dat_mdu$y_range <- y_range
      dat_mdu$sd_ratio <- sd_ratio
      dat_mdu$max_beta <- max_beta
    }

    setup_ast <- if (prior_type == "weak") {
      quote({
        N <- nrow(Y)
        M <- ncol(Y)
        D <- ndim
        half_range <- diff(y_range) / 2
        mu_alpha_init <- mean(y_range)
        alpha_sd <- half_range
        sigma_rate <- 1 / (half_range * sd_ratio)
        beta_rate <- 1 / max_beta
      })
    } else {
      quote({
        N <- nrow(Y)
        M <- ncol(Y)
        D <- ndim
      })
    }

    param_ast <- if (alpha_type == "random") {
      bquote({
        mu_alpha <- Dim()
        alpha_raw <- Dim(M, random = TRUE)
        sigma_alpha <- Dim(1, lower = 0)
        beta <- Dim(lower = 0)
        sigma <- Dim(M, lower = 0)
        delta <- Dim(c(M, D), type = "centered_tri", random = TRUE)
        theta <- Dim(c(N, D), random = TRUE)
      })
    } else {
      bquote({
        alpha <- Dim()
        beta <- Dim(lower = 0)
        sigma <- Dim(M, lower = 0)
        delta <- Dim(c(M, D), type = "centered_tri", random = TRUE)
        theta <- Dim(c(N, D), random = TRUE)
      })
    }

    transform_ast <- if (alpha_type == "random") {
      quote({
        alpha <- mu_alpha + alpha_raw * sigma_alpha
      })
    } else {
      NULL
    }

    loop_ast <- if (distance == "squared" && alpha_type == "random") {
      quote(for (i in 1:N) {
        for (j in 1:M) {
          if (!is.na(Y[i, j])) {
            d_ij <- squared_distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha[j] - d_ij)
            Y[i, j] ~ normal(mu_ij, sigma[j])
          }
        }
      })
    } else if (distance == "squared") {
      quote(for (i in 1:N) {
        for (j in 1:M) {
          if (!is.na(Y[i, j])) {
            d_ij <- squared_distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha - d_ij)
            Y[i, j] ~ normal(mu_ij, sigma[j])
          }
        }
      })
    } else if (alpha_type == "random") {
      quote(for (i in 1:N) {
        for (j in 1:M) {
          if (!is.na(Y[i, j])) {
            d_ij <- distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha[j] - d_ij)
            Y[i, j] ~ normal(mu_ij, sigma[j])
          }
        }
      })
    } else {
      quote(for (i in 1:N) {
        for (j in 1:M) {
          if (!is.na(Y[i, j])) {
            d_ij <- distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha - d_ij)
            Y[i, j] ~ normal(mu_ij, sigma[j])
          }
        }
      })
    }

    model_exprs <- list(loop_ast)
    model_exprs[[length(model_exprs) + 1]] <- quote(delta ~ centered_tri_multi_normal(delta_sd))
    model_exprs[[length(model_exprs) + 1]] <- quote(for (d in 1:D) theta[, d] ~ normal(0, theta_sd))
    if (alpha_type == "random") {
      model_exprs[[length(model_exprs) + 1]] <- quote(alpha_raw ~ normal(0, 1))
      model_exprs[[length(model_exprs) + 1]] <- quote(sigma_alpha ~ exponential(sigma_alpha_rate))
    }
    if (prior_type %in% c("normal", "weak")) {
      if (alpha_type == "fix") {
        model_exprs[[length(model_exprs) + 1]] <- quote(alpha ~ normal(mu_alpha_init, alpha_sd))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- quote(mu_alpha ~ normal(mu_alpha_init, alpha_sd))
      }
      model_exprs[[length(model_exprs) + 1]] <- quote(beta ~ exponential(beta_rate))
      model_exprs[[length(model_exprs) + 1]] <- quote(sigma ~ exponential(sigma_rate))
    }
    model_ast <- as.call(c(list(as.name("{")), model_exprs))
    code_obj <- list(setup = setup_ast, parameters = param_ast)
    if (!is.null(transform_ast)) code_obj$transform <- transform_ast
    code_obj$model <- model_ast
    if (isTRUE(WAIC)) {
      rating_ll_ast <- if (distance == "squared" && alpha_type == "random") {
        quote({
          log_lik <- numeric(N * M)
          idx_ll <- 1
          for (i in 1:N) for (j in 1:M) {
            d_ij <- squared_distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha[j] - d_ij)
            log_lik[idx_ll] <- normal_lpdf(Y[i, j], mu_ij, sigma[j])
            idx_ll <- idx_ll + 1
          }
        })
      } else if (distance == "squared") {
        quote({
          log_lik <- numeric(N * M)
          idx_ll <- 1
          for (i in 1:N) for (j in 1:M) {
            d_ij <- squared_distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha - d_ij)
            log_lik[idx_ll] <- normal_lpdf(Y[i, j], mu_ij, sigma[j])
            idx_ll <- idx_ll + 1
          }
        })
      } else if (alpha_type == "random") {
        quote({
          log_lik <- numeric(N * M)
          idx_ll <- 1
          for (i in 1:N) for (j in 1:M) {
            d_ij <- distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha[j] - d_ij)
            log_lik[idx_ll] <- normal_lpdf(Y[i, j], mu_ij, sigma[j])
            idx_ll <- idx_ll + 1
          }
        })
      } else {
        quote({
          log_lik <- numeric(N * M)
          idx_ll <- 1
          for (i in 1:N) for (j in 1:M) {
            d_ij <- distance(theta[i, ], delta[j, ]) + distance_eps
            mu_ij <- beta * (alpha - d_ij)
            log_lik[idx_ll] <- normal_lpdf(Y[i, j], mu_ij, sigma[j])
            idx_ll <- idx_ll + 1
          }
        })
      }
      code_obj$generate <- .rtmb_waic_generate_ast(NULL, rating_ll_ast)
    }
    code_obj$env <- parent.frame()

    if (is.null(init)) {
      init <- make_init_mdu(Y, D, distance = distance, alpha = alpha_type, distance_eps = distance_eps)
      if (alpha_type == "random") {
        alpha_init <- init$alpha
        sigma_alpha_init <- stats::sd(alpha_init, na.rm = TRUE)
        if (!is.finite(sigma_alpha_init) || sigma_alpha_init <= 1e-8) sigma_alpha_init <- 1
        init$alpha_raw <- (alpha_init - mean(alpha_init, na.rm = TRUE)) / sigma_alpha_init
        init$mu_alpha <- mean(alpha_init, na.rm = TRUE)
        init$sigma_alpha <- sigma_alpha_init
        init$alpha <- NULL
      }
    }

    par_names_list <- list(
      sigma = item_names,
      delta = list(item_names, dim_names),
      theta = list(person_names, dim_names)
    )
    if (alpha_type == "random") par_names_list$alpha_raw <- item_names

    if (is.null(view)) view <- c("alpha", "beta", "sigma", "delta")
  } else {
    dat_mdu <- list(
      sets = choice_data$S,
      ndim = D,
      distance_eps = distance_eps,
      mu_alpha_init = mu_alpha_init,
      alpha_sd = alpha_sd,
      delta_sd = delta_sd,
      theta_sd = theta_sd,
      sigma_alpha_rate = sigma_alpha_rate,
      lambda_rate = lambda_rate
    )
    if (method == "Best") {
      dat_mdu$Y_best <- choice_data$Y_best
    } else if (identical(choice_data$input, "Y_dif")) {
      dat_mdu$Y_dif <- choice_data$Y_dif
    } else {
      dat_mdu$Best <- choice_data$Best
      dat_mdu$Worst <- choice_data$Worst
    }

    setup_ast <- if (method == "Best") {
      quote({
        S <- sets
        N <- nrow(Y_best)
        P <- nrow(S)
        C <- ncol(S)
        M <- max(S)
        D <- ndim
      })
    } else if (identical(choice_data$input, "Y_dif")) {
      quote({
        S <- sets
        N <- nrow(Y_dif)
        P <- nrow(S)
        C <- ncol(S)
        M <- max(S)
        D <- ndim
      })
    } else {
      quote({
        S <- sets
        N <- nrow(Best)
        P <- nrow(S)
        C <- ncol(S)
        M <- max(S)
        D <- ndim
        Y_dif <- make_ydif_from_bw(Best, Worst, S)
      })
    }

    param_ast <- if (alpha_type == "random") {
      bquote({
        delta <- Dim(c(M, D), type = "centered_tri", random = TRUE)
        theta <- Dim(c(N, D), random = TRUE)
        alpha_raw <- Dim(M, type = "centered", random = TRUE)
        sigma_alpha <- Dim(1, lower = 0)
        lambda <- Dim(1, lower = 0)
      })
    } else {
      bquote({
        delta <- Dim(c(M, D), type = "centered_tri", random = TRUE)
        theta <- Dim(c(N, D), random = TRUE)
        alpha <- Dim()
        lambda <- Dim(1, lower = 0)
      })
    }

    transform_ast <- if (alpha_type == "random") {
      quote({
        alpha <- alpha_raw * sigma_alpha
      })
    } else {
      NULL
    }

    utility_ast <- if (distance == "squared" && alpha_type == "random") {
      quote(for (m in 1:M) {
        U[m] <- alpha[m] - squared_distance(theta[n, ], delta[m, ]) - distance_eps
      })
    } else if (distance == "squared") {
      quote(for (m in 1:M) {
        U[m] <- alpha - squared_distance(theta[n, ], delta[m, ]) - distance_eps
      })
    } else if (alpha_type == "random") {
      quote(for (m in 1:M) {
        U[m] <- alpha[m] - distance(theta[n, ], delta[m, ]) - distance_eps
      })
    } else {
      quote(for (m in 1:M) {
        U[m] <- alpha - distance(theta[n, ], delta[m, ]) - distance_eps
      })
    }

    likelihood_ast <- if (method == "Best") {
      bquote(for (n in 1:N) {
        U <- rep(alpha[1] * 0, M)
        .(utility_ast)
        for (p in 1:P) {
          U_set <- U[S[p, ]]
          Y_best[n, p] ~ categorical_logit(lambda * U_set)
        }
      })
    } else {
      bquote(for (n in 1:N) {
        U <- rep(alpha[1] * 0, M)
        .(utility_ast)
        for (p in 1:P) {
          U_set <- U[S[p, ]]
          Y_dif[n, p] ~ bw_categorical_logit(U_set, lambda)
        }
      })
    }

    model_exprs <- list(likelihood_ast)
    model_exprs[[length(model_exprs) + 1]] <- quote(delta ~ centered_tri_multi_normal(delta_sd))
    model_exprs[[length(model_exprs) + 1]] <- quote(for (d in 1:D) theta[, d] ~ normal(0, theta_sd))
    if (alpha_type == "random") {
      model_exprs[[length(model_exprs) + 1]] <- quote(alpha_raw ~ centered_multi_normal(1))
      model_exprs[[length(model_exprs) + 1]] <- quote(sigma_alpha ~ exponential(sigma_alpha_rate))
    } else if (prior_type %in% c("normal", "weak")) {
      model_exprs[[length(model_exprs) + 1]] <- quote(alpha ~ normal(mu_alpha_init, alpha_sd))
    }
    if (prior_type %in% c("normal", "weak")) {
      model_exprs[[length(model_exprs) + 1]] <- quote(lambda ~ exponential(lambda_rate))
    }
    model_ast <- as.call(c(list(as.name("{")), model_exprs))
    code_obj <- list(setup = setup_ast, parameters = param_ast)
    if (!is.null(transform_ast)) code_obj$transform <- transform_ast
    code_obj$model <- model_ast
    if (isTRUE(WAIC)) {
      choice_ll_ast <- if (method == "Best") {
        bquote({
          log_lik <- numeric(N * P)
          idx_ll <- 1
          for (n in 1:N) {
            U <- rep(alpha[1] * 0, M)
            .(utility_ast)
            for (p in 1:P) {
              U_set <- U[S[p, ]]
              log_lik[idx_ll] <- categorical_logit_lpmf(Y_best[n, p], lambda * U_set)
              idx_ll <- idx_ll + 1
            }
          }
        })
      } else {
        bquote({
          log_lik <- numeric(N * P)
          idx_ll <- 1
          for (n in 1:N) {
            U <- rep(alpha[1] * 0, M)
            .(utility_ast)
            for (p in 1:P) {
              U_set <- U[S[p, ]]
              log_lik[idx_ll] <- bw_categorical_logit_lpmf(Y_dif[n, p], U_set, lambda)
              idx_ll <- idx_ll + 1
            }
          }
        })
      }
      code_obj$generate <- .rtmb_waic_generate_ast(NULL, choice_ll_ast)
    }
    code_obj$env <- parent.frame()

    if (is.null(init)) {
      init <- .make_init_mdu_choice(choice_data$score_mat, D)
      if (alpha_type == "fix") init <- init[setdiff(names(init), c("alpha_raw", "sigma_alpha"))]
      if (alpha_type == "fix") init$alpha <- 0
    }

    par_names_list <- list(
      delta = list(item_names, dim_names),
      theta = list(person_names, dim_names)
    )
    if (alpha_type == "random") par_names_list$alpha_raw <- item_names

    if (is.null(view)) {
      view <- if (alpha_type == "random") {
        c("lambda", "sigma_alpha", "alpha", "delta")
      } else {
        c("lambda", "alpha", "delta")
      }
    }
  }

  if (!is.null(init)) {
    if (!is.null(init$phi_raw) && is.null(init$alpha_raw)) {
      init$alpha_raw <- init$phi_raw
      init$phi_raw <- NULL
    }
    if (!is.null(init$sigma_phi) && is.null(init$sigma_alpha)) {
      init$sigma_alpha <- init$sigma_phi
      init$sigma_phi <- NULL
    }
    if (alpha_type == "fix") {
      init$alpha_raw <- NULL
      init$sigma_alpha <- NULL
    }
  }

  obj <- rtmb_model(
    data = dat_mdu,
    code = code_obj,
    par_names = par_names_list,
    init = init,
    fixed = fixed,
    view = view
  )
  obj$type <- "mdu"
  obj$extra <- list(
    source = "wrapper",
    prior_type = prior$type,
    method = method,
    alpha = alpha_type,
    input = if (method == "rating") "rating" else if (method == "MDS") "distance" else choice_data$input,
    distance = distance,
    marginal = if (method == "MDS") "delta" else c("delta", "theta"),
    sets = if (method %in% c("rating", "MDS")) NULL else choice_data$S
  )
  obj
}

  # Prepare a subset of the BigFive dataset for factor analysis
  data(BigFive, package = "BayesRTMB")
  fa_data <- BigFive[, 1:10]

  # --- 1. Standard Exploratory Factor Analysis (1 Factor) ---
  fit_fa1 <- rtmb_fa(data = fa_data, nfactors = 1)

  # Maximum A Posteriori (MAP) estimation
  map_fa1 <- fit_fa1$optimize()
  map_fa1$summary()



  # --- 2. Factor Analysis with Rotation and Factor Scores (2 Factors) ---
  # Extract 2 factors, apply Promax rotation during model fitting, and calculate factor scores
  fit_fa2 <- rtmb_fa(data = fa_data, nfactors = 2, rotate = "promax", score = TRUE)

  # Setting se_method = "sampling" enables standard errors and 95% CIs
  # for transformed and generated quantities, such as factor scores and post-hoc rotations.
  # (It uses multivariate normal sampling from the unconstrained parameter space)
  map_fa2 <- fit_fa2$optimize(se_method = "sampling")
  map_fa2$summary()

  # Post-hoc rotation using the fa_rotate() method
  # You can also apply other rotation methods (e.g., "varimax" from the stats package)
  # to the unrotated loading matrix ("L") after estimation.
  map_fa2$fa_rotate(target = "L", rotate = "varimax")

  # The post-hoc rotated loadings are automatically stored with the method's
  # suffix (e.g., "L_varimax")
  map_fa2$summary("L_varimax")



  # --- 3. Regularized Factor Analysis using Spike-and-Slab Prior (SSP) ---
  # Specifying 'rotate = "ssp"' enables sparse loading matrix estimation
  fit_ssp <- rtmb_fa(data = fa_data, nfactors = 2, rotate = "ssp")

  map_ssp <- fit_ssp$optimize()
  map_ssp$summary()

#' RTMB-based Log-linear analysis (Poisson regression)
#'
#' @description
#' Performs Bayesian or Frequentist log-linear analysis (Poisson regression)
#' on a contingency table or raw data.
#'
#' @param formula A formula (e.g., `~ A + B + A:B`) or a contingency table.
#' @param data A data frame (required if `formula` is used).
#' @param prior An object of class "rtmb_prior" specifying the prior distribution.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments passed to `rtmb_glm()`.
#' @return An `RTMB_Model` object.
#' @example inst/examples/ex_loglinear.R
#' @export
rtmb_loglinear <- function(formula, data, prior = prior_flat(), fixed = NULL,
                           WAIC = FALSE, ...) {

  if (is.null(prior)) prior <- prior_flat()
  if (!inherits(prior, "rtmb_prior")) {
    stop("prior must be an object of class 'rtmb_prior'.", call. = FALSE)
  }
  if (missing(data) || is.null(data)) {
    stop("'data' must be supplied as a data frame, table, or matrix.", call. = FALSE)
  }

  # 1. Data Preparation
  # Convert table or matrix to long data frame
  was_table <- FALSE
  if (inherits(data, c("table", "matrix"))) {
    data <- as.data.frame(as.table(data))
    was_table <- TRUE
  }

  # Identify variables and frequency column
  all_vars <- all.vars(formula)
  response_var <- NULL

  # Check if formula has a response
  if (length(formula) == 3) {
    response_var <- as.character(formula[[2]])
  }

  # If no response is specified
  if (is.null(response_var)) {
    if (was_table && "Freq" %in% names(data)) {
      # For tables/matrices, we automatically use the Freq column
      formula <- update(formula, Freq ~ .)
      response_var <- "Freq"
    } else {
      # For raw data frames, we aggregate
      # Check if variables exist in data
      missing_vars <- setdiff(all_vars, names(data))
      if (length(missing_vars) > 0) {
        stop(sprintf("Variables not found in data: %s", paste(missing_vars, collapse = ", ")))
      }

      # Aggregate raw observations into counts
      data_counts <- as.data.frame(table(data[all_vars]))
      # Update formula to include 'Freq' as response
      formula <- update(formula, Freq ~ .)
      data <- data_counts
      response_var <- "Freq"
    }
  }


  # Ensure response is numeric (counts)
  data[[response_var]] <- as.numeric(data[[response_var]])

  # Call the engine
  # We use rtmb_glmer which handles formulas, random effects, and priors.
  args <- list(...)
  if (!is.null(args$y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }

  # For rtmb_table, we default to prior_weak with Stan-style values if requested
  if (inherits(prior, "rtmb_prior") && identical(prior$type, "weak")) {
    # Stan-style defaults for log-linear (Poisson)
    if (is.null(prior$sd_ratio)) prior$sd_ratio <- 5.0 # For Intercept (alpha_prior_sd)
    if (is.null(prior$max_beta)) prior$max_beta <- 2.5 # For coefficients
  }

  # Call the engine
  # We use rtmb_glmer which handles formulas, random effects, and priors.

  # 2. Determine centering and predictors (to keep generate block clean in print_code)
  use_centering <- prior$type %in% c("normal", "weak", "rhs", "ssp", "jzs")
  is_centered <- use_centering

  # Identify if there are predictors
  X_tmp <- stats::model.matrix(nobars(formula), data[1, , drop = FALSE])
  K <- ncol(X_tmp) - (if (attr(stats::terms(nobars(formula)), "intercept")) 1 else 0)

  if (is_centered) {
    int_name <- as.name("Intercept_c")
    x_name <- as.name("X_c")
  } else {
    int_name <- as.name("Intercept")
    x_name <- as.name("X")
  }

  if (K > 0) {
    gen_block <- bquote({
      eta <- .(int_name) + .(x_name) %*% b
      mu <- exp(eta)
      list(mu = mu)
    })
  } else {
    gen_block <- bquote({
      eta <- rep(.(int_name), N)
      mu <- exp(eta)
      list(mu = mu)
    })
  }

  res <- rtmb_glmer(
    formula = formula,
    data = data,
    family = "poisson",
    prior = prior,
    generate = gen_block, fixed = fixed, WAIC = WAIC,
    ...
  )

  # Tag as loglinear
  res$type <- "loglinear"
  res$extra$source <- "wrapper"
  # res$extra$prior_type is maintained from rtmb_glmer
  res$extra$marginal <- if (is_centered) "Intercept_c" else "Intercept"
  if (K > 0) res$extra$marginal <- c(res$extra$marginal, "b")
  res$extra$obs_Y <- data[[response_var]]
  fit <- res

  # Tag as rtmb_loglinear for potential custom methods
  class(fit) <- c("rtmb_loglinear", class(fit))

  return(fit)
}

#' @importFrom stats .getXlevels ave chisq.test fisher.test kmeans lm na.pass optim prcomp qt rchisq setNames update
#' @importFrom utils write.csv
NULL

`%||%` <- function(a, b) {
  if (!is.null(a)) a else b
}

# Add global variables to avoid R CMD check notes
utils::globalVariables(c("N", "Y", "iteration", "chain", "value", "variable"))

#' Probability Distributions for RTMB Models
#'
#' @description
#' This page summarizes the probability distributions available within the `rtmb_code` block.
#' These functions are designed to be used with the Stan-like sampling syntax (`~`),
#' which internally adds the log-density to the model's total log-posterior.
#'
#' @details
#' \strong{Syntax Styles:}
#' In `rtmb_code`, you can specify distributions in two ways:
#' \itemize{
#'   \item \strong{Sampling Syntax (Recommended):} \code{y ~ normal(mu, sigma)}
#'   \item \strong{Explicit Function Call:} \code{lp <- lp + normal_lpdf(y, mu, sigma)}
#' }
#'
#' \strong{Continuous Distributions (LPDF):}
#' \itemize{
#'   \item \code{normal(mean, sd)}: Normal distribution.
#'   \item \code{lognormal(meanlog, sdlog)}: Lognormal distribution.
#'   \item \code{exponential(rate)}: Exponential distribution.
#'   \item \code{cauchy(location, scale)}: Cauchy distribution.
#'   \item \code{student_t(df, mu, sigma)}: Student's t-distribution.
#'   \item \code{gamma(shape, rate)}: Gamma distribution.
#'   \item \code{inverse_gamma(shape, scale)}: Inverse-gamma distribution.
#'   \item \code{beta(a, b)}: Beta distribution.
#' }
#'
#' \strong{Discrete Distributions (LPMF):}
#' \itemize{
#'   \item \code{bernoulli(prob)} / \code{bernoulli_logit(eta)}: Binary outcomes.
#'   \item \code{binomial(size, prob)} / \code{binomial_logit(size, eta)}: Binomial outcomes.
#'   \item \code{poisson(mean)}: Poisson count data.
#'   \item \code{neg_binomial_2(mu, size)}: Negative binomial (mean/dispersion parameterization).
#'   \item \code{ordered_logistic(eta, cutpoints)}: Ordered categorical outcomes.
#'   \item \code{sequential_logistic(eta, cutpoints)}: Sequential ordered categorical outcomes.
#' }
#'
#' \strong{Multivariate and Matrix Distributions:}
#' \itemize{
#'   \item \code{multi_normal(mean, Sigma)}: Standard multivariate normal distribution.
#'   \item \code{multi_student_t(df, mean, Sigma)}: Multivariate Student's t-distribution.
#'   \item \code{multi_cauchy(mean, Sigma)}: Multivariate Cauchy distribution (Student-t with df=1).
#'   \item \code{lkj_corr(eta)}: LKJ prior for correlation matrices.
#'   \item \code{dirichlet(alpha)}: Dirichlet distribution for simplexes.
#'   \item \code{lower_tri_normal(mean, sd)}: Normal distribution for elements of a lower-triangular matrix.
#'   \item \code{centered_tri_multi_normal(sigma)}: Multivariate normal for centered triangular matrices (used in identification constraints).
#'   \item \code{sufficient_multi_normal_fa(S_mat, N, y_bar, mu, psi, Lambda)}: Factor analysis likelihood using sufficient statistics (highly efficient for large sample sizes).
#' }
#'
#' \strong{Vectorization:}
#' Most univariate distributions are vectorized. If \code{y} and \code{mu} are vectors,
#' \code{y ~ normal(mu, sigma)} will calculate the sum of log-densities for all elements
#' efficiently.
#'
#' @name distributions
#' @family distributions
#' @import RTMB
NULL

#' Normal log-probability density function
#'
#' @param x Vector of quantiles.
#' @param mean Vector of means.
#' @param sd Vector of standard deviations.
#' @param sum Logical; if TRUE (default), returns the sum of log-densities. If FALSE, returns element-wise log-densities.
#' @return The sum of the log-density (scalar) or a vector of log-densities.
#' @keywords internal
normal_lpdf <- function(x, mean, sd, sum = TRUE) {
  res <- dnorm(x, mean = mean, sd = sd, log = TRUE)
  if(sum) sum(res) else res
}

#' Lognormal log-probability density function
#'
#' @param x Vector of quantiles.
#' @param meanlog Mean of the distribution on the log scale.
#' @param sdlog Standard deviation of the distribution on the log scale.
#' @return The sum of the log-density.
#' @keywords internal
lognormal_lpdf <- function(x, meanlog, sdlog, sum = TRUE) {
  res <- suppressWarnings(dlnorm(x, meanlog = meanlog, sdlog = sdlog, log = TRUE))
  if(sum) sum(res) else res
}

#' Exponential log-probability density function
#'
#' @param x Vector of quantiles.
#' @param rate Vector of rates.
#' @return The sum of the log-density.
#' @keywords internal
exponential_lpdf <- function(x, rate, sum = TRUE) {
  res <- dexp(x, rate, log = TRUE)
  if(sum) sum(res) else res
}

#' Half-Normal log-probability density function
#'
#' @param x Vector of quantiles (must be non-negative).
#' @param sd Vector of standard deviations (scale parameter).
#' @return The sum of the log-density.
#' @keywords internal
half_normal_lpdf <- function(x, sd, sum = TRUE) {
  res <- dnorm(x, mean = 0, sd = sd, log = TRUE) + log(2)
  if(sum) sum(res) else res
}

#' Beta log-probability density function
#'
#' @param x Vector of quantiles.
#' @param a Shape parameter alpha.
#' @param b Shape parameter beta.
#' @return The sum of the log-density.
#' @keywords internal
beta_lpdf <- function(x, a, b, sum = TRUE) {
  res <- dbeta(x, a, b, log = TRUE)
  if(sum) sum(res) else res
}

#' Gamma log-probability density function
#'
#' @param x Vector of quantiles.
#' @param shape Shape parameter.
#' @param rate Rate parameter.
#' @return The sum of the log-density.
#' @keywords internal
gamma_lpdf <- function(x, shape, rate, sum = TRUE) {
  res <- suppressWarnings(dgamma(x, shape = shape, rate = rate, log = TRUE))
  if(sum) sum(res) else res
}

#' Inverse-gamma log-probability density function
#'
#' @param x Vector of quantiles.
#' @param shape Shape parameter.
#' @param scale Scale parameter.
#' @return The sum of the log-density.
#' @keywords internal
inverse_gamma_lpdf <- function(x, shape, scale, sum = TRUE) {
  res <- shape * log(scale) - lgamma(shape) - (shape + 1) * log(x) - scale / x
  if(sum) sum(res) else res
}

#' Cauchy log-probability density function
#'
#' @param x Vector of quantiles.
#' @param location Location parameter.
#' @param scale Scale parameter.
#' @return The sum of the log-density.
#' @keywords internal
cauchy_lpdf <- function(x, location, scale, sum = TRUE) {
  res <- suppressWarnings(dcauchy(x, location = location, scale = scale, log = TRUE))
  if(sum) sum(res) else res
}

#' Student-t log-probability density function
#'
#' @param x Vector of quantiles.
#' @param df Degrees of freedom.
#' @param mu Location parameter.
#' @param sigma Scale parameter.
#' @return The sum of the log-density.
#' @keywords internal
student_t_lpdf <- function(x, df, mu = 0, sigma = 1, sum = TRUE) {
  res <- suppressWarnings(dt((x - mu) / sigma, df = df, log = TRUE) - log(sigma))
  if(sum) sum(res) else res
}

#' Laplace log-probability density function
#'
#' @param x Vector of quantiles.
#' @param location Location parameter.
#' @param scale Scale parameter.
#' @return The sum of the log-density.
#' @keywords internal
laplace_lpdf <- function(x, location = 0, scale = 1, sum = TRUE) {
  res <- -log(2 * scale) - sqrt((x - location)^2 + 1e-8) / scale
  if(sum) sum(res) else res
}

#' Logit-Normal log-probability density function
#'
#' @param x Vector of quantiles (must be strictly between 0 and 1).
#' @param mu Vector of means on the logit scale.
#' @param sd Vector of standard deviations on the logit scale.
#' @return The sum of the log-density.
#' @keywords internal
logit_normal_lpdf <- function(x, mu, sd, sum = TRUE) {
  logit_x <- qlogis(x)
  log_jacobian <- -log(x) - log1p(-x)
  res <- dnorm(logit_x, mean = mu, sd = sd, log = TRUE) + log_jacobian
  if(sum) sum(res) else res
}

#' Weibull log-probability density function
#'
#' @param x Vector of quantiles.
#' @param shape Shape parameter.
#' @param scale Scale parameter.
#' @return The sum of the log-density.
#' @keywords internal
weibull_lpdf <- function(x, shape, scale, sum = TRUE) {
  res <- suppressWarnings(dweibull(x, shape = shape, scale = scale, log = TRUE))
  if(sum) sum(res) else res
}

#' Uniform log-probability density function
#'
#' @param x Vector of quantiles.
#' @param a Lower limit of the distribution.
#' @param b Upper limit of the distribution.
#' @return The sum of the log-density.
#' @keywords internal
uniform_lpdf <- function(x, a, b, sum = TRUE) {
  res <- dunif(x, a, b, log = TRUE)
  if(sum) sum(res) else res
}

#' LKJ correlation log-probability density function
#'
#' @param Omega Correlation matrix.
#' @param eta Shape parameter.
#' @return The log-density.
#' @keywords internal
lkj_corr_lpdf <- function(Omega, eta = 1) {
  if (length(Omega) == 1) {
    rho <- Omega
    x <- (rho + 1) / 2
    return(dbeta(x, shape1 = eta, shape2 = eta, log = TRUE) - log(2))
  }
  # Matrix case (3 or more variables)
  K <- nrow(Omega)
  log_C <- 0
  for (k in 1:(K - 1)) {
    log_C <- log_C - (lgamma(eta + (K - 1 - k) / 2) +
                        lgamma(k / 2) -
                        lgamma(eta + (K - 1) / 2))
  }
  if (eta == 1) {
    return(log_C)
  }
  safe_Omega <- Omega + diag(1e-11, K)
  U <- chol(safe_Omega)
  log_det <- 2 * sum(log(diag(U)))
  return(log_C + (eta - 1) * log_det)
}

#' LKJ correlation log-probability density function for Cholesky factors
#'
#' @param CF_Omega Cholesky factor of a correlation matrix.
#' @param eta Shape parameter.
#' @return The log-density.
#' @keywords internal
lkj_CF_corr_lpdf <- function(CF_Omega, eta = 1) {
  K <- nrow(CF_Omega)
  log_C <- 0
  if (K > 1) {
    for (k in 1:(K - 1)) {
      log_C <- log_C - (lgamma(eta + (K - 1 - k) / 2) +
                          lgamma(k / 2) -
                          lgamma(eta + (K - 1) / 2))
    }
  }
  if (eta == 1) {
    return(log_C)
  }
  log_det <- 2 * sum(log(diag(CF_Omega)))
  return(log_C + (eta - 1) * log_det)
}

#' Bernoulli log-probability mass function
#'
#' @param x Vector of binary outcomes (0 or 1).
#' @param prob Probability of success.
#' @return The sum of the log-probability.
#' @keywords internal
bernoulli_lpmf <- function(x, prob, sum = TRUE) {
  res <- dbinom(x, 1, prob, log = TRUE)
  if(sum) sum(res) else res
}

#' Binomial log-probability mass function
#'
#' @param x Vector of quantiles (number of successes).
#' @param size Number of trials.
#' @param prob Probability of success.
#' @return The sum of the log-probability.
#' @keywords internal
binomial_lpmf <- function(x, size, prob, sum = TRUE) {
  res <- dbinom(x, size, prob, log = TRUE)
  if(sum) sum(res) else res
}

#' Bernoulli log-probability mass function with logit parameterization
#'
#' @param x Vector of binary outcomes (0 or 1).
#' @param eta Linear predictor (logit probability).
#' @return The sum of the log-probability.
#' @keywords internal
bernoulli_logit_lpmf <- function(x, eta, sum = TRUE) {
  res <- x * eta - log1p_exp(eta)
  if(sum) sum(res) else res
}

#' Binomial log-probability mass function with logit parameterization
#'
#' @param x Vector of quantiles (number of successes).
#' @param size Number of trials.
#' @param eta Linear predictor (logit probability).
#' @return The sum of the log-probability.
#' @keywords internal
binomial_logit_lpmf <- function(x, size, eta, sum = TRUE) {
  res <- lgamma(size + 1) - lgamma(x + 1) - lgamma(size - x + 1) + x * eta - size * log1p_exp(eta)
  if(sum) sum(res) else res
}

#' Poisson log-probability mass function
#'
#' @param x Vector of quantiles.
#' @param mean Expected value.
#' @return The sum of the log-probability.
#' @keywords internal
poisson_lpmf <- function(x, mean, sum = TRUE) {
  res <- dpois(x, mean, log = TRUE)
  if(sum) sum(res) else res
}

#' Negative binomial log-probability mass function
#'
#' @param x Vector of quantiles.
#' @param size Target for number of successful trials.
#' @param prob Probability of success in each trial.
#' @return The sum of the log-probability.
#' @keywords internal
neg_binomial_lpmf <- function(x, size, prob, sum = TRUE) {
  res <- suppressWarnings(dnbinom(x, size = size, prob = prob, log = TRUE))
  if(sum) sum(res) else res
}

#' Negative binomial log-probability mass function (alternative parameterization)
#'
#' @param x Vector of quantiles.
#' @param mu Mean parameter.
#' @param size Dispersion parameter.
#' @return The sum of the log-probability.
#' @keywords internal
neg_binomial_2_lpmf <- function(x, mu, size, sum = TRUE) {
  res <- lgamma(x + size) - lgamma(size) - lgamma(x + 1) +
    size * log(size) + x * log(mu) - (x + size) * log(size + mu)
  if(sum) sum(res) else res
}

#' Categorical log-probability mass function
#'
#' @param x Vector of categorical outcomes.
#' @param prob Vector or matrix of probabilities.
#' @return The sum of the log-probability.
#' @keywords internal
categorical_lpmf <- function(x, prob, sum = TRUE) {
  if (is.matrix(prob)) {
    res <- log(prob[cbind(seq_along(x), x)])
  } else {
    res <- log(prob[x])
  }
  if(sum) sum(res) else res
}

#' Categorical log-probability mass function with logit parameterization
#'
#' @param x Integer or integer vector. The observed category index.
#' @param eta A numeric vector of linear predictors (unnormalized log-probabilities).
#' @return The sum of the log-probability.
#' @keywords internal
categorical_logit_lpmf <- function(x, eta, sum = TRUE) {
  res <- eta[x] - log_sum_exp(eta)
  if(sum) sum(res) else res
}

#' Multinomial log-probability mass function
#'
#' @param x Vector of counts.
#' @param size Total number of trials.
#' @param prob Vector of probabilities for each category.
#' @return The sum of the log-probability.
#' @keywords internal
multinomial_lpmf <- function(x, size, prob) {
  sum(suppressWarnings(dmultinom(x, size = size, prob = prob, log = TRUE)))
}

#' Ordered logistic log-probability mass function
#'
#' @param x Vector of ordered categorical outcomes.
#' @param eta Linear predictor.
#' @param cutpoints Vector of cutpoints.
#' @return The sum of the log-probability.
#' @keywords internal
ordered_logistic_lpmf <- function(x, eta, cutpoints, sum = TRUE) {
  N <- length(x)
  K <- length(cutpoints) + 1

  log1p_exp <- function(v) {
    max_val <- (v + sqrt(v^2 + 1e-11)) / 2
    return(max_val + log(exp(v - max_val) + exp(-max_val)))
  }
  log1m_exp <- function(v) {
    # In ordered-logistic middle categories, v = log(F_lower) - log(F_upper)
    # is non-positive by construction. Avoid pmin()/ifelse() here because
    # comparisons on RTMB AD types are unsafe.
    return(log(-expm1(v)))
  }

  if (length(eta) == 1 && N > 1) {
    eta_vec <- rep(eta, N)
  } else {
    eta_vec <- eta
  }

  # Perform vector operations using indices
  idx_1 <- which(x == 1)
  idx_K <- which(x == K)
  idx_mid <- which(x > 1 & x < K)

  if (sum) {
    lp <- 0
    if (length(idx_1) > 0) {
      lp <- lp - sum(log1p_exp(-(cutpoints[1] - eta_vec[idx_1])))
    }
    if (length(idx_K) > 0) {
      lp <- lp - sum(log1p_exp(cutpoints[K - 1] - eta_vec[idx_K]))
    }
    if (length(idx_mid) > 0) {
      for (ii in idx_mid) {
        y <- as.integer(x[ii])
        A <- -log1p_exp(-(cutpoints[y] - eta_vec[ii]))
        B <- -log1p_exp(-(cutpoints[y - 1L] - eta_vec[ii]))
        lp <- lp + A + log1m_exp(B - A)
      }
    }
    return(lp)
  } else {
    # Initialize with AD type safety
    res <- rep(cutpoints[1] * 0, N)
    if (length(idx_1) > 0) {
      res[idx_1] <- -log1p_exp(-(cutpoints[1] - eta_vec[idx_1]))
    }
    if (length(idx_K) > 0) {
      res[idx_K] <- -log1p_exp(cutpoints[K - 1] - eta_vec[idx_K])
    }
    if (length(idx_mid) > 0) {
      for (ii in idx_mid) {
        y <- as.integer(x[ii])
        A <- -log1p_exp(-(cutpoints[y] - eta_vec[ii]))
        B <- -log1p_exp(-(cutpoints[y - 1L] - eta_vec[ii]))
        res[ii] <- A + log1m_exp(B - A)
      }
    }
    return(res)
  }
}

#' Sequential logistic log-probability mass function
#'
#' @param x Vector of ordered categorical outcomes.
#' @param eta Linear predictor.
#' @param cutpoints Vector of sequential cutpoints.
#' @return The sum of the log-probability.
#' @keywords internal
sequential_logistic_lpmf <- function(x, eta, cutpoints, sum = TRUE) {
  N <- length(x)
  K <- length(cutpoints) + 1

  log1p_exp <- function(v) {
    max_val <- (v + sqrt(v^2 + 1e-11)) / 2
    max_val + log(exp(v - max_val) + exp(-max_val))
  }

  eta_is_matrix <- is.matrix(eta)
  if (!eta_is_matrix) {
    if (length(eta) == 1 && N > 1) {
      eta_vec <- rep(eta, N)
    } else {
      eta_vec <- eta
    }
  }

  log_prob_one <- function(i, y) {
    lp_i <- cutpoints[1] * 0
    if (y > 1L) {
      for (kk in 1:(y - 1L)) {
        eta_ik <- if (eta_is_matrix) eta[i, kk] else eta_vec[i]
        v <- eta_ik - cutpoints[kk]
        lp_i <- lp_i - log1p_exp(-v)
      }
    }
    if (y < K) {
      eta_iy <- if (eta_is_matrix) eta[i, y] else eta_vec[i]
      v <- eta_iy - cutpoints[y]
      lp_i <- lp_i - log1p_exp(v)
    }
    lp_i
  }

  if (sum) {
    lp <- cutpoints[1] * 0
    for (ii in seq_len(N)) {
      lp <- lp + log_prob_one(ii, as.integer(x[ii]))
    }
    lp
  } else {
    res <- rep(cutpoints[1] * 0, N)
    for (ii in seq_len(N)) {
      res[ii] <- log_prob_one(ii, as.integer(x[ii]))
    }
    res
  }
}

#' Dirichlet log-probability density function
#'
#' @param x Vector or matrix of simplexes.
#' @param alpha Vector of concentration parameters.
#' @return The sum of the log-density.
#' @keywords internal
dirichlet_lpdf <- function(x, alpha) {
  sum((alpha - 1) * log(x)) + lgamma(sum(alpha)) - sum(lgamma(alpha))
}
#' Lower-triangular normal log-probability density function
#'
#' @param x A matrix of lower-triangular parameters.
#' @param mean Mean of the normal distribution.
#' @param sd Standard deviation of the normal distribution.
#' @return The log-density calculated only for the non-zero lower triangular elements.
#' @keywords internal
lower_tri_normal_lpdf <- function(x, mean = 0, sd = 1) {
  R <- nrow(x)
  C <- ncol(x)
  lp <- 0
  for (j in 1:min(R, C)) {
    val <- x[j:R, j]
    lp <- lp + sum(dnorm(val, mean, sd, log = TRUE))
  }
  return(lp)
}
#' Positive lower-triangular normal log-probability density function
#'
#' @param x A matrix of lower-triangular parameters (Cholesky factor with positive diagonals).
#' @param mean Mean of the normal distribution (assumed to be 0 for the half-normal correction).
#' @param sd Standard deviation of the normal distribution.
#' @return The log-density calculated for the non-zero lower triangular elements.
#' @keywords internal
positive_lower_tri_normal_lpdf <- function(x, mean = 0, sd = 1) {
  R <- nrow(x)
  C <- ncol(x)
  lp <- 0

  for (j in 1:min(R, C)) {
    val <- x[j:R, j]
    lp <- lp + sum(dnorm(val, mean, sd, log = TRUE))
  }

  num_diag <- min(R, C)
  lp <- lp + num_diag * log(2)

  return(lp)
}
#' Multivariate normal log-probability density function parameterized by Cholesky factor of correlation matrix
#'
#' @param x Vector or matrix of quantiles.
#' @param mean Vector or matrix of means.
#' @param sd Vector of standard deviations.
#' @param CF_Omega Cholesky factor of the correlation matrix.
#' @return The sum of the log-density.
#' @keywords internal
multi_normal_CF_lpdf <- function(x, mean, sd, CF_Omega, sum = TRUE) {
  if (length(sd) == 1) {
    L_Sigma <- matrix(sd, 1, 1) %*% CF_Omega
  } else {
    L_Sigma <- diag(sd) %*% CF_Omega
  }

  N <- if (is.matrix(x)) nrow(x) else 1
  K <- ncol(L_Sigma)

  log_det <- 2 * sum(log(diag(L_Sigma)))
  const <- -0.5 * (K * log(2 * pi) + log_det)

  if (is.matrix(x)) {
    resid_t <- if (is.matrix(mean)) t(x - mean) else t(x) - mean
    z <- solve(L_Sigma, resid_t)
    if (sum) {
      quad_form <- sum(z^2)
      return(N * const - 0.5 * quad_form)
    } else {
      quad_per_obs <- colSums(z^2)
      return(const - 0.5 * quad_per_obs)
    }
  } else {
    z <- solve(L_Sigma, x - mean)
    quad_form <- sum(z^2)
    return(const - 0.5 * quad_form)
  }
}

#' Multivariate normal log-probability density function parameterized by Cholesky factor of correlation matrix (FIML)
#'
#' @param x Vector or matrix of quantiles containing NAs.
#' @param mean Vector of means.
#' @param sd Vector of standard deviations.
#' @param CF_Omega Cholesky factor of the correlation matrix.
#' @return The sum of the log-density using Full Information Maximum Likelihood.
#' @keywords internal
fiml_multi_normal_CF_lpdf <- function(x, mean, sd, CF_Omega) {
  if (length(sd) == 1) {
    L_Sigma <- matrix(sd, 1, 1) %*% CF_Omega
  } else {
    L_Sigma <- diag(sd) %*% CF_Omega
  }
  Sigma <- L_Sigma %*% t(L_Sigma)

  N <- nrow(x)
  lp <- 0
  
  # Ensure positive definiteness with a scale-aware jitter
  diag_Sigma <- diag(Sigma)
  safe_Sigma <- Sigma + diag(diag_Sigma * 1e-6 + 1e-8)

  for (i in 1:N) {
    y_i <- x[i, ]
    keep <- !is.na(y_i)
    if (any(keep)) {
      y_obs <- y_i[keep]
      mean_obs <- mean[keep]
      Sigma_obs <- safe_Sigma[keep, keep, drop = FALSE]

      log_det_obj <- determinant(Sigma_obs, logarithm = TRUE)
      log_det <- log_det_obj$modulus
      k <- sum(keep)
      const <- -0.5 * (k * log(2 * pi) + log_det)

      resid <- y_obs - mean_obs
      quad_form <- sum(resid * solve(Sigma_obs, resid))
      lp <- lp + const - 0.5 * quad_form
    }
  }
  return(lp)
}

#' Multivariate normal log-probability density function
#'
#' @param x Vector or matrix of quantiles.
#' @param mean Vector or matrix of means.
#' @param Sigma Covariance matrix.
#' @return The sum of the log-density.
#' @keywords internal
multi_normal_lpdf <- function(x, mean, Sigma, sum = TRUE) {

  log_det_chol <- function(L) {
    return(2 * sum(log(diag(L))))
  }
  quad_form_chol <- function(x, L) {
    z <- solve(L, x)
    return(sum(z^2))
  }

  K <- nrow(Sigma)
  if (is.null(K) || length(Sigma) == 1) {
    res <- dnorm(x, mean = mean, sd = sqrt(Sigma + 1e-11), log = TRUE)
    return(if (sum) sum(res) else res)
  }
  # Ensure positive definiteness with a scale-aware jitter
  # This prevents singularity even if the elements of Sigma are very large or very small.
  diag_Sigma <- diag(Sigma)
  safe_Sigma <- Sigma + diag(diag_Sigma * 1e-6 + 1e-8)

  # Calculate log-determinant and quadratic form more robustly
  # For RTMB, using determinant() and solve() can sometimes be more stable than chol()
  # if the matrix is very near-singular.
  
  log_det_obj <- determinant(safe_Sigma, logarithm = TRUE)
  log_det <- log_det_obj$modulus
  
  const <- -0.5 * (K * log(2 * pi) + log_det)

  if (is.matrix(x) && ncol(x) > 1) {
    n <- nrow(x)
    resid_t <- if (is.matrix(mean)) t(x - mean) else t(x) - mean
    quad_form <- colSums(resid_t * solve(safe_Sigma, resid_t))
    res <- const - 0.5 * quad_form
    return(if (sum) sum(res) else res)
  } else {
    # Single observation
    x_vec <- if(is.matrix(x)) drop(x) else x
    mean_vec <- if(is.matrix(mean)) drop(mean) else mean
    resid <- x_vec - mean_vec
    quad_form <- sum(resid * solve(safe_Sigma, resid))
    return(const - 0.5 * quad_form)
  }
}

#' Multivariate Student-t log-probability density function
#'
#' @param x Vector or matrix of quantiles.
#' @param df Degrees of freedom.
#' @param mean Vector or matrix of means.
#' @param Sigma Scale matrix.
#' @return The sum of the log-density.
#' @keywords internal
multi_student_t_lpdf <- function(x, df, mean, Sigma, sum = TRUE) {
  # Robust dimension check for RTMB
  Sigma_val <- drop(Sigma)
  K <- if (is.matrix(Sigma)) nrow(Sigma) else length(Sigma_val)
  
  if (length(Sigma_val) == 1) {
    res <- student_t_lpdf(x, df = df, mu = mean, sigma = sqrt(Sigma_val + 1e-11), sum = FALSE)
    return(if (sum) sum(res) else res)
  }
  
  # Ensure positive definiteness with a scale-aware jitter
  diag_Sigma <- diag(Sigma)
  safe_Sigma <- Sigma + diag(diag_Sigma * 1e-6 + 1e-8)
  
  # Calculate log-determinant and quadratic form more robustly
  log_det_obj <- determinant(safe_Sigma, logarithm = TRUE)
  log_det <- log_det_obj$modulus
  
  const <- lgamma((df + K) / 2) - lgamma(df / 2) - 0.5 * (K * log(df * pi) + log_det)
  
  if (is.matrix(x) && ncol(x) > 1) {
    n <- nrow(x)
    resid_t <- if (is.matrix(mean)) t(x - mean) else t(x) - mean
    quad_form <- colSums(resid_t * solve(safe_Sigma, resid_t))
    res <- const - 0.5 * (df + K) * log1p(quad_form / df)
    return(if (sum) sum(res) else res)
  } else {
    # Single observation (vector or column-matrix)
    x_vec <- if(is.matrix(x)) drop(x) else x
    mean_vec <- if(is.matrix(mean)) drop(mean) else mean
    resid <- x_vec - mean_vec
    quad_form <- sum(resid * solve(safe_Sigma, resid))
    return(const - 0.5 * (df + K) * log1p(quad_form / df))
  }
}

#' Multivariate Cauchy log-probability density function
#'
#' @param x Vector or matrix of quantiles.
#' @param mean Vector or matrix of means.
#' @param Sigma Scale matrix.
#' @return The sum of the log-density.
#' @keywords internal
multi_cauchy_lpdf <- function(x, mean, Sigma, sum = TRUE) {
  multi_student_t_lpdf(x, df = 1, mean = mean, Sigma = Sigma, sum = sum)
}

#' Normal Mixture log-probability density function
#'
#' @param x Vector of quantiles (observations).
#' @param pi_w Vector of mixing proportions for each cluster.
#' @param mean Vector of means for each cluster.
#' @param sd Vector of standard deviations for each cluster.
#' @return The sum of the log-density.
#' @keywords internal
normal_mixture_lpdf <- function(x, pi_w, mean, sd, sum = TRUE) {
  log_pi <- log(pi_w)
  K <- length(pi_w)
  N <- length(x)

  # Fully vectorized version using matrix operations
  # log_dens: N x K matrix
  # Initialize with AD-safe values (using 0 based on log_pi type)
  log_dens <- matrix(log_pi[1] * 0, N, K)
  for (k in 1:K) {
    log_dens[, k] <- dnorm(x, mean = mean[k], sd = sd[k], log = TRUE) + log_pi[k]
  }

  # Log-Sum-Exp row-wise
  max_lp <- apply(log_dens, 1, max)
  res <- max_lp + log(rowSums(exp(log_dens - max_lp)))

  if(sum) sum(res) else res
}

#' Generic Mixture log-probability density function
#'
#' @param x Vector of quantiles.
#' @param pi_w Vector or matrix of mixing proportions.
#' @param lpdf_list A list of log-density vectors (one for each cluster).
#' @return The sum of the log-density.
#' @keywords internal
mixture_lpdf <- function(x, pi_w, lpdf_list, sum = TRUE) {
  N <- if (is.matrix(x)) nrow(x) else length(x)
  K <- length(lpdf_list)

  # log_pi: N x K matrix
  if (is.matrix(pi_w)) {
    log_pi <- log(pi_w)
  } else {
    log_pi <- matrix(log(pi_w), N, K, byrow = TRUE)
  }

  # log_dens: N x K matrix
  log_dens <- matrix(log_pi[1] * 0, N, K)
  for (k in 1:K) {
    log_dens[, k] <- lpdf_list[[k]] + log_pi[, k]
  }

  # Log-Sum-Exp row-wise
  max_lp <- apply(log_dens, 1, max)
  res <- max_lp + log(rowSums(exp(log_dens - max_lp)))

  if(sum) sum(res) else res
}


#' Centered / Centered matrix multivariate normal log-probability density function
#'
#' @param x Vector or matrix of quantiles. If a matrix, it assumes each column sums to zero.
#' @param sigma Standard deviation parameter. Can be a scalar, or a vector of length ncol(x) if x is a matrix.
#' @param K Dimension. If NULL, inferred from the length or nrow of x.
#' @return The log-density.
#' @keywords internal
centered_multi_normal_lpdf <- function(x, sigma, K = NULL) {
  if (is.matrix(x)) {
    R <- nrow(x)
    C <- ncol(x)
    if (length(sigma) == 1) {
      df <- (R - 1) * C
      lp <- -0.5 * df * log(2 * pi) - df * log(sigma) - 0.5 * sum(x^2) / sigma^2
    } else {
      lp <- 0
      for (c in 1:C) {
        x_c <- x[, c]
        sd_c <- sigma[c]
        lp <- lp - 0.5 * (R - 1) * log(2 * pi) - (R - 1) * log(sd_c) - 0.5 * sum(x_c^2) / sd_c^2
      }
    }
    return(lp)
  } else {
    if (is.null(K)) K <- length(x)
    lp <- -0.5 * (K - 1) * log(2 * pi) - (K - 1) * log(sigma) - 0.5 * sum(x^2) / sigma^2
    return(lp)
  }
}

#' Centered triangular multivariate normal log-probability density function
#'
#' @param x Matrix of quantiles.
#' @param sigma Standard deviation parameter(s).
#' @return The log-density.
#' @keywords internal
centered_tri_multi_normal_lpdf <- function(x, sigma) {
  R <- nrow(x)
  C <- ncol(x)
  max_d <- min(C, R - 1)

  lp <- 0
  if (length(sigma) == 1) {
    df <- sum(R - (1:max_d))
    lp <- -0.5 * df * log(2 * pi) - df * log(sigma) - 0.5 * sum(x^2) / sigma^2
  } else {
    # When specifying a different sigma for each dimension
    for (d in 1:max_d) {
      x_d <- x[d:R, d]
      K <- length(x_d)
      sd_d <- sigma[d]
      lp <- lp - 0.5 * (K - 1) * log(2 * pi) - (K - 1) * log(sd_d) - 0.5 * sum(x_d^2) / sd_d^2
    }
  }
  return(lp)
}
#' Positive centered triangular multivariate normal log-probability density function
#'
#' @param x Matrix of quantiles (centered triangular matrix with positive diagonals).
#' @param sigma Standard deviation parameter(s).
#' @return The log-density.
#' @keywords internal
positive_centered_tri_multi_normal_lpdf <- function(x, sigma = 1) {
  R <- nrow(x)
  C <- ncol(x)
  max_d <- min(C, R - 1)

  # Calculate the original centered_tri_multi_normal_lpdf
  lp <- centered_tri_multi_normal_lpdf(x, sigma)

  # Correct the normalizing constant for the truncated normal distribution (multiply density by 2^max_d since it's halved for each column)
  lp <- lp + max_d * log(2)

  return(lp)
}

#' Best-Worst Categorical Logit Log-PMF (RTMB optimized)
#'
#' @param x Vector of indices of the selected pairs.
#' @param U Vector of utilities for each item.
#' @param lambda Scaling parameter.
#' @return Log-probability (scalar advector).
#' @keywords internal
bw_categorical_logit_lpmf <- function(x, U, lambda = 1) {
  # 1. Handle missing values
  if (any(is.na(as.numeric(x)))) return(0)

  C <- length(U)

  # 2. Generate all pairs (i, j) where i != j using vectorized operations
  # Instead of loops, we use rep() to create index vectors
  idx_i <- rep(1:C, each = C)
  idx_j <- rep(1:C, times = C)
  mask <- idx_i != idx_j # Logical vector to exclude i == j

  # 3. Compute utility differences for all pairs at once
  # This avoids partial assignment and keeps the AD tape clean
  U_dif <- U[idx_i[mask]] - U[idx_j[mask]]

  # 4. Compute log-probabilities
  # eta is a vector of length C*(C-1)
  eta <- lambda * U_dif

  # Log-sum-exp is computed once per set
  # sum(eta[x]) handles multiple observations if x is a vector
  log_p_obs <- sum(eta[x])
  log_p_denom <- length(x) * log_sum_exp(eta)

  return(log_p_obs - log_p_denom)
}
#' Wishart log-probability density function
#'
#' @param X Symmetric positive-definite matrix (scatter matrix).
#' @param n Degrees of freedom (sample size N).
#' @param V Scale matrix (covariance matrix Omega).
#' @return The log-density with all constant terms.
#' @keywords internal
wishart_lpdf <- function(X, n, V) {
  p <- nrow(X)
  L_V <- chol(V)
  log_det_V <- 2 * sum(log(diag(L_V)))
  trace_term <- sum(diag(solve(V, X)))
  log_det_X <- determinant(X, logarithm = TRUE)$modulus
  log_gamma_p <- (p * (p - 1) / 4) * log(pi) + sum(lgamma((n - (1:p) + 1) / 2))
  lp <- (n - p - 1) / 2 * log_det_X - 0.5 * trace_term -
    (n * p / 2) * log(2) - (n / 2) * log_det_V - log_gamma_p

  return(lp)
}
#' Wishart log-probability density function parameterized by Cholesky factor of correlation matrix
#'
#' @param X Symmetric positive-definite matrix (scatter matrix).
#' @param n Degrees of freedom (sample size N).
#' @param sd Vector of standard deviations.
#' @param CF_Omega Cholesky factor of the correlation matrix.
#' @return The log-density with all constant terms.
#' @keywords internal
wishart_CF_lpdf <- function(X, n, sd, CF_Omega) {
  p <- nrow(X)

  if (length(sd) == 1) {
    L_Sigma <- matrix(sd, 1, 1) %*% CF_Omega
  } else {
    L_Sigma <- diag(sd) %*% CF_Omega
  }

  log_det_Sigma <- 2 * sum(log(diag(L_Sigma)))
  L_inv <- solve(L_Sigma)
  Sigma_inv <- t(L_inv) %*% L_inv
  trace_term <- sum(Sigma_inv * X)

  log_det_X <- determinant(X, logarithm = TRUE)$modulus
  log_gamma_p <- (p * (p - 1) / 4) * log(pi) + sum(lgamma((n - (1:p) + 1) / 2))

  lp <- (n - p - 1) / 2 * log_det_X - 0.5 * trace_term -
    (n * p / 2) * log(2) - (n / 2) * log_det_Sigma - log_gamma_p

  return(lp)
}

#' Factor analysis multivariate normal log-probability density function
#'
#' Woodbury matrix identity is used for efficient computation.
#'
#' @param x Vector or matrix of quantiles.
#' @param mu Vector of means.
#' @param Lambda Factor loading matrix (P x K).
#' @param psi Vector of unique variances (P).
#' @return The sum of the log-density.
#' @keywords internal
fa_multi_normal_lpdf <- function(x, mu, Lambda, psi, sum = TRUE) {
  P <- nrow(Lambda)
  K <- ncol(Lambda)
  inv_psi <- 1 / psi

  Lambda_scaled <- Lambda * inv_psi
  M <- diag(1, K) + (t(Lambda) %*% Lambda_scaled)
  L_M <- chol(M)
  log_det_Sigma <- sum(log(psi)) + 2 * sum(log(diag(L_M)))
  const <- -0.5 * (P * 1.83787706640935 + log_det_Sigma)

  if (is.matrix(x)) {
    N <- nrow(x)
    y_c <- t(t(x) - mu)
    z_scaled <- t(t(y_c) * inv_psi)
    z_lambda <- z_scaled %*% Lambda
    theta_T <- solve(M, t(z_lambda))

    if (sum) {
      term1 <- sum(y_c * z_scaled)
      term2 <- sum(z_lambda * t(theta_T))
      return(N * const - 0.5 * (term1 - term2))
    } else {
      term1_per_obs <- rowSums(y_c * z_scaled)
      term2_per_obs <- rowSums(z_lambda * t(theta_T))
      return(const - 0.5 * (term1_per_obs - term2_per_obs))
    }

  } else {
    # Vector case (single sample)
    y_c <- x - mu
    term1 <- sum((y_c^2) * inv_psi)

    z_scaled <- y_c * inv_psi
    z_lambda <- as.vector(t(Lambda) %*% z_scaled)
    theta <- solve(M, z_lambda)
    term2 <- sum(z_lambda * theta)

    return(const - 0.5 * (term1 - term2))
  }
}
#' Factor analysis multivariate normal log-probability density function (FIML)
#'
#' @param x Matrix of quantiles containing NAs.
#' @param mu Vector of means.
#' @param Lambda Factor loading matrix (P x K).
#' @param psi Vector of unique variances (P).
#' @return The sum of the log-density.
#' @keywords internal
fiml_multi_normal_fa_lpdf <- function(x, mu, Lambda, psi) {
  P <- nrow(Lambda)
  K <- ncol(Lambda)
  
  # Implied Covariance: Sigma = Lambda %*% t(Lambda) + diag(psi^2)
  # But for FIML, we can either construct Sigma and invert subsets, or use Woodbury on subsets.
  # Woodbury on subsets is complex because M = I + Lambda_obs' * inv_psi_obs * Lambda_obs changes per missing pattern.
  # Given typical RTMB usage, constructing Sigma and subsetting is simplest.
  Sigma <- Lambda %*% t(Lambda)
  diag(Sigma) <- diag(Sigma) + psi^2
  
  N <- nrow(x)
  lp <- 0
  
  diag_Sigma <- diag(Sigma)
  safe_Sigma <- Sigma + diag(diag_Sigma * 1e-6 + 1e-8)

  for (i in 1:N) {
    y_i <- x[i, ]
    keep <- !is.na(y_i)
    if (any(keep)) {
      y_obs <- y_i[keep]
      mean_obs <- mu[keep]
      Sigma_obs <- safe_Sigma[keep, keep, drop = FALSE]

      log_det_obj <- determinant(Sigma_obs, logarithm = TRUE)
      log_det <- log_det_obj$modulus
      k <- sum(keep)
      const <- -0.5 * (k * log(2 * pi) + log_det)

      resid <- y_obs - mean_obs
      quad_form <- sum(resid * solve(Sigma_obs, resid))
      lp <- lp + const - 0.5 * quad_form
    }
  }
  return(lp)
}

#' Sufficient statistics multivariate normal log-probability density function
#'
#' @param S_mat Deviation sum of squares matrix.
#' @param N Sample size.
#' @param y_bar Sample mean vector.
#' @param mean Mean parameter vector.
#' @param sd Standard deviation parameter vector.
#' @param CF_Omega Cholesky factor of correlation matrix.
#' @return The exact log-likelihood of the N raw observations.
#' @keywords internal
sufficient_multi_normal_CF_lpdf <- function(S_mat, N, y_bar, mean, sd, CF_Omega) {
  p <- length(y_bar)

  # 1. Calculate L_Sigma and log|Sigma|
  if (length(sd) == 1) {
    L_Sigma <- matrix(sd, 1, 1) %*% CF_Omega
  } else {
    L_Sigma <- diag(sd) %*% CF_Omega
  }
  log_det_Sigma <- 2 * sum(log(diag(L_Sigma)))

  # 2. Mahalanobis distance of the mean vector (N * (y_bar - mu)^T Sigma^-1 (y_bar - mu))
  z <- solve(L_Sigma, y_bar - mean)
  quad_mean <- sum(z^2)

  # 3. Trace term of the deviation sum of squares matrix (tr(Sigma^-1 S_mat))
  L_inv <- solve(L_Sigma)
  Sigma_inv <- t(L_inv) %*% L_inv
  trace_term <- sum(Sigma_inv * S_mat)

  # 4. Full log-likelihood of the original multivariate normal distribution
  lp <- -0.5 * N * p * log(2 * pi) -
    0.5 * N * log_det_Sigma -
    0.5 * trace_term -
    0.5 * N * quad_mean

  return(lp)
}

#' Sufficient statistics multivariate normal log-probability density function (covariance parameterization)
#'
#' @param S_mat Deviation sum of squares matrix.
#' @param N Sample size.
#' @param y_bar Sample mean vector.
#' @param mean Mean parameter vector.
#' @param Sigma Covariance matrix.
#' @return The exact log-likelihood of the N raw observations.
#' @keywords internal
sufficient_multi_normal_lpdf <- function(S_mat, N, y_bar, mean, Sigma) {
  p <- length(y_bar)

  # 1. Calculate log|Sigma|
  # Ensure positive definiteness with a scale-aware jitter
  diag_Sigma <- diag(Sigma)
  safe_Sigma <- Sigma + diag(diag_Sigma * 1e-6 + 1e-8)

  log_det_obj <- determinant(safe_Sigma, logarithm = TRUE)
  log_det_Sigma <- log_det_obj$modulus

  # 2. Mahalanobis distance of the mean vector
  resid <- y_bar - mean
  Sigma_inv <- solve(safe_Sigma)
  quad_mean <- sum(resid * (Sigma_inv %*% resid))

  # 3. Trace term of the deviation sum of squares matrix (tr(Sigma^-1 S_mat))
  trace_term <- sum(Sigma_inv * S_mat)

  # 4. Full log-likelihood
  lp <- -0.5 * N * p * log(2 * pi) -
    0.5 * N * log_det_Sigma -
    0.5 * trace_term -
    0.5 * N * quad_mean

  return(lp)
}

#' Sufficient statistics factor analysis multivariate normal log-probability density function
#'
#' @param S_mat Deviation sum of squares matrix.
#' @param N Sample size.
#' @param y_bar Sample mean vector.
#' @param mu Mean parameter vector.
#' @param Lambda Factor loading matrix (P x K).
#' @param psi Vector of unique variances (P).
#' @return The exact log-likelihood of the N raw observations.
#' @keywords internal
sufficient_multi_normal_fa_lpdf <- function(S_mat, N, y_bar, mu, psi,Lambda) {
  P <- nrow(Lambda)
  K <- ncol(Lambda)

  psi_safe <- psi^2 + 1e-8
  inv_psi <- 1 / psi_safe

  # Modification 1: Avoid recycling multiplication with vectors and use explicit matrix multiplication
  Lambda_scaled <- diag(inv_psi) %*% Lambda

  M <- diag(1, K) + (t(Lambda) %*% Lambda_scaled)
  L_M <- chol(M)

  log_det_Sigma <- sum(log(psi_safe)) + 2 * sum(log(diag(L_M)))

  # --- 1. Calculate trace term ---
  term1_trace <- sum(diag(S_mat) * inv_psi)

  Q <- t(Lambda_scaled) %*% S_mat %*% Lambda_scaled
  term2_trace <- sum(diag(solve(M, Q)))
  trace_term <- term1_trace - term2_trace

  # --- 2. Calculate mean term ---
  d <- y_bar - mu
  term1_mean <- sum((d^2) * inv_psi)

  # Modification 2: Avoid as.vector() and treat strictly as a column vector (matrix)
  d_mat <- matrix(d, ncol = 1)
  z_lambda <- t(Lambda_scaled) %*% d_mat
  theta <- solve(M, z_lambda)

  # Modification 3: Extract scalar from inner product of matrices
  term2_mean <- sum(t(z_lambda) %*% theta)

  quad_mean <- term1_mean - term2_mean

  # --- 3. Calculate full log-likelihood ---
  lp <- -0.5 * (N * P * 1.83787706640935 +
                  N * log_det_Sigma +
                  trace_term +
                  N * quad_mean)

  return(sum(lp))
}

#' Gaussian Process Log-Density (Squared Exponential Kernel)
#'
#' @description
#' Calculates the log-density of a Gaussian Process with a Squared Exponential (RBF) kernel.
#'
#' @param y Observation vector (N).
#' @param x Coordinate vector or matrix (N x D).
#' @param mean Mean vector (scalar or length N).
#' @param magnitude Signal standard deviation (alpha).
#' @param smoothing Length-scale (rho).
#' @param noise Measurement noise standard deviation (sigma).
#' @param sum Logical; whether to return the sum of log-densities.
#' @return Log-density value.
#' @export
gaussian_process_lpdf <- function(y, x, mean = 0, magnitude = 1, smoothing = 1, noise = 0.01, sum = TRUE) {
  N <- length(y)
  X <- if(is.matrix(x)) x else as.matrix(x)

  # Construct covariance matrix K
  K <- matrix(0, N, N)
  for (i in 1:N) {
    for (j in i:N) {
      # Squared Euclidean distance
      d2 <- sum((X[i, ] - X[j, ])^2)

      # Squared Exponential kernel
      val <- magnitude^2 * exp(-d2 / (2 * smoothing^2))
      K[i, j] <- val
      if (i != j) K[j, i] <- val # Symmetry
    }
  }

  # Add measurement noise and a small jitter for numerical stability
  diag(K) <- diag(K) + noise^2 + 1e-8

  # Evaluate as Multivariate Normal
  return(multi_normal_lpdf(y, mean = mean, Sigma = K, sum = sum))
}

#' Common Features and Arguments of RTMB Wrapper Functions
#'
#' @description
#' The RTMB wrapper functions (`rtmb_lm`, `rtmb_glm`, `rtmb_glmer`, `rtmb_fa`, etc.)
#' share a unified interface designed to make Bayesian and Frequentist inference
#' accessible through familiar R formulas and standard model specifications.
#'
#' @details
#' All wrapper functions in this package are built upon the same core engine.
#' This ensures that regardless of the model type, the workflow for estimation,
#' summary, and expansion remains consistent.
#'
#' \strong{1. Unified Inference Methods:}
#' Every model object returned by a wrapper function provides the following methods:
#' \itemize{
#'   \item \code{$classic()}: Performs standard frequentist estimation (MLE/REML) using `prior_flat()`. Supports robust standard errors and Satterthwaite/Between-Within degrees of freedom approximations.
#'   \item \code{$optimize()}: Performs Maximum A Posteriori (MAP) estimation (Empirical Bayes).
#'   \item \code{$sample()}: Performs MCMC sampling (NUTS via tmbstan) for full Bayesian inference.
#'   \item \code{$variational()}: Performs Variational Inference (ADVI) for fast posterior approximation.
#' }
#'
#' \strong{2. Prior API and Regularization:}
#' You can specify the prior distribution using the `prior` argument:
#' \itemize{
#'   \item \code{prior_flat()}: No additional prior (default). Required for \code{classic()} inference.
#'   \item \code{prior_normal()}: Manual normal/exponential priors for parameters.
#'   \item \code{prior_weak()}: Weakly informative priors based on \code{y_range}.
#'   \item \code{prior_rhs()}: Regularized Horseshoe prior for continuous shrinkage.
#'   \item \code{prior_ssp()}: Spike-and-Slab prior for sparse variable selection.
#' }
#' \emph{Note: When using `prior_weak()`, `prior_rhs()`, or `prior_ssp()`, you must specify \code{y_range = c(min, max)}
#' to let the model set appropriate global scales for the priors.}
#'
#' \strong{3. Model Comparison and Bayes Factors:}
#' Model comparison is performed exclusively via Bridge Sampling. To compare a full model 
#' against a nested null model, use the \code{$bayes_factor(fixed = list(...))} method on an 
#' MCMC fit object. For example, \code{fit$bayes_factor(fixed = list(b = 0))} tests the 
#' hypothesis that the fixed effect \code{b} is exactly zero.
#'
#' \strong{4. Standard Errors and Degrees of Freedom:}
#' The \code{$classic()} and \code{$optimize()} methods provide advanced summary options:
#' \itemize{
#'   \item \code{se_method}: Calculate `"robust"` (Huber-White) or `"cluster-robust"` standard errors by providing a cluster variable.
#'   \item \code{df_method}: Use `"satterthwaite"` for mixed models or `"bw"` (Between-Within) for multilevel correlation models to obtain accurate p-values and confidence intervals.
#' }
#'
#' \strong{5. Fixed parameters:}
#' Use \code{fixed = list(...)} during model construction to fix model parameters to specified values.
#' For example, \code{fixed = list(delta = 0)} fixes the parameter \code{delta} to zero.
#' This same syntax is used in \code{$bayes_factor()} to define the restricted model.
#'
#' @name rtmb_wrappers
#' @family wrappers
NULL

#' Automatic Differentiation Variational Inference (ADVI)
#'
#' @param model An RTMB objective function object (`ad_obj`).
#' @param par_list A list defining the structure of parameters to be estimated.
#' @param pl_full A list defining the full structure of parameters including random effects.
#' @param iter Integer; fixed number of iterations for the optimization. Default is 3000.
#' @param tol_rel_obj Numeric; relative tolerance for the ELBO to check convergence. Default is 0.001.
#' @param window_size Integer; size of the moving window to calculate the median ELBO. Default is 100.
#' @param num_samples Integer; number of posterior draws to generate after optimization. Default is 1000.
#' @param alpha Numeric; learning rate (step size) for the Adam optimizer. Default is 0.01.
#' @param laplace Logical; whether Laplace approximation is used. Default is FALSE.
#' @param print_freq Integer; frequency of printing progress to the console. Set to 0 to disable. Default is 500.
#' @param method Character; type of variational distribution. One of "meanfield", "fullrank", or "hybrid". Default is "meanfield".
#' @param update_progress Optional function to update a progress bar.
#' @param update_interval Integer; interval for updating the progress bar. Default is 100.
#' @return A list containing `fit`, `random_fit`, `elbo_history`, `elbo_final`, `rel_obj_final`, and `converged`.
ADVI_method <- function(model, par_list, pl_full,
                        iter = 3000, tol_rel_obj = 0.001,
                        window_size = 100, num_samples = 1000,
                        alpha = 0.01, laplace = FALSE,
                        print_freq = 500,
                        method = c("meanfield", "fullrank", "hybrid"),
                        update_progress = NULL, update_interval = 100) {

  method <- match.arg(method)

  P <- length(model$par)
  mu <- model$par
  par_names_rtmb <- names(model$par)

  # fixed and random effect index
  random_flags <- sapply(par_list, function(x) isTRUE(x$random))
  random_bases <- names(par_list)[random_flags]

  idx_random <- which(par_names_rtmb %in% random_bases)
  idx_fixed  <- which(!(par_names_rtmb %in% random_bases))

  P_fixed <- length(idx_fixed)
  P_random <- length(idx_random)

  if (method == "hybrid") {
    if (P_random == 0) {
      method <- "fullrank"
    } else if (P_fixed == 0) {
      method <- "meanfield"
    }
  }

  beta1 <- 0.9
  beta2 <- 0.999
  epsilon <- 1e-8
  entropy_const <- (P / 2) * log(2 * pi * exp(1))

  if (print_freq > 0) cat("Starting ADVI optimization with Adam...\n")

  if (method == "hybrid") {
    L_diag <- rep(-2, P_fixed)
    L_off <- rep(0, P_fixed * (P_fixed - 1) / 2)
    omega <- rep(-2, P_random)

    m_mu <- rep(0, P); v_mu <- rep(0, P)
    m_diag <- rep(0, P_fixed); v_diag <- rep(0, P_fixed)
    m_off <- rep(0, length(L_off)); v_off <- rep(0, length(L_off))
    m_omega <- rep(0, P_random); v_omega <- rep(0, P_random)
  } else if (method == "fullrank") {
    L_diag <- rep(-2, P)
    L_off <- rep(0, P * (P - 1) / 2)
    m_mu <- rep(0, P); v_mu <- rep(0, P)
    m_diag <- rep(0, P); v_diag <- rep(0, P)
    m_off <- rep(0, length(L_off)); v_off <- rep(0, length(L_off))
  } else { # meanfield
    omega <- rep(-2, P)
    m_mu <- rep(0, P); v_mu <- rep(0, P)
    m_omega <- rep(0, P); v_omega <- rep(0, P)
  }

  # --- Optimization: Pre-compute matrix indices for fullrank/hybrid ---
  if (method == "hybrid" && P_fixed > 0) {
    idx_L_lower <- which(lower.tri(matrix(NA, P_fixed, P_fixed)))
    idx_L_diag <- 1 + 0:(P_fixed - 1) * (P_fixed + 1)
  } else if (method == "fullrank") {
    idx_L_lower <- which(lower.tri(matrix(NA, P, P)))
    idx_L_diag <- 1 + 0:(P - 1) * (P + 1)
  }

  # Optimization: Initialize Adam decay terms
  b1_t <- 1
  b2_t <- 1
  # ------------------------------------------------------------

  elbo_history <- numeric(iter)
  converged <- FALSE
  rel_obj_final <- NA

  mu_history <- matrix(NA, nrow = window_size, ncol = P)

  for (t in 1:iter) {
    # Update Adam decay terms efficiently
    b1_t <- b1_t * beta1
    b2_t <- b2_t * beta2

    # --- update progress bar ---
    if (!is.null(update_progress) && t %% update_interval == 0) {
      update_progress(1)
    }

    eps <- rnorm(P)

    if (method == "hybrid") {
      eps_fixed <- eps[idx_fixed]
      eps_random <- eps[idx_random]

      L <- matrix(0, P_fixed, P_fixed)
      if (length(L_off) > 0) L[idx_L_lower] <- L_off
      L[idx_L_diag] <- exp(L_diag)
      theta_fixed <- mu[idx_fixed] + as.vector(L %*% eps_fixed)

      sigma_random <- exp(omega)
      theta_random <- mu[idx_random] + sigma_random * eps_random

      theta <- numeric(P)
      theta[idx_fixed] <- theta_fixed
      theta[idx_random] <- theta_random
    } else if (method == "fullrank") {
      L <- matrix(0, P, P)
      if (length(L_off) > 0) L[idx_L_lower] <- L_off
      L[idx_L_diag] <- exp(L_diag)
      theta <- mu + as.vector(L %*% eps)
    } else {
      sigma <- exp(omega)
      theta <- mu + sigma * eps
    }

    fn_val <- -model$fn(theta)
    gr_val <- as.vector(-model$gr(theta))

    if (!is.finite(fn_val) || any(!is.finite(gr_val))) {
      elbo_history[t] <- if(t > 1) elbo_history[t-1] else 0
      next
    }

    if (method == "hybrid") {
      elbo_history[t] <- fn_val + sum(L_diag) + sum(omega) + entropy_const

      grad_mu <- gr_val

      # fixed effect (Full-rank)
      gr_fixed <- gr_val[idx_fixed]
      grad_L_mat <- outer(gr_fixed, eps_fixed)

      grad_diag <- grad_L_mat[idx_L_diag] * exp(L_diag) + 1
      grad_off <- grad_L_mat[idx_L_lower]

      # random effect (Mean-field)
      gr_random <- gr_val[idx_random]
      grad_omega <- gr_random * (eps_random * sigma_random) + 1

      m_mu <- beta1 * m_mu + (1 - beta1) * grad_mu
      v_mu <- beta2 * v_mu + (1 - beta2) * (grad_mu^2)

      mu <- mu + alpha * (m_mu / (1 - b1_t)) / (sqrt(v_mu / (1 - b2_t)) + epsilon)

      m_diag <- beta1 * m_diag + (1 - beta1) * grad_diag
      v_diag <- beta2 * v_diag + (1 - beta2) * (grad_diag^2)
      L_diag <- L_diag + alpha * (m_diag / (1 - b1_t)) / (sqrt(v_diag / (1 - b2_t)) + epsilon)

      if (length(L_off) > 0) {
        m_off <- beta1 * m_off + (1 - beta1) * grad_off
        v_off <- beta2 * v_off + (1 - beta2) * (grad_off^2)
        L_off <- L_off + alpha * (m_off / (1 - b1_t)) / (sqrt(v_off / (1 - b2_t)) + epsilon)
      }

      if (length(omega) > 0) {
        m_omega <- beta1 * m_omega + (1 - beta1) * grad_omega
        v_omega <- beta2 * v_omega + (1 - beta2) * (grad_omega^2)
        omega <- omega + alpha * (m_omega / (1 - b1_t)) / (sqrt(v_omega / (1 - b2_t)) + epsilon)
      }
    } else if (method == "fullrank") {
      elbo_history[t] <- fn_val + sum(L_diag) + entropy_const
      grad_mu <- gr_val
      grad_L_mat <- outer(gr_val, eps)

      grad_diag <- grad_L_mat[idx_L_diag] * exp(L_diag) + 1
      grad_off <- grad_L_mat[idx_L_lower]

      m_mu <- beta1 * m_mu + (1 - beta1) * grad_mu
      v_mu <- beta2 * v_mu + (1 - beta2) * (grad_mu^2)

      mu <- mu + alpha * (m_mu / (1 - b1_t)) / (sqrt(v_mu / (1 - b2_t)) + epsilon)

      m_diag <- beta1 * m_diag + (1 - beta1) * grad_diag
      v_diag <- beta2 * v_diag + (1 - beta2) * (grad_diag^2)
      L_diag <- L_diag + alpha * (m_diag / (1 - b1_t)) / (sqrt(v_diag / (1 - b2_t)) + epsilon)

      if (length(L_off) > 0) {
        m_off <- beta1 * m_off + (1 - beta1) * grad_off
        v_off <- beta2 * v_off + (1 - beta2) * (grad_off^2)
        L_off <- L_off + alpha * (m_off / (1 - b1_t)) / (sqrt(v_off / (1 - b2_t)) + epsilon)
      }
    } else {
      elbo_history[t] <- fn_val + sum(omega) + entropy_const
      grad_mu <- gr_val
      grad_omega <- gr_val * (eps * sigma) + 1

      m_mu <- beta1 * m_mu + (1 - beta1) * grad_mu
      v_mu <- beta2 * v_mu + (1 - beta2) * (grad_mu^2)

      mu <- mu + alpha * (m_mu / (1 - b1_t)) / (sqrt(v_mu / (1 - b2_t)) + epsilon)

      m_omega <- beta1 * m_omega + (1 - beta1) * grad_omega
      v_omega <- beta2 * v_omega + (1 - beta2) * (grad_omega^2)
      omega <- omega + alpha * (m_omega / (1 - b1_t)) / (sqrt(v_omega / (1 - b2_t)) + epsilon)
    }

    idx <- (t - 1) %% window_size + 1
    mu_history[idx, ] <- mu

    if (print_freq > 0 && t %% print_freq == 0) {
      cat(sprintf("Iter %d: Approx ELBO = %.2f\n", t, elbo_history[t]))
    }
  }

  # convergence index
  check_start <- 2 * window_size
  if (iter > check_start) {
    med_prev <- median(elbo_history[(iter - 2 * window_size + 1):(iter - window_size)])
    med_curr <- median(elbo_history[(iter - window_size + 1):iter])
    rel_obj_final <- abs(med_curr - med_prev) / (abs(med_prev) + 1e-8)
    if (rel_obj_final < tol_rel_obj) {
      converged <- TRUE
    }
  }

  if (print_freq > 0) cat("Generating posterior samples from variational distribution...\n")

  fit_matrix <- matrix(NA, nrow = num_samples, ncol = P)

  if (method == "hybrid") {
    L <- matrix(0, P_fixed, P_fixed)
    if (length(L_off) > 0) L[lower.tri(L)] <- L_off
    diag(L) <- exp(L_diag)
    sigma_random <- exp(omega)

    for (i in 1:num_samples) {
      eps_samp <- rnorm(P)
      eps_fixed <- eps_samp[idx_fixed]
      eps_random <- eps_samp[idx_random]

      theta_fixed <- mu[idx_fixed] + as.vector(L %*% eps_fixed)
      theta_random <- mu[idx_random] + sigma_random * eps_random

      theta_samp <- numeric(P)
      theta_samp[idx_fixed] <- theta_fixed
      theta_samp[idx_random] <- theta_random
      fit_matrix[i, ] <- theta_samp
    }
  } else if (method == "fullrank") {
    L <- matrix(0, P, P)
    if (length(L_off) > 0) L[lower.tri(L)] <- L_off
    diag(L) <- exp(L_diag)
    for (i in 1:num_samples) {
      fit_matrix[i, ] <- mu + as.vector(L %*% rnorm(P))
    }
  } else {
    sigma <- exp(omega)
    for (i in 1:num_samples) {
      fit_matrix[i, ] <- mu + sigma * rnorm(P)
    }
  }

  base_names_full <- sub("\\[.*\\]", "", pl_full$names)
  random_flags <- sapply(par_list, function(x) isTRUE(x$random))

  if (laplace && any(random_flags)) {
    base_names_fixed <- names(par_list)[!random_flags]
    base_names_random <- names(par_list)[random_flags]
  } else {
    base_names_fixed <- names(par_list)
    base_names_random <- character(0)
  }

  fixed_names <- pl_full$names[base_names_full %in% base_names_fixed]
  random_names <- pl_full$names[base_names_full %in% base_names_random]

  fixed_idx  <- which(pl_full$names %in% fixed_names)
  random_idx <- if (length(random_names) > 0) which(pl_full$names %in% random_names) else integer(0)

  P_all_true <- length(pl_full$names)
  para_final <- array(NA, dim = c(num_samples, P_all_true))
  lp_final <- numeric(num_samples)

  for (i in 1:num_samples) {
    zeta_sample <- fit_matrix[i, ]
    lp_final[i] <- -model$fn(zeta_sample)

    if (laplace && length(model$env$random) > 0) {
      para_list_res <- model$env$parList(x = as.numeric(zeta_sample[model$env$lfixed()]))
    } else {
      para_list_res <- model$env$parList(x = as.numeric(zeta_sample))
    }

    con_list <- to_constrained(para_list_res, par_list)
    para_final[i, ] <- unlist(con_list, use.names = FALSE)
  }

  if (method == "hybrid") {
    elbo_final <- mean(lp_final) + sum(L_diag) + sum(omega) + entropy_const
  } else if (method == "fullrank") {
    elbo_final <- mean(lp_final) + sum(L_diag) + entropy_const
  } else {
    elbo_final <- mean(lp_final) + sum(omega) + entropy_const
  }

  fit <- array(NA, dim = c(num_samples, 1, length(fixed_names) + 1))
  dimnames(fit) <- list(iteration = NULL, chain = "est1", variable = c("lp", fixed_names))

  random_fit <- NULL
  if (length(random_names) > 0) {
    random_fit <- array(NA, dim = c(num_samples, 1, length(random_names)))
    dimnames(random_fit) <- list(iteration = NULL, chain = "est1", variable = random_names)
  }

  for (i in 1:num_samples) {
    fit[i, 1, 1] <- lp_final[i]
    fit[i, 1, -1] <- para_final[i, fixed_idx]
    if (!is.null(random_fit)) {
      random_fit[i, 1, ] <- para_final[i, random_idx]
    }
  }

  mu_hist_filled <- !is.na(mu_history[, 1])
  valid_hist_idx <- which(mu_hist_filled & apply(mu_history, 1, function(z) all(is.finite(z))))

  mu_hist_constrained <- matrix(
    NA,
    nrow = length(valid_hist_idx),
    ncol = length(fixed_idx) + length(random_idx)
  )
  colnames(mu_hist_constrained) <- c(fixed_names, random_names)

  for (j in seq_along(valid_hist_idx)) {
    i <- valid_hist_idx[j]
    zeta_hist <- mu_history[i, ]

    model$fn(zeta_hist)
    if (laplace && length(model$env$random) > 0) {
      para_list_res <- model$env$parList(x = as.numeric(zeta_hist[model$env$lfixed()]))
    } else {
      para_list_res <- model$env$parList(x = as.numeric(zeta_hist))
    }

    con_list <- to_constrained(para_list_res, par_list)
    para_hist_all <- unlist(con_list, use.names = FALSE)

    mu_hist_constrained[j, 1:length(fixed_idx)] <- para_hist_all[fixed_idx]
    if (length(random_idx) > 0) {
      mu_hist_constrained[j, (length(fixed_idx) + 1):ncol(mu_hist_constrained)] <- para_hist_all[random_idx]
    }
  }

  fit <- Re(fit)
  if (!is.null(random_fit)) random_fit <- Re(random_fit)

  return(list(
    fit           = fit,
    random_fit    = random_fit,
    elbo_history  = elbo_history,
    elbo_final    = elbo_final,
    rel_obj_final = rel_obj_final,
    converged     = converged,
    mu_history    = mu_hist_constrained
  ))
}

#' Find random effects terms (bars) in a formula
#'
#' @param term A formula or expression.
#' @return A list of bar terms.
#' @importFrom stats as.formula model.frame model.matrix model.response model.offset
#' @keywords internal
#' @noRd
findbars <- function(term) {
  if (is.name(term) || !is.call(term)) return(NULL)
  if (term[[1]] == as.name("(")) return(findbars(term[[2]]))
  if (term[[1]] == as.name("|") || term[[1]] == as.name("||")) return(list(term))
  res <- lapply(term, findbars)
  return(unlist(res, recursive = FALSE))
}

#' Remove random effects terms from a formula
#'
#' @param term A formula or expression.
#' @return A formula or expression with bars removed.
#' @keywords internal
#' @noRd
nobars <- function(term) {
  if (!is.language(term)) return(term)
  if (is.call(term) && (term[[1]] == as.name("|") || term[[1]] == as.name("||"))) return(NULL)
  if (length(term) == 1) return(term)
  if (length(term) == 2) {
    nb <- nobars(term[[2]])
    if (is.null(nb)) return(NULL)
    term[[2]] <- nb
    return(term)
  }
  nb2 <- nobars(term[[2]])
  nb3 <- nobars(term[[3]])
  if (is.null(nb3)) {
    if (is.call(term) && term[[1]] == as.name("~")) {
      term[[2]] <- nb2
      term[[3]] <- as.name("1")
      return(term)
    }
    return(nb2)
  }
  if (is.null(nb2)) return(nb3)
  term[[2]] <- nb2
  term[[3]] <- nb3
  return(term)
}

#' Replace bars with pluses to get all variables
#'
#' @param term A formula or expression.
#' @return A formula or expression with bars replaced by pluses.
#' @keywords internal
#' @noRd
subbars <- function(term) {
  if (is.name(term) || !is.call(term)) return(term)
  if (term[[1]] == as.name("(")) return(subbars(term[[2]]))
  if (term[[1]] == as.name("|") || term[[1]] == as.name("||")) {
    return(as.call(list(as.name("+"), subbars(term[[2]]), subbars(term[[3]]))))
  }
  if (length(term) == 1) return(term)
  res <- lapply(term, subbars)
  return(as.call(res))
}

`%||%` <- function(a, b) if (!is.null(a)) a else b

#' @keywords internal
#' @noRd
.names0 <- function(x) {
  n <- names(x)
  if (is.null(n)) character(0L) else n
}

#' @keywords internal
#' @noRd
.estimate_component_names <- function(fit, component) {
  if (component == "parameters") return(.names0(fit$model$par_list))

  if (component == "transform") {
    n <- .names0(fit$transform_dims)
    if (length(n) == 0L) n <- .names0(fit$transform)
    return(n)
  }

  if (component == "generate") {
    n <- .names0(fit$generate_dims)
    if (length(n) == 0L) n <- .names0(fit$generate)
    return(n)
  }

  stop("Unknown component: ", component, call. = FALSE)
}

#' @keywords internal
#' @noRd
.get_marginal_mode <- function(x) {
  x <- x[is.finite(x)]
  if (length(x) == 0L) return(NA_real_)
  if (length(unique(x)) == 1L) return(x[1L])
  # Avoid density estimation warning when values are nearly constant
  if (diff(range(x)) < 1e-7) return(mean(x))
  d <- density(x)
  d$x[which.max(d$y)]
}

#' @export
print.rtmb_estimate_matrix <- function(x, digits = max(3L, getOption("digits") - 2L), ...) {
  x_print <- formatC(unclass(x), format = "f", digits = digits)
  dimnames(x_print) <- dimnames(x)
  rn <- rownames(x_print)
  cn <- colnames(x_print)
  if (is.null(rn)) rn <- rep("", nrow(x_print))
  if (is.null(cn)) cn <- paste0("[,", seq_len(ncol(x_print)), "]")

  row_width <- max(nchar(rn), 0L)
  col_widths <- pmax(nchar(cn), apply(nchar(x_print), 2L, max))
  header <- paste0(
    strrep(" ", row_width + 1L),
    paste(format(cn, width = col_widths, justify = "right"), collapse = " ")
  )
  rows <- vapply(seq_len(nrow(x_print)), function(i) {
    paste0(
      format(rn[i], width = row_width, justify = "left"),
      " ",
      paste(format(x_print[i, ], width = col_widths, justify = "right"), collapse = " ")
    )
  }, character(1L))
  cat(paste(c(header, rows), collapse = "\n"), "\n", sep = "")
  invisible(x)
}

#' @keywords internal
#' @noRd
.apply_estimate_dimnames <- function(fit, name, val, dims = NULL) {
  if (is.null(dims)) {
    if (!is.null(fit$model$par_list[[name]])) {
      dims <- fit$model$par_list[[name]]$dim
    } else if (!is.null(fit$transform_dims[[name]])) {
      dims <- fit$transform_dims[[name]]
    } else if (!is.null(fit$generate_dims[[name]])) {
      dims <- fit$generate_dims[[name]]
    }
  }
  if (is.null(dims) || length(dims) == 0L) return(val)

  names_def <- fit$model$par_names[[name]]
  if (length(dims) > 1L && is.null(dim(val))) dim(val) <- dims
  if (is.null(names_def) && length(dims) == 2L) {
    y_names <- colnames(fit$model$data$Y)
    if (!is.null(y_names) && length(y_names) == dims[1L]) {
      names_def <- list(y_names, paste0("Factor", seq_len(dims[2L])))
    }
  }

  if (!is.null(names_def)) {
    if (is.list(names_def) && length(names_def) == length(dims)) {
      dimnames(val) <- names_def
    } else if (is.atomic(names_def) && length(dims) == 1L && length(names_def) == dims[1L]) {
      names(val) <- names_def
    } else if (is.atomic(names_def) && length(dims) > 1L && length(names_def) == dims[1L]) {
      dn <- vector("list", length(dims))
      dn[[1L]] <- names_def
      if (length(dims) == 2L && dims[1L] == dims[2L]) dn[[2L]] <- names_def
      dimnames(val) <- dn
    }
  }
  if (length(dim(val)) == 2L && is.numeric(val)) {
    class(val) <- unique(c("rtmb_estimate_matrix", class(val)))
  }
  val
}

#' @keywords internal
#' @noRd
.collect_point_estimates <- function(fit, type = "EAP", chains = NULL, best_chains = NULL, ...) {
  if (inherits(fit, c("Classic_Fit", "map_fit"))) {
    # Combine par, transform, and generate fields from optimization result
    res <- c(fit$par %||% list(), fit$transform %||% list(), fit$generate %||% list())
    for (name in names(res)) {
      res[[name]] <- .apply_estimate_dimnames(fit, name, res[[name]])
    }
    return(res)
  }

  if (inherits(fit, c("mcmc_fit", "advi_fit"))) {
    # Fetch draws for all variables (respecting chains/best_chains)
    samps <- fit$draws(chains = chains, best_chains = best_chains, 
                       inc_random = TRUE, inc_transform = TRUE, inc_generate = TRUE)
    
    if (type == "joint_map") {
      # Returns the sample with the highest log-posterior (joint MAP)
      # Find the iteration/chain with max lp
      lp_idx <- which(dimnames(samps)[[3]] == "lp")
      if (length(lp_idx) == 0) lp_idx <- 1
      
      lp_vals <- samps[, , lp_idx]
      max_idx <- which(lp_vals == max(lp_vals, na.rm = TRUE), arr.ind = TRUE)
      iter_idx <- max_idx[1, 1]
      chain_idx <- max_idx[1, 2]
      
      # This is our point estimate vector (flattened)
      flat_ests <- samps[iter_idx, chain_idx, ]
      names(flat_ests) <- dimnames(samps)[[3]]
    } else {
      # EAP or marginal_map
      d <- dim(samps)
      samps_flat <- matrix(samps, nrow = d[1] * d[2], ncol = d[3])
      colnames(samps_flat) <- dimnames(samps)[[3]]
      
      if (type == "EAP") {
        flat_ests <- colMeans(samps_flat, na.rm = TRUE)
      } else {
        flat_ests <- apply(samps_flat, 2, .get_marginal_mode)
      }
    }
    
    # Group back into structured list
    res <- list()
    
    # Helper to group by base name
    group_by_name <- function(base_names, dims_list = NULL) {
      for (name in base_names) {
        idx <- grep(paste0("^", name, "(\\[|$)"), names(flat_ests))
        if (length(idx) > 0) {
          val <- flat_ests[idx]
          dims <- if (!is.null(dims_list)) dims_list[[name]] else fit$model$par_list[[name]]$dim
          if (length(dims) > 1) dim(val) <- dims
          val <- .apply_estimate_dimnames(fit, name, val, dims)
          res[[name]] <<- val
        }
      }
    }
    
    group_by_name(names(fit$model$par_list))
    group_by_name(names(fit$transform_dims), fit$transform_dims)
    group_by_name(names(fit$generate_dims), fit$generate_dims)
    
    return(res)
  }
  stop("Unsupported fit object type.")
}

#' @keywords internal
#' @noRd
.select_estimates <- function(fit, est_list, pars = NULL, component = "all", drop = FALSE) {
  component <- match.arg(component, c("all", "parameters", "transform", "generate"))

  # 1. Parameter selection keywords
  if (identical(pars, "all")) {
    pars <- NULL
  } else if (identical(pars, "parameters")) {
    pars <- .estimate_component_names(fit, "parameters")
  } else if (identical(pars, "transform")) {
    pars <- .estimate_component_names(fit, "transform")
  } else if (identical(pars, "generate")) {
    pars <- .estimate_component_names(fit, "generate")
  }

  res <- select_parameters(est_list, pars)

  # 2. Component filtering
  if (component != "all") {
    keep <- .estimate_component_names(fit, component)
    res <- res[names(res) %in% keep]
  }

  if (length(res) == 0L) return(list())
  if (isTRUE(drop) && length(res) == 1L) return(res[[1L]])
  return(res)
}

#' Base class for RTMB Fit objects
#'
#' An R6 base class providing common methods for Bayesian and MAP inference results.
#'
#' @field model The `RTMB_Model` object used for estimation.
#'
#' @importFrom R6 R6Class
#' @export
RTMB_Fit_Base <- R6::R6Class(
  classname = "RTMB_Fit_Base",

  public = list(
    model = NULL,

    #' @description Abstract method to get a point estimate for a target parameter.
    #' @param target Character string specifying the target parameter name.
    #' @param ... Additional arguments.
    #' @return A numeric array or matrix of the point estimate.
    get_point_estimate = function(target, ...) {
      stop("get_point_estimate must be implemented by subclasses.")
    },

    #' @description Get point estimates for parameters, transformed parameters, and generated quantities.
    #' @param pars Optional character or numeric vector of parameter names or indices to extract.
    #'        Supports special keywords: "parameters", "transform", "generate", and "all".
    #' @param type Character string specifying the estimation type.
    #' @param component Character string specifying the component to filter by.
    #' @param chains Numeric vector of chains to include.
    #' @param best_chains Number of best chains to include.
    #' @param drop Logical; if TRUE and only one parameter is selected, return the value directly instead of a list.
    #' @param ... Additional arguments passed to draws().
    #' @return A named list of point estimates, or a single value if `drop = TRUE`.
    estimate = function(pars = NULL, 
                        type = c("mean", "EAP", "marginal_map", "joint_map", "MAP"), 
                        component = c("all", "parameters", "transform", "generate"), 
                        chains = NULL, 
                        best_chains = NULL, 
                        drop = TRUE, 
                        ...) {
      names0 <- function(x) {
        n <- names(x)
        if (is.null(n)) character(0L) else n
      }
      coalesce_null <- function(a, b) {
        if (!is.null(a)) a else b
      }
      apply_estimate_dimnames <- function(fit, name, val, dims = NULL) {
        if (is.null(dims)) {
          if (!is.null(fit$model$par_list[[name]])) {
            dims <- fit$model$par_list[[name]]$dim
          } else if (!is.null(fit$transform_dims[[name]])) {
            dims <- fit$transform_dims[[name]]
          } else if (!is.null(fit$generate_dims[[name]])) {
            dims <- fit$generate_dims[[name]]
          }
        }
        if (is.null(dims) || length(dims) == 0L) return(val)

        names_def <- fit$model$par_names[[name]]
        if (length(dims) > 1L && is.null(dim(val))) dim(val) <- dims
        if (is.null(names_def) && length(dims) == 2L) {
          y_names <- colnames(fit$model$data$Y)
          if (!is.null(y_names) && length(y_names) == dims[1L]) {
            names_def <- list(y_names, paste0("Factor", seq_len(dims[2L])))
          }
        }

        if (!is.null(names_def)) {
          if (is.list(names_def) && length(names_def) == length(dims)) {
            dimnames(val) <- names_def
          } else if (is.atomic(names_def) && length(dims) == 1L && length(names_def) == dims[1L]) {
            names(val) <- names_def
          } else if (is.atomic(names_def) && length(dims) > 1L && length(names_def) == dims[1L]) {
            dn <- vector("list", length(dims))
            dn[[1L]] <- names_def
            if (length(dims) == 2L && dims[1L] == dims[2L]) dn[[2L]] <- names_def
            dimnames(val) <- dn
          }
        }
        if (length(dim(val)) == 2L && is.numeric(val)) {
          class(val) <- unique(c("rtmb_estimate_matrix", class(val)))
        }
        val
      }
      select_parameters_local <- function(all_list, pars) {
        if (is.null(pars)) return(all_list)
        if (identical(pars, "parameters") || identical(pars, "all")) return(all_list)

        if (is.numeric(pars)) {
          res <- tryCatch(all_list[pars], error = function(e) NULL)
          if (is.null(res)) return(all_list[0])
          return(res[names(res) != ""])
        }

        if (is.character(pars)) {
          is_negative <- grepl("^-", pars)
          if (any(is_negative)) {
            if (!all(is_negative)) {
              stop("Cannot mix positive and negative parameter names in selection.")
            }
            exclude_names <- gsub("^-", "", pars)
            return(all_list[!(names(all_list) %in% exclude_names)])
          }
          return(all_list[names(all_list) %in% pars])
        }

        return(all_list)
      }
      estimate_component_names <- function(fit, component) {
        if (component == "parameters") return(names0(fit$model$par_list))

        if (component == "transform") {
          n <- names0(fit$transform_dims)
          if (length(n) == 0L) n <- names0(fit$transform)
          return(n)
        }

        if (component == "generate") {
          n <- names0(fit$generate_dims)
          if (length(n) == 0L) n <- names0(fit$generate)
          return(n)
        }

        stop("Unknown component: ", component, call. = FALSE)
      }
      marginal_mode <- function(x) {
        x <- x[is.finite(x)]
        if (length(x) == 0L) return(NA_real_)
        if (length(unique(x)) == 1L) return(x[1L])
        # Avoid density estimation warning when values are nearly constant
        if (diff(range(x)) < 1e-7) return(mean(x))
        d <- density(x)
        d$x[which.max(d$y)]
      }
      collect_point_estimates <- function(fit, type = "EAP", chains = NULL, best_chains = NULL, ...) {
        if (inherits(fit, c("Classic_Fit", "map_fit"))) {
          res <- c(coalesce_null(fit$par, list()),
                   coalesce_null(fit$transform, list()),
                   coalesce_null(fit$generate, list()))
          for (name in names(res)) {
            res[[name]] <- apply_estimate_dimnames(fit, name, res[[name]])
          }
          return(res)
        }

        if (inherits(fit, c("mcmc_fit", "advi_fit"))) {
          samps <- fit$draws(chains = chains, best_chains = best_chains,
                             inc_random = TRUE, inc_transform = TRUE, inc_generate = TRUE)

          if (type == "joint_map") {
            lp_idx <- which(dimnames(samps)[[3]] == "lp")
            if (length(lp_idx) == 0) lp_idx <- 1

            lp_vals <- samps[, , lp_idx]
            max_idx <- which(lp_vals == max(lp_vals, na.rm = TRUE), arr.ind = TRUE)
            iter_idx <- max_idx[1, 1]
            chain_idx <- max_idx[1, 2]

            flat_ests <- samps[iter_idx, chain_idx, ]
            names(flat_ests) <- dimnames(samps)[[3]]
          } else {
            d <- dim(samps)
            samps_flat <- matrix(samps, nrow = d[1] * d[2], ncol = d[3])
            colnames(samps_flat) <- dimnames(samps)[[3]]

            if (type == "EAP") {
              flat_ests <- colMeans(samps_flat, na.rm = TRUE)
            } else {
              flat_ests <- apply(samps_flat, 2, marginal_mode)
            }
          }

          res <- list()
          group_by_name <- function(base_names, dims_list = NULL) {
            for (name in base_names) {
              idx <- grep(paste0("^", name, "(\\[|$)"), names(flat_ests))
              if (length(idx) > 0) {
                val <- flat_ests[idx]
                dims <- if (!is.null(dims_list)) dims_list[[name]] else fit$model$par_list[[name]]$dim
                if (length(dims) > 1) dim(val) <- dims
                val <- apply_estimate_dimnames(fit, name, val, dims)
                res[[name]] <<- val
              }
            }
          }

          group_by_name(names(fit$model$par_list))
          group_by_name(names(fit$transform_dims), fit$transform_dims)
          group_by_name(names(fit$generate_dims), fit$generate_dims)

          return(res)
        }
        stop("Unsupported fit object type.")
      }
      select_estimates <- function(fit, est_list, pars = NULL, component = "all", drop = FALSE) {
        component <- match.arg(component, c("all", "parameters", "transform", "generate"))

        if (identical(pars, "all")) {
          pars <- NULL
        } else if (identical(pars, "parameters")) {
          pars <- estimate_component_names(fit, "parameters")
        } else if (identical(pars, "transform")) {
          pars <- estimate_component_names(fit, "transform")
        } else if (identical(pars, "generate")) {
          pars <- estimate_component_names(fit, "generate")
        }

        res <- select_parameters_local(est_list, pars)

        if (component != "all") {
          keep <- estimate_component_names(fit, component)
          res <- res[names(res) %in% keep]
        }

        if (length(res) == 0L) return(list())
        if (isTRUE(drop) && length(res) == 1L) return(res[[1L]])
        return(res)
      }

      type <- match.arg(type)
      component <- match.arg(component)
      
      # Normalization
      if (type == "mean") type <- "EAP"
      if (type == "MAP") type <- "marginal_map"

      # Determine default pars if NULL
      if (is.null(pars)) {
        if (!identical(component, "all")) {
          pars <- component
        } else if (inherits(self, c("mcmc_fit", "advi_fit"))) {
          pars <- "parameters"
        } else {
          pars <- "all"
        }
      }

      est_all <- collect_point_estimates(self, type = type, chains = chains, best_chains = best_chains, ...)
      
      # Validate pars existence if character and not keywords/regex/exclusion
      if (is.character(pars) && !any(grepl("^-", pars)) && 
          !all(pars %in% c("parameters", "transform", "generate", "all"))) {
        missing <- setdiff(pars, names(est_all))
        if (length(missing) > 0L) {
          stop(
            "Unknown estimate variable(s): ", paste(missing, collapse = ", "),
            "\nAvailable variables: ", paste(names(est_all), collapse = ", "),
            call. = FALSE
          )
        }
      }
      
      res <- select_estimates(self, est_all, pars = pars, component = component, drop = drop)
      return(res)
    },

    #' @description Calculate Expected A Posteriori (EAP) estimates from posterior samples.
    #' @param pars Optional character vector of parameter names to extract.
    #' @param chains Numeric vector of chains to include.
    #' @param best_chains Number of best chains to include.
    #' @param drop Logical; whether to drop the list if only one parameter is selected.
    #' @param ... Additional arguments passed to `estimate()`.
    #' @return A named list of EAP estimates.
    EAP = function(pars = "parameters", chains = NULL, best_chains = NULL, drop = FALSE, ...) {
      return(self$estimate(pars = pars, type = "EAP", chains = chains, best_chains = best_chains, drop = drop, ...))
    },

    #' @description Calculate Maximum A Posteriori (MAP) estimates.
    #' @param pars Optional character vector of parameter names to extract.
    #' @param chains Numeric vector of chains to include.
    #' @param best_chains Number of best chains to include.
    #' @param type Character string; "marginal" or "joint" MAP.
    #' @param drop Logical; whether to drop the list if only one parameter is selected.
    #' @param ... Additional arguments passed to `estimate()`.
    #' @return A named list of MAP estimates.
    MAP = function(pars = "parameters", chains = NULL, best_chains = NULL, 
                   type = c("marginal", "joint"), drop = FALSE, ...) {
      type <- match.arg(type)
      est_type <- if (type == "joint") "joint_map" else "marginal_map"
      return(self$estimate(pars = pars, type = est_type, chains = chains, best_chains = best_chains, drop = drop, ...))
    },

    #' @description Rotate sampled parameters.
    #' @param target Character string specifying the target variable to base the rotation on.
    #' @param reference Matrix to rotate towards. If NULL, the target's point estimate is used.
    #' @param linked Character vector of variable names to be rotated in the same direction.
    #' @param overwrite Logical; whether to overwrite the stored draws. If FALSE, adds to generated quantities. Default is FALSE.
    #' @param suffix Character string to append to the rotated variable names when overwrite is FALSE. Default is "rot".
    #' @param ... Additional arguments passed to the rotation function.
    #' @return The updated object invisibly.
    rotate = function(target, reference = NULL, linked = NULL) {
      message("Applying orthogonal Procrustes rotation (Saving to generate as _rot)...")

      target_map <- self$get_point_estimate(target)

      ref_name <- paste0(target, "_ref")
      if (!is.null(reference)) {
        self$model$data[[ref_name]] <- reference
      } else {
        self$model$data[[ref_name]] <- target_map
      }

      exprs <- list()
      exprs[[length(exprs) + 1]] <- bquote(svd_res <- svd(t(.(as.name(target))) %*% .(as.name(ref_name))))
      exprs[[length(exprs) + 1]] <- quote(Q <- svd_res$u %*% t(svd_res$v))

      target_rot <- paste0(target, "_rot")
      exprs[[length(exprs) + 1]] <- bquote(.(as.name(target_rot)) <- .(as.name(target)) %*% Q)

      if (!is.null(linked)) {
        for (l_var in linked) {
          l_rot <- paste0(l_var, "_rot")
          exprs[[length(exprs) + 1]] <- bquote(.(as.name(l_rot)) <- .(as.name(l_var)) %*% Q)
        }
      }

      ret_list <- list()
      ret_list[[target_rot]] <- as.name(target_rot)
      if (!is.null(linked)) {
        for (l_var in linked) {
          l_rot <- paste0(l_var, "_rot")
          ret_list[[l_rot]] <- as.name(l_rot)
        }
      }
      ret_list_call <- as.call(c(list(as.name("list")), ret_list))
      exprs[[length(exprs) + 1]] <- bquote(return(.(ret_list_call)))

      gen_ast <- as.call(c(list(as.name("{")), exprs))

      self$generated_quantities(gen_ast)

      if (!is.null(self$generate_fit)) {
        all_names <- c(
          if(!is.null(self$fit)) dimnames(self$fit)[[3]] else character(0),
          if(!is.null(self$transform_fit)) dimnames(self$transform_fit)[[3]] else character(0),
          dimnames(self$generate_fit)[[3]]
        )

        get_renamed_dimnames <- function(orig_var, new_var) {
          orig_pattern <- paste0("^", orig_var, "(\\[.*\\])?$")
          orig_flat_names <- grep(orig_pattern, all_names, value = TRUE)
          if (length(orig_flat_names) > 0) {
            return(sub(paste0("^", orig_var), new_var, orig_flat_names))
          }
          return(NULL)
        }

        gen_names <- dimnames(self$generate_fit)[[3]]

        new_target_names <- get_renamed_dimnames(target, target_rot)
        if (!is.null(new_target_names)) {
          target_gen_pattern <- paste0("^", target_rot, "(\\[.*\\])?$")
          idx <- grep(target_gen_pattern, gen_names)
          if (length(idx) == length(new_target_names)) {
            gen_names[idx] <- new_target_names
          }
        }

        if (!is.null(linked)) {
          for (l_var in linked) {
            l_rot <- paste0(l_var, "_rot")
            new_linked_names <- get_renamed_dimnames(l_var, l_rot)
            if (!is.null(new_linked_names)) {
              l_gen_pattern <- paste0("^", l_rot, "(\\[.*\\])?$")
              idx <- grep(l_gen_pattern, gen_names)
              if (length(idx) == length(new_linked_names)) {
                gen_names[idx] <- new_linked_names
              }
            }
          }
        }
        dimnames(self$generate_fit)[[3]] <- gen_names
      }

      invisible(self)
    },

    #' @description Rotate factor loadings and optional factor scores.
    #' @param target Character string specifying the target variable to base the rotation on.
    #' @param linked Character vector of variable names to be rotated in the same direction.
    #' @param scores Character vector of variable names to be rotated as factor scores (inverse direction).
    #' @param rotate Character string specifying the rotation method.
    #' @param ... Additional arguments passed to the rotation function.
    #' @return The updated object invisibly.
    fa_rotate = function(target = "L", linked = NULL, scores = NULL, rotate = "promax", ...) {
      message(sprintf("Applying %s rotation to %s (Saving to generate as _%s)...", rotate, target, rotate))

      if (exists(rotate, mode = "function")) {
        rot_fn <- match.fun(rotate)
        fn_call <- as.name(rotate)
      } else if (requireNamespace("GPArotation", quietly = TRUE) &&
                 exists(rotate, where = asNamespace("GPArotation"), mode = "function")) {
        rot_fn <- getFromNamespace(rotate, "GPArotation")
        fn_call <- call("::", as.name("GPArotation"), as.name(rotate))
      } else {
        stop("Rotation function not found: ", rotate, ". If this is from GPArotation, please install it using install.packages('GPArotation').")
      }

      target_map <- self$get_point_estimate(target)

      if (length(dim(target_map)) != 2) stop("fa_rotate is only applicable to matrix (2D) parameters.")

      test_rot <- rot_fn(target_map, ...)
      is_matrix_rot <- is.matrix(test_rot)
      has_phi <- !is_matrix_rot && !is.null(test_rot$Phi)

      exprs <- list()
      rot_name <- paste0(target, "_", rotate)

      exprs[[length(exprs) + 1]] <- bquote(rot_obj <- .(fn_call)(.(as.name(target))))
      ret_list <- list()

      if (is_matrix_rot) {
        exprs[[length(exprs) + 1]] <- quote(rot_mat <- unclass(rot_obj))
        ret_list[[rot_name]] <- as.name("rot_mat")
      } else {
        exprs[[length(exprs) + 1]] <- quote(rot_mat <- unclass(rot_obj$loadings))
        ret_list[[rot_name]] <- as.name("rot_mat")
        if (has_phi) {
          ret_list[["fa_cor"]] <- quote(rot_obj$Phi)
        }
      }

      if (!is.null(linked) || !is.null(scores)) {
        if (is_matrix_rot) {
          warning("The specified rotation function returns only a matrix, so the rotation matrix cannot be obtained. 'linked' and 'scores' will not be rotated.")
        } else {
          exprs[[length(exprs) + 1]] <- quote(rot_Th <- if (!is.null(rot_obj$Th)) rot_obj$Th else rot_obj$rotmat)

          if (!is.null(linked)) {
            for (l_var in linked) {
              l_rot <- paste0(l_var, "_", rotate)
              exprs[[length(exprs) + 1]] <- bquote(.(as.name(l_rot)) <- .(as.name(l_var)) %*% rot_Th)
              ret_list[[l_rot]] <- as.name(l_rot)
            }
          }

          if (!is.null(scores)) {
            exprs[[length(exprs) + 1]] <- quote(rot_Th_inv <- solve(t(rot_Th)))
            for (s_var in scores) {
              s_rot <- paste0(s_var, "_", rotate)
              exprs[[length(exprs) + 1]] <- bquote(.(as.name(s_rot)) <- .(as.name(s_var)) %*% rot_Th_inv)
              ret_list[[s_rot]] <- as.name(s_rot)
            }
          }
        }
      }

      ret_list_call <- as.call(c(list(as.name("list")), ret_list))
      exprs[[length(exprs) + 1]] <- bquote(return(.(ret_list_call)))

      gen_ast <- as.call(c(list(as.name("{")), exprs))
      self$generated_quantities(gen_ast)

      if (!is.null(self$generate_fit)) {
        all_names <- c(
          if(!is.null(self$fit)) dimnames(self$fit)[[3]] else character(0),
          if(!is.null(self$transform_fit)) dimnames(self$transform_fit)[[3]] else character(0),
          dimnames(self$generate_fit)[[3]]
        )

        get_renamed_dimnames <- function(orig_var, new_var) {
          orig_pattern <- paste0("^", orig_var, "(\\[.*\\])?$")
          orig_flat_names <- grep(orig_pattern, all_names, value = TRUE)
          if (length(orig_flat_names) > 0) {
            return(sub(paste0("^", orig_var), new_var, orig_flat_names))
          }
          return(NULL)
        }

        gen_names <- dimnames(self$generate_fit)[[3]]

        new_target_names <- get_renamed_dimnames(target, rot_name)
        if (!is.null(new_target_names)) {
          target_gen_pattern <- paste0("^", rot_name, "(\\[.*\\])?$")
          idx <- grep(target_gen_pattern, gen_names)
          if (length(idx) == length(new_target_names)) {
            gen_names[idx] <- new_target_names
          }
        }

        if (!is.null(linked)) {
          for (l_var in linked) {
            l_rot <- paste0(l_var, "_", rotate)
            new_linked_names <- get_renamed_dimnames(l_var, l_rot)
            if (!is.null(new_linked_names)) {
              l_gen_pattern <- paste0("^", l_rot, "(\\[.*\\])?$")
              idx <- grep(l_gen_pattern, gen_names)
              if (length(idx) == length(new_linked_names)) {
                gen_names[idx] <- new_linked_names
              }
            }
          }
        }

        if (!is.null(scores)) {
          for (s_var in scores) {
            s_rot <- paste0(s_var, "_", rotate)
            new_score_names <- get_renamed_dimnames(s_var, s_rot)
            if (!is.null(new_score_names)) {
              s_gen_pattern <- paste0("^", s_rot, "(\\[.*\\])?$")
              idx <- grep(s_gen_pattern, gen_names)
              if (length(idx) == length(new_score_names)) {
                gen_names[idx] <- new_score_names
              }
            }
          }
        }

        dimnames(self$generate_fit)[[3]] <- gen_names
      }

      invisible(self)
    }
  )
)

  # Simulate mediation data
  set.seed(123)
  N <- 100
  X <- rnorm(N)
  # M is influenced by X
  M <- 0.5 * X + rnorm(N, 0, 0.5)
  # Y is influenced by both X and M
  Y <- 0.3 * X + 0.8 * M + rnorm(N, 0, 0.5)
  dat <- data.frame(X, M, Y)

  # Fit a mediation model
  # The formula list specifies the two regression equations
  fit_med <- rtmb_mediation(list(M ~ X, Y ~ X + M), data = dat)
  
  # Maximum A Posteriori (MAP) estimation
  map_med <- fit_med$optimize()
  # The summary automatically calculates indirect, direct, and total effects
  map_med$summary()

#' VB fit object
#'
#' An R6 class storing posterior samples and related information
#' from Automatic Differentiation Variational Inference (ADVI).
#'
#' @field model An `RTMB_Model` object used for estimation.
#' @field fit A 3D array of posterior draws for fixed model parameters.
#' @field random_fit A 3D array of posterior draws for random effects, if available.
#' @field transform_fit A 3D array of posterior draws for transformed parameters, if available.
#' @field generate_fit A 3D array of posterior draws for generated quantities, if available.
#' @field transform_dims A list storing dimension information for transformed parameters.
#' @field generate_dims A list storing dimension information for generated quantities.
#' @field elbo_history A list of numeric vectors storing the Evidence Lower Bound (ELBO) history during optimization for each chain.
#' @field laplace Logical; whether Laplace approximation was used to marginalize random effects.
#' @field posterior_mean A named numeric vector of posterior mean estimates.
#' @field ELBO A numeric vector of final ELBO values for each chain.
#' @field rel_obj_vals A numeric vector of final relative objective tolerance values for each chain.
#' @field best_chain Integer; the index of the chain with the maximum ELBO.
#' @field mu_history Matrix of the parameter trajectory from the final window.
#'
#' @export
VB_Fit <- R6::R6Class(
  classname = "advi_fit",
  inherit = RTMB_Fit_Base,

  public = list(
    # --- Fields ---
    model          = NULL, # Reference to RTMB_Model instance
    fit            = NULL,
    random_fit     = NULL,
    transform_fit  = NULL, # Store transformed quantities
    generate_fit   = NULL, # Store generated quantities
    transform_dims = NULL, # Store dimension info for transformed quantities
    generate_dims  = NULL, # Store dimension info for generated quantities
    elbo_history   = NULL, # Store ELBO history
    laplace        = NULL,
    posterior_mean = NULL,
    ELBO           = NULL,
    rel_obj_vals   = NULL,
    best_chain     = NULL,
    mu_history     = NULL,

    # 1. Constructor
    #' @description Get point estimate for a target parameter.
    #' @param target Target parameter name.
    #' @param chains Numeric vector of chains to include. If NULL, all chains are used.
    #' @param best_chains Integer; number of best chains to retain based on ELBO.
    #' @return Matrix, array, vector, or scalar point estimate.
    get_point_estimate = function(target, chains = NULL, best_chains = NULL) {
      target_draws <- self$draws(pars = target, chains = chains, best_chains = best_chains, inc_transform = TRUE, inc_generate = TRUE)
      if (dim(target_draws)[3] == 0) stop("Parameter not found: ", target)

      lp_draws <- self$draws(pars = "lp", chains = chains, best_chains = best_chains, inc_transform = FALSE, inc_generate = FALSE)
      max_idx <- which(lp_draws == max(lp_draws, na.rm = TRUE), arr.ind = TRUE)
      best_iter <- max_idx[1, 1]
      best_chain <- max_idx[1, 2]

      t_info <- self$model$par_list[[target]]
      if (!is.null(t_info)) {
        t_dim <- t_info$dim
      } else if (!is.null(self$transform_dims[[target]])) {
        t_dim <- self$transform_dims[[target]]
      } else if (!is.null(self$generate_dims[[target]])) {
        t_dim <- self$generate_dims[[target]]
      } else {
        t_dim <- dim(target_draws)[3]
      }

      target_map_flat <- target_draws[best_iter, best_chain, ]
      target_map <- target_map_flat
      if (length(t_dim) > 1) dim(target_map) <- t_dim

      return(target_map)
    },

    #' @description Create a new `VB_Fit` object.
    #' @param model An `RTMB_Model` object.
    #' @param fit A 3D array of parameter draws.
    #' @param random_fit A 3D array of random effect draws.
    #' @param elbo_history A list of numeric vectors of ELBO values for each chain.
    #' @param laplace Logical; indicates if Laplace approximation was used.
    #' @param posterior_mean A named numeric vector of posterior means.
    #' @param ELBO A numeric vector of final ELBO values for each chain.
    #' @param rel_obj_vals A numeric vector of final relative objective tolerance values for each chain.
    #' @param best_chain Integer; the index of the chain with the maximum ELBO.
    #' @param mu_history Matrix of the parameter trajectory from the final window.
    initialize = function(model, fit, random_fit, elbo_history, laplace, posterior_mean, ELBO, rel_obj_vals, best_chain, mu_history) {
      self$model <- model
      self$fit <- fit
      self$random_fit <- random_fit
      self$elbo_history <- elbo_history
      self$laplace <- laplace
      self$posterior_mean <- posterior_mean
      self$ELBO <- ELBO
      self$rel_obj_vals <- rel_obj_vals
      self$best_chain <- best_chain
      self$mu_history <- mu_history
      self$transform_fit <- NULL
      self$transform_dims <- list()
      self$generate_fit <- NULL
      self$generate_dims <- list()
      # Ensure S3 dispatch works for base class methods
      class(self) <- c(class(self), "RTMB_Fit_Base")
    },


    #' @description Print a brief summary of the fitted object.
    #' @param ... Additional arguments passed to the `summary` method.
    #' @return The object itself, invisibly.
    print = function(...) {
      print(self$summary(...))
      invisible(self)
    },

    #' @description Extract posterior draws for selected parameters.
    #' @param pars Character or numeric vector specifying the names or indices of parameters to extract. If NULL, all available parameters are extracted.
    #' @param chains Numeric vector specifying the chains to extract. If NULL, draws from all chains are returned.
    #' @param best_chains Integer; number of best chains to retain based on ELBO. If supplied, chains with the highest ELBO are retained.
    #' @param inc_random Logical; whether to include random effects in the output. Default is FALSE.
    #' @param inc_transform Logical; whether to include transformed parameters in the output. Default is TRUE.
    #' @param inc_generate Logical; whether to include generated quantities in the output. Default is TRUE.
    #' @param best_only Logical; whether to extract only from the chain with the maximum ELBO. Default is FALSE unless explicitly requested.
    #' @return A 3D array of posterior draws `[iterations, chains, parameters]`.
    draws = function(pars = NULL, chains = NULL, best_chains = NULL,
                     inc_random = FALSE, inc_transform = TRUE, inc_generate = TRUE,
                     best_only = FALSE) {
      total_chains <- dim(self$fit)[2]
      available_chains <- seq_len(total_chains)

      if (!is.null(chains)) {
        available_chains <- intersect(available_chains, chains)
        if (length(available_chains) == 0) {
          stop("The specified chains were not found.", call. = FALSE)
        }
      }

      if (!is.null(best_chains)) {
        elbo_vals <- self$ELBO[available_chains]
        n_best <- min(best_chains, length(available_chains))
        ordered_idx <- order(elbo_vals, decreasing = TRUE, na.last = NA)
        available_chains <- available_chains[ordered_idx[seq_len(n_best)]]
      }

      available_chains <- sort(unique(available_chains))
      out_array <- self$fit[, available_chains, , drop = FALSE]

      if (inc_random && !is.null(self$random_fit)) {
        P1 <- dim(out_array)[3]; P2 <- dim(self$random_fit)[3]
        I <- dim(out_array)[1]; C <- dim(out_array)[2]
        new_out <- array(NA, dim = c(I, C, P1 + P2))
        new_out[,,1:P1] <- out_array
        new_out[,,(P1+1):(P1+P2)] <- self$random_fit[, available_chains, , drop = FALSE]
        dimnames(new_out) <- list(
          iteration = dimnames(out_array)[[1]],
          chain = dimnames(out_array)[[2]],
          variable = c(dimnames(out_array)[[3]], dimnames(self$random_fit)[[3]])
        )
        out_array <- new_out
      }

      if (inc_transform && !is.null(self$transform_fit)) {
        P1 <- dim(out_array)[3]; P2 <- dim(self$transform_fit)[3]
        I <- dim(out_array)[1]; C <- dim(out_array)[2]
        new_out <- array(NA, dim = c(I, C, P1 + P2))
        new_out[,,1:P1] <- out_array
        new_out[,,(P1+1):(P1+P2)] <- self$transform_fit[, available_chains, , drop = FALSE]
        dimnames(new_out) <- list(
          iteration = dimnames(out_array)[[1]],
          chain = dimnames(out_array)[[2]],
          variable = c(dimnames(out_array)[[3]], dimnames(self$transform_fit)[[3]])
        )
        out_array <- new_out
      }

      if (inc_generate && !is.null(self$generate_fit)) {
        P1 <- dim(out_array)[3]; P2 <- dim(self$generate_fit)[3]
        I <- dim(out_array)[1]; C <- dim(out_array)[2]
        new_out <- array(NA, dim = c(I, C, P1 + P2))
        new_out[,,1:P1] <- out_array
        new_out[,,(P1+1):(P1+P2)] <- self$generate_fit[, available_chains, , drop = FALSE]
        dimnames(new_out) <- list(
          iteration = dimnames(out_array)[[1]],
          chain = dimnames(out_array)[[2]],
          variable = c(dimnames(out_array)[[3]], dimnames(self$generate_fit)[[3]])
        )
        out_array <- new_out
      }

      P <- dim(out_array)[3]
      param_names <- dimnames(out_array)[[3]]
      if (is.null(param_names)) param_names <- paste0("V", 1:P)
      target_idx <- 1:P

      if (!is.null(pars)) {
        if (is.numeric(pars)) {
          # Support both positive and negative numeric indexing
          target_idx <- tryCatch(seq_along(param_names)[pars], error = function(e) NULL)
          if (is.null(target_idx) || length(target_idx) == 0) {
            stop("The index specified in 'pars' was not found or is invalid.", call. = FALSE)
          }
        } else if (is.character(pars)) {
          base_names <- gsub("\\[.*\\]$", "", param_names)
          
          # Check for exclusion (e.g., "-theta")
          if (any(grepl("^-", pars))) {
            exclude_names <- gsub("^-", "", pars)
            target_idx <- which(!(param_names %in% exclude_names | base_names %in% exclude_names))
          } else {
            ordered_idx <- integer(0)
            for (p in pars) {
              match_idx <- which(param_names == p | base_names == p)
              ordered_idx <- c(ordered_idx, match_idx)
            }
            target_idx <- unique(ordered_idx)
          }
          
          if (length(target_idx) == 0) {
            stop("The variable name specified in 'pars' was not found.", call. = FALSE)
          }
        } else {
          stop("'pars' must be either numeric or character.", call. = FALSE)
        }
      }

      if (best_only) {
        if (!(self$best_chain %in% available_chains)) {
          stop("The best chain is not included in the selected chains.", call. = FALSE)
        }
        best_pos <- which(available_chains == self$best_chain)[1]
        return(out_array[, best_pos, target_idx, drop = FALSE])
      }

      return(out_array[, , target_idx, drop = FALSE])
    },

    #' @description Summarize posterior draws.
    #' @param pars Character or numeric vector specifying the names or indices of parameters to summarize. If NULL, all available parameters are summarized.
    #' @param max_rows Integer; maximum number of rows to print in the summary table. Default is 10.
    #' @param digits Integer; number of decimal places to print. Default is 2.
    #' @param inc_random Logical; whether to include random effects in the summary. Default is FALSE.
    #' @param inc_transform Logical; whether to include transformed parameters in the summary. Default is TRUE.
    #' @param inc_generate Logical; whether to include generated quantities in the summary. Default is TRUE.
    #' @return A data frame containing the summarized posterior statistics.
    summary = function(pars = NULL, max_rows = 10, digits = 2,
                       inc_random = FALSE, inc_transform = TRUE, inc_generate = TRUE) {

      draws_best <- self$draws(pars = pars,
                               inc_random = inc_random,
                               inc_transform = inc_transform,
                               inc_generate = inc_generate,
                               best_only = TRUE)

      draws_all <- self$draws(pars = pars,
                              inc_random = inc_random,
                              inc_transform = inc_transform,
                              inc_generate = inc_generate,
                              best_only = FALSE)

      P <- dim(draws_best)[3]
      param_names <- dimnames(draws_best)[[3]]
      target_idx <- seq_len(P)

      if (length(target_idx) > 0 && is.null(pars)) {
        current_names <- param_names[target_idx]
        base_names <- gsub("\\[.*\\]$", "", current_names)
        target_views <- c("lp")
        if (!is.null(self$model$view)) {
          target_views <- c(target_views, self$model$view)
        }

        priority_sub_idx <- integer(0)
        for (v in target_views) {
          match_idx <- which(current_names == v | base_names == v)
          priority_sub_idx <- c(priority_sub_idx, match_idx)
        }
        priority_sub_idx <- unique(priority_sub_idx)
        other_sub_idx <- setdiff(seq_along(current_names), priority_sub_idx)
        target_idx <- target_idx[c(priority_sub_idx, other_sub_idx)]
      }

      if (!is.null(max_rows)) {
        limit <- min(length(target_idx), as.integer(max_rows))
        target_idx <- target_idx[seq_len(limit)]
      }

      max_rel_obj <- if (!is.null(self$rel_obj_vals) && length(self$rel_obj_vals) >= self$best_chain) {
        self$rel_obj_vals[self$best_chain]
      } else {
        NA_real_
      }

      res_list_sum <- vector("list", length(target_idx))

      for (i in seq_along(target_idx)) {
        p <- target_idx[i]
        vec_best <- as.vector(draws_best[, 1, p])
        valid_vec <- vec_best[is.finite(vec_best)]
        mat_all <- as.matrix(draws_all[, , p, drop = FALSE])
        var_name <- param_names[p]

        if (length(valid_vec) == 0) {
          res_list_sum[[i]] <- data.frame(
            variable = var_name,
            mean = NA, sd = NA, map = NA,
            q2.5 = NA, q97.5 = NA,
            `Max rel_obj` = max_rel_obj,
            rhat = NA,
            stringsAsFactors = FALSE,
            check.names = FALSE
          )
          next
        }

        sd_val <- sd(valid_vec)
        if (is.na(sd_val) || sd_val < 1e-10) {
          map_val <- valid_vec[1]
          q95 <- c(valid_vec[1], valid_vec[1])
        } else {
          map_val <- map_est(valid_vec)
          q95 <- quantile95(valid_vec)
        }

        if (ncol(mat_all) >= 2 && all(apply(mat_all, 2, function(z) sum(is.finite(z)) > 1))) {
          rhat_val <- r_hat(mat_all)
        } else {
          rhat_val <- NA_real_
        }

        res_list_sum[[i]] <- data.frame(
          variable = var_name,
          mean = mean(valid_vec),
          sd = sd_val,
          map = map_val,
          q2.5 = unname(q95[1]),
          q97.5 = unname(q95[2]),
          #`Max rel_obj` = max_rel_obj,
          #rhat = rhat_val,
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
      }

      res_df <- do.call(rbind, res_list_sum)
      num_cols <- sapply(res_df, is.numeric)
      res_df[num_cols] <- lapply(res_df[num_cols], function(x) {
        x[abs(x) < 1e-12 & !is.na(x)] <- 0
        x
      })
      class(res_df) <- c("summary_BayesRTMB", "data.frame")
      attr(res_df, "digits") <- digits
      return(res_df)
    },

    #' @description Plot the ELBO history to diagnose convergence.
    #' @param tail_n Integer; the number of recent iterations to plot. If NULL, plots the entire history. Default is 2000.
    #' @param ests Character string `"best"`, numeric vector of estimate indices (e.g., `c(1, 3)`), or `NULL` to plot all. Default is `NULL`.
    #' @param type Character string; the type of plot. Default is "l" (lines).
    #' @param ... Additional arguments passed to the `plot` function.
    #' @return The object itself, invisibly.
    plot_elbo = function(tail_n = 1000, ests = NULL, type = "l", ...) {
      history_list <- self$elbo_history
      num_estimate <- length(history_list)
      if (num_estimate == 0) {
        cat("No ELBO history available.\n")
        return(invisible(self))
      }

      n_total <- length(history_list[[1]])
      plot_data <- do.call(cbind, history_list)
      colnames(plot_data) <- paste0("est", seq_len(num_estimate))

      start_iter <- 1
      if (!is.null(tail_n) && tail_n > 0 && tail_n < n_total) {
        plot_data <- plot_data[(n_total - tail_n + 1):n_total, , drop = FALSE]
        start_iter <- n_total - tail_n + 1
      }

      x_axis <- seq(start_iter, n_total)
      main_title <- if (!is.null(tail_n) && tail_n < n_total) {
        sprintf("ELBO History (Last %d iterations)", tail_n)
      } else {
        "ELBO History"
      }

      best_c <- self$best_chain
      max_rel_obj_val <- self$rel_obj_vals[best_c]
      sub_title <- if(!is.na(max_rel_obj_val)) sprintf("Best (est%d) Final rel_obj: %.5f", best_c, max_rel_obj_val) else ""

      # --- Chain selection for plotting ---
      if (is.null(ests)) {
        target_idx <- seq_len(num_estimate)
      } else if (identical(ests, "best")) {
        target_idx <- best_c
      } else if (is.numeric(ests)) {
        target_idx <- intersect(ests, seq_len(num_estimate))
        if (length(target_idx) == 0) {
          cat("Specified estimates are out of bounds.\n")
          return(invisible(self))
        }
      } else {
        cat("Invalid 'ests' argument. Use NULL, \"best\", or a numeric vector.\n")
        return(invisible(self))
      }

      plot_data <- plot_data[, target_idx, drop = FALSE]

      # --- Line color and thickness adjustment ---
      # Default is gray (black if only 1), highlight blue only if best chain is included
      cols <- rep("gray", length(target_idx))
      lwds <- rep(1, length(target_idx))

      best_in_plot <- which(target_idx == best_c)
      if (length(best_in_plot) > 0) {
        cols[best_in_plot] <- "dodgerblue"
        lwds[best_in_plot] <- 2
      } else if (length(target_idx) == 1) {
        cols <- "black"
      }

      matplot(x_axis, plot_data, type = type, lty = 1, col = cols, lwd = lwds,
              xlab = "Iteration", ylab = "ELBO",
              main = main_title, sub = sub_title, ...)

      # --- Draw legend ---
      if (length(target_idx) > 1) {
        if (length(best_in_plot) > 0) {
          legend("bottomright", legend = c(paste("Best (est", best_c, ")"), "Others"),
                 col = c("dodgerblue", "gray"), lwd = c(2, 1), lty = 1, bty = "n")
        } else {
          legend("bottomright", legend = colnames(plot_data),
                 col = cols, lwd = lwds, lty = 1, bty = "n")
        }
      } else {
        legend("bottomright", legend = colnames(plot_data),
               col = cols, lwd = lwds, lty = 1, bty = "n")
      }

      invisible(self)
    },

    #' @description Plot the parameter trajectory from the final optimization window.
    #' @param pars Character vector specifying the names of parameters to plot. If NULL, plots all available parameters.
    #' @param type Character string; the type of plot. Default is "l" (lines).
    #' @param ... Additional arguments passed to the `matplot` function.
    #' @return The object itself, invisibly.
    plot_trajectory = function(pars = NULL, type = "l", ...) {
      if (is.null(self$mu_history)) {
        cat("No trajectory data available (mu_history is NULL).\n")
        return(invisible(self))
      }

      plot_data <- self$mu_history

      if (!is.null(pars)) {
        valid_pars <- character(0)
        for (p in pars) {
          # 1. Search by exact match (e.g., "delta[1]" or "mat[1,2]")
          if (p %in% colnames(plot_data)) {
            valid_pars <- c(valid_pars, p)
          } else {
            # 2. Bulk search by base name (e.g., "delta" -> "delta[1]", "mat" -> "mat[1,1]")
            pattern <- paste0("^", p, "\\[.*\\]$")
            matched <- grep(pattern, colnames(plot_data), value = TRUE)
            valid_pars <- c(valid_pars, matched)
          }
        }
        valid_pars <- unique(valid_pars)

        if (length(valid_pars) == 0) {
          cat("None of the specified parameters were found in the trajectory data.\n")
          return(invisible(self))
        }
        plot_data <- plot_data[, valid_pars, drop = FALSE]
      }

      n_steps <- nrow(plot_data)
      x_axis <- seq_len(n_steps)

      # --- Automatic adjustment algorithm for vertical axis (ylim) ---
      y_min <- min(plot_data, na.rm = TRUE)
      y_max <- max(plot_data, na.rm = TRUE)
      y_mean <- mean(plot_data, na.rm = TRUE)
      y_range <- y_max - y_min

      args_list <- list(...)

      # Auto-adjust only if ylim is not explicitly specified by user
      if (!"ylim" %in% names(args_list)) {
        # Minimum vertical axis range to secure (max of absolute 0.1 or 5% of mean)
        min_range <- max(0.1, abs(y_mean) * 0.05)

        if (y_range < min_range) {
          # If fluctuation is too small, apply minimum range to make it look "flat"
          y_min <- y_mean - min_range / 2
          y_max <- y_mean + min_range / 2
        } else {
          # If fluctuation is sufficient, add 5% margin to top and bottom
          y_min <- y_min - y_range * 0.05
          y_max <- y_max + y_range * 0.05
        }
        args_list$ylim <- c(y_min, y_max)
      }

      main_title <- sprintf("Parameter Trajectory (Last %d steps)", n_steps)

      # --- Dynamically build arguments for matplot and call ---
      args_list$x <- x_axis
      args_list$y <- plot_data
      args_list$type <- type
      args_list$lty <- 1
      args_list$xlab <- "Iterations (in final window)"
      if (!"ylab" %in% names(args_list)) args_list$ylab <- "Parameter Value"
      if (!"main" %in% names(args_list)) args_list$main <- main_title

      do.call(matplot, args_list)

      if (ncol(plot_data) <= 10) {
        legend("topright", legend = colnames(plot_data),
               col = seq_len(ncol(plot_data)), lty = 1, cex = 0.8,
               bty = "n", inset = c(0.02, 0.02))
      }

      invisible(self)
    },

    #' @description Compute transformed parameters from posterior draws.
    #' @param tran_fn An optional user-supplied function that takes data and parameter lists to return transformed quantities.
    #' @return The `VB_Fit` object itself, invisibly.
    transformed_draws = function(tran_fn = NULL) {
      all_draws <- self$draws(
        inc_random = TRUE,
        inc_transform = FALSE,
        inc_generate = FALSE,
        best_only = FALSE
      )

      iter   <- dim(all_draws)[1]
      chains <- dim(all_draws)[2]

      if (is.null(tran_fn)) tran_fn <- self$model$transform

      wrapper_tran_fn <- function(dat, param) {
        res <- list()

        if (!is.null(tran_fn)) {
          user_res <- tran_fn(dat, param)
          if (is.null(user_res)) user_res <- list()
          res <- c(res, user_res)
        }
        return(res)
      }

      test_para <- constrained_vector_to_list(all_draws[1, 1, -1], self$model$par_list)
      test_tran <- wrapper_tran_fn(self$model$data, test_para)

      if (length(test_tran) == 0) return(invisible(self))

      tran_names <- character(0)
      self$transform_dims <- list()

      for (name in names(test_tran)) {
        val <- test_tran[[name]]
        len <- length(val)
        dim_val <- dim(val)
        if (is.null(dim_val)) dim_val <- len
        self$transform_dims[[name]] <- dim_val

        names_def <- self$model$par_names[[name]]
        flat_nms <- generate_flat_names(name, dim_val, names_def)
        tran_names <- c(tran_names, flat_nms)
      }

      tran_array <- array(NA, dim = c(iter, chains, length(tran_names)))
      dimnames(tran_array) <- list(
        iteration = NULL,
        chain = paste0("est", seq_len(chains)),
        variable = tran_names
      )

      pb <- txtProgressBar(min = 0, max = iter * chains, style = 3)
      counter <- 0
      for (c in seq_len(chains)) {
        for (i in seq_len(iter)) {
          p_list <- constrained_vector_to_list(all_draws[i, c, -1], self$model$par_list)
          res <- wrapper_tran_fn(self$model$data, p_list)
          tran_array[i, c, ] <- unlist(res, use.names = FALSE)
          counter <- counter + 1
          setTxtProgressBar(pb, counter)
        }
      }
      close(pb)

      self$transform_fit <- tran_array
      return(invisible(self))
    },

    #' @description Compute generated quantities from posterior draws.
    #' @param code An `rtmb_code(\{ ... \})` or `\{ ... \}` block containing the logic
    #' to be calculated using posterior samples.
    #' @return The `VB_Fit` object itself (invisibly).
    #' Results are appended to the `generate_fit` field.
    generated_quantities = function(code) {
      raw_code <- substitute(code)

      if (is.name(raw_code)) {
        evaluated <- tryCatch(eval(raw_code, envir = parent.frame()), error = function(e) NULL)
        if (is.language(evaluated) || is.call(evaluated)) {
          code <- evaluated
          raw_code <- evaluated
        }
      }

      if (is.call(raw_code) && identical(raw_code[[1]], as.name("rtmb_code"))) {
        parsed_code <- eval(raw_code, envir = parent.frame())
        if (!"generate" %in% names(parsed_code)) {
          stop("There is no 'generate' block in rtmb_code().")
        }
        gen_ast <- parsed_code$generate
      } else if (is.call(raw_code) && identical(raw_code[[1]], as.name("{"))) {
        gen_ast <- raw_code
      } else if (is.call(code) && identical(code[[1]], as.name("{"))) {
        gen_ast <- code
      } else if (is.list(code) && "generate" %in% names(code)) {
        gen_ast <- code$generate
      } else {
        stop("'code' must be specified in the format rtmb_code(generate = { ... }) or { ... }.")
      }

      gen_fn <- eval(bquote(transform_code(.(gen_ast))))
      environment(gen_fn) <- parent.env(globalenv())

      # 3. Get all samples
      all_draws <- self$draws(inc_random = TRUE, inc_transform = TRUE, inc_generate = FALSE, best_only = FALSE)
      iter   <- dim(all_draws)[1]
      chains <- dim(all_draws)[2]

      # 4. Get dimension information
      test_p_list <- constrained_vector_to_list(all_draws[1, 1, -1], self$model$par_list)
      if (!is.null(self$model$transform)) {
        tran_res <- self$model$transform(self$model$data, test_p_list)
        if (is.list(tran_res)) test_p_list <- c(test_p_list, tran_res)
      }

      if (!is.null(self$generate_fit)) {
        existing_gq <- dimnames(self$generate_fit)[[3]]
        for (g_name in names(self$generate_dims)) {
          g_dim <- self$generate_dims[[g_name]]
          flat_nms <- generate_flat_names(g_name, g_dim, self$model$par_names[[g_name]])
          # Extract only those existing in generate_fit
          if (all(flat_nms %in% existing_gq)) {
            val <- self$generate_fit[1, 1, flat_nms]
            if (length(g_dim) > 1) dim(val) <- g_dim
            test_p_list[[g_name]] <- val
          }
        }
      }
      test_gq <- gen_fn(self$model$data, test_p_list)

      if (is.null(test_gq) || length(test_gq) == 0) return(invisible(self))

      gq_names <- character(0)
      for (name in names(test_gq)) {
        val <- test_gq[[name]]
        dim_val <- if (is.null(dim(val))) length(val) else dim(val)
        self$generate_dims[[name]] <- dim_val
        gq_names <- c(gq_names, generate_flat_names(name, dim_val, self$model$par_names[[name]]))
      }

      # 5. Array for storing results
      new_gq_array <- array(NA, dim = c(iter, chains, length(gq_names)))
      dimnames(new_gq_array) <- list(iteration = NULL, chain = paste0("est", seq_len(chains)), variable = gq_names)

      # 6. Execute for all estimates and samples
      pb <- txtProgressBar(min = 0, max = iter * chains, style = 3)
      counter <- 0
      for (c in seq_len(chains)) {
        for (i in seq_len(iter)) {
          p_list <- constrained_vector_to_list(all_draws[i, c, -1], self$model$par_list)
          if (!is.null(self$model$transform)) {
            tran_res <- self$model$transform(self$model$data, p_list)
            if (is.list(tran_res)) p_list <- c(p_list, tran_res)
          }
          if (!is.null(self$generate_fit)) {
            existing_gq <- dimnames(self$generate_fit)[[3]]
            for (g_name in names(self$generate_dims)) {
              g_dim <- self$generate_dims[[g_name]]
              flat_nms <- generate_flat_names(g_name, g_dim, self$model$par_names[[g_name]])
              # Extract only those existing in generate_fit
              if (all(flat_nms %in% existing_gq)) {
                val <- self$generate_fit[i, c, flat_nms]
                if (length(g_dim) > 1) dim(val) <- g_dim
                p_list[[g_name]] <- val
              }
            }
          }
          res <- gen_fn(self$model$data, p_list)
          new_gq_array[i, c, ] <- unlist(res, use.names = FALSE)
          counter <- counter + 1
          setTxtProgressBar(pb, counter)
        }
      }
      close(pb)

      # 7. Merge with existing results
      if (is.null(self$generate_fit)) {
        self$generate_fit <- new_gq_array
      } else {
        old_gq <- self$generate_fit
        merged_gq <- array(NA, dim = c(iter, chains, dim(old_gq)[3] + dim(new_gq_array)[3]))
        merged_gq[,,1:dim(old_gq)[3]] <- old_gq
        merged_gq[,,(dim(old_gq)[3]+1):dim(merged_gq)[3]] <- new_gq_array
        dimnames(merged_gq) <- list(NULL, dimnames(old_gq)[[2]], c(dimnames(old_gq)[[3]], gq_names))
        self$generate_fit <- merged_gq
      }

      cat("Generated quantities added to samples.\n")
      invisible(self)
    },

    #' @description Compute WAIC from pointwise generated log likelihood.
    #' @param ... Additional arguments passed to `draws()`, such as `chains` or `best_chains`.
    #' @return A `waic_BayesRTMB` object.
    WAIC = function(...) {
      .waic_from_fit_draws(self, ...)
    },

    #' @description Run basic diagnostics for the variational fit.
    #' @param ... Additional arguments passed to `diagnose_vb_fit()`.
    #' @return A `diagnose_BayesRTMB` object.
    diagnose = function(...) {
      diagnose_vb_fit(self, ...)
    }
  )
)

  # Estimate the correlation between two variables in the debate dataset
  data(debate, package = "BayesRTMB")

  fit_corr <- rtmb_corr(cbind(sat, perf), data = debate)

  \donttest{
  mcmc_corr <- fit_corr$sample(sampling = 500, warmup = 500, chains = 2)
  mcmc_corr$summary()

  bf_corr <- mcmc_corr$bayes_factor(fixed = list(corr = 0))
  print(bf_corr)
  }

.diagnostic_row <- function(check, status, message) {
  data.frame(
    check = check,
    status = status,
    message = message,
    stringsAsFactors = FALSE
  )
}

.diagnostic_status <- function(checks) {
  if (is.null(checks) || nrow(checks) == 0L) return("ok")
  if (any(checks$status == "problem", na.rm = TRUE)) "problem"
  else if (any(checks$status == "warning", na.rm = TRUE)) "warning"
  else "ok"
}

.new_diagnose_result <- function(fit_type, checks, details = list()) {
  out <- list(
    fit_type = fit_type,
    status = .diagnostic_status(checks),
    checks = checks,
    details = details
  )
  class(out) <- "diagnose_BayesRTMB"
  out
}

.has_generated_log_lik <- function(fit) {
  if (!is.null(fit$generate) && !is.null(fit$generate$log_lik)) return(TRUE)
  if (!is.null(fit$generate_fit)) {
    gnames <- dimnames(fit$generate_fit)[[3L]]
    if (any(gnames == "log_lik" | grepl("^log_lik\\[", gnames))) return(TRUE)
  }
  if (!is.null(fit$se_samples) && !is.null(fit$se_samples$gq) &&
      !is.null(fit$se_samples$gq$log_lik)) return(TRUE)
  FALSE
}

.summarize_se_tables <- function(...) {
  tabs <- list(...)
  tabs <- tabs[vapply(tabs, is.data.frame, logical(1))]
  if (length(tabs) == 0L) return(NULL)
  do.call(rbind, tabs)
}

diagnose_map_fit <- function(fit, ...) {
  checks <- list()
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "convergence",
    if (identical(fit$convergence, 0L) || identical(fit$convergence, 0)) "ok" else "warning",
    if (identical(fit$convergence, 0L) || identical(fit$convergence, 0)) "optimizer converged" else paste("optimizer convergence code:", fit$convergence)
  )
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "finite_objective",
    if (is.finite(fit$objective)) "ok" else "problem",
    if (is.finite(fit$objective)) "objective is finite" else "objective is not finite"
  )

  if (identical(fit$ci_method, "none") || is.null(fit$sd_rep)) {
    checks[[length(checks) + 1L]] <- .diagnostic_row("hessian", "skipped", "SE calculation was not requested")
  } else {
    pd <- fit$sd_rep$pdHess
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "hessian",
      if (isTRUE(pd)) "ok" else "warning",
      if (isTRUE(pd)) "pdHess is TRUE" else "pdHess is not TRUE; Wald SE may be unreliable"
    )
  }

  if (!is.null(fit$fallback_needed)) {
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "se_fallback",
      if (isTRUE(fit$fallback_needed)) "warning" else "ok",
      if (isTRUE(fit$fallback_needed)) "Hessian/SE fallback was used" else "no Hessian/SE fallback recorded"
    )
  }

  if (!is.null(fit$vcov_unc)) {
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "vcov_unc",
      if (all(is.finite(fit$vcov_unc))) "ok" else "warning",
      if (all(is.finite(fit$vcov_unc))) "unconstrained covariance is finite" else "unconstrained covariance contains non-finite values"
    )
  }

  se_tab <- .summarize_se_tables(fit$df_fixed, fit$random_effects, fit$df_transform, fit$df_generate)
  if (!is.null(se_tab) && "Std. Error" %in% names(se_tab)) {
    se_vals <- se_tab[["Std. Error"]]
    bad <- !is.na(se_vals) & !is.finite(se_vals)
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "standard_errors",
      if (any(bad)) "warning" else "ok",
      if (any(bad)) paste(sum(bad), "standard errors are non-finite") else "reported standard errors are finite or NA by design"
    )
  }

  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "generated_log_lik",
    if (.has_generated_log_lik(fit)) "ok" else "skipped",
    if (.has_generated_log_lik(fit)) "pointwise log_lik is available for WAIC" else "pointwise log_lik is not available"
  )

  .new_diagnose_result("MAP_Fit", do.call(rbind, checks), list(
    marginal_vars = fit$marginal_vars,
    laplace_random_vars = fit$laplace_random_vars,
    ci_method = fit$ci_method
  ))
}

diagnose_mcmc_fit <- function(fit, rhat_warning = 1.01, rhat_problem = 1.05,
                              ess_warning = 400, ess_problem = 100, ...) {
  checks <- list()
  ndraws <- dim(fit$fit)[1L]
  nchains <- dim(fit$fit)[2L]
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "draw_count",
    if (ndraws * nchains < ess_problem) "warning" else "ok",
    paste("retained draws:", ndraws * nchains, "from", nchains, "chain(s)")
  )

  lp_idx <- which(dimnames(fit$fit)[[3L]] == "lp")
  if (length(lp_idx) == 1L) {
    lp <- fit$fit[, , lp_idx, drop = TRUE]
    nonfinite <- sum(!is.finite(lp))
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "finite_lp",
      if (nonfinite == 0L) "ok" else "warning",
      if (nonfinite == 0L) "all retained lp values are finite" else paste(nonfinite, "retained lp values are non-finite")
    )
  }

  if (!is.null(fit$accept)) {
    min_accept <- min(fit$accept, na.rm = TRUE)
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "acceptance",
      if (min_accept < 0.4) "problem" else if (min_accept < 0.6) "warning" else "ok",
      sprintf("minimum mean chain acceptance is %.3f", min_accept)
    )
  }

  if (!is.null(fit$treedepth)) {
    max_td <- max(fit$treedepth, na.rm = TRUE)
    max_allowed <- fit$max_treedepth %||% NA
    td_status <- if (is.finite(max_allowed) && max_td >= max_allowed) "warning" else "ok"
    td_msg <- if (is.finite(max_allowed)) {
      sprintf("maximum treedepth is %s (limit %s)", max_td, max_allowed)
    } else {
      sprintf("maximum treedepth is %s", max_td)
    }
    checks[[length(checks) + 1L]] <- .diagnostic_row("treedepth", td_status, td_msg)
  }

  summ <- tryCatch(fit$summary(max_rows = NULL, inc_random = FALSE, inc_transform = FALSE, inc_generate = FALSE), error = function(e) NULL)
  if (is.data.frame(summ)) {
    if ("rhat" %in% names(summ)) {
      max_rhat <- suppressWarnings(max(summ$rhat, na.rm = TRUE))
      if (is.finite(max_rhat)) {
        checks[[length(checks) + 1L]] <- .diagnostic_row(
          "rhat",
          if (max_rhat > rhat_problem) "problem" else if (max_rhat > rhat_warning) "warning" else "ok",
          sprintf("maximum R-hat is %.3f", max_rhat)
        )
      }
    }
    ess_cols <- intersect(c("ess_bulk", "ess_tail"), names(summ))
    if (length(ess_cols) > 0L) {
      min_ess <- suppressWarnings(min(as.matrix(summ[ess_cols]), na.rm = TRUE))
      if (is.finite(min_ess)) {
        checks[[length(checks) + 1L]] <- .diagnostic_row(
          "ess",
          if (min_ess < ess_problem) "problem" else if (min_ess < ess_warning) "warning" else "ok",
          sprintf("minimum reported ESS is %.1f", min_ess)
        )
      }
    }
  }

  if (!is.null(fit$pd_error_count)) {
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "pd_errors",
      if (sum(fit$pd_error_count, na.rm = TRUE) > 0L) "warning" else "ok",
      paste("positive-definite/singularity fallback count:", sum(fit$pd_error_count, na.rm = TRUE))
    )
  }

  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "generated_log_lik",
    if (.has_generated_log_lik(fit)) "ok" else "skipped",
    if (.has_generated_log_lik(fit)) "pointwise log_lik is available for WAIC" else "pointwise log_lik is not available"
  )

  .new_diagnose_result("MCMC_Fit", do.call(rbind, checks), list(laplace = fit$laplace))
}

diagnose_vb_fit <- function(fit, rel_obj_warning = 1e-3, rel_obj_problem = 1e-2, ...) {
  checks <- list()
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "ELBO",
    if (all(is.finite(fit$ELBO))) "ok" else "problem",
    if (all(is.finite(fit$ELBO))) "final ELBO values are finite" else "ELBO contains non-finite values"
  )
  if (!is.null(fit$rel_obj_vals)) {
    best_rel <- suppressWarnings(min(fit$rel_obj_vals, na.rm = TRUE))
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "relative_objective",
      if (best_rel > rel_obj_problem) "problem" else if (best_rel > rel_obj_warning) "warning" else "ok",
      sprintf("best final relative objective value is %.3g", best_rel)
    )
  }
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "best_chain",
    if (!is.null(fit$best_chain) && is.finite(fit$best_chain)) "ok" else "warning",
    paste("best chain:", fit$best_chain %||% NA)
  )
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "generated_log_lik",
    if (.has_generated_log_lik(fit)) "ok" else "skipped",
    if (.has_generated_log_lik(fit)) "pointwise log_lik is available for WAIC" else "pointwise log_lik is not available"
  )
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "variational_note",
    "skipped",
    "ADVI is an approximate posterior; uncertainty and WAIC may be optimistic relative to MCMC"
  )
  .new_diagnose_result("VB_Fit", do.call(rbind, checks), list(laplace = fit$laplace))
}

diagnose_classic_fit <- function(fit, ...) {
  checks <- list()
  checks[[length(checks) + 1L]] <- .diagnostic_row(
    "finite_logLik",
    if (is.finite(as.numeric(fit$logLik()))) "ok" else "warning",
    if (is.finite(as.numeric(fit$logLik()))) "logLik is finite" else "logLik is not finite or unavailable"
  )
  se_tab <- .summarize_se_tables(fit$fit, fit$random_effects, fit$df_transform, fit$df_generate)
  if (!is.null(se_tab) && "Std. Error" %in% names(se_tab)) {
    se_vals <- se_tab[["Std. Error"]]
    bad <- !is.na(se_vals) & !is.finite(se_vals)
    checks[[length(checks) + 1L]] <- .diagnostic_row(
      "standard_errors",
      if (any(bad)) "warning" else "ok",
      if (any(bad)) paste(sum(bad), "standard errors are non-finite") else "reported standard errors are finite or NA by design"
    )
  }
  checks[[length(checks) + 1L]] <- .diagnostic_row("WAIC", "skipped", "WAIC is not defined for Classic_Fit")
  .new_diagnose_result("Classic_Fit", do.call(rbind, checks), list(df_method = fit$df_method))
}

#' @export
print.diagnose_BayesRTMB <- function(x, ...) {
  cat("BayesRTMB diagnostics\n")
  cat("Fit:", x$fit_type, "\n")
  cat("Status:", x$status, "\n\n")
  checks <- x$checks
  if (!is.null(checks) && nrow(checks) > 0L) {
    for (i in seq_len(nrow(checks))) {
      cat(sprintf("[%s] %s: %s\n", checks$status[i], checks$check[i], checks$message[i]))
    }
  }
  invisible(x)
}

.inference_pipeline <- function(self, private, laplace, init, num_estimate, control,
                               optimizer, method, map, fixed, se, se_method,
                               num_samples, seed, marginal, df_method, view, ...) {

  se_method <- match.arg(se_method, c("wald", "sampling", "none"))
  se_enabled <- !identical(se_method, "none")
  se <- isTRUE(se) && se_enabled

  if (is.null(marginal)) marginal <- "none"
  target_vars <- private$.resolve_marginal_vars(marginal)
  marginal_flag <- length(target_vars) > 0L

  dot_args <- list(...)
  requested_contrasts <- dot_args$contrasts %||% self$contrasts
  if (!is.null(requested_contrasts)) {
    old_contrasts <- options()$contrasts
    on.exit(options(contrasts = old_contrasts), add = TRUE)
    if (requested_contrasts == "sum") options(contrasts = c("contr.sum", "contr.poly"))
    else if (requested_contrasts == "treatment") options(contrasts = c("contr.treatment", "contr.poly"))
    self$contrasts <- requested_contrasts
  }

  # Delegate to the corrected apply_constraints
  if (!is.null(fixed)) {
    new_model <- self$fixed_model(fixed)

    return(.inference_pipeline(
      self = new_model, private = new_model$.__enclos_env__$private,
      laplace = laplace, init = new_model$init, num_estimate = num_estimate, control = control,
      optimizer = optimizer, method = method, map = map, fixed = NULL,
      se = se, se_method = se_method, num_samples = num_samples, seed = seed,
      marginal = marginal, df_method = df_method, view = view, ...
    ))
  }

  if (se_method == "sampling") se_sampling <- TRUE else se_sampling <- FALSE

  # Resolve DF settings using helper
  df_settings <- .resolve_optimize_df_settings(df_method, marginal_flag, se_method, marginal)
  auto_df <- df_settings$auto_df
  show_df <- df_settings$show_df
  df_t <- df_settings$df_t
  df_method_l <- df_settings$df_method_norm

  # --- Warnings for ignored df_method ---
  if (!se_enabled) {
    explicit_df_requested <- is.numeric(df_method) || (is.character(df_method) && !(tolower(df_method) %in% c("auto", "inf", "infinite", "none")))
    if (explicit_df_requested) {
      warning("df_method is ignored when se_method = 'none'.", call. = FALSE)
    }
  } else if (!marginal_flag) {
    explicit_df_requested <- is.numeric(df_method) || (is.character(df_method) && !(tolower(df_method) %in% c("auto", "inf", "infinite", "none")))
    if (explicit_df_requested) {
      if (identical(marginal, "auto")) {
        warning("df_method is ignored because marginal = 'auto' resolved to no parameters. Set model$extra$marginal or specify marginal = c(...) explicitly.", call. = FALSE)
      } else if (identical(marginal, "none")) {
        warning("df_method is ignored when marginal = 'none' in optimize(). Use marginal to enable finite-df adjustment, or set df_method = 'none'.", call. = FALSE)
      }
    }
  }



  # --- Logic for using the common RTMB fitting engine ---
  raw <- private$.fit_rtmb(
    laplace = laplace, init = init, num_estimate = num_estimate, control = control,
    optimizer = optimizer, method = method, map = map, se = se,
    reml = marginal_flag, target_vars = target_vars,
    jacobian_target = NULL,  # Let .fit_rtmb decide after marginalization handling
    apply_prior_correction = TRUE
  )

  # Extract results and essential dimensions
  ad_setup       <- raw$ad_setup
  ad_obj         <- raw$ad_obj
  opt            <- raw$opt
  sd_rep         <- if (se_enabled) raw$sd_rep else NULL
  unc_est_vec    <- raw$par_unc
  L_u_total      <- raw$L_u_total
  unc_se_vec     <- if (se_enabled) raw$se_unc else rep(NA_real_, L_u_total)
  Cov_u          <- if (se_enabled) raw$vcov_unc else matrix(NA_real_, L_u_total, L_u_total)
  con_est_list   <- raw$par
  target_map     <- raw$map
  idx_fix_active <- raw$idx_fix_active
  idx_ran_full   <- raw$idx_ran_full
  idx_ran        <- raw$idx_ran
  factor_levels_seen <- raw$factor_levels_seen
  unc_est_list   <- raw$par_unc_list
  unc_se_list    <- raw$se_unc_list
  opt_history    <- raw$opt_history
  fallback_needed <- raw$fallback_needed

  con_se_list <- list(); con_lower_list <- list(); con_upper_list <- list(); samps_con <- list(); samps_tran <- list(); samps_gq <- list()

  dH_list <- NULL; V_opt <- NULL; active_idx <- NULL; dH_theta <- dH_beta <- V_theta <- V_beta <- NULL
  if (any(!is.infinite(df_t))) {
    est_dfs_all <- rep(df_t[1], L_u_total)
  } else if (auto_df) {
    # Perform Satterthwaite approximation
    satt_res <- self$calculate_satterthwaite_df(ad_obj, idx_fix_active, L_u_total, opt$par, max_df = 1e6, silent = FALSE, return_sensitivities = TRUE)
    if (is.list(satt_res)) {
      est_dfs_all <- satt_res$df
      dH_list <- satt_res$sensitivities
      V_opt <- satt_res$V
    } else {
      est_dfs_all <- satt_res
    }
    dH_theta <- dH_list
    V_theta <- V_opt

    if (marginal_flag && length(ad_obj$env$random) > 0) {
      full_par_names_mapped <- names(ad_obj$env$last.par)[idx_ran]
      target_ran_idx <- which(full_par_names_mapped %in% target_vars)
      if (length(target_ran_idx) > 0) {
        reml_res <- self$calculate_reml_satterthwaite_df(ad_obj, opt$par, target_ran_idx, silent = FALSE, return_sensitivities = TRUE)
        if (is.list(reml_res)) {
          active_idx <- idx_ran_full[target_ran_idx]
          est_dfs_all[active_idx] <- reml_res$df
          dH_beta <- reml_res$sensitivities
          V_beta <- reml_res$V_beta
        }
      }
    }
    ad_obj$fn(opt$par)
  } else {
    est_dfs_all <- rep(Inf, L_u_total)
  }

  if (se_sampling && se_enabled) {
    set.seed(seed)
    if (!getOption("BayesRTMB.silent", FALSE)) cat(sprintf("Using simulation-based error propagation (%d samples)...\n", num_samples))
    Cov_u_valid <- Cov_u[idx_fix_active, idx_fix_active, drop = FALSE]; mu_valid <- unc_est_vec[idx_fix_active]
    eig <- eigen(Cov_u_valid, symmetric = TRUE); eig$values <- pmax(eig$values, 1e-8); Cov_u_pd <- eig$vectors %*% diag(eig$values) %*% t(eig$vectors)
    raw_samples <- MASS::mvrnorm(num_samples, mu = mu_valid, Sigma = Cov_u_pd); active_dfs <- est_dfs_all[idx_fix_active]
    for (j in 1:ncol(raw_samples)) {
      df_j <- active_dfs[j]
      if (is.finite(df_j)) { sigma_j <- sqrt(Cov_u_pd[j, j]); z <- (raw_samples[, j] - mu_valid[j]) / (sigma_j + 1e-12); w <- rchisq(num_samples, df = df_j); raw_samples[, j] <- mu_valid[j] + z * sigma_j * sqrt(df_j / pmax(w, 1e-6)) }
    }
    u_samples_mat <- matrix(rep(unc_est_vec, each = num_samples), nrow = num_samples); u_samples_mat[, idx_fix_active] <- raw_samples
    idx_curr <- 1
    for (name in names(self$par_list)) {
      L_u <- self$par_list[[name]]$unc_length
      if (L_u > 0) {
        map_f <- if (!is.null(target_map[[name]])) target_map[[name]] else NULL
        for (i in 1:L_u) { pos_last <- idx_curr + i - 1; if (pos_last %in% idx_ran_full) next; if (!is.null(map_f) && !is.na(map_f[i])) { lvl <- as.character(map_f[i]); mapped_opt_idx <- factor_levels_seen[[lvl]]; if (!is.null(mapped_opt_idx)) u_samples_mat[, pos_last] <- raw_samples[, mapped_opt_idx] } }
        idx_curr <- idx_curr + L_u
      }
    }
    if (laplace && length(idx_ran_full) > 0) for (ridx in idx_ran_full) u_samples_mat[, ridx] <- rnorm(num_samples, mean = unc_est_vec[ridx], sd = unc_se_vec[ridx])

    local_data <- self$data; local_par_list <- self$par_list; local_transform <- self$transform; local_generate <- self$generate
    test_con_list <- self$to_constrained(unconstrained_vector_to_list(unc_est_vec, local_par_list, map = target_map))
    for (name in names(test_con_list)) samps_con[[name]] <- matrix(NA_real_, num_samples, length(as.numeric(test_con_list[[name]])))
    if (!is.null(local_transform)) {
      test_tran <- local_transform(local_data, test_con_list)
      if (!is.null(test_tran)) { for (name in names(test_tran)) samps_tran[[name]] <- matrix(NA_real_, num_samples, length(as.numeric(test_tran[[name]]))); test_con_list <- c(test_con_list, test_tran) }
    }
    if (!is.null(local_generate)) {
      test_gq <- local_generate(local_data, test_con_list)
      if (!is.null(test_gq)) for (name in names(test_gq)) samps_gq[[name]] <- matrix(NA_real_, num_samples, length(as.numeric(test_gq[[name]])))
    }

    process_sample <- function(s, u_vec) {
      tmp_con_list <- self$to_constrained(unconstrained_vector_to_list(u_vec, local_par_list))
      for (name in names(tmp_con_list)) samps_con[[name]][s, ] <<- as.numeric(tmp_con_list[[name]])
      if (!is.null(local_transform)) {
        tmp_tran <- local_transform(local_data, tmp_con_list); if (!is.null(tmp_tran)) { for (name in names(tmp_tran)) samps_tran[[name]][s, ] <<- as.numeric(tmp_tran[[name]]); tmp_con_list <- c(tmp_con_list, tmp_tran) }
      }
      if (!is.null(local_generate)) {
        tmp_gq <- local_generate(local_data, tmp_con_list); if (!is.null(tmp_gq)) for (name in names(tmp_gq)) samps_gq[[name]][s, ] <<- as.numeric(tmp_gq[[name]])
      }
    }
    for (s in 1:num_samples) process_sample(s, u_samples_mat[s, ])

    for (name in names(self$par_list)) {
      mat <- samps_con[[name]]; p_info <- self$par_list[[name]]
      con_se_list[[name]] <- if (is.null(mat)) rep(NA, p_info$length) else apply(mat, 2, sd, na.rm = TRUE)
      con_lower_list[[name]] <- if (is.null(mat)) rep(NA, p_info$length) else apply(mat, 2, quantile, 0.025, na.rm = TRUE)
      con_upper_list[[name]] <- if (is.null(mat)) rep(NA, p_info$length) else apply(mat, 2, quantile, 0.975, na.rm = TRUE)
      if (length(p_info$dim) > 1) { dim(con_se_list[[name]]) <- dim(con_lower_list[[name]]) <- dim(con_upper_list[[name]]) <- p_info$dim }
    }
  } else if (se_enabled) {
    u_idx_current <- 1
    for (name in names(self$par_list)) {
      p_info <- self$par_list[[name]]; u_val <- unc_est_list[[name]]; u_se <- unc_se_list[[name]]; c_val <- con_est_list[[name]]
      L_u <- p_info$unc_length; L_c <- p_info$length; u_indices <- if (L_u > 0) u_idx_current:(u_idx_current + L_u - 1) else integer(0); u_idx_current <- u_idx_current + L_u
      transform_single <- function(u) { tmp_list <- unc_est_list; tmp_list[[name]] <- u; as.numeric(self$to_constrained(tmp_list)[[name]]) }
      J <- matrix(0, L_c, L_u); for (i in seq_len(L_u)) { u_tmp <- u_val; u_tmp[i] <- u_tmp[i] + 1e-5; J[, i] <- (transform_single(u_tmp) - as.numeric(c_val)) / 1e-5 }
      c_se <- numeric(L_c); if (L_u > 0) { Cov_sub <- Cov_u[u_indices, u_indices, drop = FALSE]; for (j in 1:L_c) c_se[j] <- sqrt(sum((J[j, ] %*% Cov_sub) * J[j, ], na.rm = TRUE)) }
      con_se_list[[name]] <- if (length(p_info$dim) > 1) structure(c_se, dim = p_info$dim) else c_se
      u_dfs <- est_dfs_all[u_indices]; u_q_95 <- ifelse(is.na(u_dfs) | is.infinite(u_dfs), qnorm(0.975), qt(0.975, df = pmax(u_dfs, 2.1)))
      if (p_info$bounds == "corr_matrix" || p_info$type == "corr_matrix") {
        rho_val <- as.numeric(c_val); rho_clipped <- pmax(pmin(rho_val, 0.9999), -0.9999); z_val <- atanh(rho_clipped); z_se <- c_se / (1 - rho_clipped^2 + 1e-10); u_q_95_m <- if (any(is.na(u_dfs) | is.infinite(u_dfs))) qnorm(0.975) else qt(0.975, df = pmax(mean(u_dfs, na.rm = TRUE), 2.1))
        c_low <- tanh(z_val - u_q_95_m * z_se); c_up <- tanh(z_val + u_q_95_m * z_se); is_fixed <- c_se < 1e-10 & abs(rho_val - 1) < 1e-8; if (any(is_fixed)) { c_low[is_fixed] <- 1.0; c_up[is_fixed] <- 1.0 }
      } else {
        u_low <- u_val - u_q_95 * u_se; u_up <- u_val + u_q_95 * u_se; c_low_raw <- transform_single(u_low); c_up_raw <- transform_single(u_up); c_low <- pmin(c_low_raw, c_up_raw); c_up <- pmax(c_low_raw, c_up_raw)
      }
      con_lower_list[[name]] <- if (length(p_info$dim) > 1) structure(c_low, dim = p_info$dim) else c_low
      con_upper_list[[name]] <- if (length(p_info$dim) > 1) structure(c_up, dim = p_info$dim) else c_up
    }
  } else {
    # SE disabled: use point estimates and NA for uncertainty
    for (name in names(self$par_list)) {
      p_info <- self$par_list[[name]]
      L_c <- p_info$length
      con_se_list[[name]] <- rep(NA_real_, L_c)
      con_lower_list[[name]] <- rep(NA_real_, L_c)
      con_upper_list[[name]] <- rep(NA_real_, L_c)
      if (length(p_info$dim) > 1) {
        dim(con_se_list[[name]]) <- dim(con_lower_list[[name]]) <- dim(con_upper_list[[name]]) <- p_info$dim
      }
    }
  }


  df_list_con <- list(); df_list_unc <- unconstrained_vector_to_list(est_dfs_all, self$par_list)
  for (name in names(self$par_list)) {
    p <- self$par_list[[name]]; if (p$unc_length == p$length) df_list_con[[name]] <- df_list_unc[[name]]
    else { df_val <- rep(Inf, p$length); if (p$unc_length > 0) { m_df <- mean(df_list_unc[[name]], na.rm = TRUE); if (is.finite(m_df)) df_val[] <- m_df }; df_list_con[[name]] <- df_val }
  }

  .apply_df_map <- function(df, df_map) {
    if (is.null(df) || is.null(df_map) || length(df_map) == 0 || !("df" %in% names(df))) return(df)

    rn <- rownames(df)
    if (is.null(rn)) return(df)

    for (i in seq_along(rn)) {
      nm <- rn[i]
      base <- sub("\\[.*$", "", nm)

      if (nm %in% names(df_map)) {
        df$df[i] <- as.numeric(df_map[[nm]])
      } else if (base %in% names(df_map)) {
        df$df[i] <- as.numeric(df_map[[base]])
      }
    }
    df
  }

  build_summary_df <- function(target_random = FALSE) {
    names_vec <- c(); est_vec <- c(); se_vec <- c(); low_vec <- c(); up_vec <- c(); df_vec <- c()
    for (name in names(self$par_list)) {
      p_info <- self$par_list[[name]]; if (isTRUE(p_info$random) == target_random) {
        f_names <- generate_flat_names(name, p_info$dim, self$par_names[[name]]); names_vec <- c(names_vec, f_names); est_vec <- c(est_vec, as.numeric(con_est_list[[name]])); se_vec <- c(se_vec, as.numeric(con_se_list[[name]])); low_vec <- c(low_vec, as.numeric(con_lower_list[[name]])); up_vec <- c(up_vec, as.numeric(con_upper_list[[name]])); df_vec <- c(df_vec, if (!is.null(df_list_con[[name]])) as.numeric(df_list_con[[name]]) else rep(Inf, p_info$length))
      }
    }
    if (length(names_vec) == 0) return(NULL)
    res_df <- data.frame(Estimate = est_vec, `Std. Error` = se_vec, `Lower 95%` = low_vec, `Upper 95%` = up_vec, check.names = FALSE); if (show_df) res_df$df <- df_vec; rownames(res_df) <- names_vec; return(res_df)
  }
  df_fixed <- build_summary_df(FALSE); df_random <- if (laplace) build_summary_df(TRUE) else NULL

  df_map <- self$extra$df_map
  if (!is.null(df_map)) {
    df_fixed    <- .apply_df_map(df_fixed, df_map)
    df_random   <- .apply_df_map(df_random, df_map)
  }
  con_est_vec <- unlist(con_est_list, use.names = FALSE); tran_list <- if (!is.null(self$transform)) tryCatch(self$transform(self$data, con_est_list), error = function(e) NULL) else NULL
  gq_list <- if (!is.null(self$generate)) tryCatch(self$generate(self$data, if (!is.null(tran_list)) c(con_est_list, tran_list) else con_est_list), error = function(e) NULL) else NULL

  build_derived_summary <- function(func, base_out, is_generate = FALSE, dH_list_in = NULL, V_opt_in = NULL, is_reml_in = FALSE, active_idx_in = NULL, dH_theta_in = NULL, V_theta_in = NULL, theta_idx_in = NULL) {
    if (is.null(func) || is.null(base_out) || length(base_out) == 0) return(NULL)
    flat_base <- unlist(base_out, use.names = FALSE); L_out <- length(flat_base); names_vec <- c()
    for (name in names(base_out)) names_vec <- c(names_vec, generate_flat_names(name, if (is.null(dim(base_out[[name]]))) length(base_out[[name]]) else dim(base_out[[name]]), self$par_names[[name]]))
    
    # 1. Delta method Jacobian calculation (needed for both SE and DF if not sampling)
    J <- NULL
    if (se_enabled && (show_df || !se_sampling)) {
      J <- matrix(0, L_out, L_u_total)
      for (i in 1:L_u_total) {
        u_tmp <- unc_est_vec
        u_tmp[i] <- u_tmp[i] + 1e-5
        tmp_con <- self$to_constrained(unconstrained_vector_to_list(u_tmp, self$par_list))
        if (is_generate && !is.null(self$transform)) {
          ut <- tryCatch(self$transform(self$data, tmp_con), error = function(e) NULL)
          if (!is.null(ut)) tmp_con <- c(tmp_con, ut)
        }
        tmp_out <- tryCatch(func(self$data, tmp_con), error = function(e) NULL)
        if (!is.null(tmp_out)) J[, i] <- (unlist(tmp_out, use.names = FALSE) - flat_base) / 1e-5
      }
    }

    # 2. Degrees of Freedom
    derived_dfs <- rep(Inf, L_out)
    if (show_df && se_enabled && auto_df && !is.null(dH_theta_in) && !is.null(V_theta_in) && !is.null(J)) {
       # Use the new Satterthwaite delta method helper
       derived_dfs <- .satterthwaite_df_delta(
         J_full = J,
         V_theta = V_theta_in,
         dH_theta = dH_theta_in,
         idx_theta_full = theta_idx_in %||% idx_fix_active,
         V_beta = if (is_reml_in) V_opt_in else NULL,
         dH_beta = if (is_reml_in) dH_list_in else NULL,
         idx_beta_full = if (is_reml_in) active_idx_in else NULL,
         max_df = NULL
       )
    } else if (show_df && !is.infinite(df_t[1])) {
       derived_dfs <- rep(df_t[1], L_out)
    }

    # 3. Uncertainty (SE/CI)
    if (!se_enabled) {
      se_out <- low_out <- up_out <- rep(NA_real_, L_out)
    } else if (se_sampling) {
      # FIXED Sampling Branch: match samples to specific output names
      samps_list <- if (is_generate) samps_gq else samps_tran
      target_names <- names(base_out)
      samps_sub <- samps_list[target_names]
      samps_sub <- samps_sub[!vapply(samps_sub, is.null, logical(1))]
      
      if (length(samps_sub) == 0L) {
        se_out <- low_out <- up_out <- rep(NA_real_, L_out)
      } else {
        mat_all <- do.call(cbind, unname(samps_sub))
        if (ncol(mat_all) != L_out) {
          se_out <- low_out <- up_out <- rep(NA_real_, L_out)
        } else {
          se_out <- apply(mat_all, 2, stats::sd, na.rm = TRUE)
          low_out <- apply(mat_all, 2, stats::quantile, 0.025, na.rm = TRUE)
          up_out <- apply(mat_all, 2, stats::quantile, 0.975, na.rm = TRUE)
        }
      }
    } else {
      # delta method SE
      se_out <- rep(NA_real_, L_out)
      smry_rep <- if (!is.null(sd_rep)) {
        tryCatch(
          suppressWarnings(as.data.frame(summary(sd_rep, select = "report"))),
          error = function(e) NULL
        )
      } else {
        NULL
      }
      if (!is.null(smry_rep)) {
        m_idx <- match(names_vec, rownames(smry_rep))
        valid <- !is.na(m_idx)
        if (any(valid)) se_out[valid] <- smry_rep$`Std. Error`[m_idx[valid]]
      }
      for (j in which(is.na(se_out))) {
        se_out[j] <- sqrt(abs(sum((J[j, ] %*% Cov_u) * J[j, ])))
      }
      
      crit <- ifelse(is.finite(derived_dfs), 
                     qt(0.975, df = pmax(derived_dfs, 2.1)), 
                     qnorm(0.975))
      low_out <- flat_base - crit * se_out
      up_out <- flat_base + crit * se_out
      
      corr_pat <- "^(corr|pcorr|B_corr|W_corr)(\\[|$)"
      is_corr <- grepl(corr_pat, names_vec)
      for (j in which(is_corr)) {
        r <- pmax(pmin(flat_base[j], 0.9999), -0.9999)
        z <- atanh(r)
        se_z <- se_out[j] / (1 - r^2 + 1e-10)
        
        cj <- crit[j]
        low_out[j] <- tanh(z - cj * se_z)
        up_out[j] <- tanh(z + cj * se_z)
      }
    }
    
    res_df <- data.frame(Estimate = flat_base, `Std. Error` = se_out, `Lower 95%` = low_out, `Upper 95%` = up_out, check.names = FALSE)
    if (show_df) {
      res_df$df <- derived_dfs
    }
    rownames(res_df) <- names_vec
    return(res_df)
  }

  df_transform <- build_derived_summary(self$transform, tran_list, FALSE, if (marginal_flag) dH_beta else dH_list, if (marginal_flag) V_beta else V_opt, marginal_flag, if (marginal_flag) active_idx else NULL, if (marginal_flag) dH_theta else NULL, if (marginal_flag) V_theta else NULL, if (marginal_flag) idx_fix_active else NULL)
  if (!is.null(df_map)) df_transform <- .apply_df_map(df_transform, df_map)


  # Execute calculation if a user-defined generate block exists
  df_generate <- build_derived_summary(self$generate, gq_list, TRUE, if (marginal_flag) dH_beta else dH_list, if (marginal_flag) V_beta else V_opt, marginal_flag, if (marginal_flag) active_idx else NULL, if (marginal_flag) dH_theta else NULL, if (marginal_flag) V_theta else NULL, if (marginal_flag) idx_fix_active else NULL)
  if (!is.null(df_map)) df_generate <- .apply_df_map(df_generate, df_map)

  log_ml <- NA
  if (!is.null(sd_rep) && !is.null(sd_rep$cov.fixed) && !fallback_needed) {
    eig <- tryCatch(eigen(sd_rep$cov.fixed, symmetric = TRUE), error = function(e) NULL)
    if (!is.null(eig) && all(eig$values > 1e-8)) {
      lj_missing <- if (laplace) calc_log_jacobian(unc_est_list, self$par_list, FALSE) - calc_log_jacobian(unc_est_list, self$par_list, TRUE) else calc_log_jacobian(unc_est_list, self$par_list, FALSE)
      log_ml <- -opt$objective + lj_missing + (length(opt$par) / 2) * log(2 * pi) + 0.5 * sum(log(eig$values)) - self$prior_correction
    }
  }



  is_se_sampling <- identical(se_method, "sampling")

  res_obj <- MAP_Fit$new(
    model = self, par_vec = con_est_vec, par = con_est_list,
    objective = opt$objective, log_ml = log_ml, convergence = opt$convergence,
    sd_rep = sd_rep, df_fixed = df_fixed, random_effects = df_random,
    df_transform = df_transform, df_generate = df_generate,
    opt_history = opt_history, transform = tran_list, generate = gq_list,
    se_samples = if (is_se_sampling && se_enabled) list(con = samps_con, tran = samps_tran, gq = samps_gq) else NULL,
    par_unc = unc_est_vec, ci_method = se_method, laplace = laplace,
    vcov_unc = Cov_u,
    map = target_map,
    marginal_vars = target_vars,
    laplace_random_vars = ad_setup$use_random,
    idx_fix_active = idx_fix_active, show_df = show_df,
    view = view,
    fallback_needed = fallback_needed
  )
  return(res_obj)
}

.resolve_optimize_df_settings <- function(df_method, marginal_flag, se_method, marginal) {
  if (is.null(df_method)) df_method <- "auto"
  df_method_l <- if (is.character(df_method)) tolower(df_method) else "numeric"

  if (identical(df_method_l, "bw")) {
    stop(
      "`df_method = 'bw'` is not supported in optimize(). ",
      "Use `df_method = 'satterthwaite'`, 'auto', 'inf', or 'none'.",
      call. = FALSE
    )
  }

  auto_df <- FALSE
  show_df <- FALSE
  df_t <- Inf

  if (identical(se_method, "none")) {
    # se_method = "none" always disables finite-df
  } else if (!marginal_flag) {
    # No marginalization means no finite-df by default in MAP
  } else if (is.numeric(df_method)) {
    df_t <- as.numeric(df_method)
    show_df <- TRUE
  } else if (df_method_l %in% c("auto", "satterthwaite")) {
    auto_df <- TRUE
    show_df <- TRUE
  } else if (df_method_l %in% c("inf", "infinite", "none")) {
    # Keep defaults (Inf, FALSE)
  } else {
    stop("Invalid df_method.", call. = FALSE)
  }

  return(list(
    auto_df = auto_df,
    show_df = show_df,
    df_t = df_t,
    df_method_norm = if (is.numeric(df_method)) "numeric" else df_method_l
  ))
}

.satterthwaite_df_delta <- function(J_full, V_theta, dH_theta, idx_theta_full, V_beta = NULL, dH_beta = NULL, idx_beta_full = NULL, max_df = NULL) {
  L_out <- nrow(J_full)
  df_out <- rep(Inf, L_out)
  rel_tol <- 1e-10

  # Optimized block (theta)
  j_theta <- J_full[, idx_theta_full, drop = FALSE]
  P_theta <- length(idx_theta_full)
  
  # Optional marginalized block (beta)
  has_beta <- !is.null(V_beta) && is.matrix(V_beta) && 
              !is.null(dH_beta) && is.list(dH_beta) &&
              !is.null(idx_beta_full) && length(idx_beta_full) > 0L

  if (has_beta) {
    j_beta <- J_full[, idx_beta_full, drop = FALSE]
    P_beta <- length(idx_beta_full)
  }

  for (i in 1:L_out) {
    ji_t <- j_theta[i, , drop = FALSE]
    v_i <- as.numeric(ji_t %*% V_theta %*% t(ji_t))
    
    grad_v <- rep(0, P_theta)
    for (k in 1:P_theta) {
      dh <- dH_theta[[k]]
      
      # Guard against NULL or dimension mismatch in numerical sensitivities
      if (is.null(dh)) next
      if (!is.matrix(dh)) next
      if (nrow(dh) != nrow(V_theta) || ncol(dh) != ncol(V_theta)) next

      # dV_theta/dtheta_k = -V_theta %*% dh %*% V_theta
      dV_k <- - V_theta %*% dh %*% V_theta
      grad_v[k] <- as.numeric(ji_t %*% dV_k %*% t(ji_t))
    }
    
    if (has_beta) {
      ji_b <- j_beta[i, , drop = FALSE]
      v_i <- v_i + as.numeric(ji_b %*% V_beta %*% t(ji_b))
      
      for (k in 1:P_theta) {
         # dV_beta/dtheta_k is directly provided in dH_beta
         dV_b_k <- dH_beta[[k]]
         
         if (is.null(dV_b_k)) next
         if (!is.matrix(dV_b_k)) next
         if (nrow(dV_b_k) != nrow(V_beta) || ncol(dV_b_k) != ncol(V_beta)) next

         grad_v[k] <- grad_v[k] + as.numeric(ji_b %*% dV_b_k %*% t(ji_b))
      }
    }

    if (all(abs(grad_v) < 1e-30)) next
    
    var_vi <- as.numeric(t(grad_v) %*% V_theta %*% grad_v)
    
    # Numerical stability: If variance of v_i is extremely small relative to v_i^2, 
    # it means sensitivity is negligible, so DF should be Infinite.
    if (!is.na(var_vi) && is.finite(var_vi) && var_vi > rel_tol * (v_i^2) && v_i > 0) {
      df_out[i] <- 2 * (v_i^2) / var_vi
    }
  }

  if (!is.null(max_df)) df_out[df_out > max_df] <- Inf
  df_out[!is.finite(df_out)] <- Inf
  df_out <- pmax(df_out, 2.1)
  
  return(df_out)
}

#' MAP fit object
#'
#' An R6 class storing optimization results from maximum a posteriori
#' (MAP) estimation.
#'
#' @field model The `RTMB_Model` object used for estimation.
#' @field par_vec Parameter vector on the unconstrained scale (constrained values unlisted).
#' @field par Parameter list on the constrained scale.
#' @field par_unc Parameter vector on the unconstrained scale (raw unconstrained values).
#' @field ci_method Method used for CI estimation ("wald", "sampling", or "none").
#' @field objective RTMB objective function object.
#' @field log_ml Log marginal likelihood or related model criterion.
#' @field convergence Optimizer convergence code.
#' @field sd_rep Standard deviation report object.
#' @field df_fixed Summary table for fixed-effect parameters.
#' @field random_effects Random effect estimates.
#' @field df_transform Summary table for transformed parameter estimates.
#' @field df_generate Summary table for generated quantity estimates.
#' @field opt_history A vector of optimize objective history.
#' @field transform List of transformed parameters maintaining their original dimensions.
#' @field generate List of generated quantities maintaining their original dimensions.
#' @field se_samples List of simulated samples for standard error estimation.
#' @field laplace Logical; whether Laplace approximation was used.
#' @field vcov_unc Variance-covariance matrix of parameters in unconstrained space.
#' @field map List; the parameter mapping used.
#' @field marginal_vars Character vector of parameter names requested through `optimize(marginal = ...)`.
#' @field laplace_random_vars Character vector of all parameter names passed to `MakeADFun(random = ...)` during Laplace approximation.
#' @field idx_fix_active Numeric vector; mapping between active parameters and full unconstrained vector.
#' @field show_df Logical; whether to display degrees of freedom in the summary output.
#' @field view Character vector of parameter names to prioritize in summary.
#' @field fallback_needed Logical; whether Hessian/SE fallback was used during optimization.
#'
#' @export
MAP_Fit <- R6::R6Class(
  classname = "map_fit",
  inherit = RTMB_Fit_Base,

  public = list(
    # --- Fields ---
    model          = NULL,
    par_vec        = NULL,
    par            = NULL,
    objective      = NULL,
    log_ml         = NULL,
    convergence    = NULL,
    sd_rep         = NULL,
    df_fixed       = NULL,
    random_effects = NULL,
    df_transform   = NULL,
    df_generate    = NULL,
    opt_history    = NULL,
    transform      = NULL,
    generate       = NULL,
    se_samples     = NULL,
    par_unc        = NULL,
    ci_method      = NULL,
    laplace        = NULL,
    map            = NULL,
    vcov_unc       = NULL,
    marginal_vars  = NULL,
    laplace_random_vars = NULL,
    idx_fix_active = NULL,
    show_df        = TRUE,
    view           = NULL,
    fallback_needed = NULL,

    #' @description Get point estimate for a target parameter.
    #' @param target Target parameter name.
    #' @param ... Additional arguments, ignored for MAP fits.
    #' @return Matrix, array, vector, or scalar point estimate.
    get_point_estimate = function(target, ...) {
      if (!is.null(self$par[[target]])) return(self$par[[target]])
      if (!is.null(self$transform[[target]])) return(self$transform[[target]])
      if (!is.null(self$generate[[target]])) return(self$generate[[target]])
      stop("Parameter not found: ", target)
    },



    #' @description Create a new `MAP_Fit` object.
    #' @param model The `RTMB_Model` object used for estimation.
    #' @param par_vec Parameter vector on the unconstrained scale (constrained values unlisted).
    #' @param par Parameter list on the constrained scale.
    #' @param objective The objective function value at the optimum.
    #' @param log_ml Log marginal likelihood.
    #' @param convergence Optimizer convergence code.
    #' @param sd_rep The `sdreport` object from TMB.
    #' @param df_fixed Data frame of fixed effects estimates and CIs.
    #' @param random_effects Data frame of random effects estimates and CIs.
    #' @param df_transform Data frame of transformed parameters.
    #' @param df_generate Data frame of generated quantities.
    #' @param opt_history Data frame of optimization history.
    #' @param transform List of transformed parameters maintaining their original dimensions.
    #' @param generate List of generated quantities maintaining their original dimensions.
    #' @param se_samples List of simulated samples for standard error estimation.
    #' @param par_unc Parameter vector on the unconstrained scale (raw values).
    #' @param ci_method Method used for CI estimation ("wald", "sampling", or "none").
    #' @param laplace Logical; whether Laplace approximation was used.
    #' @param vcov_unc Variance-covariance matrix of parameters in unconstrained space.
    #' @param map List; the parameter mapping used.
    #' @param marginal_vars Character vector of parameter names requested through `optimize(marginal = ...)`.
    #' @param laplace_random_vars Character vector of all parameter names passed to `MakeADFun(random = ...)` during Laplace approximation.
    #' @param idx_fix_active Numeric vector; mapping between active parameters and full unconstrained vector.
    #' @param show_df Logical; whether to display degrees of freedom in the summary output. Default is TRUE.
    #' @param view Character vector of parameter names to prioritize in summary.
    #' @param fallback_needed Logical; whether Hessian/SE fallback was used during optimization.
    initialize = function(model, par_vec = NULL, par = NULL, objective = NULL, log_ml = NULL,
                          convergence = NULL, sd_rep = NULL, df_fixed = NULL, random_effects = NULL,
                          df_transform = NULL, df_generate = NULL, opt_history = NULL,
                          transform = NULL, generate = NULL, se_samples = NULL, par_unc = NULL,
                          ci_method = "wald", laplace = TRUE, map = NULL, vcov_unc = NULL,
                          marginal_vars = NULL, laplace_random_vars = NULL, idx_fix_active = NULL,
                          show_df = TRUE, view = NULL, fallback_needed = NULL) {
      self$model <- model
      self$par_vec <- par_vec
      self$par <- par
      self$objective <- objective
      self$log_ml <- log_ml
      self$convergence <- convergence
      self$sd_rep <- sd_rep
      self$df_fixed <- df_fixed
      self$random_effects <- random_effects
      self$df_transform <- df_transform
      self$df_generate <- df_generate
      self$opt_history <- opt_history
      self$transform <- transform
      self$generate <- generate
      self$se_samples <- se_samples
      self$par_unc <- par_unc
      self$ci_method <- ci_method
      self$vcov_unc <- vcov_unc
      self$laplace <- laplace
      self$map <- map
      self$marginal_vars <- marginal_vars
      self$laplace_random_vars <- laplace_random_vars
      self$idx_fix_active <- idx_fix_active
      self$show_df <- show_df
      self$view <- view
      self$fallback_needed <- fallback_needed

      if (!is.null(self$par_vec)) self$par_vec <- Re(self$par_vec)
      if (!is.null(self$par)) self$par <- lapply(self$par, Re)
      if (!is.null(self$random_effects) && !is.data.frame(self$random_effects)) {
        self$random_effects <- lapply(self$random_effects, Re)
      }
      if (!is.null(self$transform)) self$transform <- lapply(self$transform, Re)
      if (!is.null(self$generate)) self$generate <- lapply(self$generate, Re)
    },

    #' @description Return random effect estimates as a named list.
    #' @return A named list of random effect estimates.
    ranef = function() {
      if (is.null(self$random_effects)) {
        message("No random effects found in this model.")
        return(NULL)
      }
      
      # Extract only the parameters that are marked as random
      random_names <- names(self$model$par_list)[sapply(self$model$par_list, function(x) isTRUE(x$random))]
      return(self$par[random_names])
    },

    #' @description Extract samples from the asymptotic posterior distribution.
    #' @param pars Character or numeric vector specifying the names or indices of parameters to extract.
    #' @param inc_random Logical; whether to include random effects.
    #' @param inc_transform Logical; whether to include transformed parameters.
    #' @param inc_generate Logical; whether to include generated quantities.
    #' @param ... Ignored.
    #' @return An array of samples [iterations, 1, parameters].
    draws = function(pars = NULL, inc_random = FALSE, inc_transform = TRUE, inc_generate = TRUE, ...) {
      # 1. Determine number of samples
      num_samples <- 1
      if (!is.null(self$se_samples)) {
        num_samples <- nrow(self$se_samples$con[[1]])
      }

      # 2. Identify candidate base variables
      all_base_vars <- names(self$model$par_list)
      if (!inc_random) {
        random_flags <- sapply(self$model$par_list, function(x) isTRUE(x$random))
        all_base_vars <- all_base_vars[!random_flags]
      }
      
      if (inc_transform && !is.null(self$transform)) all_base_vars <- c(all_base_vars, names(self$transform))
      if (inc_generate && !is.null(self$generate)) all_base_vars <- c(all_base_vars, names(self$generate))

      # 3. Narrow down base variables if pars is provided as character
      target_base_vars <- all_base_vars
      if (!is.null(pars) && is.character(pars)) {
        pars_base <- unique(gsub("\\[.*\\]$", "", pars))
        # We need to include base variables that are directly in pars_base,
        # but also we don't know if a flat name matches until we expand.
        # To be safe and efficient, we take the union of all_base_vars and pars_base.
        target_base_vars <- intersect(all_base_vars, pars_base)
        if (length(target_base_vars) == 0) target_base_vars <- all_base_vars
      }

      # 4. Collect and flatten samples for target base variables
      flat_list <- list()
      flat_names_vec <- c()

      for (v in target_base_vars) {
        # Determine group
        group <- if (v %in% names(self$model$par_list)) "con" 
                 else if (v %in% names(self$transform)) "tran"
                 else if (v %in% names(self$generate)) "gq"
                 else NULL
        
        if (is.null(group)) next

        # Get samples matrix [num_samples, length(v)]
        samps <- if (!is.null(self$se_samples[[group]][[v]])) {
          self$se_samples[[group]][[v]]
        } else {
          val <- if (group == "con") self$par[[v]] else self[[if(group=="gq")"generate" else "transform"]][[v]]
          matrix(rep(as.numeric(val), each = num_samples), nrow = num_samples)
        }

        flat_list[[v]] <- samps
        
        # Get flat names
        v_info <- self$model$par_list[[v]]
        v_dim <- if (!is.null(v_info)) v_info$dim else dim(self[[if(group=="gq")"generate" else "transform"]][[v]])
        if (is.null(v_dim)) v_dim <- length(as.numeric(self[[if(group=="gq")"generate" else "transform"]][[v]]))
        v_names_def <- self$model$par_names[[v]]
        flat_names_vec <- c(flat_names_vec, generate_flat_names(v, v_dim, v_names_def))
      }

      if (length(flat_list) == 0) stop("No matching parameters found.")

      res_mat <- do.call(cbind, unname(flat_list))
      colnames(res_mat) <- flat_names_vec

      # 5. Final filtering by pars (handles both base names and flattened names)
      if (!is.null(pars)) {
        if (is.numeric(pars)) {
          target_idx <- pars[pars >= 1 & pars <= ncol(res_mat)]
        } else if (is.character(pars)) {
          base_names <- gsub("\\[.*\\]$", "", flat_names_vec)
          target_idx <- integer(0)
          for (p in pars) {
            match_idx <- which(flat_names_vec == p | base_names == p)
            target_idx <- c(target_idx, match_idx)
          }
          target_idx <- unique(target_idx)
        } else {
          stop("'pars' must be numeric or character.")
        }
        
        if (length(target_idx) == 0) stop("No matching parameters found in 'pars'.")
        res_mat <- res_mat[, target_idx, drop = FALSE]
        flat_names_vec <- flat_names_vec[target_idx]
      }

      # 6. Convert to array [iterations, chains=1, parameters]
      res_array <- array(res_mat, dim = c(num_samples, 1, ncol(res_mat)))
      dimnames(res_array) <- list(iteration = 1:num_samples, chain = 1, variable = flat_names_vec)

      return(res_array)
    },

    #' @description Summarize MAP estimates.
    #' @param pars Character vector specifying the names of parameters to summarize. If NULL, all available parameters are summarized.
    #' @param max_rows Maximum number of rows to print in summaries. Default is 10.
    #' @param digits Number of digits to print.
    #' @param ranef Logical; whether to also display random effect estimates. Default is FALSE.
    #' @param view Character vector of parameter names to prioritize or filter by.
    #' @return A summary object, typically a data frame.
    summary = function(pars = NULL, max_rows = 10, digits = 5, ranef = FALSE, view = NULL) {
      # Defensive check for pars
      vars <- if (is.null(pars)) names(self$par) else pars
      if (is.numeric(vars) && !is.null(self$par)) vars <- names(self$par)[vars]
      vars <- as.character(vars) # Fix for environment subsetting error

      cat("\nCall:\nMAP Estimation via RTMB\n")
      cat(sprintf("\nNegative Log-Posterior: %.2f\n", self$objective))

      if (!is.null(self$log_ml) && !is.na(self$log_ml)) {
        cat(sprintf("Approx. Log Marginal Likelihood (Laplace): %.2f\n", self$log_ml))
      } else {
        cat("Approx. Log Marginal Likelihood (Laplace): NA\n")
      }

      if (!is.null(self$random_effects) && !isTRUE(ranef)) {
        cat("Note: Random effects are stored in $random_effects (use ranef = TRUE to show them)\n")
      }

      target_views <- c()
      if (!is.null(view)) {
        target_views <- c(target_views, view)
      } else if (!is.null(self$view)) {
        target_views <- c(target_views, self$view)
      } else if (!is.null(self$model$view)) {
        target_views <- c(target_views, self$model$view)
      }

      view_matches_ranef <- FALSE
      if (length(target_views) > 0 && is.data.frame(self$random_effects) && nrow(self$random_effects) > 0) {
        re_names <- rownames(self$random_effects)
        re_base_names <- gsub("\\[.*\\]$", "", re_names)
        for (v in target_views) {
          if (any(re_names == v | re_base_names == v)) {
            view_matches_ranef <- TRUE
            break
          }
        }
      }

      all_dfs <- list()
      if (!is.null(self$df_fixed) && nrow(self$df_fixed) > 0) all_dfs$fixed <- self$df_fixed
      
      # Automatically include random effects if requested by pars, ranef=TRUE, or view.
      include_ranef <- isTRUE(ranef) || !is.null(pars) || view_matches_ranef
      if (include_ranef && is.data.frame(self$random_effects) && nrow(self$random_effects) > 0) {
        all_dfs$random <- self$random_effects
      }
      if (!is.null(self$df_transform) && nrow(self$df_transform) > 0) all_dfs$transform <- self$df_transform
      if (!is.null(self$df_generate) && nrow(self$df_generate) > 0) all_dfs$generate <- self$df_generate

      if (length(all_dfs) == 0) {
        cat("\nNo parameters to display.\n")
        return(invisible(NULL))
      }

      df_combined <- do.call(rbind, unname(all_dfs))

      if (!is.null(pars)) {
        var_names <- rownames(df_combined)
        base_names <- gsub("\\[.*\\]$", "", var_names)
        
        ordered_idx <- integer(0)
        for (p in vars) {
          match_idx <- which(var_names == p | base_names == p)
          ordered_idx <- c(ordered_idx, match_idx)
        }
        df_combined <- df_combined[unique(ordered_idx), , drop = FALSE]
      }

      if (nrow(df_combined) == 0) {
        cat("\nNo matching parameters found.\n")
        return(invisible(df_combined))
      }

      if (nrow(df_combined) > 0 && is.null(pars)) {
        var_names <- rownames(df_combined)
        base_names <- gsub("\\[.*\\]$", "", var_names)

        priority_idx <- integer(0)
        for (v in target_views) {
          match_idx <- which(var_names == v | base_names == v)
          priority_idx <- c(priority_idx, match_idx)
        }
        priority_idx <- unique(priority_idx)
        other_idx <- setdiff(seq_along(var_names), priority_idx)
        df_combined <- df_combined[c(priority_idx, other_idx), , drop = FALSE]
      }

      ci_label <- "95% Wald CI"
      if (!is.null(self$ci_method)) {
        if (self$ci_method == "sampling") ci_label <- "95% Sampling-based CI"
      }
      cat(sprintf("\nPoint Estimates and %s:\n", ci_label))

      num_cols <- sapply(df_combined, is.numeric)
      df_combined[num_cols] <- lapply(df_combined[num_cols], function(x) {
        x[abs(x) < 1e-12 & !is.na(x)] <- 0
        return(x)
      })

      # Display all estimated quantities including SE and CI (excluding only metadata if needed)
      df_display <- df_combined
      
      # Ensure consistent column ordering for readability (support both DF and df)
      # Order: Estimate, Std. Error, [CIs], DF/df, [Other]
      desired_order <- c("Estimate", "Std. Error", "Lower 95%", "Upper 95%", "Lower 2.5%", "Upper 97.5%", "df")
      current_names <- colnames(df_display)
      order_to_use <- intersect(desired_order, current_names)
      other_names <- setdiff(current_names, desired_order)
      df_display <- df_display[, c(order_to_use, other_names), drop = FALSE]

      out_df <- data.frame(variable = rownames(df_display), df_display, check.names = FALSE, stringsAsFactors = FALSE)
      
      # --- Hide DF column if requested ---
      if (!isTRUE(self$show_df) && "df" %in% names(out_df)) {
         out_df$df <- NULL
      }
      
      rownames(out_df) <- NULL

      if (!is.null(max_rows) && nrow(out_df) > max_rows) {
        out_df <- head(out_df, max_rows)
      }

      class(out_df) <- c("summary_BayesRTMB", "data.frame")
      print(out_df, digits = digits)

      cat("\n")
      invisible(df_combined)
    },


    #' @description Print a brief summary of the fitted object.
    #' @param pars Character vector specifying the names of parameters to summarize.
    #' @param max_rows Maximum number of rows to print in summaries.
    #' @param digits Number of digits to print.
    #' @param ... Additional arguments passed to the `summary` method.
    #' @return The object itself, invisibly.
    print = function(pars = NULL, max_rows = 10, digits = 5, ...) {
      self$summary(pars = pars, max_rows = max_rows, digits = digits, ...)
      invisible(self)
    },

    #' @description Compute generated quantities from the MAP estimate.
    #' @param code An `rtmb_code(\{ ... \})` or `\{ ... \}` block containing the logic to be calculated using the MAP estimate.
    #' @return The `MAP_Fit` object itself (invisibly). Results are added or updated in the `generate` list and `df_generate`.
    generated_quantities = function(code) {
      raw_code <- substitute(code)
      if (is.name(raw_code)) {
        evaluated <- tryCatch(eval(raw_code, envir = parent.frame()), error = function(e) NULL)
        if (is.language(evaluated) || is.call(evaluated)) code <- evaluated
      }

      gen_ast <- if (is.call(code) && identical(code[[1]], as.name("rtmb_code"))) code$generate else code

      gen_fn <- eval(bquote(transform_code(.(gen_ast))))
      environment(gen_fn) <- parent.env(globalenv())

      p_list <- self$par
      if (!is.null(self$transform)) p_list <- c(p_list, self$transform)
      if (!is.null(self$generate)) p_list <- c(p_list, self$generate)

      res <- gen_fn(self$model$data, p_list)

      if (is.null(self$generate)) self$generate <- list()
      for (n in names(res)) {
        self$generate[[n]] <- res[[n]]
      }

      flat_base <- unlist(res, use.names = FALSE)
      names_vec <- c()
      for (name in names(res)) {
        val <- res[[name]]
        dim_val <- dim(val)
        if (is.null(dim_val)) dim_val <- length(val)
        names_def <- self$model$par_names[[name]]
        names_vec <- c(names_vec, generate_flat_names(name, dim_val, names_def))
      }

      se_out <- rep(NA, length(flat_base))
      low_out <- rep(NA, length(flat_base))
      up_out <- rep(NA, length(flat_base))

      # Apply se_sampling if available
      if (!is.null(self$se_samples)) {
        num_samples <- nrow(self$se_samples$con[[1]])
        samps_res <- list()

        for (s in 1:num_samples) {
          tmp_p_list <- list()
          for (name in names(self$se_samples$con)) {
            val <- self$se_samples$con[[name]][s, ]
            if (length(dim(self$par[[name]])) > 1) dim(val) <- dim(self$par[[name]])
            tmp_p_list[[name]] <- val
          }
          if (!is.null(self$se_samples$tran)) {
            for (name in names(self$se_samples$tran)) {
              val <- self$se_samples$tran[[name]][s, ]
              if (length(dim(self$transform[[name]])) > 1) dim(val) <- dim(self$transform[[name]])
              tmp_p_list[[name]] <- val
            }
          }
          if (!is.null(self$se_samples$gq)) {
            for (name in names(self$se_samples$gq)) {
              val <- self$se_samples$gq[[name]][s, ]
              if (length(dim(self$generate[[name]])) > 1) dim(val) <- dim(self$generate[[name]])
              tmp_p_list[[name]] <- val
            }
          }

          tmp_res <- tryCatch(gen_fn(self$model$data, tmp_p_list), error = function(e) NULL)
          if (!is.null(tmp_res)) {
            for (name in names(tmp_res)) {
              val <- as.numeric(tmp_res[[name]])
              if (is.null(samps_res[[name]])) samps_res[[name]] <- matrix(NA, num_samples, length(val))
              samps_res[[name]][s, ] <- val
            }
          }
        }

        if (length(samps_res) > 0) {
          if (is.null(self$se_samples$gq)) self$se_samples$gq <- list()
          for (name in names(samps_res)) {
            self$se_samples$gq[[name]] <- samps_res[[name]]
          }

          mat_list <- list()
          for (name in names(res)) {
            if (!is.null(samps_res[[name]])) {
              mat_list[[name]] <- samps_res[[name]]
            } else {
              mat_list[[name]] <- matrix(NA, num_samples, length(res[[name]]))
            }
          }
          mat_all <- do.call(cbind, unname(mat_list))

          se_out <- apply(mat_all, 2, sd, na.rm = TRUE)
          low_out <- apply(mat_all, 2, quantile, probs = 0.025, na.rm = TRUE)
          up_out <- apply(mat_all, 2, quantile, probs = 0.975, na.rm = TRUE)
        }
      }

      new_df <- data.frame(
        Estimate     = flat_base,
        `Std. Error` = se_out,
        `Lower 95%`  = low_out,
        `Upper 95%`  = up_out,
        row.names    = names_vec,
        check.names  = FALSE
      )

      if (is.null(self$df_generate)) {
        self$df_generate <- new_df
      } else {
        overlap <- intersect(rownames(self$df_generate), rownames(new_df))
        if (length(overlap) > 0) {
          self$df_generate[overlap, ] <- new_df[overlap, ]
          new_df <- new_df[!(rownames(new_df) %in% overlap), , drop = FALSE]
        }
        if (nrow(new_df) > 0) {
          self$df_generate <- rbind(self$df_generate, new_df)
        }
      }

      cat("Generated quantities updated.\n")
      invisible(self)
    },

    #' @description Compute an approximate WAIC from sampling-based uncertainty propagation.
    #' @return A `waic_BayesRTMB` object.
    WAIC = function() {
      if (is.null(self$se_samples) || is.null(self$se_samples$gq) ||
          is.null(self$se_samples$gq$log_lik)) {
        stop(
          "WAIC for MAP_Fit requires pointwise `log_lik` samples. ",
          "Use optimize(se_method = 'sampling') with WAIC = TRUE, or use sample()/variational().",
          call. = FALSE
        )
      }
      .compute_waic_from_log_lik(self$se_samples$gq$log_lik)
    },

    #' @description Run basic diagnostics for the MAP fit.
    #' @param ... Additional arguments passed to `diagnose_map_fit()`.
    #' @return A `diagnose_BayesRTMB` object.
    diagnose = function(...) {
      diagnose_map_fit(self, ...)
    },

    #' @description Calculate Profile Likelihood confidence intervals for specific parameters.
    #' @param pars Character vector of parameter names to profile. If NULL, all fixed parameters are profiled.
    #' @param level Confidence level (default is 0.95).
    #' @param trace Logical; whether to print profiling progress. Default is FALSE.
    #' @param digits Integer; number of decimal places to print. Default is 5.
    #' @param show_plot Logical; whether to plot the profile likelihood curves. Default is FALSE.
    #' @param quiet Logical; whether to suppress text output. Default is FALSE.
    #' @param jacobian Character; "none" (default), "random", or "all". Whether to include Jacobian adjustments for transformations.
    #' @param ... Additional arguments passed to TMB::tmbprofile (e.g., ytol).
    #' @return A data frame containing the profile-based confidence intervals, with the raw profile objects stored in the "profiles" attribute.
    profile = function(pars = NULL, level = 0.95, trace = FALSE, digits = 5, show_plot = FALSE, quiet = FALSE, jacobian = "none", ...) {
      if (!quiet) cat("Estimating confidence intervals via Profile Likelihood...\n")
      
      # 1. Re-build ad_obj using optimized values and specific jacobian adjustment
      # Use stored marginal_vars to ensure marginal likelihood is profiled if applicable
      if (jacobian == "none" && isTRUE(self$laplace)) jacobian <- "random"
      
      # Check if any target parameters are currently marginalized
      if (!is.null(pars) && !is.null(self$marginal_vars)) {
          target_base_names <- unique(gsub("\\[.*\\]$", "", pars))
          conflicting_vars <- intersect(target_base_names, self$marginal_vars)
          if (length(conflicting_vars) > 0) {
              stop(sprintf("Parameter(s) '%s' are marginalized (integrated out) and cannot be profiled. 
To profile these parameters, please re-run optimize() without including them in 'marginal'.", 
                           paste(conflicting_vars, collapse = ", ")), call. = FALSE)
          }
      }
      
      # Use stored laplace_random_vars to ensure all random components are integrated in the re-built object
      profile_random_vars <- self$laplace_random_vars %||% self$marginal_vars
      
      ad_setup <- self$model$build_ad_obj(init = self$par, laplace = self$laplace, 
                                          map = self$map, jacobian_target = jacobian,
                                          .marginal_vars = profile_random_vars)
      ad_obj <- ad_setup$ad_obj
      
      # Sync internal state
      ad_obj$fn(ad_obj$par)
      
      # 2. Identify indices
      all_fixed_names <- rownames(self$df_fixed)
      if (is.null(all_fixed_names)) {
        stop("No fixed parameters found to profile.")
      }
      
      if (is.null(pars)) {
        target_indices <- seq_along(all_fixed_names)
        target_names <- all_fixed_names
      } else {
        target_indices <- integer(0)
        target_names <- character(0)
        
        for (p in pars) {
          idx <- which(all_fixed_names == p)
          if (length(idx) == 0) {
            idx <- grep(paste0("^", p, "(\\[|$)"), all_fixed_names)
          }
          if (length(idx) > 0) {
            target_indices <- c(target_indices, idx)
            target_names <- c(target_names, all_fixed_names[idx])
          } else {
            warning(sprintf("Parameter '%s' not found in estimated fixed effects.", p), call. = FALSE)
          }
        }
        target_indices <- unique(target_indices)
        target_names <- unique(target_names)
      }
      
      if (length(target_indices) == 0) {
        stop("No valid parameters selected for profiling.", call. = FALSE)
      }
      
      res_mat <- matrix(NA, nrow = length(target_indices), ncol = 2)
      low_label <- sprintf("Lower %g%%", (1 - level)/2 * 100)
      up_label <- sprintf("Upper %g%%", (1 - (1 - level)/2) * 100)
      colnames(res_mat) <- c(low_label, up_label)
      rownames(res_mat) <- target_names

      # Use the stored idx_fix_active for reliable index mapping
      idx_fix_active <- self$idx_fix_active
      
      if (is.null(idx_fix_active)) {
        stop("idx_fix_active not found in MAP_Fit object. Profile calculation unavailable.")
      }

      prof_list <- list()
      
      # 3. Profile
      for (i in seq_along(target_indices)) {
        idx <- target_indices[i]
        p_name <- target_names[i]
        if (!quiet) {
          cat(sprintf("  Profiling %d/%d (%s)...                                                                \r", i, length(target_indices), p_name))
          utils::flush.console()
        }
        
        args_list <- list(...)
        if (!"ytol" %in% names(args_list)) {
          args_list$ytol <- stats::qchisq(level, df = 1) / 2 + 2
        }
        args_list$obj <- ad_obj
        args_list$name <- idx
        args_list$trace <- trace
        
        prof <- tryCatch(suppressWarnings(do.call(TMB::tmbprofile, args_list)), error = function(e) NULL)
        if (!is.null(prof)) {
          prof_list[[p_name]] <- prof
          if (show_plot) {
            plot(prof, main = paste("Profile Likelihood:", p_name))
          }
          ci <- tryCatch(suppressWarnings(confint(prof, level = level)), error = function(e) NULL)
          if (!is.null(ci)) {
            # Transform CIs back to constrained scale
            transform_val <- function(u) {
              if (is.na(u)) return(NA)
              u_full <- self$par_unc
              u_full[idx_fix_active[idx]] <- u

              c_list <- to_constrained(unconstrained_vector_to_list(u_full, self$model$par_list), self$model$par_list)

              # Extract the specific value for p_name
              p_base <- gsub("\\[.*\\]$", "", p_name)
              p_val <- as.numeric(c_list[[p_base]])
              
              # Find the index of p_name within the flattened names of p_base
              p_info <- self$model$par_list[[p_base]]
              p_names_def <- self$model$par_names[[p_base]]
              all_p_names <- generate_flat_names(p_base, p_info$dim, p_names_def)
              
              p_idx <- which(all_p_names == p_name)
              if (length(p_idx) == 0) p_idx <- 1
              
              return(p_val[p_idx])
            }
            low_c <- transform_val(ci[1])
            up_c <- transform_val(ci[2])
            res_mat[i, ] <- c(min(low_c, up_c, na.rm = TRUE), max(low_c, up_c, na.rm = TRUE))
          }
        }
      }
      if (!quiet) cat("\nDone.\n")
      
      # 4. Summary
      res_df <- data.frame(
        variable = target_names,
        Estimate = format(round(self$df_fixed[target_names, "Estimate"], digits), nsmall = digits),
        check.names = FALSE,
        stringsAsFactors = FALSE
      )
      res_df[[low_label]] <- format(round(res_mat[, 1], digits), nsmall = digits)
      res_df[[up_label]] <- format(round(res_mat[, 2], digits), nsmall = digits)
      
      rownames(res_df) <- NULL
      
      if (!quiet) {
        cat("\nProfile Likelihood Confidence Intervals:\n")
        # Format for pretty printing: left align the variable names
        res_df_print <- res_df
        res_df_print$variable <- format(res_df_print$variable, justify = "left")
        print(res_df_print, row.names = FALSE, right = FALSE)
        cat("\n")
      }
      
      attr(res_df, "profiles") <- prof_list
      return(invisible(res_df))
    }
  )
)

#' @export
summary.map_fit <- function(object, ...) {
  object$summary(...)
}

#' @export
print.map_fit <- function(x, ...) {
  x$summary()
  invisible(x)
}

# Helper to handle wide-to-long conversion for repeated measures
.handle_wide_to_long <- function(formula, data, within, factors) {
  lhs <- formula[[2]]
  if (!(is.call(lhs) && identical(lhs[[1]], as.name("cbind")))) {
    return(list(formula = formula, data = data, factors = factors))
  }

  # 1. Expand columns in cbind()
  # Support cbind(v1:v4) or cbind(v1, v2, ...)
  data_names <- names(data)
  resp_cols <- character(0)
  
  for (i in 2:length(lhs)) {
    arg <- lhs[[i]]
    if (is.call(arg) && identical(arg[[1]], as.name(":"))) {
      start_name <- as.character(arg[[2]])
      end_name <- as.character(arg[[3]])
      idx_start <- which(data_names == start_name)
      idx_end <- which(data_names == end_name)
      if (length(idx_start) == 0 || length(idx_end) == 0) {
        stop(sprintf("Range columns '%s' or '%s' not found in data.", start_name, end_name))
      }
      resp_cols <- c(resp_cols, data_names[idx_start:idx_end])
    } else {
      resp_cols <- c(resp_cols, as.character(arg))
    }
  }
  
  # Validate columns exist
  missing_cols <- setdiff(resp_cols, data_names)
  if (length(missing_cols) > 0) {
    stop(sprintf("Response columns not found in data: %s", paste(missing_cols, collapse = ", ")))
  }

  n_cols <- length(resp_cols)

  # 2. Handle 'within' inference or validation
  if (is.null(within)) {
    # Infer from RHS
    rhs_vars <- all.vars(formula[[3]])
    missing_vars <- setdiff(rhs_vars, data_names)
    
    if (length(missing_vars) == 1) {
      within <- setNames(list(n_cols), missing_vars)
    } else if (length(missing_vars) > 1) {
      stop(sprintf("Multiple variables in RHS are not in data (%s). Please specify 'within' list.", paste(missing_vars, collapse = ", ")))
    } else {
      stop("LHS is cbind() but no within-factor found in RHS that is not in data. Please specify 'within'.")
    }
  }

  n_levels <- prod(as.numeric(within))
  if (n_cols != n_levels) {
    stop(sprintf("Number of response columns (%d) does not match total levels in 'within' (%d).", n_cols, n_levels))
  }

  # 3. Create within-subjects grid (last factor changes fastest)
  within_list <- lapply(within, function(x) seq_len(x))
  within_grid <- expand.grid(rev(within_list))
  within_grid <- within_grid[, rev(seq_along(within)), drop = FALSE]
  colnames(within_grid) <- names(within)

  # 4. Reshape data to long format
  N_rows <- nrow(data)
  
  # Repeat rows of original data
  long_data <- data[rep(seq_len(N_rows), each = n_cols), , drop = FALSE]
  # Remove wide columns
  long_data <- long_data[, setdiff(names(long_data), resp_cols), drop = FALSE]
  
  # Add within-factors
  for (fn in names(within)) {
    long_data[[fn]] <- rep(within_grid[[fn]], N_rows)
  }
  
  # Extract response values row by row
  Y_wide <- as.matrix(data[, resp_cols, drop = FALSE])
  long_data$RTMB_Response <- as.vector(t(Y_wide))
  
  # 5. Remove NA trials
  long_data <- long_data[!is.na(long_data$RTMB_Response), , drop = FALSE]
  if (nrow(long_data) == 0) stop("All observations removed due to NA in response columns.")

  # 6. Update formula and factors
  new_formula <- formula
  new_formula[[2]] <- as.name("RTMB_Response")
  
  new_factors <- unique(c(factors, names(within)))

  return(list(formula = new_formula, data = long_data, factors = new_factors))
}

# Internal helper to merge priors without removing NULL values
.merge_prior <- function(default, user) {
  if (is.null(user)) return(default)
  res <- user
  for (name in names(default)) {
    if (!(name %in% names(user))) {
      res[[name]] <- default[[name]]
    }
  }
  return(res)
}


#' Specify a flat prior
#'
#' @description
#' `prior_flat()` specifies that no additional prior density is added by the
#' wrapper. This is the only prior type allowed for `classic()`.
#'
#' @return A list with class `"rtmb_prior"` and `type = "flat"`.
#' @export
prior_flat <- function() {
  res <- list(type = "flat")
  class(res) <- "rtmb_prior"
  res
}


#' Specify normal/exponential priors for MAP and Bayesian inference
#'
#' @description
#' `prior_normal()` specifies normal priors for location parameters and
#' exponential priors for scale parameters. It is intended for MAP and Bayesian
#' inference, not for `classic()`.
#'
#' @param Intercept_sd Standard deviation for the intercept prior. If `NULL`, no intercept prior is added.
#' @param b_sd Standard deviation for coefficient priors. If `NULL`, no coefficient prior is added.
#' @param mu_sd Standard deviation for mean/intercept priors. If `NULL`, no mean prior is added.
#' @param sigma_rate Rate for residual standard deviation priors. If `NULL`, no sigma prior is added.
#' @param tau_rate Rate for random-effect standard deviation priors. If `NULL`, no tau prior is added.
#' @param ... Optional wrapper-specific hyperparameters.
#'
#' @return A list with class `"rtmb_prior"` and `type = "normal"`.
#' @export
prior_normal <- function(
  Intercept_sd = 10,
  mu_sd = 10,
  b_sd = 10,
  sigma_rate = 5,
  tau_rate = 5,
  ...
) {
  res <- list(
    type = "normal",
    Intercept_sd = Intercept_sd,
    mu_sd = mu_sd,
    b_sd = b_sd,
    sigma_rate = sigma_rate,
    tau_rate = tau_rate,
    ...
  )
  class(res) <- "rtmb_prior"
  res
}


#' Specify a flat prior
#'
#' @description
#' Deprecated alias of `prior_flat()`.
#'
#' @return A list with class `"rtmb_prior"` and `type = "flat"`.
#' @export
prior_uniform <- function() {
  prior_flat()
}

#' Specify a weakly informative prior
#'
#' @param sd_ratio Ratio of the prior standard deviation to the half-range of the response variable. Default is 0.5.
#' @param max_beta Maximum expected effect size. Default is 1.0.
#' @param ... Optional hyperparameters for other distributions.
#' @return A list with class "rtmb_prior"
#' @export
prior_weak <- function(sd_ratio = 0.5, max_beta = 1.0, ...) {
  res <- list(type = "weak", sd_ratio = sd_ratio, max_beta = max_beta, ...)
  class(res) <- "rtmb_prior"
  return(res)
}

#' Specify a JZS (Jeffrey-Zellner-Siow) prior for t-tests
#'
#' @param r Numeric; scale parameter for the Cauchy prior on effect size delta. Default is 0.707.
#' @return A list with class "rtmb_prior"
#' @export
prior_jzs <- function(r = 0.707) {
  res <- list(type = "jzs", r = r)
  class(res) <- "rtmb_prior"
  return(res)
}

#' Specify a Spike-and-Slab prior for variable selection
#'
#' @param ssp_ratio Prior probability of inclusion for each variable. Default is 0.25.
#' @param max_beta Maximum expected effect size. Default is 1.0.
#' @param ... Optional hyperparameters for other distributions.
#' @return A list with class "rtmb_prior"
#' @export
prior_ssp <- function(ssp_ratio = 0.25, max_beta = 1.0, ...) {
  res <- list(type = "ssp", ssp_ratio = ssp_ratio, max_beta = max_beta, ...)
  class(res) <- "rtmb_prior"
  return(res)
}

#' Specify a Regularized Horseshoe prior for continuous shrinkage
#'
#' @param expected_vars Expected number of non-zero variables. Default is 3.
#' @param slab_scale Scale parameter for the slab distribution. Default is 2.0.
#' @param slab_df Degrees of freedom for the slab distribution. Default is 4.0.
#' @param ... Optional hyperparameters for other distributions.
#' @return A list with class "rtmb_prior"
#' @export
prior_rhs <- function(expected_vars = 3, slab_scale = 2.0, slab_df = 4.0, ...) {
  res <- list(type = "rhs", expected_vars = expected_vars, slab_scale = slab_scale, slab_df = slab_df, ...)
  class(res) <- "rtmb_prior"
  return(res)
}

#' RTMB-based IRT (Item Response Theory) Wrapper
#'
#' @description
#' Performs Item Response Theory modeling (1PL, 2PL, 3PL, and Graded Response Model) using RTMB.
#' Missing values (NA) in the data are automatically removed internally for efficient computation.
#'
#' @param data A data frame or matrix of item responses (N persons x J items).
#' @param model Character string for the model type: "1PL", "2PL", or "3PL".
#' @param type Character string for the data type: "binary" or "ordered".
#' @param prior Prior configuration: `prior_flat()`, `prior_normal()`, or `prior_weak()`.
#'   Hyperparameters can be specified within these functions (e.g., `prior_normal(b_sd = 5)`).
#'   Available parameters for IRT: `a_rate` (discrimination), `b_sd` (difficulty), `c_alpha`/`c_beta` (guessing).
#' @param init List of initial values.
#' @param fixed A named list of parameter values to fix (optional).
#' @param view Character vector of parameter names to prioritize in summary.
#' @param missing Missing value handling strategy: "listwise" (default) or "fiml" (Full Information Maximum Likelihood).
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @example inst/examples/ex_irt.R
#' @export
rtmb_irt <- function(data, model = c("2PL", "1PL", "3PL"), type = c("binary", "ordered"),
                     prior = prior_flat(), 
                     init = NULL, fixed = NULL, view = NULL,
                     missing = c("listwise", "fiml"), WAIC = FALSE) {

  missing <- match.arg(missing)

  model <- match.arg(model)
  type <- match.arg(type)

  if (model == "3PL" && type == "ordered") {
    stop("The 3PL model only supports 'binary' data.")
  }

  if (is.data.frame(data)) {
    if (!all(sapply(data, is.numeric))) {
      stop("All variables in the data must be numeric. Character or factor variables are not supported.", call. = FALSE)
    }
  } else if (!is.numeric(data) && !is.logical(data)) {
    stop("The data matrix must be numeric.", call. = FALSE)
  }

  Y <- as.matrix(data)
  
  if (missing == "listwise") {
    Y <- na.omit(Y)
  }

  if (type == "binary") {
    unique_vals <- na.omit(unique(as.vector(Y)))
    if (length(unique_vals) > 2 || any(!unique_vals %in% c(0, 1))) {
      stop("Binary IRT models (type = 'binary') require responses to be exactly 0, 1, or NA. Found other values.", call. = FALSE)
    }
  }

  # Get row and column names
  item_names <- colnames(Y)
  if (is.null(item_names)) item_names <- paste0("Item", 1:ncol(Y))
  person_names <- rownames(Y)
  if (is.null(person_names)) person_names <- paste0("Person", 1:nrow(Y))

  # Prior Handling
  if (is.null(prior)) prior <- prior_flat()

  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), or prior_weak().",
      call. = FALSE
    )
  }

  prior_type <- prior$type
  
  # Extract hyperparameters from prior object if they exist
  a_rate  <- prior$a_rate
  b_sd    <- if (!is.null(prior$b_sd)) prior$b_sd else prior$mu_sd
  c_alpha <- prior$c_alpha
  c_beta  <- prior$c_beta

  # Default weak values
  if (prior_type == "weak") {
    if (is.null(a_rate)) a_rate <- 1
    if (is.null(b_sd)) b_sd <- 3
    if (is.null(c_alpha)) c_alpha <- 1
    if (is.null(c_beta)) c_beta <- 9
  }
  theta_sd <- 1 # Fixed to 1 for identification by default

  # --- Construction of Setup Block ---
  setup_exprs <- list()
  setup_exprs[[1]] <- quote(obs_data <- which(!is.na(Y), arr.ind = TRUE))
  setup_exprs[[2]] <- quote(person_idx <- as.integer(obs_data[, "row"]))
  setup_exprs[[3]] <- quote(item_idx <- as.integer(obs_data[, "col"]))
  setup_exprs[[4]] <- quote(Y_obs <- Y[obs_data])
  setup_exprs[[5]] <- quote(N_persons <- nrow(Y))
  setup_exprs[[6]] <- quote(N_items <- ncol(Y))
  setup_exprs[[7]] <- quote(N_obs <- length(Y_obs))

  if (type == "ordered") {
    setup_exprs[[8]] <- quote(if (min(Y_obs) == 0) Y_obs <- Y_obs + 1)
    setup_exprs[[9]] <- quote(K_cat <- max(Y_obs))
  }
  setup_ast <- as.call(c(list(as.name("{")), setup_exprs))

  tmp_env <- list2env(list(Y = Y))
  eval(setup_ast, tmp_env)

  # --- Construction of Parameters Block ---
  param_exprs <- list()
  param_exprs[[length(param_exprs) + 1]] <- quote(theta <- Dim(N_persons, random = TRUE))

  if (type == "binary") {
    param_exprs[[length(param_exprs) + 1]] <- quote(b <- Dim(N_items))
  } else {
    param_exprs[[length(param_exprs) + 1]] <- quote(b <- Dim(c(N_items, K_cat - 1), type = "ordered"))
  }

  if (model %in% c("2PL", "3PL")) {
    param_exprs[[length(param_exprs) + 1]] <- quote(a <- Dim(N_items, lower = 0))
  }
  if (model == "3PL") {
    param_exprs[[length(param_exprs) + 1]] <- quote(c <- Dim(N_items, lower = 0, upper = 1))
  }
  param_ast <- as.call(c(list(as.name("{")), param_exprs))

  # --- Construction of Model Block ---
  loop_body <- list()
  loop_body[[1]] <- quote(p <- person_idx[i])
  loop_body[[2]] <- quote(j <- item_idx[i])
  loop_body[[3]] <- quote(y <- Y_obs[i])

  if (type == "binary") {
    if (model == "1PL") {
      loop_body[[4]] <- quote(eta <- theta[p] - b[j])
    } else {
      loop_body[[4]] <- quote(eta <- a[j] * (theta[p] - b[j]))
    }
    if (model == "3PL") {
      loop_body[[5]] <- quote(prob <- c[j] + (1 - c[j]) * plogis(eta))
      loop_body[[6]] <- quote(y ~ bernoulli(prob))
    } else {
      loop_body[[5]] <- quote(y ~ bernoulli_logit(eta))
    }
  } else if (type == "ordered") {
    if (model == "1PL") {
      loop_body[[4]] <- quote(eta <- theta[p])
    } else {
      loop_body[[4]] <- quote(eta <- a[j] * theta[p])
    }
    loop_body[[5]] <- quote(y ~ ordered_logistic(eta, b[j, ]))
  }

  loop_ast <- quote(for (i in 1:N_obs) {})
  loop_ast[[4]] <- as.call(c(list(as.name("{")), loop_body))

  model_exprs <- list()
  model_exprs[[length(model_exprs) + 1]] <- "# Likelihood"
  model_exprs[[length(model_exprs) + 1]] <- loop_ast

  model_exprs[[length(model_exprs) + 1]] <- "# Priors"
  if (prior_type %in% c("normal", "weak")) {
    if (model %in% c("2PL", "3PL") && !is.null(a_rate)) {
      model_exprs[[length(model_exprs) + 1]] <- bquote(a ~ exponential(.(a_rate)))
    }
    if (!is.null(b_sd)) {
      if (type == "binary") {
        model_exprs[[length(model_exprs) + 1]] <- bquote(b ~ normal(0, .(b_sd)))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(for (j in 1:N_items) b[j, ] ~ normal(0, .(b_sd)))
      }
    }
    if (model == "3PL" && !is.null(c_alpha) && !is.null(c_beta)) {
      model_exprs[[length(model_exprs) + 1]] <- bquote(c ~ beta(.(c_alpha), .(c_beta)))
    }
  }
  model_exprs[[length(model_exprs) + 1]] <- bquote(theta ~ normal(0, .(theta_sd)))

  model_ast <- as.call(c(list(as.name("{")), model_exprs))

  # --- Mapping of Parameter Names ---
  code_obj <- list(setup = setup_ast, parameters = param_ast, model = model_ast)

  if (isTRUE(WAIC)) {
    gq_body <- list(
      quote(log_lik <- numeric(N_obs)),
      quote(for (i in 1:N_obs) {})
    )
    gq_loop_body <- list(
      quote(p <- person_idx[i]),
      quote(j <- item_idx[i]),
      quote(y <- Y_obs[i])
    )

    if (type == "binary") {
      if (model == "1PL") {
        gq_loop_body[[length(gq_loop_body) + 1]] <- quote(eta <- theta[p] - b[j])
      } else {
        gq_loop_body[[length(gq_loop_body) + 1]] <- quote(eta <- a[j] * (theta[p] - b[j]))
      }
      if (model == "3PL") {
        gq_loop_body[[length(gq_loop_body) + 1]] <- quote(prob <- c[j] + (1 - c[j]) * plogis(eta))
        gq_loop_body[[length(gq_loop_body) + 1]] <- quote(log_lik[i] <- bernoulli_lpmf(y, prob))
      } else {
        gq_loop_body[[length(gq_loop_body) + 1]] <- quote(log_lik[i] <- bernoulli_logit_lpmf(y, eta))
      }
    } else {
      if (model == "1PL") {
        gq_loop_body[[length(gq_loop_body) + 1]] <- quote(eta <- theta[p])
      } else {
        gq_loop_body[[length(gq_loop_body) + 1]] <- quote(eta <- a[j] * theta[p])
      }
      gq_loop_body[[length(gq_loop_body) + 1]] <- quote(log_lik[i] <- ordered_logistic_lpmf(y, eta, b[j, ]))
    }

    gq_body[[2]][[4]] <- as.call(c(list(as.name("{")), gq_loop_body))
    code_obj$generate <- .rtmb_waic_generate_ast(NULL, as.call(c(list(as.name("{")), gq_body)))
  }

  par_names_list <- list()
  par_names_list$theta <- person_names
  if (type == "binary") {
    par_names_list$b <- item_names
  } else {
    par_names_list$b <- list(item_names, paste0("Threshold", 1:(tmp_env$K_cat - 1)))
  }
  if (model %in% c("2PL", "3PL")) par_names_list$a <- item_names
  if (model == "3PL") par_names_list$c <- item_names

  view_vars <- c()
  if (model %in% c("2PL", "3PL")) view_vars <- c(view_vars, "a")
  view_vars <- c(view_vars, "b")
  if (model == "3PL") view_vars <- c(view_vars, "c")

  # --- Automatic Setting of Initial Values ---
  if (is.null(init)) {
    init <- list()
    if (type == "binary") {
      init$b <- rep(0, length(item_names))
    }
    if (model %in% c("2PL", "3PL")) init$a <- rep(1.0, length(item_names))
    if (model == "3PL") init$c <- rep(0.1, length(item_names))
  }

  obj <- rtmb_model(data = list(Y = Y), code = code_obj, par_names = par_names_list,
                    init = init, fixed = fixed, view = view_vars)
  obj$type <- "irt"
  obj$extra <- list(
    source = "wrapper",
    prior_type = prior$type,
    marginal = "theta"
  )

  return(obj)
}

  # --- 1. Binary Data (e.g., correct/incorrect answers) ---
  # Simulate binary response data for 100 persons and 5 items
  set.seed(123)
  bin_data <- matrix(rbinom(500, size = 1, prob = 0.6), nrow = 100, ncol = 5)
  colnames(bin_data) <- paste0("Item", 1:5)

  # Introduce some missing values (NA) to demonstrate automatic handling
  #bin_data[sample(1:500, 10)] <- NA

  # Fit a 2-Parameter Logistic (2PL) model
  fit_2pl <- rtmb_irt(data = bin_data, model = "2PL", type = "binary")

  # Maximum A Posteriori (MAP) estimation
  map_2pl <- fit_2pl$optimize()
  map_2pl$summary()

  # --- 2. Ordered Data (e.g., Likert scale) ---
  # Simulate ordered response data (categories 1 to 5) with a true latent structure
  set.seed(123)
  N <- 100
  J <- 5
  K <- 4

  # True parameters
  theta <- rnorm(N, 0, 1)          # Person abilities (latent traits)
  a <- runif(J, 0.8, 2.0)          # Item discriminations

  # Item-specific thresholds (must be strictly increasing)
  b <- matrix(NA, nrow = J, ncol = K - 1)
  for (j in 1:J) {
    b[j, ] <- sort(c(-1.5, 0, 1.5) + rnorm(3, 0, 0.2))
  }

  ord_data <- matrix(NA, nrow = N, ncol = J)
  colnames(ord_data) <- paste0("Item", 1:J)

  # Generate responses based on the Graded Response Model
  for (i in 1:N) {
    for (j in 1:J) {
      # Latent continuous response (eta + standard logistic noise)
      eta <- a[j] * theta[i]
      y_star <- eta + rlogis(1)

      # Apply thresholds to determine the observed category (1 to 4)
      category <- 1
      for (k in 1:(K - 1)) {
        if (y_star > b[j, k]) category <- k + 1
      }
      ord_data[i, j] <- category
    }
  }

  # Fit a Graded Response Model (2PL for ordered data)
  fit_ord <- rtmb_irt(data = ord_data, model = "2PL", type = "ordered")
  fit_ord$print_code()

  map_ord <- fit_ord$optimize()
  map_ord$summary()

#' @noRd
.print_code_impl <- function(self, private) {
  if (is.null(self$code)) {
    cat("Model code is not available (Not built with rtmb_code).\n")
    return(invisible(self))
  }

  cat("=== RTMB Model Code ===\n\n")
  cat("rtmb_code(\n")

  blocks <- setdiff(names(self$code), "env")
  is_empty_block <- function(expr) {
    is.call(expr) &&
      identical(expr[[1]], as.name("{")) &&
      length(as.list(expr)) == 1L
  }
  blocks <- blocks[!(blocks == "setup" & vapply(self$code[blocks], is_empty_block, logical(1)))]
  n_blocks <- length(blocks)

  for (i in seq_along(blocks)) {
    block <- blocks[i]
    expr <- self$code[[block]]

    lines <- deparse(expr, width.cutoff = 500L, control = "useSource")
    lines <- sapply(lines, function(x) {
      m <- regexpr("^ +", x)
      if (m > 0) {
        n_spaces <- attr(m, "match.length")
        new_spaces <- strrep(" ", n_spaces / 2)
        sub("^ +", new_spaces, x)
      } else {
        x
      }
    }, USE.NAMES = FALSE)

    lines <- gsub('^(\\s*)"#(.*)"$', "\\1#\\2", lines)
    lines <- paste0("  ", lines)

    if (trimws(lines[1]) == "{") {
      lines[1] <- paste0("  ", block, " = {")
    }

    if (i < n_blocks) {
      lines[length(lines)] <- paste0(lines[length(lines)], ", ")
    }

    cat(paste(lines, collapse = "\n"))
    cat("\n")
  }
  cat(")\n")
  return(invisible(self))
}

#' @noRd
.get_n_obs_impl <- function(self, private) {
  if (is.null(self$code$model)) return(NULL)

  get_base_name <- function(x) {
    if (is.name(x)) return(as.character(x))
    if (is.call(x) && identical(x[[1]], as.name("["))) return(get_base_name(x[[2]]))
    return(NULL)
  }

  data_names <- names(self$data)

  rewrite_to_count <- function(x) {
    if (is.call(x)) {
      if (identical(x[[1]], as.name("~"))) {
        target <- x[[2]]
        base_name <- get_base_name(target)
        if (!is.null(base_name) && base_name %in% data_names) {
          return(call("<-", as.name("n_obs"),
                      call("+", as.name("n_obs"),
                           call("if", call("is.matrix", target),
                                call("nrow", target), call("length", target)))))
        } else {
          return(quote(NULL))
        }
      }
      x[] <- lapply(as.list(x), rewrite_to_count)
    }
    return(x)
  }

  raw_expr <- self$code$model
  count_expr <- rewrite_to_count(raw_expr)

  body_list <- c(
    list(as.name("{")),
    quote(RTMB::getAll(dat, par)),
    quote(n_obs <- 0),
    if (is.call(count_expr) && identical(count_expr[[1]], as.name("{"))) as.list(count_expr)[-1] else list(count_expr),
    quote(return(n_obs))
  )

  fn_expr <- call("function", as.pairlist(alist(dat = , par = )), as.call(body_list))
  count_fn <- eval(fn_expr, envir = asNamespace("BayesRTMB"))
  test_para <- self$get_par_list()

  n_total <- tryCatch({
    count_fn(self$data, test_para)
  }, error = function(e) return(NA_integer_))

  return(n_total)
}

#' Restore MCMC Fit from CSV
#'
#' @param model An RTMB_Model object.
#' @param name Base name of the saved CSVs.
#' @param dir Directory where CSVs are saved. Default is "BayesRTMB_mcmc".
#' @param chains Number of chains. Default is 4.
#' @param laplace Logical; whether Laplace approximation was used. Default is FALSE.
#' @return An MCMC_Fit object.
#' @export
read_mcmc_csv <- function(model, name, dir = "BayesRTMB_mcmc", chains = 4, laplace = FALSE) {

  test_file <- file.path(dir, paste0(name, "-1.csv"))
  if (!file.exists(test_file)) stop(paste("File not found:", test_file))

  # --- Fix: Add check.names = FALSE ---
  test_dat <- read.csv(test_file, header = TRUE, check.names = FALSE)
  n_samples <- nrow(test_dat)

  orig_pl <- model$par_list
  random_flags <- vapply(orig_pl, function(x) isTRUE(x$random), logical(1))

  if (laplace && any(random_flags)) {
    pl_fixed  <- parse_parameters(orig_pl[!random_flags], model$par_names)
    pl_random <- parse_parameters(orig_pl[random_flags], model$par_names)
  } else {
    pl_fixed   <- model$pl_full
    pl_random  <- NULL
  }

  P_fixed <- length(pl_fixed$names)
  P_random <- if(!is.null(pl_random)) length(pl_random$names) else 0

  fit <- array(NA, dim=c(n_samples, chains, P_fixed + 1))
  dimnames(fit) <- list(iteration = NULL, chain = paste0("chain", 1:chains), variable = c("lp", pl_fixed$names))

  if (P_random > 0) {
    random_fit <- array(NA, dim=c(n_samples, chains, P_random))
    dimnames(random_fit) <- list(iteration = NULL, chain = paste0("chain", 1:chains), variable = pl_random$names)
  } else {
    random_fit <- NULL
  }

  accept_mat <- array(NA, dim=c(n_samples, chains))
  td_mat <- array(NA, dim=c(n_samples, chains))
  eps_vec <- numeric(chains)

  for (c in 1:chains) {
    file_path <- file.path(dir, paste0(name, "-", c, ".csv"))
    if (!file.exists(file_path)) stop(paste("File not found:", file_path))

    # --- Fix: Add check.names = FALSE ---
    dat <- read.csv(file_path, header = TRUE, check.names = FALSE)

    if (nrow(dat) != n_samples) {
      warning(sprintf("Chain %d has %d iterations, expected %d", c, nrow(dat), n_samples))
    }

    accept_mat[, c] <- dat$accept
    td_mat[, c] <- dat$treedepth
    eps_vec[c] <- dat$eps[1]

    fit[, c, "lp"] <- dat$lp
    for (p_name in pl_fixed$names) {
      fit[, c, p_name] <- dat[[p_name]]
    }

    if (P_random > 0) {
      for (p_name in pl_random$names) {
        random_fit[, c, p_name] <- dat[[p_name]]
      }
    }
  }

  eps_chains <- eps_vec
  accept_chains <- apply(accept_mat, 2, mean)
  treedepth_chains <- apply(td_mat, 2, max)
  names(eps_chains) <- names(accept_chains) <- names(treedepth_chains) <- paste0("chain", 1:chains)

  posterior_mean <- numeric(length(model$pl_full$names))
  names(posterior_mean) <- model$pl_full$names
  fixed_mean <- apply(fit[, , -1, drop = FALSE], 3, mean)
  posterior_mean[names(fixed_mean)] <- fixed_mean

  if (!is.null(random_fit)) {
    random_mean <- apply(random_fit, 3, mean)
    posterior_mean[names(random_mean)] <- random_mean
  }

  res_obj <- MCMC_Fit$new(
    model          = model,
    fit            = fit,
    random_fit     = random_fit,
    eps            = eps_chains,
    accept         = accept_chains,
    treedepth      = treedepth_chains,
    laplace        = laplace,
    posterior_mean = posterior_mean
  )

  has_tran <- !is.null(model$transform)
  has_generate <- !is.null(model$generate)
  has_cf_corr <- any(sapply(model$par_list, function(x) x$type == "CF_corr"))

  if (has_tran || has_cf_corr) res_obj$transformed_draws(model$transform)
  if (has_generate) res_obj$generated_quantities(model$generate)

  return(res_obj)
}
# Helper to assign class to summary results
as_summary_df <- function(df) {
  class(df) <- c("summary_BayesRTMB", "data.frame")
  return(df)
}

#' print for summary_BayesRTMB class
#' @method print summary_BayesRTMB
#' @param x An object of class summary_BayesRTMB
#' @param digits integer
#' @param ... Additional arguments.
#' @export
print.summary_BayesRTMB <- function(x, digits = NULL,...) {
  # Ensure x is treated as a data frame
  df <- as.data.frame(x)

  if (is.null(digits)) {
    digits <- attr(x, "digits")
    if (is.null(digits)) digits <- 2
  }

  # 1. Convert each column to string (specify display format)
  out_char <- as.data.frame(lapply(names(df), function(cn) {
    val <- df[[cn]]
    if (is.numeric(val)) {
      if (grepl("ess|iter|count|df", cn, ignore.case = TRUE)) {
        # ESS, DF and similar metrics are integers (including NA handling)
        ifelse(is.na(val), "NA", sprintf("%.0f", val))
      } else if (cn == "t value" || cn == "Pr(>|t|)" || cn == "Pr") {
        # Specific formatting for classic frequentist results
        fmt <- "%.3f"
        res <- ifelse(is.na(val), "NA", sprintf(fmt, val))
        if (cn == "Pr(>|t|)" || cn == "Pr") {
          # Omit leading zero for p-values
          res <- gsub("^0\\.", ".", res)
          res <- gsub("^-0\\.", "-.", res) # Handle rare case of -0
        }
        res
      } else {
        fmt <- paste0("%.", digits, "f")
        ifelse(is.na(val), "NA", sprintf(fmt, val))
      }
    } else {
      ifelse(is.na(val), "NA", as.character(val))
    }
  }), stringsAsFactors = FALSE)
  colnames(out_char) <- names(df)

  col_widths <- sapply(seq_along(names(out_char)), function(i) {
    max(nchar(names(out_char)[i]), nchar(out_char[, i]), na.rm = TRUE)
  })

  # 2. Calculate column widths
  col_widths <- sapply(seq_along(names(out_char)), function(i) {
    max(nchar(names(out_char)[i]), nchar(out_char[, i]))
  })

  # 3. Display header (all titles right-aligned)
  header_parts <- sapply(seq_along(col_widths), function(i) {
    sprintf(paste0("%", col_widths[i], "s"), names(out_char)[i])
  })
  cat(paste(header_parts, collapse = "  "), "\n")

  # 4. Display data (only the first column is left-aligned)
  for (r in seq_len(nrow(out_char))) {
    row_parts <- sapply(seq_along(col_widths), function(c) {
      if (c == 1) {
        # Left-align only the variable name data
        sprintf(paste0("%-", col_widths[c], "s"), out_char[r, c])
      } else {
        # Right-align numeric data
        sprintf(paste0("%", col_widths[c], "s"), out_char[r, c])
      }
    })
    cat(paste(row_parts, collapse = "  "), "\n")
  }

  return(invisible(x))
}

#' @import MASS
#' @importFrom grDevices col2rgb hcl.colors rgb
#' @importFrom graphics abline axis legend lines matplot mtext pairs par points polygon segments strwidth text
#' @importFrom stats Gamma binomial coef cor cov delete.response density dunif gaussian glm median model.frame model.matrix model.offset model.response na.omit nlminb poisson quantile rnorm runif sd terms var
#' @importFrom utils capture.output getFromNamespace modifyList read.csv
NULL


#' Create an RTMB_Model Object
#'
#' @description
#' The `rtmb_model` function acts as the core constructor for compiling and combining
#' user-defined data with the model code (defined via `rtmb_code`). It generates an
#' `RTMB_Model` (R6 class) instance, which serves as the foundation for performing
#' Bayesian inference, including Maximum A Posteriori (MAP) estimation, Variational
#' Inference (ADVI), and Markov Chain Monte Carlo (MCMC) sampling.
#'
#' @details
#' \strong{Model Compilation and Pre-checking:}
#' When this function is called, it evaluates the provided data and model blocks.
#' It performs a "sandbox execution" (pre-check) using dummy initial values to dynamically
#' detect common structural errors—such as undefined variables, out-of-bounds indices,
#' or incompatible matrix operations—before proceeding to the computationally expensive
#' Automatic Differentiation (MakeADFun) phase. Cryptic backend errors are caught and
#' translated into user-friendly hints.
#'
#' \strong{Writing AD-Compatible Code (Important):}
#' To ensure the model is differentiable, you must follow specific syntax rules
#' when writing code within \code{rtmb_code}. Avoid discrete branching (\code{if}, \code{ifelse})
#' based on parameters and use numerically stable functions.
#' See \code{\link{rtmb_syntax}} for a detailed guide on Automatic Differentiation requirements.
#'
#' \strong{Initial Values (\code{init}):}
#' Initial parameter values can be specified as a flat numeric vector or a named list.
#' If a partial list is provided, the unspecified parameters are automatically initialized
#' with random values drawn from a uniform distribution on the unconstrained scale.
#' If \code{init = NULL}, all parameters are initialized randomly.
#'
#' \strong{Parameter Labeling (\code{par_names}):}
#' By default, vector or matrix parameters are displayed with numeric indices (e.g., \code{mu[1]}).
#' You can pass a named list of character vectors to \code{par_names} to assign meaningful
#' labels to specific dimensions (e.g., \code{mu[Control]}), vastly improving the readability
#' of summary outputs and trace plots.
#'
#' \strong{Available Inference Methods on the Returned Object:}
#' The returned \code{RTMB_Model} instance provides the following core methods:
#' \itemize{
#'   \item \code{$optimize(...)}: Performs Maximum A Posteriori (MAP) or Maximum Likelihood estimation. Returns a \code{MAP_Fit} object.
#'   \item \code{$sample(...)}: Draws posterior samples using the NUTS (No-U-Turn Sampler) algorithm. Returns an \code{MCMC_Fit} object.
#'   \item \code{$variational(...)}: Performs Mean-field or Full-rank Automatic Differentiation Variational Inference (ADVI). Returns a \code{VB_Fit} object.

#' }
#'
#' @param data A named list containing observation data and constants (e.g., sample size, matrices) used in the model.
#' @param code A model definition block generated by \code{rtmb_code(...)} (must include \code{parameters} and \code{model} blocks).
#' @param par_names A named list of character vectors corresponding to the dimensions of specific parameters (optional).
#' @param init A list or numeric vector of initial values for parameters (optional). If not specified, initialized randomly.
#' @param view Character vector of parameter names to be displayed preferentially at the top when outputting results like \code{summary()} (optional).
#' @param fixed A named list of parameter values to fix (optional). Useful for scoring or plug-in estimation where some parameters (e.g., item parameters) are fixed to known values.

#'
#' @return An \code{RTMB_Model} class instance with a compiled and pre-tested automatic differentiation function.
#'
#' @examples
#' \donttest{
#' # Simulate data for 3 groups
#' set.seed(123)
#' N <- 60
#' group_idx <- sample(1:3, N, replace = TRUE)
#' group_names <- c("Control", "Treatment_A", "Treatment_B")
#'
#' # True group means: Control = 0, Treatment_A = 2, Treatment_B = -1
#' true_means <- c(0, 2, -1)
#' y <- true_means[group_idx] + rnorm(N, mean = 0, sd = 0.5)
#'
#' data_list <- list(N = N, K = 3, group_idx = group_idx, y = y)
#'
#' # Define the model using rtmb_code
#' model_code <- rtmb_code(
#'   parameters = {
#'     mu    = Dim(K)             # Vector of length K (group means)
#'     sigma = Dim(1, lower = 0)  # Scalar (residual standard deviation)
#'   },
#'   model = {
#'     # Priors
#'     for (k in 1:K) mu[k] ~ normal(0, 10)
#'     sigma ~ exponential(1)
#'
#'     # Likelihood
#'     for (i in 1:N) {
#'       y[i] ~ normal(mu[group_idx[i]], sigma)
#'     }
#'   }
#' )
#'
#' # --- 1. Basic Model Creation ---
#' # Create the RTMB_Model object
#' mod_basic <- rtmb_model(
#'   data = data_list,
#'   code = model_code
#' )
#'
#' # Perform Maximum A Posteriori (MAP) estimation
#' map_basic <- mod_basic$optimize()
#' # The summary displays default parameter names: mu[1], mu[2], mu[3]
#' map_basic$summary()
#'
#' # --- 2. Optional: Adding Custom Parameter Names and initial values ---
#' # You can optionally use 'par_names' to assign meaningful labels
#' # to vector or matrix elements for easier interpretation.
#' mod_named <- rtmb_model(
#'   data = data_list,
#'   code = model_code,
#'   init = list(mu = rep(0, 3), sigma = 1),
#'   par_names = list(mu = group_names)
#' )
#'
#' map_named <- mod_named$optimize()
#' # The summary now displays: mu[Control], mu[Treatment_A], mu[Treatment_B]
#' map_named$summary()
#' }
#' @param silent Logical; if TRUE, suppresses diagnostic messages during model creation. Default is FALSE.
#'
#' @return An \code{RTMB_Model} class instance with a compiled and pre-tested automatic differentiation function.
#'
#' @export
rtmb_model <- function(data, code, par_names = list(), init = NULL, view = NULL, fixed = NULL, silent = FALSE) {

  if (is.null(silent)) silent <- FALSE
  if (getOption("BayesRTMB.silent", FALSE)) silent <- TRUE

  old_silent <- options(BayesRTMB.silent = silent)
  on.exit(options(old_silent), add = TRUE)

  if (!"parameters" %in% names(code)) stop("The 'parameters = { ... }' block is required in code.")
  if (!"model" %in% names(code)) stop("The 'model = { ... }' block is required in code.")

  data <- validate_data(data)

  # --- 1. Static validation of Data block (optional) ---
  if ("data" %in% names(code)) {
    data_vars <- all.vars(code$data)
    missing_data <- setdiff(data_vars, names(data))
    if (length(missing_data) > 0) {
      stop(sprintf("[Declaration error in 'data' block]\nThe declared variable '%s' does not exist in the provided data list (%s).",
                   missing_data[1], paste(names(data), collapse = ", ")), call. = FALSE)
    }
  }

  if ("setup" %in% names(code)) {
    # Extract data into R environment and evaluate setup code within it
    dat_env <- list2env(data, parent = parent.frame())
    tryCatch({
      eval(code$setup, envir = dat_env)
    }, error = function(e) {
      stop(sprintf("[Error in 'setup' block] An error occurred during data preprocessing:\n  %s", conditionMessage(e)), call. = FALSE)
    })
    # Return all created/modified variables to the data list after evaluation
    data <- env_to_ordered_list(dat_env, data, code$setup)
  }

  # --- 2. Static validation and evaluation of Parameters block ---
  param_exprs <- as.list(code$parameters)[-1]
  evaluated_par_list <- list()
  eval_env <- list2env(data, parent = parent.frame())

  for (e in param_exprs) {
    if (is.null(e)) next
    if (is.call(e) && (identical(e[[1]], as.name("=")) || identical(e[[1]], as.name("<-")))) {
      v_name <- as.character(e[[2]])
      if (identical(v_name, "lp")) {
        stop(
          "[Error in 'parameters' block]\n",
          "'lp' is a reserved variable used internally for accumulating log probability and cannot be declared as a parameter.",
          call. = FALSE
        )
      }
      d_expr <- e[[3]]

      used_vars <- all.vars(d_expr)
      missing_vars <- setdiff(used_vars, names(data))

      if (length(missing_vars) > 0) {
        stop(sprintf("[Error in 'parameters' block]\nUndefined variable '%s' is used.\n  [Location]: %s\n  * Please check if it is included in the 'data' list.",
                     missing_vars[1], paste(deparse(e), collapse = " ")), call. = FALSE)
      }

      p_obj <- eval(d_expr, envir = eval_env)
      
      # Handle initial values from 'init' argument
      if (!is.null(init)) {
        if (is.list(init) && !is.null(init[[v_name]])) {
          p_obj$init <- init[[v_name]]
        } else if (is.numeric(init)) {
          # If init is a flat vector, it's more complex to handle here without to_constrained logic
          # For now, handle list-based init which is standard for wrappers
        }
      }
      
      evaluated_par_list[[v_name]] <- p_obj
    } else {
      stop("Format error in 'parameters' block: ", paste(deparse(e), collapse = " "))
    }
  }

  for (v_name in names(evaluated_par_list)) {
    p <- evaluated_par_list[[v_name]]
    if (!is.null(p$type) && p$type %in% c("CF_cov", "CF_corr", "cov_matrix", "corr_matrix")) {
      if (length(p$dim) == 3) {
        if (p$dim[2] != p$dim[3]) {
          stop(sprintf("[Parameter definition error] Variable '%s' is specified as type = '%s', but its matrix slices do not have square dimensions (Specified dimension: %s).",
                       v_name, p$type, paste(p$dim, collapse = " x ")), call. = FALSE)
        }
      } else if (length(p$dim) < 2 || p$dim[1] != p$dim[2]) {
        stop(sprintf("[Parameter definition error] Variable '%s' is specified as type = '%s', but it does not have square matrix dimensions (Specified dimension: %s).",
                     v_name, p$type, paste(p$dim, collapse = " x ")), call. = FALSE)
      }
    }
  }

  check_unsupported_funcs <- function(expr) {
    if (is.call(expr)) {
      func_name <- as.character(expr[[1]])
      if (length(func_name) == 1 && func_name %in% c("ifelse", "apply", "sapply", "lapply", "tapply", "vapply")) {
        stop(sprintf("[Syntax error in 'model' block] Function '%s' is used, which is not supported by automatic differentiation (AD).\n  * Workaround for conditionals: Substitute with arithmetic operations like '(condition) * A + (!condition) * B'.\n  * Workaround for loop functions: Rewrite as a standard 'for' loop.", func_name), call. = FALSE)
      }
      lapply(as.list(expr)[-1], check_unsupported_funcs)
    }
  }
  if ("model" %in% names(code)) {
    check_unsupported_funcs(code$model)
  }
  original_code <- code
  code <- inject_transform_adreports(code)

  # --- 3. Dynamic compilation (model, transform, generate) ---
  user_env <- if (!is.null(code$env)) code$env else parent.frame()

  exec_model_ast <- code$model

  comp_model <- eval(bquote(model_code(.(code$model), env = user_env)))

  comp_transform <- NULL
  if ("transform" %in% names(code)) {
    comp_transform <- eval(bquote(transform_code(.(code$transform), env = user_env)))
  }

  comp_generate <- NULL
  if ("generate" %in% names(code)) {
    comp_generate <- eval(bquote(transform_code(.(code$generate), env = user_env)))
  }

  # --- 4. Pre-check (Sandbox execution) ---
  # --- 4. Pre-check (Sandbox execution) ---
  if (!silent) {
    cat("Pre-checking model code...\n")
  }

  test_unc_list <- lapply(evaluated_par_list, function(p) {
    rnorm(p$unc_length, mean = 0, sd = 0.1)
  })
  test_para <- to_constrained(test_unc_list, evaluated_par_list)

  if (!is.null(comp_transform)) {
    test_tran <- with_rtmb_error_handling({ comp_transform(data, test_para) }, "transform")
    if (is.list(test_tran)) {
      # Check for name collisions between transform outputs and parameter names
      colliding_names <- intersect(names(test_tran), names(evaluated_par_list))
      if (length(colliding_names) > 0) {
        warning(sprintf(
          "[Warning in 'transform' block] The following variable names in 'transform' conflict with parameter names: %s. ",
          paste(colliding_names, collapse = ", ")
        ),
        "This may cause unexpected behavior during model evaluation. ",
        "Consider renaming the transform variables to avoid conflicts.",
        call. = FALSE)
      }
      test_para <- utils::modifyList(test_para, test_tran)
    }
  }

  with_rtmb_error_handling({ 
    comp_model(data, test_para) 
  }, "model")

  if (!is.null(comp_generate)) {
    with_rtmb_error_handling({ comp_generate(data, test_para) }, "generate")
  }

  # --- 5. Create safe wrappers for execution ---
  safe_model <- function(dat, para) { with_rtmb_error_handling({ comp_model(dat, para) }, "model") }


  safe_transformed <- NULL
  if (!is.null(comp_transform)) {
    safe_transformed <- function(dat, para) {
      res <- with_rtmb_error_handling({ comp_transform(dat, para) }, "transform")
      if (!is.list(res)) stop("The return value must be a 'named list'.")
      res
    }
    attr(safe_transformed, "raw_expr") <- code$transform
  }

  safe_generate <- NULL
  if (!is.null(comp_generate)) {
    safe_generate <- function(dat, para) {
      res <- with_rtmb_error_handling({ comp_generate(dat, para) }, "generate")
      if (!is.list(res)) stop("The return value must be a 'named list'.")
      res
    }
    attr(safe_generate, "raw_expr") <- code$generate
  }

  # --- 6. Pass to R6 class ---
  obj <- RTMB_Model$new(
    data       = data,
    par_list   = evaluated_par_list,
    log_prob   = safe_model,
    transform  = safe_transformed,
    generate   = safe_generate,
    par_names  = par_names,
    init       = init,
    view       = view,
    code       = original_code
  )


  if (!is.null(fixed)) {
    obj <- obj$fixed_model(fixed)
  }



  return(obj)
}

report <- function(...) invisible(NULL)

# --- Helper functions for AST exploration ---

# Check if a specific function call (e.g., "report") is included in the expression
has_function_call <- function(expr, func_name) {
  if (is.atomic(expr) || is.name(expr)) return(FALSE)
  if (is.call(expr)) {
    if (identical(expr[[1]], as.name(func_name))) return(TRUE)
    return(any(sapply(as.list(expr)[-1], has_function_call, func_name = func_name)))
  }
  return(FALSE)
}

extract_report_vars <- function(expr) {
  vars <- character()
  if (is.atomic(expr) || is.name(expr)) return(vars)
  if (is.call(expr)) {
    if (identical(expr[[1]], as.name("report"))) {
      for (i in seq_along(expr)[-1]) {
        if (is.name(expr[[i]])) vars <- c(vars, as.character(expr[[i]]))
      }
    } else {
      for (i in seq_along(expr)[-1]) {
        vars <- c(vars, extract_report_vars(expr[[i]]))
      }
    }
  }
  return(unique(vars))
}

# Remove report() calls from AST to avoid runtime errors
remove_report_calls <- function(expr) {
  if (is.atomic(expr) || is.name(expr)) return(expr)
  if (is.call(expr)) {
    if (identical(expr[[1]], as.name("report"))) return(NULL)
    new_args <- lapply(as.list(expr), remove_report_calls)
    new_args <- Filter(Negate(is.null), new_args)
    if (length(new_args) > 0) return(as.call(new_args)) else return(NULL)
  }
  return(expr)
}

# Extract all assigned variable names
get_assigned_vars <- function(expr) {
  vars <- character()
  if (is.atomic(expr) || is.name(expr)) return(vars)
  if (is.call(expr)) {
    if (identical(expr[[1]], as.name("<-")) || identical(expr[[1]], as.name("="))) {
      if (is.name(expr[[2]])) vars <- c(vars, as.character(expr[[2]]))
    }
    for (i in seq_along(expr)[-1]) {
      vars <- c(vars, get_assigned_vars(expr[[i]]))
    }
  }
  return(unique(vars))
}

# Inject ADREPORT into the model block based on the transform content
inject_transform_adreports <- function(code) {
  if (is.null(code$transform)) return(code)

  # 1. Check if report() is explicitly specified
  explicit_vars <- extract_report_vars(code$transform)

  if (length(explicit_vars) > 0) {
    # If explicit, target only those variables. Remove the report() call from transform
    target_vars <- explicit_vars
    code$transform <- remove_report_calls(code$transform)
  } else {
    # If not explicitly specified, target all variables assigned in transform
    target_vars <- get_assigned_vars(code$transform)
  }

  # 2. Add ADREPORT(var) to the end of the model block
  if (length(target_vars) > 0 && !is.null(code$model)) {
    ad_exprs <- lapply(target_vars, function(v) call("ADREPORT", as.name(v)))
    if (is.call(code$model) && identical(code$model[[1]], as.name("{"))) {
      # Inject into { ... }
      code$model <- as.call(c(as.list(code$model), ad_exprs))
    }
  }
  return(code)
}

#' Catch and Translate RTMB Errors into User-Friendly Messages
#'
#' @description
#' This internal helper function wraps an R expression in a [tryCatch()] block to
#' capture cryptic error messages generated by the RTMB automatic differentiation
#' engine or C++ backend, translating them into descriptive hints.
#'
#' @details
#' It uses regular expressions to categorize common RTMB failure modes, such as:
#' \itemize{
#'   \item **Missing Objects/Functions**: Detects if a variable or a specific distribution
#'   function (e.g., `normal_lpdf`) is missing from the environment.
#'   \item **Index Errors**: Identifies subscript out of bounds errors.
#'   \item **AD Type Destruction**: Catches the "lost class attribute" error, often caused
#'   by incorrect initialization of vectors (e.g., using `numeric(N)` and partial assignment
#'   inside loops).
#'   \item **Matrix Dimension Mismatch**: Identifies non-conformable arguments in
#'   matrix multiplication.
#' }
#'
#' @param expr An R expression to be evaluated, typically representing the logic within
#' a specific model block.
#' @param block_name A character string identifying the context of the error
#' (e.g., "parameters", "model", "transform", or "generate").
#'
#' @return The result of evaluating the \code{expr}. If an error occurs, it throws
#' a formatted [stop()] message.
#'
#' @keywords internal
with_rtmb_error_handling <- function(expr, block_name) {
  tryCatch({
    expr
  }, error = function(e) {
    msg <- conditionMessage(e)
    call_expr <- conditionCall(e)

    call_str <- if (!is.null(call_expr)) {
      code_snippet <- paste(deparse(call_expr), collapse = " ")
      if (nchar(code_snippet) > 100) paste0(substr(code_snippet, 1, 97), "...") else code_snippet
    } else {
      "Unknown location"
    }

    # 1. Use of non-existent variables
    if (grepl("object '.*' not found", msg)) {
      var_name <- sub(".*object '(.*)'.*", "\\1", msg)
      stop(sprintf("[Error in '%s' block] Undefined variable '%s' is used.\n  [Location]: %s\n  * Please check if it is included in 'data' or 'parameters'.",
                   block_name, var_name, call_str), call. = FALSE)
    }

    # 2. Use of non-existent functions
    if (grepl("could not find function", msg)) {
      func_name <- sub(".*function [\"']?(.*?)[\"']? could not.*", "\\1", msg)
      stop(sprintf("[Error in '%s' block] Undefined function '%s' is used.\n  [Location]: %s",
                   block_name, func_name, call_str), call. = FALSE)
    }

    # 3. Index error
    if (grepl("subscript out of bounds", msg)) {
      stop(sprintf("[Error in '%s' block] Array or matrix subscript out of bounds.\n  [Location]: %s",
                   block_name, call_str), call. = FALSE)
    }

    # 4. Destruction of advector / Failure of type conversion
    if (grepl("lost class attribute", msg) || grepl("Invalid argument to 'advector'", msg) ||
        grepl("non-numeric argument to mathematical function", msg)) {
      stop(sprintf("[Error in '%s' block] Automatic differentiation (AD) type was lost or an invalid operation was performed.\n  [Location]: %s\n  * Possible cause: Assigning values to an empty vector initialized with 'numeric(N)' (turns into a list type).\n  * Solutions:\n    1. Vectorize the calculation to avoid loops.\n    2. Initialize while preserving AD type, e.g., 'vec <- rep(theta[1]*0, N)'.\n    3. Use 'advector(numeric(N))'.",
                   block_name, call_str), call. = FALSE)
    }

    # 5. Matrix dimension mismatch
    if (grepl("non-conformable arguments", msg)) {
      stop(sprintf("[Error in '%s' block] Non-conformable arguments in matrix multiplication (%%*%%) or operations.\n  [Location]: %s\n  * Note: R's standard 'automatic recycling rule' for vectors of different lengths does not apply to RTMB matrix operations. Ensure dimensions match exactly or use rep() to align sizes explicitly.",
                   block_name, call_str), call. = FALSE)
    }

    # Other errors
    stop(sprintf("[Error in '%s' block]\n  [Message]: %s\n  [Location]: %s",
                 block_name, msg, call_str), call. = FALSE)
  })
}

#' Internal function to convert an environment to a list while maintaining original order
#' @keywords internal
env_to_ordered_list <- function(env, orig_list, code_ast = NULL) {
  env_list <- as.list(env)
  orig_names <- names(orig_list)

  if (!is.null(code_ast)) {
    extract_assigned_vars <- function(expr) {
      vars <- character()
      if (is.call(expr)) {
        if (identical(expr[[1]], as.name("<-")) || identical(expr[[1]], as.name("="))) {
          if (is.name(expr[[2]])) vars <- c(vars, as.character(expr[[2]]))
        } else if (identical(expr[[1]], as.name("{"))) {
          if (length(expr) > 1) {
            for (i in seq_along(expr)[-1]) {
              vars <- c(vars, extract_assigned_vars(expr[[i]]))
            }
          }
        }
      }
      return(unique(vars))
    }

    assigned_vars <- extract_assigned_vars(code_ast)
    assigned_vars <- intersect(assigned_vars, names(env_list))
    new_names <- intersect(assigned_vars, setdiff(names(env_list), orig_names))
    other_new_names <- setdiff(names(env_list), c(orig_names, new_names))
    new_names <- c(new_names, other_new_names)
  } else {
    new_names <- setdiff(names(env_list), orig_names)
  }

  return(env_list[c(intersect(orig_names, names(env_list)), new_names)])
}

#' Internal function to search AST and inject namespace to package functions
#' @param expr Unevaluated code (AST)
#' @param pkg Target package name
#' @keywords internal
inject_namespace <- function(expr, pkg = "BayesRTMB") {
  if (is.atomic(expr) || is.name(expr)) {
    return(expr)
  }

  if (is.call(expr)) {
    if (identical(expr[[1]], quote(`::`)) || identical(expr[[1]], quote(`:::`))) {
      return(expr)
    }

    func_name <- as.character(expr[[1]])

    if (length(func_name) == 1) {
      if (exists(func_name, envir = asNamespace(pkg), inherits = FALSE)) {
        expr[[1]] <- call(":::", as.name(pkg), as.name(func_name))
      }
    }

    if (length(expr) > 1) {
      for (i in seq_along(expr)[-1]) {
        if (!identical(expr[[i]], quote(expr=))) {
          expr[[i]] <- inject_namespace(expr[[i]], pkg)
        }
      }
    }
  }

  return(expr)
}

#' Define an RTMB Model with Stan-like Syntax
#'
#' @description
#' \code{rtmb_code} is the primary interface for defining Bayesian or Frequentist models
#' within the RTMB framework. It allows you to organize model logic into functional
#' blocks while utilizing a Stan-like \code{~} operator for specifying priors and likelihoods.
#'
#' @details
#' \strong{Key Blocks and Logic:}
#' \itemize{
#'   \item \code{setup}: Pre-compilation data processing.
#'   \item \code{parameters}: Declaration of estimated parameters. See \code{\link{parameter_types}} for available constraints.
#'   \item \code{transform}: Definition of intermediate variables. Use \code{\link{math_functions}} for stable AD calculations.
#'   \item \code{model}: Likelihood and Priors. Refer to \code{\link{distributions}} for available sampling statements.
#'   \item \code{generate}: Post-hoc calculation of predictions or diagnostics.
#' }
#'
#' \strong{Automatic Differentiation (AD) Note:}
#' All code within \code{parameters}, \code{transform}, and \code{model} is
#' automatically differentiated by RTMB. To ensure numerical stability, use
#' provided utility functions like \code{\link{log_sum_exp}} or \code{\link{inv_logit}}
#' instead of raw algebraic implementations.
#'
#' @param ... Model definition blocks.
#' @return A list of unevaluated code blocks.
#' @seealso
#' \code{\link{rtmb_model}} for building the model object.
#' \code{\link{parameter_types}} for constraining parameters.
#' \code{\link{distributions}} for likelihood functions.
#' \code{\link{math_functions}} for numerical utilities.
#'
#' @examples
#' \donttest{
#' # Simulate data for a linear regression
#' set.seed(123)
#' N <- 100
#' x <- rnorm(N)
#' y <- 2.0 + 1.5 * x + rnorm(N, mean = 0, sd = 1)
#' data_list <- list(N = N, x = x, y = y)
#'
#' # Define the model using rtmb_code
#' code <- rtmb_code(
#'   setup = {
#'     # Center the predictor variable (executed once)
#'     x_centered <- x - mean(x)
#'   },
#'   parameters = {
#'     # Define parameters and their constraints
#'     alpha = Dim(1)
#'     beta  = Dim(1)
#'     sigma = Dim(1, lower = 0)
#'   },
#'   transform = {
#'     # Calculate the linear predictor
#'     mu <- alpha + beta * x_centered
#'   },
#'   model = {
#'     # Priors
#'     alpha ~ normal(0, 10)
#'     beta  ~ normal(0, 10)
#'     sigma ~ exponential(1)
#'
#'     # Likelihood (Vectorized)
#'     y ~ normal(mu, sigma)
#'   },
#'   generate = {
#'     # Calculate generated quantities
#'     y_pred <- mu
#'
#'     # Must return a named list
#'     list(y_pred = y_pred)
#'   }
#' )
#'
#' # Create the model object
#' mod <- rtmb_model(data = data_list, code = code)
#'
#' # Fit the model using MAP estimation
#' map_res <- mod$optimize()
#' map_res$summary(pars = c("alpha", "beta", "sigma"))
#'
#' # The generated quantity 'y_pred' can also be summarized
#' map_res$summary("y_pred", max_rows = 5)
#' }
#' @export
rtmb_code <- function(...) {
  args <- as.list(match.call())[-1]
  code_list <- list()
  valid_blocks <- c("setup", "data", "parameters", "transform", "model", "generate")

  if (!is.null(names(args))) {
    for (name in names(args)) {
      if (name == "") next
      if (name %in% valid_blocks) {
        code_list[[name]] <- args[[name]]
      } else {
        stop(sprintf("Unknown block '%s' is specified. Valid blocks are %s.",
                     name, paste(valid_blocks, collapse = ", ")), call. = FALSE)
      }
    }

  } else if (length(args) == 1 && is.null(names(args))) {
    ast <- args[[1]]
    if (is.call(ast) && identical(ast[[1]], as.name("{"))) {
      exprs <- as.list(ast)[-1]

      for (e in exprs) {
        if (is.null(e)) next

        if (is.call(e)) {
          block_name <- as.character(e[[1]])
          if (block_name %in% valid_blocks) {
            code_list[[block_name]] <- if (length(e) >= 2) e[[2]] else quote({})
          } else {
            stop(sprintf("Unknown block '%s' is specified. Valid blocks are %s.",
                         block_name, paste(valid_blocks, collapse = ", ")), call. = FALSE)
          }
        } else {
          stop(sprintf("Invalid syntax in rtmb_code: '%s'",
                       paste(deparse(e), collapse = " ")), call. = FALSE)
        }
      }
    } else {
      stop("Invalid syntax in rtmb_code(). Separate blocks with commas, or enclose the entire definition in {}.", call. = FALSE)
    }
  } else {
    stop("Invalid syntax in rtmb_code(). Separate blocks with commas, or enclose the entire definition in {}.", call. = FALSE)
  }

  code_list$env <- parent.frame()
  return(code_list)
}


#' Model Code Wrapper for RTMB
#'
#' @param expr A block of code containing model description.
#' @param env Environment to assign to the generated function.
#' @return A standard R function object taking (dat, par).
model_code <- function(expr, env = parent.frame()) {
  raw_expr <- substitute(expr)

  if (is.name(raw_expr)) {
    evaluated <- eval(raw_expr, envir = env)
    if (is.language(evaluated)) {
      raw_expr <- evaluated
    }
  }

  # --- Inject namespace into standard math functions etc. ---
  raw_expr <- inject_namespace(raw_expr, pkg = "BayesRTMB")

  rewrite_formula <- function(x) {
    if (is.call(x)) {
      if (identical(x[[1]], as.name("~"))) {
        target <- x[[2]]
        dist_call <- x[[3]]

        if (!is.call(dist_call)) {
          stop(sprintf("The right side of '~' must be a distribution function call. (e.g., %s( ... ))",
                       as.character(dist_call)), call. = FALSE)
        }

        dist_name_raw <- dist_call[[1]]
        if (is.call(dist_name_raw) && identical(dist_name_raw[[1]], as.name(":::"))) {
          dist_name <- as.character(dist_name_raw[[3]])
        } else {
          dist_name <- as.character(dist_name_raw)
        }
        dist_args <- as.list(dist_call[-1])

        name_lpdf <- paste0(dist_name, "_lpdf")
        name_lpmf <- paste0(dist_name, "_lpmf")

        exists_in_pkg <- function(fname) {
          if (length(fname) > 1) return(FALSE)
          requireNamespace("BayesRTMB", quietly = TRUE) &&
            exists(fname, mode = "function", envir = asNamespace("BayesRTMB"), inherits = FALSE)
        }

        use_pkg_namespace <- FALSE

        if (exists_in_pkg(name_lpdf)) {
          actual_name <- name_lpdf
          use_pkg_namespace <- TRUE
        } else if (exists_in_pkg(name_lpmf)) {
          actual_name <- name_lpmf
          use_pkg_namespace <- TRUE
        } else if (exists(name_lpdf, mode = "function", envir = env)) {
          actual_name <- name_lpdf
        } else if (exists(name_lpmf, mode = "function", envir = env)) {
          actual_name <- name_lpmf
        } else {
          actual_name <- name_lpdf # Default
        }

        if (use_pkg_namespace) {
          func_call <- call(":::", as.name("BayesRTMB"), as.name(actual_name))
        } else {
          func_call <- as.name(actual_name)
        }

        new_call <- as.call(c(
          as.name("<-"),
          as.name("lp"),
          as.call(c(
            as.name("+"),
            as.name("lp"),
            as.call(c(func_call, target, dist_args))
          ))
        ))
        return(new_call)
      }
      x[] <- lapply(x, rewrite_formula)
    }
    return(x)
  }

  processed_expr <- rewrite_formula(raw_expr)

  if (is.call(processed_expr) && identical(processed_expr[[1]], as.name("{"))) {
    expr_elements <- as.list(processed_expr)[-1]
  } else {
    expr_elements <- list(processed_expr)
  }

  body_list <- c(
    list(as.name("{")),
    quote(RTMB::getAll(dat, par)),
    quote(lp <- 0),
    unname(expr_elements),
    quote(return(lp))
  )
  body_expr <- as.call(body_list)

  args <- as.pairlist(alist(dat = , par = ))
  fn_expr <- call("function", args, body_expr)

  log_prob_fn <- eval(fn_expr, envir = env)
  attr(log_prob_fn, "raw_expr") <- raw_expr

  return(log_prob_fn)
}


#' Transformed Code Wrapper for RTMB
#'
#' @param expr A block of code containing calculations for transformed parameters.
#' @param env Environment to assign to the generated function.
#' @return A function taking (dat, par) that returns a named list.
transform_code <- function(expr, env = parent.frame()) {
  raw_expr <- substitute(expr)

  raw_expr <- inject_namespace(raw_expr, pkg = "BayesRTMB")

  if (is.call(raw_expr) && identical(raw_expr[[1]], as.name("{"))) {
    expr_elements <- as.list(raw_expr)[-1]
  } else {
    expr_elements <- list(raw_expr)
  }

  if (length(expr_elements) == 0) {
    body_list <- c(
      list(as.name("{")),
      quote(RTMB::getAll(dat, par)),
      list(quote(return(list())))
    )
    body_expr <- as.call(body_list)
    args <- as.pairlist(alist(dat = , par = ))
    fn_expr <- call("function", args, body_expr)
    fn <- eval(fn_expr, envir = env)
    attr(fn, "raw_expr") <- raw_expr
    return(fn)
  }

  last_elem <- expr_elements[[length(expr_elements)]]

  is_explicit_return <- FALSE
  if (is.call(last_elem)) {
    if (identical(last_elem[[1]], as.name("list"))) {
      is_explicit_return <- TRUE
      ret_call <- last_elem
    } else if (identical(last_elem[[1]], as.name("return"))) {
      is_explicit_return <- TRUE
      ret_call <- last_elem[[2]]
    }
  } else if (is.name(last_elem)) {
    is_explicit_return <- TRUE
    ret_call <- last_elem
  }

  # --- report() AST
  reported_vars <- character()

  strip_report <- function(e) {
    if (is.atomic(e) || is.name(e)) return(e)
    if (is.call(e)) {
      if (identical(e[[1]], as.name("{"))) {
        new_args <- list()
        for (i in seq_along(e)[-1]) {
          sub_e <- e[[i]]
          if (is.call(sub_e) && identical(sub_e[[1]], as.name("report"))) {
            if (length(sub_e) > 1 && is.name(sub_e[[2]])) {
              reported_vars <<- c(reported_vars, as.character(sub_e[[2]]))
            }
            next
          }
          new_args <- c(new_args, list(strip_report(sub_e)))
        }
        return(as.call(c(list(as.name("{")), new_args)))
      }
      for (i in seq_along(e)[-1]) {
        e[[i]] <- strip_report(e[[i]])
      }
    }
    return(e)
  }

  raw_expr_stripped <- strip_report(raw_expr)
  reported_vars <- unique(reported_vars)

  if (is.call(raw_expr_stripped) && identical(raw_expr_stripped[[1]], as.name("{"))) {
    expr_elements <- as.list(raw_expr_stripped)[-1]
  } else {
    expr_elements <- if (is.null(raw_expr_stripped)) list() else list(raw_expr_stripped)
  }

  if (is_explicit_return) {
    expr_elements <- expr_elements[-length(expr_elements)]
  } else {
    # report()
    if (length(reported_vars) > 0) {
      defined_vars <- reported_vars
    } else {
      find_assignments <- function(x) {
        if (is.call(x)) {
          if (identical(x[[1]], as.name("<-")) || identical(x[[1]], as.name("="))) {
            if (is.name(x[[2]])) return(as.character(x[[2]]))
          }
          return(unique(unlist(lapply(as.list(x), find_assignments))))
        }
        return(NULL)
      }
      defined_vars <- find_assignments(raw_expr)
    }

    ret_list_args <- lapply(defined_vars, as.name)
    names(ret_list_args) <- defined_vars
    ret_call <- as.call(c(list(as.name("list")), ret_list_args))
  }

  body_list <- c(
    list(as.name("{")),
    quote(RTMB::getAll(dat, par)),
    unname(expr_elements),
    list(as.call(list(as.name("return"), ret_call)))
  )
  body_expr <- as.call(body_list)

  args <- as.pairlist(alist(dat = , par = ))
  fn_expr <- call("function", args, body_expr)
  fn <- eval(fn_expr, envir = env)
  attr(fn, "raw_expr") <- raw_expr

  return(fn)
}

#' Code block for parameter definitions
#'
#' @description
#' Defines the list of parameters to be passed to `rtmb_model`.
#' By writing in the format `variable_name = Dim(...)` within a block enclosed in `{}`,
#' evaluation is delayed, enabling strict error checking during model construction.
#'
#' @param expr A parameter definition expression enclosed in `{}`.
#' @return A lazily evaluated list of parameters.
parameters_code <- function(expr) {
  ast <- substitute(expr)

  if (!is.call(ast) || !identical(ast[[1]], as.name("{"))) {
    stop("parameters_code must receive a block enclosed in {}.\nExample: parameters_code({ beta = Dim(K) })", call. = FALSE)
  }

  exprs <- as.list(ast)[-1]
  param_list <- list()

  for (e in exprs) {
    if (is.null(e)) next

    if (is.call(e) && (identical(e[[1]], as.name("=")) || identical(e[[1]], as.name("<-")))) {
      var_name <- as.character(e[[2]])
      param_list[[var_name]] <- e[[3]]
    } else {
      stop(sprintf("Invalid syntax in parameters_code: '%s'\nPlease describe in the format 'variable_name = Dim(...)'.",
                   paste(deparse(e), collapse = " ")), call. = FALSE)
    }
  }

  return(param_list)
}


#' Pre-validation of data and parameters
#'
#' @description
#' Before passing data to `rtmb_model`, it checks for the presence of R-specific data types (such as data.frame)
#' and missing values, and outputs errors or warnings accordingly.
#'
#' @param dat_list A list of data to be passed to the model (usually containing matrices or numeric vectors).
#'
#' @return Returns invisible `NULL` on success. Interrupts execution with `stop()` or issues `warning()` if issues are found.
#'
#' @details
#' RTMB's automatic differentiation engine requires pure `matrix` or `numeric` types.
#' If a user mistakenly passes a `data.frame`, incomprehensible errors occur during the construction of the computation graph.
#' This function catches such issues in advance.
validate_data <- function(dat_list) {
  for (name in names(dat_list)) {
    if (inherits(dat_list[[name]], "rtmb_setup_df")) next
    if (is.data.frame(dat_list[[name]])) {
      dat_list[[name]] <- as.matrix(dat_list[[name]])
    }
  }
  for (name in names(dat_list)) {
    x <- dat_list[[name]]
    if (inherits(x, "formula") || is.list(x) || is.function(x)) next
  }
  return(dat_list)
}


#' Safe RTMB model construction (with error message translation)
#'
#' @description
#' Wraps `rtmb_model` to translate cryptic error messages generated during `MakeADFun`
#' execution (originating from C++/RTMB) into user-friendly hints.
#'
#' @param data A list of data that has passed `validate_data`.
#' @param code Code blocks for likelihood and priors defined with `model_code({})`.
#' @param par_names A list of variable names corresponding to the dimensions of each parameter (optional).
#' @param init A list or numeric vector of initial values (optional).
#' @param view Character vector of parameter names to be displayed preferentially in summary outputs (optional).

#'
#' @return Returns an `rtmb_model` object (R6 class instance) upon successful compilation.
#'
#' @export
safe_rtmb_model <- function(data, code, par_names = list(), init = NULL, view = NULL) {
  data <- validate_data(data)

  tryCatch({
    obj <- rtmb_model(data = data, code = code, par_names = par_names,
                      init = init, view = view)
    return(obj)

  }, error = function(e) {
    err_msg <- conditionMessage(e)

    if (grepl("lost class attribute", err_msg) || grepl("Invalid argument to 'advector'", err_msg)) {
      stop("[Model construction error] Automatic differentiation (AD) type was lost during execution.\n",
           "  * Cause: You may be performing sequential assignments within a 'for' loop to an empty 'array()' or using 'rep()' inside 'model_code'.\n",
           "  * Solution: Use vectorized matrix calculations, or use 'numeric(M)' etc. for initialization.\n",
           "[Original error]: ", err_msg, call. = FALSE)
    }

    if (grepl("illegal operation", err_msg) || grepl("not a valid 'advector'", err_msg)) {
      stop("[Model construction error] An invalid operation or type mismatch occurred.\n",
           "  * Cause: Dimensions may not match around matrix multiplication (%*%), or type specification in Dim() (e.g., CF_cov, lower_tri) may contradict the actual matrix size.\n",
           "[Original error]: ", err_msg, call. = FALSE)
    }

    if (grepl("requires numeric/complex matrix/vector", err_msg)) {
      stop("[Data type error] A different type was passed where a numeric vector or matrix is required.\n",
           "  * Cause: Are observation data or indices passed as data.frame or list?\n",
           "[Original error]: ", err_msg, call. = FALSE)
    }

    stop("[Unexpected error]\n", err_msg, call. = FALSE)
  })
}

#' RTMB-based Factor Analysis Wrapper
#'
#' @description
#' Performs exploratory factor analysis (EFA) using RTMB. It supports standard
#' rotation methods (e.g., varimax, promax) as well as regularized factor analysis
#' using a Spike-and-Slab Prior (SSP) for estimating sparse loading matrices.
#'
#' @param data Observation data frame or matrix (N x J).
#' @param nfactors Number of factors (K).
#' @param rotate String specifying the rotation method (e.g., "varimax", "promax", "ssp"). If NULL, no rotation is applied. Specifying "ssp" performs regularized factor analysis.
#' @param score Logical; if TRUE, factor scores are calculated in the generate block (default is FALSE).
#' @param prior Prior configuration: `prior_flat()`, `prior_normal()`, or `prior_weak()`.
#'   Hyperparameters can be specified within these functions (e.g., `prior_normal(mean_sd = 10, sd_rate = 10)`).
#'   Available parameters for FA: `mean_sd`, `sd_rate`, `loadings_sd`, and `ssp_ratio` (if `rotate = "ssp"`).
#' @param y_range A numeric vector of length 2 specifying the theoretical min and max values of the items.
#'   Used to construct weakly informative priors when `prior = prior_weak()`.
#' @param init List of initial values.
#' @param fixed A named list of parameter values to fix (optional).
#' @param missing Missing value handling strategy: "listwise" (default) or "fiml" (Full Information Maximum Likelihood).
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @example inst/examples/ex_fa.R
#' @export
rtmb_fa <- function(data, nfactors = 1, rotate = NULL, score = FALSE,
                    prior = prior_flat(),
                    y_range = NULL,
                    init = NULL, fixed = NULL,
                    missing = c("listwise", "fiml"), WAIC = FALSE) {
  missing <- match.arg(missing)
  if (isTRUE(WAIC) && missing != "listwise") {
    stop("WAIC = TRUE is currently supported for rtmb_fa() only with missing = 'listwise'.", call. = FALSE)
  }
  if (!is.numeric(nfactors) || length(nfactors) != 1L || is.na(nfactors) ||
      nfactors < 1 || nfactors != as.integer(nfactors)) {
    stop("'nfactors' must be a positive integer.", call. = FALSE)
  }

  if (is.data.frame(data)) {
    if (!all(sapply(data, is.numeric))) {
      stop("All variables in the data must be numeric. Character or factor variables are not supported.", call. = FALSE)
    }
  } else if (!is.numeric(data) && !is.logical(data)) {
    stop("The data matrix must be numeric.", call. = FALSE)
  }
  Y <- as.matrix(data)
  if (missing == "listwise") {
    Y <- na.omit(Y)
  }
  K <- as.integer(nfactors)
  J <- ncol(Y)
  N <- nrow(Y)

  if (is.null(J) || J < 2L) {
    stop("Factor analysis requires at least two observed variables.", call. = FALSE)
  }
  if (is.null(N) || N < 2L) {
    stop("Factor analysis requires at least two complete observations.", call. = FALSE)
  }
  if (K >= J) stop("The number of factors (K) must be less than the number of observed variables (J).")

  var_names <- colnames(data)
  if (is.null(var_names)) var_names <- paste0("V", 1:J)
  factor_names <- paste0("Factor", 1:K)
  colnames(Y) <- var_names

  # --- 1. Pre-calculate Sufficient Statistics in R (Handling NAs) ---
  if (any(is.na(Y))) {
    Y_bar <- apply(Y, 2, mean, na.rm = TRUE)
    S_Y <- cov(Y, use = "pairwise.complete.obs") * (N - 1)
  } else {
    Y_bar <- apply(Y, 2, mean)
    S_Y <- cov(Y) * (N - 1)
  }

  # --- 2. Prior Handling (Extracting from prior object) ---
  if (is.null(prior)) prior <- prior_flat()

  # Automatically switch to prior_weak() if y_range is provided and prior is default flat
  if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }

  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), or prior_weak().",
      call. = FALSE
    )
  }

  prior_type <- prior$type
  
  # Error handling: if weak is requested but y_range is missing
  if (prior_type == "weak" && is.null(y_range)) {
    stop("When using 'prior_weak()', you must specify 'y_range' (e.g., y_range = c(1, 5)) to define the scaling of the priors.", call. = FALSE)
  }

  # Default/Initial settings
  prior_mean_center <- 0
  prior_mean_sd <- if (prior_type == "normal") prior$mu_sd else prior$mean_sd
  prior_sd_rate <- if (prior_type == "normal") prior$sigma_rate else prior$sd_rate
  prior_loadings_sd <- if (prior_type == "normal") prior$b_sd else prior$loadings_sd
  ssp_ratio <- if (!is.null(prior$ssp_ratio)) prior$ssp_ratio else 0.25

  # Weak Information Prior Logic (GLMER style)
  if (prior_type == "weak") {
    sd_ratio <- if (!is.null(prior$sd_ratio)) prior$sd_ratio else 0.5
    half_d_y <- diff(y_range) / 2
    base_scale <- half_d_y * sd_ratio
    
    prior_mean_center <- mean(y_range)
    prior_mean_sd <- half_d_y
    prior_sd_rate <- 1.0 / base_scale
    prior_loadings_sd <- base_scale
  }

  dat_fa <- list(
    Y = Y,
    nfactors = K,
    prior_mean_center = prior_mean_center,
    prior_mean_sd = prior_mean_sd,
    prior_sd_rate = prior_sd_rate,
    prior_loadings_sd = prior_loadings_sd
  )

  # Determine if SSP model is used
  is_ssp <- !is.null(rotate) && rotate == "ssp"

  # --- 3. Simplified Setup AST ---
  if (missing == "listwise") {
    setup_ast <- quote({
      N <- nrow(Y)
      J <- ncol(Y)
      K <- nfactors
      Y_bar <- colMeans(Y)
      S_Y <- cov(Y) * (N - 1)
    })
  } else {
    setup_ast <- quote({
      N <- nrow(Y)
      J <- ncol(Y)
      K <- nfactors
    })
  }

  if (is_ssp) {
    dat_fa$ssp_ratio <- ssp_ratio
    if (score || isTRUE(WAIC)) dat_fa$Y <- Y

    # Automatic generation of initial values for SSP
    if (is.null(init)) {
      tryCatch({
        eig <- eigen(S_Y / (N - 1))
        L_pca <- eig$vectors[, 1:K, drop = FALSE] %*% diag(sqrt(pmax(eig$values[1:K], 0.01)), nrow = K, ncol = K)
        var_Y <- diag(S_Y / (N - 1))

        if (K > 1) {
          pm <- stats::promax(L_pca, m = 4)
          init_Lambda <- unclass(pm$loadings)
          T_mat <- pm$rotmat
          Phi <- cov2cor(solve(t(T_mat) %*% T_mat))
          init_CF_Omega <- t(chol(Phi))
        } else {
          init_Lambda <- L_pca
          init_CF_Omega <- matrix(1, nrow = 1, ncol = 1)
          Phi <- matrix(1, nrow = 1, ncol = 1)
        }
        h2_raw <- rowSums(init_Lambda * (init_Lambda %*% Phi))
        init_sd <- sqrt(pmax(var_Y - h2_raw, 0.01 * var_Y))
        L_std <- init_Lambda / sqrt(var_Y)
        init_r <- ifelse(abs(L_std) > 0.2, 0.9, 0.1)

        init <- list(mean = Y_bar, Lambda_star = init_Lambda, sd = init_sd, CF_Omega = init_CF_Omega, r = init_r, tau = matrix(1.0, nrow = J, ncol = K))
      }, error = function(e) { init <- NULL })
    }

    param_ast <- quote({
      mean = Dim(dim = J)
      Lambda_star = Dim(c(J, K))
      r = Dim(c(J, K), lower = 0.001, upper = 0.999)
      tau = Dim(c(J, K), lower = 0)
      sd = Dim(J, lower = 0)
      CF_Omega = Dim(c(K, K), type = "CF_corr")
    })

    tran_ast <- quote({
      Lambda <- Lambda_star * r * tau
      h2 <- rowSums(Lambda * (Lambda %*% CF_Omega))
      var_Y <- h2 + sd^2
      sd_Y <- sqrt(var_Y)
      L <- Lambda / sd_Y
      fa_cor <- CF_Omega %*% t(CF_Omega)
    })

    model_exprs <- list()
    # In SSP case, Lambda is a transformed variable. 
    # S_Y ~ sufficient_multi_normal_fa(N, Y_bar, mean, sd, Lambda %*% CF_Omega)
    # Note: Lambda here is the loadings *before* correlation matrix rotation.
    if (missing == "listwise") {
      model_exprs[[1]] <- quote(S_Y ~ sufficient_multi_normal_fa(N, Y_bar, mean, sd, Lambda %*% CF_Omega))
    } else {
      model_exprs[[1]] <- quote(Y ~ fiml_multi_normal_fa(mean, Lambda %*% CF_Omega, sd))
    }
    if (prior_type == "normal") {
       if (!is.null(prior_mean_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(mean ~ normal(0, .(prior_mean_sd)))
       if (!is.null(prior_sd_rate)) model_exprs[[length(model_exprs) + 1]] <- bquote(sd ~ exponential(.(prior_sd_rate)))
    } else if (prior_type == "weak") {
       if (!is.null(prior_mean_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(mean ~ normal(prior_mean_center, prior_mean_sd))
       if (!is.null(prior_sd_rate)) model_exprs[[length(model_exprs) + 1]] <- bquote(sd ~ exponential(prior_sd_rate))
    }
    model_exprs[[length(model_exprs) + 1]] <- quote(CF_Omega ~ lkj_CF_corr(1))
    model_exprs[[length(model_exprs) + 1]] <- bquote(mu_r <- log(ssp_ratio / (1 - ssp_ratio)))
    model_exprs[[length(model_exprs) + 1]] <- quote(r ~ logit_normal(mu_r, 3))
    model_exprs[[length(model_exprs) + 1]] <- quote(tau ~ exponential(1))
    model_exprs[[length(model_exprs) + 1]] <- quote(Lambda_star ~ laplace(0, 1))
    model_ast <- as.call(c(list(as.name("{")), model_exprs))

    base_gq <- quote({
      Sigma <- Lambda %*% fa_cor %*% t(Lambda) + diag(sd^2)
      var_total <- diag(Sigma)
      var_common <- rowSums(Lambda * (Lambda %*% CF_Omega))
      communality <- var_common / var_total
      out <- list(communality = communality)
    })
    waic_expr <- if (isTRUE(WAIC)) {
      quote({ out$log_lik <- multi_normal_lpdf(Y, mean, Sigma, sum = FALSE) })
    } else {
      quote({})
    }
    score_expr <- if (score) {
      quote({
        # Use explicit matrix creation to ensure AD type is preserved
        Y_c <- Y - matrix(mean, nrow = N, ncol = J, byrow = TRUE)
        out$score <- Y_c %*% solve(Sigma, Lambda %*% fa_cor)
      })
    } else quote({})
    ret_expr <- quote({ return(out) })
    gq_ast <- as.call(c(list(as.name("{")), as.list(base_gq)[-1], as.list(waic_expr)[-1], as.list(score_expr)[-1], as.list(ret_expr)[-1]))
    
    code_obj <- list(setup = setup_ast, parameters = param_ast, transform = tran_ast, model = model_ast, generate = gq_ast, env = parent.frame())
    p_names <- list(
      mean = var_names,
      Lambda_star = list(var_names, factor_names),
      r = list(var_names, factor_names),
      tau = list(var_names, factor_names),
      sd = var_names,
      CF_Omega = factor_names,
      Lambda = list(var_names, factor_names),
      L = list(var_names, factor_names),
      fa_cor = factor_names,
      communality = var_names
    )
    if (score) {
      ind_names <- rownames(data); if (is.null(ind_names)) ind_names <- paste0("Id", 1:N)
      p_names[["score"]] <- list(ind_names, factor_names)
    }
    obj <- rtmb_model(data = dat_fa, code = code_obj, par_names = p_names, init = init, view = c("L", "sd", "fa_cor"), fixed = fixed)
    obj$type <- "fa"
    obj$extra <- list(
      source = "wrapper",
      prior_type = if (is_ssp) "ssp" else prior$type,
      marginal = "mean"
    )
    return(obj)

  } else {
    # --- Standard rotation logic ---
    if (score || isTRUE(WAIC)) dat_fa$Y <- Y
    if (is.null(init)) {
      tryCatch({
        eig <- eigen(S_Y / (N - 1))
        L_pca <- eig$vectors[, 1:K, drop = FALSE] %*% diag(sqrt(pmax(eig$values[1:K], 0.01)), nrow=K, ncol=K)
        L_top <- L_pca[1:K, , drop = FALSE]; qr_res <- qr(t(L_top)); Q_mat <- qr.Q(qr_res)
        init_loadings <- L_pca %*% Q_mat
        init_sd <- pmax(diag(S_Y / (N - 1)) - rowSums(init_loadings^2), 0.01)^0.5
        for (j in 1:J) for (k in 1:K) if (j < k) init_loadings[j, k] <- 0
        init <- list(mean = Y_bar, L_raw = init_loadings, sd = init_sd)
      }, error = function(e) { init <- NULL })
    }

    param_ast <- quote({
      mean <- Dim(dim = J)
      L_raw <- Dim(dim = c(J, K), type = "lower_tri")
      sd <- Dim(dim = J, lower = 0)
    })

    tran_ast <- quote({
      h2 <- rowSums(L_raw^2)
      var_Y <- h2 + sd^2
      sd_Y <- sqrt(var_Y)
      L <- L_raw/sd_Y
    })

    model_exprs <- list()
    # Standard FA case: L_raw is a parameter (lower triangular matrix)
    if (missing == "listwise") {
      model_exprs[[1]] <- quote(S_Y ~ sufficient_multi_normal_fa(N, Y_bar, mean, sd, L_raw))
    } else {
      model_exprs[[1]] <- quote(Y ~ fiml_multi_normal_fa(mean, L_raw, sd))
    }
    if (prior_type == "normal") {
       if (!is.null(prior_mean_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(mean ~ normal(0, .(prior_mean_sd)))
       if (!is.null(prior_sd_rate)) model_exprs[[length(model_exprs) + 1]] <- bquote(sd ~ exponential(.(prior_sd_rate)))
       if (!is.null(prior_loadings_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(L_raw ~ lower_tri_normal(0, .(prior_loadings_sd)))
    } else if (prior_type == "weak") {
       if (!is.null(prior_mean_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(mean ~ normal(0, prior_mean_sd))
       if (!is.null(prior_sd_rate)) model_exprs[[length(model_exprs) + 1]] <- bquote(sd ~ exponential(prior_sd_rate))
       if (!is.null(prior_loadings_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(L_raw ~ lower_tri_normal(0, prior_loadings_sd))
    }
    model_ast <- as.call(c(list(as.name("{")), model_exprs))

    base_gq <- quote({
      Sigma <- L_raw %*% t(L_raw) + diag(sd^2)
      var_total <- diag(Sigma); var_common <- rowSums(L_raw^2); communality <- var_common / var_total
      out <- list(communality = communality)
    })
    waic_expr <- if (isTRUE(WAIC)) {
      quote({ out$log_lik <- multi_normal_lpdf(Y, mean, Sigma, sum = FALSE) })
    } else {
      quote({})
    }

    if (!is.null(rotate)) {
      rot_loadings_name <- paste0("L_", rotate)
      if (exists(rotate, mode = "function")) { rot_fn <- match.fun(rotate); fn_call <- as.name(rotate) }
      else if (requireNamespace("GPArotation", quietly = TRUE) && exists(rotate, where = asNamespace("GPArotation"), mode = "function")) {
        rot_fn <- getFromNamespace(rotate, "GPArotation"); fn_call <- call("::", as.name("GPArotation"), as.name(rotate))
      } else stop("Rotation function not found: ", rotate)
      
      dummy_L <- matrix(rnorm(J * K), J, K); test_rot <- rot_fn(dummy_L)
      is_matrix_rot <- is.matrix(test_rot); has_phi <- !is_matrix_rot && !is.null(test_rot$Phi)
      if (is_matrix_rot) {
        rot_expr <- bquote({
          rot_obj <- .(fn_call)(L)
          out[[.(rot_loadings_name)]] <- unclass(rot_obj)
          dimnames(out[[.(rot_loadings_name)]]) <- list(.(var_names), .(factor_names))
          class(out[[.(rot_loadings_name)]]) <- c("rtmb_estimate_matrix", "matrix", "array")
        })
        score_expr <- if (score) bquote({
          Y_c <- Y - matrix(mean, nrow = N, ncol = J, byrow = TRUE)
          rot_raw <- unclass(.(fn_call)(L_raw)); if (!is.matrix(rot_raw)) rot_raw <- unclass(rot_raw$loadings)
          out$score <- Y_c %*% solve(Sigma, rot_raw)
        }) else quote({})
      } else {
        if (has_phi) {
          rot_expr <- bquote({
            rot_obj <- .(fn_call)(L)
            out$fa_cor <- rot_obj$Phi
            out[[.(rot_loadings_name)]] <- unclass(rot_obj$loadings)
            dimnames(out[[.(rot_loadings_name)]]) <- list(.(var_names), .(factor_names))
            class(out[[.(rot_loadings_name)]]) <- c("rtmb_estimate_matrix", "matrix", "array")
          })
          score_expr <- if (score) bquote({
            Y_c <- Y - matrix(mean, nrow = N, ncol = J, byrow = TRUE)
            rot_raw_obj <- .(fn_call)(L_raw); out$score <- Y_c %*% solve(Sigma, unclass(rot_raw_obj$loadings) %*% rot_raw_obj$Phi)
          }) else quote({})
        } else {
          rot_expr <- bquote({
            rot_obj <- .(fn_call)(L)
            out[[.(rot_loadings_name)]] <- unclass(rot_obj$loadings)
            dimnames(out[[.(rot_loadings_name)]]) <- list(.(var_names), .(factor_names))
            class(out[[.(rot_loadings_name)]]) <- c("rtmb_estimate_matrix", "matrix", "array")
          })
          score_expr <- if (score) bquote({
            Y_c <- Y - matrix(mean, nrow = N, ncol = J, byrow = TRUE)
            out$score <- Y_c %*% solve(Sigma, unclass(.(fn_call)(L_raw)$loadings))
          }) else quote({})
        }
      }
    } else { has_phi <- FALSE; rot_expr <- quote({}); score_expr <- if (score) quote({ Y_c <- Y - matrix(mean, nrow = N, ncol = J, byrow = TRUE); out$score <- Y_c %*% solve(Sigma, L_raw) }) else quote({}) }

    ret_expr <- quote({ return(out) })
    gq_ast <- as.call(c(list(as.name("{")), as.list(base_gq)[-1], as.list(waic_expr)[-1], as.list(rot_expr)[-1], as.list(score_expr)[-1], as.list(ret_expr)[-1]))
    
    code_obj <- list(setup = setup_ast, parameters = param_ast, transform = tran_ast, model = model_ast, generate = gq_ast, env = parent.frame())
    p_names <- list(
      mean = var_names,
      L_raw = list(var_names, factor_names),
      sd = var_names,
      L = list(var_names, factor_names),
      communality = var_names
    )
    if (!is.null(rotate)) {
      p_names[[paste0("L_", rotate)]] <- list(var_names, factor_names)
      if (has_phi) p_names[["fa_cor"]] <- factor_names
    }
    if (score) { ind_names <- rownames(data); if (is.null(ind_names)) ind_names <- paste0("Id", 1:N); p_names[["score"]] <- list(ind_names, factor_names) }

    target_view <- if (!is.null(rotate)) c(paste0("L_", rotate), "sd", "fa_cor") else c("L", "sd", "fa_cor")
    obj <- rtmb_model(data = dat_fa, code = code_obj, par_names = p_names, init = init, view = target_view, fixed = fixed)
    obj$type <- "fa"
    obj$extra <- list(
      source = "wrapper",
      prior_type = prior$type,
      marginal = "mean"
    )
    return(obj)
  }
}

.calculate_satterthwaite_df_impl <- function(self, private, ad_obj, idx_fix_active = NULL, L_u_total = NULL, opt_par = NULL, max_df = NULL, silent = FALSE, return_sensitivities = FALSE) {
  # if (!silent) cat("Estimating Satterthwaite degrees of freedom...\n")

  par <- if (!is.null(opt_par)) opt_par else ad_obj$par
  P <- length(par)
  total_len <- if (!is.null(L_u_total)) L_u_total else P
  df_full <- rep(Inf, total_len)

  # Step 1: Compute Hessian at optimum via Jacobian of analytical gradient
  # For marginal likelihood (Laplace), numerical differentiation on the gradient is most reliable

  H0 <- tryCatch(private$.simple_jacobian(ad_obj$gr, par), warning = function(w) NULL, error = function(e) NULL)

  if (is.null(H0) || any(!is.finite(H0))) {
    if (!silent) cat("SE warning: Hessian computation failed in Satterthwaite calculation. Using normal approximation.\n")
    return(df_full)
  }
  V <- tryCatch(solve(H0), error = function(e) {
    if (!silent) cat("SE warning: Hessian matrix was singular in Satterthwaite calculation; using MASS::ginv().\n")
    tryCatch(MASS::ginv(H0), error = function(e2) NULL)
  })
  if (is.null(V)) {
    if (!silent) cat("SE warning: Satterthwaite covariance calculation failed. Using normal approximation.\n")
    return(df_full)
  }

  # Step 2: Compute dV_ii/dtheta_k via central differences of the Hessian
  # Optimized: Compute dH/dtheta_k once per k and vectorize diag(V dH V)
  # Improved: Smaller eps base for better sensitivity to 3rd derivatives
  # Improved: Slightly larger eps base to smooth out 3rd derivative noise
  eps <- 1e-3 * pmax(abs(par), 0.1)
  grad_V_diag <- matrix(0, nrow = P, ncol = P) # grad_V_diag[k, i] = dV_ii / dtheta_k
  dH_list <- if (return_sensitivities) vector("list", P) else NULL

  for (k in 1:P) {
    par_plus <- par_minus <- par
    par_plus[k] <- par[k] + eps[k]
    par_minus[k] <- par[k] - eps[k]

    H_plus <- tryCatch(private$.simple_jacobian(ad_obj$gr, par_plus), warning = function(w) NULL, error = function(e) NULL)
    H_minus <- tryCatch(private$.simple_jacobian(ad_obj$gr, par_minus), warning = function(w) NULL, error = function(e) NULL)

    if (is.null(H_plus) || is.null(H_minus)) next

    # Third derivative approximation: dH/dtheta_k
    dH_k <- (H_plus - H_minus) / (2 * eps[k])
    if (return_sensitivities) dH_list[[k]] <- dH_k

    # dV/dtheta_k = -V %*% dH_k %*% V
    # Diagonal elements: diag(dV/dtheta_k) = -rowSums((V %*% dH_k) * V)
    grad_V_diag[k, ] <- -rowSums((V %*% dH_k) * V)
  }

  rel_tol <- 1e-10
  df_par <- rep(Inf, P)
  for (i in 1:P) {
    grad_vi <- grad_V_diag[, i]
    if (all(abs(grad_vi) < 1e-30)) next

    # Var(V_ii) = grad(V_ii)^T %*% V %*% grad(V_ii)
    var_vi <- as.numeric(t(grad_vi) %*% V %*% grad_vi)
    
    # Numerical stability: If variance of V_ii is extremely small relative to V_ii^2, 
    # it means sensitivity is negligible, so DF should be Infinite.
    if (!is.na(var_vi) && is.finite(var_vi) && var_vi > rel_tol * (V[i, i]^2) && V[i, i] > 0) {
      df_par[i] <- 2 * (V[i, i]^2) / var_vi
    } else {
      df_par[i] <- Inf
    }
  }

  if (!is.null(max_df)) {
    df_par[df_par > max_df] <- Inf
  }
  df_par[!is.finite(df_par)] <- Inf
  df_par <- pmax(df_par, 2.1)

  # Map to full parameter vector
  if (!is.null(idx_fix_active) && length(idx_fix_active) == P) {
    df_full[idx_fix_active] <- df_par
  } else if (P == total_len) {
    df_full <- df_par
  }

  finite_dfs <- df_par[is.finite(df_par)]

  if (return_sensitivities) {
    return(list(df = df_full, sensitivities = dH_list, V = V))
  }
  return(df_full)
}

.calculate_reml_satterthwaite_df_impl <- function(self, private, ad_obj, opt_par, beta_idx, max_df = NULL, silent = FALSE, return_sensitivities = FALSE) {
  if (!silent) cat("Estimating Satterthwaite degrees of freedom for fixed effects (REML)...\n")

  P <- length(opt_par)
  n_beta <- length(beta_idx)

  # Helper to get covariance of joint Hessian for beta
  get_beta_cov = function(theta) {
    p_full <- ad_obj$env$last.par.best
    idx_fixed <- ad_obj$env$lfixed()
    p_full[idx_fixed] <- theta

    H <- tryCatch(ad_obj$env$spHess(p_full, random = TRUE), warning = function(w) NULL, error = function(e) NULL)
    if (is.null(H)) {
      return(NULL)
    }

    # Convert to dense matrix for solve to ensure extraction works reliably
    V_full <- tryCatch(solve(as.matrix(H)), warning = function(w) NULL, error = function(e) NULL)
    if (is.null(V_full)) {
      return(NULL)
    }

    as.matrix(V_full[beta_idx, beta_idx, drop = FALSE])
  }

  # Early return helper to ensure consistent return types
  fallback_reml_result <- function() {
    if (return_sensitivities) {
      return(list(
        df = rep(Inf, n_beta),
        se = rep(NA_real_, n_beta),
        sensitivities = vector("list", P), # Keep list structure
        V_theta = NULL,
        V_beta = NULL
      ))
    }
    list(
      df = rep(Inf, n_beta),
      se = rep(NA_real_, n_beta)
    )
  }

  V_beta_0 <- tryCatch(get_beta_cov(opt_par), error = function(e) {
    return(NULL)
  })
  if (is.null(V_beta_0)) {
    return(fallback_reml_result())
  }
  V_beta_diag_0 <- diag(V_beta_0)

  # Covariance of variance components (theta)
  H_theta <- tryCatch(private$.simple_jacobian(ad_obj$gr, opt_par), warning = function(w) NULL, error = function(e) NULL)
  if (is.null(H_theta)) {
    return(fallback_reml_result())
  }

  V_theta <- tryCatch(solve(H_theta), error = function(e) {
    if (!silent) cat("SE warning: Hessian matrix was singular in REML Satterthwaite calculation; using MASS::ginv().\n")
    tryCatch(MASS::ginv(H_theta), error = function(e2) NULL)
  })
  if (is.null(V_theta)) {
    if (!silent) cat("SE warning: REML Satterthwaite covariance calculation failed. Using infinite degrees of freedom.\n")
    return(fallback_reml_result())
  }

  # Gradient of V_beta with respect to theta
  eps <- 1e-5 * pmax(abs(opt_par), 0.1)
  grad_V_beta_diag <- matrix(0, nrow = P, ncol = n_beta)
  dV_beta_list <- if (return_sensitivities) vector("list", P) else NULL

  for (k in 1:P) {
    p_plus <- p_minus <- opt_par
    p_plus[k] <- opt_par[k] + eps[k]
    p_minus[k] <- opt_par[k] - eps[k]

    v_plus <- tryCatch(get_beta_cov(p_plus), warning = function(w) NULL, error = function(e) NULL)
    v_minus <- tryCatch(get_beta_cov(p_minus), warning = function(w) NULL, error = function(e) NULL)

    if (is.null(v_plus) || is.null(v_minus)) next
    dV_k <- (v_plus - v_minus) / (2 * eps[k])
    if (return_sensitivities) dV_beta_list[[k]] <- dV_k
    grad_V_beta_diag[k, ] <- diag(dV_k)
  }

  dfs <- rep(Inf, n_beta)
  rel_tol <- 1e-10
  for (i in 1:n_beta) {
    grad_vi <- grad_V_beta_diag[, i]
    var_vi <- as.numeric(t(grad_vi) %*% V_theta %*% grad_vi)
    v_ii <- V_beta_diag_0[i]
    
    if (!is.na(var_vi) && is.finite(var_vi) && var_vi > rel_tol * (v_ii^2) && v_ii > 0) {
      dfs[i] <- 2 * (v_ii^2) / var_vi
    } else {
      dfs[i] <- Inf
    }
  }

  if (!is.null(max_df)) {
    dfs[dfs > max_df] <- Inf
  }
  dfs[!is.finite(dfs)] <- Inf
  dfs <- pmax(dfs, 2.1)
  if (return_sensitivities) {
    return(list(df = dfs, se = sqrt(V_beta_diag_0), sensitivities = dV_beta_list, V_theta = V_theta, V_beta = V_beta_0))
  }
  return(list(df = dfs, se = sqrt(V_beta_diag_0)))
}

  # Simulate rating data for Multidimensional Unfolding (MDU)
  set.seed(123)
  N <- 50  # Number of persons
  M <- 10  # Number of items
  D <- 2   # Number of dimensions
  
  # True person and item coordinates in a 2D space
  theta <- matrix(rnorm(N * D), N, D)
  delta <- matrix(rnorm(M * D), M, D)
  
  # Generate distance-like ratings (smaller means more preferred)
  Y <- matrix(NA, N, M)
  for(i in 1:N) {
    for(j in 1:M) {
      Y[i, j] <- sum((theta[i,] - delta[j,])^2) + rnorm(1, 0, 0.5)
    }
  }

  # Fit a 2-dimensional MDU model
  fit_mdu <- rtmb_mdu(Y, ndim = 2, distance = "squared", method = "rating")
  
  # MAP estimation
  map_mdu <- fit_mdu$optimize()
  map_mdu$summary()
  
  # Note: MDU models have many parameters, so MCMC sampling might take time.
  \donttest{
  # mcmc_mdu <- fit_mdu$sample(sampling = 500, warmup = 500, chains = 2)
  }

  # Simulate 1D mixture data (2 components)
  set.seed(123)
  N <- 100
  group <- rbinom(N, 1, 0.4)
  y <- ifelse(group == 1, rnorm(N, 5, 1), rnorm(N, 0, 1))
  dat <- data.frame(y)

  # Fit a 1D Gaussian mixture model with 2 components
  fit_mix <- rtmb_mixture(y ~ 1, k = 2, data = dat)
  
  # MAP estimation
  map_mix <- fit_mix$optimize()
  map_mix$summary()

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)

## ----eval=FALSE---------------------------------------------------------------
# library(BayesRTMB)
# set.seed(123)
# 
# # Data generation (2 groups)
# y1 <- rnorm(30, mean = 0, sd = 1)
# y2 <- rnorm(30, mean = 1, sd = 1)
# 
# # Method passing vectors directly
# mdl_ttest <- rtmb_ttest(y1, y2)
# 
# # Method using formula format (when you have a dataframe)
# # df <- data.frame(Y = c(y1, y2), group = rep(c("A", "B"), each = 30))
# # mdl_ttest <- rtmb_ttest(Y ~ group, data = df)
# 
# # MCMC estimation
# fit_ttest <- mdl_ttest$sample()
# fit_ttest$summary()

## ----eval=FALSE---------------------------------------------------------------
# # Display the internally generated RTMB code
# mdl_ttest$print_code()

## ----eval=FALSE---------------------------------------------------------------
# # Compare with the model where effect size delta = 0
# bf <- fit_ttest$bayes_factor(fixed = list(delta = 0))
# bf

## ----eval=FALSE---------------------------------------------------------------
# data(debate)
# 
# # Model predicting sat using talk and skill
# mdl_lm <- rtmb_lm(sat ~ talk + skill, data = debate)
# 
# # Quick check via MAP estimation
# fit_lm <- mdl_lm$optimize()
# fit_lm$summary()

## ----eval=FALSE---------------------------------------------------------------
# # Create a model using weakly informative priors (assuming sat ranges from 1 to 5)
# mdl_lm_weak <- rtmb_lm(sat ~ talk + skill,
#   data = debate,
#   use_weak_info = TRUE,
#   y_range = c(1, 5)
# )
# mdl_lm_weak$print_code()

## ----eval=FALSE---------------------------------------------------------------
# # Logistic regression (family = "bernoulli")
# mdl_glm <- rtmb_glm(cond ~ sat + skill,
#   data = debate,
#   family = "bernoulli"
# )
# 
# fit_glm <- mdl_glm$sample()
# fit_glm$summary()

## ----eval=FALSE---------------------------------------------------------------
# # Random intercept model
# mdl_glmer <- rtmb_glmer(sat ~ talk + (1 | group),
#   data = debate
# )
# 
# # When including random effects, optimize() with Laplace approximation is fast
# opt_glmer <- mdl_glmer$optimize(laplace = TRUE)
# opt_glmer$summary()

## ----eval=FALSE---------------------------------------------------------------
# mdl_glmer <-
#   rtmb_glmer(
#     sat ~ talk + perf + skill + cond + (1 | group),
#     data = debate,
#     penalty = "ssp",
#     y_range = c(1, 5)
#   )
# 
# mcmc_glmer <- mdl_glmer$sample(parallel = TRUE)
# mcmc_glmer

## ----eval=FALSE---------------------------------------------------------------
# mcmc_glmer$draws("b[cond]") |> plot_dens()

## ----eval=FALSE---------------------------------------------------------------
# # Linear regression with interaction between talk and cond
# mdl_int <- rtmb_lm(sat ~ talk * perf, data = debate)
# fit_int <- mdl_int$sample()
# 
# # Visualize the interaction effect
# # For continuous moderators, it automatically shows Mean ± 1SD
# ce <- conditional_effects(fit_int, effect = "talk:perf")
# plot(ce)

## ----eval=FALSE---------------------------------------------------------------
# # Calculate simple slopes of 'talk' for each level of 'cond'
# se <- simple_effects(fit_int, effect = "talk:perf")
# print(se)

## ----eval=FALSE---------------------------------------------------------------
# data(BigFive)
# 
# # Correlation between BF1 and BF6
# mdl_corr2 <- rtmb_corr(BigFive[, c("BF1", "BF6")])
# mcmc_corr2 <- mdl_corr2$sample()
# mcmc_corr2$summary()

## ----eval=FALSE---------------------------------------------------------------
# # Compare against the zero-correlation model (fixed = list(corr = 0))
# bf_corr <- mcmc_corr2$bayes_factor(fixed = list(corr = 0))
# bf_corr

## ----eval=FALSE---------------------------------------------------------------
# # Estimate the correlation matrix of 5 variables
# mdl_corr_mat <- rtmb_corr(BigFive[, 1:5])
# opt_corr_mat <- mdl_corr_mat$optimize()
# opt_corr_mat$summary()

## ----eval=FALSE---------------------------------------------------------------
# # 3-factor model, specifying Promax rotation
# mdl_fa <- rtmb_fa(BigFive, nfactors = 5, rotate = "promax")
# 
# opt_fa <- mdl_fa$optimize(se_sampling = TRUE)
# 
# # Check the factor loadings (L), etc.
# opt_fa$summary()

## ----eval=FALSE---------------------------------------------------------------
# opt_fa$generate$L_promax |> sort_loadings()

## ----eval=FALSE---------------------------------------------------------------
# mdl_fa <- rtmb_fa(BigFive, nfactors = 5, rotate = "promax", score = TRUE)
# 
# opt_fa <- mdl_fa$optimize()
# 
# opt_fa$generate$score |> head()

## ----eval=FALSE---------------------------------------------------------------
# mdl_fa <- rtmb_fa(BigFive, nfactors = 5, rotate = "ssp")
# 
# vb_fa <- mdl_fa$variational(iter = 5000, parallel = TRUE)
# vb_fa

## ----eval=FALSE---------------------------------------------------------------
# vb_fa$MAP("L") |> sort_loadings()

## ----eval=FALSE---------------------------------------------------------------
# data(BigFive)
# # Select items 2, 7, 12, 17 (Agreeableness)
# dat_irt <- BigFive[, c("BF2", "BF7", "BF12", "BF17")]
# 
# # Fit a 2PL Graded Response Model (Ordered Logistic)
# mdl_irt <- rtmb_irt(dat_irt, model = "2PL", type = "ordered")
# 
# # Fast estimation using MAP with Laplace approximation for latent traits
# opt_irt <- mdl_irt$optimize(se_sampling = TRUE)
# opt_irt

## ----eval=FALSE---------------------------------------------------------------
# # Calculate item curves
# ic <- item_curve(opt_irt)
# 
# # Plot curves (shows category probabilities for each item)
# plot(ic)

## ----eval=FALSE---------------------------------------------------------------
# # Item Information Function: shows which items provide the most information
# ii <- item_info(opt_irt)
# plot(ii)
# 
# # Test Information Function: total information of the test/scale
# ti <- test_info(opt_irt)
# plot(ti)

  # --- 1. Linear Regression (rtmb_lm) ---
  # Fit a linear regression model using the debate dataset
  data(debate, package = "BayesRTMB")
  fit_lm <- rtmb_lm(sat ~ talk + perf, data = debate)
  map_lm <- fit_lm$optimize()
  map_lm$summary()

  # --- 2. Generalized Linear Model (rtmb_glm) ---
  # Fit a logistic regression model using the debate dataset
  data(debate, package = "BayesRTMB")
  fit_glm <- rtmb_glm(cond ~ talk + sat, data = debate, family = "bernoulli")
  map_glm <- fit_glm$optimize()
  map_glm$summary()

  # --- 3. Generalized Linear Mixed Model (rtmb_glmer) ---
  # Fit a linear mixed-effects model using the debate dataset
  data(debate, package = "BayesRTMB")
  fit_glmer <- rtmb_glmer(talk ~ cond + (1 | group), data = debate, family = "gaussian")

  # MAP estimation using Laplace approximation for random effects
  map_glmer <- fit_glmer$optimize(laplace = TRUE)
  map_glmer$summary()

  # MCMC sampling (chains and iterations reduced for faster execution)
  \donttest{
  mcmc_glmer <- fit_glmer$sample(sampling = 500, warmup = 500, chains = 2)
  mcmc_glmer$summary()
  }

  # --- 4. Linear Mixed Model (rtmb_lmer) ---
  # A convenient wrapper for Gaussian mixed models (identical to rtmb_glmer with family="gaussian")
  fit_lmer <- rtmb_lmer(sat ~ talk + (1 | group), data = debate)
  map_lmer <- fit_lmer$optimize()
  map_lmer$summary()

  # --- 5. Regularized Regression (Variable Selection) ---
  # You can apply regularization to the fixed effects to shrink noise variables towards zero.
  # Use prior = prior_rhs() for the Regularized Horseshoe prior,
  # or prior_ssp() for the Spike-and-Slab prior.
  # Note: When using regularization, you must specify 'y_range' (the theoretical minimum and maximum
  # values of the response variable) to automatically set up the required weakly informative priors.

  # Fit a linear regression using debate predictors with the Horseshoe prior
  fit_rhs <- rtmb_lm(
    sat ~ talk + perf + skill,
    data = debate,
    prior = prior_rhs(),
    y_range = c(1, 5)
  )
  map_rhs <- fit_rhs$optimize()
  # Summarize only the fixed effects (slopes)
  map_rhs$summary("b")

  # Fit a linear regression with the Spike-and-Slab prior
  fit_ssp <- rtmb_lm(
    sat ~ talk + perf + skill,
    data = debate,
    prior = prior_ssp(),
    y_range = c(1, 5)
  )
  map_ssp <- fit_ssp$optimize()
  map_ssp$summary("b")

#' RTMB-based Linear Mixed Model (LMM) wrapper function
#'
#' @param formula Formula
#' @param data Data frame
#' @param laplace Logical; whether to marginalize random effects using Laplace approximation
#' @param prior An object of class `"rtmb_prior"`.
#' Use `prior_flat()` for no prior, `prior_normal()` for default normal/exponential priors,
#' or `prior_weak()`, `prior_rhs()`, `prior_ssp()` for weakly informative or regularized Bayesian inference.
#' Default is `prior_flat()`.
#' @param y_range Theoretical minimum and maximum values of the response variable
#' @param init Initial values

#' @param gmc Character vector of variable names for GMC
#' @param centering Alias for `gmc`.
#' @param cwc List for CWC
#' @param view Character vector of parameter names to prioritize in summary.
#' @param factors Character vector of variable names to be treated as factors.
#' @param contrasts Character string specifying the contrast type ("treatment" or "sum").
#' @param sigma_by Character vector specifying variables to group residual variance by (heteroscedasticity).
#' @param resid_corr Residual correlation structure (e.g., "ar1", "cs", "un", "toep").
#' @param resid_time Variable name for time points in residual correlation.
#' @param resid_group Variable name for grouping in residual correlation.
#' @param within Optional list for wide-to-long conversion. For repeated measures data in wide format,
#' specify the factor names and their levels, e.g., \code{list(Time = 4)} or \code{list(A = 2, B = 3)}.
#' The total number of levels must match the number of columns in \code{cbind()} on the LHS.
#' If omitted and the LHS is \code{cbind()}, the within-factor name is inferred from RHS variables not present in the data.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param missing Missing value handling strategy: "listwise".
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments passed to \code{rtmb_model()}.
#'
#' @return RTMB_Model object
#' @export
#' @example inst/examples/ex_lm.R
rtmb_lmer <- function(formula, data, laplace = TRUE,
                       prior = prior_flat(),
                       y_range = NULL,
                       init = NULL,
                       fixed = NULL,
                       gmc = NULL,
                       centering = NULL,
                       cwc = NULL,
                       view = NULL,
                       sigma_by = NULL,
                       factors = NULL,
                       contrasts = "treatment",
                       resid_corr = NULL,
                       resid_time = NULL,
                       resid_group = NULL,
                       within = NULL,
                       missing = c("listwise", "fiml"),
                       WAIC = FALSE,
                       ...) {
  missing <- match.arg(missing)
  rtmb_glmer(formula = formula, data = data, family = "gaussian",
             laplace = laplace,
             prior = prior,
             y_range = y_range,
             init = init,
             fixed = fixed,
             gmc = gmc,
             centering = centering,
             cwc = cwc,
             view = view,
             sigma_by = sigma_by,
             factors = factors,
             contrasts = contrasts,
             resid_corr = resid_corr,
             resid_time = resid_time,
             resid_group = resid_group,
             within = within,
             missing = missing,
             WAIC = WAIC,
             .force_sum = TRUE)
}

#' RTMB-based GLM wrapper function (no random effects)
#'
#' @param formula Formula
#' @param data Data frame
#' @param family Character string of the distribution family (e.g., "gaussian", "binomial", "poisson")
#' @param prior An object of class `"rtmb_prior"`.
#' Use `prior_flat()` for no prior, `prior_normal()` for default normal/exponential priors,
#' or `prior_weak()`, `prior_rhs()`, `prior_ssp()` for weakly informative or regularized Bayesian inference.
#' Default is `prior_flat()`.
#' @param y_range Theoretical minimum and maximum values of the response variable as a vector c(min, max). Specifying this automatically enables weakly informative priors.
#' @param init List of initial values

#' @param gmc Character vector of variable names for GMC
#' @param centering Alias for `gmc`.
#' @param factors Character vector of variable names to be treated as factors.
#' @param contrasts Character string specifying the contrast type ("treatment" or "sum").
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param view Optional character vector of parameter names to show first in summary.
#' @param missing Missing value handling strategy: "listwise".
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments passed to \code{rtmb_glmer()}.
#' @example inst/examples/ex_lm.R
#' @export
rtmb_glm <- function(formula, data, family = "gaussian",
                       prior = prior_flat(),
                       y_range = NULL,
                       init = NULL, fixed = NULL,
                       gmc = NULL,
                       centering = NULL,
                       view = NULL,
                       factors = NULL,
                       contrasts = "treatment",
                       missing = c("listwise", "fiml"), WAIC = FALSE, ...) {
  missing <- match.arg(missing)
  rtmb_glmer(formula = formula, data = data, family = family,
             laplace = FALSE,
             prior = prior,
             y_range = y_range,
             init = init,
             fixed = fixed,
             gmc = gmc,
             centering = centering,
             view = view,
             factors = factors,
             contrasts = contrasts, missing = missing, WAIC = WAIC, ...)
}

#' RTMB-based Linear Regression wrapper function
#'
#' @param formula Formula (e.g., Y ~ X1 + X2)
#' @param data Data frame
#' @param prior An object of class `"rtmb_prior"`.
#' Use `prior_flat()` for no prior, `prior_normal()` for default normal/exponential priors,
#' or `prior_weak()`, `prior_rhs()`, `prior_ssp()` for weakly informative or regularized Bayesian inference.
#' Default is `prior_flat()`.
#' @param y_range Theoretical minimum and maximum values of the response variable as a vector c(min, max). Specifying this automatically enables weakly informative priors.
#' @param init List of initial values.

#' @param gmc Character vector of variable names for GMC
#' @param centering Alias for `gmc`.
#' @param factors Character vector of variable names to be treated as factors.
#' @param contrasts Character string specifying the contrast type ("treatment" or "sum").
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param view Optional character vector of parameter names to show first in summary.
#' @param missing Missing value handling strategy: "listwise".
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments passed to \code{rtmb_glmer()}.
#' @example inst/examples/ex_lm.R
#' @export
rtmb_lm <- function(formula, data,
                    prior = prior_flat(),
                    y_range = NULL,
                    init = NULL, fixed = NULL,
                    gmc = NULL,
                    centering = NULL,
                    view = NULL,
                    factors = NULL,
                    contrasts = "treatment",
                    missing = c("listwise", "fiml"), WAIC = FALSE, ...) {
  missing <- match.arg(missing)
  rtmb_glmer(formula = formula, data = data, family = "gaussian",
             laplace = FALSE,
             prior = prior,
             y_range = y_range,
             init = init,
             fixed = fixed,
             gmc = gmc,
             centering = centering,
             view = view,
             factors = factors,
             contrasts = contrasts, missing = missing, WAIC = WAIC, ...)
}

test_that("classic FA summary omits test columns", {
  fit <- rtmb_fa(scale(mtcars[, c("mpg", "disp", "hp")]), nfactors = 1)$classic()
  coefs <- fit$summary(max_rows = 100)$coefficients

  expect_false("df" %in% names(coefs))
  expect_false("t value" %in% names(coefs))
  expect_false("z value" %in% names(coefs))
  expect_false("Pr" %in% names(coefs))
})

test_that("classic wrapper scale rows omit test output", {
  fit_lm <- rtmb_lm(mpg ~ wt, data = mtcars)$classic()
  coefs_lm <- fit_lm$summary(max_rows = 100)$coefficients

  expect_true("sigma" %in% rownames(coefs_lm))
  expect_true(is.na(coefs_lm["sigma", "t value"]))
  expect_equal(coefs_lm["sigma", "Pr"], "")

  scale_like <- data.frame(
    Estimate = c(1, 1),
    `Std. Error` = c(0.1, 0.1),
    `Lower 95%` = c(0.8, 0.8),
    `Upper 95%` = c(1.2, 1.2),
    df = c(10, 10),
    check.names = FALSE,
    row.names = c("sigma1", "sd_group")
  )
  fit_mock <- Classic_Fit$new(model = list(type = "mediation"), df_fixed = scale_like, sd_rep = list())
  coefs_mock <- fit_mock$summary(max_rows = 100)$coefficients
  expect_true(all(is.na(coefs_mock[, "t value"])))
  expect_true(all(coefs_mock[, "Pr"] == ""))

  skip_if_not_installed("lme4")
  data("sleepstudy", package = "lme4")

  fit <- rtmb_lmer(Reaction ~ Days + (1 | Subject), data = sleepstudy)$classic()
  coefs <- fit$summary(max_rows = 100)$coefficients
  scale_rows <- grepl("^(sigma|sd)", rownames(coefs))

  expect_true(any(scale_rows))
  expect_true(all(is.na(coefs[scale_rows, "t value"])))
  expect_true(all(coefs[scale_rows, "Pr"] == ""))
})

#' BayesRTMB: Bayesian Modeling with RTMB
#'
#' @description
#' The `BayesRTMB` package provides an intuitive, R-based Bayesian modeling
#' environment using the `RTMB` package as its computational backend.
#' It allows users to perform Maximum A Posteriori (MAP) estimation,
#' Markov Chain Monte Carlo (MCMC) sampling, and Automatic Differentiation
#' Variational Inference (ADVI) for hierarchical and complex non-linear models
#' through a unified and user-friendly interface.
#'
#' @details
#' The standard workflow of the package consists of three main steps:
#'
#' 1. **Model Definition**: Use [rtmb_code()] to define the model structure
#'    (likelihood, prior distributions, transformations, etc.) using standard R syntax.
#' 2. **Model Instantiation**: Pass the data and the generated model code to
#'    [rtmb_model()] to create an `RTMB_Model` R6 object.
#' 3. **Estimation**: Call the appropriate method on the model object:
#'    * `$optimize()`: For MAP estimation and Laplace approximation.
#'    * `$sample()`: For MCMC sampling using the No-U-Turn Sampler (NUTS).
#'    * `$variational()`: For approximate Bayesian inference using ADVI.
#'
#' For more detailed examples and tutorials, please refer to the package vignettes.
#'
#' @seealso
#' Useful links:
#' * [https://github.com/norimune/BayesRTMB](https://github.com/norimune/BayesRTMB)
#' * Report bugs at [https://github.com/norimune/BayesRTMB/issues](https://github.com/norimune/BayesRTMB/issues)
#'
#' @keywords internal
"_PACKAGE"

#' Classic fit object
#'
#' @description
#' An R6 class representing the results of a classical (frequentist) estimation.
#'
#' @field model The `RTMB_Model` object used for estimation.
#' @field fit The result of the estimation (dataframe or lm object).
#' @field par a named list of parameter estimates.
#' @field vcov Variance-covariance matrix of fixed effects.
#' @field se_method Character string specifying the method used for standard errors.
#' @field cluster Character string specifying the cluster variable name, if any.
#' @field robust_type Character string specifying the robust standard error correction type, if used.
#' @field bootstrap_results A matrix containing bootstrap samples, if applicable.
#' @field test_results List of additional test results (e.g., chisq.test).
#' @field view Character vector of parameter names to prioritize in summary.
#' @field par_vec Numeric vector of parameter estimates.
#' @field objective Final objective value.
#' @field log_lik Log-likelihood.
#' @field restricted_log_lik Restricted log-likelihood, if classical estimation used REML-style marginalization.
#' @field rss Residual sum of squares for classical linear models, if available.
#' @field df_residual Residual degrees of freedom for classical linear models, if available.
#' @field convergence Convergence code.
#' @field sd_rep TMB sdreport object.
#' @field df_fixed Dataframe of fixed effects results.
#' @field random_effects Dataframe of random effects results.
#' @field df_transform Dataframe of transformed parameters.
#' @field df_generate Dataframe of generated quantities.
#' @field opt_history Dataframe of optimization history.
#' @field transform List of transformed parameters.
#' @field generate List of generated quantities.
#' @field se_samples List of simulated samples for SE estimation.
#' @field par_unc Numeric vector of unconstrained parameter estimates.
#' @field vcov_unc Variance-covariance matrix of parameters in unconstrained space.
#' @field ci_method Method used for CI estimation.
#' @field laplace Whether Laplace approximation was used.
#' @field map Parameter mapping used.
#' @field df_method Character string specifying the degrees of freedom calculation method.
#' @field idx_fix_active Numeric vector; mapping between active parameters and full unconstrained vector.
#' @field show_df Logical; whether to display degrees of freedom in the summary output.
#' @importFrom stats AIC BIC logLik anova
#' @export
Classic_Fit <- R6::R6Class(
  classname = "Classic_Fit",
  inherit = RTMB_Fit_Base,
  public = list(
    model = NULL,
    fit = NULL,
    par = NULL,
    par_vec = NULL,
    vcov = NULL,

    #' @description Get point estimate for a target parameter.
    #' @param target Target parameter name.
    #' @param ... Additional arguments, ignored for classic fits.
    #' @return Matrix, array, vector, or scalar point estimate.
    get_point_estimate = function(target, ...) {
      if (!is.null(self$par[[target]])) return(self$par[[target]])
      if (!is.null(self$transform[[target]])) return(self$transform[[target]])
      if (!is.null(self$generate[[target]])) return(self$generate[[target]])
      stop("Parameter not found: ", target)
    },
    objective = NULL,
    log_lik = NULL,
    restricted_log_lik = NULL,
    rss = NULL,
    df_residual = NULL,
    convergence = NULL,
    sd_rep = NULL,
    df_fixed = NULL,
    random_effects = NULL,
    df_transform = NULL,
    df_generate = NULL,
    opt_history = NULL,
    transform = NULL,
    generate = NULL,
    se_samples = NULL,
    par_unc = NULL,
    vcov_unc = NULL,
    bootstrap_results = NULL,
    test_results = list(),
    se_method = "wald",
    ci_method = "wald",
    laplace = TRUE,
    map = NULL,
    cluster = NULL,
    view = NULL,
    df_method = "auto",
    idx_fix_active = NULL,
    show_df = TRUE,
    robust_type = NULL,

    #' @description Create a new `Classic_Fit` object.
    #' @param model The `RTMB_Model` object.
    #' @param par_vec Numeric vector of parameter estimates.
    #' @param par List of parameter estimates.
    #' @param objective Final objective value.
    #' @param log_lik Full log-likelihood used for information criteria.
    #' @param restricted_log_lik Restricted log-likelihood, if classical estimation used REML-style marginalization.
    #' @param rss Residual sum of squares for classical linear models, if available.
    #' @param df_residual Residual degrees of freedom for classical linear models, if available.
    #' @param convergence Convergence code.
    #' @param sd_rep TMB sdreport object.
    #' @param df_fixed Dataframe of fixed effects results.
    #' @param random_effects Dataframe of random effects results.
    #' @param df_transform Dataframe of transformed parameters.
    #' @param df_generate Dataframe of generated quantities.
    #' @param opt_history Dataframe of optimization history.
    #' @param transform List of transformed parameters.
    #' @param generate List of generated quantities.
    #' @param se_samples List of simulated samples for SE estimation.
    #' @param par_unc Parameter vector on unconstrained scale.
    #' @param vcov_unc Covariance matrix on unconstrained scale.
    #' @param ci_method Method used for CI estimation.
    #' @param laplace Whether Laplace approximation was used.
    #' @param map Parameter mapping used.
    #' @param test_results List of additional test results (e.g., chisq.test).
    #' @param view Character vector of parameter names to prioritize in summary.
    #' @param fit Legacy argument for backward compatibility (maps to df_fixed).
    #' @param vcov Variance-covariance matrix of parameters.
    #' @param df_method Method for degrees of freedom calculation.
    #' @param idx_fix_active Numeric vector; mapping between active parameters and full unconstrained vector.
    #' @param show_df Logical; whether to display degrees of freedom in the summary output.
    #' @param ... Additional arguments passed to the constructor.
    initialize = function(model, par_vec = NULL, par = NULL, objective = NULL, log_lik = NULL,
                          restricted_log_lik = NULL,
                          convergence = NULL, sd_rep = NULL, df_fixed = NULL, random_effects = NULL,
                          df_transform = NULL, df_generate = NULL, opt_history = NULL,
                          transform = NULL, generate = NULL, se_samples = NULL, par_unc = NULL,
                          vcov_unc = NULL, ci_method = "wald", laplace = TRUE, map = NULL,
                          test_results = list(), view = NULL, fit = NULL, vcov = NULL, 
                          df_method = "auto", idx_fix_active = NULL, show_df = TRUE,
                          rss = NULL, df_residual = NULL, ...) {
      self$model <- model
      self$par_vec <- par_vec
      self$par <- if (!is.null(par)) par else if (!is.null(df_fixed)) self$.construct_par_list(df_fixed) else if (!is.null(fit)) self$.construct_par_list(fit) else NULL
      self$objective <- objective
      
      # Handle log_ml fallback from ...
      dots <- list(...)
      if (is.null(log_lik) && !is.null(dots$log_ml)) {
        log_lik <- dots$log_ml
      }
      self$log_lik <- log_lik
      self$restricted_log_lik <- restricted_log_lik
      self$rss <- rss
      self$df_residual <- df_residual
      
      self$convergence <- convergence
      self$sd_rep <- sd_rep
      self$df_fixed <- if (!is.null(df_fixed)) df_fixed else fit
      self$fit <- self$df_fixed # Legacy compatibility
      self$random_effects <- random_effects
      self$df_transform <- df_transform
      self$df_generate <- df_generate
      self$opt_history <- opt_history
      self$transform <- transform
      self$generate <- generate
      self$se_samples <- se_samples
      self$par_unc <- par_unc
      self$vcov_unc <- vcov_unc
      self$ci_method <- ci_method
      self$se_method <- ci_method # Legacy compatibility
      self$laplace <- laplace
      self$map <- map
      self$test_results <- test_results
      self$view <- view
      self$vcov <- vcov
      self$df_method <- df_method
      self$idx_fix_active <- idx_fix_active
      self$show_df <- show_df
    },

    #' @description Compute robust standard errors (sandwich estimator).
    #' @param cluster Character; variable name for clustering.
    #' @param type Character; "HC3" (default), "HC0", or "HC1" for non-clustered
    #'   robust SE; "CR1" or "CR0" for clustered robust SE.
    #' @param inplace Logical; if FALSE, return a robust-SE copy without modifying the original fit.
    #'   If TRUE, update the object in place.
    #' @param ... Additional arguments.
    #' @return Self.
    robust_se = function(cluster = NULL, type = c("HC3", "HC0", "HC1", "CR1", "CR0"), inplace = FALSE, ...) {
      type <- match.arg(type)
      cluster_context <- !is.null(cluster) || !is.null(self$model$extra$glmer_info)
      if (isTRUE(cluster_context) && type %in% c("HC3", "HC0", "HC1")) type <- "CR1"
      if (!isTRUE(cluster_context) && type %in% c("CR1", "CR0")) type <- "HC3"

      if (!isTRUE(inplace)) {
        out <- self$clone(deep = TRUE)
        return(out$robust_se(cluster = cluster, type = type, inplace = TRUE, ...))
      }

      # --- 1. Detect if GLMM ---
      if (!is.null(self$model$extra$glmer_info)) {
        return(private$.compute_glmer_robust_se(cluster = cluster, type = type, update = TRUE))
      }

      # --- 2. Fallback to OLS-based sandwich for LM ---
      if (!is.null(self$model$type) && self$model$type == "lm") {
        return(private$.compute_lm_robust_se(cluster = cluster, type = type, update = TRUE))
      }

      # --- 3. Score-based sandwich for GLM ---
      if (!is.null(self$model$type) && self$model$type == "glm") {
        return(private$.compute_glm_robust_se(cluster = cluster, type = type, update = TRUE))
      }

      stop("robust_se() is currently supported for 'lm', 'glm', and mixed models ('lmer'/'glmer').", call. = FALSE)
    },

    #' @description (Deprecated) Use robust_se() instead.
    #' @param ... Arguments passed to robust_se(), including inplace.
    #' @return Self.
    compute_robust = function(...) {
      warning("compute_robust() is deprecated. Use robust_se() instead.", call. = FALSE)
      self$robust_se(...)
    },

    #' @description Compute nonparametric bootstrap standard errors and confidence intervals.
    #' Currently implemented only for mediation models.
    #' Bootstrap refits mediation equations with base R lm.fit()/glm.fit() when possible,
    #' and falls back to RTMB refits for unsupported families.
    #' @param n_boot Integer; number of bootstrap samples.
    #' @param seed Optional integer random seed.
    #' @param inplace Logical; if FALSE, return a bootstrap-SE copy without modifying the original fit.
    #'   If TRUE, update the object in place.
    #' @param ... Additional arguments.
    #' @return A Classic_Fit object.
    bootstrap = function(n_boot = 1000, seed = NULL, inplace = FALSE, ...) {
      self$compute_bootstrap(
        n_boot = n_boot,
        seed = seed,
        inplace = inplace,
        ...
      )
    },

    #' @description Compute nonparametric bootstrap standard errors and confidence intervals.
    #' Currently implemented only for mediation models.
    #' Bootstrap refits mediation equations with base R lm.fit()/glm.fit() when possible,
    #' and falls back to RTMB refits for unsupported families.
    #' @param n_boot Integer; number of bootstrap samples.
    #' @param seed Optional integer random seed.
    #' @param inplace Logical; if FALSE, return a bootstrap-SE copy without modifying the original fit.
    #'   If TRUE, update the object in place.
    #' @param ... Additional arguments.
    #' @return A Classic_Fit object.
    compute_bootstrap = function(n_boot = 1000, seed = NULL, inplace = FALSE, ...) {
      if (!identical(self$model$type, "mediation")) {
        stop(
          "bootstrap() is currently implemented only for mediation models. ",
          "Use robust_se() for lm/glmer robust standard errors.",
          call. = FALSE
        )
      }

      if (!isTRUE(inplace)) {
        out <- self$clone(deep = TRUE)
        return(out$compute_bootstrap(
          n_boot = n_boot,
          seed = seed,
          inplace = TRUE,
          ...
        ))
      }

      if (!is.numeric(n_boot) || length(n_boot) != 1L || is.na(n_boot) || n_boot < 1) {
        stop("n_boot must be a positive integer.", call. = FALSE)
      }

      n_boot <- as.integer(n_boot)

      extract_estimates <- function(fit) {
        dfs <- list(
          fixed = fit$df_fixed,
          random = fit$random_effects,
          transform = fit$df_transform,
          generate = fit$df_generate
        )
        keep <- vapply(dfs, is.data.frame, logical(1))
        dfs <- dfs[keep]
        if (length(dfs) == 0L) return(stats::setNames(numeric(0), character(0)))
        out_df <- private$.rbind_result_dfs(dfs)
        out_df <- out_df[!duplicated(rownames(out_df)), , drop = FALSE]
        out <- out_df$Estimate
        names(out) <- rownames(out_df)
        out
      }

      update_df_with_boot <- function(df, boot_se, boot_lower, boot_upper) {
        if (!is.data.frame(df)) return(df)

        common <- intersect(rownames(df), names(boot_se))
        if (length(common) == 0L) return(df)

        df[common, "Std. Error"] <- boot_se[common]
        df[common, "Lower 95%"] <- boot_lower[common]
        df[common, "Upper 95%"] <- boot_upper[common]

        stat_col <- if ("z value" %in% names(df)) {
          "z value"
        } else if ("t value" %in% names(df)) {
          "t value"
        } else {
          NULL
        }

        if (!is.null(stat_col)) {
          df[common, stat_col] <- df[common, "Estimate"] /
            pmax(df[common, "Std. Error"], 1e-12)
        }

        if ("Pr" %in% names(df) && !is.null(stat_col)) {
          if ("df" %in% names(df)) {
            df[common, "Pr"] <- 2 * stats::pt(
              -abs(df[common, stat_col]),
              df = df[common, "df"]
            )
          } else {
            df[common, "Pr"] <- 2 * stats::pnorm(
              -abs(df[common, stat_col])
            )
          }
        }

        df
      }

      compute_mediation_boot_estimates <- function(dat_b, target_names, formula, family) {
        n_eq <- length(formula)
        if (!is.list(family)) family <- rep(list(family), n_eq)
        if (length(family) != n_eq) stop("Length of family list must match length of formula list.")

        coefs <- vector("list", n_eq)
        x_names <- vector("list", n_eq)
        resp_names <- character(n_eq)
        out <- stats::setNames(rep(NA_real_, length(target_names)), target_names)

        for (j in seq_len(n_eq)) {
          f <- formula[[j]]
          fam <- family[[j]]
          fam_name <- if (inherits(fam, "family")) fam$family else as.character(fam)[1]
          mf <- stats::model.frame(f, data = dat_b)
          y <- stats::model.response(mf)
          X <- stats::model.matrix(f, data = mf)
          cn <- colnames(X)
          cn[cn == "(Intercept)"] <- "Intercept"
          x_names[[j]] <- cn
          resp_names[j] <- as.character(f[[2]])

          if (fam_name == "gaussian") {
            fit_j <- stats::lm.fit(x = X, y = as.numeric(y))
            beta <- as.numeric(fit_j$coefficients)
            names(beta) <- cn
            coefs[[j]] <- beta
            b_names <- paste0("b", j, "[", cn, "]")
            out[intersect(b_names, target_names)] <- beta[match(intersect(b_names, target_names), b_names)]
            sigma_name <- paste0("sigma", j)
            if (sigma_name %in% target_names) {
              out[[sigma_name]] <- sqrt(mean(fit_j$residuals^2, na.rm = TRUE))
            }
          } else if (fam_name %in% c("binomial", "bernoulli")) {
            fit_j <- stats::glm.fit(x = X, y = y, family = stats::binomial())
            beta <- as.numeric(fit_j$coefficients)
            names(beta) <- cn
            coefs[[j]] <- beta
            b_names <- paste0("b", j, "[", cn, "]")
            out[intersect(b_names, target_names)] <- beta[match(intersect(b_names, target_names), b_names)]
          } else if (fam_name == "poisson") {
            fit_j <- stats::glm.fit(x = X, y = y, family = stats::poisson())
            beta <- as.numeric(fit_j$coefficients)
            names(beta) <- cn
            coefs[[j]] <- beta
            b_names <- paste0("b", j, "[", cn, "]")
            out[intersect(b_names, target_names)] <- beta[match(intersect(b_names, target_names), b_names)]
          } else {
            stop(sprintf("Unsupported mediation bootstrap family: %s", fam_name), call. = FALSE)
          }
        }

        mediators <- intersect(resp_names, unique(unlist(x_names)))
        indeps <- setdiff(unique(unlist(x_names)), c(resp_names, "Intercept"))
        if (length(indeps) == 0L && length(mediators) > 0L) {
          indeps <- setdiff(x_names[[1]], "Intercept")
        }

        for (iv in indeps) {
          for (m in mediators) {
            idx_m_resp <- which(resp_names == m)
            idx_m_pred <- which(vapply(x_names, function(x) m %in% x, logical(1)))
            if (length(idx_m_resp) == 0L || length(idx_m_pred) == 0L) next

            for (m_resp in idx_m_resp) {
              if (!(iv %in% names(coefs[[m_resp]]))) next
              a_val <- coefs[[m_resp]][[iv]]

              for (dv_idx in idx_m_pred) {
                if (dv_idx == m_resp) next
                dv_name <- resp_names[[dv_idx]]
                if (!(m %in% names(coefs[[dv_idx]]))) next
                b_val <- coefs[[dv_idx]][[m]]

                ie_name <- paste0("IE_", iv, "_", m, "_", dv_name)
                if (ie_name %in% target_names) out[[ie_name]] <- a_val * b_val

                if (iv %in% names(coefs[[dv_idx]])) {
                  de_val <- coefs[[dv_idx]][[iv]]
                  de_name <- paste0("DE_", iv, "_", dv_name)
                  te_name <- paste0("TE_", iv, "_", m, "_", dv_name)
                  if (de_name %in% target_names) out[[de_name]] <- de_val
                  if (te_name %in% target_names) out[[te_name]] <- a_val * b_val + de_val
                }
              }
            }
          }
        }

        out
      }

      use_fast_mediation_bootstrap <- function(family) {
        fam_list <- if (is.list(family)) family else list(family)
        fam_names <- vapply(fam_list, function(fam) {
          if (inherits(fam, "family")) fam$family else as.character(fam)[1]
        }, character(1))
        all(fam_names %in% c("gaussian", "binomial", "bernoulli", "poisson"))
      }

      base_est <- extract_estimates(self)
      target_names <- names(base_est)
      if (length(target_names) == 0L) {
        stop("No estimates are available for bootstrap.", call. = FALSE)
      }

      med_info <- self$model$extra$mediation %||% list()
      formula <- med_info$formula %||% self$model$formula
      family <- med_info$family %||% self$model$family %||% "gaussian"
      view <- med_info$view %||% self$model$view
      dat <- self$model$raw_data

      if (is.null(formula) || is.null(dat)) {
        stop(
          "Mediation bootstrap requires the original formula and raw data stored in the model.",
          call. = FALSE
        )
      }
      dat <- as.data.frame(dat)
      n <- nrow(dat)
      if (is.na(n) || n <= 0L) stop("Original data are empty.", call. = FALSE)

      if (!is.null(seed)) set.seed(seed)

      boot_mat <- matrix(NA_real_, nrow = n_boot, ncol = length(target_names))
      colnames(boot_mat) <- target_names
      ok <- rep(FALSE, n_boot)
      fast_boot <- use_fast_mediation_bootstrap(family)

      show_progress <- !getOption("BayesRTMB.silent", FALSE)
      if (show_progress) {
        cat(sprintf("Performing mediation bootstrap estimation (n = %d)...\n", n_boot))
        pb <- utils::txtProgressBar(min = 0, max = n_boot, style = 3)
        on.exit(close(pb), add = TRUE)
      }

      old_silent <- options(BayesRTMB.silent = TRUE)
      on.exit(options(old_silent), add = TRUE)

      for (i in seq_len(n_boot)) {
        idx <- sample.int(n, size = n, replace = TRUE)
        dat_b <- dat[idx, , drop = FALSE]

        boot_est <- tryCatch({
          if (fast_boot) {
            compute_mediation_boot_estimates(
              dat_b = dat_b,
              target_names = target_names,
              formula = formula,
              family = family
            )
          } else {
            boot_mdl <- rtmb_mediation(
              formula = formula,
              data = dat_b,
              family = family,
              prior = prior_flat(),
              view = view
            )
            boot_fit <- boot_mdl$classic()
            extract_estimates(boot_fit)
          }
        }, error = function(e) NULL)

        if (!is.null(boot_est) && length(boot_est) > 0L) {
          common <- intersect(target_names, names(boot_est))
          if (length(common) > 0L) {
            boot_mat[i, common] <- boot_est[common]
            ok[i] <- TRUE
          }
        }

        if (show_progress) utils::setTxtProgressBar(pb, i)
      }

      if (!any(ok)) {
        stop("All bootstrap fits failed.", call. = FALSE)
      }
      if (sum(ok) < n_boot) {
        warning(
          sprintf("Only %d of %d bootstrap fits succeeded.", sum(ok), n_boot),
          call. = FALSE
        )
      }

      boot_mat_ok <- boot_mat[ok, , drop = FALSE]
      boot_se <- apply(boot_mat_ok, 2, stats::sd, na.rm = TRUE)
      boot_lower <- apply(boot_mat_ok, 2, stats::quantile, probs = 0.025, na.rm = TRUE)
      boot_upper <- apply(boot_mat_ok, 2, stats::quantile, probs = 0.975, na.rm = TRUE)

      boot_lower <- as.numeric(boot_lower)
      boot_upper <- as.numeric(boot_upper)
      names(boot_lower) <- names(boot_upper) <- colnames(boot_mat_ok)

      self$bootstrap_results <- boot_mat_ok
      self$se_method <- "bootstrap"
      self$ci_method <- "bootstrap"
      self$cluster <- NULL
      self$robust_type <- NULL
      self$vcov <- stats::cov(boot_mat_ok, use = "pairwise.complete.obs")

      self$df_fixed <- update_df_with_boot(self$df_fixed, boot_se, boot_lower, boot_upper)
      self$random_effects <- update_df_with_boot(self$random_effects, boot_se, boot_lower, boot_upper)
      self$df_transform <- update_df_with_boot(self$df_transform, boot_se, boot_lower, boot_upper)
      self$df_generate <- update_df_with_boot(self$df_generate, boot_se, boot_lower, boot_upper)
      self$fit <- self$df_fixed

      return(self)
    },

    #' @description Get the AIC of the fitted model.
    AIC = function() {
      ll <- self$logLik()
      k <- as.numeric(attr(ll, "df"))
      if (length(k) == 0 || is.na(k)) k <- 0
      return(-2 * as.numeric(ll) + 2 * k[1])
    },

    #' @description Get the BIC of the fitted model.
    BIC = function() {
      ll <- self$logLik()
      k <- as.numeric(attr(ll, "df"))
      n <- as.numeric(attr(ll, "nobs"))
      if (length(k) == 0 || is.na(k)) k <- 0
      if (length(n) == 0 || is.na(n) || n <= 0) n <- 1
      return(-2 * as.numeric(ll) + log(n[1]) * k[1])
    },

    #' @description WAIC is not defined for classical fits.
    WAIC = function() {
      stop(
        "WAIC requires posterior draws of pointwise log_lik. ",
        "Use sample() or variational() on a model created with WAIC = TRUE.",
        call. = FALSE
      )
    },

    #' @description Run basic diagnostics for the classical fit.
    #' @param ... Additional arguments passed to `diagnose_classic_fit()`.
    #' @return A `diagnose_BayesRTMB` object.
    diagnose = function(...) {
      diagnose_classic_fit(self, ...)
    },

    #' @description Print the fit results.
    #' @param ... Additional arguments passed to `summary()`.
    print = function(...) {
      print(self$summary(...))
      invisible(self)
    },

    #' @description Get the Log-Likelihood of the fitted model.
    logLik = function() {
      if (inherits(self$fit, c("lm", "glm"))) {
        return(stats::logLik(self$fit))
      }

      val <- if (!is.null(self$log_lik)) {
        self$log_lik
      } else if (!is.null(self$model$extra[["loglik"]])) {
        self$model$extra[["loglik"]]
      } else {
        NA_real_
      }

      df_val <- if (!is.null(self$model$extra[["df"]])) {
        self$model$extra[["df"]]
      } else if (is.data.frame(self$fit)) {
        nrow(self$fit)
      } else {
        length(self$par_vec)
      }

      nobs_val <- if (!is.null(self$model$extra[["nobs"]])) {
        self$model$extra[["nobs"]]
      } else if (!is.null(self$model$raw_data)) {
        nrow(as.data.frame(self$model$raw_data))
      } else {
        NA_real_
      }

      res <- val
      attr(res, "df") <- df_val
      attr(res, "nobs") <- nobs_val
      class(res) <- "logLik"
      res
    },

    #' @description EAP estimates are not available for Classic_Fit.
    #' @param ... Ignored.
    EAP = function(...) {
      stop(
        "EAP() is not available for Classic_Fit. Use estimate() instead.",
        call. = FALSE
      )
    },

    #' @description MAP estimates are not available for Classic_Fit.
    #' @param ... Ignored.
    MAP = function(...) {
      stop(
        "MAP() is not available for Classic_Fit. Use estimate() instead.",
        call. = FALSE
      )
    },

    #' @description Display a summary of the estimation results.
    #' @param view Character vector of parameter names to prioritize or filter by.
    #' @param digits Number of digits to print for estimates.
    #' @param max_rows Maximum number of rows to display in the coefficient table.
    #' @param ranef Logical; whether to show random effects in the summary.
    summary = function(view = NULL, digits = 5, max_rows = 10, ranef = FALSE) {
      if (!is.numeric(digits)) digits <- 5
      res <- list(
        type = self$model$type,
        family = self$model$family,
        se_method = self$se_method,
        cluster = self$cluster,
        robust_type = self$robust_type,
        bootstrap = if (!is.null(self$bootstrap_results)) nrow(self$bootstrap_results) else NULL,
        logLik = self$logLik(),
        AIC = self$AIC(),
        BIC = self$BIC(),
        extra = self$model$extra,
        test_results = self$test_results,
        digits = digits
      )

      if (is.data.frame(self$fit) || inherits(self$fit, "lm")) {
        # Combine all result dataframes (fixed, random, transform, generate)
        all_dfs <- list()
        if (is.data.frame(self$fit)) all_dfs$fixed <- self$fit
        
        if (!is.null(self$random_effects) && is.data.frame(self$random_effects)) {
           all_dfs$random <- self$random_effects
        }
        if (!is.null(self$df_transform) && is.data.frame(self$df_transform)) {
           all_dfs$transform <- self$df_transform
        }
        if (!is.null(self$df_generate) && is.data.frame(self$df_generate)) {
           all_dfs$generate <- self$df_generate
        }

        if (length(all_dfs) > 0) {
           df_all <- private$.rbind_result_dfs(all_dfs)
           # Remove duplicates (preferring earlier ones)
           df_all <- df_all[!duplicated(rownames(df_all)), , drop = FALSE]
           # Track which rows are random effects
           is_re <- if (!is.null(self$random_effects)) rownames(df_all) %in% rownames(self$random_effects) else rep(FALSE, nrow(df_all))
        } else if (inherits(self$fit, "lm")) {
           # Handle lm case specifically
           s_lm <- summary(self$fit)
           df_all <- as.data.frame(s_lm$coefficients)
        } else {
           df_all <- self$fit
        }

        # --- Correlation Model Filtering ---
        if (!is.null(self$model$type) && self$model$type == "corr") {
           keep <- rep(TRUE, nrow(df_all))
           seen_pairs <- list()
           for (i in 1:nrow(df_all)) {
             nm <- rownames(df_all)[i]
             est <- as.numeric(df_all$Estimate[i])
             se <- as.numeric(df_all$`Std. Error`[i])
             if (!is.na(est) && abs(est - 1) < 1e-7 && (is.na(se) || se < 1e-10)) {
               keep[i] <- FALSE
               next
             }
             if (grepl("\\[.*,.*\\]", nm)) {
               parts <- gsub(".*\\[(.*)\\]", "\\1", nm); bits <- strsplit(parts, ",")[[1]]
               if (length(bits) == 2) {
                 pair_id <- paste(sort(trimws(bits)), collapse = "_")
                 prefix <- gsub("\\[.*", "", nm); pair_key <- paste0(prefix, "_", pair_id)
                 if (pair_key %in% names(seen_pairs)) { keep[i] <- FALSE } else {
                   seen_pairs[[pair_key]] <- TRUE
                   if (trimws(bits[1]) == trimws(bits[2])) keep[i] <- FALSE
                 }
               }
             }
           }
           df_all <- df_all[keep, , drop = FALSE]
           if (exists("is_re")) is_re <- is_re[keep]
        }

        df_print <- df_all
        
        # --- Handle lm specifically if needed ---
        if (inherits(self$fit, "lm")) {
           s_lm <- summary(self$fit)
           is_asymptotic <- inherits(self$model, c("rtmb_loglinear", "rtmb_table"))
           df_print <- as.data.frame(s_lm$coefficients)
           if (!is.null(self$vcov)) {
             new_se <- sqrt(diag(self$vcov))
             n_match <- min(nrow(df_print), length(new_se))
             df_print[1:n_match, 2] <- new_se[1:n_match]
             df_print[1:n_match, 3] <- df_print$Estimate[1:n_match] / pmax(df_print[1:n_match, 2], 1e-12)
           }
           if (is_asymptotic) {
             colnames(df_print) <- c("Estimate", "Std. Error", "z value", "Pr")
             df_print$df <- Inf
           } else {
             colnames(df_print) <- c("Estimate", "Std. Error", "t value", "Pr")
             df_print$df <- self$fit$df.residual
           }
           rownames(df_print)[rownames(df_print) == "(Intercept)"] <- "Intercept"
        }

        # --- View Prioritization Logic ---
        # Distinction: if 'view' is explicitly passed, we apply STRICT filtering.
        view_explicit <- !is.null(view)
        view_to_use <- if (view_explicit) view else if (!is.null(self$view)) self$view else self$model$view
        
        # If it's a correlation model, default is c("corr", "mean", "sd")
        if (is.null(view_to_use) && !is.null(self$model$type) && self$model$type == "corr") {
           view_to_use <- c("corr", "mean", "sd")
        }
        
        priority_idx <- integer(0)
        cf_idx <- integer(0)
        
        if (!is.null(view_to_use)) {
          var_names <- rownames(df_print)
          base_names <- gsub("\\[.*\\]$", "", var_names)
          
          # Pass 1: Exact matches and base names for ALL keywords
          for (v in view_to_use) {
            m1 <- which(var_names == v | base_names == v)
            priority_idx <- c(priority_idx, setdiff(m1, priority_idx))
          }
          
          # Pass 2: Standard prefix matches (e.g. corr[...]) for ALL keywords
          for (v in view_to_use) {
            # Specifically exclude CF_ prefix here to prevent it from jumping ahead
            m2 <- which(grepl(paste0("^", v), var_names) & !grepl("^CF_", var_names))
            priority_idx <- c(priority_idx, setdiff(m2, priority_idx))
          }
          
          # Pass 3: Internal CF_ matches (e.g. CF_corr[...]) for ALL keywords - ALWAYS LAST
          for (v in view_to_use) {
            m3 <- which(grepl(paste0("^CF_", v), var_names))
            cf_idx <- c(cf_idx, setdiff(m3, cf_idx))
          }
          
          priority_idx <- unique(priority_idx)
          cf_idx <- setdiff(unique(cf_idx), priority_idx)
          
          # Reorder based on hierarchy: Standard matches > CF matches > Others
          final_order <- c(priority_idx, cf_idx)
          
          if (view_explicit) {
             # STRICT FILTERING: Only keep matched rows
             other_idx <- integer(0)
          } else {
             # PRIORITIZATION ONLY: Keep everything else at the bottom
             other_idx <- setdiff(seq_len(nrow(df_print)), final_order)
          }
          final_order <- c(final_order, other_idx)
          
          # Update is_re and df_print
          if (exists("is_re")) is_re <- is_re[final_order]
          matched_indices <- seq_along(priority_idx)
          if (length(cf_idx) > 0) matched_indices <- c(matched_indices, (length(priority_idx)+1):(length(priority_idx)+length(cf_idx)))
          
          df_print <- df_print[final_order, , drop = FALSE]
        }
        
        # --- Random Effect Filtering ---
        if (!isTRUE(ranef) && exists("is_re")) {
           # Keep if NOT a random effect OR it was explicitly matched by view
           keep_mask <- !is_re
           if (exists("matched_indices") && length(matched_indices) > 0) {
              keep_mask[matched_indices] <- TRUE
           }
           df_print <- df_print[keep_mask, , drop = FALSE]
        }
        
        # --- Truncation Logic ---
        if (!is.null(max_rows) && nrow(df_print) > max_rows) {
           res$truncated <- TRUE
           res$total_rows <- nrow(df_print)
           res$max_rows <- max_rows
        } else {
           res$truncated <- FALSE
        }

        is_asymptotic <- (!is.null(self$model$type) &&
                            self$model$type %in% c("glm", "glmer", "table", "loglinear", "fa", "irt", "mixture")) ||
          inherits(self$model, "rtmb_loglinear")
        hide_tests <- !is.null(self$model$type) && self$model$type %in% c("mdu", "fa")

        # --- Add t/z-test results for classic mode ---
        if (!is.null(self$sd_rep)) {
           u_est <- if (!is.null(self$sd_rep$par.fixed)) self$sd_rep$par.fixed else numeric(0)
           u_se <- if (!is.null(self$sd_rep$cov.fixed)) sqrt(diag(self$sd_rep$cov.fixed)) else rep(NA, length(u_est))
           u_t <- ifelse(is.na(u_se), NA_real_, u_est / pmax(u_se, 1e-12))
           
           # Map to df_print rows
           u_idx_map <- list()
           if (length(u_est) > 0) {
             for (nm in unique(names(u_est))) {
                u_idx_map[[nm]] <- which(names(u_est) == nm)
             }
           }
           
           u_counter <- list()
           row_t <- rep(NA, nrow(df_print))
           
           for (i in 1:nrow(df_print)) {
              row_nm <- rownames(df_print)[i]
              base_nm <- gsub("\\[.*\\]$", "", row_nm)
              
              if (base_nm %in% names(u_idx_map)) {
                 if (is.null(u_counter[[base_nm]])) u_counter[[base_nm]] <- 1
                 curr_idx <- u_counter[[base_nm]]
                 if (curr_idx <= length(u_idx_map[[base_nm]])) {
                    real_idx <- u_idx_map[[base_nm]][curr_idx]
                    row_t[i] <- u_t[real_idx]
                    u_counter[[base_nm]] <- curr_idx + 1
                 }
              }
           }
           
           # Fallback to constrained ratio for parameters not in par.fixed (e.g. transformed)
           missing_t <- is.na(row_t)
           if (any(missing_t)) {
              row_t[missing_t] <- ifelse(is.na(df_print$`Std. Error`[missing_t]), NA_real_,
                df_print$Estimate[missing_t] / pmax(df_print$`Std. Error`[missing_t], 1e-12))
           }

           if (identical(self$se_method, "bootstrap")) {
              row_t <- ifelse(is.na(df_print$`Std. Error`), NA_real_,
                df_print$Estimate / pmax(df_print$`Std. Error`, 1e-12))
           } else if (self$se_method %in% c("robust", "cluster-robust") && !is.null(self$vcov)) {
              robust_rows <- intersect(rownames(df_print), rownames(self$vcov))
              if (length(robust_rows) > 0L) {
                robust_idx <- match(robust_rows, rownames(df_print))
                row_t[robust_idx] <- ifelse(is.na(df_print$`Std. Error`[robust_idx]), NA_real_,
                  df_print$Estimate[robust_idx] / pmax(df_print$`Std. Error`[robust_idx], 1e-12))
              }
           } else if (!identical(self$se_method, "wald")) {
              row_t <- ifelse(is.na(df_print$`Std. Error`), NA_real_,
                df_print$Estimate / pmax(df_print$`Std. Error`, 1e-12))
           }
           
           if (is_asymptotic) {
                df_print$`z value` <- row_t
                if ("df" %in% names(df_print)) df_print$df <- NULL # Hide DF for asymptotic models
             } else {
                df_print$`t value` <- row_t
             }
             
             if (!is.null(df_print$df)) {
                df_print$Pr <- 2 * pt(-abs(row_t), df = df_print$df)
             } else {
                df_print$Pr <- 2 * pnorm(-abs(row_t))
             }

             if (!is.null(self$model$type) && self$model$type == "ttest") {
                test_rows <- rownames(df_print) == "diff"
                if ("t value" %in% names(df_print)) df_print$`t value`[!test_rows] <- NA_real_
                if ("z value" %in% names(df_print)) df_print$`z value`[!test_rows] <- NA_real_
                df_print$Pr[!test_rows] <- NA_real_
             }

             if (!is.null(self$model$type) && self$model$type %in% c("lm", "glm", "lmer", "glmer", "mediation")) {
                row_base <- gsub("\\[.*\\]$", "", rownames(df_print))
                scale_rows <- grepl("^sigma", row_base) | grepl("^sd", row_base)
                if (any(scale_rows)) {
                   if ("t value" %in% names(df_print)) df_print$`t value`[scale_rows] <- NA_real_
                   if ("z value" %in% names(df_print)) df_print$`z value`[scale_rows] <- NA_real_
                   df_print$Pr[scale_rows] <- NA_real_
                }
             }

             if (hide_tests) {
                df_print$`z value` <- NULL
                df_print$`t value` <- NULL
                df_print$Pr <- NULL
             }
        }

        # --- Hide DF column if requested ---
        if (!isTRUE(self$show_df) && "df" %in% names(df_print)) {
           df_print$df <- NULL
        }

        # 1. Round main numeric columns
        cols_to_round <- setdiff(names(df_print), c("Pr", "df"))
        for (col in cols_to_round) {
          if (is.numeric(df_print[[col]])) {
            df_print[[col]] <- round(df_print[[col]], digits)
          }
        }

        # 2. Round df to 1 decimal place
        if ("df" %in% names(df_print) && is.numeric(df_print$df)) {
          df_print$df <- round(df_print$df, 1)
        }

        # 3. Format Pr
        if (!is.null(df_print$Pr)) {
          # Standard significance symbols
          sig <- symnum(as.numeric(df_print$Pr), corr = FALSE, na = FALSE,
                        cutpoints = c(0, 0.001, 0.01, 0.05, 0.1, 1),
                        symbols = c("***", "**", "*", ".", " "))

          # Format Pr values (e.g., .0000, .0123)
          formatted_pr <- sapply(df_print$Pr, function(p) {
            if (is.na(p)) return("")
            p_val <- as.numeric(p)
            if (p_val < 0.0001) return("<.0001")
             fmt <- paste0("%.", digits, "f")
             s <- sprintf(fmt, p_val)
             sub("^0", "", s) # Remove leading zero
          })

          df_print$Pr <- formatted_pr
          df_print$sig <- as.character(sig)
        }

        # 4. Reorder columns for consistency
        if (is_asymptotic) {
          desired_order <- c("Estimate", "Std. Error", "Lower 95%", "Upper 95%", "df", "z value", "Pr")
        } else {
          desired_order <- c("Estimate", "Std. Error", "Lower 95%", "Upper 95%", "df", "t value", "Pr")
        }
        current_names <- names(df_print)

        # Filter existing columns
        order_to_use <- intersect(desired_order, current_names)
        other_names <- setdiff(current_names, c(desired_order, "sig"))

        # Construct final selection
        final_cols <- c(order_to_use, other_names)
        if ("sig" %in% current_names) {
          final_cols <- c(final_cols, "sig")
        }

        df_final <- df_print[, final_cols, drop = FALSE]

        # Remove df for asymptotic models if requested
        if (is_asymptotic && "df" %in% names(df_final)) {
          df_final$df <- NULL
        }

        # Rename sig to blank just before printing
        if ("sig" %in% names(df_final)) {
           names(df_final)[names(df_final) == "sig"] <- ""
        }

        res$coefficients <- df_final

        # --- Enhanced Output for LM/GLM ---
        if (inherits(self$fit, c("lm", "glm"))) {
          s_lm <- summary(self$fit)
          res$lm_info <- list(
            dispersion = if (inherits(self$fit, "glm")) s_lm$dispersion else NULL,
            null_deviance = if (inherits(self$fit, "glm")) s_lm$null.deviance else NULL,
            df_null = if (inherits(self$fit, "glm")) s_lm$df.null else NULL,
            deviance = if (inherits(self$fit, "glm")) s_lm$deviance else NULL,
            df_residual = s_lm$df.residual,
            sigma = if (inherits(self$fit, "lm")) s_lm$sigma else NULL,
            df_lm = if (inherits(self$fit, "lm")) s_lm$df else NULL,
            r_squared = if (inherits(self$fit, "lm")) s_lm$r.squared else NULL,
            adj_r_squared = if (inherits(self$fit, "lm")) s_lm$adj.r.squared else NULL,
            fstatistic = if (inherits(self$fit, "lm")) s_lm$fstatistic else NULL
          )
        }
      } else {
        res$fit_summary <- self$fit
      }

      class(res) <- "summary_Classic_Fit"
      return(res)
    },



    #' @description Perform ANOVA (Wald F-tests / Chisq-tests) on the fitted model.
    #' @param method Character; "reml" (standard) or "ls" (experimental).
    #' @param type Integer; Type of Sum of Squares (only Type III supported currently).
    #' @return A data frame containing the ANOVA table.
    anova = function(method = c("reml", "ls"), type = 3) {
      # --- 1. Restricted Model Types ---
      restricted_types <- c("ttest", "corr", "fa", "irt")
      if (!is.null(self$model$type) && self$model$type %in% restricted_types) {
        stop(sprintf("anova() is not supported for '%s' models.", self$model$type), call. = FALSE)
      }

      if (!is.null(self$model$type) && self$model$type == "table") {
        obs_tab <- self$model$extra$tab
        N_tot <- sum(obs_tab)
        R_dim <- nrow(obs_tab)
        C_dim <- ncol(obs_tab)

        # Combine all result dataframes to find 'mu'
        all_dfs <- list()
        if (!is.null(self$df_fixed)) all_dfs$fixed <- self$df_fixed
        if (!is.null(self$df_transform)) all_dfs$transform <- self$df_transform
        if (!is.null(self$df_generate)) all_dfs$generate <- self$df_generate
        
        df_combined <- if (length(all_dfs) > 0) {
          tmp <- private$.rbind_result_dfs(all_dfs)
          tmp[!duplicated(rownames(tmp)), , drop = FALSE]
        } else data.frame()
        
        df_mu <- df_combined[grepl("^mu(\\[|$)", rownames(df_combined)), , drop = FALSE]

        if (nrow(df_mu) == R_dim * C_dim) {
          p_est <- df_mu$Estimate / N_tot
          p_mat <- matrix(p_est, nrow = R_dim, ncol = C_dim)
          p_row <- rowSums(p_mat)
          p_col <- colSums(p_mat)
          E_mat <- matrix(0, nrow = R_dim, ncol = C_dim)
          for (i in 1:R_dim) for (j in 1:C_dim) E_mat[i, j] <- p_row[i] * p_col[j] * N_tot
          chisq_stat <- sum((as.vector(obs_tab) - as.vector(E_mat))^2 / as.vector(E_mat))
          df_val <- (R_dim - 1) * (C_dim - 1)
          p_val <- stats::pchisq(chisq_stat, df = df_val, lower.tail = FALSE)

           chisq_res <- data.frame(
             `X-squared` = round(chisq_stat, 5), 
             df = round(df_val, 1), 
             `p-value` = round(p_val, 5), 
             check.names = FALSE
           )
           rownames(chisq_res) <- "Pearson's Chi-squared (from est. prob)"
        } else {
          chisq_res <- NULL
        }

        fisher_res <- if (!is.null(self$test_results$fisher)) self$test_results$fisher else self$model$extra$fisher
        res <- list(chisq = chisq_res, fisher = fisher_res)
        class(res) <- "anova_rtmb_table"
        return(res)
      }

      old_opts <- NULL
      if (!is.null(self$model$contrasts)) {
        if (self$model$contrasts == "sum") {
          old_opts <- options(contrasts = c("contr.sum", "contr.poly"))
        } else if (self$model$contrasts == "treatment") {
          old_opts <- options(contrasts = c("contr.treatment", "contr.poly"))
        }
      }
      if (!is.null(old_opts)) on.exit(options(old_opts), add = TRUE)

      method <- match.arg(method)
      if (is.null(self$model$extra$X_assign)) stop("ANOVA requires term assignments (X_assign).")
      V_full <- self$vcov
      beta_full <- if (is.data.frame(self$fit)) self$fit$Estimate else stats::coef(self$fit)

      full_names <- if (is.data.frame(self$fit)) rownames(self$fit) else names(beta_full)
      if (inherits(self$fit, "lm")) {
        full_names[full_names == "(Intercept)"] <- "Intercept"
      }
      names(beta_full) <- full_names

      assign_idx <- self$model$extra$X_assign

      if (inherits(self$fit, "lm")) {
        fe_idx <- seq_along(beta_full)
      } else {
        # Expand patterns for identifying fixed effects (adding beta, mu, delta, etc.)
        fix_pats <- c("Intercept", "Intercept_c", "b", "mean", "prob", "beta", "mu", "delta", "diff", "mean_diff")
        fe_regex <- paste0("^(", paste(fix_pats, collapse="|"), ")($|\\[)")
        fe_idx <- which(grepl(fe_regex, names(beta_full)))
      }

      if (length(fe_idx) == 0) stop("Could not identify fixed effects for ANOVA.")

      beta <- beta_full[fe_idx]
      fe_names_actual <- names(beta)
      V <- V_full[fe_names_actual, fe_names_actual, drop = FALSE]

      full_assign <- c(0, assign_idx)

      ct_setting <- if (!is.null(self$model$contrasts)) self$model$contrasts else "sum"
      if (ct_setting != "sum") {
        formula <- nobars(self$model$formula)
        predictor_vars <- all.vars(delete.response(terms(formula)))
        relevant_data <- self$model$raw_data[, predictor_vars, drop = FALSE]
        if (is.null(relevant_data)) relevant_data <- self$model$data[, predictor_vars, drop = FALSE]
        levs <- lapply(relevant_data, function(x) {
          if (is.factor(x) || is.character(x)) {
            levels(as.factor(x))
          } else {
            mean(x, na.rm = TRUE)
          }
        })
        grid <- expand.grid(levs)

        old_opts2 <- options(contrasts = if (ct_setting == "treatment")
          c("contr.treatment", "contr.poly") else options()$contrasts)
        X_curr <- model.matrix(delete.response(terms(formula)), grid)
        options(old_opts2)

        old_opts3 <- options(contrasts = c("contr.sum", "contr.poly"))
        X_sum <- model.matrix(delete.response(terms(formula)), grid)
        options(old_opts3)

        if (ncol(X_curr) == length(beta)) {
          M <- MASS::ginv(X_sum) %*% X_curr
          beta <- as.numeric(M %*% beta)
          names(beta) <- colnames(X_sum)
          V <- M %*% V %*% t(M)
          assign_idx <- attr(X_sum, "assign")
        }
      }

      terms <- self$model$extra$X_terms
      res_list <- list()

      has_int <- any(grepl("^Intercept", names(beta))) || any(full_assign == 0)
      all_assign <- seq_along(terms)
      all_term_names <- terms

      for (i in seq_along(all_assign)) {
        a_id <- all_assign[i]
        t_name <- all_term_names[i]

        idx <- which(assign_idx == a_id)
        if (length(idx) == 0) next

        # --- Special Case: Calculate F-value by squaring t-value for 1-DF terms (continuous or 2-level factors) ---
        # Attempt direct extraction from original beta_full and V_full to avoid contrast transformation issues
        is_single <- length(idx) == 1
        W <- NULL
        if (is_single) {
          term_name_in_beta <- names(beta)[idx]
          # Look for the corresponding coefficient in the original estimation results (beta_full)
          # Try both b[name] format and the name itself
          orig_match <- which(names(beta_full) == term_name_in_beta | names(beta_full) == paste0("b[", term_name_in_beta, "]"))
          if (length(orig_match) == 1) {
            b_val <- beta_full[orig_match]
            se_val <- sqrt(V_full[orig_match, orig_match])
            W <- (b_val / se_val)^2
          }
        }

        # Standard Wald test if not 1-DF or if the above failed
        if (is.null(W)) {
          L <- matrix(0, nrow = length(idx), ncol = length(beta))
          for (j in seq_along(idx)) L[j, idx[j]] <- 1

          LVL <- L %*% V %*% t(L)
          inv_LVL <- try(solve(LVL), silent = TRUE)
          if (inherits(inv_LVL, "try-error")) inv_LVL <- MASS::ginv(LVL)

          W <- as.numeric(t(L %*% beta) %*% inv_LVL %*% (L %*% beta))
        }

        df1 <- length(idx)
        f_val <- W / df1

        fit_df_col <- if ("df" %in% names(self$fit)) self$fit$df else if ("DF" %in% names(self$fit)) self$fit$DF else NULL
        if (is.data.frame(self$fit) && !is.null(fit_df_col)) {
          # Align with beta_full indices
          term_name_in_beta <- names(beta)[idx]
          orig_match_indices <- sapply(term_name_in_beta, function(nm) {
            m <- which(names(beta_full) == nm | names(beta_full) == paste0("b[", nm, "]"))
            if (length(m) > 0) m[1] else NA
          })
          df2 <- if (any(!is.na(orig_match_indices))) min(fit_df_col[orig_match_indices], na.rm = TRUE) else Inf
        } else if (inherits(self$fit, "lm")) {
          df2 <- self$fit$df.residual
        } else {
          df2 <- Inf
        }

        # --- Decision: Use F-test or Chi-squared test format ---
        # Unify to F-test format unless it is an asymptotic model (Poisson, Binomial, Table, etc.)
        is_asymp <- (!is.null(self$model$family) && self$model$family %in% c("poisson", "binomial", "bernoulli", "multinomial", "ordered")) || 
                    (!is.null(self$model$type) && self$model$type %in% c("loglinear", "table"))
        
        if (is_asymp) {
          # Asymptotic model: Chi-squared test format
          p_val <- stats::pchisq(W, df1, lower.tail = FALSE)
          res_list[[t_name]] <- data.frame(`df` = df1, `Chisq` = W, `Pr(>Chisq)` = p_val, row.names = t_name, check.names = FALSE)
        } else {
          # Regression models: Unify to F-test format (including cases where denominator DF is Inf)
          p_val <- if (is.infinite(df2) || df2 <= 0) stats::pchisq(W, df1, lower.tail = FALSE) 
                   else stats::pf(f_val, df1, df2, lower.tail = FALSE)
          res_list[[t_name]] <- data.frame(`num_df` = df1, `den_df` = df2, `F value` = f_val, `Pr(>F)` = p_val, row.names = t_name, check.names = FALSE)
        }
      }

      res_df <- do.call(rbind, res_list)

      class(res_df) <- c("anova_rtmb", "anova", "data.frame")
      heading <- if ("Chisq" %in% names(res_df)) "ANOVA Table (Wald Chisq tests)" else "ANOVA Table (Wald F-tests)"
      attr(res_df, "heading") <- heading

      # --- Added: R-squared and overall model test (for rtmb_lm) ---
      is_lm <- !is.null(self$model$type) && self$model$type == "lm"
      if (is_lm && !is.null(self$model$data$Y)) {
        Y <- self$model$data$Y
        TSS <- sum((Y - mean(Y))^2)
        sigma_est <- beta_full["sigma"]
        if (!is.na(sigma_est) && TSS > 0) {
          # Obtain residual degrees of freedom
          df_resid <- if (is.null(res_df$den_df)) (length(Y) - sum(res_df$num_df) - 1) else min(res_df$den_df, na.rm = TRUE)
          if (is.infinite(df_resid)) df_resid <- length(Y) - sum(res_df$num_df) - 1
          
          RSS <- (sigma_est^2) * df_resid
          r2 <- 1 - (RSS / TSS)
          adj_r2 <- 1 - (1 - r2) * (length(Y) - 1) / pmax(df_resid, 1)

          # Overall model Wald test (all predictors vs. Intercept)
          all_pred_idx <- which(assign_idx > 0)
          if (length(all_pred_idx) > 0) {
            L_all <- matrix(0, nrow = length(all_pred_idx), ncol = length(beta))
            for (j in seq_along(all_pred_idx)) L_all[j, all_pred_idx[j]] <- 1
            LVL_all <- L_all %*% V %*% t(L_all)
            W_all <- as.numeric(t(L_all %*% beta) %*% MASS::ginv(LVL_all) %*% (L_all %*% beta))
            f_all <- W_all / length(all_pred_idx)
            p_all <- stats::pf(f_all, length(all_pred_idx), df_resid, lower.tail = FALSE)
            
            attr(res_df, "lm_info") <- list(
              r_squared = r2,
              adj_r_squared = adj_r2,
              fstatistic = c(value = f_all, numdf = length(all_pred_idx), dendf = df_resid),
              p_value = p_all,
              sigma = sigma_est,
              df_resid = df_resid
            )
          }
        }
      }
      return(res_df)
    },

    #' @description Calculate Least Squares Means (Marginal Means) and contrasts.
    #' @param specs Character vector of factors to calculate means for.
    #' @param pairwise Logical; whether to perform pairwise comparisons.
    #' @param simple Character vector of factors to hold constant for simple main effects.
    #' @param adjust Character; p-value adjustment method (e.g., "bonferroni", "holm", "none").
    #' @param protect Logical; whether to use hierarchical (protected) testing.
    #' @return A data frame containing the marginal means or contrasts.
    lsmeans = function(specs = NULL, pairwise = FALSE, simple = NULL, adjust = "holm", protect = FALSE) {
      # --- 1. Restricted Model Types ---
      restricted_types <- c("ttest", "corr", "fa", "irt", "table")
      if (!is.null(self$model$type) && self$model$type %in% restricted_types) {
        stop(sprintf("lsmeans() is not supported for '%s' models.", self$model$type), call. = FALSE)
      }

      # --- 2. Check for missing specs ---
      if (is.null(specs)) {
        formula <- self$model$formula
        data <- if (!is.null(self$model$raw_data)) as.data.frame(self$model$raw_data) else as.data.frame(self$model$data)
        vars <- all.vars(nobars(formula))[-1]
        is_cat <- sapply(data[vars], function(x) is.factor(x) || is.character(x))
        cat_vars <- vars[is_cat]
        
        if (length(cat_vars) == 0) {
          stop("lsmeans() is only available for models with categorical factors.", call. = FALSE)
        } else {
          stop(sprintf("Please specify 'specs' (e.g., fit$lsmeans('%s')).\nAvailable factors: %s", 
                       cat_vars[1], paste(cat_vars, collapse = ", ")), call. = FALSE)
        }
      }

      # Ensure consistent contrasts for reference grid construction
      old_opts <- NULL
      if (!is.null(self$model$contrasts)) {
        if (self$model$contrasts == "sum") {
          old_opts <- options(contrasts = c("contr.sum", "contr.poly"))
        } else if (self$model$contrasts == "treatment") {
          old_opts <- options(contrasts = c("contr.treatment", "contr.poly"))
        }
      }
      if (!is.null(old_opts)) on.exit(options(old_opts), add = TRUE)

      if (!is.null(self$model$raw_data)) {
        data <- as.data.frame(self$model$raw_data)
      } else if (!is.null(self$model$data)) {
        data <- as.data.frame(self$model$data)
      } else {
        stop("lsmeans requires the original data.")
      }
      formula <- self$model$formula

      # 1. Identify all factors in the model
      vars <- all.vars(nobars(formula))[-1]
      is_cat <- sapply(data[vars], function(x) is.factor(x) || is.character(x))
      cat_vars <- vars[is_cat]

      full_specs <- unique(c(specs, simple))
      if (!all(full_specs %in% cat_vars)) stop("specs/simple must be categorical factors in the model.")

      # 2. Create a reference grid for all categorical factors
      # Reorder variables so that specs vary fastest and simple varies slowest among the targeted specs
      other_vars <- setdiff(cat_vars, full_specs)
      ordered_grid_vars <- c(other_vars, full_specs)
      grid_list <- lapply(data[ordered_grid_vars], function(x) levels(as.factor(x)))
      ref_grid <- expand.grid(grid_list)

      # 3. Handle continuous covariates by setting them to their mean
      cont_vars <- vars[!is_cat]
      for (v in cont_vars) {
        ref_grid[[v]] <- mean(data[[v]], na.rm = TRUE)
      }

      # 4. Generate model matrix for the reference grid
      mf_orig <- model.frame(nobars(formula), data)
      orig_terms <- delete.response(terms(mf_orig))

      ct_setting <- if (!is.null(self$model$contrasts)) self$model$contrasts else "sum"
      ct_list <- list()
      for (v in cat_vars) {
        ct_list[[v]] <- if (ct_setting == "treatment") "contr.treatment" else "contr.sum"
      }

      X_grid_raw <- model.matrix(orig_terms, ref_grid, contrasts.arg = ct_list)

      # Match columns with the original model
      orig_cols <- self$model$extra$X_colnames
      if (!is.null(orig_cols)) {
        grid_cols <- colnames(X_grid_raw)
        # Normalize BOTH to 'Intercept' for matching
        grid_cols[grid_cols == "(Intercept)"] <- "Intercept"
        orig_cols_norm <- orig_cols
        orig_cols_norm[orig_cols_norm == "(Intercept)"] <- "Intercept"
        
        col_idx <- match(orig_cols_norm, grid_cols)
        if (any(is.na(col_idx))) {
          missing_cols <- orig_cols[is.na(col_idx)]
          stop(sprintf("Could not match lsmeans grid columns.\nMissing from grid: %s\nAvailable in grid: %s", 
                       paste(missing_cols, collapse = ", "), 
                       paste(grid_cols, collapse = ", ")), call. = FALSE)
        }
        X_grid <- X_grid_raw[, col_idx, drop = FALSE]
      } else {
        X_grid <- X_grid_raw
      }

      beta_vals <- if (is.data.frame(self$fit)) self$fit$Estimate else stats::coef(self$fit)
      fe_idx <- which(grepl("^(Intercept|Intercept_c|b\\[)", rownames(self$fit)))
      beta_match <- beta_vals[fe_idx]
      fe_names_match <- rownames(self$fit)[fe_idx]
      V_match <- self$vcov[fe_names_match, fe_names_match, drop = FALSE]

      # Group by specs + simple with descriptive labels
      label_grid <- ref_grid[full_specs]
      for (v in full_specs) {
        label_grid[[v]] <- paste0(v, "=", label_grid[[v]])
      }
      groups_full <- interaction(label_grid, drop = TRUE, sep = ":")
      levels(groups_full) <- unique(as.character(groups_full))
      unique_groups_full <- levels(groups_full)

      # Calculate L-vectors for each group
      L_list <- list()
      for (grp in unique_groups_full) {
        idx <- which(groups_full == grp)
        L_list[[grp]] <- colMeans(X_grid[idx, , drop = FALSE])
      }

      if (!pairwise) {
        # --- Marginal Means ---
        res_list <- list()
        for (grp in unique_groups_full) {
          L <- L_list[[grp]]
          est <- as.numeric(L %*% beta_match)
          se <- sqrt(as.numeric(t(L) %*% V_match %*% L))
          df_val <- self$.get_lsmeans_df(specs)

          res_list[[grp]] <- data.frame(
            estimate = est, `Std. Error` = se, df = df_val,
            `Lower 95%` = est + qt(0.025, df_val) * se,
            `Upper 95%` = est + qt(0.975, df_val) * se,
            row.names = grp, check.names = FALSE
          )
        }
        res <- do.call(rbind, res_list)
        class(res) <- c("rtmb_lsmeans", class(res))
        return(res)
      } else {
        # --- Pairwise Comparisons & Simple Main Effects ---
        results_by_group <- list()
        
        # Moderator levels
        if (!is.null(simple)) {
          mod_grid <- unique(ref_grid[simple])
          focal_grid <- unique(ref_grid[specs])

          for (row_m in 1:nrow(mod_grid)) {
            m_vals <- mod_grid[row_m, , drop = FALSE]
            m_label <- paste(sapply(1:ncol(m_vals), function(k) paste0(names(m_vals)[k], "=", m_vals[1, k])), collapse = ":")
            
            # --- Simple Main Effect (SME) ---
            L_group <- list()
            group_names <- c()
            for (i in 1:nrow(focal_grid)) {
                f_label <- paste(sapply(1:ncol(focal_grid), function(k) paste0(names(focal_grid)[k], "=", focal_grid[i, k])), collapse=":")
                nm <- paste(c(f_label, m_label), collapse = ":")
                L_group[[i]] <- L_list[[nm]]
                group_names[i] <- nm
            }
            
            if (length(L_group) > 1) {
              L_sme <- as.matrix(do.call(rbind, lapply(2:length(L_group), function(i) L_group[[i]] - L_group[[1]])))
              V_match_mat <- as.matrix(V_match)
              
              est_sme <- as.numeric(L_sme %*% beta_match)
              V_sme <- L_sme %*% V_match_mat %*% t(L_sme)
              W_val <- as.numeric(t(est_sme) %*% MASS::ginv(V_sme) %*% est_sme)
              num_df <- nrow(L_sme)
              den_df <- self$.get_lsmeans_df(specs)
              f_val <- W_val / num_df
              p_sme <- pf(f_val, num_df, den_df, lower.tail = FALSE)
            } else {
              f_val <- NA; num_df <- 0; den_df <- self$.get_lsmeans_df(specs); p_sme <- NA
            }
            
            sme_row <- data.frame(
               Source = paste("Simple Main Effect of", paste(specs, collapse=":")),
               `F value` = f_val, num_df = num_df, den_df = den_df, Pr = p_sme,
               check.names = FALSE
            )
            
            group_contrasts <- list()
            for (i in 1:(nrow(focal_grid)-1)) {
              for (j in (i+1):nrow(focal_grid)) {
                L_diff <- L_group[[i]] - L_group[[j]]
                contrast_label <- paste0("(", group_names[i], ") - (", group_names[j], ")")
                group_contrasts[[contrast_label]] <- self$.calc_contrast(L_diff, specs)
              }
            }
            pairwise_df <- do.call(rbind, group_contrasts)
            if (adjust != "none") pairwise_df$Pr <- p.adjust(pairwise_df$Pr, method = adjust)
            if (protect && !is.na(p_sme) && p_sme > 0.05) pairwise_df$Pr <- 1.0
            
            results_by_group[[m_label]] <- list(sme = sme_row, pairwise = pairwise_df)
          }
        } else {
           group_contrasts <- list()
           for (i in 1:(length(unique_groups_full)-1)) {
             for (j in (i+1):length(unique_groups_full)) {
               name_i <- unique_groups_full[i]
               name_j <- unique_groups_full[j]
               L_diff <- L_list[[name_i]] - L_list[[name_j]]
               contrast_label <- paste0("(", name_i, ") - (", name_j, ")")
               group_contrasts[[contrast_label]] <- self$.calc_contrast(L_diff, specs)
             }
           }
           pairwise_df <- do.call(rbind, group_contrasts)
           if (adjust != "none") pairwise_df$Pr <- p.adjust(pairwise_df$Pr, method = adjust)
           results_by_group[["All"]] <- list(pairwise = pairwise_df)
        }

        class(results_by_group) <- c("rtmb_lsmeans_grouped", "list")
        attr(results_by_group, "adjustment") <- adjust
        attr(results_by_group, "protect") <- protect
        return(results_by_group)
      }
    },

    #' @description (Internal) Calculate metrics for a contrast.
    #' @param L Contrast matrix.
    #' @param specs Variable names for lsmeans.
    #' @return A data frame with estimate, SE, df, t-value, and p-value.
    .calc_contrast = function(L, specs) {
      beta_vals <- if (is.data.frame(self$fit)) self$fit$Estimate else stats::coef(self$fit)
      fe_idx <- which(grepl("^(Intercept|Intercept_c|b\\[)", rownames(self$fit)))
      beta_match <- beta_vals[fe_idx]
      V_match <- self$vcov[fe_idx, fe_idx]
      est <- as.numeric(L %*% beta_match)
      se <- sqrt(as.numeric(t(L) %*% V_match %*% L))
      df_val <- self$.get_lsmeans_df(specs)
      t_val <- est / pmax(se, 1e-12)
      p_val <- 2 * pt(-abs(t_val), df = df_val)
      data.frame(estimate = est, `Std. Error` = se, df = df_val, `t value` = t_val, Pr = p_val, check.names = FALSE)
    },

    #' @description (Internal) Get representative DF for lsmeans.
    #' @param specs Variable names for lsmeans.
    #' @return Degrees of freedom.
    .get_lsmeans_df = function(specs) {
      df_val <- Inf
      if (is.data.frame(self$fit) && !is.null(self$fit$df)) {
        match_pattern <- paste0("^b\\[(", paste(specs, collapse="|"), ")")
        match_idx <- grepl(match_pattern, rownames(self$fit))
        if (any(match_idx)) df_val <- min(self$fit$df[match_idx], na.rm = TRUE)
      } else if (inherits(self$fit, "lm")) {
        df_val <- self$fit$df.residual
      }
      return(df_val)
    },

    #' @description (Internal) Construct a list of parameters from the fit.
    #' @param fit The fit result (dataframe or lm object).
    #' @return A named list of parameters.
    .construct_par_list = function(fit) {
      par_list <- list()
      if (inherits(fit, "lm")) {
        coefs <- stats::coef(fit)
        par_list$Intercept <- coefs["(Intercept)"]
        par_list$b <- coefs[names(coefs) != "(Intercept)"]
        return(par_list)
      }
      if (is.data.frame(fit) && "Estimate" %in% names(fit)) {
        est <- fit$Estimate; names(est) <- rownames(fit)
        par_list$Intercept <- est["Intercept"]
        par_list$b <- est[grepl("^b\\[", names(est))]
        par_list$sd <- est[grepl("^sd\\[", names(est))]
        par_list$sigma <- est[grepl("^sigma", names(est))]
        if (!is.null(self$model$type) && self$model$type == "corr") {
          par_list$corr <- est[grepl("^corr\\[", names(est))]
        }
        par_list$IE <- est[grepl("^IE_", names(est))]
      }
      return(par_list)
    }
  ),
  private = list(
    .rbind_result_dfs = function(dfs) {
      dfs <- dfs[vapply(dfs, is.data.frame, logical(1))]
      if (length(dfs) == 0L) return(data.frame())

      all_cols <- unique(unlist(lapply(dfs, names), use.names = FALSE))
      dfs <- lapply(dfs, function(df) {
        missing_cols <- setdiff(all_cols, names(df))
        for (col in missing_cols) df[[col]] <- NA
        df[, all_cols, drop = FALSE]
      })

      do.call(rbind, unname(dfs))
    },
    .update_fixed_effects_with_vcov = function(V) {
      se <- sqrt(diag(V))

      for (nm in names(se)) {
        if (!nm %in% rownames(self$fit)) next

        self$fit[nm, "Std. Error"] <- se[[nm]]

        stat_col <- if ("z value" %in% names(self$fit)) "z value" else "t value"
        self$fit[nm, stat_col] <- self$fit[nm, "Estimate"] / pmax(se[[nm]], 1e-12)

        if ("df" %in% names(self$fit) && is.finite(self$fit[nm, "df"])) {
          crit <- qt(0.975, df = self$fit[nm, "df"])
          self$fit[nm, "Pr"] <- 2 * pt(-abs(self$fit[nm, stat_col]), df = self$fit[nm, "df"])
        } else {
          crit <- qnorm(0.975)
          self$fit[nm, "Pr"] <- 2 * pnorm(-abs(self$fit[nm, stat_col]))
        }

        self$fit[nm, "Lower 95%"] <- self$fit[nm, "Estimate"] - crit * se[[nm]]
        self$fit[nm, "Upper 95%"] <- self$fit[nm, "Estimate"] + crit * se[[nm]]
      }

      self$df_fixed <- self$fit
      invisible(self)
    },
    .compute_lm_robust_se = function(cluster = NULL, type = c("HC3", "HC0", "HC1", "CR1", "CR0"), update = TRUE) {
      type <- match.arg(type)
      if (is.null(cluster) && type %in% c("CR1", "CR0")) type <- "HC3"
      if (!is.null(cluster) && type %in% c("HC3", "HC0", "HC1")) type <- "CR1"
      self$se_method <- if (is.null(cluster)) "robust" else "cluster-robust"
      self$cluster <- cluster

      formula <- nobars(self$model$formula)
      if (!is.null(self$model$raw_data)) {
        dat <- as.data.frame(self$model$raw_data)
      } else {
        dat <- as.data.frame(self$model$data)
      }
      mf <- model.frame(formula, dat)
      X <- model.matrix(formula, mf)
      y <- model.response(mf)

      if (is.data.frame(self$fit)) {
        beta_all <- self$fit$Estimate
        names(beta_all) <- rownames(self$fit)
        fe_names_in_fit <- names(beta_all)[grepl("^(Intercept|Intercept_c|b\\[)", names(beta_all))]
      } else if (inherits(self$fit, "lm")) {
        beta_all <- stats::coef(self$fit)
        fe_names_in_fit <- names(beta_all)
      } else {
        stop(paste("Unsupported fit type:", class(self$fit)[1]))
      }

      if (length(fe_names_in_fit) == 0) stop("No fixed effects found in fit object.")

      X_cols <- colnames(X)
      idx_map <- integer(length(fe_names_in_fit))
      for (i in seq_along(fe_names_in_fit)) {
        fname <- fe_names_in_fit[i]
        pos <- which(X_cols == fname)
        if (length(pos) == 0) {
          if (tolower(fname) == "intercept" || fname == "intercept_c" || fname == "(intercept)") {
            pos <- which(tolower(X_cols) == "intercept" | tolower(X_cols) == "(intercept)")
          } else {
            vname <- gsub("^b\\[(.*)\\]$", "\\1", fname)
            pos <- which(X_cols == vname)
          }
        }
        if (length(pos) > 0) idx_map[i] <- pos[1]
      }

      keep <- idx_map > 0
      if (!any(keep)) stop("None of the fixed effects could be matched.")

      fe_names_in_fit <- fe_names_in_fit[keep]
      beta <- beta_all[fe_names_in_fit]
      idx_map <- idx_map[keep]
      X_subset <- X[, idx_map, drop = FALSE]

      XtX <- t(X_subset) %*% X_subset
      bread_ols <- tryCatch(solve(XtX), error = function(e) MASS::ginv(XtX))
      res <- as.numeric(y - X_subset %*% beta)

      if (is.null(cluster)) {
        h <- tryCatch(diag(X_subset %*% bread_ols %*% t(X_subset)), error = function(e) rep(0, length(res)))
        omega <- res^2
        if (type == "HC1") {
          n <- nrow(X_subset); k <- ncol(X_subset)
          omega <- omega * (n / pmax(n - k, 1))
        } else if (type == "HC3") {
          omega <- omega / pmax(1 - h, 1e-8)^2
        }
        meat <- t(X_subset) %*% (as.numeric(omega) * X_subset)
        V_final <- bread_ols %*% meat %*% bread_ols
      } else {
        if (!cluster %in% names(dat)) stop(paste("Cluster variable", cluster, "not found."))
        grp <- as.factor(dat[[cluster]])
        scores <- X_subset * as.numeric(res)
        cluster_scores <- aggregate(scores, by = list(grp), sum)[, -1, drop = FALSE]
        meat <- t(as.matrix(cluster_scores)) %*% as.matrix(cluster_scores)
        m <- length(unique(grp)); n <- nrow(X_subset); k <- ncol(X_subset)
        adj <- if (type == "CR1") (m / (m - 1)) * ((n - 1) / (n - k)) else 1
        V_final <- adj * bread_ols %*% meat %*% bread_ols
      }

      self$vcov <- V_final
      colnames(self$vcov) <- rownames(self$vcov) <- fe_names_in_fit

      self$se_method <- if (is.null(cluster)) "robust" else "cluster-robust"
      self$ci_method <- self$se_method
      self$cluster <- cluster
      self$robust_type <- type

      if (update) private$.update_fixed_effects_with_vcov(self$vcov)
      
      message(sprintf("%s standard errors calculated.", tools::toTitleCase(self$se_method)))
      return(self)
    },
      .compute_glm_robust_se = function(cluster = NULL, type = c("HC3", "HC0", "HC1", "CR1", "CR0"), update = TRUE) {
        type <- match.arg(type)
        if (is.null(cluster) && type %in% c("CR1", "CR0")) type <- "HC3"
        if (!is.null(cluster) && type %in% c("HC3", "HC0", "HC1")) type <- "CR1"

        formula <- nobars(self$model$formula)
        dat <- if (!is.null(self$model$raw_data)) {
          as.data.frame(self$model$raw_data)
        } else {
          as.data.frame(self$model$data)
        }
        mf <- stats::model.frame(formula, dat)
        X <- stats::model.matrix(formula, mf)
        y_raw <- stats::model.response(mf)

        beta_all <- if (is.data.frame(self$fit)) {
          est <- self$fit$Estimate
          names(est) <- rownames(self$fit)
          est
        } else {
          stats::coef(self$fit)
        }
        fe_names_in_fit <- names(beta_all)[grepl("^(Intercept|Intercept_c|b\\[)", names(beta_all))]
        if (length(fe_names_in_fit) == 0L) stop("No fixed effects found in fit object.", call. = FALSE)

        X_cols <- colnames(X)
        idx_map <- integer(length(fe_names_in_fit))
        for (i in seq_along(fe_names_in_fit)) {
          fname <- fe_names_in_fit[i]
          pos <- which(X_cols == fname)
          if (length(pos) == 0L) {
            if (tolower(fname) == "intercept" || fname == "intercept_c" || fname == "(intercept)") {
              pos <- which(tolower(X_cols) == "intercept" | tolower(X_cols) == "(intercept)")
            } else {
              vname <- gsub("^b\\[(.*)\\]$", "\\1", fname)
              pos <- which(X_cols == vname)
            }
          }
          if (length(pos) > 0L) idx_map[i] <- pos[1]
        }

        keep <- idx_map > 0
        if (!any(keep)) stop("None of the fixed effects could be matched.", call. = FALSE)
        fe_names_in_fit <- fe_names_in_fit[keep]
        beta <- beta_all[fe_names_in_fit]
        idx_map <- idx_map[keep]
        X_subset <- X[, idx_map, drop = FALSE]

        family <- self$model$family %||% "gaussian"
        family <- if (inherits(family, "family")) family$family else as.character(family)[1]
        if (!family %in% c("binomial", "bernoulli", "poisson", "gamma")) {
          stop(
            "robust_se() for glm currently supports binomial/bernoulli, poisson, and gamma families.",
            call. = FALSE
          )
        }

        prior_weights <- stats::model.weights(mf)
        if (is.null(prior_weights)) prior_weights <- rep(1, nrow(X_subset))

        if (family %in% c("binomial", "bernoulli")) {
          if (is.matrix(y_raw) && ncol(y_raw) == 2L) {
            binom_n <- rowSums(y_raw)
            y <- y_raw[, 1] / pmax(binom_n, 1)
            prior_weights <- prior_weights * binom_n
          } else {
            y <- as.numeric(y_raw)
            if (is.factor(y_raw)) y <- as.numeric(y_raw) - 1
          }
        } else {
          y <- as.numeric(y_raw)
        }

        eta <- as.numeric(X_subset %*% beta)
        if (family %in% c("binomial", "bernoulli")) {
          mu <- stats::plogis(eta)
          dmu_deta <- mu * (1 - mu)
          var_mu <- mu * (1 - mu)
        } else if (family == "poisson") {
          mu <- exp(eta)
          dmu_deta <- mu
          var_mu <- mu
        } else {
          mu <- exp(eta)
          dmu_deta <- mu
          var_mu <- mu^2
        }
        var_mu <- pmax(var_mu, 1e-12)

        W <- as.numeric(prior_weights) * (dmu_deta^2) / var_mu
        bread <- tryCatch(solve(t(X_subset) %*% (W * X_subset)), error = function(e) MASS::ginv(t(X_subset) %*% (W * X_subset)))
        score <- X_subset * as.numeric(prior_weights * (y - mu) * dmu_deta / var_mu)

        if (is.null(cluster)) {
          if (type %in% c("HC1", "HC3")) {
            n <- nrow(X_subset); k <- ncol(X_subset)
            if (type == "HC1") {
              score <- score * sqrt(n / pmax(n - k, 1))
            } else {
              Xw <- X_subset * sqrt(W)
              h <- tryCatch(diag(Xw %*% bread %*% t(Xw)), error = function(e) rep(0, n))
              score <- score / pmax(1 - h, 1e-8)
            }
          }
          meat <- t(score) %*% score
          V_final <- bread %*% meat %*% bread
        } else {
          if (!cluster %in% names(dat)) stop(paste("Cluster variable", cluster, "not found."), call. = FALSE)
          grp <- as.factor(dat[[cluster]])
          cluster_scores <- stats::aggregate(score, by = list(grp), sum)[, -1, drop = FALSE]
          meat <- t(as.matrix(cluster_scores)) %*% as.matrix(cluster_scores)
          m <- length(unique(grp)); n <- nrow(X_subset); k <- ncol(X_subset)
          adj <- if (type == "CR1") (m / (m - 1)) * ((n - 1) / pmax(n - k, 1)) else 1
          V_final <- adj * bread %*% meat %*% bread
        }

        self$vcov <- V_final
        colnames(self$vcov) <- rownames(self$vcov) <- fe_names_in_fit
        self$se_method <- if (is.null(cluster)) "robust" else "cluster-robust"
        self$ci_method <- self$se_method
        self$cluster <- cluster
        self$robust_type <- type

        if (update) private$.update_fixed_effects_with_vcov(self$vcov)

        message(sprintf("%s standard errors calculated.", tools::toTitleCase(self$se_method)))
        return(self)
      },
      .compute_glmer_robust_se = function(cluster = NULL, type = c("CR1", "CR0"), update = TRUE) {
        type <- match.arg(type)
        info <- self$model$extra$glmer_info
        
        if (!is.null(self$map) && length(self$map) > 0L) {
          stop("robust_se() for mixed models with mapped/fixed parameters is not supported yet.", call. = FALSE)
        }
        
        if (info$num_bars != 1L) {
          stop("robust_se() for mixed models currently supports exactly one grouping factor.", call. = FALSE)
        }

        gvar <- info$group_labels[[1]]
        if (is.null(cluster)) cluster <- gvar
        if (!identical(cluster, gvar)) {
          stop(sprintf("robust_se() for mixed models currently requires `cluster` to be the random-effect grouping variable '%s'.", gvar), call. = FALSE)
        }

        raw_data <- if (!is.null(self$model$raw_data)) self$model$raw_data else self$model$data
        if (!cluster %in% names(raw_data)) stop(paste("Cluster variable", cluster, "not found."))
        
        grp_factor <- as.factor(raw_data[[cluster]])
        clusters <- levels(grp_factor)
        m <- length(clusters)
        n <- length(grp_factor)

        # Verify one-to-one mapping
        idx_name <- info$group_idx_names[1]
        re_grp_idx <- self$model$data[[idx_name]]
        tab <- table(re_grp_idx, grp_factor)
        if (any(rowSums(tab > 0) != 1L)) {
          stop("Each random-effect level must belong to exactly one robust cluster.", call. = FALSE)
        }

        obs_to_cluster <- as.integer(grp_factor)
        re_level_to_cluster <- as.integer(apply(tab > 0, 1, function(x) which(x > 0)[1]))

        # 3. Build AD object for score calculation
        message("Preparing parameters for robust AD object...")
        
        tmp_par_list <- self$model$par_list
        tmp_par_list[[info$obs_weight_name]] <- list(
          type = "vector", length = n, dim = n, unc_length = n,
          bounds = "none", random = FALSE
        )
        for (b in 1:info$num_bars) {
          L_w <- info$num_groups_list[[b]]
          tmp_par_list[[info$weight_names[b]]] <- list(
            type = "vector", length = L_w, dim = L_w, unc_length = L_w,
            bounds = "none", random = FALSE
          )
        }

        tmp_data <- self$model$data
        tmp_data[[info$obs_weight_name]] <- NULL
        for (wn in info$weight_names) tmp_data[[wn]] <- NULL

        # Initial unconstrained parameters
        init_unc_list <- self$model$unconstrained_vector_to_list(self$par_unc)
        init_unc_list[[info$obs_weight_name]] <- rep(1, n)
        for (b in 1:info$num_bars) {
          init_unc_list[[info$weight_names[b]]] <- rep(1, info$num_groups_list[[b]])
        }

        random_names <- names(self$model$par_list)[vapply(self$model$par_list, function(p) isTRUE(p$random), logical(1))]
        
        f_ad_score <- function(y_unc_list) {
          para <- to_constrained(y_unc_list, tmp_par_list)
          -self$model$log_prob(dat = tmp_data, par = para)
        }

        obj_score <- RTMB::MakeADFun(
          func = f_ad_score,
          parameters = init_unc_list,
          random = random_names,
          map = self$map,
          intern = TRUE,
          silent = TRUE
        )
        message("AD object built. Calculating Hessian...")
        H <- obj_score$he(obj_score$par)
        message("Hessian calculated.")
        
        fe_pars <- intersect(c("Intercept", "Intercept_c", "b"), names(self$model$par_list))
        k_p <- 0
        for (nm in fe_pars) k_p <- k_p + self$model$par_list[[nm]]$unc_length
        
        n_weights <- n + sum(unlist(info$num_groups_list))
        n_total <- length(obj_score$par)
        n_theta <- n_total - n_weights
        theta_indices <- 1:n_theta
        
        meat <- matrix(0, n_theta, n_theta)
        
        current_pos <- n_theta + 1
        obs_weight_indices <- current_pos:(current_pos + n - 1)
        current_pos <- current_pos + n
        re_weight_indices <- list()
        for (b in 1:info$num_bars) {
          n_lev <- info$num_groups_list[[b]]
          re_weight_indices[[b]] <- current_pos:(current_pos + n_lev - 1)
          current_pos <- current_pos + n_lev
        }
        
        for (i in 1:m) {
          s_i <- rep(0, n_theta)
          obs_idx <- which(obs_to_cluster == i)
          if (length(obs_idx) > 0) {
            s_i <- s_i + rowSums(H[theta_indices, obs_weight_indices[obs_idx], drop = FALSE])
          }
          for (b in 1:info$num_bars) {
            lev_idx <- which(re_level_to_cluster == i)
            if (length(lev_idx) > 0) {
              s_i <- s_i + rowSums(H[theta_indices, re_weight_indices[[b]][lev_idx], drop = FALSE])
            }
          }
          meat <- meat + s_i %*% t(s_i)
        }
        
        H_theta <- H[theta_indices, theta_indices, drop = FALSE]
        H_inv <- tryCatch(solve(H_theta), error = function(e) MASS::ginv(H_theta))
        
        # 4. Final Calculations and Object Updates
        if (type == "CR1") {
          if (m <= 1L) {
            stop("CR1 robust SE requires at least two clusters.", call. = FALSE)
          }
          if (n <= k_p) {
            warning("CR1 finite-sample correction skipped because n <= p.", call. = FALSE)
            adj <- m / (m - 1)
          } else {
            adj <- (m / (m - 1)) * ((n - 1) / (n - k_p))
          }
        } else {
          adj <- 1
        }
        
        V_unc <- adj * H_inv %*% meat %*% H_inv
        
        # Correctly identify fixed-effect indices from AD object names
        par_names <- names(obj_score$par)
        fe_idx <- which(par_names %in% c("Intercept", "Intercept_c", "b"))
        
        # Standardized labels for summary alignment
        fe_labels <- unlist(lapply(fe_pars, function(nm) {
          generate_flat_names(
            base_name = nm,
            dims = self$model$par_list[[nm]]$dim,
            names_def = self$model$par_names[[nm]]
          )
        }), use.names = FALSE)
        
        if (length(fe_idx) != length(fe_labels)) {
          stop("Failed to align fixed-effect indices with displayed coefficient names.", call. = FALSE)
        }
        
        V_beta <- V_unc[fe_idx, fe_idx, drop = FALSE]
        rownames(V_beta) <- colnames(V_beta) <- fe_labels
        
        # Update object metadata
        self$vcov <- V_beta
        self$se_method <- "cluster-robust"
        self$ci_method <- "cluster-robust"
        self$cluster <- cluster
        self$robust_type <- type
        
        if (update) private$.update_fixed_effects_with_vcov(V_beta)
        
        message(sprintf("%s standard errors calculated.", tools::toTitleCase(self$se_method)))
        return(self)
      }
    )
  )

#' @export
summary.Classic_Fit <- function(object, ...) {
  object$summary(...)
}

#' @export
print.summary_Classic_Fit <- function(x, ...) {
  digits <- if (!is.null(x$digits)) x$digits else 5
  if (!is.null(x$type) && x$type == "table") { cat("\nContingency Table Analysis\n")
  } else if (!is.null(x$type) && x$type == "loglinear") { cat("\nLog-Linear Model Analysis\n")
  } else { cat("\nCall:\n"); type_label <- if (is.null(x$type)) "Generic Model" else x$type; cat(paste("Classical estimation via", type_label), "\n") }
  if (!is.null(x$se_method) && x$se_method != "wald") {
    method_label <- switch(
      x$se_method,
      robust = if (!is.null(x$cluster)) {
        paste("Robust (Cluster:", x$cluster, ")")
      } else {
        paste0("Robust (", x$robust_type %||% "HC3", ")")
      },
      `cluster-robust` = paste0(
        "Cluster-robust",
        if (!is.null(x$cluster)) paste0(" (cluster = ", x$cluster, ")") else "",
        if (!is.null(x$robust_type)) paste0(", ", x$robust_type) else ""
      ),
      bootstrap = paste0("Bootstrap (", x$bootstrap, " samples)"),
      "Standard"
    )
    cat(sprintf("Standard Errors: %s\n", method_label))
  }
  if (!is.null(x$logLik) && !is.na(x$logLik)) cat(sprintf("\nLog-Likelihood: %.3f, AIC: %.3f, BIC: %.3f\n", as.numeric(x$logLik), x$AIC, x$BIC))
  if (!is.null(x$coefficients)) {
    if (!is.null(x$type) && x$type == "table") {
      cat("\n---\n"); obs_tab <- x$extra$tab; N_tot <- sum(obs_tab); R_dim <- nrow(obs_tab); C_dim <- ncol(obs_tab)
      df_mu <- x$coefficients[grepl("^mu(\\[|$)", rownames(x$coefficients)), , drop = FALSE]
      if (nrow(df_mu) == R_dim * C_dim) {
        p_est <- df_mu$Estimate / N_tot; p_mat <- matrix(p_est, nrow = R_dim, ncol = C_dim); p_row <- rowSums(p_mat); p_col <- colSums(p_mat)
        E_mat <- matrix(0, nrow = R_dim, ncol = C_dim); for (i in 1:R_dim) for (j in 1:C_dim) E_mat[i, j] <- p_row[i] * p_col[j] * N_tot
        cat("Cell Probabilities (p) and Confidence Intervals:\n"); df_p <- df_mu
         # Correct order: Calculate then Round
         df_p$Estimate <- round(df_mu$Estimate / N_tot, digits)
         df_p$`Std. Error` <- round(df_mu$`Std. Error` / N_tot, digits)
         if ("Lower 95%" %in% names(df_p)) { 
           df_p$`Lower 95%` <- round(df_mu$`Lower 95%` / N_tot, digits)
           df_p$`Upper 95%` <- round(df_mu$`Upper 95%` / N_tot, digits)
         }
         rownames(df_p) <- gsub("^mu\\[", "p[", rownames(df_p)); cols_to_show <- intersect(c("Estimate", "Std. Error", "Lower 95%", "Upper 95%", "z value", "t value", "Pr"), names(df_p))
         print(df_p[, cols_to_show, drop = FALSE], quote = FALSE, right = TRUE)
         cat("\nExpected Counts (Independence) and Pearson Residuals:\n"); E_vec <- as.vector(E_mat); obs_vec <- as.vector(obs_tab); residuals <- (obs_vec - E_vec) / pmax(sqrt(E_vec), 1e-8)
         df_resid <- data.frame(Expected = round(E_vec, digits), Residual = round(residuals, digits), row.names = gsub("^mu\\[", "E\\[", rownames(df_mu)))
        print(df_resid, quote = FALSE, right = TRUE)
      } else {
        # Fallback for classic mode or models without mu parameters
        cat("\nTest Results:\n")
        print(x$coefficients, quote = FALSE, right = TRUE)
      }
    } else if (!is.null(x$type) && x$type == "loglinear") {
      cat("\nLog-linear Parameters and Confidence Intervals:\n"); df_params <- x$coefficients[!grepl("^mu\\[", rownames(x$coefficients)), , drop = FALSE]
      print(df_params[, intersect(c("Estimate", "Std. Error", "Lower 95%", "Upper 95%", "z value", "t value", "Pr"), names(df_params)), drop = FALSE], quote = FALSE, right = TRUE)
      df_mu <- x$coefficients[grepl("^mu\\[", rownames(x$coefficients)), , drop = FALSE]
      if (nrow(df_mu) > 0 && !is.null(x$extra$obs_Y)) {
        cat("\n---\nExpected Counts (Fitted) and Pearson Residuals:\n"); E_vec <- df_mu$Estimate; obs_vec <- x$extra$obs_Y; residuals <- (obs_vec - E_vec) / pmax(sqrt(E_vec), 1e-8)
        df_resid <- data.frame(Observed = obs_vec, Expected = round(E_vec, digits), Residual = round(residuals, digits), row.names = rownames(df_mu))
        max_r <- if (!is.null(x$max_rows)) x$max_rows else 20
        if (nrow(df_resid) > max_r) { print(df_resid[1:max_r, , drop=FALSE], quote=FALSE, right=TRUE); cat(sprintf("... (omitted %d rows)\n", nrow(df_resid) - max_r))
        } else { print(df_resid, quote = FALSE, right = TRUE) }
      }
    } else {
      cat("\nPoint Estimates and Confidence Intervals:\n"); df_to_print <- x$coefficients
      if (isTRUE(x$truncated)) df_to_print <- df_to_print[1:x$max_rows, , drop = FALSE]
      print(df_to_print, quote = FALSE, right = TRUE)
      if (isTRUE(x$truncated)) cat(sprintf("... (omitted %d parameters; use summary(max_rows = ...) to show more)\n", x$total_rows - x$max_rows))
    }
    if (!is.null(x$lm_info)) {
      cat("\n---\n"); info <- x$lm_info
      if (!is.null(info$dispersion)) { cat(sprintf("Dispersion parameter for %s family taken to be %s\n", x$family, format(round(info$dispersion, digits), nsmall = digits)))
        cat(sprintf("Null deviance: %s on %d degrees of freedom\n", format(round(info$null_deviance, 2), nsmall = 2), info$df_null))
        cat(sprintf("Residual deviance: %s on %d degrees of freedom\n", format(round(info$deviance, 2), nsmall = 2), info$df_residual))
      } else if (!is.null(info$sigma)) { cat(sprintf("Residual standard error: %s on %d degrees of freedom\n", format(round(info$sigma, digits), nsmall = digits), info$df_residual))
        cat(sprintf("Multiple R-squared: %s, Adjusted R-squared: %s\n", format(round(info$r_squared, 4), nsmall = 4), format(round(info$adj_r_squared, 4), nsmall = 4)))
        if (!is.null(info$fstatistic)) { f <- info$fstatistic; p_f <- stats::pf(f[1], f[2], f[3], lower.tail = FALSE)
          cat(sprintf("F-statistic: %s on %d and %d DF, p-value: %s\n", format(round(f[1], 2), nsmall = 2), f[2], f[3], if (p_f < 0.001) "< .001" else format(round(p_f, 4), nsmall = 4)))
        }
      }
    }
  } else if (!is.null(x$fit_summary)) { print(x$fit_summary) }
  invisible(x)
}

#' @export
print.anova_rtmb_table <- function(x, ...) { cat("\nContingency Table Analysis Tests\n- \n"); if (!is.null(x$chisq)) print(x$chisq, row.names = TRUE, right = TRUE)
  if (!is.null(x$fisher) && !inherits(x$fisher, "try-error")) { cat("\nFisher's Exact Test for Count Data\np-value = ", sprintf("%.5f", x$fisher$p.value), "\n") }
  invisible(x)
}

#' @export
anova.Classic_Fit <- function(object, ...) {
  dots <- list(...)
  is_fit <- vapply(dots, inherits, logical(1), what = "Classic_Fit")

  if (length(dots) > 0L && any(is_fit)) {
    if (!all(is_fit)) {
      stop("When comparing models, all arguments after `object` must be Classic_Fit objects.", call. = FALSE)
    }

    call <- match.call(expand.dots = FALSE)
    dot_exprs <- as.list(call$...)
    labels <- c(deparse1(call$object), vapply(dot_exprs, deparse1, character(1)))
    fits <- c(list(object), dots)
    if (all(vapply(fits, .is_classic_lm_fit, logical(1)))) {
      return(.anova_classic_lm_f(fits, labels = labels))
    }
    return(.anova_classic_lrt(fits, labels = labels))
  }

  object$anova(...)
}

.is_classic_lm_fit <- function(x) {
  inherits(x, "Classic_Fit") &&
    !is.null(x$model) &&
    identical(x$model$type, "lm") &&
    is.finite(x$rss) &&
    is.finite(x$df_residual)
}

.anova_classic_lm_f <- function(fits, labels = NULL) {
  if (length(fits) < 2L) {
    stop("At least two Classic_Fit objects are required for model comparison.", call. = FALSE)
  }
  if (is.null(labels) || length(labels) != length(fits)) {
    labels <- paste0("Model ", seq_along(fits))
  }

  ll <- lapply(fits, stats::logLik)
  loglik <- vapply(ll, as.numeric, numeric(1))
  df <- vapply(ll, function(x) as.numeric(attr(x, "df"))[1], numeric(1))
  nobs <- vapply(ll, function(x) as.numeric(attr(x, "nobs"))[1], numeric(1))
  rss <- vapply(fits, function(x) as.numeric(x$rss)[1], numeric(1))
  res_df <- vapply(fits, function(x) as.numeric(x$df_residual)[1], numeric(1))

  if (length(unique(nobs[!is.na(nobs)])) > 1L) {
    warning("Models appear to have different numbers of observations; F tests may not be valid.", call. = FALSE)
  }

  ord <- order(df, rss, decreasing = c(FALSE, TRUE))
  labels <- labels[ord]
  fits <- fits[ord]
  loglik <- loglik[ord]
  df <- df[ord]
  rss <- rss[ord]
  res_df <- res_df[ord]

  df_diff <- c(NA_real_, diff(df))
  rss_diff <- c(NA_real_, -diff(rss))
  f_val <- rep(NA_real_, length(fits))
  p_val <- rep(NA_real_, length(fits))
  scale <- rss[length(rss)] / res_df[length(res_df)]
  df2 <- res_df[length(res_df)]
  ok <- !is.na(df_diff) & df_diff > 0 & !is.na(rss_diff) & rss_diff >= 0 &
    is.finite(scale) & scale > 0 & is.finite(df2) & df2 > 0
  f_val[ok] <- (rss_diff[ok] / df_diff[ok]) / scale
  p_val[ok] <- stats::pf(f_val[ok], df1 = df_diff[ok], df2 = df2, lower.tail = FALSE)

  res <- data.frame(
    Model = labels,
    Df = df,
    Res.Df = res_df,
    RSS = rss,
    `Df Diff` = df_diff,
    `Sum of Sq` = rss_diff,
    F = f_val,
    `Pr(>F)` = p_val,
    logLik = loglik,
    AIC = vapply(fits, stats::AIC, numeric(1)),
    BIC = vapply(fits, stats::BIC, numeric(1)),
    check.names = FALSE
  )
  rownames(res) <- seq_len(nrow(res))
  class(res) <- c("anova_classic_lm_f", "anova", "data.frame")
  attr(res, "heading") <- "Analysis of Variance Table (F tests)"
  res
}

.anova_classic_lrt <- function(fits, labels = NULL) {
  if (length(fits) < 2L) {
    stop("At least two Classic_Fit objects are required for model comparison.", call. = FALSE)
  }

  if (is.null(labels) || length(labels) != length(fits)) {
    labels <- paste0("Model ", seq_along(fits))
  }

  ll <- lapply(fits, stats::logLik)
  loglik <- vapply(ll, as.numeric, numeric(1))
  df <- vapply(ll, function(x) {
    val <- attr(x, "df")
    if (is.null(val) || length(val) == 0L) NA_real_ else as.numeric(val)[1]
  }, numeric(1))
  nobs <- vapply(ll, function(x) {
    val <- attr(x, "nobs")
    if (is.null(val) || length(val) == 0L) NA_real_ else as.numeric(val)[1]
  }, numeric(1))

  if (anyNA(loglik)) {
    stop("Likelihood-ratio comparison requires all models to have a finite logLik().", call. = FALSE)
  }
  if (anyNA(df)) {
    stop("Likelihood-ratio comparison requires all logLik() values to have a valid df attribute.", call. = FALSE)
  }

  known_nobs <- nobs[!is.na(nobs)]
  if (length(unique(known_nobs)) > 1L) {
    warning("Models appear to have different numbers of observations; likelihood-ratio tests may not be valid.", call. = FALSE)
  }

  ord <- order(df, loglik)
  labels <- labels[ord]
  loglik <- loglik[ord]
  df <- df[ord]
  fits <- fits[ord]

  aic <- vapply(fits, stats::AIC, numeric(1))
  bic <- vapply(fits, stats::BIC, numeric(1))
  chisq <- c(NA_real_, 2 * diff(loglik))
  df_diff <- c(NA_real_, diff(df))
  p_val <- rep(NA_real_, length(fits))
  ok <- !is.na(chisq) & !is.na(df_diff) & df_diff > 0
  p_val[ok] <- stats::pchisq(chisq[ok], df = df_diff[ok], lower.tail = FALSE)

  res <- data.frame(
    Model = labels,
    Df = df,
    logLik = loglik,
    AIC = aic,
    BIC = bic,
    Chisq = chisq,
    `Chi Df` = df_diff,
    `Pr(>Chisq)` = p_val,
    check.names = FALSE
  )
  rownames(res) <- seq_len(nrow(res))
  class(res) <- c("anova_classic_lrt", "anova", "data.frame")
  attr(res, "heading") <- "Likelihood Ratio Tests"
  res
}

#' @export
print.anova_classic_lrt <- function(x, digits = max(3L, getOption("digits") - 2L), ...) {
  heading <- attr(x, "heading")
  if (!is.null(heading)) cat(heading, "\n", sep = "")
  x_print <- x
  class(x_print) <- "data.frame"
  print(x_print, digits = digits, na.print = "", row.names = FALSE, ...)
  invisible(x)
}

#' @export
print.anova_classic_lm_f <- print.anova_classic_lrt

#' @export
print.Classic_Fit <- function(x, ...) x$print(...)

#' Least Squares Means (Marginal Means)
#'
#' @description
#' Calculate least squares means (also known as marginal means or predicted means)
#' for categorical factors in a fitted model. It also supports pairwise comparisons
#' and simple main effects.
#'
#' @param object A fitted model object (e.g., `Classic_Fit`).
#' @param specs A character vector of factor names to calculate means for.
#' @param ... Additional arguments passed to the method.
#' @export
lsmeans <- function(object, specs, ...) UseMethod("lsmeans")

#' @export
lsmeans.Classic_Fit <- function(object, specs, ...) object$lsmeans(specs, ...)

#' @export
print.rtmb_lsmeans <- function(x, ...) {
  fmt_df <- function(v) sapply(round(v, 1), function(z) sprintf("%g", z))
  fmt_5 <- function(v) sprintf("%.5f", v)
  
  disp <- as.data.frame(x)
  
  cols_5 <- intersect(names(disp), c("estimate", "Std. Error", "Lower 95%", "Upper 95%"))
  for (col in cols_5) disp[[col]] <- fmt_5(disp[[col]])
  
  if ("df" %in% names(disp)) disp$df <- fmt_df(disp$df)
  
  print(disp, ...)
  invisible(x)
}

#' @export
print.rtmb_lsmeans_grouped <- function(x, ...) {
  adj <- attr(x, "adjustment"); prot <- attr(x, "protect"); cat("\nPost-hoc Analysis: Simple Main Effects and Pairwise Comparisons\n")
  cat(sprintf("P-value adjustment: %s (applied within groups)\n", adj)); if (prot) cat("Hierarchical testing (Protected): Pairwise results only shown if SME is significant.\n")
  fmt_df <- function(v) sapply(round(v, 1), function(z) sprintf("%g", z)); fmt_5 <- function(v) sprintf("%.5f", v); fmt_p <- function(v) sub("^0", "", sprintf("%.5f", v))
  for (grp in names(x)) { cat("\n", rep("-", nchar(grp) + 10), "\nGroup: ", grp, "\n", rep("-", nchar(grp) + 10), "\n", sep="")
    if (!is.null(x[[grp]]$sme)) { cat("\nSimple Main Effect (Wald F-test):\n"); sme_disp <- x[[grp]]$sme; sme_disp$`F value` <- fmt_5(sme_disp$`F value`); sme_disp$den_df <- fmt_df(sme_disp$den_df); sme_disp$Pr <- fmt_p(sme_disp$Pr); print(sme_disp, row.names = FALSE) }
    cat("\nPairwise Comparisons:\n"); pw_disp <- x[[grp]]$pairwise; pw_disp$estimate <- fmt_5(pw_disp$estimate); pw_disp$`Std. Error` <- fmt_5(pw_disp$`Std. Error`); pw_disp$df <- fmt_df(pw_disp$df); pw_disp$`t value` <- fmt_5(pw_disp$`t value`); pw_disp$Pr <- fmt_p(pw_disp$Pr); print(pw_disp); cat("\n")
  }
  invisible(x)
}

#' @export
logLik.Classic_Fit <- function(object, ...) object$logLik()

#' @export
AIC.Classic_Fit <- function(object, ..., k = 2) object$AIC()

#' @export
BIC.Classic_Fit <- function(object, ...) object$BIC()

#' @export
print.anova_rtmb <- function(x, ...) {
  # Use the standard anova print for the table
  # stats:::print.anova will handle the 'heading' attribute
  # Temporarily remove class to avoid infinite recursion
  x_copy <- x
  class(x_copy) <- c("anova", "data.frame")
  print(x_copy, ...)
  
  # Print LM info if available
  info <- attr(x, "lm_info")
  if (!is.null(info)) {
    cat("---\n")
    cat(sprintf("Residual standard error: %.4f on %.1f degrees of freedom\n", info$sigma, info$df_resid))
    cat(sprintf("Multiple R-squared: %.4f,  Adjusted R-squared: %.4f\n", info$r_squared, info$adj_r_squared))
    if (!is.null(info$fstatistic)) {
       cat(sprintf("F-statistic: %.4f on %d and %.1f DF,  p-value: %s\n", 
                   info$fstatistic["value"], 
                   as.integer(info$fstatistic["numdf"]), 
                   info$fstatistic["dendf"],
                   format.pval(info$p_value)))
    }
  }
  invisible(x)
}

#' Fit a Correlation Model using RTMB
#'
#' @description
#' `rtmb_corr` fits a correlation model to estimate means, standard deviations, and correlation structures.
#' It supports simple correlation, multilevel correlation, and classical frequentist estimation.
#'
#' @param x A matrix, data frame, formula, or expression (e.g., \code{cbind(V1, V2)}) of response variables.
#' @param data An optional data frame containing the variables.
#' @param ID A character string or expression specifying the group ID variable for multilevel models.
#' @param covariates Optional numeric matrix or data frame of covariates to be included in the joint MVN model.
#' @param method Correlation method for \code{classic()}: \code{"pearson"}, \code{"spearman"}, or \code{"reml"}.
#' @param prior Prior configuration object: `prior_flat()`, `prior_normal()`, or `prior_weak()`. Default is `prior_flat()`.
#' @param y_range Optional numeric vector or matrix defining the theoretical range (min, max) of response variables.
#' Required when using \code{prior_weak()}. Can be a vector of length 2 (applies to all variables) or a matrix/list of length P.
#' @param init Optional list of initial values.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param missing Missing value handling strategy: "listwise" (default), "pairwise", or "fiml" (Full Information Maximum Likelihood).
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#'
#' @example inst/examples/ex_corr.R
#' @export
rtmb_corr <- function(x = NULL, data = NULL, ID = NULL,
                      covariates = NULL,
                       method = c("pearson", "spearman", "reml"),
                       prior = prior_flat(), y_range = NULL,
                       init = NULL, fixed = NULL,
                       missing = c("listwise", "fiml", "pairwise"),
                       WAIC = FALSE) {

  missing <- match.arg(missing)
  method <- match.arg(method)
  if (missing == "pairwise" && method == "reml") {
    stop("missing = 'pairwise' is only available when method is 'pearson' or 'spearman'.")
  }
  if (isTRUE(WAIC) && missing != "listwise") {
    stop("WAIC = TRUE is currently supported for rtmb_corr() only with missing = 'listwise'.", call. = FALSE)
  }
  x_expr <- substitute(x)
  id_expr <- substitute(ID)
  data_eval <- data
  if (!is.null(data_eval) && is.matrix(data_eval)) {
    data_eval <- as.data.frame(data_eval)
  }

  eval_col_range <- function(expr, data_env) {
    if (is.null(data_env) || !is.call(expr)) return(NULL)
    data_names <- names(data_env)
    if (is.null(data_names)) return(NULL)

    range_expr <- expr
    if (identical(expr[[1]], as.name("cbind")) && length(expr) == 2L) {
      range_expr <- expr[[2]]
    }
    if (!is.call(range_expr) || !identical(range_expr[[1]], as.name(":"))) {
      return(NULL)
    }

    left <- as.character(range_expr[[2]])
    right <- as.character(range_expr[[3]])
    if (length(left) != 1L || length(right) != 1L) return(NULL)
    if (!left %in% data_names || !right %in% data_names) return(NULL)

    idx <- match(c(left, right), data_names)
    data_env[, seq.int(idx[1L], idx[2L]), drop = FALSE]
  }

  # Evaluation logic for response variables (Y_mat)
  if (!is.null(data_eval)) {
    # Evaluate x in the context of data (NSE support for cbind(a, b))
    Y_mat <- eval_col_range(x_expr, data_eval)
    if (is.null(Y_mat)) {
      Y_mat <- eval(x_expr, data_eval, parent.frame())
    }
    if (is.null(Y_mat)) {
      Y_mat <- data_eval
    }
  } else {
    Y_mat <- x
  }

  # Handle if Y_mat is a formula
  if (inherits(Y_mat, "formula")) {
    formula <- Y_mat
    lhs_expr <- formula[[2]]

    # 1. Try to evaluate LHS directly (handles cbind(y1, y2) or Y_df)
    Y_mat <- try(eval(lhs_expr, data_eval, parent.frame()), silent = TRUE)

    # 2. If eval(lhs) failed, try a safer model.frame call
    if (inherits(Y_mat, "try-error") || is.null(Y_mat)) {
      resp_formula <- bquote(.(lhs_expr) ~ 1)
      mf_y <- try(model.frame(resp_formula, data = data_eval, na.action = na.pass), silent = TRUE)
      if (!inherits(mf_y, "try-error")) {
         Y_mat <- model.response(mf_y)
         if (is.null(Y_mat)) Y_mat <- mf_y[, 1, drop = FALSE]
      } else {
         # Last resort: check if it's a name in data
         id_name <- as.character(lhs_expr)
         if (!is.null(data_eval) && id_name %in% names(data_eval)) {
            Y_mat <- data_eval[[id_name]]
         }
      }
    }

    # Final check if it's a list (not df)
    if (is.list(Y_mat) && !is.data.frame(Y_mat)) {
       Y_mat <- do.call(cbind, Y_mat)
    }

    # If formula has RHS, extract covariates
    if (length(formula) == 3 && is.null(covariates)) {
       rhs_expr <- formula[[3]]
       cov_formula <- bquote(~ .(rhs_expr))
       mf_x <- try(model.frame(cov_formula, data = data_eval, na.action = na.pass), silent = TRUE)
       if (inherits(mf_x, "try-error")) {
          # Try to subset data to only RHS variables to avoid errors from other complex columns
          rhs_vars <- all.vars(rhs_expr)
          data_sub <- if (!is.null(data_eval)) data_eval[, intersect(rhs_vars, names(data_eval)), drop = FALSE] else data_eval
          mf_x <- model.frame(cov_formula, data = data_sub, na.action = na.pass)
       }
       covariates <- model.matrix(attr(mf_x, "terms"), mf_x)
       # Remove intercept
       if ("(Intercept)" %in% colnames(covariates)) {
         covariates <- covariates[, colnames(covariates) != "(Intercept)", drop = FALSE]
       }
    }
  } else {
    formula <- NULL
    # If Y_mat is a character vector of names, subset from data
    if (is.character(Y_mat) && length(Y_mat) > 1 && !is.null(data_eval)) {
      Y_mat <- data_eval[, Y_mat, drop = FALSE]
    }
    # Handle list of vectors
    if (is.list(Y_mat) && !is.data.frame(Y_mat)) {
       Y_mat <- do.call(cbind, Y_mat)
    }
  }

  # Handle covariates (if provided separately or extracted from formula)
  X_mat <- NULL
  if (!is.null(covariates)) {
     if (is.matrix(covariates) || is.data.frame(covariates)) {
        X_mat <- as.matrix(covariates)
     } else if (inherits(covariates, "formula")) {
        mf_x <- model.frame(covariates, data = data_eval, na.action = na.pass)
        X_mat <- model.matrix(covariates, mf_x)
        if ("(Intercept)" %in% colnames(X_mat)) {
          X_mat <- X_mat[, colnames(X_mat) != "(Intercept)", drop = FALSE]
        }
     } else if (is.character(covariates) && !is.null(data_eval)) {
        X_mat <- as.matrix(data_eval[, covariates, drop = FALSE])
     }
  }

  if (!is.null(X_mat)) {
    # Ensure numeric
    X_mat <- X_mat[, sapply(as.data.frame(X_mat), is.numeric), drop = FALSE]
  }

  # Parse ID (NSE support for ID = group)
  id_val <- NULL
  if (!is.null(id_expr)) {
    if (!is.null(data_eval)) {
      # Try to evaluate ID in data
      id_val <- try(eval(id_expr, data_eval, parent.frame()), silent = TRUE)
      if (inherits(id_val, "try-error") || is.null(id_val)) {
        # Fallback: check if ID name is in data
        id_name <- as.character(id_expr)
        if (id_name %in% names(data_eval)) {
          id_val <- data_eval[[id_name]]
        } else {
          id_val <- ID # Literal value
        }
      }
    } else if (is.data.frame(Y_mat)) {
      # If data is NULL but x is a dataframe, check if ID is a column name
      id_name <- as.character(id_expr)
      if (id_name %in% names(Y_mat)) {
        id_val <- Y_mat[[id_name]]
        # We will remove this column from Y_mat later
      } else {
        id_val <- ID
      }
    } else {
      id_val <- ID
    }

    # If the response matrix still contains the ID column, remove it
    id_name_str <- as.character(id_expr)
    if (is.data.frame(Y_mat) && id_name_str %in% colnames(Y_mat)) {
       Y_mat[[id_name_str]] <- NULL
    }
  }

  if (is.data.frame(Y_mat)) {
    if (!all(sapply(Y_mat, is.numeric))) {
      stop("All variables in the response data must be numeric. Character or factor variables are not supported.", call. = FALSE)
    }
  } else if (!is.numeric(Y_mat) && !is.logical(Y_mat)) {
    stop("The response data matrix must be numeric.", call. = FALSE)
  }

  if (missing == "listwise") {
    if (!is.null(X_mat)) {
      valid_idx <- stats::complete.cases(Y_mat, X_mat)
      Y_mat <- Y_mat[valid_idx, , drop = FALSE]
      X_mat <- X_mat[valid_idx, , drop = FALSE]
      if (length(id_val) == length(valid_idx)) id_val <- id_val[valid_idx]
    } else {
      valid_idx <- stats::complete.cases(Y_mat)
      Y_mat <- Y_mat[valid_idx, , drop = FALSE]
      if (length(id_val) == length(valid_idx)) id_val <- id_val[valid_idx]
    }
  } else if (missing == "pairwise" && anyNA(Y_mat)) {
     # No row deletion for pairwise
  }

  Y_mat <- as.matrix(Y_mat)

  P_y <- ncol(Y_mat)
  target_names <- colnames(Y_mat)
  if (is.null(target_names)) target_names <- paste0("Y", 1:P_y)

  # Combine with covariates for Joint MVN
  if (!is.null(X_mat)) {
     P_x <- ncol(X_mat)
     control_names <- colnames(X_mat)
     if (is.null(control_names)) control_names <- paste0("X", 1:P_x)
     Y_mat <- cbind(Y_mat, X_mat)
     colnames(Y_mat) <- c(target_names, control_names)
  } else {
     P_x <- 0
     control_names <- NULL
  }

  N <- nrow(Y_mat)
  P <- ncol(Y_mat)
  if (P < 1) stop("No numeric columns found for correlation analysis.")
  if (N < 2) {
    stop("Correlation analysis requires at least two complete observations.", call. = FALSE)
  }
  if (P == 1 && is.null(id_val)) {
    stop("Correlation analysis requires at least two variables.", call. = FALSE)
  }

  var_names <- colnames(Y_mat)
  if (is.null(var_names)) var_names <- paste0("V", 1:P)
  colnames(Y_mat) <- var_names



  # --- RTMB Implementation ---
  if (is.null(prior)) {
    prior <- prior_flat()
  }

  if (!is.null(id_val)) {
     # Multilevel correlation mode
     if (length(id_val) != N) {
       stop("'ID' must have the same length as the response data after missing-value handling.", call. = FALSE)
     }
     if (anyNA(id_val)) {
       stop("'ID' must not contain missing values after missing-value handling.", call. = FALSE)
     }
     group_factor <- as.factor(id_val)
     group_id <- as.integer(group_factor)
     group_names <- levels(group_factor)
     J <- length(group_names)
     group_counts <- tabulate(group_id, nbins = J)
     if (J < 2L) {
       stop("Multilevel correlation requires at least two ID groups.", call. = FALSE)
     }
     if (all(group_counts < 2L)) {
       stop(
         "Multilevel correlation requires at least one ID group with two or more observations. ",
         "All supplied ID groups contain only one observation, so within-group variation cannot be estimated.",
         call. = FALSE
       )
     }

     # Automatically switch to prior_weak() if y_range is provided and prior is default flat
     if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
       prior <- prior_weak()
     }

     if (!inherits(prior, "rtmb_prior")) {
       stop(
         "prior must be an object of class 'rtmb_prior'. ",
         "Use prior_flat(), prior_normal(), or prior_weak().",
         call. = FALSE
       )
     }

     prior_type <- prior$type

     is_flat <- identical(prior_type, "flat")
     is_normal <- identical(prior_type, "normal")
     is_weak <- identical(prior_type, "weak")

     if (is_normal) {
       if (is.null(prior$mu_sd) && !is.null(prior$Intercept_sd)) {
         prior$mu_sd <- prior$Intercept_sd
       }
     }

     use_weak_info <- prior_type %in% c("weak")
     multivariate <- P > 1

     setup_exprs <- list()
     if (use_weak_info) {
       if (is.null(y_range)) {
         stop("Specifying 'y_range' is required when using prior_weak().")
       }
       if (is.list(y_range)) {
         Y_range_mat <- do.call(rbind, lapply(y_range, as.numeric))
       } else if (is.vector(y_range) && length(y_range) == 2) {
         Y_range_mat <- matrix(rep(as.numeric(y_range), each = P), P, 2)
       } else {
         Y_range_mat <- as.matrix(y_range)
       }

       half_d_y <- (Y_range_mat[, 2] - Y_range_mat[, 1]) / 2
       mid_y_val <- (Y_range_mat[, 2] + Y_range_mat[, 1]) / 2
       base_scale <- half_d_y * prior$sd_ratio

       setup_exprs[[length(setup_exprs) + 1]] <- quote(mid_y <- mid_y)
       setup_exprs[[length(setup_exprs) + 1]] <- quote(alpha_prior_sd <- alpha_prior_sd)
       setup_exprs[[length(setup_exprs) + 1]] <- quote(sigma_rate_vec <- sigma_rate_vec)
     }
     setup_ast <- as.call(c(list(as.name("{")), setup_exprs))

     param_exprs <- list(as.name("{"))
     param_exprs[[length(param_exprs) + 1]] <- bquote(mu <- Dim(.(P), random = TRUE))
     param_exprs[[length(param_exprs) + 1]] <- bquote(u <- Dim(c(J, .(P)), random = TRUE))

     param_exprs[[length(param_exprs) + 1]] <- bquote(sigma_between <- Dim(.(P), lower = 0))
     param_exprs[[length(param_exprs) + 1]] <- bquote(sigma_within <- Dim(.(P), lower = 0))

     if (multivariate) {
       param_exprs[[length(param_exprs) + 1]] <- bquote(L_corr_between <- Dim(c(.(P), .(P)), type = "CF_corr"))
       param_exprs[[length(param_exprs) + 1]] <- bquote(L_corr_within <- Dim(c(.(P), .(P)), type = "CF_corr"))
     }
     param_ast <- as.call(param_exprs)

     model_exprs <- list(as.name("{"))
      if (multivariate) {
        if (!is.null(prior$lkj_eta)) {
          model_exprs[[length(model_exprs) + 1]] <- bquote(L_corr_between ~ lkj_CF_corr(.(prior$lkj_eta)))
          model_exprs[[length(model_exprs) + 1]] <- bquote(L_corr_within ~ lkj_CF_corr(.(prior$lkj_eta)))
        }
        model_exprs[[length(model_exprs) + 1]] <- quote(u ~ multi_normal_CF(mean = rep(0, P), sd = sigma_between, CF_Omega = L_corr_between))
      } else {
       model_exprs[[length(model_exprs) + 1]] <- quote(u ~ normal(0, sigma_between))
     }

     model_exprs[[length(model_exprs) + 1]] <- quote(Y_pred <- u[group_id, ] + rep(mu, each = N))
     if (multivariate) {
       model_exprs[[length(model_exprs) + 1]] <- quote(Y ~ multi_normal_CF(mean = Y_pred, sd = sigma_within, CF_Omega = L_corr_within))
     } else {
       model_exprs[[length(model_exprs) + 1]] <- quote(Y ~ normal(Y_pred, sigma_within))
     }

     if (prior_type == "weak") {
       model_exprs[[length(model_exprs) + 1]] <- quote(sigma_between ~ exponential(sigma_rate_vec))
       model_exprs[[length(model_exprs) + 1]] <- quote(sigma_within ~ exponential(sigma_rate_vec))
       model_exprs[[length(model_exprs) + 1]] <- quote(mu ~ normal(mid_y, alpha_prior_sd))
      } else if (prior_type == "normal") {
        if (!is.null(prior$sigma_rate)) {
          model_exprs[[length(model_exprs) + 1]] <- bquote(sigma_between ~ exponential(.(prior$sigma_rate)))
          model_exprs[[length(model_exprs) + 1]] <- bquote(sigma_within ~ exponential(.(prior$sigma_rate)))
        }
        if (!is.null(prior$mu_sd) || !is.null(prior$Intercept_sd)) {
          mu_sd_val <- if (!is.null(prior$mu_sd)) prior$mu_sd else prior$Intercept_sd
          model_exprs[[length(model_exprs) + 1]] <- bquote(mu ~ normal(0, .(mu_sd_val)))
        }
      }
     model_ast <- as.call(model_exprs)

      transform_exprs <- list(as.name("{"))
      transform_exprs[[length(transform_exprs) + 1]] <- quote(ICC <- (sigma_between^2) / (sigma_between^2 + sigma_within^2))
      if (multivariate) {
        transform_exprs[[length(transform_exprs) + 1]] <- quote(B_corr <- L_corr_between %*% t(L_corr_between))
        transform_exprs[[length(transform_exprs) + 1]] <- quote(W_corr <- L_corr_within %*% t(L_corr_within))

        if (P_x > 0) {
           transform_exprs[[length(transform_exprs) + 1]] <- quote(R_yy <- W_corr[1:P_y, 1:P_y])
           transform_exprs[[length(transform_exprs) + 1]] <- quote(R_yx <- W_corr[1:P_y, (P_y+1):P])
           transform_exprs[[length(transform_exprs) + 1]] <- quote(R_xx <- W_corr[(P_y+1):P, (P_y+1):P])
           transform_exprs[[length(transform_exprs) + 1]] <- quote(P_cov_w <- R_yy - R_yx %*% solve(R_xx) %*% t(R_yx))
           transform_exprs[[length(transform_exprs) + 1]] <- quote(D_w <- diag(1 / sqrt(diag(P_cov_w))))
           transform_exprs[[length(transform_exprs) + 1]] <- quote(W_pcorr <- D_w %*% P_cov_w %*% D_w)

           transform_exprs[[length(transform_exprs) + 1]] <- quote(R_yy_b <- B_corr[1:P_y, 1:P_y])
           transform_exprs[[length(transform_exprs) + 1]] <- quote(R_yx_b <- B_corr[1:P_y, (P_y+1):P])
           transform_exprs[[length(transform_exprs) + 1]] <- quote(R_xx_b <- B_corr[(P_y+1):P, (P_y+1):P])
           transform_exprs[[length(transform_exprs) + 1]] <- quote(P_cov_b <- R_yy_b - R_yx_b %*% solve(R_xx_b) %*% t(R_yx_b))
           transform_exprs[[length(transform_exprs) + 1]] <- quote(D_b <- diag(1 / sqrt(diag(P_cov_b))))
           transform_exprs[[length(transform_exprs) + 1]] <- quote(B_pcorr <- D_b %*% P_cov_b %*% D_b)
           transform_exprs[[length(transform_exprs) + 1]] <- quote(report(ICC, B_corr, W_corr, W_pcorr, B_pcorr))
        } else {
           transform_exprs[[length(transform_exprs) + 1]] <- quote(report(ICC, B_corr, W_corr))
        }
      } else {
        transform_exprs[[length(transform_exprs) + 1]] <- quote(report(ICC))
      }
      transform_ast <- as.call(transform_exprs)

     generate_ast <- NULL
     if (isTRUE(WAIC)) {
       generate_ast <- if (multivariate) {
         quote({
           Y_pred <- u[group_id, ] + rep(mu, each = N)
           log_lik <- multi_normal_CF_lpdf(Y, Y_pred, sigma_within, L_corr_within, sum = FALSE)
         })
       } else {
         quote({
           Y_pred <- u[group_id, ] + rep(mu, each = N)
           log_lik <- normal_lpdf(Y, Y_pred, sigma_within, sum = FALSE)
         })
       }
     }

     mdl_code <- list(setup = setup_ast, parameters = param_ast, transform = transform_ast, model = model_ast, env = parent.frame())
     if (!is.null(generate_ast)) mdl_code$generate <- .rtmb_waic_generate_ast(NULL, generate_ast)
     class(mdl_code) <- "rtmb_code"

     v_names <- list(mu = var_names, sigma_between = var_names, sigma_within = var_names)
     v_names$u <- list(group_names, var_names)
     v_names$ICC <- var_names

     if (multivariate) {
       v_names$B_corr <- list(var_names, var_names)
       v_names$W_corr <- list(var_names, var_names)
       if (P_x > 0) {
           v_names$B_pcorr <- list(target_names, target_names)
           v_names$W_pcorr <- list(target_names, target_names)
       }
     }

     data_list <- list(Y = Y_mat, group_id = group_id, N = N, P = P, J = J, P_y = P_y, P_x = P_x)
     if (use_weak_info) {
       data_list$mid_y <- mid_y_val
       data_list$alpha_prior_sd <- half_d_y
       data_list$sigma_rate_vec <- 1.0 / base_scale
     }

     init_list <- list(mu = colMeans(Y_mat))
     init_list$sigma_between <- apply(Y_mat, 2, sd) * 0.5
     init_list$sigma_within <- apply(Y_mat, 2, sd) * 0.5
     init_list$u <- matrix(0, J, P)

     view_order <- c("pcorr", "B_corr", "W_corr", "corr", "ICC", "mu", "sigma_between", "sigma_within", "sigma")

     obj <- rtmb_model(data_list, mdl_code, par_names = v_names, init = init_list, fixed = fixed, view = view_order)
     obj$raw_data <- data

     obj$type <- "corr"
     obj$extra <- list(
       source = "wrapper",
       prior_type = prior$type,
       marginal = "mu",
       corr_method = "reml",
       missing = missing
     )

     # Set degrees of freedom map for BW method
     # Between: J, Within: N - J - P
     df_map <- list()
     # Level-2 (Between)
     df_between <- pmax(J, 1)
     df_map$mu <- df_between
     df_map$sigma_between <- df_between
     df_map$B_corr <- df_between
     df_map$B_pcorr <- df_between
     df_map$ICC <- df_between
     
     # Level-1 (Within)
     df_within <- pmax(N - J - P, 1)
     df_map$sigma_within <- df_within
     df_map$W_corr <- df_within
     df_map$W_pcorr <- df_within
     
     obj$extra$df_map <- df_map

     return(obj)
  } else {
     # Simple correlation mode
     # Automatically switch to prior_weak() if y_range is provided and prior is default flat
     if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
       prior <- prior_weak()
     }

     if (!inherits(prior, "rtmb_prior")) {
       stop(
         "prior must be an object of class 'rtmb_prior'. ",
         "Use prior_flat(), prior_normal(), or prior_weak().",
         call. = FALSE
       )
     }

     prior_type <- prior$type

     is_flat <- identical(prior_type, "flat")
     is_normal <- identical(prior_type, "normal")
     is_weak <- identical(prior_type, "weak")

     if (is_normal) {
       if (is.null(prior$mu_sd) && !is.null(prior$Intercept_sd)) {
         prior$mu_sd <- prior$Intercept_sd
       }
     }

     setup_exprs <- list(as.name("{"))
     setup_exprs[[length(setup_exprs) + 1]] <- quote(N <- nrow(Y))
     setup_exprs[[length(setup_exprs) + 1]] <- quote(P <- ncol(Y))
     if (missing == "listwise") {
       setup_exprs[[length(setup_exprs) + 1]] <- quote(Y_bar <- colMeans(Y))
       setup_exprs[[length(setup_exprs) + 1]] <- quote(S_Y <- cov(Y) * (N - 1))
     } else if (missing == "pairwise") {
       setup_exprs[[length(setup_exprs) + 1]] <- quote(Y_bar <- colMeans(Y, na.rm = TRUE))
       setup_exprs[[length(setup_exprs) + 1]] <- quote(S_Y <- cov(Y, use = "pairwise.complete.obs") * (N - 1))
     }

     if (prior_type == "weak") {
       if (is.null(y_range)) {
         stop("y_range is required when using prior_weak().")
       }
       if (is.list(y_range)) {
         y_range_mat <- do.call(rbind, y_range)
       } else if (is.vector(y_range) && length(y_range) == 2) {
         y_range_mat <- matrix(y_range, P, 2, byrow = TRUE)
       } else {
         y_range_mat <- as.matrix(y_range)
       }

       mid_y_val <- (y_range_mat[, 1] + y_range_mat[, 2]) / 2
       alpha_prior_sd_val <- (y_range_mat[, 2] - y_range_mat[, 1]) / 2
       sigma_rate_val <- 1 / (alpha_prior_sd_val * 0.5)

       setup_exprs[[length(setup_exprs) + 1]] <- quote(mid_y <- mid_y)
       setup_exprs[[length(setup_exprs) + 1]] <- quote(alpha_prior_sd <- alpha_prior_sd)
       setup_exprs[[length(setup_exprs) + 1]] <- quote(sigma_rate_vec <- sigma_rate_vec)
     }
     setup_ast <- as.call(setup_exprs)

     param_exprs <- list(as.name("{"))
     param_exprs[[length(param_exprs) + 1]] <- quote(mean <- Dim(P, random = TRUE))
     param_exprs[[length(param_exprs) + 1]] <- quote(sd   <- Dim(P, lower = 0))

     if (P == 2) {
       param_exprs[[length(param_exprs) + 1]] <- bquote(corr <- Dim(lower = -1, upper = 1))
     } else {
       param_exprs[[length(param_exprs) + 1]] <- quote(CF_corr <- Dim(c(P, P), type = "CF_corr"))
     }
     param_ast <- as.call(param_exprs)

     model_exprs <- list(as.name("{"))
     if (prior_type == "weak") {
       model_exprs[[length(model_exprs) + 1]] <- quote(mean ~ normal(mid_y, alpha_prior_sd))
       model_exprs[[length(model_exprs) + 1]] <- quote(sd ~ exponential(sigma_rate_vec))
      } else if (prior_type == "normal") {
        if (!is.null(prior$mu_sd) || !is.null(prior$Intercept_sd)) {
          mu_sd_val <- if (!is.null(prior$mu_sd)) prior$mu_sd else prior$Intercept_sd
          model_exprs[[length(model_exprs) + 1]] <- bquote(mean ~ normal(0, .(mu_sd_val)))
        }
        if (!is.null(prior$sigma_rate)) {
          model_exprs[[length(model_exprs) + 1]] <- bquote(sd ~ exponential(.(prior$sigma_rate)))
        }
      }

      if (P == 2) {
        if (!is.null(prior$lkj_eta)) {
          model_exprs[[length(model_exprs) + 1]] <- bquote(corr ~ lkj_corr(.(prior$lkj_eta)))
        }
        model_exprs[[length(model_exprs) + 1]] <- quote(CF_corr <- matrix(corr * 0, nrow = 2, ncol = 2))
        model_exprs[[length(model_exprs) + 1]] <- quote(CF_corr[1, 1] <- 1)
        model_exprs[[length(model_exprs) + 1]] <- quote(CF_corr[2, 1] <- corr)
        model_exprs[[length(model_exprs) + 1]] <- quote(CF_corr[2, 2] <- sqrt(1 - corr^2))
      } else {
        if (!is.null(prior$lkj_eta)) {
          model_exprs[[length(model_exprs) + 1]] <- bquote(CF_corr ~ lkj_CF_corr(.(prior$lkj_eta)))
        }
      }

      if (missing %in% c("listwise", "pairwise")) {
        model_exprs[[length(model_exprs) + 1]] <- quote(S_Y ~ sufficient_multi_normal_CF(N, Y_bar, mean, sd, CF_corr))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- quote(Y ~ fiml_multi_normal_CF(mean, sd, CF_corr))
      }
     model_ast <- as.call(model_exprs)

     transform_exprs <- list(as.name("{"))
     if (P > 2) {
       transform_exprs[[length(transform_exprs) + 1]] <- quote(corr <- CF_corr %*% t(CF_corr))

       if (P_x > 0) {
          transform_exprs[[length(transform_exprs) + 1]] <- quote(R_yy <- corr[1:P_y, 1:P_y])
          transform_exprs[[length(transform_exprs) + 1]] <- quote(R_yx <- corr[1:P_y, (P_y+1):P])
          transform_exprs[[length(transform_exprs) + 1]] <- quote(R_xx <- corr[(P_y+1):P, (P_y+1):P])
          transform_exprs[[length(transform_exprs) + 1]] <- quote(P_cov <- R_yy - R_yx %*% solve(R_xx) %*% t(R_yx))
          transform_exprs[[length(transform_exprs) + 1]] <- quote(D <- diag(1 / sqrt(diag(P_cov))))
          transform_exprs[[length(transform_exprs) + 1]] <- quote(pcorr <- D %*% P_cov %*% D)
          transform_exprs[[length(transform_exprs) + 1]] <- quote(report(corr, pcorr))
       } else {
          transform_exprs[[length(transform_exprs) + 1]] <- quote(report(corr))
       }
     }
     transform_ast <- as.call(transform_exprs)

     generate_ast <- NULL
     if (isTRUE(WAIC)) {
       generate_ast <- if (P == 1) {
         quote({
           log_lik <- normal_lpdf(Y, mean, sd, sum = FALSE)
         })
       } else {
         quote({
           log_lik <- multi_normal_CF_lpdf(Y, mean, sd, CF_corr, sum = FALSE)
         })
       }
     }

     mdl_code <- list(setup = setup_ast, parameters = param_ast, transform = transform_ast, model = model_ast, env = parent.frame())
     if (!is.null(generate_ast)) mdl_code$generate <- .rtmb_waic_generate_ast(NULL, generate_ast)
     class(mdl_code) <- "rtmb_code"

     dat_list <- list(Y = Y_mat, P_y = P_y, P_x = P_x)
     if (prior_type == "weak") {
       dat_list$mid_y <- mid_y_val
       dat_list$alpha_prior_sd <- alpha_prior_sd_val
       dat_list$sigma_rate_vec <- sigma_rate_val
     }

     v_names <- list(mean = var_names, sd = var_names)
     if (P == 2) {
       v_names$corr <- "rho"
     } else {
       v_names$corr <- list(var_names, var_names)
       if (P_x > 0) v_names$pcorr <- list(target_names, target_names)
     }

     init_list <- if (is.null(init)) {
       list(mean = colMeans(Y_mat), sd = apply(Y_mat, 2, sd))
     } else init

     view_vars <- if (P_x > 0) c("pcorr", "B_corr", "W_corr", "corr", "mean", "sd") else c("B_corr", "W_corr", "corr", "mean", "sd")
     obj <- rtmb_model(data = dat_list, code = mdl_code, par_names = v_names, init = init_list, fixed = fixed, view = view_vars)

    obj$raw_data <- data
    obj$type <- "corr"
    obj$extra <- list(
      source = "wrapper",
      prior_type = prior$type,
      marginal = "mean",
      corr_method = method,
      missing = missing
    )

    return(obj)
  }
}

#' @noRd
.sample_impl <- function(self, private, sampling, warmup, chains, thin, seed, delta, 
                         max_treedepth, parallel, laplace, init, init_jitter, 
                         save_csv, map, fixed) {
  if (!is.null(fixed)) {
    return(private$.dispatch_fixed(.method_to_call = "sample", sampling = sampling, warmup = warmup, chains = chains, thin = thin,
      seed = seed, delta = delta, max_treedepth = max_treedepth,
      parallel = parallel, laplace = laplace, init = init,
      init_jitter = init_jitter, save_csv = save_csv, map = map, fixed = fixed))
  }

  set.seed(seed)
  orig_pl <- self$par_list
  data_na_summary <- function(dat) {
    if (is.null(dat)) return(character(0))
    out <- character(0)
    for (nm in names(dat)) {
      x <- dat[[nm]]
      if (is.atomic(x) || is.matrix(x) || is.array(x) || is.data.frame(x)) {
        n_na <- sum(is.na(x))
        if (n_na > 0L) out <- c(out, sprintf("%s (%d NA)", nm, n_na))
      }
    }
    out
  }
  stop_nonfinite_lp <- function(context) {
    na_vars <- data_na_summary(self$data)
    if (length(na_vars) > 0L) {
      stop(
        context,
        " produced non-finite log-probability values. The model data contain missing values in: ",
        paste(na_vars, collapse = ", "),
        ". If these NA values are not handled explicitly in the model code, remove/impute them or use a wrapper/missing-data option that supports NA.",
        call. = FALSE
      )
    }
    stop(context, " produced non-finite log-probability values.", call. = FALSE)
  }
  mcmc_pd_error_to_neginf <- isTRUE(getOption("BayesRTMB.mcmc_pd_error_to_neginf", TRUE))
  is_positive_definite_error <- function(msg) {
    grepl(
      paste(
        c(
          "not positive definite",
          "positive[- ]definite",
          "leading minor",
          "computationally singular",
          "reciprocal condition number",
          "\\bsingular\\b",
          "Cholesky",
          "\\bchol\\b",
          "\\bLLT\\b",
          "\\bLDLT\\b"
        ),
        collapse = "|"
      ),
      msg,
      ignore.case = TRUE
    )
  }
  wrap_mcmc_pd_errors <- function(ad_obj) {
    if (!mcmc_pd_error_to_neginf) return(ad_obj)
    pd_error_state <- new.env(parent = emptyenv())
    pd_error_state$post_warmup <- FALSE
    pd_error_state$count <- 0L
    ad_obj$rtmb_pd_error_state <- pd_error_state
    orig_fn <- ad_obj$fn
    orig_gr <- ad_obj$gr
    ad_obj$fn <- function(x, ...) {
      tryCatch(
        orig_fn(x, ...),
        error = function(e) {
          if (is_positive_definite_error(conditionMessage(e))) {
            if (isTRUE(pd_error_state$post_warmup)) pd_error_state$count <- pd_error_state$count + 1L
            return(Inf)
          }
          stop(e)
        }
      )
    }
    ad_obj$gr <- function(x, ...) {
      n <- length(x)
      tryCatch(
        orig_gr(x, ...),
        error = function(e) {
          if (is_positive_definite_error(conditionMessage(e))) {
            if (isTRUE(pd_error_state$post_warmup)) pd_error_state$count <- pd_error_state$count + 1L
            return(rep(0, n))
          }
          stop(e)
        }
      )
    }
    ad_obj
  }

  if (!is.null(save_csv)) {
    if (!is.list(save_csv)) stop("save_csv must be specified in the format list(name='...', dir='...').")
    save_name <- if (!is.null(save_csv$name)) save_csv$name else "model"
    save_dir <- if (!is.null(save_csv$dir)) save_csv$dir else "BayesRTMB_mcmc"
    save_freq <- if (!is.null(save_csv$freq)) save_csv$freq else 0
    if (!dir.exists(save_dir)) dir.create(save_dir, recursive = TRUE, showWarnings = FALSE)
    save_info <- list(name = save_name, dir = save_dir, freq = save_freq)
  } else {
    save_info <- NULL
  }

  random_flags <- vapply(orig_pl, function(x) isTRUE(x$random), logical(1))

  if (laplace && any(random_flags)) {
    pl_fixed <- parse_parameters(orig_pl[!random_flags], self$par_names)
    pl_random <- parse_parameters(orig_pl[random_flags], self$par_names)
    fixed_idx <- which(self$pl_full$names %in% pl_fixed$names)
    random_idx <- which(self$pl_full$names %in% pl_random$names)
  } else {
    pl_fixed <- self$pl_full
    pl_random <- NULL
    fixed_idx <- 1:length(self$pl_full$names)
    random_idx <- integer(0)
  }

  P_fixed <- length(pl_fixed$names)
  P_random <- if (!is.null(pl_random)) length(pl_random$names) else 0

  if (parallel) {
    if (!requireNamespace("future", quietly = TRUE) ||
        !requireNamespace("future.apply", quietly = TRUE) ||
        !requireNamespace("progressr", quietly = TRUE)) {
      stop(
        "parallel = TRUE for sample() requires the suggested packages ",
        "'future', 'future.apply', and 'progressr'. ",
        "Please install them or use parallel = FALSE.",
        call. = FALSE
      )
    }
    if (inherits(future::plan(), "sequential")) {
      if (.Platform$OS.type == "unix") future::plan(future::multicore, workers = chains)
      else future::plan(future::multisession, workers = chains)
    }
    cat(paste0("Starting parallel sampling (chains = ", chains, ")...\n"))
  } else {
    cat(paste0("Starting sequential sampling (chains = ", chains, ")...\n"))
  }

  # --- [IMPORTANT] Data extraction to avoid serialization ---
  local_data <- self$data
  local_par_list <- self$par_list
  local_pl_full <- self$pl_full
  local_map <- if (is.null(map)) self$map else utils::modifyList(as.list(self$map), as.list(map))
  # Normalize map factor levels for consistent MakeADFun behavior
  if (!is.null(local_map)) {
    local_map <- lapply(local_map, function(m) {
      if (is.null(m)) return(NULL)
      vals <- as.character(m)
      factor(vals, levels = unique(vals[!is.na(vals)]))
    })
  }
  local_log_prob <- self$log_prob
  local_transform <- self$transform
  local_fixed_prior_specs <- self$fixed_prior_specs
  local_code_model <- self$code$model
  
  # Pre-calculate shadow list for fixed-prior removal
  env_shadow_local <- list2env(local_data, parent = asNamespace("BayesRTMB"))
  idx_s <- 1
  for (nm in names(local_par_list)) {
    p <- local_par_list[[nm]]
    shadow_vec <- local_pl_full$names[idx_s:(idx_s + p$length - 1)]
    if (length(p$dim) > 1) dim(shadow_vec) <- p$dim
    env_shadow_local[[nm]] <- shadow_vec
    idx_s <- idx_s + p$length
  }
  if (!is.null(local_transform)) {
    par_only <- mget(names(local_par_list), envir = env_shadow_local, inherits = FALSE)
    tran_res_shadow <- tryCatch(suppressMessages(local_transform(local_data, par_only)), error = function(e) list())
    for (nm in names(tran_res_shadow)) env_shadow_local[[nm]] <- tran_res_shadow[[nm]]
  }
  local_shadow_list <- as.list(env_shadow_local)

  random_effs <- names(local_par_list)[random_flags]
  use_random <- if (laplace && length(random_effs) > 0) random_effs else NULL
  base_init <- self$prepare_init(init)

  # [Fix] Prepare f_ad beforehand (to avoid capturing private)
  f_ad_global <- private$.build_f_ad(
    data_local = local_data,
    par_list_local = local_par_list,
    log_prob_local = local_log_prob,
    transform_local = local_transform,
    jacobian_target = "all",
    adreport = FALSE,
    fixed_prior_specs_local = local_fixed_prior_specs,
    code_model_local = local_code_model,
    shadow_list_local = local_shadow_list
  )

  run_chain <- function(c, f_ad, p_callback = NULL) {
    library(BayesRTMB)
    unc_init_list <- to_unconstrained(constrained_vector_to_list(base_init, local_par_list), local_par_list)
    unc_init_vec <- unlist(unc_init_list, use.names = FALSE)

    if (init_jitter > 0) {
      jitter_vec <- rnorm(length(unc_init_vec), mean = 0, sd = init_jitter)
      if (!is.null(local_map)) {
        idx <- 1
        for (name in names(local_par_list)) {
          L_unc <- local_par_list[[name]]$unc_length
          if (L_unc > 0) {
            if (!is.null(local_map[[name]])) {
              is_fixed <- is.na(local_map[[name]])
              jitter_vec[idx:(idx + L_unc - 1)][is_fixed] <- 0
            }
            idx <- idx + L_unc
          }
        }
      }
      unc_init_vec <- unc_init_vec + jitter_vec
    }
    unc_init_list_new <- unconstrained_vector_to_list(unc_init_vec, local_par_list)

    # Use the f_ad provided from outside
    ad_obj <- tryCatch({
      RTMB::MakeADFun(func = f_ad, parameters = unc_init_list_new, random = use_random, map = local_map, silent = TRUE)
    }, error = function(e) stop("Failed to setup MakeADFun in parallel worker.\n[Error]: ", e$message, call. = FALSE))
    ad_obj <- wrap_mcmc_pd_errors(ad_obj)

    init_lp <- tryCatch(-ad_obj$fn(ad_obj$par), error = function(e) NA_real_)
    if (!is.finite(init_lp)) {
      stop_nonfinite_lp(sprintf("MCMC initialization for chain %d", c))
    }

    res <- NUTS_method(model = ad_obj, sampling = sampling, warmup = warmup, delta = delta, max_treedepth = max_treedepth, chain = c, update_progress = p_callback, laplace = laplace, save_info = save_info)
    if (is.null(res$lp) || all(!is.finite(res$lp))) {
      stop_nonfinite_lp(sprintf("MCMC sampling for chain %d", c))
    }

    P_all_true <- length(local_pl_full$names)
    iter_total <- sampling + warmup
    para_final <- array(NA, dim = c(iter_total, P_all_true))

    for (i in 1:iter_total) {
      x_in <- as.numeric(res$para_fixed[i, ])
      if (laplace && length(ad_obj$env$random) > 0) {
        ad_obj$fn(x_in)
        para_list <- ad_obj$env$parList()
      } else {
        para_list <- ad_obj$env$parList(x = x_in)
      }
      con_list <- to_constrained(para_list, local_par_list)
      para_final[i, ] <- unlist(con_list, use.names = FALSE)
    }
    res$para <- para_final
    res$para_full <- NULL
    return(res)
  }

  results_list <- list()
  if (parallel) {
    iter <- sampling + warmup
    total_updates <- chains * (1 + floor(iter / 100))
    progressr::with_progress({
      p <- progressr::progressor(steps = total_updates)
      results_list <- withCallingHandlers({
        future.apply::future_lapply(1:chains, function(c) {
          run_chain(c, f_ad = f_ad_global, p_callback = function(msg = "", amt = 1, ...) {
            if (is.numeric(msg)) { amt <- msg; msg <- "" }
            p(amount = amt, message = as.character(msg))
          })
        }, future.seed = TRUE, future.packages = c("RTMB", "BayesRTMB"), future.globals = TRUE)
      }, warning = function(w) { if (grepl("package:BayesRTMB", conditionMessage(w))) invokeRestart("muffleWarning") })
    })
  } else {
    results_list <- lapply(1:chains, function(c) run_chain(c, f_ad = f_ad_global, p_callback = NULL))
  }

  mcmc_index <- seq(from = (warmup + 1), to = (warmup + sampling), by = thin)
  accept_mat <- array(NA, dim = c(length(mcmc_index), chains))
  td_mat <- array(NA, dim = c(length(mcmc_index), chains))
  eps_vec <- numeric(chains)
  pd_error_counts <- integer(chains)
  fit <- array(NA, dim = c(length(mcmc_index), chains, P_fixed + 1))
  dimnames(fit) <- list(iteration = NULL, chain = paste0("chain", 1:chains), variable = c("lp", pl_fixed$names))

  if (P_random > 0) {
    random_fit <- array(NA, dim = c(length(mcmc_index), chains, P_random))
    dimnames(random_fit) <- list(iteration = NULL, chain = paste0("chain", 1:chains), variable = pl_random$names)
  } else { random_fit <- NULL }

  for (c in 1:chains) {
    res <- results_list[[c]]
    pd_error_counts[c] <- if (!is.null(res$pd_error_count)) res$pd_error_count else 0L
    fit[, c, 1] <- res$lp[mcmc_index]
    for (j in 1:P_fixed) fit[, c, j + 1] <- res$para[mcmc_index, fixed_idx[j]]
    if (P_random > 0) { for (j in 1:P_random) random_fit[, c, j] <- res$para[mcmc_index, random_idx[j]] }
    accept_mat[, c] <- res$accept[mcmc_index]; td_mat[, c] <- res$treedepth[mcmc_index]; eps_vec[c] <- res$eps
  }
  if (all(!is.finite(fit[, , "lp"]))) {
    stop_nonfinite_lp("MCMC retained samples")
  }
  if (any(!is.finite(fit[, , "lp"]))) {
    warning(
      "Some retained MCMC draws have non-finite log-probability values. ",
      "They will be ignored by summaries that use finite values, but diagnostics should be interpreted carefully.",
      call. = FALSE
    )
  }
  if (sum(pd_error_counts) > 0L) {
    warning(
      "During post-warmup MCMC sampling, ",
      sum(pd_error_counts),
      " matrix positive-definite/singularity errors were treated as lp = -Inf.",
      call. = FALSE
    )
  }

  eps_chains <- eps_vec; accept_chains <- apply(accept_mat, 2, mean); treedepth_chains <- apply(td_mat, 2, max)
  names(eps_chains) <- names(accept_chains) <- names(treedepth_chains) <- paste0("chain", 1:chains)

  posterior_mean <- numeric(length(self$pl_full$names)); names(posterior_mean) <- self$pl_full$names
  fixed_mean <- apply(fit[, , -1, drop = FALSE], 3, mean); posterior_mean[names(fixed_mean)] <- fixed_mean
  if (!is.null(random_fit)) { random_mean <- apply(random_fit, 3, mean); posterior_mean[names(random_mean)] <- random_mean }

  if (!is.null(save_info)) {
    for (c in 1:chains) {
      backup_file <- file.path(save_info$dir, paste0(save_info$name, "-", c, ".csv"))
      df_metrics <- data.frame(iteration = mcmc_index, accept = accept_mat[, c], treedepth = td_mat[, c], eps = eps_vec[c])
      df_out <- if (!is.null(random_fit)) cbind(df_metrics, as.data.frame(fit[, c, ]), as.data.frame(random_fit[, c, ])) else cbind(df_metrics, as.data.frame(fit[, c, ]))
      write.csv(df_out, file = backup_file, row.names = FALSE)
    }
  }

  res_obj <- MCMC_Fit$new(
    model = self, fit = fit, random_fit = random_fit,
    eps = eps_chains, accept = accept_chains, treedepth = treedepth_chains,
    laplace = laplace, posterior_mean = posterior_mean,
    max_treedepth = max_treedepth,
    pd_error_count = pd_error_counts
  )
  if (!is.null(self$transform)) res_obj$transformed_draws(self$transform)
  if (!is.null(self$generate)) res_obj$generated_quantities(self$code$generate)
  return(res_obj)
}

#' @noRd
.variational_impl <- function(self, private, iter, tol_rel_obj, window_size, num_samples, 
                              num_estimate, alpha, laplace, print_freq, 
                              method = c("meanfield", "fullrank", "hybrid"), 
                              parallel, seed, init, save_csv, map, fixed) {
  if (!is.null(fixed)) {
    return(private$.dispatch_fixed(.method_to_call = "variational", iter = iter, tol_rel_obj = tol_rel_obj, window_size = window_size,
      num_samples = num_samples, num_estimate = num_estimate, alpha = alpha,
      laplace = laplace, print_freq = print_freq, method = method,
      parallel = parallel, seed = seed, init = init, save_csv = save_csv, map = map, fixed = fixed))
  }

  set.seed(seed); method <- match.arg(method)
  if (!is.null(save_csv)) {
    save_name <- if (!is.null(save_csv$name)) save_csv$name else "model_vb"
    save_dir <- if (!is.null(save_csv$dir)) save_csv$dir else "BayesRTMB_vb"
    if (!dir.exists(save_dir)) dir.create(save_dir, recursive = TRUE, showWarnings = FALSE)
    save_info <- list(name = save_name, dir = save_dir)
  } else { save_info <- NULL }

  if (parallel && num_estimate > 1) {
    if (!requireNamespace("future", quietly = TRUE) ||
        !requireNamespace("future.apply", quietly = TRUE)) {
      stop(
        "parallel = TRUE for variational() requires the suggested packages ",
        "'future' and 'future.apply'. Please install them or use parallel = FALSE.",
        call. = FALSE
      )
    }
    if (inherits(future::plan(), "sequential")) {
      if (.Platform$OS.type == "unix") future::plan(future::multicore, workers = num_estimate)
      else future::plan(future::multisession, workers = num_estimate)
    }
    cat(paste0("Starting parallel VB estimation (num_estimate = ", num_estimate, ")...\n"))
  } else { cat(paste0("Starting sequential VB estimation (num_estimate = ", num_estimate, ")...\n")) }

  # --- Data extraction to avoid serialization ---
  local_data <- self$data; local_par_list <- self$par_list; local_pl_full <- self$pl_full
  local_map <- if (is.null(map)) self$map else utils::modifyList(as.list(self$map), as.list(map))
  # Normalize map factor levels for consistent MakeADFun behavior
  if (!is.null(local_map)) {
    local_map <- lapply(local_map, function(m) {
      if (is.null(m)) return(NULL)
      vals <- as.character(m)
      factor(vals, levels = unique(vals[!is.na(vals)]))
    })
  }
  local_log_prob <- self$log_prob; local_transform <- self$transform
  local_fixed_prior_specs <- self$fixed_prior_specs
  local_code_model <- self$code$model
  
  # Pre-calculate shadow list for fixed-prior removal
  env_shadow_local <- list2env(local_data, parent = asNamespace("BayesRTMB"))
  idx_s <- 1
  for (nm in names(local_par_list)) {
    p <- local_par_list[[nm]]
    shadow_vec <- local_pl_full$names[idx_s:(idx_s + p$length - 1)]
    if (length(p$dim) > 1) dim(shadow_vec) <- p$dim
    env_shadow_local[[nm]] <- shadow_vec
    idx_s <- idx_s + p$length
  }
  if (!is.null(local_transform)) {
    par_only <- mget(names(local_par_list), envir = env_shadow_local, inherits = FALSE)
    tran_res_shadow <- tryCatch(suppressMessages(local_transform(local_data, par_only)), error = function(e) list())
    for (nm in names(tran_res_shadow)) env_shadow_local[[nm]] <- tran_res_shadow[[nm]]
  }
  local_shadow_list <- as.list(env_shadow_local)

  random_effs <- names(local_par_list)[sapply(local_par_list, function(x) isTRUE(x$random))]
  use_random <- if (laplace && length(random_effs) > 0) random_effs else NULL
  base_init <- self$prepare_init(init)

  # Prepare f_ad beforehand to avoid capturing private environment
  f_ad_global <- private$.build_f_ad(
    data_local = local_data, 
    par_list_local = local_par_list, 
    log_prob_local = local_log_prob, 
    transform_local = local_transform, 
    jacobian_target = "all", 
    adreport = FALSE,
    fixed_prior_specs_local = local_fixed_prior_specs,
    code_model_local = local_code_model,
    shadow_list_local = local_shadow_list
  )

  run_advi_worker <- function(c, f_ad, p_callback = NULL, p_interval = 0) {
    library(BayesRTMB)
    unc_init_list <- to_unconstrained(constrained_vector_to_list(base_init, local_par_list), local_par_list)
    unc_init_vec <- unlist(unc_init_list, use.names = FALSE)
    unc_init_list_new <- unconstrained_vector_to_list(unc_init_vec, local_par_list)

    # Use the f_ad provided from outside
    ad_obj <- tryCatch({
      RTMB::MakeADFun(func = f_ad, parameters = unc_init_list_new, random = use_random, map = local_map, silent = TRUE)
    }, error = function(e) stop("Failed to setup MakeADFun in parallel worker.\n[Error]: ", e$message, call. = FALSE))

    if (!is.null(use_random)) {
      orig_fn <- ad_obj$fn; orig_gr <- ad_obj$gr; idx_fixed_mask <- ad_obj$env$lfixed(); n_fixed <- sum(idx_fixed_mask)
      ad_obj$fn <- function(x, ...) { if (length(x) != n_fixed) x <- x[idx_fixed_mask]; orig_fn(x, ...) }
      ad_obj$gr <- function(x, ...) { if (length(x) != n_fixed) { g <- orig_gr(x[idx_fixed_mask], ...); g_full <- rep(0, length(x)); g_full[idx_fixed_mask] <- g; return(g_full) }; return(orig_gr(x, ...)) }
    }

    res <- ADVI_method(model = ad_obj, par_list = local_par_list, pl_full = local_pl_full, iter = iter, tol_rel_obj = tol_rel_obj, window_size = window_size, num_samples = num_samples, alpha = alpha, laplace = laplace, print_freq = if (is.null(p_callback)) print_freq else 0, method = method, update_progress = p_callback, update_interval = p_interval)
    return(res)
  }

  results_list <- list()
  if (parallel && num_estimate > 1) {
    if (requireNamespace("progressr", quietly = TRUE)) {
      progressr::handlers(global = TRUE); update_interval <- max(1, floor(iter / 100)); total_steps <- ceiling(iter / update_interval) * num_estimate
      results_list <- progressr::with_progress({
        p <- progressr::progressor(steps = total_steps)
        withCallingHandlers({
          future.apply::future_lapply(1:num_estimate, function(c) {
            run_advi_worker(c = c, f_ad = f_ad_global, p_callback = function(amount = 1) p(amount = amount), p_interval = update_interval)
          }, future.seed = TRUE, future.packages = c("RTMB", "BayesRTMB"), future.globals = TRUE)
        }, warning = function(w) { if (grepl("BayesRTMB", conditionMessage(w))) invokeRestart("muffleWarning") })
      })
    } else {
      results_list <- future.apply::future_lapply(1:num_estimate, function(c) run_advi_worker(c, f_ad = f_ad_global, p_callback = NULL, p_interval = 0), future.seed = TRUE, future.packages = c("RTMB", "BayesRTMB"), future.globals = TRUE)
    }
  } else { results_list <- lapply(1:num_estimate, function(c) { if (print_freq > 0) cat(sprintf("\n--- Starting VB estimation: est%d ---\n", c)); run_advi_worker(c, f_ad = f_ad_global, p_callback = NULL, p_interval = 0) }) }

  P_fixed <- dim(results_list[[1]]$fit)[3] - 1; P_random <- if (!is.null(results_list[[1]]$random_fit)) dim(results_list[[1]]$random_fit)[3] else 0
  fit <- array(NA, dim = c(num_samples, num_estimate, P_fixed + 1)); dimnames(fit) <- list(iteration = NULL, chain = paste0("est", 1:num_estimate), variable = dimnames(results_list[[1]]$fit)[[3]])
  if (P_random > 0) { random_fit <- array(NA, dim = c(num_samples, num_estimate, P_random)); dimnames(random_fit) <- list(iteration = NULL, chain = paste0("est", 1:num_estimate), variable = dimnames(results_list[[1]]$random_fit)[[3]]) } else { random_fit <- NULL }

  elbo_history_list <- list(); elbo_final_vec <- numeric(num_estimate); rel_obj_vec <- numeric(num_estimate)
  for (c in 1:num_estimate) {
    res <- results_list[[c]]; fit[, c, ] <- res$fit[, 1, ]
    if (P_random > 0) random_fit[, c, ] <- res$random_fit[, 1, ]
    elbo_history_list[[c]] <- res$elbo_history - self$prior_correction
    elbo_final_vec[c] <- res$elbo_final - self$prior_correction; rel_obj_vec[c] <- res$rel_obj_final
  }

  if (!is.null(save_info)) {
    for (c in 1:num_estimate) {
      backup_file <- file.path(save_info$dir, paste0(save_info$name, "-", c, ".csv"))
      df_out <- if (!is.null(random_fit)) cbind(iteration = 1:num_samples, as.data.frame(fit[, c, ]), as.data.frame(random_fit[, c, ])) else cbind(iteration = 1:num_samples, as.data.frame(fit[, c, ]))
      write.csv(df_out, file = backup_file, row.names = FALSE)
    }
  }

  best_chain <- which.max(elbo_final_vec)
  posterior_mean <- numeric(length(self$pl_full$names)); names(posterior_mean) <- self$pl_full$names
  fixed_mean <- apply(fit[, best_chain, -1, drop = FALSE], 3, mean); posterior_mean[names(fixed_mean)] <- fixed_mean
  if (!is.null(random_fit)) { random_mean <- apply(random_fit[, best_chain, , drop = FALSE], 3, mean); posterior_mean[names(random_mean)] <- random_mean }

  cat("\nConvergence Diagnostics per estimate:\n")
  for (c in 1:num_estimate) {
    status <- if (!is.na(rel_obj_vec[c]) && rel_obj_vec[c] < tol_rel_obj) "Converged" else "Not Converged"
    best_marker <- if (c == best_chain) "  <-- BEST" else ""
    cat(sprintf("  est%d: ELBO = %10.2f, Final rel_obj = %.5f (%s)%s\n", c, elbo_final_vec[c], rel_obj_vec[c], status, best_marker))
  }
  
  res_obj <- VB_Fit$new(model = self, fit = fit, random_fit = random_fit, elbo_history = elbo_history_list, laplace = laplace, posterior_mean = posterior_mean, ELBO = elbo_final_vec, rel_obj_vals = rel_obj_vec, best_chain = best_chain, mu_history = results_list[[best_chain]]$mu_history)
  if (!is.null(self$transform)) { cat("Calculating transformed parameters...\n"); res_obj$transformed_draws(self$transform) }
  if (!is.null(self$generate)) {
    # --- Post-calculation of generated quantities ---
    # Execute the model's generate block for each posterior sample
    cat("Calculating generated quantities...\n"); res_obj$generated_quantities(self$code$generate) 
  }
  return(res_obj)
}

.log_mean_exp <- function(x) {
  x <- x[is.finite(x)]
  if (length(x) == 0L) return(NA_real_)
  m <- max(x)
  m + log(mean(exp(x - m)))
}

.as_log_lik_matrix <- function(x) {
  if (length(dim(x)) == 3L) {
    dn <- dim(x)
    matrix(x, nrow = dn[1L] * dn[2L], ncol = dn[3L])
  } else {
    mat <- as.matrix(x)
    if (ncol(mat) == 0L) stop("No pointwise log_lik values were found.", call. = FALSE)
    mat
  }
}

.compute_waic_from_log_lik <- function(log_lik_draws) {
  mat <- .as_log_lik_matrix(log_lik_draws)
  if (nrow(mat) < 2L) {
    stop(
      "WAIC requires more than one posterior or approximate posterior draw of pointwise log_lik.",
      call. = FALSE
    )
  }

  lppd_i <- apply(mat, 2L, .log_mean_exp)
  p_waic_i <- apply(mat, 2L, stats::var, na.rm = TRUE)
  p_waic_i[!is.finite(p_waic_i)] <- NA_real_
  elpd_i <- lppd_i - p_waic_i

  pointwise <- data.frame(
    lppd = lppd_i,
    p_waic = p_waic_i,
    WAIC = elpd_i,
    `-2 WAIC` = -2 * elpd_i,
    check.names = FALSE
  )

  out <- list(
    WAIC = sum(elpd_i, na.rm = TRUE),
    minus2_WAIC = -2 * sum(elpd_i, na.rm = TRUE),
    lppd = sum(lppd_i, na.rm = TRUE),
    p_waic = sum(p_waic_i, na.rm = TRUE),
    pointwise = pointwise,
    nobs = ncol(mat),
    ndraws = nrow(mat)
  )
  class(out) <- "waic_BayesRTMB"
  out
}

#' @export
print.waic_BayesRTMB <- function(x, digits = max(3L, getOption("digits") - 2L), ...) {
  cat("WAIC\n")
  cat(sprintf("  WAIC (elpd scale): %.*f\n", digits, x$WAIC))
  cat(sprintf("  -2 WAIC: %.*f\n", digits, x$minus2_WAIC))
  cat(sprintf("  p_waic: %.*f\n", digits, x$p_waic))
  cat(sprintf("  lppd: %.*f\n", digits, x$lppd))
  cat(sprintf("  nobs: %d, draws: %d\n", x$nobs, x$ndraws))
  invisible(x)
}

.waic_from_fit_draws <- function(fit, ...) {
  arr <- tryCatch(
    fit$draws(pars = "log_lik", inc_random = FALSE, inc_transform = FALSE, inc_generate = TRUE, ...),
    error = function(e) NULL
  )
  if (is.null(arr) || length(arr) == 0L || dim(arr)[3L] == 0L) {
    stop(
      "WAIC requires pointwise generated quantity `log_lik`. ",
      "Create the model with WAIC = TRUE or add `log_lik` with generated_quantities().",
      call. = FALSE
    )
  }
  .compute_waic_from_log_lik(arr)
}

.rtmb_block_exprs <- function(ast) {
  if (is.null(ast)) return(list())
  if (is.call(ast) && identical(ast[[1L]], as.name("{"))) as.list(ast)[-1L] else list(ast)
}

.rtmb_merge_generate_ast <- function(existing = NULL, extra = NULL) {
  exprs <- c(.rtmb_block_exprs(existing), .rtmb_block_exprs(extra))
  if (length(exprs) == 0L) return(NULL)
  as.call(c(list(as.name("{")), exprs))
}

.rtmb_assignment_names <- function(exprs) {
  find_assignments <- function(x) {
    if (!is.call(x)) return(character())
    if ((identical(x[[1L]], as.name("<-")) || identical(x[[1L]], as.name("="))) &&
        is.name(x[[2L]])) {
      return(as.character(x[[2L]]))
    }
    unique(unlist(lapply(as.list(x)[-1L], find_assignments), use.names = FALSE))
  }
  unique(unlist(lapply(exprs, find_assignments), use.names = FALSE))
}

.rtmb_list_call_add_log_lik <- function(ret_call, log_lik_name = "log_lik") {
  if (is.call(ret_call) && identical(ret_call[[1L]], as.name("list"))) {
    args <- as.list(ret_call)[-1L]
    arg_names <- names(args)
    if (is.null(arg_names)) arg_names <- rep("", length(args))
    if (!log_lik_name %in% arg_names) {
      args[[length(args) + 1L]] <- as.name(log_lik_name)
      arg_names <- c(arg_names, log_lik_name)
      names(args) <- arg_names
    }
    return(as.call(c(list(as.name("list")), args)))
  }
  NULL
}

.rtmb_waic_generate_ast <- function(existing = NULL, waic = NULL, log_lik_name = "log_lik") {
  waic_exprs <- .rtmb_block_exprs(waic)
  existing_exprs <- .rtmb_block_exprs(existing)
  if (length(waic_exprs) == 0L) return(existing)

  if (length(existing_exprs) == 0L) {
    ret_call <- as.call(c(list(as.name("list")), stats::setNames(list(as.name(log_lik_name)), log_lik_name)))
    return(as.call(c(list(as.name("{")), waic_exprs, list(as.call(list(as.name("return"), ret_call))))))
  }

  last_idx <- length(existing_exprs)
  last_expr <- existing_exprs[[last_idx]]
  if (is.call(last_expr) && identical(last_expr[[1L]], as.name("return"))) {
    ret_expr <- last_expr[[2L]]
    ret_call <- .rtmb_list_call_add_log_lik(ret_expr, log_lik_name)
    if (!is.null(ret_call)) {
      existing_exprs[[last_idx]] <- as.call(list(as.name("return"), ret_call))
      return(as.call(c(list(as.name("{")), existing_exprs[-last_idx], waic_exprs, existing_exprs[last_idx])))
    }
    if (is.name(ret_expr)) {
      add_expr <- as.call(list(as.name("<-"), as.call(list(as.name("$"), ret_expr, as.name(log_lik_name))), as.name(log_lik_name)))
      return(as.call(c(list(as.name("{")), existing_exprs[-last_idx], waic_exprs, list(add_expr), existing_exprs[last_idx])))
    }
  }

  if (is.call(last_expr) && identical(last_expr[[1L]], as.name("list"))) {
    ret_call <- .rtmb_list_call_add_log_lik(last_expr, log_lik_name)
    existing_exprs[[last_idx]] <- as.call(list(as.name("return"), ret_call))
    return(as.call(c(list(as.name("{")), existing_exprs[-last_idx], waic_exprs, existing_exprs[last_idx])))
  }

  if (is.name(last_expr)) {
    add_expr <- as.call(list(as.name("<-"), as.call(list(as.name("$"), last_expr, as.name(log_lik_name))), as.name(log_lik_name)))
    return(as.call(c(list(as.name("{")), existing_exprs[-last_idx], waic_exprs, list(add_expr), list(as.call(list(as.name("return"), last_expr))))))
  }

  returned_names <- .rtmb_assignment_names(existing_exprs)
  ret_args <- lapply(returned_names, as.name)
  names(ret_args) <- returned_names
  ret_args[[length(ret_args) + 1L]] <- as.name(log_lik_name)
  names(ret_args)[length(ret_args)] <- log_lik_name
  ret_call <- as.call(c(list(as.name("list")), ret_args))
  as.call(c(list(as.name("{")), existing_exprs, waic_exprs, list(as.call(list(as.name("return"), ret_call)))))
}

#' Beverage Preference Data
#'
#' @description
#' A dataset containing preference ratings for various beverages.
#' This data was used to demonstrate a random effect model in metric
#' multidimensional unfolding.
#'
#' @format A data frame (or matrix) with [INSERT NUMBER OF ROWS] rows and [INSERT NUMBER OF COLUMNS] columns.
#' \describe{
#'   \item{AJ}{Apple Juice preference rating}
#'   \item{BT}{Black Tea preference rating}
#'   \item{CF}{Coffee preference rating}
#'   \item{CL}{Cola preference rating}
#'   \item{GT}{Green Tea preference rating}
#'   \item{OJ}{Orange Juice preference rating}
#'   \item{OT}{Oolong Tea preference rating}
#' }
#'
#' @source
#' Adachi, K. (2000). A Random Effect Model in Metric Multidimensional Unfolding.
#' \emph{The Japanese Journal of Behaviormetrics}, 27(1), 12-23.
#' \doi{10.2333/jbhmk.27.12}
#'
#' @keywords datasets
"beverage"


#' Big Five Personality Traits Data
#'
#' @description
#' A dataset containing responses to 20 items measuring the Big Five personality traits.
#' This dataset was originally collected by the package author.
#' The items are arranged in a cyclical order representing the five factors:
#' Extraversion, Neuroticism, Openness, Agreeableness, and Conscientiousness.
#'
#' @format A data frame with 10 rows and 20 columns:
#' \describe{
#'   \item{BF1}{Extraversion item 1}
#'   \item{BF2}{Neuroticism item 1}
#'   \item{BF3}{Openness item 1}
#'   \item{BF4}{Agreeableness item 1}
#'   \item{BF5}{Conscientiousness item 1}
#'   \item{BF6}{Extraversion item 2}
#'   \item{BF7}{Neuroticism item 2}
#'   \item{BF8}{Openness item 2}
#'   \item{BF9}{Agreeableness item 2}
#'   \item{BF10}{Conscientiousness item 2}
#'   \item{BF11}{Extraversion item 3}
#'   \item{BF12}{Neuroticism item 3}
#'   \item{BF13}{Openness item 3}
#'   \item{BF14}{Agreeableness item 3}
#'   \item{BF15}{Conscientiousness item 3}
#'   \item{BF16}{Extraversion item 4}
#'   \item{BF17}{Neuroticism item 4}
#'   \item{BF18}{Openness item 4}
#'   \item{BF19}{Agreeableness item 4}
#'   \item{BF20}{Conscientiousness item 4}
#' }
#'
#' @source
#' Originally collected by the package author.
#'
#' @keywords datasets
"BigFive"

#' Debate Simulation Data
#'
#' @description
#' A simulated dataset containing individual satisfaction scores and related
#' variables from a 3-person group debate experiment. This dummy data was
#' created by the package author to demonstrate hierarchical (multilevel) modeling.
#'
#' @format A data frame with 300 rows and 6 columns:
#' \describe{
#'   \item{group}{Group identifier. Each group consists of 3 members.}
#'   \item{sat}{Self-reported satisfaction level of the individual after the debate.}
#'   \item{talk}{Amount of talk or utterances by the individual during the debate.}
#'   \item{perf}{Evaluated performance of the group debate (group-level variable).}
#'   \item{skill}{Evaluated conversation skill of the individual.}
#'   \item{cond}{Experimental condition indicator (e.g., 0 = control, 1 = treatment).}
#' }
#'
#' @source
#' Simulated dummy data created by the package author.
#'
#' @keywords datasets
"debate"

#' Social Skills Training Data
#'
#' @description
#' A small dataset containing repeated outcome measurements from a social
#' skills training study. The outcome was measured at four time points for
#' participants assigned to a control or intervention condition.
#'
#' @format A data frame with 12 rows and 8 columns:
#' \describe{
#'   \item{ID}{Participant identifier.}
#'   \item{time1}{Outcome score at the first measurement occasion.}
#'   \item{time2}{Outcome score at the second measurement occasion.}
#'   \item{time3}{Outcome score at the third measurement occasion.}
#'   \item{time4}{Outcome score at the fourth measurement occasion.}
#'   \item{a}{Training condition indicator: 0 = control group, 1 = intervention group.}
#'   \item{b}{Qualitative baseline skill classification.}
#'   \item{age}{Participant age.}
#' }
#'
#' @source
#' Simulated dummy data created by the package author.
#'
#' @keywords datasets
"training"

#' Plot posterior densities for MCMC samples
#'
#' Draw kernel density plots for each parameter across chains.
#'
#' @param x A 3D array of posterior samples with dimensions
#'   `(iterations, chains, variables)`.
#'   A 2D matrix `(iterations, chains)` is also allowed and is treated
#'   as a single variable.
#' @param mono Logical. If `TRUE`, plot in monochrome. If `FALSE`,
#'   use blue shades.
#'
#' @return No return value. This function is called for its side effect
#'   of plotting.
#'
#' @importFrom graphics plot.new title
#' @export
plot_dens <- function(x, mono = FALSE) {
  if (is.null(dim(x))) {
    stop("`x` must be a matrix or a 3D array.", call. = FALSE)
  }

  dims <- dim(x)

  if (length(dims) == 2) {
    x <- array(x, dim = c(dims[1], dims[2], 1))
    dims <- dim(x)
  }

  if (length(dims) != 3) {
    stop("`x` must have dimension (iterations, chains, variables).", call. = FALSE)
  }

  n_chains <- dims[2]
  n_variables <- dims[3]

  parnames <- dimnames(x)[[3]]
  if (is.null(parnames)) {
    parnames <- paste0("V", seq_len(n_variables))
  }

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  n_cols <- ceiling(sqrt(n_variables))
  n_rows <- ceiling(n_variables / n_cols)

  par(
    mfrow = c(n_rows, n_cols),
    mar = c(4, 4, 3, 1),
    oma = c(0, 0, 2, 0)
  )

  if (isTRUE(mono)) {
    colors <- rep("black", n_chains)
  } else {
    colors <- grDevices::colorRampPalette(c("blue4", "lightblue"))(n_chains)
  }

  for (i in seq_len(n_variables)) {
    dens_list <- vector("list", n_chains)
    max_dens <- 0
    min_x <- Inf
    max_x <- -Inf

    for (n in seq_len(n_chains)) {
      valid_data <- x[, n, i]
      valid_data <- valid_data[is.finite(valid_data)]

      if (length(valid_data) > 1) {
        if (stats::var(valid_data) == 0) {
          valid_data <- jitter(valid_data, amount = 1e-5)
        }
        dens_list[[n]] <- stats::density(valid_data)
        max_dens <- max(max_dens, max(dens_list[[n]]$y))
        min_x <- min(min_x, min(dens_list[[n]]$x))
        max_x <- max(max_x, max(dens_list[[n]]$x))
      }
    }

    valid_idx <- which(!vapply(dens_list, is.null, logical(1)))

    if (length(valid_idx) == 0) {
      plot.new()
      title(main = parnames[i])
      mtext("No finite draws", side = 3, line = -1.5, cex = 0.9)
      next
    }

    plot_idx <- valid_idx[1]

    plot(
      dens_list[[plot_idx]],
      lty = plot_idx,
      col = colors[plot_idx],
      xlab = "",
      ylab = "Density",
      main = parnames[i],
      xlim = c(min_x, max_x),
      ylim = c(0, max_dens * 1.15)
    )

    legend_text <- paste0("chain", plot_idx)
    legend_lty <- plot_idx
    legend_col <- colors[plot_idx]

    if (length(valid_idx) > 1) {
      for (n in valid_idx[-1]) {
        lines(dens_list[[n]], lty = n, col = colors[n])
        legend_text <- c(legend_text, paste0("chain", n))
        legend_lty <- c(legend_lty, n)
        legend_col <- c(legend_col, colors[n])
      }
    }

    legend(
      "topright",
      legend = legend_text,
      lty = legend_lty,
      col = legend_col,
      bg = "transparent",
      bty = "n",
      cex = 0.75
    )
  }

  mtext("Density Plot", outer = TRUE, cex = 1.2, font = 2)
}

#' Plot MCMC trace plots
#'
#' Draw trace plots for each parameter across chains.
#'
#' @param x A 3D array of posterior samples with dimensions
#'   `(iterations, chains, variables)`.
#'   A 2D matrix `(iterations, chains)` is also allowed and is treated
#'   as a single variable.
#' @param mono Logical. If `TRUE`, plot in monochrome. If `FALSE`,
#'   use blue shades.
#'
#' @return No return value. This function is called for its side effect
#'   of plotting.
#'
#' @export
plot_trace <- function(x, mono = FALSE) {
  if (is.null(dim(x))) {
    stop("`x` must be a matrix or a 3D array.", call. = FALSE)
  }

  dims <- dim(x)

  if (length(dims) == 2) {
    x <- array(x, dim = c(dims[1], dims[2], 1))
    dims <- dim(x)
  }

  if (length(dims) != 3) {
    stop("`x` must have dimension (iterations, chains, variables).", call. = FALSE)
  }

  iter <- dims[1]
  n_chains <- dims[2]
  n_variables <- dims[3]

  parnames <- dimnames(x)[[3]]
  if (is.null(parnames)) {
    parnames <- paste0("V", seq_len(n_variables))
  }

  if (isTRUE(mono)) {
    ltys <- seq_len(n_chains)
    colors <- rep("black", n_chains)
  } else {
    ltys <- rep(1, n_chains)
    colors <- grDevices::colorRampPalette(c("blue4", "lightblue"))(n_chains)
  }

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  # Determine grid layout automatically based on the number of variables
  n_cols <- ceiling(sqrt(n_variables))
  n_rows <- ceiling(n_variables / n_cols)

  par(
    mfrow = c(n_rows, n_cols),
    mar = c(4, 4, 3, 1),
    oma = c(0, 0, 2, 0)
  )

  for (i in seq_len(n_variables)) {
    min_val <- min(x[, , i], na.rm = TRUE)
    max_val <- max(x[, , i], na.rm = TRUE)

    # Add 5% padding to the top and bottom of the Y-axis
    pad <- 0.05 * (max_val - min_val)
    if (is.na(pad) || pad == 0) pad <- 1e-8
    y_limit <- c(min_val - pad, max_val + pad)

    plot(
      seq_len(iter),
      x[, 1, i],
      type = "l",
      ylim = y_limit,
      col = colors[1],
      lty = ltys[1],
      xlab = "Iteration",
      ylab = "Value",
      main = parnames[i]
    )

    legend_text <- "chain1"

    if (n_chains > 1) {
      for (j in 2:n_chains) {
        lines(seq_len(iter), x[, j, i], col = colors[j], lty = ltys[j])
        legend_text <- c(legend_text, paste0("chain", j))
      }
    }

    legend(
      "topright",
      legend = legend_text,
      col = colors,
      lty = ltys,
      bg = "transparent",
      bty = "n",
      cex = 0.75
    )
  }

  mtext("Trace Plot", outer = TRUE, cex = 1.2, font = 2)
}

#' Plot autocorrelation for one variable across chains
#'
#' Draw autocorrelation plots for a selected parameter across all chains.
#'
#' @param x A 3D array of posterior samples with dimensions
#'   `(iterations, chains, variables)`.
#' @param var_idx Integer. Index of the variable to plot.
#'
#' @return No return value. This function is called for its side effect
#'   of plotting.
#'
#' @export
plot_acf <- function(x, var_idx = 1) {
  if (is.null(dim(x)) || length(dim(x)) != 3) {
    stop("`x` must be a 3D array with dimension (iterations, chains, variables).",
         call. = FALSE)
  }

  if (!is.numeric(var_idx) || length(var_idx) != 1) {
    stop("`var_idx` must be a single integer.", call. = FALSE)
  }

  n_variables <- dim(x)[3]
  if (var_idx < 1 || var_idx > n_variables) {
    stop("`var_idx` is out of range.", call. = FALSE)
  }

  parnames <- dimnames(x)[[3]]
  if (is.null(parnames)) {
    parnames <- paste0("V", seq_len(n_variables))
  }

  parname <- parnames[var_idx]
  n_chains <- dim(x)[2]

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  par(
    mfrow = c(n_chains, 1),
    mar = c(2, 4.1, 1, 2),
    oma = c(1, 0, 2, 0)
  )

  for (i in seq_len(n_chains)) {
    ac_data <- stats::acf(x[, i, var_idx], plot = FALSE)

    plot(
      x = as.vector(ac_data$lag),
      y = as.vector(ac_data$acf),
      type = "h",
      lwd = 2,
      col = "black",
      xlab = "Lag",
      ylab = paste0("chain", i),
      main = ""
    )
    abline(h = 0)
  }

  mtext(paste0("Autocorrelation (", parname, ")"), outer = TRUE, cex = 1.2, font = 2)
}

#' Plot parameter estimates and credible intervals (Forest Plot)
#'
#' @param x A 3D array of posterior samples with dimensions `(iterations, chains, variables)`.
#' @param prob Numeric. Probability mass for the credible interval (default is 0.95).
#' @param point_estimate Character. Point estimate shown as dots. One of
#'   `"median"` (default), `"EAP"`/`"mean"`, `"MAP"`/`"marginal_map"`, or
#'   `"joint_map"`. `"joint_map"` uses the draw with the largest `lp` variable.
#'
#' @return No return value.
#' @export
plot_forest <- function(x, prob = 0.95,
                        point_estimate = c("median", "EAP", "mean", "MAP", "marginal_map", "joint_map")) {
  if (is.null(dim(x)) || length(dim(x)) != 3) {
    stop("`x` must be a 3D array with dimension (iterations, chains, variables).", call. = FALSE)
  }
  point_estimate <- match.arg(point_estimate)

  n_variables <- dim(x)[3]
  parnames <- dimnames(x)[[3]]
  if (is.null(parnames)) {
    parnames <- paste0("V", seq_len(n_variables))
  }

  # Combine all chains and compute summary statistics for each parameter
  lower_prob <- (1 - prob) / 2
  upper_prob <- 1 - lower_prob

  summaries <- t(apply(x, 3, function(v) {
    v_flat <- as.vector(v[!is.na(v)])
    c(
      mean = mean(v_flat),
      median = median(v_flat),
      lower = quantile(v_flat, probs = lower_prob, names = FALSE),
      upper = quantile(v_flat, probs = upper_prob, names = FALSE)
    )
  }))

  point_values <- switch(
    point_estimate,
    median = summaries[, "median"],
    EAP = summaries[, "mean"],
    mean = summaries[, "mean"],
    MAP = apply(x, 3, function(v) .get_marginal_mode(as.vector(v))),
    marginal_map = apply(x, 3, function(v) .get_marginal_mode(as.vector(v))),
    joint_map = {
      lp_idx <- match("lp", parnames)
      if (is.na(lp_idx)) {
        stop("`point_estimate = 'joint_map'` requires a variable named `lp` in `x`.", call. = FALSE)
      }
      lp_vals <- x[, , lp_idx]
      max_idx <- which(lp_vals == max(lp_vals, na.rm = TRUE), arr.ind = TRUE)
      if (nrow(max_idx) == 0L) {
        stop("Failed to find the maximum `lp` value.", call. = FALSE)
      }
      x[max_idx[1, 1], max_idx[1, 2], ]
    }
  )

  point_label <- switch(
    point_estimate,
    median = "median",
    EAP = "EAP",
    mean = "EAP",
    MAP = "MAP",
    marginal_map = "MAP",
    joint_map = "joint MAP"
  )

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  # Expand left margin to accommodate parameter names
  par(mar = c(5, 6, 3, 2))

  # Reverse Y-axis to order from bottom to top
  y_pos <- seq(n_variables, 1, by = -1)

  plot(
    x = point_values,
    y = y_pos,
    xlim = range(c(summaries[, "lower"], summaries[, "upper"], point_values), na.rm = TRUE),
    ylim = c(0.5, n_variables + 0.5),
    type = "n",
    yaxt = "n",
    ylab = "",
    xlab = "Estimate",
    main = sprintf("Forest Plot (%g%% CI, point = %s)", prob * 100, point_label)
  )

  abline(v = 0, lty = 2, col = "gray50")

  # Draw credible interval lines and selected point estimates
  segments(
    x0 = summaries[, "lower"], y0 = y_pos,
    x1 = summaries[, "upper"], y1 = y_pos,
    lwd = 2, col = "black"
  )
  points(point_values, y_pos, pch = 16, col = "black", cex = 1.2)

  axis(2, at = y_pos, labels = parnames, las = 1, tick = FALSE, line = -0.5)
}

#' Plot Multidimensional Unfolding Configuration
#'
#' @description
#' Draws an item-person map for multidimensional unfolding models. Item
#' locations are shown as labels, optional blue circles show item alpha/radius
#' values, and the respondent locations are summarized by a two-
#' dimensional kernel density contour.
#'
#' @param delta Item coordinate matrix (M items x D dimensions), or a fitted
#'   object with an `EAP()` method.
#' @param theta Person coordinate matrix (N persons x D dimensions). If `delta`
#'   is a fitted object and `theta` is `NULL`, it is extracted from the fit.
#' @param item_alpha Optional item alpha vector used for item radii. If `delta`
#'   is a fitted object and `item_alpha` is `NULL`, `alpha` is extracted when
#'   available.
#' @param phi Deprecated alias for `item_alpha`.
#' @param dims Integer vector of length 2 specifying dimensions to plot.
#' @param radius Numeric plot radius. If `NULL`, a radius is chosen from the
#'   coordinates.
#' @param signs Numeric vector of length 2. Use `-1` to flip an axis.
#' @param item_labels Optional item labels. Defaults to row names or item
#'   numbers.
#' @param show_phi Logical; whether to draw circles based on item alpha.
#' @param show_density Logical; whether to draw density contours for `theta`.
#' @param circle_scale Numeric multiplier for item radii.
#' @param alpha Circle transparency.
#' @param contour_n Grid size passed to `MASS::kde2d()`.
#' @param distance Character; `"auto"`, `"squared"`, or `"euclidean"`. Used
#'   to transform item alpha values into plotted radii. `"auto"` uses the fit's
#'   stored distance when available.
#' @param point_estimate Character; point estimate used when `delta` is a fit
#'   object. Passed to `estimate()`.
#' @param main,xlab,ylab Plot title and axis labels.
#' @param ... Additional arguments passed to `plot()`.
#'
#' @return Invisibly returns a list with plotted `delta`, `theta`, and
#'   `item_alpha`.
#' @export
plot_mdu <- function(delta, theta = NULL, item_alpha = NULL, phi = NULL,
                     dims = c(1, 2), radius = NULL, signs = c(1, 1),
                     item_labels = NULL, show_phi = TRUE, show_density = TRUE,
                     circle_scale = 1, alpha = 0.2, contour_n = 60,
                     distance = c("auto", "squared", "euclidean"),
                     point_estimate = c("EAP", "MAP", "mean", "marginal_map", "joint_map"),
                     main = "MDU Configuration", xlab = NULL, ylab = NULL, ...) {
  point_estimate <- match.arg(point_estimate)
  distance <- match.arg(distance)

  if (!is.matrix(delta) && is.function(delta$estimate)) {
    fit <- delta
    if (distance == "auto") distance <- fit$extra$distance %||% "squared"
    est_type <- switch(
      point_estimate,
      EAP = "EAP",
      mean = "mean",
      MAP = "MAP",
      marginal_map = "marginal_map",
      joint_map = "joint_map"
    )
    est <- fit$estimate(type = est_type, pars = "all", drop = FALSE)
    if (is.null(est$delta)) {
      stop("The fitted object does not contain 'delta'.", call. = FALSE)
    }
    delta <- est$delta
    if (is.null(theta)) theta <- est$theta
    if (is.null(item_alpha)) item_alpha <- est$alpha
    if (is.null(item_alpha) && is.null(phi)) phi <- est$phi
  }
  if (distance == "auto") distance <- "squared"
  if (is.null(item_alpha) && !is.null(phi)) item_alpha <- phi

  delta <- as.matrix(delta)
  if (is.null(theta)) {
    stop("'theta' must be supplied unless 'delta' is a fitted object containing theta.", call. = FALSE)
  }
  theta <- as.matrix(theta)

  D <- ncol(delta)
  if (length(dims) != 2L || any(dims < 1) || any(dims > D)) {
    stop("'dims' must be an integer vector of length 2 within the coordinate dimensions.", call. = FALSE)
  }
  if (ncol(theta) < max(dims)) {
    stop("'theta' does not have enough columns for the requested 'dims'.", call. = FALSE)
  }
  if (length(signs) != 2L || any(!signs %in% c(-1, 1))) {
    stop("'signs' must be a numeric vector of length 2 containing only -1 or 1.", call. = FALSE)
  }

  x_delta <- signs[1] * delta[, dims[1]]
  y_delta <- signs[2] * delta[, dims[2]]
  x_theta <- signs[1] * theta[, dims[1]]
  y_theta <- signs[2] * theta[, dims[2]]

  if (is.null(item_labels)) {
    item_labels <- rownames(delta)
    if (is.null(item_labels)) item_labels <- seq_len(nrow(delta))
  }
  if (length(item_labels) != nrow(delta)) {
    stop("'item_labels' must have the same length as nrow(delta).", call. = FALSE)
  }

  finite_coords <- c(x_delta, y_delta, x_theta, y_theta)
  finite_coords <- finite_coords[is.finite(finite_coords)]
  if (is.null(radius)) {
    radius <- max(abs(finite_coords), na.rm = TRUE)
    if (!is.finite(radius) || radius <= 0) radius <- 1
    radius <- radius * 1.1
  }

  if (is.null(xlab)) xlab <- paste0("Dim ", dims[1])
  if (is.null(ylab)) ylab <- paste0("Dim ", dims[2])

  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))
  par(pty = "s")

  plot(
    c(-radius, radius), c(-radius, radius),
    type = "n", asp = 1, xlab = xlab, ylab = ylab, main = main, ...
  )
  abline(h = 0, v = 0, col = "gray85", lty = 3)

  if (isTRUE(show_density)) {
    ok <- is.finite(x_theta) & is.finite(y_theta)
    if (sum(ok) > 5 && stats::sd(x_theta[ok]) > 0 && stats::sd(y_theta[ok]) > 0) {
      knl <- MASS::kde2d(
        x_theta[ok], y_theta[ok],
        n = contour_n,
        lims = c(-radius, radius, -radius, radius)
      )
      graphics::contour(knl, add = TRUE, drawlabels = TRUE, col = "gray25", lwd = 0.8)
    }
  }

  if (isTRUE(show_phi) && !is.null(item_alpha)) {
    item_alpha <- as.numeric(item_alpha)
    if (length(item_alpha) == 1L) {
      item_radius <- rep(0, nrow(delta))
    } else if (length(item_alpha) == nrow(delta)) {
      item_radius <- pmax(item_alpha - min(item_alpha, na.rm = TRUE), 0)
      if (distance == "squared") item_radius <- sqrt(item_radius)
    } else {
      stop("'item_alpha' must have length 1 or the same length as nrow(delta).", call. = FALSE)
    }
    max_item_radius <- max(item_radius, na.rm = TRUE)
    if (is.finite(max_item_radius) && max_item_radius > 0) {
      item_radius <- item_radius / max_item_radius * radius * 0.35 * circle_scale
      circle_col <- grDevices::rgb(0.1, 0.25, 0.9, alpha = alpha)
      angle <- seq(0, 2 * pi, length.out = 160)
      for (m in seq_along(item_radius)) {
        if (is.finite(item_radius[m]) && item_radius[m] > 0) {
          lines(
            x_delta[m] + item_radius[m] * cos(angle),
            y_delta[m] + item_radius[m] * sin(angle),
            col = circle_col
          )
        }
      }
    }
  }

  points(x_theta, y_theta, pch = 16, col = grDevices::rgb(0, 0, 0, 0.12), cex = 0.45)
  text(x_delta, y_delta, labels = item_labels, font = 2, cex = 0.9)

  invisible(list(delta = cbind(x_delta, y_delta), theta = cbind(x_theta, y_theta), item_alpha = item_alpha))
}

#' Plot pairs for posterior samples
#'
#' Draw a scatterplot matrix to examine posterior correlations between parameters.
#'
#' @param x A 3D array of posterior samples with dimensions `(iterations, chains, variables)`.
#' @param pars Character vector or integer vector specifying which parameters to plot.
#'   If NULL (default), up to the first 10 parameters are plotted to prevent overplotting.
#'
#' @return No return value.
#' @export
plot_pairs <- function(x, pars = NULL) {
  if (is.null(dim(x)) || length(dim(x)) != 3) {
    stop("`x` must be a 3D array with dimension (iterations, chains, variables).", call. = FALSE)
  }

  n_variables <- dim(x)[3]
  parnames <- dimnames(x)[[3]]
  if (is.null(parnames)) {
    parnames <- paste0("V", seq_len(n_variables))
  }

  # Filter target parameters for display
  if (!is.null(pars)) {
    if (is.character(pars)) {
      idx <- match(pars, parnames)
      if (any(is.na(idx))) stop("Some specified parameters were not found.", call. = FALSE)
      plot_idx <- idx
    } else if (is.numeric(pars)) {
      plot_idx <- pars
    }
  } else {
    # Default to maximum of 10 parameters (more would be slow and cluttered)
    plot_idx <- seq_len(min(n_variables, 10))
  }

  if (length(plot_idx) < 2) {
    stop("At least 2 parameters are required for a pairs plot.", call. = FALSE)
  }

  # Combine all chains into a 2D matrix
  mat <- apply(x[, , plot_idx, drop = FALSE], 3, as.vector)
  colnames(mat) <- parnames[plot_idx]

  # Custom function to display correlation coefficients in the upper panels
  panel.cor <- function(x, y, digits = 2, prefix = "", cex.cor, ...) {
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- cor(x, y, use = "complete.obs")
    txt <- format(c(r, 0.123456789), digits = digits)[1]
    txt <- paste0(prefix, txt)
    if(missing(cex.cor)) cex.cor <- 0.8/strwidth(txt)
    # Adjust text size based on the strength of the correlation
    text(0.5, 0.5, txt, cex = cex.cor * (0.5 + abs(r) / 2))
  }

  # Custom function to draw semi-transparent points to visualize density
  panel.scatter <- function(x, y, ...) {
    points(x, y, pch = 16, col = rgb(0, 0, 0, alpha = 0.2), cex = 0.5)
  }

  pairs(mat,
        lower.panel = panel.scatter,
        upper.panel = panel.cor,
        gap = 0.2,
        main = "Pairs Plot")
}

#' Plot least-squares marginal means
#'
#' Calculate and plot marginal means for a fitted model.
#'
#' @param fit A fitted model object (e.g. `Classic_Fit`).
#' @param specs Character vector specifying the variables for which marginal means are calculated.
#' @param ... Additional arguments passed to \code{\link{lsmeans}} or the plot method.
#'
#' @return Invisibly returns the plotted object.
#' @export
plot_lsmeans <- function(fit, specs, ...) {
  res <- lsmeans(fit, specs, ...)
  plot(res, ...)
}

#' Plot conditional effects
#'
#' Calculate and plot conditional effects for a fitted model.
#'
#' @param fit A fitted model object (e.g., `MCMC_Fit`, `MAP_Fit`, `VB_Fit`).
#' @param effect Name of the variable to visualize (e.g., "X1" or "X1:X2").
#' @param ... Additional arguments passed to \code{\link{conditional_effects}} or the plot method.
#'
#' @return Invisibly returns the plotted object of class \code{ce_rtmb}.
#' @export
plot_conditional_effects <- function(fit, effect, ...) {
  res <- conditional_effects(fit, effect, ...)
  plot(res, ...)
}

#' Plot item/category response curves
#'
#' Calculate and plot item/category response curves for a fitted IRT model.
#'
#' @param fit A fitted IRT model object.
#' @param ... Additional arguments passed to \code{\link{item_curve}} or the plot method.
#'
#' @return Invisibly returns the plotted object of class \code{rtmb_item_curve}.
#' @export
plot_item_curve <- function(fit, ...) {
  res <- item_curve(fit, ...)
  plot(res, ...)
}

#' Plot item information functions
#'
#' Calculate and plot item information functions for a fitted IRT model.
#'
#' @param fit A fitted IRT model object.
#' @param ... Additional arguments passed to \code{\link{item_info}} or the plot method.
#'
#' @return Invisibly returns the plotted object of class \code{rtmb_item_info}.
#' @export
plot_item_info <- function(fit, ...) {
  res <- item_info(fit, ...)
  plot(res, ...)
}

#' Plot test information function
#'
#' Calculate and plot test information function for a fitted IRT model.
#'
#' @param fit A fitted IRT model object.
#' @param ... Additional arguments passed to \code{\link{test_info}} or the plot method.
#'
#' @return Invisibly returns the plotted object of class \code{rtmb_test_info}.
#' @export
plot_test_info <- function(fit, ...) {
  res <- test_info(fit, ...)
  plot(res, ...)
}

#' @keywords internal
#' @noRd
dmvnorm_log <- function(x, mean, sigma) {
  if (is.vector(x)) {
    x <- matrix(x, nrow = 1)
  }
  k <- ncol(x)
  U <- chol(sigma)
  log_det <- 2 * sum(log(diag(U)))
  
  x_centered <- sweep(x, 2, mean, "-")
  y_t <- forwardsolve(t(U), t(x_centered))
  mahalanobis_dist <- colSums(y_t^2)
  
  -(k / 2) * log(2 * pi) - 0.5 * log_det - 0.5 * mahalanobis_dist
}

#' MCMC fit object
#'
#' An R6 class storing posterior samples and related information
#' from MCMC estimation.
#'
#' @param model An `RTMB_Model` object used for estimation.
#' @param fit Posterior draws for model parameters.
#' @param random_fit Posterior draws for random effects, if available.
#' @param transform_fit Posterior draws for transformed parameters, if available.
#' @param transform_dims Dimension information for transformed parameters.
#' @param generate_fit Posterior draws for generated quantities, if available.
#' @param generate_dims Dimension information for generated quantities.
#' @param eps Step size used by the sampler.
#' @param accept Acceptance statistics from sampling.
#' @param treedepth Tree depth used in HMC/NUTS sampling.
#' @param laplace Logical; whether Laplace approximation was used.
#' @param posterior_mean Posterior mean estimates.
#' @param pars Names of parameters to extract or summarize.
#' @param chains Chains to include.
#' @param inc_random Logical; whether to include random effects.
#' @param inc_transform Logical; whether to include transformed parameters.
#' @param inc_generate Logical; whether to include generated quantities.
#' @param max_rows Maximum number of rows to print in summaries.
#' @param digits Number of digits to print.
#' @param method Method name.
#' @param seed Random seed.
#' @param tran_fn A function for transformed parameters.
#' @param gq_fn A function for generated quantities.
#' @param target Target variable(s) for rotation or relabeling.
#' @param type Rotation type.
#' @param linked_straight Linked variables transformed in the same direction.
#' @param linked_inverse Linked variables transformed in the opposite direction.
#' @param overwrite Logical; whether to overwrite stored draws.
#' @param linked Linked variables.
#' @param loadings Factor loading variables.
#' @param scores Factor score variables.
#' @param linked_loadings Linked loading variables.
#' @param ... Additional arguments.
#'
#' @field model An `RTMB_Model` object used for estimation.
#' @field fit Posterior draws for model parameters.
#' @field random_fit Posterior draws for random effects.
#' @field transform_fit Posterior draws for transformed parameters.
#' @field transform_dims Dimension information for transformed parameters.
#' @field generate_fit Posterior draws for generated quantities.
#' @field generate_dims Dimension information for generated quantities.
#' @field eps Step size used by the sampler.
#' @field accept Acceptance statistics from sampling.
#' @field treedepth Tree depth used in HMC/NUTS sampling.
#' @field laplace Logical; whether Laplace approximation was used.
#' @field posterior_mean Posterior mean estimates.
#' @field log_ml Numeric value storing the calculated log marginal likelihood from bridge sampling.
#' @field comparison_fit An \code{MCMC_Fit} object containing the fitted comparison model.
#' @field max_treedepth Maximum tree depth requested for NUTS/HMC.
#' @field pd_error_count Positive-definite/singularity errors treated as lp = -Inf by chain.
#'


#' @export
MCMC_Fit <- R6::R6Class(
  classname = "mcmc_fit",
  inherit = RTMB_Fit_Base,

  public = list(
    # --- Validation and aggregation of sampling results ---
    model          = NULL, # Reference to RTMB_Model instance
    fit            = NULL,
    random_fit     = NULL,
    transform_fit  = NULL, # Store transformed quantities
    generate_fit   = NULL, # Store generated quantities
    transform_dims = NULL, # Store dimension info for transformed quantities
    generate_dims  = NULL, # Store dimension info for generated quantities
    eps            = NULL,
    accept         = NULL,
    treedepth      = NULL,
    laplace        = NULL,
    posterior_mean = NULL,
    log_ml          = NULL,
    comparison_fit = NULL,
    max_treedepth  = NULL,
    pd_error_count = NULL,

    # 1. Constructor
    #' @description Get point estimate for a target parameter.
    #' @param target Target parameter name.
    #' @param chains Numeric vector of chains to include. If NULL, all chains are used.
    #' @param best_chains Integer; number of best chains to retain based on mean log-posterior.
    #' @return Matrix, array, vector, or scalar point estimate.
    get_point_estimate = function(target, chains = NULL, best_chains = NULL) {
      target_draws <- self$draws(pars = target, chains = chains, best_chains = best_chains, inc_transform = TRUE, inc_generate = TRUE)
      if (dim(target_draws)[3] == 0) stop("Parameter not found: ", target)

      lp_draws <- self$draws(pars = "lp", chains = chains, best_chains = best_chains, inc_transform = FALSE, inc_generate = FALSE)
      if (all(is.na(lp_draws))) stop("Log-probability ('lp') contains only NAs.")
      max_idx <- which(lp_draws == max(lp_draws, na.rm = TRUE), arr.ind = TRUE)
      if (nrow(max_idx) == 0) stop("Failed to find the maximum log-probability.")
      best_iter <- max_idx[1, 1]
      best_chain <- max_idx[1, 2]

      t_info <- self$model$par_list[[target]]
      if (!is.null(t_info)) {
        t_dim <- t_info$dim
      } else if (!is.null(self$transform_dims[[target]])) {
        t_dim <- self$transform_dims[[target]]
      } else if (!is.null(self$generate_dims[[target]])) {
        t_dim <- self$generate_dims[[target]]
      } else {
        t_dim <- dim(target_draws)[3]
      }

      target_map_flat <- target_draws[best_iter, best_chain, ]
      target_map <- target_map_flat
      if (length(t_dim) > 1) dim(target_map) <- t_dim

      return(target_map)
    },


    #' @description Create a new `MCMC_Fit` object.
    #' @param model An `RTMB_Model` object used for estimation.
    #' @param fit Posterior draws for model parameters.
    #' @param random_fit Posterior draws for random effects, if available.
    #' @param eps Step size used by the sampler.
    #' @param accept Acceptance statistics from sampling.
    #' @param treedepth Tree depth used in HMC/NUTS sampling.
    #' @param laplace Logical; whether Laplace approximation was used.
    #' @param posterior_mean Posterior mean estimates.
    #' @param max_treedepth Maximum tree depth requested for NUTS/HMC.
    #' @param pd_error_count Positive-definite/singularity errors treated as `lp = -Inf` by chain.
    initialize = function(model, fit, random_fit, eps, accept, treedepth, laplace,
                          posterior_mean, max_treedepth = NULL, pd_error_count = NULL) {
      self$model <- model
      self$fit <- fit
      self$random_fit <- random_fit
      self$eps <- eps
      self$accept <- accept
      self$treedepth <- treedepth
      self$laplace <- laplace
      self$posterior_mean <- posterior_mean
      self$max_treedepth <- max_treedepth
      self$pd_error_count <- pd_error_count
      self$transform_fit <- NULL
      self$transform_dims <- list()
      self$generate_fit <- NULL
      self$generate_dims <- list()
      # Ensure S3 dispatch works for base class methods
      class(self) <- c(class(self), "RTMB_Fit_Base")
    },

    #' @description Print a brief summary of the fitted object.
    #' @return The object itself, invisibly.
    print = function(...) {
      print(self$summary(...))
      invisible(self)
    },

    #' @description Extract posterior draws for selected parameters.
    #' @param pars Character or numeric vector specifying the names or indices of parameters to extract. If NULL, all available parameters are extracted.
    #' @param chains Numeric vector specifying the chains to extract. If NULL, draws from all chains are returned.
    #' @param best_chains Integer; number of best chains to retain based on mean log-posterior (lp).
    #' @param inc_random Logical; whether to include random effects in the output. Default is FALSE.
    #' @param inc_transform Logical; whether to include transformed parameters in the output. Default is TRUE.
    #' @param inc_generate Logical; whether to include generated quantities in the output. Default is TRUE.
    #' @return Posterior draws.
    draws = function(pars = NULL, chains = NULL, best_chains = NULL,
                     inc_random = FALSE, inc_transform = TRUE, inc_generate = TRUE) {

      # 1. Determine chains to use
      total_chains <- dim(self$fit)[2]
      available_chains <- 1:total_chains

      if (!is.null(chains)) {
        available_chains <- intersect(available_chains, chains)
        if (length(available_chains) == 0) {
          stop("The specified chains were not found.", call. = FALSE)
        }
      }

      if (!is.null(best_chains)) {
        lp_idx <- which(dimnames(self$fit)[[3]] == "lp")
        if (length(lp_idx) == 0) lp_idx <- 1

        # Calculate mean of lp for available_chains
        lp_means <- apply(self$fit[, available_chains, lp_idx, drop = FALSE], 2, mean, na.rm = TRUE)
        n_best <- min(best_chains, length(available_chains))
        ordered_idx <- order(lp_means, decreasing = TRUE)
        selected_rel_idx <- ordered_idx[1:n_best]
        available_chains <- available_chains[selected_rel_idx]
      }

      available_chains <- sort(available_chains)

      # 2. Extract and bind arrays
      out_array <- self$fit[, available_chains, , drop = FALSE]

      if (inc_random && !is.null(self$random_fit)) {
        P1 <- dim(out_array)[3]; P2 <- dim(self$random_fit)[3]
        I <- dim(out_array)[1]; C <- length(available_chains)
        new_out <- array(NA, dim = c(I, C, P1 + P2))
        new_out[,,1:P1] <- out_array
        new_out[,,(P1+1):(P1+P2)] <- self$random_fit[, available_chains, , drop = FALSE]
        dimnames(new_out) <- list(
          iteration = dimnames(out_array)[[1]],
          chain = dimnames(out_array)[[2]],
          variable = c(dimnames(out_array)[[3]], dimnames(self$random_fit)[[3]])
        )
        out_array <- new_out
      }

      if (inc_transform && !is.null(self$transform_fit)) {
        P1 <- dim(out_array)[3]; P2 <- dim(self$transform_fit)[3]
        I <- dim(out_array)[1]; C <- length(available_chains)
        new_out <- array(NA, dim = c(I, C, P1 + P2))
        new_out[,,1:P1] <- out_array
        new_out[,,(P1+1):(P1+P2)] <- self$transform_fit[, available_chains, , drop = FALSE]
        dimnames(new_out) <- list(
          iteration = dimnames(out_array)[[1]],
          chain = dimnames(out_array)[[2]],
          variable = c(dimnames(out_array)[[3]], dimnames(self$transform_fit)[[3]])
        )
        out_array <- new_out
      }

      if (inc_generate && !is.null(self$generate_fit)) {
        P1 <- dim(out_array)[3]; P2 <- dim(self$generate_fit)[3]
        I <- dim(out_array)[1]; C <- length(available_chains)
        new_out <- array(NA, dim = c(I, C, P1 + P2))
        new_out[,,1:P1] <- out_array
        new_out[,,(P1+1):(P1+P2)] <- self$generate_fit[, available_chains, , drop = FALSE]
        dimnames(new_out) <- list(
          iteration = dimnames(out_array)[[1]],
          chain = dimnames(out_array)[[2]],
          variable = c(dimnames(out_array)[[3]], dimnames(self$generate_fit)[[3]])
        )
        out_array <- new_out
      }

      P <- dim(out_array)[3]
      param_names <- dimnames(out_array)[[3]]
      if (is.null(param_names)) param_names <- paste0("V", 1:P)

      target_idx <- 1:P

      if (!is.null(pars)) {
        if (is.numeric(pars)) {
          # Support both positive and negative numeric indexing
          # In R, array[,,pars] just works for both!
          # But we want to validate if possible
          target_idx <- tryCatch(seq_along(param_names)[pars], error = function(e) NULL)
          if (is.null(target_idx) || length(target_idx) == 0) {
            stop("The index specified in 'pars' was not found or is invalid.", call. = FALSE)
          }
        } else if (is.character(pars)) {
          base_names <- gsub("\\[.*\\]$", "", param_names)
          
          # Check for exclusion (e.g., "-theta")
          if (any(grepl("^-", pars))) {
            exclude_names <- gsub("^-", "", pars)
            target_idx <- which(!(param_names %in% exclude_names | base_names %in% exclude_names))
          } else {
            ordered_idx <- integer(0)
            for (p in pars) {
              match_idx <- which(param_names == p | base_names == p)
              ordered_idx <- c(ordered_idx, match_idx)
            }
            target_idx <- unique(ordered_idx)
          }
          
          if (length(target_idx) == 0) {
            stop("The variable name specified in 'pars' was not found.", call. = FALSE)
          }
        } else {
          stop("'pars' must be either numeric or character.", call. = FALSE)
        }
      }

      return(out_array[, , target_idx, drop = FALSE])
    },

    #' @description Summarize posterior draws.
    #' @param pars Character or numeric vector specifying the names or indices of parameters to summarize. If NULL, all available parameters are summarized.
    #' @param chains Numeric vector specifying the chains to extract. If NULL, draws from all chains are used.
    #' @param best_chains Integer; number of best chains to retain based on mean log-posterior (lp).
    #' @param max_rows Integer; maximum number of rows to print in the summary table. Default is 10.
    #' @param digits Integer; number of decimal places to print. Default is 2.
    #' @param inc_random Logical; whether to include random effects in the summary. Default is FALSE.
    #' @param inc_transform Logical; whether to include transformed parameters in the summary. Default is TRUE.
    #' @param inc_generate Logical; whether to include generated quantities in the summary. Default is TRUE.
    #' @return A summary object.
    summary = function(pars = NULL, chains = NULL, best_chains = NULL,
                       max_rows = 10, digits = 2,
                       inc_random = FALSE, inc_transform = TRUE, inc_generate = TRUE){
      draws_array <- self$draws(pars = pars,
                                chains = chains,
                                best_chains = best_chains,
                                inc_random = inc_random,
                                inc_transform = inc_transform,
                                inc_generate = inc_generate)

      P <- dim(draws_array)[3]
      param_names <- dimnames(draws_array)[[3]]

      target_idx <- 1:P

      # --- Priority sorting by lp and model$view ---
      if (length(target_idx) > 0 && is.null(pars)) {
        current_names <- param_names[target_idx]
        base_names <- gsub("\\[.*\\]$", "", current_names)

        # Always prioritize lp
        target_views <- c("lp")
        if (!is.null(self$model$view)) {
          target_views <- c(target_views, self$model$view)
        }

        priority_sub_idx <- integer(0)
        for (v in target_views) {
          match_idx <- which(current_names == v | base_names == v)
          priority_sub_idx <- c(priority_sub_idx, match_idx)
        }
        priority_sub_idx <- unique(priority_sub_idx)
        other_sub_idx <- setdiff(seq_along(current_names), priority_sub_idx)
        target_idx <- target_idx[c(priority_sub_idx, other_sub_idx)]
      }

      if (!is.null(max_rows)) {
        limit <- min(length(target_idx), as.integer(max_rows))
        target_idx <- target_idx[1:limit]
      }

      res_list_sum <- vector("list", length(target_idx))

      for (i in seq_along(target_idx)) {
        p <- target_idx[i]
        mat_p <- as.matrix(draws_array[, , p])
        vec_p <- as.vector(mat_p)
        valid_vec <- vec_p[is.finite(vec_p)]

        if (length(valid_vec) == 0) {
          res_list_sum[[i]] <-
            data.frame(
              variable = param_names[p],
              mean = NA,
              sd = NA,
              map = NA,
              q2.5 = NA,
              q97.5 = NA,
              ess_bulk = NA,
              ess_tail = NA,
              rhat = NA,
              stringsAsFactors = FALSE
            )
          next
        }

        sd_val <- sd(valid_vec)
        if (is.na(sd_val) || sd_val < 1e-10) {
          map_val <- valid_vec[1]; q95 <- c(valid_vec[1], valid_vec[1])
          rhat_val <- NA; ebulk_val <- NA; etail_val <- NA
        } else {
          map_val   <- map_est(valid_vec)
          q95       <- quantile95(valid_vec)
          rhat_val  <- r_hat(mat_p)
          ebulk_val <- ess_bulk(mat_p)
          etail_val <- ess_tail95(mat_p)
        }

        res_list_sum[[i]] <- data.frame(
          variable = param_names[p],
          mean     = mean(valid_vec),
          sd       = sd_val,
          map      = map_val,
          q2.5     = unname(q95[1]),
          q97.5    = unname(q95[2]),
          ess_bulk = ebulk_val,
          ess_tail = etail_val,
          rhat     = rhat_val,
          stringsAsFactors = FALSE
        )
      }
      res_df <- do.call(rbind, res_list_sum)
      # Replace extremely small values close to zero with exact 0 in numeric columns of the dataframe
      num_cols <- sapply(res_df, is.numeric)
      res_df[num_cols] <- lapply(res_df[num_cols], function(x) {
        x[abs(x) < 1e-12 & !is.na(x)] <- 0
        return(x)
      })
      class(res_df) <- c("summary_BayesRTMB", "data.frame")
      attr(res_df, "digits") <- digits
      return(res_df)
    },

    #' @description Transform posterior draws to the unconstrained scale.
    #' @return Posterior draws on the unconstrained scale.
    unconstrain_draws = function() {
      I <- dim(self$fit)[1]
      C <- dim(self$fit)[2]
      orig_pl <- self$model$par_list
      random_flags <- vapply(orig_pl, function(x) isTRUE(x$random), logical(1))

      target_par_list <- if (self$laplace && any(random_flags)) orig_pl[!random_flags] else orig_pl
      draws_ob <- self$fit[, , -1, drop = FALSE]

      # --- Modified: Calculate the number of effective parameters considering map ---
      map_list <- self$model$map
      if (!is.null(map_list)) {
        Q <- 0
        for (name in names(target_par_list)) {
          L <- target_par_list[[name]]$unc_length
          if (L > 0) {
            if (!is.null(map_list[[name]])) {
              Q <- Q + sum(!is.na(map_list[[name]]))
            } else {
              Q <- Q + L
            }
          }
        }
      } else {
        Q <- sum(vapply(target_par_list, function(x) x$unc_length, numeric(1)))
      }

      draws_uc <- array(NA, dim = c(I, C, Q))

      for (i in 1:I) {
        for (j in 1:C) {
          con_vec <- draws_ob[i, j, ]
          con_list <- constrained_vector_to_list(con_vec, target_par_list)
          unc_list <- to_unconstrained(con_list, target_par_list)

          # --- Modified: Extract only non-fixed parameters if map is specified ---
          if (!is.null(map_list)) {
            unc_vec <- c()
            for (name in names(target_par_list)) {
              val <- unc_list[[name]]
              if (length(val) > 0) {
                if (!is.null(map_list[[name]])) {
                  unc_vec <- c(unc_vec, val[!is.na(map_list[[name]])])
                } else {
                  unc_vec <- c(unc_vec, val)
                }
              }
            }
            draws_uc[i, j, ] <- unc_vec
          } else {
            draws_uc[i, j, ] <- unlist(unc_list, use.names = FALSE)
          }
        }
      }
      return(matrix(draws_uc, nrow = I * C, ncol = Q))
    },

    #' @description Evaluate log-probability values.
    #' @param safe Logical; whether to wrap the evaluation in a tryCatch block. Default is FALSE for speed.
    #' @return Numeric vector of log-probability values.
    log_prob = function(safe = FALSE) {
      ad_setup <- self$model$build_ad_obj(init = self$posterior_mean, laplace = self$laplace, jacobian_target = "all")
      ad_obj <- ad_setup$ad_obj

      if (safe) {
        fn <- function(q) {
          val <- tryCatch({ -ad_obj$fn(q) }, error = function(e) -Inf)
          if (is.na(val) || is.nan(val)) return(-Inf)
          return(val)
        }
      } else {
        fn <- function(q) {
          val <- -ad_obj$fn(q)
          if (is.na(val) || is.nan(val)) return(-Inf)
          return(val)
        }
      }
      return(fn)
    },

    #' @description Estimate the marginal likelihood by bridge sampling.
    #' @param method Character; the method to use for bridge sampling (e.g., "warp3", "normal"). Default is "warp3".
    #' @param use_neff Logical; whether to use the effective sample size (ESS) to adjust for autocorrelation. Default is TRUE.
    #' @param seed Integer; random seed for reproducibility. Default is NULL.
    #' @param max_iter Integer; maximum number of iterations for the estimation algorithm. Default is 100.
    #' @return Bridge sampling result.
    # Calculation of log marginal likelihood for Bridge Sampling
    bridgesampling = function(method = "normal", use_neff = TRUE, seed = NULL, max_iter = 100) {
      if (!is.null(seed)) set.seed(seed)

      draws_uc <- self$unconstrain_draws()
      log_prob_fn <- self$log_prob()

      N_total <- nrow(draws_uc)
      Q <- ncol(draws_uc)

      # Handle models with no free parameters
      if (Q == 0) {
        logml.bs <- log_prob_fn(numeric(0))
        
        correction <- self$model$prior_correction
        if (!is.null(correction) && correction != 0) {
          logml.bs <- logml.bs - correction
        }
        
        cat(sprintf("Bridge Sampling Converged (Fixed Point): LogML = %.3f\n", logml.bs))
        res <- logml.bs
        attr(res, "error") <- 0
        attr(res, "ess") <- N_total
        self$log_ml <- res
        return(invisible(res))
      }

      index1 <- seq(1, N_total - 1, by = 2)
      index2 <- seq(2, N_total, by = 2)
      z_post <- draws_uc[index1, , drop = FALSE]
      z_fit  <- draws_uc[index2, , drop = FALSE]

      M1 <- nrow(z_post)
      M2 <- nrow(z_fit)
      M  <- M1 + M2

      # Pre-computation: parameter estimation of proposal distribution
      meanz <- apply(z_fit, 2, mean)
      covz <- cov(z_fit)

      # Calculate log-posterior for posterior samples
      lp_post <- apply(z_post, 1, log_prob_fn)

      if (isTRUE(use_neff)) {
        n_eff <- tryCatch({
          ess_basic(lp_post)
        }, error = function(e) M1)
        if (is.na(n_eff) || n_eff <= 0) n_eff <- M1
      } else {
        n_eff <- M1
      }

      # Optimal weights based on Meng & Wong
      S1 <- n_eff / (n_eff + M2)
      S2 <- M2 / (n_eff + M2)

      if (method == "normal") {
        z_propose <- MASS::mvrnorm(M2, meanz, covz)
        log_propose <- function(z) dmvnorm_log(z, meanz, covz)

        # Reuse pre-calculated lp_post
        log_L1 <- lp_post - log_propose(z_post)
        log_L2 <- apply(z_propose, 1, log_prob_fn) - log_propose(z_propose)

        valid_log_L1 <- log_L1[is.finite(log_L1)]
        if (length(valid_log_L1) == 0) stop("No valid posterior samples found.")
        log_Lm <- median(valid_log_L1)

        L1 <- exp(log_L1 - log_Lm)
        L2 <- exp(log_L2 - log_Lm)
        L1[is.na(L1) | is.nan(L1)] <- 0
        L2[is.na(L2) | is.nan(L2)] <- 0

        ml_t <- exp(log_prob_fn(meanz) - log_Lm)
        if (is.na(ml_t) || is.nan(ml_t) || ml_t == 0) ml_t <- 1e-10

      } else if (method == "warp3") {
        L_chol <- t(chol(covz + diag(0.0001, Q)))
        log_det_L <- sum(log(diag(L_chol)))

        eta_post <- t(solve(L_chol, t(sweep(z_post, 2, meanz, "-"))))
        eta_prop <- matrix(rnorm(M2 * Q), nrow = M2, ncol = Q)

        log_propose_std <- function(eta) {
          sum(dnorm(eta, mean = 0, sd = 1, log = TRUE))
        }

        log_target_warp3 <- function(eta) {
          z_plus  <- meanz + as.vector(L_chol %*% eta)
          z_minus <- meanz - as.vector(L_chol %*% eta)

          lp_plus  <- log_prob_fn(z_plus)
          lp_minus <- log_prob_fn(z_minus)

          max_lp <- max(lp_plus, lp_minus)
          if (is.infinite(max_lp)) return(-Inf)

          log_mix <-
            log(0.5) + max_lp + log(exp(lp_plus - max_lp) + exp(lp_minus - max_lp))
          return(log_mix + log_det_L)
        }

        log_L1 <- apply(eta_post, 1, log_target_warp3) - apply(eta_post, 1, log_propose_std)
        log_L2 <- apply(eta_prop, 1, log_target_warp3) - apply(eta_prop, 1, log_propose_std)

        valid_log_L1 <- log_L1[is.finite(log_L1)]
        if (length(valid_log_L1) == 0) stop("No valid posterior samples found.")
        log_Lm <- median(valid_log_L1)

        L1 <- exp(log_L1 - log_Lm)
        L2 <- exp(log_L2 - log_Lm)
        L1[is.na(L1) | is.nan(L1)] <- 0
        L2[is.na(L2) | is.nan(L2)] <- 0

        ml_t <- exp(log_target_warp3(rep(0, Q)) - log_Lm)
        if (is.na(ml_t) || is.nan(ml_t) || ml_t == 0) ml_t <- 1e-10

      } else {
        stop("The 'method' argument must be either 'normal' or 'warp3'.")
      }

      Trial <- max_iter
      for(t in 1:Trial){
        ml_t_old <- ml_t
        bunbo <- 0
        bunshi <- 0
        for(m in 1:M1) bunbo <- bunbo + 1/(S1*L1[m] + S2*ml_t)
        for(m in 1:M2) bunshi <- bunshi + L2[m] / (S1*L2[m] + S2*ml_t)

        ml_t <- (bunshi/M2) / (bunbo/M1)

        if (is.na(ml_t) || is.nan(ml_t) || ml_t <= 0) {
          warning("ml_t became 0 or NA during calculation.")
          break
        }
        if(abs(log(ml_t) - log(ml_t_old)) < 0.000001){
          break
        }
      }

      logml.bs <- as.numeric(log(ml_t) + log_Lm)

      correction <- self$model$prior_correction
      if (!is.null(correction) && correction != 0) {
        logml.bs <- logml.bs - correction
      }

      # 2. Calculation of approximate standard error
      res <- logml.bs

      if (method == "normal") {
        f1_vec <- ml_t / (S1 * L1 + S2 * ml_t)
        f2_vec <- L2 / (S1 * L2 + S2 * ml_t)

        V1 <- var(f1_vec)
        V2 <- var(f2_vec)
        E1 <- mean(f1_vec)
        E2 <- mean(f2_vec)

        re2 <- V1 / (n_eff * E1^2) + V2 / (M2 * E2^2)
        error_logml <- sqrt(re2)

        cat(sprintf("Bridge Sampling Converged: LogML = %.3f (Error = %.4f, ESS = %.1f)\n", logml.bs, error_logml, n_eff))
        attr(res, "error") <- error_logml

      } else {
        # Return NA if error approximation is difficult, such as in warp3
        cat(sprintf("Bridge Sampling Converged: LogML = %.3f (ESS = %.1f)\n", logml.bs, n_eff))
        attr(res, "error") <- NA_real_
      }

      attr(res, "ess") <- n_eff
      self$log_ml <- res

      return(invisible(res))
    },

    #' @description Compute WAIC from pointwise generated log likelihood.
    #' @param ... Additional arguments passed to `draws()`, such as `chains` or `best_chains`.
    #' @return A `waic_BayesRTMB` object.
    WAIC = function(...) {
      .waic_from_fit_draws(self, ...)
    },

    #' @description Run basic diagnostics for the MCMC fit.
    #' @param ... Additional arguments passed to `diagnose_mcmc_fit()`.
    #' @return A `diagnose_BayesRTMB` object.
    diagnose = function(...) {
      diagnose_mcmc_fit(self, ...)
    },

    #' @description Calculate the Bayes factor by marginal-likelihood model comparison.
    #'
    #' @details
    #' Bayes factors are computed as ratios of marginal likelihoods estimated
    #' by bridge sampling. The comparison model can be constructed by fixing
    #' parameters with `fixed = list(...)`, or supplied as an already fitted
    #' `MCMC_Fit` object via `comparison_fit`.
    #' Bayes factors are computed only by marginal-likelihood model comparison.
    #'
    #' @param fixed Named list of parameter values used to construct the comparison model.
    #'   For example, \code{fixed = list(delta = 0)} or
    #'   \code{fixed = list("b[x]" = 0)}.
    #' @param comparison_fit Optional `MCMC_Fit` object for an already fitted comparison model.
    #' @param bs_method Character; the method to use for bridge sampling ("normal" or "warp3").
    #' @param error_threshold Numeric; threshold for the approximate error warning.
    #' @param ... Additional arguments passed to `sample()` when fitting the comparison model (e.g., \code{chains = 4}, \code{sampling = 4000}).
    #' @return A list of class \code{bayes_factor_rtmb} containing Bayes factor results.
    bayes_factor = function(
      fixed = NULL,
      comparison_fit = NULL,
      bs_method = "normal",
      error_threshold = 0.2,
      ...
    ) {
      fixed_supplied <- !is.null(fixed)
      fit_supplied <- !is.null(comparison_fit)

      if (identical(fixed_supplied, fit_supplied)) {
        stop(
          "Specify exactly one of `fixed` or `comparison_fit`.",
          call. = FALSE
        )
      }

      if (fixed_supplied) {
        if (!is.list(fixed) || is.null(names(fixed)) || any(names(fixed) == "")) {
          stop(
            "`fixed` must be a named list, e.g. `fixed = list(delta = 0)` ",
            "or `fixed = list(\"b[x]\" = 0)`.",
            call. = FALSE
          )
        }
      }

      if (fit_supplied) {
        if (!(inherits(comparison_fit, "R6") && "bridgesampling" %in% names(comparison_fit))) {
          stop(
            "`comparison_fit` must be an MCMC_Fit-like object with a `bridgesampling()` method.",
            call. = FALSE
          )
        }
      }

      # 1. Full model marginal likelihood
      if (is.null(self$log_ml)) {
        cat("Calculating marginal likelihood for the full model...\n")
        self$log_ml <- self$bridgesampling(method = bs_method)
      }
      log_ml1 <- self$log_ml

      # 2. Comparison model marginal likelihood
      if (fixed_supplied) {
        mdl_comp <- self$model$fixed_model(fixed = fixed)

        sample_args <- list(...)

        if (is.null(sample_args$sampling)) {
          sample_args$sampling <- dim(self$fit)[1]
        }
        if (is.null(sample_args$chains)) {
          sample_args$chains <- dim(self$fit)[2]
        }

        cat("\n--- Sampling from the comparison model ---\n")
        fit_comp <- do.call(mdl_comp$sample, sample_args)

        cat("\n--- Calculating marginal likelihood for the comparison model ---\n")
        fit_comp$log_ml <- fit_comp$bridgesampling(method = bs_method)

        log_ml2 <- fit_comp$log_ml
        self$comparison_fit <- fit_comp

      } else {
        if (is.null(comparison_fit$log_ml)) {
          cat("Calculating marginal likelihood for the comparison model...\n")
          comparison_fit$log_ml <- comparison_fit$bridgesampling(method = bs_method)
        }

        log_ml2 <- comparison_fit$log_ml
        self$comparison_fit <- comparison_fit
      }

      # 3. Bayes factor
      val1 <- as.numeric(log_ml1)
      val2 <- as.numeric(log_ml2)

      log_bf <- val1 - val2
      bf <- exp(log_bf)

      err1 <- attr(log_ml1, "error")
      err2 <- attr(log_ml2, "error")
      has_error <- !is.null(err1) && !is.null(err2) && !is.na(err1) && !is.na(err2)

      if (has_error) {
        log_bf_err <- sqrt(err1^2 + err2^2)

        if (log_bf_err > error_threshold) {
          warning(
            sprintf(
              "The estimation error of the log Bayes factor (%.3f) exceeds the threshold (%g). ",
              log_bf_err,
              error_threshold
            ),
            "Consider increasing the number of MCMC samples or ESS.",
            call. = FALSE,
            immediate. = TRUE
          )
        }
      } else {
        log_bf_err <- NA_real_
      }

      if (is.na(bf) || is.nan(bf)) {
        evidence <- "Indeterminate"
      } else if (bf > 100) {
        evidence <- "Decisive evidence for Model 1"
      } else if (bf > 10) {
        evidence <- "Strong evidence for Model 1"
      } else if (bf > 3) {
        evidence <- "Substantial evidence for Model 1"
      } else if (bf > 1) {
        evidence <- "Anecdotal evidence for Model 1"
      } else if (bf == 1) {
        evidence <- "No evidence"
      } else if (bf >= 1 / 3) {
        evidence <- "Anecdotal evidence for Model 2"
      } else if (bf >= 1 / 10) {
        evidence <- "Substantial evidence for Model 2"
      } else if (bf >= 1 / 100) {
        evidence <- "Strong evidence for Model 2"
      } else {
        evidence <- "Decisive evidence for Model 2"
      }

      res <- list(
        BF12 = bf,
        log_BF12 = log_bf,
        log_ml_model1 = val1,
        log_ml_model2 = val2,
        log_BF_error = log_bf_err,
        evidence = evidence,
        fixed = fixed,
        comparison_fit = self$comparison_fit
      )

      class(res) <- "bayes_factor_rtmb"
      res
    },


    #' @description Compute transformed parameters from posterior draws.
    #' @return Transformed parameter draws.
    transformed_draws = function(tran_fn = NULL) {
      all_draws <- self$draws(
        inc_random = TRUE,
        inc_transform = FALSE,
        inc_generate = FALSE
      )

      iter   <- dim(all_draws)[1]
      chains <- dim(all_draws)[2]

      if (is.null(tran_fn)) tran_fn <- self$model$transform

      wrapper_tran_fn <- function(dat, param) {
        res <- list()

        if (!is.null(tran_fn)) {
          user_res <- tran_fn(dat, param)

          if (is.null(user_res)) {
            user_res <- list()
          } else if (!is.list(user_res)) {
            stop("'transformed_parameters' must return a list.", call. = FALSE)
          }

          dup_names <- intersect(names(res), names(user_res))
          if (length(dup_names) > 0) {
            stop(
              sprintf(
                "Transformed parameter names are duplicated: %s",
                paste(dup_names, collapse = ", ")
              ),
              call. = FALSE
            )
          }

          res <- c(res, user_res)
        }

        return(res)
      }

      test_para <- constrained_vector_to_list(all_draws[1, 1, -1], self$model$par_list)
      test_tran <- wrapper_tran_fn(self$model$data, test_para)

      if (length(test_tran) == 0) return(invisible(self))

      tran_names <- character(0)
      self$transform_dims <- list()

      for (name in names(test_tran)) {
        val <- test_tran[[name]]
        len <- length(val)
        dim_val <- dim(val)
        if (is.null(dim_val)) dim_val <- len
        self$transform_dims[[name]] <- dim_val

        names_def <- self$model$par_names[[name]]
        flat_nms <- generate_flat_names(name, dim_val, names_def)
        tran_names <- c(tran_names, flat_nms)
      }

      tran_array <- array(NA, dim = c(iter, chains, length(tran_names)))
      dimnames(tran_array) <- list(
        iteration = NULL,
        chain = paste0("chain", seq_len(chains)),
        variable = tran_names
      )

      cat("Calculating transformed parameters...\n")

      total_steps <- iter * chains
      pb <- txtProgressBar(min = 0, max = total_steps, style = 3)
      counter <- 0

      update_interval <- max(1, floor(total_steps / 100))

      for (c in seq_len(chains)) {
        for (i in seq_len(iter)) {
          p_list <- constrained_vector_to_list(all_draws[i, c, -1], self$model$par_list)
          res <- wrapper_tran_fn(self$model$data, p_list)
          tran_array[i, c, ] <- unlist(res, use.names = FALSE)

          counter <- counter + 1
          if (counter %% update_interval == 0) {
            setTxtProgressBar(pb, counter)
          }
        }
      }
      setTxtProgressBar(pb, total_steps)
      close(pb)

      self$transform_fit <- tran_array
      return(invisible(self))
    },

    #' @description Compute generated quantities from posterior draws.
    #' @param code An `rtmb_code(\{ ... \})` or `\{ ... \}` block containing the logic
    #' to be calculated using posterior samples.
    #' @return The `MCMC_Fit` object itself (invisibly).
    #' Results are appended to the `generate_fit` field.
    generated_quantities = function(code) {
      raw_code <- substitute(code)

      if (is.name(raw_code)) {
        evaluated <- tryCatch(eval(raw_code, envir = parent.frame()), error = function(e) NULL)
        if (is.language(evaluated) || is.call(evaluated)) {
          code <- evaluated
          raw_code <- evaluated
        }
      }

      if (is.call(raw_code) && identical(raw_code[[1]], as.name("rtmb_code"))) {
        parsed_code <- eval(raw_code, envir = parent.frame())
        if (!"generate" %in% names(parsed_code)) {
          stop("There is no 'generate' block in rtmb_code().")
        }
        gen_ast <- parsed_code$generate
      } else if (is.call(raw_code) && identical(raw_code[[1]], as.name("{"))) {
        gen_ast <- raw_code
      } else if (is.call(code) && identical(code[[1]], as.name("{"))) {
        gen_ast <- code
      } else if (is.list(code) && "generate" %in% names(code)) {
        gen_ast <- code$generate
      } else {
        stop("'code' must be specified in the format rtmb_code(generate = { ... }) or { ... }.")
      }

      cat("Running generated_quantities...\n")

      gen_fn <- eval(bquote(transform_code(.(gen_ast))))
      environment(gen_fn) <- parent.env(globalenv())

      all_draws <- self$draws(
        inc_random = TRUE,
        inc_transform = TRUE,
        inc_generate = FALSE
      )

      iter   <- dim(all_draws)[1]
      chains <- dim(all_draws)[2]

      test_p_list <- constrained_vector_to_list(all_draws[1, 1, -1], self$model$par_list)
      if (!is.null(self$model$transform)) {
        test_tran <- self$model$transform(self$model$data, test_p_list)
        if (is.list(test_tran)) test_p_list <- c(test_p_list, test_tran)
      }

      if (!is.null(self$generate_fit)) {
        existing_gq <- dimnames(self$generate_fit)[[3]]
        for (g_name in names(self$generate_dims)) {
          g_dim <- self$generate_dims[[g_name]]
          flat_nms <- generate_flat_names(g_name, g_dim, self$model$par_names[[g_name]])
          if (all(flat_nms %in% existing_gq)) {
            val <- self$generate_fit[1, 1, flat_nms]
            if (length(g_dim) > 1) dim(val) <- g_dim
            test_p_list[[g_name]] <- val
          }
        }
      }

      test_gq <- gen_fn(self$model$data, test_p_list)

      if (is.null(test_gq) || length(test_gq) == 0) {
        cat("No generated quantities returned.\n")
        return(invisible(self))
      }

      gq_names <- character(0)
      if (is.null(self$generate_dims)) self$generate_dims <- list()

      for (name in names(test_gq)) {
        val <- test_gq[[name]]
        dim_val <- dim(val)
        if (is.null(dim_val)) dim_val <- length(val)
        self$generate_dims[[name]] <- dim_val

        names_def <- self$model$par_names[[name]]
        flat_nms <- generate_flat_names(name, dim_val, names_def)
        gq_names <- c(gq_names, flat_nms)
      }

      # (Omitted)
      new_gq_array <- array(NA, dim = c(iter, chains, length(gq_names)))
      dimnames(new_gq_array) <- list(
        iteration = NULL,
        chain = paste0("chain", seq_len(chains)),
        variable = gq_names
      )

      total_steps <- iter * chains
      pb <- txtProgressBar(min = 0, max = total_steps, style = 3)
      counter <- 0

      update_interval <- max(1, floor(total_steps / 100))

      for (c in seq_len(chains)) {
        for (i in seq_len(iter)) {
          p_list <- constrained_vector_to_list(all_draws[i, c, -1], self$model$par_list)
          if (!is.null(self$model$transform)) {
            tran_res <- self$model$transform(self$model$data, p_list)
            if (is.list(tran_res)) p_list <- c(p_list, tran_res)
          }
          if (!is.null(self$generate_fit)) {
            existing_gq <- dimnames(self$generate_fit)[[3]]
            for (g_name in names(self$generate_dims)) {
              g_dim <- self$generate_dims[[g_name]]
              flat_nms <- generate_flat_names(g_name, g_dim, self$model$par_names[[g_name]])
              if (all(flat_nms %in% existing_gq)) {
                val <- self$generate_fit[i, c, flat_nms]
                if (length(g_dim) > 1) dim(val) <- g_dim
                p_list[[g_name]] <- val
              }
            }
          }
          res <- gen_fn(self$model$data, p_list)
          new_gq_array[i, c, ] <- unlist(res, use.names = FALSE)

          counter <- counter + 1

          if (counter %% update_interval == 0) {
            setTxtProgressBar(pb, counter)
          }
        }
      }
      setTxtProgressBar(pb, total_steps)
      close(pb)

      if (is.null(self$generate_fit)) {
        self$generate_fit <- new_gq_array
      } else {
        old_gq <- self$generate_fit
        I <- dim(old_gq)[1]; C <- dim(old_gq)[2]
        P1 <- dim(old_gq)[3]; P2 <- dim(new_gq_array)[3]
        merged_gq <- array(NA, dim = c(I, C, P1 + P2))
        merged_gq[,,1:P1] <- old_gq
        merged_gq[,,(P1+1):(P1+P2)] <- new_gq_array
        dimnames(merged_gq) <- list(dimnames(old_gq)[[1]], dimnames(old_gq)[[2]], c(dimnames(old_gq)[[3]], dimnames(new_gq_array)[[3]]))
        self$generate_fit <- merged_gq
      }

      cat("Generated quantities added to samples.\n")
      invisible(self)
    },

    #' @description Resolve label switching in posterior draws.
    #' @param target Character string specifying the target variable to base the relabeling on.
    #' @param linked Character vector of variable names to be relabeled in the same order as the target. Default is NULL.
    #' @param overwrite Logical; whether to overwrite the stored draws in the current object. Default is TRUE.
    #' @param scalar_fns A named list of functions to apply to scalar variables for relabeling. Default is an empty list.
    #' @return Relabeled draws or updated object.
    resolve_switching = function(target, linked = NULL, overwrite = TRUE, scalar_fns = list()) {
      cat(sprintf("Resolving label switching based on '%s'...\n", target))

      obj <- if (isTRUE(overwrite)) self else self$clone(deep = TRUE)

      f_arr <- obj$fit
      r_arr <- obj$random_fit
      t_arr <- obj$transform_fit
      g_arr <- obj$generate_fit

      v_names_f <- dimnames(f_arr)[[3]]
      v_names_r <- if (!is.null(r_arr)) dimnames(r_arr)[[3]] else character(0)
      v_names_t <- if (!is.null(t_arr)) dimnames(t_arr)[[3]] else character(0)
      v_names_g <- if (!is.null(g_arr)) dimnames(g_arr)[[3]] else character(0)

      get_var_info <- function(vname) {
        pattern <- paste0("^", vname, "\\[")

        idx_f <- grep(pattern, v_names_f)
        if (length(idx_f) > 0) return(list(loc = "fixed", idx = idx_f, dim = obj$model$par_list[[vname]]$dim))
        idx_r <- grep(pattern, v_names_r)
        if (length(idx_r) > 0) return(list(loc = "random", idx = idx_r, dim = obj$model$par_list[[vname]]$dim))
        idx_t <- grep(pattern, v_names_t)
        if (length(idx_t) > 0) return(list(loc = "tran", idx = idx_t, dim = obj$transform_dims[[vname]]))
        idx_g <- grep(pattern, v_names_g)
        if (length(idx_g) > 0) return(list(loc = "gq", idx = idx_g, dim = obj$generate_dims[[vname]]))

        idx_f0 <- which(v_names_f == vname)
        if (length(idx_f0) == 1) return(list(loc = "fixed_scalar", idx = idx_f0, dim = 1))
        idx_r0 <- which(v_names_r == vname)
        if (length(idx_r0) == 1) return(list(loc = "random_scalar", idx = idx_r0, dim = 1))
        idx_t0 <- which(v_names_t == vname)
        if (length(idx_t0) == 1) return(list(loc = "tran_scalar", idx = idx_t0, dim = 1))
        idx_g0 <- which(v_names_g == vname)
        if (length(idx_g0) == 1) return(list(loc = "gq_scalar", idx = idx_g0, dim = 1))

        stop(paste0("Label switching failed: Variable '", vname, "' not found."))
      }

      t_info <- get_var_info(target)

      if (grepl("scalar", t_info$loc)) stop(paste0("Target variable '", target, "' must be a vector, not a scalar."))
      if (length(t_info$dim) != 1) stop(paste0("Target variable '", target, "' must be a vector."))

      K <- t_info$dim[1]
      if (length(t_info$idx) != K) stop(paste0("Target variable '", target, "' has inconsistent dimension information."))

      linked_info_list <- list()
      if (!is.null(linked)) {
        for (lvar in linked) {
          l_info <- get_var_info(lvar)

          if (grepl("scalar", l_info$loc)) {
            linked_info_list[[lvar]] <- l_info
          } else {
            if (length(l_info$dim) != 1) {
              warning(sprintf("Variable '%s' is not a vector. Skipping.", lvar))
            } else if (l_info$dim[1] != K) {
              warning(sprintf("Length of '%s' (%d) != target '%s' (%d). Skipping.", lvar, l_info$dim[1], target, K))
            } else {
              linked_info_list[[lvar]] <- l_info
            }
          }
        }
      }

      iter_total   <- dim(f_arr)[1]
      chains_total <- dim(f_arr)[2]

      get_values <- function(arr, i, c, idx) { arr[i, c, idx] }
      set_values <- function(arr, i, c, idx, value) {
        arr[i, c, idx] <- value
        arr
      }

      for (c in seq_len(chains_total)) {
        for (i in seq_len(iter_total)) {

          if (t_info$loc == "fixed") t_vals <- get_values(f_arr, i, c, t_info$idx)
          else if (t_info$loc == "random") t_vals <- get_values(r_arr, i, c, t_info$idx)
          else if (t_info$loc == "tran") t_vals <- get_values(t_arr, i, c, t_info$idx)
          else if (t_info$loc == "gq") t_vals <- get_values(g_arr, i, c, t_info$idx)
          else stop("Target must be a vector variable.")

          ord <- order(t_vals)

          if (any(ord != seq_len(K))) {
            if (t_info$loc == "fixed") f_arr <- set_values(f_arr, i, c, t_info$idx, t_vals[ord])
            else if (t_info$loc == "random") r_arr <- set_values(r_arr, i, c, t_info$idx, t_vals[ord])
            else if (t_info$loc == "tran") t_arr <- set_values(t_arr, i, c, t_info$idx, t_vals[ord])
            else if (t_info$loc == "gq") g_arr <- set_values(g_arr, i, c, t_info$idx, t_vals[ord])

            for (lvar in names(linked_info_list)) {
              l_info <- linked_info_list[[lvar]]

              if (l_info$loc == "fixed") f_arr <- set_values(f_arr, i, c, l_info$idx, f_arr[i, c, l_info$idx][ord])
              else if (l_info$loc == "random") r_arr <- set_values(r_arr, i, c, l_info$idx, r_arr[i, c, l_info$idx][ord])
              else if (l_info$loc == "tran") t_arr <- set_values(t_arr, i, c, l_info$idx, t_arr[i, c, l_info$idx][ord])
              else if (l_info$loc == "gq") g_arr <- set_values(g_arr, i, c, l_info$idx, g_arr[i, c, l_info$idx][ord])
              else if (K == 2) {
                s_fn <- function(x) 1 - x
                if (!is.null(scalar_fns) && lvar %in% names(scalar_fns)) {
                  s_fn <- scalar_fns[[lvar]]
                }

                if (l_info$loc == "fixed_scalar") f_arr[i, c, l_info$idx] <- s_fn(f_arr[i, c, l_info$idx])
                else if (l_info$loc == "random_scalar") r_arr[i, c, l_info$idx] <- s_fn(r_arr[i, c, l_info$idx])
                else if (l_info$loc == "tran_scalar") t_arr[i, c, l_info$idx] <- s_fn(t_arr[i, c, l_info$idx])
                else if (l_info$loc == "gq_scalar") g_arr[i, c, l_info$idx] <- s_fn(g_arr[i, c, l_info$idx])
              }
            }
          }
        }
      }

      obj$fit <- f_arr
      obj$random_fit <- r_arr
      obj$transform_fit <- t_arr
      obj$generate_fit <- g_arr

      fixed_mean_new <- apply(obj$fit[, , -1, drop = FALSE], 3, mean)
      new_posterior_mean <- obj$posterior_mean
      new_posterior_mean[names(fixed_mean_new)] <- fixed_mean_new

      if (!is.null(obj$random_fit)) {
        random_mean_new <- apply(obj$random_fit, 3, mean)
        new_posterior_mean[names(random_mean_new)] <- random_mean_new
      }

      obj$posterior_mean <- new_posterior_mean

      if (isTRUE(overwrite)) invisible(self) else obj
    }

  )
)

#' Fit a Latent Rank Theory (LRT) Model
#'
#' @description
#' Fits a Latent Rank Theory model, which is a mixture model with ordered ranks
#' and Gaussian Process smoothing on the mean profiles.
#'
#' @param formula A formula specifying the response variable(s).
#' @param k Number of ranks (mixture components).
#' @param rank_coords Optional numeric vector of coordinates for each rank. Default is 1:k.
#' @param covariance Covariance structure: "diagonal", "diagonal_equal", "full", "full_equal", or "full_equal_corr".
#' @param data A data frame containing the variables.
#' @param magnitude Signal standard deviation for the GP prior. If NULL, it is estimated.
#' @param smoothing Length-scale for the GP prior. If NULL, it is estimated.
#' @param noise Measurement noise for the GP prior (default is 0.01).
#' @param prob_smoothing Logical; whether to apply smoothing to the class membership probabilities.
#' @param link Link function for class probabilities: "ordered" or "sequential".
#' @param prior Prior configuration: `prior_flat()`, `prior_normal()`,
#'   `prior_weak()`, `prior_rhs()`, or `prior_ssp()`. Default is
#'   `prior_flat()`. If `y_range` is supplied with the default flat prior,
#'   the wrapper automatically switches to `prior_weak()`.
#' @param y_range Optional numeric vector or matrix defining the theoretical range (min, max) of response variables.
#'   Specifying this automatically enables weakly informative priors if `prior` is `prior_flat()`.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param two_stage Logical; if TRUE, estimate the latent-rank measurement model
#'   first and then estimate the rank regression with delta-method uncertainty
#'   propagation. Currently supported for `$optimize()` only.
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments passed to `rtmb_model`.
#' @return A \code{RTMB_Model} object.
#' @export
rtmb_lrt <- function(formula, k = 3, data = NULL,
                     rank_coords = NULL,
                     covariance = c("diagonal", "diagonal_equal", "full", "full_equal", "full_equal_corr"),
                     magnitude = NULL, smoothing = NULL, noise = 0.01,
                     prob_smoothing = FALSE,
                     link = c("ordered", "sequential"),
                     prior = prior_flat(), y_range = NULL, fixed = NULL,
                     two_stage = FALSE, WAIC = FALSE, ...) {

  dots <- list(...)
  if ("2stage" %in% names(dots)) {
    two_stage <- isTRUE(dots[["2stage"]])
  }
  wrapper_optimize_defaults <- list()
  if (!is.null(dots$num_estimate)) {
    wrapper_optimize_defaults$num_estimate <- dots$num_estimate
  }

  if (is.null(rank_coords)) rank_coords <- 1:k

  if (is.null(prior)) {
    prior <- prior_flat()
  }

  # Automatically switch to prior_weak() if y_range is provided and prior is default flat
  if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }

  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), prior_weak(), prior_rhs(), or prior_ssp().",
      call. = FALSE
    )
  }

  # NSE for formula: handle case where formula is just a variable name in data
  formula_expr <- substitute(formula)
  formula_val <- try(formula, silent = TRUE)

  if (inherits(formula_val, "try-error") ||
      (!inherits(formula_val, "formula") && !is.matrix(formula_val) && !is.vector(formula_val))) {
    if (!is.null(data) && all(all.vars(formula_expr) %in% names(data))) {
      formula <- as.formula(call("~", formula_expr, 1))
    } else if (!inherits(formula_val, "try-error")) {
      formula <- formula_val
    } else {
      stop(paste0("Object '", deparse(formula_expr), "' not found."))
    }
  } else {
    formula <- formula_val
  }

  covariance <- match.arg(covariance)
  link <- match.arg(link)
  K_mix <- k
  prior_type <- prior$type

  if (prior_type == "flat") {
    # no merge
  } else if (prior_type == "normal") {
    normal_defaults <- list(
      Intercept_sd = NULL,
      mu_sd = NULL,
      b_sd = NULL,
      sigma_rate = NULL,
      lkj_eta = NULL,
      mag_rate = NULL,
      smooth_rate = NULL,
      cutpoint_sd = NULL
    )
    prior <- .merge_prior(normal_defaults, prior)
  } else {
    default_prior <- list(Intercept_sd = 10, mu_sd = 10, b_sd = 10, sigma_rate = 1, lkj_eta = 1.0, mag_rate = 1.0, smooth_rate = 1.0, cutpoint_sd = 2.5)
    prior <- .merge_prior(default_prior, prior)
  }

  # Sync mu_sd and Intercept_sd
  if (prior_type == "normal") {
    if (is.null(prior$mu_sd) && !is.null(prior$Intercept_sd)) {
      prior$mu_sd <- prior$Intercept_sd
    }
  }

  regularization <- if (prior_type %in% c("rhs", "ssp")) prior_type else "none"
  use_weak_info <- prior_type %in% c("weak", "rhs", "ssp")

  if (use_weak_info) {
    if (is.null(prior$max_beta)) prior$max_beta <- 1.0
    if (is.null(prior$expected_vars)) prior$expected_vars <- 3
    if (is.null(prior$slab_scale)) prior$slab_scale <- 2.0
    if (is.null(prior$slab_df)) prior$slab_df <- 4.0
    if (is.null(prior$ssp_ratio)) prior$ssp_ratio <- 0.25
  }

  setup_from_formula <- inherits(formula, "formula") && !is.null(data)

  # Data parsing
  if (is.matrix(formula) || is.vector(formula)) {
    Y_mat <- as.matrix(formula)
    X_prob <- NULL
  } else if (!inherits(formula, "formula")) {
    var_name <- as.character(formula)
    if (!is.null(data) && var_name %in% names(data)) {
      Y_mat <- as.matrix(data[[var_name]])
    } else {
      Y_mat <- as.matrix(formula)
    }
    X_prob <- NULL
  } else {
    mf_y <- model.frame(formula, data = data)
    Y_mat <- model.response(mf_y)
    if (is.null(Y_mat)) Y_mat <- as.matrix(mf_y)

    # Check if formula has RHS covariates for prob
    rhs <- formula[[3]]
    is_empty_rhs <- (is.numeric(rhs) && rhs == 1) || (is.symbol(rhs) && rhs == "1")

    if (!is_empty_rhs) {
      X_prob <- model.matrix(formula, data = data)
      if ("(Intercept)" %in% colnames(X_prob)) {
        X_prob <- X_prob[, colnames(X_prob) != "(Intercept)", drop = FALSE]
      }
      if (ncol(X_prob) == 0) X_prob <- NULL
    } else {
      X_prob <- NULL
    }
  }

  if (setup_from_formula) {
    setup_vars <- all.vars(formula)
    missing_vars <- setdiff(setup_vars, names(data))
    if (length(missing_vars) > 0) {
      stop(
        "The following variables in formula are not found in data: ",
        paste(missing_vars, collapse = ", "),
        call. = FALSE
      )
    }
    setup_df <- as.data.frame(data)[, setup_vars, drop = FALSE]
    class(setup_df) <- c("rtmb_setup_df", class(setup_df))
  }

  Y_mat <- as.matrix(Y_mat)
  N_obs <- nrow(Y_mat)
  P_dim <- ncol(Y_mat)
  if (is.null(colnames(Y_mat))) colnames(Y_mat) <- paste0("Y", 1:P_dim)
  has_cov_prob <- !is.null(X_prob)
  K_prob <- if (has_cov_prob) ncol(X_prob) else 0

  if (isTRUE(two_stage) && !has_cov_prob) {
    stop("two_stage = TRUE requires covariates on the right-hand side of the formula.", call. = FALSE)
  }

  multivariate <- P_dim > 1
  is_diag <- covariance %in% c("diagonal", "diagonal_equal")
  is_sigma_equal <- covariance %in% c("diagonal_equal", "full_equal", "full_equal_corr")

  # --- 1. Setup ---
  if (setup_from_formula) {
    setup_exprs <- list(
      as.name("{"),
      quote(mf <- model.frame(formula, df)),
      quote(Y <- model.response(mf)),
      quote(if (is.null(Y)) Y <- as.matrix(mf)),
      quote(Y <- as.matrix(Y)),
      quote(N <- nrow(Y)),
      quote(P <- ncol(Y)),
      quote(K <- K),
      quote(rank_coords <- rank_coords)
    )
  } else {
    setup_exprs <- list(
      as.name("{"),
      quote(N <- N),
      quote(K <- K),
      quote(P <- P),
      quote(rank_coords <- rank_coords)
    )
  }
  if (has_cov_prob) {
    if (setup_from_formula) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_prob <- model.matrix(formula, mf))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(if ("(Intercept)" %in% colnames(X_prob)) X_prob <- X_prob[, colnames(X_prob) != "(Intercept)", drop = FALSE])
      setup_exprs[[length(setup_exprs) + 1]] <- quote(K_prob <- ncol(X_prob))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_means <- matrix(colMeans(X_prob), 1, K_prob))
    } else {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(K_prob <- K_prob)
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_means <- matrix(X_means, 1, K_prob))
    }
    if (regularization == "rhs") {
      # RHS setup (cap expected_vars to prevent zero-division)
      p0_rhs <- min(prior$expected_vars, K_prob - 1)
      if (p0_rhs < 1) p0_rhs <- 1
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) {
        bquote(tau0 <- .(p0_rhs) / (K_prob - .(p0_rhs)) / sqrt(N))
      } else {
        quote(tau0 <- tau0)
      }
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) bquote(half_slab_df <- .(prior$slab_df) / 2) else quote(half_slab_df <- half_slab_df)
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) bquote(half_slab_scale2 <- .(prior$slab_scale)^2 / 2) else quote(half_slab_scale2 <- half_slab_scale2)
    } else if (regularization == "ssp") {
      # SSP setup
      setup_exprs[[length(setup_exprs) + 1]] <- if (setup_from_formula) bquote(tau_scale <- .(prior$max_beta) / 1.96) else quote(tau_scale <- tau_scale)
    }
  }
  setup_ast <- as.call(setup_exprs)

  # --- 2. Parameters ---
  param_exprs <- list(as.name("{"))
  param_exprs[[length(param_exprs) + 1]] <- bquote(mu_p <- Dim(c(.(P_dim), .(K_mix)), type = "ordered"))

  # Sigma parameterization based on covariance
  if (is_sigma_equal) {
    param_exprs[[length(param_exprs) + 1]] <- bquote(sigma <- Dim(.(P_dim), lower = 0))
  } else {
    param_exprs[[length(param_exprs) + 1]] <- bquote(sigma <- Dim(c(.(K_mix), .(P_dim)), lower = 0))
  }

  if (multivariate && !is_diag) {
    if (covariance == "full") {
      param_exprs[[length(param_exprs) + 1]] <- bquote(L_corr <- Dim(c(.(K_mix), .(P_dim), .(P_dim)), type = "CF_corr"))
    } else {
      param_exprs[[length(param_exprs) + 1]] <- bquote(L_corr <- Dim(c(.(P_dim), .(P_dim)), type = "CF_corr"))
    }
  }

  if (has_cov_prob) {
    if (link == "ordered") {
      if (regularization == "rhs") {
        param_exprs[[length(param_exprs) + 1]] <- bquote(z_b <- Dim(.(K_prob)))
        param_exprs[[length(param_exprs) + 1]] <- bquote(w_lambda_b <- Dim(.(K_prob), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(w_tau_b <- Dim(lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(c2_b <- Dim(lower = 0))
      } else if (regularization == "ssp") {
        param_exprs[[length(param_exprs) + 1]] <- bquote(b_raw <- Dim(.(K_prob)))
        param_exprs[[length(param_exprs) + 1]] <- bquote(r_b <- Dim(.(K_prob), lower = 0, upper = 1))
        param_exprs[[length(param_exprs) + 1]] <- bquote(tau_b <- Dim(lower = 0))
      } else {
        param_exprs[[length(param_exprs) + 1]] <- bquote(b <- Dim(.(K_prob)))
      }
      param_exprs[[length(param_exprs) + 1]] <- bquote(cutpoints <- Dim(.(K_mix - 1), type = "ordered"))
    } else {
      # Sequential: Separate b for each step
      if (regularization == "rhs") {
        param_exprs[[length(param_exprs) + 1]] <- bquote(z_b <- Dim(c(.(K_prob), .(K_mix - 1))))
        param_exprs[[length(param_exprs) + 1]] <- bquote(w_lambda_b <- Dim(c(.(K_prob), .(K_mix - 1)), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(w_tau_b <- Dim(.(K_mix - 1), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(c2_b <- Dim(.(K_mix - 1), lower = 0))
      } else if (regularization == "ssp") {
        param_exprs[[length(param_exprs) + 1]] <- bquote(b_raw <- Dim(c(.(K_prob), .(K_mix - 1))))
        param_exprs[[length(param_exprs) + 1]] <- bquote(r_b <- Dim(c(.(K_prob), .(K_mix - 1)), lower = 0, upper = 1))
        param_exprs[[length(param_exprs) + 1]] <- bquote(tau_b <- Dim(.(K_mix - 1), lower = 0))
      } else {
        param_exprs[[length(param_exprs) + 1]] <- bquote(b <- Dim(c(.(K_prob), .(K_mix - 1))))
      }
      param_exprs[[length(param_exprs) + 1]] <- bquote(alpha <- Dim(.(K_mix - 1)))
    }
  } else {
    if (prob_smoothing) {
      param_exprs[[length(param_exprs) + 1]] <- bquote(logit_theta <- Dim(.(K_mix)))
      param_exprs[[length(param_exprs) + 1]] <- quote(magnitude_theta <- Dim(lower = 0))
    } else {
      param_exprs[[length(param_exprs) + 1]] <- bquote(theta <- Dim(.(K_mix), type = "simplex"))
    }
  }

  if (is.null(magnitude)) param_exprs[[length(param_exprs) + 1]] <- quote(magnitude <- Dim(lower = 0))
  if (is.null(smoothing)) param_exprs[[length(param_exprs) + 1]] <- quote(smoothing <- Dim(lower = 0))
  param_ast <- as.call(param_exprs)

  # --- 3. Model ---
  model_exprs <- list(as.name("{"))

  # Regularization transformations
  if (has_cov_prob) {
    if (regularization == "rhs") {
      model_exprs[[length(model_exprs) + 1]] <- quote(lambda_b <- 1 / sqrt(w_lambda_b))
      model_exprs[[length(model_exprs) + 1]] <- quote(tau_hs_b <- tau0 / sqrt(w_tau_b))
      if (link == "ordered") {
        model_exprs[[length(model_exprs) + 1]] <- quote(lambda_tilde_b <- sqrt(c2_b * lambda_b^2 / (c2_b + tau_hs_b^2 * lambda_b^2)))
        model_exprs[[length(model_exprs) + 1]] <- quote(b <- z_b * tau_hs_b * lambda_tilde_b)
      } else {
        model_exprs[[length(model_exprs) + 1]] <- quote(b <- matrix(0, K_prob, K-1))
        model_exprs[[length(model_exprs) + 1]] <- quote(for (k in 1:(K-1)) {
          lambda_tilde_k <- sqrt(c2_b[k] * lambda_b[, k]^2 / (c2_b[k] + tau_hs_b[k]^2 * lambda_b[, k]^2))
          b[, k] <- z_b[, k] * tau_hs_b[k] * lambda_tilde_k
        })
      }
    } else if (regularization == "ssp") {
      model_exprs[[length(model_exprs) + 1]] <- quote(b <- b_raw * r_b * tau_b)
    }
  }

  model_exprs[[length(model_exprs) + 1]] <- quote(mu <- t(mu_p))

  if (has_cov_prob) {
    model_exprs[[length(model_exprs) + 1]] <- quote(pi_mat <- matrix(mu[1] * 0, N, K))
    if (link == "ordered") {
      model_exprs[[length(model_exprs) + 1]] <- quote(eta <- X_prob %*% b)
      model_exprs[[length(model_exprs) + 1]] <- quote(for (i in 1:N) {
        F_prev <- 0
        for (k in 1:(K-1)) {
          F_k <- inv_logit(cutpoints[k] - eta[i])
          pi_mat[i, k] <- F_k - F_prev
          F_prev <- F_k
        }
        pi_mat[i, K] <- 1 - F_prev
      })
    } else {
      # Sequential with Step-specific b
      model_exprs[[length(model_exprs) + 1]] <- quote(eta_mat <- X_prob %*% b)
      model_exprs[[length(model_exprs) + 1]] <- quote(for (i in 1:N) {
        q_cum <- 1
        for (k in 1:(K-1)) {
          q_k <- inv_logit(alpha[k] + eta_mat[i, k])
          pi_mat[i, k] <- q_cum * (1 - q_k)
          q_cum <- q_cum * q_k
        }
        pi_mat[i, K] <- q_cum
      })
    }
    model_exprs[[length(model_exprs) + 1]] <- quote(log_pi_mat <- log(pi_mat + 1e-15))
  } else if (prob_smoothing) {
    model_exprs[[length(model_exprs) + 1]] <- quote(theta <- softmax(logit_theta))
  }

  model_exprs[[length(model_exprs) + 1]] <- quote(lp <- mu[1] * 0)
  model_exprs[[length(model_exprs) + 1]] <- quote(log_dens_mat <- matrix(mu[1] * 0, N, K))

  # Likelihood Calculation
  if (is_diag) {
    dist_body <- bquote({
      ld <- Y[1] * 0
      m_vec <- mu[k, ]
      s_vec <- if (.(is_sigma_equal)) sigma else sigma[k, ]
      for (p in 1:P) {
        ld <- ld + normal_lpdf(Y[, p], m_vec[p], s_vec[p], sum = FALSE)
      }
      log_dens_mat[, k] <- ld
    })
  } else {
    # Full covariance
    dist_body <- bquote({
      m_vec <- mu[k, ]
      s_vec <- if (.(is_sigma_equal)) sigma else sigma[k, ]
      L_mat <- if (.(covariance == "full")) matrix(L_corr[k, , ], P, P) else matrix(L_corr, P, P)
      log_dens_mat[, k] <- multi_normal_CF_lpdf(Y, mean = as.vector(m_vec), sd = as.vector(s_vec), CF_Omega = L_mat, sum = FALSE)
    })
  }
  model_exprs[[length(model_exprs) + 1]] <- as.call(list(as.name("for"), as.name("k"), quote(1:K), dist_body))

  if (has_cov_prob) {
    model_exprs[[length(model_exprs) + 1]] <- quote(lp <- lp + sum(log_sum_exp(log_pi_mat + log_dens_mat)))
  } else {
    model_exprs[[length(model_exprs) + 1]] <- quote(log_theta <- log(theta))
    model_exprs[[length(model_exprs) + 1]] <- quote(lp <- lp + sum(log_sum_exp(t(t(log_dens_mat) + log_theta))))
  }

  # GP Prior (Items)
  gp_mag <- if (is.null(magnitude)) quote(magnitude) else magnitude
  gp_sm  <- if (is.null(smoothing)) quote(smoothing) else smoothing
  model_exprs[[length(model_exprs) + 1]] <- bquote(for (p in 1:P) {
    lp <- lp + gaussian_process_lpdf(mu[, p], x = rank_coords, magnitude = .(gp_mag), smoothing = .(gp_sm), noise = .(noise))
  })

  if (prob_smoothing && !has_cov_prob) {
    model_exprs[[length(model_exprs) + 1]] <- bquote(lp <- lp + gaussian_process_lpdf(logit_theta, x = rank_coords, magnitude = magnitude_theta, smoothing = .(gp_sm), noise = .(noise)))
  }

  # Priors
  if (use_weak_info) {
    model_exprs[[length(model_exprs) + 1]] <- bquote(sigma ~ exponential(.(prior$sigma_rate)))
    if (is.null(magnitude)) model_exprs[[length(model_exprs) + 1]] <- bquote(magnitude ~ exponential(.(prior$mag_rate)))
    if (is.null(smoothing)) model_exprs[[length(model_exprs) + 1]] <- bquote(smoothing ~ exponential(.(prior$smooth_rate)))
    if (prob_smoothing && !has_cov_prob) {
      model_exprs[[length(model_exprs) + 1]] <- bquote(magnitude_theta ~ exponential(.(prior$mag_rate)))
    }

    if (multivariate && !is_diag) {
      if (covariance == "full") {
        model_exprs[[length(model_exprs) + 1]] <- bquote(for (k in 1:K) matrix(L_corr[k, , ], P, P) ~ lkj_CF_corr(.(prior$lkj_eta)))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(L_corr ~ lkj_CF_corr(.(prior$lkj_eta)))
      }
    }

    if (has_cov_prob) {
      if (regularization == "rhs") {
        model_exprs[[length(model_exprs) + 1]] <- quote(z_b ~ normal(0, 1))
        model_exprs[[length(model_exprs) + 1]] <- quote(w_lambda_b ~ gamma(0.5, 0.5))
        model_exprs[[length(model_exprs) + 1]] <- quote(w_tau_b ~ gamma(0.5, 0.5))
        model_exprs[[length(model_exprs) + 1]] <- quote(c2_b ~ inverse_gamma(half_slab_df, half_slab_scale2))
      } else if (regularization == "ssp") {
        model_exprs[[length(model_exprs) + 1]] <- quote(b_raw ~ laplace(0, 0.5))
        model_exprs[[length(model_exprs) + 1]] <- bquote(mu_r <- log(.(prior$ssp_ratio) / (1 - .(prior$ssp_ratio))))
        model_exprs[[length(model_exprs) + 1]] <- quote(r_b ~ logit_normal(mu_r, 3))
        model_exprs[[length(model_exprs) + 1]] <- quote(tau_b ~ exponential(1 / tau_scale))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(b ~ normal(0, .(prior$b_sd)))
      }

      if (link == "ordered") {
        model_exprs[[length(model_exprs) + 1]] <- bquote(cutpoints ~ normal(0, .(prior$cutpoint_sd)))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(alpha ~ normal(0, .(prior$cutpoint_sd)))
      }
    }
    if (!has_cov_prob && !prob_smoothing) {
      model_exprs[[length(model_exprs) + 1]] <- bquote(theta ~ dirichlet(rep(1, K)))
    }
  } else if (prior_type == "normal") {
    if (!is.null(prior$sigma_rate)) model_exprs[[length(model_exprs) + 1]] <- bquote(sigma ~ exponential(.(prior$sigma_rate)))
    if (has_cov_prob && !is.null(prior$b_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(b ~ normal(0, .(prior$b_sd)))
    if (is.null(magnitude) && !is.null(prior$mag_rate)) model_exprs[[length(model_exprs) + 1]] <- bquote(magnitude ~ exponential(.(prior$mag_rate)))
    if (is.null(smoothing) && !is.null(prior$smooth_rate)) model_exprs[[length(model_exprs) + 1]] <- bquote(smoothing ~ exponential(.(prior$smooth_rate)))
    if (has_cov_prob) {
       if (link == "ordered") {
         if (!is.null(prior$cutpoint_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(cutpoints ~ normal(0, .(prior$cutpoint_sd)))
       } else {
         if (!is.null(prior$cutpoint_sd)) model_exprs[[length(model_exprs) + 1]] <- bquote(alpha ~ normal(0, .(prior$cutpoint_sd)))
       }
    }

    if (!is.null(prior$lkj_eta) && multivariate && !is_diag) {
      if (covariance == "full") {
        model_exprs[[length(model_exprs) + 1]] <- bquote(for (k in 1:K) matrix(L_corr[k, , ], P, P) ~ lkj_CF_corr(.(prior$lkj_eta)))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(L_corr ~ lkj_CF_corr(.(prior$lkj_eta)))
      }
    }
  }
  model_ast <- as.call(model_exprs)

  # --- 4. Generate ---
  generate_exprs <- list(as.name("{"))
  generate_exprs[[length(generate_exprs) + 1]] <- quote(mu <- t(mu_p))
  if (has_cov_prob) {
    if (regularization == "rhs") {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(lambda_b <- 1 / sqrt(w_lambda_b))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(tau_hs_b <- tau0 / sqrt(w_tau_b))
      if (link == "ordered") {
        generate_exprs[[length(generate_exprs) + 1]] <- quote(lambda_tilde_b <- sqrt(c2_b * lambda_b^2 / (c2_b + tau_hs_b^2 * lambda_b^2)))
        generate_exprs[[length(generate_exprs) + 1]] <- quote(b <- z_b * tau_hs_b * lambda_tilde_b)
      } else {
        generate_exprs[[length(generate_exprs) + 1]] <- quote(b <- matrix(0, K_prob, K-1))
        generate_exprs[[length(generate_exprs) + 1]] <- quote(for (k in 1:(K-1)) {
          lambda_tilde_k <- sqrt(c2_b[k] * lambda_b[, k]^2 / (c2_b[k] + tau_hs_b[k]^2 * lambda_b[, k]^2))
          b[, k] <- z_b[, k] * tau_hs_b[k] * lambda_tilde_k
        })
      }
    } else if (regularization == "ssp") {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(b <- b_raw * r_b * tau_b)
    }
    if (link == "ordered") {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(eta_mean <- X_means %*% b)
      generate_exprs[[length(generate_exprs) + 1]] <- quote(F_prev <- 0)
      generate_exprs[[length(generate_exprs) + 1]] <- quote(prob_mean <- numeric(K))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(for (k in 1:(K - 1)) {
        F_k <- inv_logit(cutpoints[k] - eta_mean)
        prob_mean[k] <- F_k - F_prev
        F_prev <- F_k
      })
      generate_exprs[[length(generate_exprs) + 1]] <- quote(prob_mean[K] <- 1 - F_prev)
    } else {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(eta_mean_mat <- X_means %*% b)
      generate_exprs[[length(generate_exprs) + 1]] <- quote(q_cum <- 1)
      generate_exprs[[length(generate_exprs) + 1]] <- quote(prob_mean <- numeric(K))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(for (k in 1:(K - 1)) {
        q_k <- inv_logit(alpha[k] + eta_mean_mat[k])
        prob_mean[k] <- q_cum * (1 - q_k)
        q_cum <- q_cum * q_k
      })
      generate_exprs[[length(generate_exprs) + 1]] <- quote(prob_mean[K] <- q_cum)
    }
  } else {
    if (prob_smoothing) generate_exprs[[length(generate_exprs) + 1]] <- quote(theta <- softmax(logit_theta))
    generate_exprs[[length(generate_exprs) + 1]] <- quote(prob_mean <- theta)
  }

  if (!is_diag) {
    generate_exprs[[length(generate_exprs) + 1]] <- bquote(corr <- if (.(covariance == "full")) array(mu[1] * 0, dim = c(K, P, P)) else matrix(mu[1] * 0, P, P))
    if (covariance == "full") {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(for (k in 1:K) {
        L_mat <- matrix(L_corr[k, , ], P, P)
        corr[k, , ] <- L_mat %*% t(L_mat)
      })
    } else {
      generate_exprs[[length(generate_exprs) + 1]] <- quote(L_mat <- matrix(L_corr, P, P))
      generate_exprs[[length(generate_exprs) + 1]] <- quote(corr <- L_mat %*% t(L_mat))
    }
  }
  generate_base_exprs <- generate_exprs
  if (isTRUE(WAIC)) {
    waic_exprs <- list(as.name("{"))
    waic_exprs[[length(waic_exprs) + 1]] <- quote(log_dens_mat <- matrix(mu[1] * 0, N, K))
    waic_exprs[[length(waic_exprs) + 1]] <- as.call(list(as.name("for"), as.name("k"), quote(1:K), dist_body))
    if (has_cov_prob) {
      waic_exprs[[length(waic_exprs) + 1]] <- quote(pi_mat <- matrix(mu[1] * 0, N, K))
      if (link == "ordered") {
        waic_exprs[[length(waic_exprs) + 1]] <- quote(eta <- X_prob %*% b)
        waic_exprs[[length(waic_exprs) + 1]] <- quote(for (i in 1:N) {
          F_prev <- 0
          for (k in 1:(K - 1)) {
            F_k <- inv_logit(cutpoints[k] - eta[i])
            pi_mat[i, k] <- F_k - F_prev
            F_prev <- F_k
          }
          pi_mat[i, K] <- 1 - F_prev
        })
      } else {
        waic_exprs[[length(waic_exprs) + 1]] <- quote(eta_mat <- X_prob %*% b)
        waic_exprs[[length(waic_exprs) + 1]] <- quote(for (i in 1:N) {
          q_cum <- 1
          for (k in 1:(K - 1)) {
            q_k <- inv_logit(alpha[k] + eta_mat[i, k])
            pi_mat[i, k] <- q_cum * (1 - q_k)
            q_cum <- q_cum * q_k
          }
          pi_mat[i, K] <- q_cum
        })
      }
      waic_exprs[[length(waic_exprs) + 1]] <- quote(log_lik <- log_sum_exp(log(pi_mat + 1e-15) + log_dens_mat))
    } else {
      if (prob_smoothing) {
        waic_exprs[[length(waic_exprs) + 1]] <- quote(theta <- softmax(logit_theta))
      }
      waic_exprs[[length(waic_exprs) + 1]] <- quote(log_lik <- log_sum_exp(t(t(log_dens_mat) + log(theta))))
    }
  }
  generate_ast <- if (isTRUE(WAIC)) {
    .rtmb_waic_generate_ast(as.call(generate_base_exprs), as.call(waic_exprs))
  } else {
    as.call(generate_base_exprs)
  }

  mdl_code <- list(setup = setup_ast, parameters = param_ast, model = model_ast, generate = generate_ast, env = parent.frame())
  class(mdl_code) <- "rtmb_code"

  v_names <- list(mu = list(paste0("Rank", 1:K_mix), colnames(Y_mat)))
  v_names$sigma <- if (is_sigma_equal) colnames(Y_mat) else list(paste0("Rank", 1:K_mix), colnames(Y_mat))

  if (!is_diag) {
    if (covariance == "full") {
      v_names$L_corr <- list(paste0("Rank", 1:K_mix), colnames(Y_mat), colnames(Y_mat))
      v_names$corr <- list(paste0("Rank", 1:K_mix), colnames(Y_mat), colnames(Y_mat))
    } else {
      v_names$L_corr <- list(colnames(Y_mat), colnames(Y_mat))
      v_names$corr <- list(colnames(Y_mat), colnames(Y_mat))
    }
  }

  if (has_cov_prob) {
    if (link == "ordered") {
      v_names$b <- colnames(X_prob)
      v_names$cutpoints <- paste0("C", 1:(K_mix - 1))
    } else {
      step_labels <- paste0(1:(K_mix - 1), "->", 2:K_mix)
      v_names$b <- list(colnames(X_prob), step_labels)
      v_names$alpha <- step_labels
    }
  }

  data_list <- if (setup_from_formula) {
    list(df = setup_df, formula = formula, K = K_mix, rank_coords = rank_coords)
  } else {
    list(Y = Y_mat, N = N_obs, K = K_mix, P = P_dim, rank_coords = rank_coords)
  }
  if (has_cov_prob) {
    if (!setup_from_formula) {
      data_list$X_prob <- X_prob
      data_list$K_prob <- K_prob
      data_list$X_means <- as.vector(colMeans(X_prob))
    }
    if (regularization == "rhs") {
      p0 <- min(prior$expected_vars, K_prob - 1)
      if (p0 < 1) p0 <- 1
      if (!setup_from_formula) {
        data_list$tau0 <- p0 / (K_prob - p0) / sqrt(N_obs)
        data_list$half_slab_df <- prior$slab_df / 2
        data_list$half_slab_scale2 <- prior$slab_scale^2 / 2
      }
    } else if (regularization == "ssp") {
      if (!setup_from_formula) data_list$tau_scale <- prior$max_beta / 1.96
    }
  }

  init_list <- list()
  if (nrow(Y_mat) >= K_mix) {
    # PCA-based ordering for LRT initial values
    # We use PC1 to capture the dominant direction of variation
    pc1 <- try(prcomp(Y_mat, rank. = 1)$x[, 1], silent = TRUE)
    if (!inherits(pc1, "try-error")) {
      # Split by quantiles of PC1
      groups <- cut(pc1, breaks = quantile(pc1, probs = seq(0, 1, length.out = K_mix + 1)), include.lowest = TRUE, labels = FALSE)
      mu_init_p <- matrix(0, P_dim, K_mix)
      for (k in 1:K_mix) {
        idx <- which(groups == k)
        if (length(idx) > 0) {
          mu_init_p[, k] <- colMeans(Y_mat[idx, , drop = FALSE])
        } else {
          mu_init_p[, k] <- colMeans(Y_mat)
        }
      }
      # Initial mu_p: handle potential NAs from colMeans
      mu_init_p[is.na(mu_init_p)] <- 0
      # Ensure it's ordered along the average direction
      if (mean(mu_init_p[, 1]) > mean(mu_init_p[, K_mix])) {
        mu_init_p <- mu_init_p[, K_mix:1]
      }
      # Add small jitter to ensure strict ordering for AD setup
      # AND handle any individual items that might be decreasing
      for (k in 2:K_mix) {
        for (p in 1:P_dim) {
          if (mu_init_p[p, k] <= mu_init_p[p, k-1]) {
            mu_init_p[p, k] <- mu_init_p[p, k-1] + 1e-3
          }
        }
      }
      init_list$mu_p <- mu_init_p

      # Initial b and cutpoints if covariates present
      if (has_cov_prob) {
        # Proxy regression of ranks on X
        fit_b <- try(lm(groups ~ X_prob), silent = TRUE)
        if (!inherits(fit_b, "try-error")) {
          b_est <- coef(fit_b)
          # Exclude intercept if it was included (lm usually does)
          if ("(Intercept)" %in% names(b_est)) b_est <- b_est[names(b_est) != "(Intercept)"]
          b_est[is.na(b_est)] <- 0

          if (regularization == "rhs") {
            init_list$z_b <- if (link == "ordered") as.vector(b_est) else matrix(b_est, ncol(X_prob), K_mix - 1)
            init_list$w_lambda_b <- if (link == "ordered") rep(1, length(init_list$z_b)) else matrix(1, ncol(X_prob), K_mix - 1)
            init_list$w_tau_b <- if (link == "ordered") 1 else rep(1, K_mix - 1)
            init_list$c2_b <- if (link == "ordered") 1 else rep(1, K_mix - 1)
          } else if (regularization == "ssp") {
            init_list$b_raw <- if (link == "ordered") as.vector(b_est) else matrix(b_est, ncol(X_prob), K_mix - 1)
            init_list$r_b <- if (link == "ordered") rep(0.5, length(init_list$b_raw)) else matrix(0.5, ncol(X_prob), K_mix - 1)
            init_list$tau_b <- if (link == "ordered") 1 else rep(1, K_mix - 1)
          } else {
            if (link == "ordered") {
              init_list$b <- as.vector(b_est)
            } else {
              init_list$b <- matrix(b_est, ncol(X_prob), K_mix - 1)
            }
          }
        }

        # Thresholds from group proportions
        props <- table(factor(groups, levels = 1:K_mix)) / length(groups)
        F_k <- cumsum(props)
        jitter <- (1:(K_mix - 1)) * 1e-4
        if (link == "ordered") {
          init_list$cutpoints <- qlogis(pmin(pmax(F_k[1:(K_mix-1)], 0.01), 0.99)) + jitter
        } else {
          # Sequential alphas: q_k = P(Z>k|Z>=k)
          q_vals <- (1 - F_k[1:(K_mix-1)]) / (1 - c(0, F_k)[1:(K_mix-1)])
          init_list$alpha <- qlogis(pmin(pmax(q_vals, 0.01), 0.99)) + jitter
        }
      }
    }
  }

  if (is.null(init_list$mu_p)) {
    mu_init <- matrix(0, P_dim, K_mix)
    for (p in 1:P_dim) {
      r_y <- range(Y_mat[, p])
      mu_init[p, ] <- seq(r_y[1], r_y[2], length.out = K_mix)
    }
    init_list$mu_p <- mu_init
  }

  if (!is.null(Y_mat)) {
    s_init <- apply(Y_mat, 2, sd)
    if (is_sigma_equal) {
      init_list$sigma <- s_init
    } else {
      init_list$sigma <- matrix(s_init, K_mix, P_dim, byrow = TRUE)
    }
  }

  view_order <- c("prob_mean", "mu", "sigma")
  if (has_cov_prob) {
    view_order <- if (link == "ordered") c("b", "cutpoints", view_order) else c("b", "alpha", view_order)
  }
  if (!is_diag) view_order <- c(view_order, "corr")

  mdl <- rtmb_model(data_list, mdl_code, par_names = v_names, init = init_list, fixed = fixed, view = view_order)
  mdl$type <- "lrt"
  mdl$extra <- list(
    source = "wrapper",
    prior_type = prior$type,
    marginal = "mu_p",
    two_stage = isTRUE(two_stage),
    lrt = list(
      formula = formula,
      data = if (setup_from_formula) setup_df else NULL,
      k = K_mix,
      rank_coords = rank_coords,
      covariance = covariance,
      magnitude = magnitude,
      smoothing = smoothing,
      noise = noise,
      prob_smoothing = prob_smoothing,
      link = link,
      prior = prior,
      y_range = y_range,
      optimize_defaults = wrapper_optimize_defaults,
      multivariate = multivariate,
      has_cov_prob = has_cov_prob
    )
  )
  return(mdl)
}

.lrt_response_only_formula <- function(formula) {
  if (!inherits(formula, "formula") || length(formula) < 3L) return(formula)
  stats::as.formula(call("~", formula[[2L]], 1), env = environment(formula))
}

.lrt_log_dmvn <- function(Y, mean, sigma, corr = NULL) {
  Y <- as.matrix(Y)
  P <- ncol(Y)
  if (is.null(corr)) corr <- diag(P)
  S <- diag(as.numeric(sigma), P, P) %*% corr %*% diag(as.numeric(sigma), P, P)
  S <- S + diag(1e-8, P)
  R <- chol(S)
  z <- backsolve(R, t(sweep(Y, 2, mean, "-")), transpose = TRUE)
  quad <- colSums(z^2)
  -0.5 * (P * log(2 * pi) + 2 * sum(log(diag(R))) + quad)
}

.lrt_rank_weights_from_fit <- function(Y, fit, covariance) {
  Y <- as.matrix(Y)
  mu <- t(fit$par$mu_p)
  K <- nrow(mu)
  P <- ncol(mu)
  theta <- fit$par$theta
  if (is.null(theta)) theta <- rep(1 / K, K)
  theta <- pmax(as.numeric(theta), 1e-12)
  theta <- theta / sum(theta)

  sigma <- fit$par$sigma
  is_sigma_equal <- covariance %in% c("diagonal_equal", "full_equal", "full_equal_corr")
  is_diag <- covariance %in% c("diagonal", "diagonal_equal")

  corr_obj <- fit$generate$corr
  log_dens <- matrix(NA_real_, nrow(Y), K)
  for (kk in seq_len(K)) {
    sig_k <- if (is_sigma_equal) as.numeric(sigma) else as.numeric(sigma[kk, ])
    if (is_diag) {
      ld <- rep(0, nrow(Y))
      for (pp in seq_len(P)) {
        ld <- ld + stats::dnorm(Y[, pp], mean = mu[kk, pp], sd = sig_k[pp], log = TRUE)
      }
      log_dens[, kk] <- ld
    } else {
      corr_k <- if (covariance == "full") {
        matrix(corr_obj[kk, , ], P, P)
      } else if (!is.null(corr_obj)) {
        matrix(corr_obj, P, P)
      } else {
        diag(P)
      }
      log_dens[, kk] <- .lrt_log_dmvn(Y, mean = mu[kk, ], sigma = sig_k, corr = corr_k)
    }
  }

  log_w <- sweep(log_dens, 2, log(theta), "+")
  mx <- apply(log_w, 1, max)
  W <- exp(log_w - mx)
  W / rowSums(W)
}

.make_lrt_rank_regression_model <- function(W, X_prob, link, prior, init = NULL) {
  W <- as.matrix(W)
  X_prob <- as.matrix(X_prob)
  K <- ncol(W)
  K_prob <- ncol(X_prob)
  link <- match.arg(link, c("ordered", "sequential"))
  prior_type <- prior$type %||% "flat"

  if (prior_type %in% c("rhs", "ssp")) {
    stop("two_stage = TRUE currently does not support prior_rhs() or prior_ssp().", call. = FALSE)
  }

  setup_ast <- quote({
    N <- nrow(W)
    K <- ncol(W)
    K_prob <- ncol(X_prob)
    X_means <- matrix(colMeans(X_prob), 1, K_prob)
  })

  if (link == "ordered") {
    param_ast <- quote({
      b <- Dim(K_prob)
      cutpoints <- Dim(K - 1, type = "ordered")
    })
    model_exprs <- list(
      as.name("{"),
      quote(lp <- b[1] * 0),
      quote(for (i in 1:N) {
        eta_i <- sum(X_prob[i, ] * b)
        F_prev <- 0
        for (k in 1:(K - 1)) {
          F_k <- inv_logit(cutpoints[k] - eta_i)
          p_k <- F_k - F_prev
          lp <- lp + W[i, k] * log(p_k + 1e-15)
          F_prev <- F_k
        }
        lp <- lp + W[i, K] * log(1 - F_prev + 1e-15)
      })
    )
    generate_ast <- quote({
      eta_mean <- X_means %*% b
      F_prev <- 0
      prob_mean <- numeric(K)
      for (k in 1:(K - 1)) {
        F_k <- inv_logit(cutpoints[k] - eta_mean)
        prob_mean[k] <- F_k - F_prev
        F_prev <- F_k
      }
      prob_mean[K] <- 1 - F_prev
    })
    par_names <- list(b = colnames(X_prob), cutpoints = paste0("C", seq_len(K - 1)))
  } else {
    param_ast <- quote({
      b <- Dim(c(K_prob, K - 1))
      alpha <- Dim(K - 1)
    })
    model_exprs <- list(
      as.name("{"),
      quote(lp <- b[1] * 0),
      quote(for (i in 1:N) {
        eta_i <- X_prob[i, ] %*% b
        q_cum <- 1
        for (k in 1:(K - 1)) {
          q_k <- inv_logit(alpha[k] + eta_i[k])
          p_k <- q_cum * (1 - q_k)
          lp <- lp + W[i, k] * log(p_k + 1e-15)
          q_cum <- q_cum * q_k
        }
        lp <- lp + W[i, K] * log(q_cum + 1e-15)
      })
    )
    generate_ast <- quote({
      eta_mean_mat <- X_means %*% b
      q_cum <- 1
      prob_mean <- numeric(K)
      for (k in 1:(K - 1)) {
        q_k <- inv_logit(alpha[k] + eta_mean_mat[k])
        prob_mean[k] <- q_cum * (1 - q_k)
        q_cum <- q_cum * q_k
      }
      prob_mean[K] <- q_cum
    })
    step_labels <- paste0(seq_len(K - 1), "->", 2:K)
    par_names <- list(b = list(colnames(X_prob), step_labels), alpha = step_labels)
  }

  if (prior_type %in% c("weak", "normal")) {
    if (!is.null(prior$b_sd)) {
      model_exprs[[length(model_exprs) + 1]] <- bquote(b ~ normal(0, .(prior$b_sd)))
    }
    if (!is.null(prior$cutpoint_sd)) {
      if (link == "ordered") {
        model_exprs[[length(model_exprs) + 1]] <- bquote(cutpoints ~ normal(0, .(prior$cutpoint_sd)))
      } else {
        model_exprs[[length(model_exprs) + 1]] <- bquote(alpha ~ normal(0, .(prior$cutpoint_sd)))
      }
    }
  }

  data_list <- list(W = W, X_prob = X_prob)
  code <- list(
    setup = setup_ast,
    parameters = param_ast,
    model = as.call(model_exprs),
    generate = generate_ast,
    env = parent.frame()
  )
  class(code) <- "rtmb_code"

  if (is.null(init)) {
    init <- list(b = if (link == "ordered") rep(0, K_prob) else matrix(0, K_prob, K - 1))
    avg_w <- pmin(pmax(cumsum(colMeans(W))[seq_len(K - 1)], 0.01), 0.99)
    if (link == "ordered") {
      init$cutpoints <- stats::qlogis(avg_w) + seq_len(K - 1) * 1e-4
    } else {
      q_vals <- (1 - avg_w) / pmax(1 - c(0, avg_w)[seq_len(K - 1)], 1e-6)
      init$alpha <- stats::qlogis(pmin(pmax(q_vals, 0.01), 0.99)) + seq_len(K - 1) * 1e-4
    }
  }

  obj <- rtmb_model(data_list, code, par_names = par_names, init = init, view = c("b", "cutpoints", "alpha", "prob_mean"))
  obj$type <- "lrt_two_stage_regression"
  obj$extra <- list(source = "wrapper", prior_type = prior_type, marginal = "b")
  obj
}

.lrt_two_stage_fit_regression <- function(W, X_prob, link, prior, init = NULL,
                                          optimizer = "nlminb", method = "BFGS",
                                          control = list(), se_method = "wald",
                                          num_estimate = 1) {
  reg_model <- .make_lrt_rank_regression_model(W, X_prob, link, prior, init = init)
  reg_model$optimize(
    laplace = FALSE,
    num_estimate = num_estimate,
    optimizer = optimizer,
    method = method,
    control = control,
    se_method = se_method,
    df_method = "none"
  )
}

.lrt_two_stage_optimize <- function(self, laplace = TRUE, init = NULL, num_estimate = 1,
                                    control = list(), optimizer = "nlminb", method = "BFGS",
                                    map = NULL, fixed = NULL, se_method = c("wald", "sampling", "none"),
                                    num_samples = 1000, seed = 123, marginal = "none",
                                    df_method = "auto", view = NULL, ...) {
  se_method <- match.arg(se_method)
  if (identical(se_method, "sampling")) {
    stop("two_stage = TRUE currently supports se_method = 'wald' or 'none' only.", call. = FALSE)
  }
  if (!is.null(fixed) || !is.null(map)) {
    stop("two_stage = TRUE currently does not support fixed or map in optimize().", call. = FALSE)
  }

  cfg <- self$extra$lrt
  if (is.null(cfg) || !isTRUE(cfg$has_cov_prob)) {
    stop("two_stage estimation requires an rtmb_lrt model with covariates.", call. = FALSE)
  }
  if (identical(num_estimate, 1) && !is.null(cfg$optimize_defaults$num_estimate)) {
    num_estimate <- cfg$optimize_defaults$num_estimate
  }

  dot_args <- list(...)
  delta_eps <- dot_args$two_stage_delta_eps %||% 1e-4
  delta_method <- isTRUE(dot_args$two_stage_delta %||% TRUE)

  stage1_formula <- .lrt_response_only_formula(cfg$formula)
  if (!getOption("BayesRTMB.silent", FALSE)) {
    cat("Starting two-stage LRT optimization...\n")
    cat("Stage 1/2: estimating latent ranks without covariates...\n")
  }
  stage1_model <- rtmb_lrt(
    formula = stage1_formula,
    k = cfg$k,
    data = cfg$data,
    rank_coords = cfg$rank_coords,
    covariance = cfg$covariance,
    magnitude = cfg$magnitude,
    smoothing = cfg$smoothing,
    noise = cfg$noise,
    prob_smoothing = FALSE,
    link = cfg$link,
    prior = cfg$prior,
    y_range = cfg$y_range,
    two_stage = FALSE
  )
  stage1_fit <- stage1_model$optimize(
    laplace = laplace,
    init = init,
    num_estimate = num_estimate,
    control = control,
    optimizer = optimizer,
    method = method,
    se_method = se_method,
    num_samples = num_samples,
    seed = seed,
    marginal = marginal,
    df_method = df_method,
    view = view
  )

  Y <- self$data$Y
  X_prob <- self$data$X_prob
  W <- .lrt_rank_weights_from_fit(Y, stage1_fit, cfg$covariance)

  if (!getOption("BayesRTMB.silent", FALSE)) {
    cat("Stage 2/2: estimating rank regression with fixed soft rank weights...\n")
  }
  stage2_fit <- .lrt_two_stage_fit_regression(
    W = W,
    X_prob = X_prob,
    link = cfg$link,
    prior = cfg$prior,
    optimizer = optimizer,
    method = method,
    control = control,
    se_method = se_method,
    num_estimate = num_estimate
  )

  delta_applied <- FALSE
  delta_se_cond <- delta_se_extra <- NULL
  if (delta_method && identical(se_method, "wald") && !is.null(stage1_fit$vcov_unc)) {
    if (!getOption("BayesRTMB.silent", FALSE)) {
      cat("Applying delta-method correction for Stage 1 uncertainty...\n")
    }
    base_est <- stage2_fit$df_fixed$Estimate
    active <- stage1_fit$idx_fix_active %||% seq_along(stage1_fit$par_unc)
    V_eta <- stage1_fit$vcov_unc[active, active, drop = FALSE]
    J <- matrix(0, length(base_est), length(active))
    init_stage2 <- stage2_fit$par
    old_silent <- options(BayesRTMB.silent = TRUE)
    on.exit(options(old_silent), add = TRUE)

    for (jj in seq_along(active)) {
      u_tmp <- stage1_fit$par_unc
      step <- delta_eps * (abs(u_tmp[active[jj]]) + 1)
      u_tmp[active[jj]] <- u_tmp[active[jj]] + step
      par_tmp <- stage1_model$to_constrained(unconstrained_vector_to_list(u_tmp, stage1_model$par_list))
      gen_tmp <- if (!is.null(stage1_model$generate)) {
        tryCatch(stage1_model$generate(stage1_model$data, par_tmp), error = function(e) stage1_fit$generate)
      } else {
        stage1_fit$generate
      }
      fit_tmp <- list(par = par_tmp, generate = gen_tmp)
      W_tmp <- .lrt_rank_weights_from_fit(Y, fit_tmp, cfg$covariance)
      reg_tmp <- tryCatch(
        .lrt_two_stage_fit_regression(
          W = W_tmp,
          X_prob = X_prob,
          link = cfg$link,
          prior = cfg$prior,
          init = init_stage2,
          optimizer = optimizer,
          method = method,
          control = control,
          se_method = "none",
          num_estimate = 1
        ),
        error = function(e) NULL
      )
      if (!is.null(reg_tmp) && !is.null(reg_tmp$df_fixed)) {
        J[, jj] <- (reg_tmp$df_fixed$Estimate - base_est) / step
      }
    }

    V_extra <- J %*% V_eta %*% t(J)
    se_cond <- stage2_fit$df_fixed$`Std. Error`
    delta_se_cond <- se_cond
    delta_se_extra <- sqrt(pmax(diag(V_extra), 0))
    se_total <- sqrt(pmax(se_cond^2 + diag(V_extra), 0))
    stage2_fit$df_fixed$`Std. Error` <- se_total
    stage2_fit$df_fixed$`Lower 95%` <- base_est - stats::qnorm(0.975) * se_total
    stage2_fit$df_fixed$`Upper 95%` <- base_est + stats::qnorm(0.975) * se_total
    delta_applied <- TRUE
  }

  stage2_fit$model$type <- "lrt"
  stage2_fit$model$extra$two_stage <- TRUE
  stage2_fit$model$extra$two_stage_details <- list(
    stage1_fit = stage1_fit,
    weights = W,
    stage1_objective = stage1_fit$objective,
    stage2_objective = stage2_fit$objective,
    correction = if (delta_applied) "delta" else "none",
    se_conditional = delta_se_cond,
    se_stage1 = delta_se_extra
  )
  class(stage2_fit) <- unique(c("rtmb_lrt_two_stage", class(stage2_fit)))
  stage2_fit
}

#' @export
print.rtmb_lrt_two_stage <- function(x, ...) {
  x$print(...)
  details <- x$model$extra$two_stage_details %||% list()
  cat("\nTwo-stage LRT note:\n")
  cat("  The displayed objective is the Stage 2 rank-regression objective.\n")
  cat("  It is not directly comparable to the joint LRT objective.\n")
  if (!is.null(details$correction)) {
    cat(sprintf("  Stage 1 uncertainty correction: %s\n", details$correction))
  }
  if (!is.null(details$stage1_objective)) {
    cat(sprintf("  Stage 1 latent-rank objective: %.2f\n", details$stage1_objective))
  }
  if (!is.null(details$stage2_objective)) {
    cat(sprintf("  Stage 2 rank-regression objective: %.2f\n", details$stage2_objective))
  }
  invisible(x)
}

# Standalone implementation of rank-normalized R-hat and ESS
# Based on Vehtari et al. (2021) and the monitornew.R implementation.

#' Compute optimal FFT size
#'
#' @importFrom stats median quantile var density
#' @keywords internal
#' @noRd
fft_next_good_size <- function(N) {
  if (N <= 2) return(2)
  while (TRUE) {
    m <- N
    while ((m %% 2) == 0) m <- m / 2
    while ((m %% 3) == 0) m <- m / 3
    while ((m %% 5) == 0) m <- m / 5
    if (m <= 1) return(N)
    N <- N + 1
  }
}

#' @keywords internal
#' @noRd
autocovariance <- function(y) {
  N <- length(y)
  M <- fft_next_good_size(N)
  Mt2 <- 2 * M
  yc <- y - mean(y)
  yc <- c(yc, rep.int(0, Mt2 - N))
  transform <- stats::fft(yc)
  ac <- stats::fft(Conj(transform) * transform, inverse = TRUE)
  ac <- Re(ac)[1:N] / (N^2 * 2)
  return(ac)
}

#' @keywords internal
#' @noRd
backtransform_ranks <- function(r, c = 3/8) {
  S <- length(r)
  (r - c) / (S - 2 * c + 1)
}

#' @keywords internal
#' @noRd
z_scale <- function(x) {
  r <- rank(x, ties.method = "average")
  z <- stats::qnorm(backtransform_ranks(r, c = 3/8))
  z[is.na(x)] <- NA
  if (!is.null(dim(x))) {
    z <- array(z, dim = dim(x), dimnames = dimnames(x))
  }
  return(z)
}

#' @keywords internal
#' @noRd
split_chains <- function(sims) {
  if (is.vector(sims)) {
    dim(sims) <- c(length(sims), 1)
  }
  niter <- dim(sims)[1]
  if (niter < 2) return(sims)
  half <- niter / 2
  cbind(sims[1:floor(half), , drop = FALSE], sims[ceiling(half + 1):niter, , drop = FALSE])
}

#' @keywords internal
#' @noRd
is_constant <- function(x, tol = .Machine$double.eps) {
  if (all(is.na(x))) return(TRUE)
  abs(max(x, na.rm = TRUE) - min(x, na.rm = TRUE)) < tol
}

#' @keywords internal
#' @noRd
rhat_rfun <- function(sims) {
  if (anyNA(sims)) return(NA)
  if (any(!is.finite(sims))) return(NaN)
  if (is_constant(sims)) return(1.0)
  if (is.vector(sims)) dim(sims) <- c(length(sims), 1)
  
  chains <- ncol(sims)
  n_samples <- nrow(sims)
  if (chains < 2) return(NA)
  
  chain_mean <- colMeans(sims)
  chain_var <- apply(sims, 2, var)
  
  var_between <- n_samples * var(chain_mean)
  var_within <- mean(chain_var)
  
  if (var_within < .Machine$double.eps) return(1.0)
  sqrt((var_between / var_within + n_samples - 1) / n_samples)
}

#' @keywords internal
#' @noRd
ess_rfun <- function(sims) {
  if (is.vector(sims)) dim(sims) <- c(length(sims), 1)
  chains <- ncol(sims)
  n_samples <- nrow(sims)
  if (n_samples < 3L || anyNA(sims)) return(NA)
  if (any(!is.finite(sims))) return(NaN)
  if (is_constant(sims)) return(NA)
  
  acov <- lapply(seq_len(chains), function(i) autocovariance(sims[, i]))
  acov <- do.call(cbind, acov) * n_samples / (n_samples - 1)
  chain_mean <- colMeans(sims)
  mean_var <- mean(acov[1, ])
  var_plus <- mean_var * (n_samples - 1) / n_samples
  if (chains > 1) var_plus <- var_plus + var(chain_mean)
  
  rho_hat_t <- rep.int(0, n_samples)
  t <- 0
  rho_hat_even <- 1
  rho_hat_t[t + 1] <- rho_hat_even
  rho_hat_odd <- 1 - (mean_var - mean(acov[t + 2, ])) / var_plus
  rho_hat_t[t + 2] <- rho_hat_odd
  
  while (t < nrow(acov) - 5 && !is.nan(rho_hat_even + rho_hat_odd) && (rho_hat_even + rho_hat_odd > 0)) {
    t <- t + 2
    rho_hat_even <- 1 - (mean_var - mean(acov[t + 1, ])) / var_plus
    rho_hat_odd <- 1 - (mean_var - mean(acov[t + 2, ])) / var_plus
    if ((rho_hat_even + rho_hat_odd) >= 0) {
      rho_hat_t[t + 1] <- rho_hat_even
      rho_hat_t[t + 2] <- rho_hat_odd
    } else {
      break
    }
  }
  max_t <- t
  if (rho_hat_even > 0) rho_hat_t[max_t + 1] <- rho_hat_even
  
  t <- 0
  while (t <= max_t - 4) {
    t <- t + 2
    if (rho_hat_t[t + 1] + rho_hat_t[t + 2] > rho_hat_t[t - 1] + rho_hat_t[t]) {
      rho_hat_t[t + 1] <- (rho_hat_t[t - 1] + rho_hat_t[t]) / 2
      rho_hat_t[t + 2] <- rho_hat_t[t + 1]
    }
  }
  
  S <- chains * n_samples
  tau_hat <- -1 + 2 * sum(rho_hat_t[1:max_t]) + rho_hat_t[max_t + 1]
  tau_hat <- max(tau_hat, 1 / log10(S))
  return(S / tau_hat)
}

# --- Public Interface Functions ---

#' Calculate Rank-normalized Split-R-hat
#' @param sims A matrix of samples (iterations x chains).
#' @return A numeric value.
#' @export
r_hat <- function(sims) {
  if (is_constant(sims)) return(1.0)
  bulk_rhat <- rhat_rfun(z_scale(split_chains(sims)))
  sims_folded <- abs(sims - median(sims))
  tail_rhat <- rhat_rfun(z_scale(split_chains(sims_folded)))
  max(bulk_rhat, tail_rhat, na.rm = TRUE)
}

#' Calculate Bulk Effective Sample Size
#' @param sims A matrix of samples (iterations x chains).
#' @return A numeric value.
#' @export
ess_bulk <- function(sims) {
  ess_rfun(z_scale(split_chains(sims)))
}

#' Calculate Tail Effective Sample Size (at 2.5\% and 97.5\% quantiles)
#' @param sims A matrix of samples (iterations x chains).
#' @return A numeric value.
#' @export
ess_tail95 <- function(sims) {
  q025 <- (sims <= quantile(sims, 0.025)) + 0
  q025_ess <- ess_rfun(split_chains(q025))
  q975 <- (sims <= quantile(sims, 0.975)) + 0
  q975_ess <- ess_rfun(split_chains(q975))
  min(q025_ess, q975_ess, na.rm = TRUE)
}

#' Basic Effective Sample Size for a single chain or pooled chains
#' @param sims A numeric vector or matrix of samples.
#' @return A numeric value.
#' @export
ess_basic <- function(sims) {
  ess_rfun(split_chains(sims))
}

#' Maximum A Posteriori (MAP) Estimate
#' @param z A numeric vector of samples.
#' @return A numeric value.
#' @export
map_est <- function(z) {
  if (abs(max(z) - min(z)) < 1e-10) return(z[1])
  d <- density(z)
  d$x[which.max(d$y)]
}

#' Calculate 95\% Quantiles
#' @param x A numeric vector.
#' @return A numeric vector of length 2.
#' @export
quantile95 <- function(x) {
  quantile(x, probs = c(0.025, 0.975), names = TRUE)
}

#' RTMB-based Bayesian two-sample t-test wrapper function
#'
#' @description
#' Performs a Bayesian or Frequentist two-sample t-test using RTMB.
#'
#' @param x Numeric vector of responses for group 1, a formula (e.g., `y ~ group`), or a column name (unquoted) if `data` is provided.
#' @param y Numeric vector of responses for group 2, or a column name (unquoted) if `data` is provided. Required if `x` is not a formula.
#' @param data Data frame containing the variables.
#' @param r Numeric; Cauchy prior scale for the effect size (delta). Default is 0.707.
#' @param paired Logical; whether to perform a paired t-test.
#' @param ID Character; name of the ID variable for paired t-tests (required for formula input with paired = TRUE).
#' @param y_range Theoretical minimum and maximum values of the response variable as a vector c(min, max). Required when using weakly informative priors.
#' @param prior An object of class `"rtmb_prior"`.
#' Use `prior_flat()` for no prior, `prior_normal()` for default normal/exponential priors,
#' or `prior_weak()` for weakly informative Bayesian inference.
#' Default is `prior_flat()`.
#' @param init List of initial values.
#' @param var.equal Logical; whether to assume equal variances. Default is TRUE.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param missing Missing value handling strategy: "listwise".
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param ... Additional arguments.
#' @return An `RTMB_Model` object.
#'
#' @details
#' For classic inference, heteroscedastic two-sample t-tests use the same
#' RTMB Satterthwaite machinery as `optimize(marginal = ..., df_method = "satterthwaite")`.
#' The result is model-based and reproducible from the printed model code.
#' This corresponds to the Welch-type unequal-variance t-test, but the
#' degrees of freedom are computed by the package's internal Satterthwaite
#' procedure rather than by a separate closed-form formula.
#' When `prior_jzs()` is combined with `var.equal = FALSE`, BayesRTMB uses
#' a Welch-style JZS extension: the effect size `delta` is an explicit
#' parameter with a Cauchy prior, and the group mean difference is scaled by
#' the root-mean-square of the two group standard deviations.
#'
#' @example inst/examples/ex_ttest.R
#' @export
rtmb_ttest <- function(x, y = NULL, data = NULL, r = 0.707,
                       paired = FALSE, ID = NULL,
                       y_range = NULL,
                       prior = prior_flat(),
                       init = NULL, fixed = NULL,
                       var.equal = TRUE, missing = c("listwise", "fiml"),
                       WAIC = FALSE, ...) {

  missing <- match.arg(missing)
  x_expr <- substitute(x)
  y_expr <- substitute(y)

  # --- 1. Data Extraction ---
  is_formula <- is.call(x_expr) && (x_expr[[1]] == quote(`~`))
  formula_input <- is_formula
  raw_response <- NULL
  raw_group_idx <- NULL
  raw_id <- NULL
  if (is_formula) {
    x <- eval(x_expr, parent.frame())
    mf <- if (is.null(data)) model.frame(x, parent.frame()) else model.frame(x, data)
    response <- mf[[1]]; group <- as.factor(mf[[2]])
    levs <- levels(group)
    if (length(levs) != 2) {
      stop(
        sprintf("The grouping variable must have exactly 2 levels, but found %d: %s",
                length(levs), paste(levs, collapse = ", ")),
        call. = FALSE
      )
    }
    x_label <- levs[1]; y_label <- levs[2]
    
    if (!is.numeric(response) && !is.logical(response)) {
      stop("The response variable must be numeric. Character or factor variables are not supported.", call. = FALSE)
    }
    raw_response <- as.numeric(response)
    raw_group_idx <- as.integer(group)
    if (paired) {
      if (is.null(ID)) {
        stop("'ID' is required for paired t-tests with formula input (e.g., ID = 'subject_id').", call. = FALSE)
      }
      id_val <- if (!is.null(data)) data[[ID]] else eval(substitute(ID), parent.frame())
      if (is.null(id_val)) {
        stop(sprintf("ID variable '%s' not found in data.", as.character(substitute(ID))), call. = FALSE)
      }
      if (length(id_val) != nrow(mf)) {
        mf_rows <- suppressWarnings(as.integer(rownames(mf)))
        if (length(mf_rows) == nrow(mf) && !anyNA(mf_rows) && max(mf_rows) <= length(id_val)) {
          id_val <- id_val[mf_rows]
        } else {
          stop("Could not align 'ID' with the rows used in the paired t-test.", call. = FALSE)
        }
      }
      raw_id <- id_val
      d1 <- mf[group == levs[1], ]; d1$id <- id_val[group == levs[1]]
      d2 <- mf[group == levs[2], ]; d2$id <- id_val[group == levs[2]]
      common_ids <- intersect(d1$id, d2$id)
      Y1 <- d1[match(common_ids, d1$id), 1]; Y2 <- d2[match(common_ids, d2$id), 1]
    } else {
      Y1 <- as.numeric(response[group == levs[1]])
      Y2 <- as.numeric(response[group == levs[2]])
      if (missing == "listwise") {
         Y1 <- as.numeric(na.omit(Y1))
         Y2 <- as.numeric(na.omit(Y2))
      }
    }
  } else {
    x_label <- deparse(x_expr); y_label <- deparse(y_expr)
    if (identical(y_expr, quote(NULL))) {
      stop("'y' is required when 'x' is not a formula.", call. = FALSE)
    }
    if (!is.null(data)) {
      Y1 <- if (x_label %in% names(data)) data[[x_label]] else eval(x_expr, parent.frame())
      Y2 <- if (y_label %in% names(data)) data[[y_label]] else eval(y_expr, parent.frame())
    } else {
      Y1 <- eval(x_expr, parent.frame()); Y2 <- eval(y_expr, parent.frame())
    }
    
    if ((!is.numeric(Y1) && !is.logical(Y1)) || (!is.numeric(Y2) && !is.logical(Y2))) {
      stop("Both variables (x and y) must be numeric. Character or factor variables are not supported.", call. = FALSE)
    }
    if (paired) {
       if (length(Y1) != length(Y2)) stop("For paired t-tests without a formula, x and y must have the same length.", call. = FALSE)
       if (missing == "listwise") {
         valid_idx <- !is.na(Y1) & !is.na(Y2)
         Y1 <- Y1[valid_idx]
         Y2 <- Y2[valid_idx]
       }
    } else {
       if (missing == "listwise") {
         Y1 <- as.numeric(na.omit(Y1)); Y2 <- as.numeric(na.omit(Y2))
       }
    }
  }
  if (paired) {
    if (length(Y1) < 2L || length(Y2) < 2L) {
      stop("Paired t-tests require at least two complete pairs.", call. = FALSE)
    }
  } else {
    if (length(Y1) < 2L || length(Y2) < 2L) {
      stop("Two-sample t-tests require at least two observations in each group.", call. = FALSE)
    }
  }
  levs <- c(x_label, y_label)

  # --- 2. Prior Setup ---
  if (is.null(prior)) prior <- prior_flat()

  # Automatically switch to prior_weak() if y_range is provided and prior is default flat
  if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }
  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), prior_jzs(), or prior_weak().",
      call. = FALSE
    )
  }
  prior_type <- prior$type
  is_jzs <- prior_type == "jzs"
  is_weak <- prior_type == "weak"
  is_normal <- prior_type == "normal"
  use_delta_param <- is_jzs
  if (is_jzs) r <- prior$r
  
  if (is_normal) {
    if (is.null(prior$mu_sd) && !is.null(prior$Intercept_sd)) {
      prior$mu_sd <- prior$Intercept_sd
    }
  }

  # --- 3. AST Construction ---
  setup_exprs <- list(as.name("{"))
  if (formula_input) {
    if (paired) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(d1 <- data.frame(Y = Y[G == 1], ID = ID[G == 1]))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(d2 <- data.frame(Y = Y[G == 2], ID = ID[G == 2]))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(common_ids <- intersect(d1$ID, d2$ID))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(Y1 <- d1$Y[match(common_ids, d1$ID)])
      setup_exprs[[length(setup_exprs) + 1]] <- quote(Y2 <- d2$Y[match(common_ids, d2$ID)])
      setup_exprs[[length(setup_exprs) + 1]] <- quote(diffs <- Y1 - Y2)
    } else {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(Y1 <- as.numeric(na.omit(Y[G == 1])))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(Y2 <- as.numeric(na.omit(Y[G == 2])))
    }
  }
  if (is_weak) {
    if (is.null(y_range)) {
      stop("Specifying 'y_range' (theoretical minimum and maximum values of the response) is required when using prior_weak().", call. = FALSE)
    }
    
    # Common setup values
    sd_ratio_val <- prior$sd_ratio
    max_beta_val <- prior$max_beta
    
    setup_exprs[[length(setup_exprs) + 1]] <- bquote(half_d_y <- .(diff(y_range) / 2))
    setup_exprs[[length(setup_exprs) + 1]] <- bquote(base_scale <- half_d_y * .(sd_ratio_val))
    setup_exprs[[length(setup_exprs) + 1]] <- quote(alpha_prior_sd <- half_d_y)
    if (paired) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(x_sd <- 0.5)
    } else {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(n1 <- length(Y1))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(n2 <- length(Y2))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(p1 <- n1 / (n1 + n2))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(x_sd <- sqrt(p1 * (1 - p1)))
    }
    setup_exprs[[length(setup_exprs) + 1]] <- bquote(diff_prior_sd <- .(max_beta_val) * base_scale / x_sd)
    setup_exprs[[length(setup_exprs) + 1]] <- bquote(mid_y <- .(if (paired) 0 else mean(y_range)))
    setup_exprs[[length(setup_exprs) + 1]] <- quote(sigma_rate_weak <- 1.0 / base_scale)
  }
  setup_ast <- as.call(setup_exprs)

  if (paired) {
    dat <- if (formula_input) list(Y = raw_response, G = raw_group_idx, ID = raw_id, r = r) else list(diffs = Y1 - Y2, r = r)
    param_ast <- if (use_delta_param) {
      quote({ sd_diff = Dim(1, lower = 0); delta = Dim(1) })
    } else {
      quote({ sd_diff = Dim(1, lower = 0); diff = Dim(1) })
    }
    tran_ast <- if (use_delta_param) {
      quote({ diff <- delta * sd_diff })
    } else {
      quote({ delta <- diff / sd_diff })
    }
    model_body <- list(quote(diffs ~ normal(diff, sd_diff)))
    if (is_jzs) model_body[[length(model_body)+1]] <- quote(delta ~ cauchy(0, r))
    if (is_weak) {
      model_body[[length(model_body)+1]] <- quote(diff ~ normal(0, diff_prior_sd))
      model_body[[length(model_body)+1]] <- quote(sd_diff ~ exponential(sigma_rate_weak))
    }
    if (is_normal) {
      if (!is.null(prior$b_sd)) {
        model_body[[length(model_body)+1]] <- bquote(diff ~ normal(0, .(prior$b_sd)))
      }
      if (!is.null(prior$sigma_rate)) {
        model_body[[length(model_body)+1]] <- bquote(sd_diff ~ exponential(.(prior$sigma_rate)))
      }
    }
    view_vars <- c("diff", "delta", "sd_diff")
    marginal <- if (use_delta_param) "delta" else "diff"
    waic_ast <- quote({
      log_lik <- normal_lpdf(diffs, diff, sd_diff, sum = FALSE)
    })
  } else {
    dat <- if (formula_input) list(Y = raw_response, G = raw_group_idx, r = r) else list(Y1 = Y1, Y2 = Y2, r = r)
    
    # Structure for independent t-tests
    if (!var.equal) {
      if (use_delta_param) {
        param_ast <- quote({ total_mean = Dim(1); sd = Dim(2, lower = 0); delta = Dim(1) })
        tran_ast <- quote({
          sd_pooled <- sqrt((sd[1]^2 + sd[2]^2)/2)
          diff <- delta * sd_pooled
          mean0 <- total_mean + diff/2
          mean1 <- total_mean - diff/2
        })
        view_vars <- c("diff", "delta", "total_mean", "mean0", "mean1", "sd")
        marginal <- c("total_mean", "delta")
      } else {
        # Heteroscedastic (Welch style): Use group means as parameters to trigger Satterthwaite on diff.
        param_ast <- quote({ mean0 = Dim(1); mean1 = Dim(1); sd = Dim(2, lower = 0) })
        tran_ast <- quote({
          diff <- mean0 - mean1
          total_mean <- (mean0 + mean1)/2
          sd_pooled <- sqrt((sd[1]^2 + sd[2]^2)/2)
          delta <- diff / sd_pooled
        })
        view_vars <- c("diff", "delta", "mean0", "mean1", "sd")
        marginal <- c("mean0", "mean1")
      }
      model_body <- list(quote(Y1 ~ normal(mean0, sd[1])), quote(Y2 ~ normal(mean1, sd[2])))
      waic_ast <- quote({
        log_lik <- c(
          normal_lpdf(Y1, mean0, sd[1], sum = FALSE),
          normal_lpdf(Y2, mean1, sd[2], sum = FALSE)
        )
      })
    } else {
      # Homoscedastic (Standard pooled): Use total_mean and diff as parameters to fix DF at N-K
      param_ast <- if (use_delta_param) {
        quote({ total_mean = Dim(1); sd = Dim(1, lower = 0); delta = Dim(1) })
      } else {
        quote({ total_mean = Dim(1); sd = Dim(1, lower = 0); diff = Dim(1) })
      }
      tran_ast <- if (use_delta_param) {
        quote({ diff <- delta * sd; mean0 <- total_mean + diff/2; mean1 <- total_mean - diff/2 })
      } else {
        quote({ delta <- diff / sd; mean0 <- total_mean + diff/2; mean1 <- total_mean - diff/2 })
      }
      model_body <- list(quote(Y1 ~ normal(mean0, sd)), quote(Y2 ~ normal(mean1, sd)))
      view_vars <- c("diff", "delta", "total_mean", "sd")
      marginal <- if (use_delta_param) c("total_mean", "delta") else c("total_mean", "diff")
      waic_ast <- quote({
        log_lik <- c(
          normal_lpdf(Y1, mean0, sd, sum = FALSE),
          normal_lpdf(Y2, mean1, sd, sum = FALSE)
        )
      })
    }
    
    if (is_jzs) {
       model_body[[length(model_body)+1]] <- quote(delta ~ cauchy(0, r))
    }
    if (is_weak) {
      if (!var.equal) {
        model_body[[length(model_body)+1]] <- quote(mean0 ~ normal(mid_y, alpha_prior_sd))
        model_body[[length(model_body)+1]] <- quote(mean1 ~ normal(mid_y, alpha_prior_sd))
      } else {
        model_body[[length(model_body)+1]] <- quote(total_mean ~ normal(mid_y, alpha_prior_sd))
        model_body[[length(model_body)+1]] <- quote(diff ~ normal(0, diff_prior_sd))
      }
      model_body[[length(model_body)+1]] <- quote(sd ~ exponential(sigma_rate_weak))
    }
    if (is_normal) {
      if (!var.equal) {
        mu_sd <- if (!is.null(prior$mu_sd)) prior$mu_sd else prior$Intercept_sd
        if (!is.null(mu_sd)) {
          model_body[[length(model_body)+1]] <- bquote(mean0 ~ normal(0, .(mu_sd)))
          model_body[[length(model_body)+1]] <- bquote(mean1 ~ normal(0, .(mu_sd)))
        }
      } else {
        mu_sd <- if (!is.null(prior$mu_sd)) prior$mu_sd else prior$Intercept_sd
        if (!is.null(mu_sd)) model_body[[length(model_body)+1]] <- bquote(total_mean ~ normal(0, .(mu_sd)))
        if (!is.null(prior$b_sd)) model_body[[length(model_body)+1]] <- bquote(diff ~ normal(0, .(prior$b_sd)))
      }
      if (!is.null(prior$sigma_rate)) model_body[[length(model_body)+1]] <- bquote(sd ~ exponential(.(prior$sigma_rate)))
    }
  }

  model_ast <- as.call(c(list(as.name("{")), model_body))
  code_obj <- if (isTRUE(WAIC)) {
    waic_ast <- .rtmb_waic_generate_ast(NULL, waic_ast)
    eval(substitute(
      rtmb_code(setup = S, parameters = P, transform = T, model = M, generate = G),
      list(S = setup_ast, P = param_ast, T = tran_ast, M = model_ast, G = waic_ast)
    ))
  } else {
    eval(substitute(
      rtmb_code(setup = S, parameters = P, transform = T, model = M),
      list(S = setup_ast, P = param_ast, T = tran_ast, M = model_ast)
    ))
  }

  tmp_env <- list2env(dat)
  ordered_data <- env_to_ordered_list(tmp_env, dat, setup_ast)
  obj <- rtmb_model(data = ordered_data, code = code_obj, fixed = fixed, view = view_vars)

  obj$type <- "ttest"
  obj$extra <- list(
    source = "wrapper",
    prior_type = prior$type,
    marginal = marginal,
    levs = levs,
    paired = paired,
    var_equal = var.equal
  )

  return(obj)
}

#' Calculate Conditional Effects
#'
#' @description
#' Calculate and visualize the predicted values (marginal effects) of a variable,
#' potentially conditional on the levels of another variable (interaction).
#'
#' @param fit Model fit object (e.g., `mcmc_fit`, `map_fit`).
#' @param effect Name of the variable to visualize (e.g., "X1" or "X1:X2").
#' @param prob Probability for the credible/confidence interval (default is 0.95).
#' @param sd_multiplier Numeric. Multiplier for standard deviation when splitting continuous moderators (default is 1).
#' @param ... Additional arguments.
#'
#' @return An object of class `ce_rtmb` containing the predicted values and their credible intervals.
#'
#' @examples
#' \donttest{
#'   data(debate, package = "BayesRTMB")
#'   fit <- rtmb_lm(sat ~ talk * perf, data = debate)
#'   map_fit <- fit$optimize()
#'   ce <- conditional_effects(map_fit, effect = "talk:perf")
#'   plot(ce)
#'   summary(ce)
#' }
#' @export
conditional_effects <- function(fit, effect, prob = 0.95, sd_multiplier = 1, ...) {
  UseMethod("conditional_effects")
}

#' @method conditional_effects default
#' @export
conditional_effects.default <- function(fit, effect, prob = 0.95, sd_multiplier = 1, ...) {
  stop(sprintf("No conditional_effects method for object of class '%s'.", class(fit)[1]))
}

#' Calculate conditional effects for MCMC fit objects
#' @method conditional_effects mcmc_fit
#' @param fit An object of class `MCMC_Fit`.
#' @param effect Name of the explanatory variable to visualize (e.g., "X1" or "X1:X2").
#' @param prob Probability for the credible/confidence interval (default is 0.95).
#' @param sd_multiplier Numeric. Multiplier for standard deviation when splitting continuous moderators (default is 1).
#' @param resolution Grid resolution to calculate for continuous variables (default is 100).
#' @param ... Additional arguments.
#'
#' @return A `ce_rtmb` object.
#' @export
conditional_effects.mcmc_fit <- function(fit, effect, prob = 0.95, sd_multiplier = 1, resolution = 100, ...) {
  model_obj <- fit$model
  if (is.null(model_obj$formula) || is.null(model_obj$raw_data)) {
    stop("This model object does not contain a formula or the original data.")
  }

  form <- model_obj$formula
  raw_data <- model_obj$raw_data
  fam <- model_obj$family
  X_mean <- model_obj$data$X_mean # For back-transforming Intercept_c if needed

  # Check for interaction and split
  eff_vars <- strsplit(effect, ":")[[1]]
  if (length(eff_vars) > 2) {
    stop("Interaction plots for 3 or more variables are not currently supported.")
  }
  eff1 <- eff_vars[1]
  eff2 <- if (length(eff_vars) == 2) eff_vars[2] else NULL

  # 1. Create base data frame (fixing other variables to mean/mode)
  base_data <- lapply(raw_data, function(x) {
    if (is.numeric(x)) {
      mean(x, na.rm = TRUE)
    } else if (is.factor(x) || is.character(x)) {
      tbl <- table(x)
      factor(names(tbl)[which.max(tbl)], levels = levels(as.factor(x)))
    } else {
      x[1]
    }
  })
  base_data <- as.data.frame(base_data)

  # 2. Create newdata moving only the effect variable
  val1 <- raw_data[[eff1]]
  if (is.null(val1)) stop(sprintf("Variable '%s' not found in the data.", eff1))

  is_numeric1 <- is.numeric(val1)
  if (is_numeric1) {
    seq1 <- seq(min(val1, na.rm = TRUE), max(val1, na.rm = TRUE), length.out = resolution)
  } else {
    seq1 <- sort(unique(val1))
  }

  if (!is.null(eff2)) {
    val2 <- raw_data[[eff2]]
    if (is.null(val2)) stop(sprintf("Variable '%s' not found in the data.", eff2))

    # If the second variable is continuous and has many unique values, restrict to 3 representative points (Mean-1SD, Mean, Mean+1SD)
    if (is.numeric(val2) && length(unique(val2)) > 5) {
      seq2 <- c(mean(val2, na.rm=TRUE) - sd_multiplier * sd(val2, na.rm=TRUE),
                mean(val2, na.rm=TRUE),
                mean(val2, na.rm=TRUE) + sd_multiplier * sd(val2, na.rm=TRUE))
      seq2 <- round(seq2, 2) # Round for better display
    } else {
      seq2 <- sort(unique(val2))
    }

    grid_data <- expand.grid(eff1 = seq1, eff2 = seq2)
    names(grid_data) <- c(eff1, eff2)

    newdata <- base_data[rep(1, nrow(grid_data)), , drop = FALSE]
    newdata[[eff1]] <- grid_data[[eff1]]
    newdata[[eff2]] <- grid_data[[eff2]]
  } else {
    newdata <- base_data[rep(1, length(seq1)), , drop = FALSE]
    newdata[[eff1]] <- seq1
  }

  # 3. Create design matrix
  rhs <- delete.response(terms(form))
  # Ensure all factor levels are known to model.matrix
  mf_raw <- model.frame(rhs, data = raw_data, na.action = na.pass)
  xlev <- .getXlevels(terms(rhs), mf_raw)
  X_new <- model.matrix(rhs, data = newdata, xlev = xlev)

  # 4. Obtain posterior samples (fixed effects)
  # Handle both wrapper functions (b, Intercept_c) and custom models (beta)
  all_pars <- dimnames(fit$draws(inc_transform=TRUE, inc_generate=FALSE))[[3]]
  
  if (any(grepl("^beta(\\[|$)", all_pars))) {
    beta_samples <- fit$draws(pars = "beta", inc_random = FALSE, inc_transform = FALSE, inc_generate = FALSE)
  } else if (any(grepl("^b(\\[|$)", all_pars))) {
    b_samps <- fit$draws(pars = "b", inc_random = FALSE, inc_transform = FALSE, inc_generate = FALSE)
    
    # Try to get Intercept (transformed) or Intercept_c
    if ("Intercept" %in% all_pars) {
      int_samps <- fit$draws(pars = "Intercept", inc_transform = TRUE)
    } else if ("Intercept_c" %in% all_pars) {
      # Back-calculate Intercept from Intercept_c if X_mean is available
      ic_samps <- fit$draws(pars = "Intercept_c")
      if (!is.null(X_mean)) {
        b_mat <- matrix(b_samps, nrow = dim(b_samps)[1] * dim(b_samps)[2], ncol = dim(b_samps)[3])
        int_val <- as.numeric(ic_samps) - (b_mat %*% as.numeric(X_mean))
        int_samps <- array(int_val, dim = c(dim(b_samps)[1], dim(b_samps)[2], 1))
      } else {
        int_samps <- ic_samps
      }
    } else {
      int_samps <- NULL
    }
    
    # Combine Intercept and b to match model.matrix output (Intercept first)
    if (!is.null(int_samps)) {
      I <- dim(b_samps)[1]; C <- dim(b_samps)[2]; P <- dim(b_samps)[3]
      beta_samples <- array(NA, dim = c(I, C, P + 1))
      beta_samples[,,1] <- int_samps
      beta_samples[,,2:(P+1)] <- b_samps
    } else {
      beta_samples <- b_samps
    }
  } else {
    stop("Could not find fixed effect parameters (b or beta) in the fit object.")
  }

  I <- dim(beta_samples)[1]
  C <- dim(beta_samples)[2]
  P <- dim(beta_samples)[3]
  beta_flat <- matrix(beta_samples, nrow = I * C, ncol = P)

  # Check dimension compatibility
  if (ncol(X_new) != ncol(beta_flat)) {
    # If mismatch, try to adjust X_new (e.g., if Intercept is missing in beta but present in X_new)
    if (ncol(X_new) == ncol(beta_flat) + 1 && colnames(X_new)[1] == "(Intercept)") {
      X_new <- X_new[, -1, drop = FALSE]
    } else if (ncol(beta_flat) == ncol(X_new) + 1) {
       # Maybe beta has intercept but X_new doesn't (rare but possible)
       X_new <- cbind(1, X_new)
    } else {
      stop(sprintf("Dimension mismatch: Design matrix has %d columns, but fixed effects have %d parameters.", ncol(X_new), ncol(beta_flat)))
    }
  }

  # 5. Calculate linear predictor
  eta <- X_new %*% t(beta_flat)

  # 6. Convert to expected value via inverse link function
  if (is.null(fam)) fam <- "gaussian"
  inv_link <- switch(fam,
                     "gaussian" = , "lognormal" = , "student_t" = function(x) x,
                     "poisson" = , "neg_binomial" = , "gamma" = exp,
                     "bernoulli" = , "binomial" = plogis,
                     function(x) x
  )
  mu <- inv_link(eta)

  # 7. Summarize posterior distribution
  alpha <- 1 - prob
  lower_q <- alpha / 2
  upper_q <- 1 - alpha / 2

  res_df <- data.frame(
    estimate   = apply(mu, 1, mean),
    lower      = apply(mu, 1, quantile, probs = lower_q),
    upper      = apply(mu, 1, quantile, probs = upper_q)
  )
  res_df <- cbind(newdata[, eff_vars, drop = FALSE], res_df)

  # 8. Combine results into a list and assign class
  res <- list(data = res_df, effect_vars = eff_vars, is_numeric = is_numeric1)
  class(res) <- "ce_rtmb"
  return(res)
}

#' @method conditional_effects map_fit
#' @export
conditional_effects.map_fit <- conditional_effects.mcmc_fit

#' @method conditional_effects advi_fit
#' @export
conditional_effects.advi_fit <- conditional_effects.mcmc_fit

#' @method conditional_effects vb_fit
#' @export
conditional_effects.vb_fit <- conditional_effects.mcmc_fit

#' Plot method for ce_rtmb class (Base R)
#' @method plot ce_rtmb
#' @param x An object of class ce_rtmb
#' @param ... Additional arguments.
#' @export
plot.ce_rtmb <- function(x, ...) {
  df <- x$data
  eff_vars <- x$effect_vars
  eff1 <- eff_vars[1]
  has_interaction <- length(eff_vars) > 1

  x_val <- df[[eff1]]
  y_est <- df$estimate
  y_low <- df$lower
  y_up  <- df$upper

  if (!has_interaction) {
    # --- Without interaction ---
    col_line <- rgb(0, 0.45, 0.7)
    col_ribbon <- rgb(0, 0.45, 0.7, 0.2)

    if (x$is_numeric) {
      plot(x_val, y_est, type = "n", ylim = range(c(y_low, y_up)),
           xlab = eff1, ylab = "Predicted value", main = paste("Conditional effect of", eff1), ...)
      polygon(c(x_val, rev(x_val)), c(y_low, rev(y_up)), col = col_ribbon, border = NA)
      lines(x_val, y_est, col = col_line, lwd = 2)
    } else {
      x_fct <- as.factor(x_val)
      x_num <- as.numeric(x_fct)
      lvls <- levels(x_fct)
      
      plot(x_num, y_est, type = "n", ylim = range(c(y_low, y_up)),
           xlim = c(0.5, length(lvls) + 0.5), xaxt = "n",
           xlab = eff1, ylab = "Predicted value", main = paste("Conditional effect of", eff1), ...)
      axis(1, at = 1:length(lvls), labels = lvls)
      segments(x0 = x_num, y0 = y_low, x1 = x_num, y1 = y_up, col = col_line, lwd = 2)
      points(x_num, y_est, col = col_line, pch = 16, cex = 1.5)
    }

  } else {
    # --- With interaction ---
    eff2 <- eff_vars[2]
    groups <- unique(df[[eff2]])
    n_groups <- length(groups)

    # Create color palette based on number of groups
    cols_line <- hcl.colors(n_groups, palette = "Dark 2")
    cols_ribbon <- sapply(cols_line, function(col) {
      rgb_val <- col2rgb(col) / 255
      rgb(rgb_val[1], rgb_val[2], rgb_val[3], 0.2) # Transparency 0.2
    })

    if (x$is_numeric) {
      plot(x_val, y_est, type = "n", ylim = range(c(y_low, y_up)),
           xlab = eff1, ylab = "Predicted value",
           main = paste("Conditional effect of", eff1, "by", eff2), ...)

      for (i in seq_along(groups)) {
        idx <- df[[eff2]] == groups[i]
        xv <- df[[eff1]][idx]
        polygon(c(xv, rev(xv)), c(df$lower[idx], rev(df$upper[idx])), col = cols_ribbon[i], border = NA)
        lines(xv, df$estimate[idx], col = cols_line[i], lwd = 2)
      }
    } else {
      x_fct <- as.factor(x_val)
      x_num <- as.numeric(x_fct)
      lvls <- levels(x_fct)

      plot(x_num, y_est, type = "n", ylim = range(c(y_low, y_up)),
           xlim = c(0.5, length(lvls) + 0.5), xaxt = "n",
           xlab = eff1, ylab = "Predicted value",
           main = paste("Conditional effect of", eff1, "by", eff2), ...)
      axis(1, at = 1:length(lvls), labels = lvls)

      # Shift X-axis slightly to avoid overlapping error bars
      offset_step <- 0.1
      offsets <- seq(-offset_step * (n_groups-1)/2, offset_step * (n_groups-1)/2, length.out = n_groups)

      for (i in seq_along(groups)) {
        idx <- df[[eff2]] == groups[i]
        xv <- x_num[idx] + offsets[i]
        segments(x0 = xv, y0 = df$lower[idx], x1 = xv, y1 = df$upper[idx], col = cols_line[i], lwd = 2)
        points(xv, df$estimate[idx], col = cols_line[i], pch = 16, cex = 1.5)
      }
    }

    # Add legend
    legend("topright", title = eff2, legend = format(groups, digits = 3),
           col = cols_line, lty = 1, pch = ifelse(x$is_numeric, NA, 16), lwd = 2, bty = "n")
  }
  invisible(x)
}

#' Print method for ce_rtmb class (automatically calls plot)
#' @method print ce_rtmb
#' @param x An object of class ce_rtmb
#' @param ... Additional arguments.
#' @export
print.ce_rtmb <- function(x, ...) {
  plot(x, ...)
  invisible(x)
}

#' Summary method for ce_rtmb class
#' @method summary ce_rtmb
#' @param object An object of class ce_rtmb
#' @param ... Additional arguments.
#' @export
summary.ce_rtmb <- function(object, ...) {
  return(object$data)
}


#' Calculate Simple Effects
#'
#' @description
#' Calculate the effect of a focal variable at different levels of a moderator.
#' For categorical focal variables, it calculates pairwise differences (contrasts).
#' For continuous focal variables, it calculates the slope (simple slopes).
#'
#' @param fit Model fit object (e.g., `map_fit`, `mcmc_fit`).
#' @param effect Character string of the interaction (e.g., "A:B"). The first variable is the focal variable.
#' @param prob Probability for the credible/confidence interval (default is 0.95).
#' @param sd_multiplier Multiplier for SD for continuous moderators (default is 1).
#' @param ... Additional arguments.
#'
#' @return A `ce_simple` object (data frame) containing the estimated effects and their credible intervals.
#'
#' @examples
#' \donttest{
#'   data(debate, package = "BayesRTMB")
#'   fit <- rtmb_lm(sat ~ talk * perf, data = debate)
#'   map_fit <- fit$optimize()
#'   # Effect of talk at different levels of performance
#'   se <- simple_effects(map_fit, effect = "talk:perf")
#'   print(se)
#' }
#' @export
simple_effects <- function(fit, effect, prob = 0.95, sd_multiplier = 1, ...) {
  UseMethod("simple_effects")
}

#' Simple effects for MCMC fit objects
#' @method simple_effects mcmc_fit
#' @param fit An object of class `MCMC_Fit`.
#' @param effect Interaction term (e.g., "A:B").
#' @param prob Probability for credible intervals.
#' @param sd_multiplier Multiplier for SD for continuous moderators.
#' @param ... Additional arguments.
#' @export
simple_effects.mcmc_fit <- function(fit, effect, prob = 0.95, sd_multiplier = 1, ...) {
  eff_vars <- strsplit(effect, ":")[[1]]
  if (length(eff_vars) != 2) {
    stop("simple_effects requires an interaction of exactly two variables (e.g., 'A:B').")
  }
  
  focal <- eff_vars[1]
  mod <- eff_vars[2]
  
  # 1. Reuse conditional_effects logic to get predicted values
  # We use a higher resolution for moderators if continuous
  ce <- conditional_effects(fit, effect = effect, resolution = 10, sd_multiplier = sd_multiplier, ...)
  df <- ce$data
  
  # Identify levels/values of the moderator
  mod_vals <- unique(df[[mod]])
  
  res_list <- list()
  
  if (ce$is_numeric) {
    # Simple Slopes for continuous focal variable
    eps <- 1e-4
    results <- data.frame()
    
    # [Internal Helper] Get predicted samples (shared with categorical logic)
    get_pred_samples <- function(fit, newdata) {
      model_obj <- fit$model
      form <- model_obj$formula
      raw_data <- model_obj$raw_data
      X_mean <- model_obj$data$X_mean
      rhs <- delete.response(terms(form))
      
      # Ensure all factor levels are known to model.matrix
      mf_raw <- model.frame(rhs, data = raw_data, na.action = na.pass)
      xlev <- .getXlevels(terms(rhs), mf_raw)
      X_new <- model.matrix(rhs, data = newdata, xlev = xlev)
      
      all_pars <- dimnames(fit$draws(inc_transform=TRUE))[[3]]
      if (any(grepl("^beta(\\[|$)", all_pars))) {
        beta_samples <- fit$draws(pars = "beta")
      } else {
        b_samps <- fit$draws(pars = "b")
        if ("Intercept" %in% all_pars) {
          int_samps <- fit$draws(pars = "Intercept", inc_transform = TRUE)
        } else if ("Intercept_c" %in% all_pars) {
          ic_samps <- fit$draws(pars = "Intercept_c")
          b_mat <- matrix(b_samps, nrow = dim(b_samps)[1] * dim(b_samps)[2], ncol = dim(b_samps)[3])
          int_val <- as.numeric(ic_samps) - (b_mat %*% as.numeric(X_mean))
          int_samps <- array(int_val, dim = c(dim(b_samps)[1], dim(b_samps)[2], 1))
        } else {
          int_samps <- NULL
        }
        if (!is.null(int_samps)) {
          I <- dim(b_samps)[1]; C <- dim(b_samps)[2]; P <- dim(b_samps)[3]
          beta_samples <- array(NA, dim = c(I, C, P + 1))
          beta_samples[,,1] <- int_samps
          beta_samples[,,2:(P+1)] <- b_samps
        } else {
          beta_samples <- b_samps
        }
      }
      
      I <- dim(beta_samples)[1]; C <- dim(beta_samples)[2]; P <- dim(beta_samples)[3]
      beta_flat <- matrix(beta_samples, nrow = I * C, ncol = P)
      
      if (ncol(X_new) == ncol(beta_flat) + 1 && colnames(X_new)[1] == "(Intercept)") {
        X_new <- X_new[, -1, drop = FALSE]
      }
      
      eta <- X_new %*% t(beta_flat)
      inv_link <- switch(model_obj$family %||% "gaussian",
                         "gaussian" = function(x) x, "poisson" = exp, "bernoulli" = plogis, function(x) x)
      return(inv_link(eta))
    }

    for (mv in mod_vals) {
      # Use the mean of the focal variable as the point of evaluation for the slope
      f_val <- mean(df[[focal]], na.rm=TRUE)
      
      # Create newdata for x and x + eps
      # We take one row of original data as template to keep other variables at their baseline
      template_data <- fit$model$raw_data[1, , drop=FALSE]
      # Set moderator value
      template_data[[mod]] <- mv
      
      sub_newdata1 <- template_data
      sub_newdata1[[focal]] <- f_val
      sub_newdata2 <- template_data
      sub_newdata2[[focal]] <- f_val + eps
      
      combined_newdata <- rbind(sub_newdata1, sub_newdata2)
      pred_samples <- get_pred_samples(fit, combined_newdata)
      
      # Slope = (y2 - y1) / eps
      slope_samples <- (pred_samples[2, ] - pred_samples[1, ]) / eps
      
      res_row <- data.frame(
        moderator = mod,
        mod_value = mv,
        term = paste("Slope of", focal),
        estimate = mean(slope_samples),
        lower = quantile(slope_samples, (1-prob)/2),
        upper = quantile(slope_samples, 1-(1-prob)/2)
      )
      results <- rbind(results, res_row)
    }
    
    names(results)[2] <- mod
    class(results) <- c("ce_simple", "data.frame")
    return(results)
    
  } else {
    # Pairwise differences for categorical focal variable
    focal_lvls <- unique(df[[focal]])
    if (length(focal_lvls) < 2) stop("Focal variable must have at least 2 levels.")
    
    # We need the posterior samples of the predicted values to calculate the differences
    # (re-running the core logic of conditional_effects but keeping samples)
    
    # [Internal Helper] Get predicted samples
    get_pred_samples <- function(fit, newdata) {
      model_obj <- fit$model
      form <- model_obj$formula
      X_mean <- model_obj$data$X_mean
      rhs <- delete.response(terms(form))
      X_new <- model.matrix(rhs, data = newdata)
      
      all_pars <- dimnames(fit$draws(inc_transform=TRUE))[[3]]
      if (any(grepl("^beta(\\[|$)", all_pars))) {
        beta_samples <- fit$draws(pars = "beta")
      } else {
        b_samps <- fit$draws(pars = "b")
        if ("Intercept" %in% all_pars) {
          int_samps <- fit$draws(pars = "Intercept", inc_transform = TRUE)
        } else if ("Intercept_c" %in% all_pars) {
          ic_samps <- fit$draws(pars = "Intercept_c")
          b_mat <- matrix(b_samps, nrow = dim(b_samps)[1] * dim(b_samps)[2], ncol = dim(b_samps)[3])
          int_val <- as.numeric(ic_samps) - (b_mat %*% as.numeric(X_mean))
          int_samps <- array(int_val, dim = c(dim(b_samps)[1], dim(b_samps)[2], 1))
        } else {
          int_samps <- NULL
        }
        if (!is.null(int_samps)) {
          I <- dim(b_samps)[1]; C <- dim(b_samps)[2]; P <- dim(b_samps)[3]
          beta_samples <- array(NA, dim = c(I, C, P + 1))
          beta_samples[,,1] <- int_samps
          beta_samples[,,2:(P+1)] <- b_samps
        } else {
          beta_samples <- b_samps
        }
      }
      
      I <- dim(beta_samples)[1]; C <- dim(beta_samples)[2]; P <- dim(beta_samples)[3]
      beta_flat <- matrix(beta_samples, nrow = I * C, ncol = P)
      
      if (ncol(X_new) == ncol(beta_flat) + 1 && colnames(X_new)[1] == "(Intercept)") {
        X_new <- X_new[, -1, drop = FALSE]
      }
      
      eta <- X_new %*% t(beta_flat)
      inv_link <- switch(model_obj$family %||% "gaussian",
                         "gaussian" = function(x) x, "poisson" = exp, "bernoulli" = plogis, function(x) x)
      return(inv_link(eta))
    }
    
    # Calculate differences for each level of moderator
    results <- data.frame()
    
    for (mv in mod_vals) {
      sub_newdata <- df[df[[mod]] == mv, names(df) %in% names(fit$model$raw_data)]
      # Ensure focal levels are in consistent order
      sub_newdata <- sub_newdata[order(sub_newdata[[focal]]), ]
      
      pred_samples <- get_pred_samples(fit, sub_newdata) # (N_levels x N_samples)
      
      # All pairwise differences
      lvls <- sub_newdata[[focal]]
      for (i in 1:(length(lvls)-1)) {
        for (j in (i+1):length(lvls)) {
          diff_samples <- pred_samples[j, ] - pred_samples[i, ]
          
          res_row <- data.frame(
            moderator = mod,
            mod_value = mv,
            term = paste(lvls[j], "-", lvls[i]),
            estimate = mean(diff_samples),
            lower = quantile(diff_samples, (1-prob)/2),
            upper = quantile(diff_samples, 1-(1-prob)/2)
          )
          results <- rbind(results, res_row)
        }
      }
    }
    
    names(results)[2] <- mod
    class(results) <- c("ce_simple", "data.frame")
    return(results)
  }
}

#' @method simple_effects map_fit
#' @export
simple_effects.map_fit <- simple_effects.mcmc_fit
#' @method simple_effects advi_fit
#' @export
simple_effects.advi_fit <- simple_effects.mcmc_fit
#' @method simple_effects vb_fit
#' @export
simple_effects.vb_fit <- simple_effects.mcmc_fit

#' Print simple effects
#' @param x A `ce_simple` object.
#' @param digits Number of digits to print.
#' @param ... Additional arguments.
#' @export
print.ce_simple <- function(x, digits = 3, ...) {
  cat("--- Simple Effects Analysis ---\n")
  
  df <- as.data.frame(x)
  # Format numeric columns
  num_cols <- sapply(df, is.numeric)
  df[num_cols] <- lapply(df[num_cols], function(v) {
    sprintf(paste0("%.", digits, "f"), v)
  })
  
  print.data.frame(df, row.names = FALSE, quote = FALSE)
  cat("\n")
  invisible(x)
}


#' Calculate Item Information Function
#' @param x An object of class RTMB_Fit_Base
#' @param ... Additional arguments.
#' @examples
#' \donttest{
#'   fit <- rtmb_irt(data = BigFive[, 1:5], model = "2PL", type = "ordered")
#'   map_fit <- fit$optimize()
#'   ii <- item_info(map_fit)
#'   plot(ii)
#' }
#' @export
item_info <- function(x, ...) UseMethod("item_info")

#' Calculate Test Information Function
#' @param x An object of class RTMB_Fit_Base
#' @param ... Additional arguments.
#' @examples
#' \donttest{
#'   fit <- rtmb_irt(data = BigFive[, 1:5], model = "2PL", type = "ordered")
#'   map_fit <- fit$optimize()
#'   ti <- test_info(map_fit)
#'   plot(ti)
#' }
#' @export
test_info <- function(x, ...) UseMethod("test_info")

#' Item Information Function for RTMB_Fit_Base
#' @method item_info RTMB_Fit_Base
#' @param x An object of class RTMB_Fit_Base.
#' @param theta_seq Sequence of trait values (ability) to evaluate.
#' @param items Index or item names to restrict the calculation to specific items (optional).
#' @param ... Additional arguments.
#' @export
item_info.RTMB_Fit_Base <- function(x, theta_seq = seq(-4, 4, length.out = 100), items = NULL, ...) {
  est <- if (!is.null(x$par)) x$par else x$EAP()
  b <- est$b



  # Fix: Refer to the correct item names (par_names) saved during model construction instead of names(b)
  par_names_b <- x$model$par_names$b

  # Get data type and item names
  if (is.matrix(b)) {
    type <- "ordered"
    J <- nrow(b)
    K_cat <- ncol(b) + 1
    item_names <- if (is.list(par_names_b)) par_names_b[[1]] else paste0("Item", 1:J)
  } else {
    type <- "binary"
    J <- length(b)
    item_names <- if (!is.null(par_names_b)) par_names_b else paste0("Item", 1:J)
  }

  # Identify indices of target items
  if (!is.null(items)) {
    if (is.character(items)) {
      target_idx <- match(items, item_names)
      if (any(is.na(target_idx))) {
        warning("Specified item names not found: ", paste(items[is.na(target_idx)], collapse = ", "))
        target_idx <- target_idx[!is.na(target_idx)]
      }
    } else {
      target_idx <- items[items >= 1 & items <= J]
    }
    if (length(target_idx) == 0) stop("No valid items selected.")
  } else {
    target_idx <- 1:J
  }

  a <- if (!is.null(est$a)) est$a else rep(1.0, J)
  c_param <- if (!is.null(est$c)) est$c else rep(0.0, J)

  info_mat <- matrix(0, nrow = length(theta_seq), ncol = length(target_idx))
  colnames(info_mat) <- item_names[target_idx]

  for (idx in seq_along(target_idx)) {
    j <- target_idx[idx]
    if (type == "binary") {
      eta <- a[j] * (theta_seq - b[j])
      P <- c_param[j] + (1 - c_param[j]) * plogis(eta)
      Q <- 1 - P

      info_mat[, idx] <- (a[j]^2 * Q / P) * ((P - c_param[j]) / (1 - c_param[j]))^2

    } else if (type == "ordered") {
      P_star <- matrix(0, nrow = length(theta_seq), ncol = K_cat + 1)
      P_star[, 1] <- 0.0 # P(Y <= 0) = 0
      P_star[, K_cat + 1] <- 1.0 # P(Y <= K) = 1

      for (k in 1:(K_cat - 1)) {
        P_star[, k + 1] <- plogis(b[j, k] - a[j] * theta_seq)
      }

      item_info_j <- rep(0, length(theta_seq))
      for (k in 1:K_cat) {
        Pk <- P_star[, k + 1] - P_star[, k] # P(Y = k) = P(Y <= k) - P(Y <= k-1)

        dP_star_k   <- -a[j] * P_star[, k] * (1 - P_star[, k])
        dP_star_kp1 <- -a[j] * P_star[, k + 1] * (1 - P_star[, k + 1])

        if (k == 1) dP_star_k <- rep(0, length(theta_seq))
        if (k == K_cat) dP_star_kp1 <- rep(0, length(theta_seq))

        dPk <- dP_star_kp1 - dP_star_k

        Pk_safe <- pmax(Pk, 1e-10)
        item_info_j <- item_info_j + (dPk^2) / Pk_safe
      }
      info_mat[, idx] <- item_info_j
    }
  }

  res <- list(theta = theta_seq, info = info_mat)
  class(res) <- "rtmb_item_info"
  return(res)
}

#' @method item_info mcmc_fit
#' @export
item_info.mcmc_fit <- item_info.RTMB_Fit_Base
#' @method item_info advi_fit
#' @export
item_info.advi_fit <- item_info.RTMB_Fit_Base
#' @method item_info map_fit
#' @export
item_info.map_fit <- item_info.RTMB_Fit_Base

#' @method test_info RTMB_Fit_Base
#' @export
test_info.RTMB_Fit_Base <- function(x, theta_seq = seq(-4, 4, length.out = 100), ...) {
  ii <- item_info(x, theta_seq, ...)
  test_info_vec <- rowSums(ii$info)

  res <- list(theta = theta_seq, info = test_info_vec)
  class(res) <- "rtmb_test_info"
  return(res)
}

#' @method test_info mcmc_fit
#' @export
test_info.mcmc_fit <- test_info.RTMB_Fit_Base
#' @method test_info advi_fit
#' @export
test_info.advi_fit <- test_info.RTMB_Fit_Base
#' @method test_info map_fit
#' @export
test_info.map_fit <- test_info.RTMB_Fit_Base

# --- Plotting Methods ---

#' @method plot rtmb_item_info
#' @export
plot.rtmb_item_info <- function(x, legend = TRUE, ...) {
  matplot(x$theta, x$info, type = "l", lty = 1,
          xlab = expression(theta ~ "(Ability)"), ylab = "Information",
          main = "Item Information Functions", ...)

  # Add legend (if legend = TRUE)
  if (legend && ncol(x$info) > 0) {
    legend("topright", legend = colnames(x$info), col = 1:ncol(x$info), lty = 1, cex = 0.8)
  }
}

#' @method plot rtmb_test_info
#' @export
plot.rtmb_test_info <- function(x, ...) {
  plot(x$theta, x$info, type = "l", lwd = 2, col = "blue",
       xlab = expression(theta ~ "(Ability)"), ylab = "Information",
       main = "Test Information Function", ...)
}

#' @method print rtmb_item_info
#' @export
print.rtmb_item_info <- function(x, ...) {
  plot(x, ...)
  invisible(x)
}

#' @method print rtmb_test_info
#' @export
print.rtmb_test_info <- function(x, ...) {
  plot(x, ...)
  invisible(x)
}


#' Calculate Item Response Curve / Category Response Curve
#' @export
#' @param x An object of class
#' @param ... Additional arguments.
item_curve <- function(x, ...) UseMethod("item_curve")

#' Item Response Curve for RTMB_Fit_Base
#' @method item_curve RTMB_Fit_Base
#' @param x An object of class RTMB_Fit_Base.
#' @param theta_seq Sequence of trait values (ability) to evaluate.
#' @param items Index or item names to restrict the calculation to specific items (optional).
#' @param ... Additional arguments.
#' @examples
#' \donttest{
#'   fit <- rtmb_irt(data = BigFive[, 1:5], model = "2PL", type = "ordered")
#'   map_fit <- fit$optimize()
#'   ic <- item_curve(map_fit)
#'   plot(ic)
#' }
#' @export
item_curve.RTMB_Fit_Base <- function(x, theta_seq = seq(-4, 4, length.out = 100), items = NULL, ...) {
  est <- if (!is.null(x$par)) x$par else x$EAP()
  b <- est$b



  par_names_b <- x$model$par_names$b

  # Get data type and item names
  if (is.matrix(b)) {
    type <- "ordered"
    J <- nrow(b)
    K_cat <- ncol(b) + 1
    item_names <- if (is.list(par_names_b)) par_names_b[[1]] else paste0("Item", 1:J)
  } else {
    type <- "binary"
    J <- length(b)
    item_names <- if (!is.null(par_names_b)) par_names_b else paste0("Item", 1:J)
  }

  # Identify indices of target items
  if (!is.null(items)) {
    if (is.character(items)) {
      target_idx <- match(items, item_names)
      if (any(is.na(target_idx))) {
        warning("Specified item names not found: ", paste(items[is.na(target_idx)], collapse = ", "))
        target_idx <- target_idx[!is.na(target_idx)]
      }
    } else {
      target_idx <- items[items >= 1 & items <= J]
    }
    if (length(target_idx) == 0) stop("No valid items selected.")
  } else {
    target_idx <- 1:J
  }

  a <- if (!is.null(est$a)) est$a else rep(1.0, J)
  c_param <- if (!is.null(est$c)) est$c else rep(0.0, J)

  out_curves <- list()

  for (idx in seq_along(target_idx)) {
    j <- target_idx[idx]

    if (type == "binary") {
      # Probability of correct response for binary model
      eta <- a[j] * (theta_seq - b[j])
      P <- c_param[j] + (1 - c_param[j]) * plogis(eta)

      out_curves[[item_names[j]]] <- matrix(P, ncol = 1)
      colnames(out_curves[[item_names[j]]]) <- "P(Y=1)"

    } else if (type == "ordered") {
      # Category selection probability for ordered model (GRM)
      eta <- a[j] * theta_seq

      # Cumulative probability P(Y <= k)
      P_cum <- matrix(1.0, nrow = length(theta_seq), ncol = K_cat + 1)
      P_cum[, 1] <- 0.0 # P(Y <= 0) = 0

      for (k in 1:(K_cat - 1)) {
        P_cum[, k + 1] <- plogis(b[j, k] - eta)
      }
      # P_cum[, K_cat + 1] is initialized to 1.0 (P(Y <= K) = 1)

      # Probability of each category P(Y = k) = P(Y <= k) - P(Y <= k-1)
      P_cat <- matrix(0, nrow = length(theta_seq), ncol = K_cat)
      for (k in 1:K_cat) {
        P_cat[, k] <- P_cum[, k + 1] - P_cum[, k]
      }

      colnames(P_cat) <- paste0("Cat", 1:K_cat)
      out_curves[[item_names[j]]] <- P_cat
    }
  }

  res <- list(theta = theta_seq, curves = out_curves, type = type)
  class(res) <- "rtmb_item_curve"
  return(res)
}

#' @method item_curve mcmc_fit
#' @export
item_curve.mcmc_fit <- item_curve.RTMB_Fit_Base
#' @method item_curve advi_fit
#' @export
item_curve.advi_fit <- item_curve.RTMB_Fit_Base
#' @method item_curve map_fit
#' @export
item_curve.map_fit <- item_curve.RTMB_Fit_Base

#' @method plot rtmb_item_curve
#' @export
plot.rtmb_item_curve <- function(x, legend = TRUE, ...) {
  n_items <- length(x$curves)

  if (x$type == "binary") {
    # Binary model: Draw all items overlaid on a single plot
    P_mat <- do.call(cbind, x$curves)
    matplot(x$theta, P_mat, type = "l", lty = 1, ylim = c(0, 1),
            xlab = expression(theta ~ "(Ability)"), ylab = "Probability",
            main = "Item Response Curves", ...)

    if (legend && ncol(P_mat) > 0) {
      legend("topleft", legend = names(x$curves), col = 1:n_items, lty = 1, cex = 0.8)
    }

  } else if (x$type == "ordered") {
    # Ordered model: Draw category response curves in separate panels per item
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))

    if (n_items > 1) {
      nr <- ceiling(sqrt(n_items))
      nc <- ceiling(n_items / nr)
      par(mfrow = c(nr, nc))
    }

    for (item_name in names(x$curves)) {
      P_mat <- x$curves[[item_name]]
      matplot(x$theta, P_mat, type = "l", lty = 1:ncol(P_mat), ylim = c(0, 1),
              xlab = expression(theta), ylab = "Probability",
              main = paste("Category Curves:", item_name), ...)

      if (legend) {
        legend("topleft", legend = colnames(P_mat), col = 1:ncol(P_mat),
               lty = 1:ncol(P_mat), cex = 0.7)
      }
    }
  }
}

#' @method print rtmb_item_curve
#' @export
print.rtmb_item_curve <- function(x, ...) {
  plot(x, ...)
  invisible(x)
}
#' Sort and display factor loadings neatly
#'
#' @param loadings Matrix, data frame, or list of factor loadings (if list, the first element is used)
#' @param cutoff Absolute loadings below this value will be displayed as blank (default is 0.0)
#' @param round_digits Number of decimal places to display (default is 3)
#' @return Sorted loading matrix (returned invisibly, allowing assignment to a variable)
#' @export
sort_loadings <- function(loadings, cutoff = 0.0, round_digits = 3) {

  if (is.list(loadings) && !is.data.frame(loadings)) {
    if (length(loadings) == 0) stop("The input list is empty.")
    loadings <- loadings[[1]]
  }

  load_mat <- as.matrix(loadings)

  if (is.null(rownames(load_mat))) {
    rownames(load_mat) <- paste0("V", 1:nrow(load_mat))
  }

  max_factor <- apply(abs(load_mat), 1, which.max)
  max_loading <- apply(abs(load_mat), 1, max)

  df_order <- data.frame(
    var_name = rownames(load_mat),
    factor = max_factor,
    loading = max_loading,
    stringsAsFactors = FALSE
  )

  df_order <- df_order[order(df_order$factor, -df_order$loading), ]
  sorted_mat <- load_mat[df_order$var_name, , drop = FALSE]

  print_mat <- matrix("", nrow = nrow(sorted_mat), ncol = ncol(sorted_mat))
  rownames(print_mat) <- rownames(sorted_mat)
  colnames(print_mat) <- colnames(sorted_mat)

  # Create a string to identify zeros like "-.000"
  zero_str <- paste0("-.", paste(rep("0", round_digits), collapse = ""))

  for (i in 1:nrow(sorted_mat)) {
    for (j in 1:ncol(sorted_mat)) {
      val <- sorted_mat[i, j]
      if (abs(val) >= cutoff) {
        s <- sprintf(paste0("%.", round_digits, "f"), val)
        s <- sub("^0\\.", ".", s)
        s <- sub("^-0\\.", "-.", s)

        # Replace "-.000" with ".000"
        if (s == zero_str) {
          s <- sub("^-", "", s)
        }

        print_mat[i, j] <- s
      }
    }
  }

  print_df <- as.data.frame(print_mat, stringsAsFactors = FALSE)
  print(print_df, quote = FALSE, right = TRUE)

  return(invisible(sorted_mat))
}
#' Calculate Bayes Factor
#'
#' @description
#' Compare two models by calculating the Bayes Factor based on their
#' marginal likelihoods.
#'
#' @param logml1 The first model fit (e.g., `mcmc_fit`) or its log-marginal likelihood.
#' @param logml2 The second model fit (e.g., `mcmc_fit`) or its log-marginal likelihood.
#' @param error_threshold Threshold for warning about high bridge-sampling error (default 0.2).
#'
#' @return An object of class `bayes_factor` containing the Bayes factor, log Bayes factor, 
#'   approximate estimation error, and interpretation.
#'
#' @examples
#' \donttest{
#'   # Compare two models using Bayes Factor
#'   data(debate, package = "BayesRTMB")
#'   fit1 <- rtmb_lm(sat ~ talk, data = debate)
#'   fit2 <- rtmb_lm(sat ~ talk + perf, data = debate)
#'   map1 <- fit1$optimize()
#'   map2 <- fit2$optimize()
#'   bf <- bayes_factor(map1, map2)
#'   print(bf)
#' }
#' @export
bayes_factor <- function(logml1, logml2, error_threshold = 0.2) {
  # Helper function to extract or calculate log marginal likelihood
  get_lml <- function(x, name = "Model") {
    if (inherits(x, "mcmc_fit")) {
      if (is.null(x$log_ml)) {
        cat(sprintf("Calculating marginal likelihood for %s...\n", name))
        x$log_ml <- x$bridgesampling()
      }
      return(x$log_ml)
    } else if (inherits(x, "map_fit")) {
      return(x$log_ml)
    } else if (inherits(x, "advi_fit")) {
      # Use the best ELBO as an approximation of log marginal likelihood
      return(max(x$ELBO, na.rm = TRUE))
    } else if (is.numeric(x)) {
      return(x)
    } else {
      stop(sprintf("Unsupported object type for %s. Must be numeric or a fit object.", name))
    }
  }

  val1_obj <- get_lml(logml1, "Model 1")
  val2_obj <- get_lml(logml2, "Model 2")

  # Strip attributes (error, ess) and calculate as pure numbers
  val1 <- as.numeric(val1_obj)
  val2 <- as.numeric(val2_obj)

  # Calculate log Bayes factor and Bayes factor
  log_bf <- val1 - val2
  bf <- exp(log_bf)

  # Error propagation (calculate if logml1 and logml2 contain "error" attribute)
  err1 <- attr(val1_obj, "error")
  err2 <- attr(val2_obj, "error")

  has_error <- !is.null(err1) && !is.null(err2) && !is.na(err1) && !is.na(err2)

  if (has_error) {
    # Standard error assuming the two MCMC samplings are independent
    log_bf_err <- sqrt(err1^2 + err2^2)
    if (log_bf_err > error_threshold) {
      warning(
        sprintf(
          "The estimation error of the log Bayes factor (%.3f) exceeds the threshold (%g). Interpretation of the results may be unstable.\n",
          log_bf_err, error_threshold
        ),
        "Consider increasing the number of MCMC samples or ESS to improve precision.\n",
        call. = FALSE, immediate. = TRUE
      )
    }
  } else {
    log_bf_err <- NA_real_
  }

  # Interpretation of evidence strength based on Jeffreys (1961) / Kass & Raftery (1995)
  if (is.na(bf) || is.nan(bf)) {
    evidence <- "Indeterminate"
  } else if (bf > 100) {
    evidence <- "Decisive evidence for Model 1"
  } else if (bf > 10) {
    evidence <- "Strong evidence for Model 1"
  } else if (bf > 3) {
    evidence <- "Substantial evidence for Model 1"
  } else if (bf > 1) {
    evidence <- "Anecdotal evidence for Model 1"
  } else if (bf == 1) {
    evidence <- "No evidence"
  } else if (bf >= 1/3) {
    evidence <- "Anecdotal evidence for Model 2"
  } else if (bf >= 1/10) {
    evidence <- "Substantial evidence for Model 2"
  } else if (bf >= 1/100) {
    evidence <- "Strong evidence for Model 2"
  } else {
    evidence <- "Decisive evidence for Model 2"
  }

  res <- list(
    BF12 = bf,
    log_BF12 = log_bf,
    log_BF_error = log_bf_err,
    evidence = evidence
  )

  class(res) <- "bayes_factor"
  return(res)
}

#' Print method for bayes_factor objects
#' @param x An object of class bayes_factor.
#' @param digits Number of decimal places to display (default is 4).
#' @param ... Additional arguments.
#' @export
print.bayes_factor <- function(x, digits = 4, ...) {
  cat("Bayes Factor (BF12) :", round(x$BF12, digits), "\n")

  if (!is.na(x$log_BF_error)) {
    cat(sprintf("Log Bayes Factor    : %.4f (Approx. Error = %.4f)\n", x$log_BF12, x$log_BF_error))
  } else {
    cat(sprintf("Log Bayes Factor    : %.4f\n", x$log_BF12))
  }

  cat("Evidence            :", x$evidence, "\n")
  invisible(x)
}

#' Print method for bayes_factor_rtmb objects
#' @param x An object of class bayes_factor_rtmb.
#' @param digits Number of decimal places to display (default is 4).
#' @param ... Additional arguments.
#' @export
print.bayes_factor_rtmb <- function(x, digits = 4, ...) {
  cat("--- Bayes Factor Analysis (Bridge Sampling) ---\n")
  cat("Bayes Factor (BF12) :", round(x$BF12, digits), "\n")

  if (!is.na(x$log_BF_error)) {
    cat(sprintf("Log Bayes Factor    : %.4f (Approx. Error = %.4f)\n", x$log_BF12, x$log_BF_error))
  } else {
    cat(sprintf("Log Bayes Factor    : %.4f\n", x$log_BF12))
  }

  cat("Evidence            :", x$evidence, "\n")
  
  if (!is.null(x$fixed)) {
    cat("Comparison model    : Parameters fixed at", deparse(x$fixed), "\n")
  }
  
  cat("\n")
  invisible(x)
}

#' Reconstruct an Observation-Level Random-Effect Design Matrix
#'
#' @description
#' Converts the transposed block random-effect design matrix used internally by
#' `rtmb_glmer()` into an observation-level matrix. This is kept as a small
#' utility for users who want to work from the block matrix returned by
#' `make_glmer_re_terms()`.
#'
#' @param Zt Transposed block random-effect design matrix.
#' @param group_idx Integer group index for each observation.
#' @param N Number of observations. Defaults to `length(group_idx)`.
#'
#' @return A matrix with `N` rows and one column per random-effect coefficient.
#' @export
make_glmer_Z_matrix <- function(Zt, group_idx, N = length(group_idx)) {
  num_groups <- max(group_idx)
  num_ranef <- nrow(Zt) / num_groups
  Z_mat <- matrix(0, nrow = N, ncol = num_ranef)
  for (i in seq_len(N)) {
    g <- group_idx[i]
    Z_mat[i, ] <- Zt[((g - 1) * num_ranef + 1):(g * num_ranef), i]
  }
  Z_mat
}

.deparse_glmer_term <- function(x) {
  paste(deparse(x), collapse = " ")
}

.expand_glmer_bars <- function(bars) {
  expanded_bars <- list()
  if (length(bars) > 0) {
    for (bar in bars) {
      grp_expr <- bar[[3]]
      term_labels <- attr(terms(as.formula(paste("~", .deparse_glmer_term(grp_expr)))), "term.labels")
      for (label in term_labels) {
        new_bar <- bar
        new_bar[[3]] <- parse(text = label)[[1]]
        expanded_bars[[length(expanded_bars) + 1]] <- new_bar
      }
    }
  }
  expanded_bars
}

.make_glmer_Zt <- function(Z_mat, group_idx, num_groups) {
  N <- nrow(Z_mat)
  num_ranef <- ncol(Z_mat)
  Zt <- matrix(0, nrow = num_groups * num_ranef, ncol = N)
  for (i in seq_len(N)) {
    g <- group_idx[i]
    Zt[((g - 1) * num_ranef + 1):(g * num_ranef), i] <- Z_mat[i, ]
  }
  Zt
}

.select_glmer_setup_data <- function(formula, data, resid_group = NULL, resid_time = NULL) {
  has_random <- !is.null(findbars(formula))
  mf_form <- if (has_random) subbars(formula) else formula
  if (!inherits(mf_form, "formula")) mf_form <- as.formula(mf_form, env = environment(formula))
  if (!is.null(resid_group)) mf_form <- update(mf_form, as.formula(paste("~ . +", resid_group)))
  if (!is.null(resid_time)) mf_form <- update(mf_form, as.formula(paste("~ . +", resid_time)))

  vars <- unique(all.vars(mf_form))
  missing_vars <- setdiff(vars, names(data))
  if (length(missing_vars) > 0) {
    stop("Variables not found in data: ", paste(missing_vars, collapse = ", "), call. = FALSE)
  }
  as.data.frame(data)[, vars, drop = FALSE]
}

#' Prepare GLMM Formula Components
#'
#' @description
#' Builds the response, fixed-effect model matrix, and lme4-style random-effect
#' design components used by `rtmb_glmer()`. This is the user-facing helper that
#' reproduces what the wrapper places in the generated `setup` block.
#'
#' @param formula lme4-style formula.
#' @param data Data frame.
#' @param family Character string of the distribution family.
#' @param resid_group Optional residual-correlation grouping variable.
#' @param resid_time Optional residual-correlation time variable.
#' @param within Optional list for wide-to-long conversion when the response is
#'   written as `cbind(...)`.
#' @param factors Optional character vector of variables to treat as factors.
#' @param missing Missing value handling strategy: "listwise".
#'
#' @return A list containing `Y`, `X`, `trials`, `offset`, `N`, fixed-effect
#' metadata, and random-effect terms.
#' @export
make_glmer_re_terms <- function(formula, data, family = "gaussian",
                                resid_group = NULL, resid_time = NULL,
                                within = NULL, factors = NULL,
                                missing = "listwise") {
  # Expand dot (.) in formula if present
  if (!is.null(data) && inherits(formula, "formula")) {
    terms_expanded <- try(terms(formula, data = data), silent = TRUE)
    if (!inherits(terms_expanded, "try-error")) {
      formula <- formula(terms_expanded)
    }
  }

  if (family == "gaussian" && is.call(formula[[2]]) && identical(formula[[2]][[1]], as.name("cbind"))) {
    processed <- .handle_wide_to_long(formula, data, within, factors)
    formula <- processed$formula
    data <- processed$data
    factors <- processed$factors
  }

  if (!is.null(factors)) {
    data <- as.data.frame(data)
    for (f in factors) {
      if (f %in% names(data)) data[[f]] <- as.factor(data[[f]])
    }
  }

  data <- .select_glmer_setup_data(formula, data, resid_group = resid_group, resid_time = resid_time)
  has_random <- !is.null(findbars(formula))
  fixed_form <- if (has_random) nobars(formula) else formula
  mf_form <- if (has_random) subbars(formula) else formula
  if (!inherits(fixed_form, "formula")) fixed_form <- as.formula(fixed_form, env = environment(formula))
  if (!inherits(mf_form, "formula")) mf_form <- as.formula(mf_form, env = environment(formula))

  if (!is.null(resid_group)) mf_form <- update(mf_form, as.formula(paste("~ . +", resid_group)))
  if (!is.null(resid_time)) mf_form <- update(mf_form, as.formula(paste("~ . +", resid_time)))

  na_act <- if (missing == "listwise") na.omit else na.pass
  mf <- model.frame(mf_form, data = data, na.action = na_act)
  Y <- model.response(mf)
  X_full <- model.matrix(fixed_form, mf)
  X_assign <- attr(X_full, "assign")
  X_terms <- attr(terms(fixed_form), "term.labels")
  offset <- model.offset(mf)
  N <- nrow(mf)

  num_categories <- NULL
  if (is.matrix(Y)) {
    if (ncol(Y) == 2 && family == "binomial") {
      trials <- as.numeric(Y[, 1] + Y[, 2])
      Y <- as.numeric(Y[, 1])
    } else if (ncol(Y) == 1) {
      Y <- as.numeric(Y[, 1])
      trials <- rep(1, length(Y))
    } else {
      stop("Invalid matrix format for the response variable.")
    }
  } else {
    trials <- rep(1, length(Y))
    if (is.factor(Y)) {
      Y <- if (family %in% c("bernoulli", "binomial")) as.numeric(Y) - 1 else as.numeric(Y)
    } else {
      Y <- as.numeric(Y)
    }
  }

  if (family %in% c("ordered", "sequential")) {
    if (any(!is.finite(Y)) || any(abs(Y - round(Y)) > .Machine$double.eps^0.5) || any(Y < 1)) {
      stop(
        "Ordinal responses must be category codes 1, 2, ..., K or an ordered/factor response. ",
        "Check that the response variable is not being centered or otherwise transformed.",
        call. = FALSE
      )
    }
    Y <- as.integer(round(Y))
    num_categories <- max(Y)
  }

  has_intercept <- "(Intercept)" %in% colnames(X_full)
  if (family %in% c("ordered", "sequential")) has_intercept <- FALSE
  fixed_colnames <- colnames(X_full)
  X <- X_full
  if ("(Intercept)" %in% colnames(X)) {
    X <- X[, colnames(X) != "(Intercept)", drop = FALSE]
  }

  bars <- if (has_random) .expand_glmer_bars(findbars(formula)) else list()
  num_bars <- length(bars)
  suffix <- function(b) if (num_bars > 1) paste0("_", b) else ""

  random_terms <- list()
  random_data <- list()
  num_ranef_list <- list()
  num_groups_list <- list()
  ranef_names_list <- list()
  group_labels_list <- list()

  if (num_bars > 0) {
    for (b in seq_len(num_bars)) {
      bar <- bars[[b]]
      re_form <- as.formula(paste("~", .deparse_glmer_term(bar[[2]])))
      Z_mat <- model.matrix(re_form, mf)

      group_var_label <- .deparse_glmer_term(bar[[3]])
      vars <- trimws(unlist(strsplit(group_var_label, ":")))
      if (length(vars) > 1) {
        flist <- interaction(mf[vars], sep = ":", drop = TRUE)
      } else {
        flist <- as.factor(mf[[vars]])
      }
      group_idx <- as.integer(flist)

      if (length(group_idx) == 0) {
        stop("Grouping factor '", group_var_label, "' resulted in an empty vector.")
      }
      if (any(is.na(group_idx))) {
        stop("NA values found in grouping factor '", group_var_label, "'. Check your data.")
      }

      num_groups <- length(levels(flist))
      num_ranef <- ncol(Z_mat)
      Zt <- .make_glmer_Zt(Z_mat, group_idx, num_groups)

      random_data[[paste0("Zt", suffix(b))]] <- Zt
      random_data[[paste0("group_idx", suffix(b))]] <- group_idx
      num_ranef_list[[b]] <- num_ranef
      num_groups_list[[b]] <- num_groups

      r_names <- colnames(Z_mat)
      r_names[r_names == "(Intercept)"] <- "Int"
      ranef_names_list[[b]] <- r_names
      group_labels_list[[b]] <- group_var_label

      random_terms[[b]] <- list(
        Z = Z_mat,
        Zt = Zt,
        group_idx = group_idx,
        group = flist,
        group_label = group_var_label,
        num_groups = num_groups,
        num_ranef = num_ranef,
        ranef_names = r_names
      )
    }
  }

  list(
    Y = Y,
    X = X,
    trials = trials,
    offset = offset,
    N = N,
    mf = mf,
    X_assign = X_assign,
    X_terms = X_terms,
    fixed_colnames = fixed_colnames,
    fixed_names = colnames(X),
    has_intercept = has_intercept,
    num_categories = num_categories,
    random = list(
      bars = bars,
      num_bars = num_bars,
      terms = random_terms,
      data = random_data,
      num_ranef_list = num_ranef_list,
      num_groups_list = num_groups_list,
      ranef_names_list = ranef_names_list,
      group_labels_list = group_labels_list
    )
  )
}

#' RTMB-based GLMM wrapper function
#'
#' @param formula lme4-style formula (e.g., Y ~ X + (1 | GID))
#' @param data Data frame
#' @param family Character string of the distribution family (e.g., "gaussian",
#'   "binomial", "poisson", "ordered", "sequential")
#' @param laplace Logical; whether to marginalize random effects using Laplace approximation
#' @param prior An object of class "rtmb_prior" specifying the prior distribution. Use prior_weak(), prior_rhs(), or prior_ssp(). Default is `prior_flat()`.
#' @param y_range Theoretical minimum and maximum values of the response variable as a vector c(min, max). Required when using weakly informative or regularized priors with continuous models.
#' @param init List of initial values (generated automatically based on glm if omitted)

#' @param gmc Character vector of variable names for Grand Mean Centering (GMC). If "all", all numeric variables are centered.
#' @param centering Alias for `gmc`.
#' @param cwc List for Centering Within Cluster (CWC). Should contain \code{cluster} (group variable) and \code{pars} (variable names to center).
#' @param view Character vector of parameter names to prioritize in summary.
#' @param factors Character vector of variable names to be treated as factors.
#' @param contrasts Character string specifying the contrast type ("treatment" or "sum").
#' @param sigma_by Character vector specifying variables to group residual variance by (heteroscedasticity).
#' @param resid_corr Residual correlation structure: "ar1" (Autoregressive), "cs" (Compound Symmetry), "toep" (Toeplitz), or "un" (Unstructured).
#' @param resid_time Variable name for time points in residual correlation.
#' @param resid_group Variable name for grouping in residual correlation.
#' @param within Optional list for wide-to-long conversion.
#' @param generate Optional expression for generated quantities.
#' @param WAIC Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.
#' @param .force_sum Logical; internal use only.
#' @param fixed Optional named list of fixed values for specific parameters.
#' @param missing Missing value handling strategy: "listwise".
#' @example inst/examples/ex_lm.R
#' @export
rtmb_glmer <- function(formula, data, family = "gaussian", laplace = FALSE,
                       prior = prior_flat(),
                       y_range = NULL,
                       init = NULL,
                       fixed = NULL,
                       gmc = NULL,
                       centering = NULL,
                       cwc = NULL,
                       view = NULL,
                       within = NULL,
                       factors = NULL,
                       contrasts = "treatment",
                       sigma_by = NULL,
                       resid_corr = NULL,
                       resid_time = NULL,
                       resid_group = NULL,
                       generate = NULL,
                       missing = c("listwise", "fiml"),
                       WAIC = FALSE,
                       .force_sum = FALSE) {

  if (!is.null(centering)) {
    if (!is.null(gmc) && !identical(gmc, centering)) {
      stop("Specify only one of 'gmc' or 'centering', or use identical values.", call. = FALSE)
    }
    gmc <- centering
  }

  valid_families <- c(
    "gaussian", "lognormal", "student_t", "bernoulli", "binomial",
    "poisson", "neg_binomial", "gamma", "ordered", "sequential"
  )
  if (!is.character(family) || length(family) != 1L || is.na(family) ||
      !(family %in% valid_families)) {
    stop(
      sprintf(
        "Invalid 'family' value. Valid options are: %s",
        paste(valid_families, collapse = ", ")
      ),
      call. = FALSE
    )
  }
  if (!is.null(contrasts) &&
      (!is.character(contrasts) || length(contrasts) != 1L ||
       !(contrasts %in% c("treatment", "sum")))) {
    stop("'contrasts' must be either 'treatment', 'sum', or NULL.", call. = FALSE)
  }

  # Expand dot (.) in formula if present
  if (!is.null(data) && inherits(formula, "formula")) {
    terms_expanded <- try(terms(formula, data = data), silent = TRUE)
    if (!inherits(terms_expanded, "try-error")) {
      formula <- formula(terms_expanded)
    }
  }

  missing <- match.arg(missing)
  # --- Handle Wide to Long conversion if needed ---
  if (family == "gaussian" && is.call(formula[[2]]) && identical(formula[[2]][[1]], as.name("cbind"))) {
    processed <- .handle_wide_to_long(formula, data, within, factors)
    formula <- processed$formula
    data <- processed$data
    factors <- processed$factors
  }

  # --- 0. Contrast Management (Automatic sum-to-zero) ---


   if (!is.null(resid_corr)) {
     valid_resid_corr <- c("ar1", "cs", "un", "toep")
     if (!(resid_corr %in% valid_resid_corr)) {
       stop(sprintf("Invalid 'resid_corr' value: '%s'. Valid options are: %s",
            resid_corr, paste(valid_resid_corr, collapse = ", ")))
     }
   }

  # --- 0. Data Preparation (Factors) ---
  if (!is.null(factors)) {
    data <- as.data.frame(data)
    for (f in factors) {
      if (f %in% names(data)) {
        data[[f]] <- as.factor(data[[f]])
      }
    }
  }

  # --- 0. Data Centering (GMC / CWC) ---
  if (!is.null(gmc) || !is.null(cwc)) {
    data_centered <- as.data.frame(data)

    # 1. Grand Mean Centering
    if (!is.null(gmc)) {
      target_gmc <- if (identical(gmc, "all")) {
        vars_in_fixed <- all.vars(nobars(formula))
        vars_in_fixed <- vars_in_fixed[-1]
        vars_in_fixed <- vars_in_fixed[vars_in_fixed %in% names(data_centered)]
        vars_in_fixed[vapply(data_centered[vars_in_fixed], is.numeric, logical(1))]
      } else {
        gmc
      }
      for (v in target_gmc) {
        if (v %in% names(data_centered)) {
          data_centered[[v]] <- data_centered[[v]] - mean(data_centered[[v]], na.rm = TRUE)
        } else {
          warning(sprintf("Variable '%s' for GMC not found in data.", v))
        }
      }
    }

    # 2. Centering Within Cluster
    if (!is.null(cwc)) {
      # Robustly extract cluster and pars
      if ("pars" %in% names(cwc)) {
        cluster_var <- cwc$cluster
        target_pars <- cwc$pars
      } else if ("cluster" %in% names(cwc)) {
        cluster_var <- cwc$cluster
        # Any elements not named "cluster" are considered target_pars
        target_pars <- unlist(cwc[names(cwc) != "cluster" | names(cwc) == ""])
      } else if (length(cwc) >= 2) {
        # Assume first is cluster, rest are pars
        cluster_var <- cwc[[1]]
        target_pars <- unlist(cwc[-1])
      } else {
        stop("Invalid 'cwc' format. Use list(cluster = ID, pars = c('var1', 'var2')) or list(ID, 'var1').")
      }

      # Resolve group ID (character or symbol)
      group_id <- if (is.character(cluster_var)) {
        data_centered[[cluster_var]]
      } else {
        # cluster_var might be a symbol (e.g., group)
        # Try to evaluate it in the context of data_centered
        tryCatch(eval(cluster_var, data_centered, parent.frame()),
                 error = function(e) NULL)
      }

      if (is.null(group_id)) {
        # If still NULL, try to deparse and check if it's a column name
        group_var_name <- deparse(cluster_var)
        if (group_var_name %in% names(data_centered)) {
          group_id <- data_centered[[group_var_name]]
        } else {
           stop("Cluster variable for CWC not found.")
        }
      }

      for (v in target_pars) {
        if (v %in% names(data_centered)) {
          group_means <- tapply(data_centered[[v]], group_id, mean, na.rm = TRUE)
          data_centered[[v]] <- data_centered[[v]] - group_means[as.character(group_id)]
        } else {
          warning(sprintf("Variable '%s' for CWC not found in data.", v))
        }
      }
    }
    data <- data_centered
  }

  # --- 0.1 Handle sigma_by (Heteroscedasticity) ---
  sigma_idx <- NULL
  num_sigma_groups <- 1
  sigma_group_names <- "sigma"
  if (!is.null(sigma_by)) {
    if (!family %in% c("gaussian", "lognormal", "student_t")) {
      stop("'sigma_by' is only supported for continuous families (gaussian, lognormal, student_t).")
    } else {
      # Identify categorical factors in sigma_by
      if (identical(sigma_by, "all")) {
        # Extract all variables in fixed part of formula and check if they are factors
        vars_in_form <- all.vars(nobars(formula))
        # Exclude response (the first variable in all.vars is the response)
        vars_in_form <- vars_in_form[-1]
        is_cat <- sapply(data[vars_in_form], function(x) is.factor(x) || is.character(x))
        target_vars <- vars_in_form[is_cat]
      } else {
        target_vars <- sigma_by
      }

      if (length(target_vars) > 0) {
        sigma_factor <- if (length(target_vars) > 1) {
          interaction(data[target_vars], drop = TRUE, sep = ":")
        } else {
          as.factor(data[[target_vars]])
        }
        sigma_idx <- as.integer(sigma_factor)
        num_sigma_groups <- nlevels(sigma_factor)
        sigma_group_names <- levels(sigma_factor)
      }
    }
  }

  # --- 0.2 Default Initial Values ---
  if (is.null(init)) {
    init <- list()
  }
  if (is.list(init)) {
    # Set safe default for residual correlation (e.g. CS, AR1) if not specified by user.
    # This avoids non-positive definite matrices during the initial check.
    if (!is.null(resid_corr) && !("rho_resid" %in% names(init)) && resid_corr %in% c("ar1", "cs", "toep")) {
      init$rho_resid <- 0.1
    }
  }

  if (is.null(prior)) {
    prior <- prior_flat()
  }

  # Automatically switch to prior_weak() if y_range is provided and prior is default flat
  if (!is.null(y_range) && inherits(prior, "rtmb_prior") && identical(prior$type, "flat")) {
    prior <- prior_weak()
  }

  if (!inherits(prior, "rtmb_prior")) {
    stop(
      "prior must be an object of class 'rtmb_prior'. ",
      "Use prior_flat(), prior_normal(), prior_weak(), prior_rhs(), or prior_ssp().",
      call. = FALSE
    )
  }

  prior_type <- prior$type

  is_flat <- identical(prior_type, "flat")
  is_normal <- identical(prior_type, "normal")
  is_weak <- identical(prior_type, "weak")
  is_jzs <- identical(prior_type, "jzs")

  regularization <- if (prior_type %in% c("rhs", "ssp")) prior_type else "none"

  use_centering <- prior_type %in% c("normal", "weak", "rhs", "ssp", "jzs")
  use_weak_info <- prior_type %in% c("weak", "rhs", "ssp", "jzs")
  use_normal_prior <- identical(prior_type, "normal")

  if (use_normal_prior) {
    if (is.null(prior$mu_sd) && !is.null(prior$Intercept_sd)) {
      prior$mu_sd <- prior$Intercept_sd
    }
  }

  has_random <- !is.null(findbars(formula))

  # If classic mode and random effects are present (or forced), we internally
  # force 'sum' contrasts for stable Type III ANOVA/DFs.
  # Otherwise, we use the user-requested contrast (defaulting to 'treatment').
  actual_contrasts <- if (.force_sum) "sum" else contrasts

  if (!is.null(actual_contrasts)) {
    if (actual_contrasts == "sum") {
      old_opts <- options(contrasts = c("contr.sum", "contr.poly"))
      on.exit(options(old_opts), add = TRUE)
    } else if (actual_contrasts == "treatment") {
      old_opts <- options(contrasts = c("contr.treatment", "contr.poly"))
      on.exit(options(old_opts), add = TRUE)
    }
  }

  if (use_weak_info) {
    if (is.null(prior$max_beta)) prior$max_beta <- 1.0
    if (is.null(prior$sd_ratio)) prior$sd_ratio <- 0.5
    if (is.null(prior$expected_vars)) prior$expected_vars <- 3
    if (is.null(prior$slab_scale)) prior$slab_scale <- 2.0
    if (is.null(prior$slab_df)) prior$slab_df <- 4.0
    if (is.null(prior$ssp_ratio)) prior$ssp_ratio <- 0.25
  }

  families_requiring_yrange <- c("gaussian", "lognormal", "student_t")
  is_continuous <- family %in% families_requiring_yrange

  if (use_weak_info && is_continuous && is.null(y_range) && prior_type != "jzs") {
    stop(sprintf("Specifying 'y_range' is required when using prior_%s() with family = '%s'.", prior_type, family))
  }

  glmer_terms <- make_glmer_re_terms(
    formula = formula,
    data = data,
    family = family,
    resid_group = resid_group,
    resid_time = resid_time,
    within = within,
    factors = factors
  )
  Y <- glmer_terms$Y
  X <- glmer_terms$X
  trials <- glmer_terms$trials
  X_assign <- glmer_terms$X_assign
  X_terms <- glmer_terms$X_terms
  offset <- glmer_terms$offset
  N <- glmer_terms$N
  mf <- glmer_terms$mf
  fixed_colnames <- glmer_terms$fixed_colnames
  fixed_names <- glmer_terms$fixed_names
  has_intercept <- glmer_terms$has_intercept
  ordered_num_categories <- glmer_terms$num_categories
  bars <- glmer_terms$random$bars
  num_bars <- glmer_terms$random$num_bars
  suffix <- function(b) if (num_bars > 1) paste0("_", b) else ""
  num_ranef_list <- glmer_terms$random$num_ranef_list
  num_groups_list <- glmer_terms$random$num_groups_list
  ranef_names_list <- glmer_terms$random$ranef_names_list
  group_labels_list <- glmer_terms$random$group_labels_list
  
  jzs_V_val <- NULL
  K_tmp <- ncol(X)
  sequential_transitions_tmp <- if (identical(family, "sequential")) ordered_num_categories - 1 else 1

  # --- Robust SE Weights ---
  dat_weights <- list(robust_obs_weight = rep(1, N))
  if (has_random && num_bars > 0) {
    for (b in 1:num_bars) {
      weight_name <- paste0("robust_re_weight", suffix(b))
      dat_weights[[weight_name]] <- rep(1, num_groups_list[[b]])
    }
  }

  # --- Automatic generation of initial values for SSP ---
  if (regularization == "ssp" && is.null(init) && K_tmp > 0) {
    rhs_mod <- NULL
    suppressMessages(capture.output({
      rhs_prior_obj <- prior_rhs(expected_vars = prior$expected_vars, slab_scale = prior$slab_scale, slab_df = prior$slab_df)
      rhs_mod <- rtmb_glmer(formula, data, family, laplace, prior = rhs_prior_obj, y_range = y_range, init = NULL)
    }))

    jac_target <- if (laplace) "random" else "none"

    ad_setup <- NULL
    suppressMessages(capture.output({
      ad_setup <- rhs_mod$build_ad_obj(laplace = laplace, jacobian_target = jac_target)
    }))

    ad_obj <- ad_setup$ad_obj
    best_obj <- Inf
    best_par <- ad_obj$par

    for (i in 1:4) {
      start_par <- if (i == 1) ad_obj$par else ad_obj$par + rnorm(length(ad_obj$par), 0, 0.5)
      opt <- suppressWarnings(try(nlminb(start = start_par, objective = ad_obj$fn, gradient = ad_obj$gr), silent = TRUE))
      if (!inherits(opt, "try-error") && !is.na(opt$objective) && opt$objective < best_obj) {
        best_obj <- opt$objective
        best_par <- opt$par
      }
    }

    ad_obj$fn(best_par)
    full_par <- if (!is.null(ad_obj$env$last.par.best)) ad_obj$env$last.par.best else ad_obj$env$last.par

    unc_est_list <- unconstrained_vector_to_list(full_par, rhs_mod$par_list)
    con_est_list <- to_constrained(unc_est_list, rhs_mod$par_list)
    if (!is.null(rhs_mod$transform)) {
      tran_res <- rhs_mod$transform(rhs_mod$data, con_est_list)
      con_est_list <- c(con_est_list, tran_res)
    }

    b_est <- con_est_list$b
    if (is.null(b_est) || length(b_est) == 0 || any(is.na(as.numeric(b_est)))) {
      b_est <- if (identical(family, "sequential")) matrix(0, K_tmp, sequential_transitions_tmp) else rep(0, K_tmp)
    } else if (identical(family, "sequential")) {
      b_est <- matrix(as.numeric(b_est), K_tmp, sequential_transitions_tmp)
    } else {
      b_est <- as.numeric(b_est)
    }

    r_init <- ifelse(abs(b_est) > mean(abs(b_est)), 0.9, 0.1)
    tau_init <- pmax(abs(b_est), 0.01)
    beta_raw_init <- b_est / (r_init * tau_init)

    init <- list(beta_raw = beta_raw_init, r = r_init, tau = tau_init)
    if (has_intercept && !is.null(con_est_list$Intercept_c)) {
      init$Intercept_c <- as.numeric(con_est_list$Intercept_c)[1]
    }
  } else if (is.null(init)) {
    tryCatch({
      fixed_formula <- nobars(formula)
      glm_fam <- switch(family, "gaussian" = gaussian(), "student_t" = gaussian(), "lognormal" = gaussian(link="log"), "bernoulli" = binomial(), "binomial" = binomial(), "poisson" = poisson(), "neg_binomial" = poisson(), "gamma" = Gamma(link="log"), gaussian())
      init_fit <- suppressWarnings(glm(fixed_formula, data = data, family = glm_fam))
      init_coef <- unname(coef(init_fit))
      init_coef[is.na(init_coef)] <- 0

      init <- list()
      if (has_intercept) {
        if (use_centering) {
          init$Intercept_c <- init_coef[1]
        } else {
          init$Intercept <- init_coef[1]
        }
      }

      if (regularization == "none" && K_tmp > 0) {
        b_init <- if (has_intercept) init_coef[-1] else init_coef
        if (family == "sequential") {
          init$b <- matrix(b_init, K_tmp, sequential_transitions_tmp)
        } else {
          init$b <- b_init
        }
      }

      # Initial values for residual correlation
      if (!is.null(resid_corr)) {
        if (resid_corr %in% c("ar1", "cs")) init$rho_resid <- 0.1
        else if (resid_corr == "toep") init$rho_resid <- rep(0.1, dat$max_T_resid - 1)
        else if (resid_corr == "un") init$L_resid <- diag(dat$max_T_resid)
      }

      if (family %in% c("gaussian", "lognormal", "student_t")) {
        init$sigma <- if (num_sigma_groups > 1) rep(stats::sd(Y) * 0.5, num_sigma_groups) else stats::sd(Y) * 0.5
      }

    }, error = function(e) init <<- NULL)
  }

  setup_df <- .select_glmer_setup_data(formula, data, resid_group = resid_group, resid_time = resid_time)
  class(setup_df) <- c("rtmb_setup_df", class(setup_df))
  dat <- list(df = setup_df, formula = formula,
              sigma_idx = sigma_idx, num_sigma_groups = num_sigma_groups)
  if (has_random && !identical(family, "gaussian")) dat$family_name <- family
  if (has_random && !is.null(resid_group)) dat$resid_group_name <- resid_group
  if (has_random && !is.null(resid_time)) dat$resid_time_name <- resid_time
  if (has_random && !is.null(within)) dat$within_info <- within
  if (has_random && !is.null(factors)) dat$factors_info <- factors

  if (!is.null(resid_corr)) {
    if (family != "gaussian") stop("Residual correlation structures are currently only supported for Gaussian models.")
    resid_corr <- tolower(resid_corr)
    if (!resid_corr %in% c("ar1", "cs", "un", "toep")) stop("Currently 'ar1', 'cs', 'un', and 'toep' are supported for resid_corr.")

    # Identify group for residuals
    if (is.null(resid_group)) {
      if (has_random) {
        resid_group_var <- as.character(bars[[1]][[3]])
      } else {
        stop("Residual correlation requires 'resid_group' (e.g., resid_group = 'ID') if no random effects are specified.")
      }
    } else {
      resid_group_var <- resid_group
    }

    if (!(resid_group_var %in% names(mf))) stop(sprintf("Residual grouping variable '%s' not found in data.", resid_group_var))

    group_resid <- as.numeric(as.factor(mf[[resid_group_var]]))
    dat$group_resid <- group_resid
    dat$num_groups_resid <- max(group_resid)
    if (!is.null(resid_time)) {
      dat$time_resid <- as.numeric(as.factor(mf[[resid_time]])) - 1
    } else {
      # Use row order within group
      dat$time_resid <- ave(rep(1, N), group_resid, FUN = seq_along) - 1
    }
    dat$max_T_resid <- max(table(group_resid))
  }

  if (!is.null(sigma_idx)) {
    dat$sigma_idx <- sigma_idx
    dat$num_sigma_groups <- num_sigma_groups
  }

  # --- Setup AST ---
  setup_exprs <- list()
  Y_setup_raw <- model.response(mf)
  
  if (is.character(Y_setup_raw)) {
    stop("The response variable must be numeric (or a factor for classification models). Character variables are not supported.", call. = FALSE)
  }
  
  if (is.factor(Y_setup_raw) && !(family %in% c("binomial", "bernoulli", "ordered", "sequential"))) {
    stop(sprintf("The response variable for family '%s' must be numeric. Factor variables are not supported.", family), call. = FALSE)
  }
  if (has_random) {
    glmer_re_call_args <- list(formula = as.name("formula"), data = as.name("df"))
    if (!identical(family, "gaussian")) glmer_re_call_args$family <- as.name("family_name")
    if (!is.null(resid_group)) glmer_re_call_args$resid_group <- as.name("resid_group_name")
    if (!is.null(resid_time)) glmer_re_call_args$resid_time <- as.name("resid_time_name")
    if (!is.null(within)) glmer_re_call_args$within <- as.name("within_info")
    if (!is.null(factors)) glmer_re_call_args$factors <- as.name("factors_info")
    glmer_re_call_args$missing <- missing
    setup_exprs[[length(setup_exprs) + 1]] <- as.call(list(as.name("<-"), as.name("res"), as.call(c(list(as.name("make_glmer_re_terms")), glmer_re_call_args))))
    setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- res$Y)
    if (family == "binomial") {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(trials <- res$trials)
    }
    setup_exprs[[length(setup_exprs) + 1]] <- quote(X <- res$X)
    if (!is.null(offset)) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(offset <- res$offset)
    }
    setup_exprs[[length(setup_exprs) + 1]] <- quote(N <- res$N)
    setup_exprs[[length(setup_exprs) + 1]] <- quote(K <- ncol(X))
    if (family %in% c("ordered", "sequential")) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(num_categories <- res$num_categories)
    }
    setup_exprs[[length(setup_exprs) + 1]] <- "# Random-effect design"
    for (b in 1:num_bars) {
      group_idx_name <- as.name(paste0("group_idx", suffix(b)))
      Z_mat_name <- as.name(paste0("Z_mat", suffix(b)))
      num_groups_name <- as.name(paste0("num_groups", suffix(b)))
      num_ranef_name <- as.name(paste0("num_ranef", suffix(b)))

      setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(group_idx_name) <- res$random$terms[[.(b)]]$group_idx)
      setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(Z_mat_name) <- res$random$terms[[.(b)]]$Z)
      setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(num_groups_name) <- res$random$terms[[.(b)]]$num_groups)
      setup_exprs[[length(setup_exprs) + 1]] <- bquote(.(num_ranef_name) <- res$random$terms[[.(b)]]$num_ranef)
    }
  } else {
    na_action_expr <- if (missing == "listwise") quote(na.omit) else quote(na.pass)
    setup_exprs[[length(setup_exprs) + 1]] <- bquote(na_act <- .(na_action_expr))
    setup_exprs[[length(setup_exprs) + 1]] <- quote(mf <- model.frame(formula, df, na.action = na_act))
    setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- model.response(mf))
    if (is.matrix(Y_setup_raw)) {
      if (ncol(Y_setup_raw) == 2 && family == "binomial") {
        setup_exprs[[length(setup_exprs) + 1]] <- quote(trials <- as.numeric(Y[, 1] + Y[, 2]))
        setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- as.numeric(Y[, 1]))
      } else if (ncol(Y_setup_raw) == 1) {
        setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- as.numeric(Y[, 1]))
        if (family == "binomial") {
          setup_exprs[[length(setup_exprs) + 1]] <- quote(trials <- rep(1, length(Y)))
        }
      }
    } else {
      if (is.factor(Y_setup_raw)) {
        if (family %in% c("bernoulli", "binomial")) {
          setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- as.numeric(Y) - 1)
        } else {
          setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- as.numeric(Y))
        }
      } else if (!is.numeric(Y_setup_raw)) {
        setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- as.numeric(Y))
      }
      if (family == "binomial") {
        setup_exprs[[length(setup_exprs) + 1]] <- quote(trials <- rep(1, length(Y)))
      }
    }
    if (family %in% c("ordered", "sequential")) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(Y <- as.integer(round(Y)))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(num_categories <- max(Y))
    }
    if ("(Intercept)" %in% fixed_colnames) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_full <- model.matrix(formula, mf))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X <- X_full[, colnames(X_full) != "(Intercept)", drop = FALSE])
    } else {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X <- model.matrix(formula, mf))
    }
    if (!is.null(offset)) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(offset <- model.offset(mf))
    }
    setup_exprs[[length(setup_exprs) + 1]] <- quote(N <- length(Y))
    setup_exprs[[length(setup_exprs) + 1]] <- quote(K <- ncol(X))
  }

  if (use_weak_info) {
    setup_exprs[[length(setup_exprs) + 1]] <- "# Prior setting"
    if (family %in% c("bernoulli", "binomial", "ordered", "sequential")) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(base_scale <- pi / sqrt(3))
      setup_exprs[[length(setup_exprs) + 1]] <- bquote(alpha_prior_sd <- base_scale * .(prior$max_beta))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(mid_y <- 0)
    } else if (family %in% c("poisson", "neg_binomial", "gamma")) {
      # For Poisson/log-link models, Stan often uses Normal(log(mean(y)), 2.5 or 5) for intercept
      # and Normal(0, 2.5) for coefficients.
      setup_exprs[[length(setup_exprs) + 1]] <- quote(mid_y <- log(mean(Y) + 0.5))
      setup_exprs[[length(setup_exprs) + 1]] <- bquote(alpha_prior_sd <- .(prior$sd_ratio))
      setup_exprs[[length(setup_exprs) + 1]] <- quote(base_scale <- 1.0)
    } else {
      if (prior_type == "jzs") {
         # For JZS, we use defaults if y_range is missing
         if (!is.null(y_range)) {
           setup_exprs[[length(setup_exprs) + 1]] <- quote(half_d_y <- diff(y_range) / 2)
           setup_exprs[[length(setup_exprs) + 1]] <- bquote(base_scale <- half_d_y * .(prior$sd_ratio))
           setup_exprs[[length(setup_exprs) + 1]] <- quote(alpha_prior_sd <- half_d_y)
           setup_exprs[[length(setup_exprs) + 1]] <- quote(mid_y <- mean(y_range))
         } else {
           # JZS defaults for alpha/sigma part if y_range is missing (though we use flat priors anyway)
           setup_exprs[[length(setup_exprs) + 1]] <- quote(base_scale <- 1.0)
           setup_exprs[[length(setup_exprs) + 1]] <- quote(alpha_prior_sd <- 10.0)
           setup_exprs[[length(setup_exprs) + 1]] <- quote(mid_y <- 0)
         }
         setup_exprs[[length(setup_exprs) + 1]] <- quote(sigma_rate <- 1.0 / base_scale)
         setup_exprs[[length(setup_exprs) + 1]] <- bquote(jzs_r <- .(prior$r))
      } else {
         setup_exprs[[length(setup_exprs) + 1]] <- quote(half_d_y <- diff(y_range) / 2)
         setup_exprs[[length(setup_exprs) + 1]] <- bquote(base_scale <- half_d_y * .(prior$sd_ratio))
         setup_exprs[[length(setup_exprs) + 1]] <- quote(alpha_prior_sd <- half_d_y)
         setup_exprs[[length(setup_exprs) + 1]] <- quote(mid_y <- mean(y_range))
         setup_exprs[[length(setup_exprs) + 1]] <- quote(sigma_rate <- 1.0 / base_scale)
      }
    }

    setup_exprs[[length(setup_exprs) + 1]] <- quote(tau_rate <- 1.0 / base_scale)

    if (K_tmp > 0) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_sd <- apply(X, 2, sd))
      if (family == "sequential") {
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(beta_prior_sd <- matrix(.(prior$max_beta) * base_scale / X_sd, K, num_categories - 1))
      } else {
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(beta_prior_sd <- .(prior$max_beta) * base_scale / X_sd)
      }
      
      if (prior_type == "jzs") {
        # Calculate (X_c' X_c / N)^-1 for multivariate JZS
        # X_c is centered X.
        X_mean_val <- apply(X, 2, mean)
        X_c_val <- X - rep(1, N) %*% t(X_mean_val)
        
        X_for_jzs <- X_c_val
        
        if (ncol(X_for_jzs) > 0) {
          # Calculate (X'X/N)^-1 with numerical safety
          XtX_inv <- solve(t(X_for_jzs) %*% X_for_jzs / N)
          # Force symmetry and add a tiny jitter to ensure PD
          jzs_V_val <- 0.5 * (XtX_inv + t(XtX_inv))
          diag(jzs_V_val) <- diag(jzs_V_val) + 1e-10
          
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(jzs_V <- .(jzs_V_val))
        } else {
          # Null model case: no predictors for JZS
          setup_exprs[[length(setup_exprs) + 1]] <- quote(jzs_V <- matrix(0, 0, 0))
        }
      }

      if (regularization == "rhs") {
        if (family == "sequential") {
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(p0 <- min(.(prior$expected_vars), K * (num_categories - 1) - 1))
          setup_exprs[[length(setup_exprs) + 1]] <- quote(K_eff <- K * (num_categories - 1))
        } else {
          setup_exprs[[length(setup_exprs) + 1]] <- bquote(p0 <- min(.(prior$expected_vars), K - 1))
          setup_exprs[[length(setup_exprs) + 1]] <- quote(K_eff <- K)
        }
        setup_exprs[[length(setup_exprs) + 1]] <- quote(if (p0 < 1) p0 <- 1)
        setup_exprs[[length(setup_exprs) + 1]] <- quote(tau0 <- (p0 / (K_eff - p0)) * (base_scale / sqrt(N)))
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(half_slab_df <- .(prior$slab_df) / 2.0)
        setup_exprs[[length(setup_exprs) + 1]] <- bquote(half_slab_scale2 <- (.(prior$slab_df) * .(prior$slab_scale)^2) / 2.0)
      } else if (regularization == "ssp") {
        if (family == "sequential") {
          setup_exprs[[length(setup_exprs) + 1]] <- quote(tau_scale <- matrix(base_scale / X_sd, K, num_categories - 1))
        } else {
          setup_exprs[[length(setup_exprs) + 1]] <- quote(tau_scale <- base_scale / X_sd)
        }
      }
    }
  }

  if (use_centering && K_tmp > 0) {
    setup_exprs[[length(setup_exprs) + 1]] <- "# Centering"
    setup_exprs[[length(setup_exprs) + 1]] <- quote(X_mean <- apply(X, 2, mean))
    if (has_intercept) {
      setup_exprs[[length(setup_exprs) + 1]] <- quote(X_c <- X - rep(1, N) %*% t(X_mean))
    }
  }
  setup_ast <- as.call(c(list(as.name("{")), setup_exprs))

  tmp_env <- list2env(dat)
  eval(setup_ast, tmp_env)
  N <- tmp_env$N; K <- tmp_env$K
  num_categories <- tmp_env$num_categories
  num_sigma_groups <- if (!is.null(tmp_env$num_sigma_groups)) tmp_env$num_sigma_groups else 1

  # --- Parameters AST ---
  param_exprs <- list()
  if (has_intercept) {
    if (use_centering) {
      param_exprs[[length(param_exprs) + 1]] <- quote(Intercept_c <- Dim(1))
    } else {
      param_exprs[[length(param_exprs) + 1]] <- quote(Intercept <- Dim(1))
    }
  }
  if (K > 0) {
    if (prior_type == "jzs") {
      if (family == "sequential") param_exprs[[length(param_exprs) + 1]] <- quote(b <- Dim(c(K, num_categories - 1)))
      else param_exprs[[length(param_exprs) + 1]] <- quote(b <- Dim(K))
    } else if (regularization == "none") {
      if (family == "sequential") param_exprs[[length(param_exprs) + 1]] <- quote(b <- Dim(c(K, num_categories - 1)))
      else param_exprs[[length(param_exprs) + 1]] <- quote(b <- Dim(K))
    } else if (regularization == "rhs") {
      if (family == "sequential") {
        param_exprs[[length(param_exprs) + 1]] <- quote(z <- Dim(c(K, num_categories - 1)))
        param_exprs[[length(param_exprs) + 1]] <- quote(lambda <- Dim(c(K, num_categories - 1), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- quote(w_lambda <- Dim(c(K, num_categories - 1), lower = 0))
      } else {
        param_exprs[[length(param_exprs) + 1]] <- quote(z <- Dim(K))
        param_exprs[[length(param_exprs) + 1]] <- quote(lambda <- Dim(K, lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- quote(w_lambda <- Dim(K, lower = 0))
      }
      param_exprs[[length(param_exprs) + 1]] <- quote(tau_hs <- Dim(1, lower = 0))
      param_exprs[[length(param_exprs) + 1]] <- quote(w_tau <- Dim(1, lower = 0))
      param_exprs[[length(param_exprs) + 1]] <- quote(c2 <- Dim(1, lower = 0))
    } else if (regularization == "ssp") {
      if (family == "sequential") {
        param_exprs[[length(param_exprs) + 1]] <- quote(beta_raw <- Dim(c(K, num_categories - 1)))
        param_exprs[[length(param_exprs) + 1]] <- quote(r <- Dim(c(K, num_categories - 1), lower = 0.001, upper = 0.999))
        param_exprs[[length(param_exprs) + 1]] <- quote(tau <- Dim(c(K, num_categories - 1), lower = 0))
      } else {
        param_exprs[[length(param_exprs) + 1]] <- quote(beta_raw <- Dim(K))
        param_exprs[[length(param_exprs) + 1]] <- quote(r <- Dim(K, lower = 0.001, upper = 0.999))
        param_exprs[[length(param_exprs) + 1]] <- quote(tau <- Dim(K, lower = 0))
      }
    }
  }

  if (has_random) {
    for (b in 1:num_bars) {
      num_ranef_val <- num_ranef_list[[b]]
      num_groups_val <- num_groups_list[[b]]
      sd_name <- as.name(paste0("sd", suffix(b)))
      r_re_name <- as.name(paste0("r_re", suffix(b)))
      CF_corr_name <- as.name(paste0("CF_corr", suffix(b)))

      if (num_ranef_val == 1) {
        param_exprs[[length(param_exprs) + 1]] <- bquote(.(sd_name) <- Dim(.(num_ranef_val), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(.(r_re_name) <- Dim(.(num_groups_val), random = TRUE))
      } else {
        param_exprs[[length(param_exprs) + 1]] <- bquote(.(sd_name) <- Dim(.(num_ranef_val), lower = 0))
        param_exprs[[length(param_exprs) + 1]] <- bquote(.(CF_corr_name) <- Dim(c(.(num_ranef_val), .(num_ranef_val)), type = "CF_corr"))
        param_exprs[[length(param_exprs) + 1]] <- bquote(.(r_re_name) <- Dim(c(.(num_groups_val), .(num_ranef_val)), random = TRUE))
      }
    }
  }

  if (family %in% c("gaussian", "lognormal", "student_t")) param_exprs[[length(param_exprs) + 1]] <- quote(sigma <- Dim(num_sigma_groups, lower = 0))
  if (!is.null(resid_corr)) {
    if (resid_corr %in% c("ar1", "cs")) {
      param_exprs[[length(param_exprs) + 1]] <- quote(rho_resid <- Dim(type = "interval", lower = -0.99, upper = 0.99))
    } else if (resid_corr == "un") {
      param_exprs[[length(param_exprs) + 1]] <- bquote(L_resid <- Dim(c(.(dat$max_T_resid), .(dat$max_T_resid)), type = "CF_corr"))
    } else if (resid_corr == "toep") {
      param_exprs[[length(param_exprs) + 1]] <- bquote(rho_resid <- Dim(.(dat$max_T_resid - 1), type = "interval", lower = -0.99, upper = 0.99))
    }
  }
  if (family == "student_t") param_exprs[[length(param_exprs) + 1]] <- quote(nu <- Dim(1, lower = 2))
  if (family == "gamma") param_exprs[[length(param_exprs) + 1]] <- quote(shape <- Dim(1, lower = 0))
  if (family == "neg_binomial") param_exprs[[length(param_exprs) + 1]] <- quote(phi <- Dim(1, lower = 0))
  if (family == "ordered") {
    param_exprs[[length(param_exprs) + 1]] <- quote(cutpoints <- Dim(num_categories - 1, type = "ordered"))
  }
  if (family == "sequential") {
    param_exprs[[length(param_exprs) + 1]] <- quote(cutpoints <- Dim(num_categories - 1))
  }
  param_ast <- as.call(c(list(as.name("{")), param_exprs))

  # --- Transform AST ---
  tran_exprs <- list()
  if (K > 0) {
    if (regularization == "rhs") {
      tran_exprs[[length(tran_exprs) + 1]] <- quote(lambda_sq <- lambda^2)
      tran_exprs[[length(tran_exprs) + 1]] <- quote(tau_sq <- tau_hs^2)
      tran_exprs[[length(tran_exprs) + 1]] <- quote(lambda_tilde <- sqrt((c2 * lambda_sq) / (c2 + tau_sq * lambda_sq)))
      tran_exprs[[length(tran_exprs) + 1]] <- quote(b <- z * lambda_tilde * tau_hs)
    } else if (regularization == "ssp") {
      tran_exprs[[length(tran_exprs) + 1]] <- quote(b <- beta_raw * r * tau)
    }
  }
  if (has_intercept) {
    if (use_centering) {
      if (K > 0) tran_exprs[[length(tran_exprs) + 1]] <- quote(Intercept <- Intercept_c - sum(X_mean * b))
      else tran_exprs[[length(tran_exprs) + 1]] <- quote(Intercept <- Intercept_c)
    }
  }
  if (has_random) {
    for (b in 1:num_bars) {
      if (num_ranef_list[[b]] > 1) {
        CF_corr_name <- as.name(paste0("CF_corr", suffix(b)))
        corr_name <- as.name(paste0("corr", suffix(b)))
        tran_exprs[[length(tran_exprs) + 1]] <- bquote(.(corr_name) <- .(CF_corr_name) %*% t(.(CF_corr_name)))
      }
    }
  }
      tran_ast <- if (length(tran_exprs) > 0) as.call(c(list(as.name("{")), tran_exprs)) else NULL

  # --- Model AST ---
  transform_exprs <- list()
  if (family == "sequential") {
    if (K > 0) transform_exprs[[1]] <- quote(eta <- X %*% b)
    else transform_exprs[[1]] <- quote(eta <- matrix(0, N, num_categories - 1))
  } else if (has_intercept) {
    if (use_centering) {
      if (K > 0) transform_exprs[[1]] <- quote(eta <- Intercept_c + X_c %*% b)
      else transform_exprs[[1]] <- quote(eta <- rep(Intercept_c, N))
    } else {
      if (K > 0) transform_exprs[[1]] <- quote(eta <- Intercept + X %*% b)
      else transform_exprs[[1]] <- quote(eta <- rep(Intercept, N))
    }
  } else {
    if (K > 0) transform_exprs[[1]] <- quote(eta <- X %*% b)
    else transform_exprs[[1]] <- quote(eta <- rep(0, N))
  }

  if (has_random) {
    for (b in 1:num_bars) {
      Z_mat_name <- as.name(paste0("Z_mat", suffix(b)))
      r_re_name <- as.name(paste0("r_re", suffix(b)))
      group_idx_name <- as.name(paste0("group_idx", suffix(b)))
      sd_name <- as.name(paste0("sd", suffix(b)))

      if (num_ranef_list[[b]] > 1) {
        if (family == "sequential") {
          transform_exprs[[length(transform_exprs) + 1]] <- bquote(
            for (i in 1:N) {
              for (k_seq in 1:(num_categories - 1)) {
                eta[i, k_seq] <- eta[i, k_seq] + sum(.(Z_mat_name)[i, ] * .(r_re_name)[.(group_idx_name)[i], ] * .(sd_name))
              }
            }
          )
        } else {
          transform_exprs[[length(transform_exprs) + 1]] <- bquote(
            for (i in 1:N) {
              eta[i] <- eta[i] + sum(.(Z_mat_name)[i, ] * .(r_re_name)[.(group_idx_name)[i], ] * .(sd_name))
            }
          )
        }
      } else {
        if (family == "sequential") {
          transform_exprs[[length(transform_exprs) + 1]] <- bquote(
            for (k_seq in 1:(num_categories - 1)) {
              eta[, k_seq] <- eta[, k_seq] + .(Z_mat_name)[, 1] * .(r_re_name)[.(group_idx_name)] * .(sd_name)
            }
          )
        } else {
          transform_exprs[[length(transform_exprs) + 1]] <- bquote(eta <- eta + .(Z_mat_name)[, 1] * .(r_re_name)[.(group_idx_name)] * .(sd_name))
        }
      }
    }
  }
  if (!is.null(offset)) {
    if (family == "sequential") {
      transform_exprs[[length(transform_exprs) + 1]] <- quote(
        for (k_seq in 1:(num_categories - 1)) {
          eta[, k_seq] <- eta[, k_seq] + offset
        }
      )
    } else {
      transform_exprs[[length(transform_exprs) + 1]] <- quote(eta <- eta + offset)
    }
  }

  ll_data_exprs <- list()
  if (!is.null(resid_corr)) {
    vi_inner_expr <- switch(resid_corr,
      "ar1" = quote(Vi[r,c] <- s_r * s_c * (rho_resid^abs(ti[r] - ti[c]))),
      "cs"  = quote(if (r == c) Vi[r,c] <- s_r^2 else Vi[r,c] <- s_r * s_c * rho_resid),
      "toep"= quote({
               lag <- abs(ti[r] - ti[c])
               if (lag == 0) Vi[r,c] <- s_r * s_c
               else if (lag <= length(rho_resid)) Vi[r,c] <- s_r * s_c * rho_resid[lag]
               else Vi[r,c] <- 0
            }),
      "un" = quote({
               if (r == c) Vi[r,c] <- s_r^2
               else {
                 idx_r <- ti[r] + 1
                 idx_c <- ti[c] + 1
                 Vi[r,c] <- s_r * s_c * R_mat_resid[idx_r, idx_c]
               }
            })
    )

    # Build model components list
    ll_data_exprs <- list()
    ll_data_exprs[[length(ll_data_exprs) + 1]] <- quote(has_sig_idx <- !is.null(sigma_idx))
    if (resid_corr == "un") {
      ll_data_exprs[[length(ll_data_exprs) + 1]] <- quote(R_mat_resid <- L_resid %*% t(L_resid))
    }

    ll_data_exprs[[length(ll_data_exprs) + 1]] <- bquote(
      for (i in 1:num_groups_resid) {
        idx <- which(group_resid == i)
        ni <- length(idx)
        ti <- time_resid[idx]
        mu_i <- eta[idx]

        Vi <- matrix(0, ni, ni)
        for (r in 1:ni) {
          s_r <- if (has_sig_idx) sigma[sigma_idx[idx[r]]] else sigma
          for (c in 1:ni) {
            s_c <- if (has_sig_idx) sigma[sigma_idx[idx[c]]] else sigma
            .(vi_inner_expr)
          }
        }
        diag(Vi) <- diag(Vi) + 1e-3
        Y[idx] ~ multi_normal(mu_i, Vi)
      }
    )
  } else {
    ll_data_exprs[[1]] <- switch(family,
                                 "gaussian" = if (!is.null(sigma_idx)) quote(Y ~ normal(eta, sigma[sigma_idx])) else quote(Y ~ normal(eta, sigma)),
                                 "lognormal" = if (!is.null(sigma_idx)) quote(Y ~ lognormal(eta, sigma[sigma_idx])) else quote(Y ~ lognormal(eta, sigma)),
                                 "student_t" = if (!is.null(sigma_idx)) quote(Y ~ student_t(nu, eta, sigma[sigma_idx])) else quote(Y ~ student_t(nu, eta, sigma)),
                                 "gamma" = quote(Y ~ gamma(shape, shape / exp(eta))),
                                 "bernoulli" = quote(Y ~ bernoulli_logit(eta)),
                                 "binomial" = quote(Y ~ binomial_logit(trials, eta)),
                                 "poisson" = quote(Y ~ poisson(exp(eta))),
                                 "neg_binomial" = quote(Y ~ neg_binomial_2(exp(eta), phi)),
                                 "ordered" = quote(Y ~ ordered_logistic(eta, cutpoints)),
                                 "sequential" = quote(Y ~ sequential_logistic(eta, cutpoints))
    )
  }

  ll_random_exprs <- list()
  if (has_random) {
    for (b in 1:num_bars) {
        num_groups_val <- num_groups_list[[b]]
        num_ranef_val <- num_ranef_list[[b]]
        if (num_ranef_val == 1) {
          r_re_name <- as.name(paste0("r_re", suffix(b)))
          ll_random_exprs[[length(ll_random_exprs) + 1]] <- bquote(.(r_re_name) ~ normal(0, 1))
        } else {
          r_re_name <- as.name(paste0("r_re", suffix(b)))
          CF_corr_name <- as.name(paste0("CF_corr", suffix(b)))
          if (!is_flat && !is.null(prior$lkj_eta)) {
            ll_random_exprs[[length(ll_random_exprs) + 1]] <- bquote(.(CF_corr_name) ~ lkj_CF_corr(.(prior$lkj_eta)))
          }
          ll_random_exprs[[length(ll_random_exprs) + 1]] <- bquote(for (j in 1:.(num_groups_val)) .(r_re_name)[j, ] ~ multi_normal_CF(rep(0, .(num_ranef_val)), rep(1, .(num_ranef_val)), .(CF_corr_name)))
        }
    }
  }

  prior_exprs <- list()
  if (prior_type %in% c("weak", "rhs", "ssp", "jzs")) {
    if (family %in% c("gaussian", "lognormal", "student_t")) {
      if (use_weak_info && prior_type == "weak") {
        prior_exprs[[length(prior_exprs) + 1]] <- quote(sigma ~ exponential(sigma_rate))
      } else if (prior_type == "jzs") {
        # Jeffreys prior for variance (p(sigma) proportional to 1/sigma)
        if (num_sigma_groups > 1) {
          prior_exprs[[length(prior_exprs) + 1]] <- quote(lp <- lp - sum(log(sigma)))
        } else {
          prior_exprs[[length(prior_exprs) + 1]] <- quote(lp <- lp - log(sigma))
        }
      } else if (!is.null(prior$sigma_rate)) {
        prior_exprs[[length(prior_exprs) + 1]] <- bquote(sigma ~ exponential(.(prior$sigma_rate)))
      }
    }

    if (family == "student_t" && !is.null(prior$nu_rate)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(nu ~ exponential(.(prior$nu_rate)))
    if (family == "gamma" && !is.null(prior$shape_rate)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(shape ~ exponential(.(prior$shape_rate)))
    if (family == "neg_binomial" && !is.null(prior$phi_rate)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(phi ~ exponential(.(prior$phi_rate)))
    if (family %in% c("ordered", "sequential") && !is.null(prior$cutpoint_sd)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(cutpoints ~ normal(0, .(prior$cutpoint_sd)))

    if (has_intercept) {
      if (use_weak_info && prior_type == "weak") {
        prior_exprs[[length(prior_exprs) + 1]] <- quote(Intercept_c ~ normal(mid_y, alpha_prior_sd))
      } else if (prior_type == "jzs") {
        # Flat prior for intercept (p(alpha) proportional to 1) - no term added
        prior_exprs[[length(prior_exprs) + 1]] <- quote("# Flat prior for Intercept (JZS)")
      } else if (!is.null(prior$Intercept_sd)) {
        prior_exprs[[length(prior_exprs) + 1]] <- bquote(Intercept_c ~ normal(0, .(prior$Intercept_sd)))
      }
    }

    if (K > 0) {
      if (prior_type == "jzs") {
        # Multivariate JZS (Single-g prior): b ~ Multivariate-Cauchy(0, r^2 * sigma^2 * (X'X/N)^-1)
        # This matches the BayesFactor package and handles correlations correctly.
        # We use bquote to embed the pre-calculated dimension.
        if (!is.null(jzs_V_val) && ncol(jzs_V_val) > 0) {
          K_jzs <- ncol(jzs_V_val)
          prior_exprs[[length(prior_exprs) + 1]] <- bquote(b ~ multi_cauchy(rep(0, .(K_jzs)), (jzs_r * sigma)^2 * jzs_V))
        }
      } else if (regularization == "none") {
        if (use_weak_info && prior_type == "weak") {
          prior_exprs[[length(prior_exprs) + 1]] <- quote(b ~ normal(0, beta_prior_sd))
        } else if (!is.null(prior$b_sd)) {
          prior_exprs[[length(prior_exprs) + 1]] <- bquote(b ~ normal(0, .(prior$b_sd)))
        }
      } else if (regularization == "rhs") {
        prior_exprs[[length(prior_exprs) + 1]] <- quote(z ~ normal(0, 1))
        prior_exprs[[length(prior_exprs) + 1]] <- quote(w_lambda ~ gamma(0.5, 0.5))
        prior_exprs[[length(prior_exprs) + 1]] <- quote(lambda ~ half_normal(1 / sqrt(w_lambda)))
        prior_exprs[[length(prior_exprs) + 1]] <- quote(w_tau ~ gamma(0.5, 0.5))
        prior_exprs[[length(prior_exprs) + 1]] <- quote(tau_hs ~ half_normal(tau0 / sqrt(w_tau)))
        prior_exprs[[length(prior_exprs) + 1]] <- quote(c2 ~ inverse_gamma(half_slab_df, half_slab_scale2))
      } else if (regularization == "ssp") {
        prior_exprs[[length(prior_exprs) + 1]] <- quote(beta_raw ~ laplace(0, 0.5))
        prior_exprs[[length(prior_exprs) + 1]] <- bquote(mu_r <- log(.(prior$ssp_ratio) / (1 - .(prior$ssp_ratio))))
        prior_exprs[[length(prior_exprs) + 1]] <- quote(r ~ logit_normal(mu_r, 3))
        prior_exprs[[length(prior_exprs) + 1]] <- quote(tau ~ exponential(1 / tau_scale))
      }
    }

    if (has_random) {
      for (b in 1:num_bars) {
        sd_name <- as.name(paste0("sd", suffix(b)))
        if (use_weak_info && prior_type == "weak") {
          prior_exprs[[length(prior_exprs) + 1]] <- bquote(.(sd_name) ~ exponential(tau_rate))
        } else if (!is_flat && !is.null(prior$sd_rate)) {
          prior_exprs[[length(prior_exprs) + 1]] <- bquote(.(sd_name) ~ exponential(.(prior$sd_rate)))
        }
      }
    }
  } else if (prior_type == "normal") {
    if (family %in% c("gaussian", "lognormal", "student_t") && !is.null(prior$sigma_rate)) {
      prior_exprs[[length(prior_exprs) + 1]] <- bquote(sigma ~ exponential(.(prior$sigma_rate)))
    }

    if (has_intercept && !is.null(prior$mu_sd)) {
      prior_exprs[[length(prior_exprs) + 1]] <- bquote(Intercept_c ~ normal(0, .(prior$mu_sd)))
    }

    if (K > 0 && !is.null(prior$b_sd)) {
      prior_exprs[[length(prior_exprs) + 1]] <- bquote(b ~ normal(0, .(prior$b_sd)))
    }

    if (has_random && !is.null(prior$tau_rate)) {
      for (b in 1:num_bars) {
        sd_name <- as.name(paste0("sd", suffix(b)))
        prior_exprs[[length(prior_exprs) + 1]] <- bquote(.(sd_name) ~ exponential(.(prior$tau_rate)))
      }
    }

    if (family == "student_t" && !is.null(prior$nu_rate)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(nu ~ exponential(.(prior$nu_rate)))
    if (family == "gamma" && !is.null(prior$shape_rate)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(shape ~ exponential(.(prior$shape_rate)))
    if (family == "neg_binomial" && !is.null(prior$phi_rate)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(phi ~ exponential(.(prior$phi_rate)))
    if (family %in% c("ordered", "sequential") && !is.null(prior$cutpoint_sd)) prior_exprs[[length(prior_exprs) + 1]] <- bquote(cutpoints ~ normal(0, .(prior$cutpoint_sd)))
  }

  model_exprs <- list()
  model_exprs[[length(model_exprs) + 1]] <- "# Transform"
  model_exprs <- c(model_exprs, transform_exprs)
  model_exprs[[length(model_exprs) + 1]] <- "# Likelihood (Data)"
  model_exprs <- c(model_exprs, ll_data_exprs)
  if (has_random) {
    model_exprs[[length(model_exprs) + 1]] <- "# Likelihood (Random)"
    model_exprs <- c(model_exprs, ll_random_exprs)
  }
  model_exprs[[length(model_exprs) + 1]] <- "# Priors"
  model_exprs <- c(model_exprs, prior_exprs)

  model_ast <- as.call(c(list(as.name("{")), model_exprs))

  waic_ast <- NULL
  if (isTRUE(WAIC)) {
    if (!is.null(resid_corr)) {
      waic_exprs <- c(transform_exprs, list(bquote({
        log_lik <- numeric(num_groups_resid)
        has_sig_idx <- !is.null(sigma_idx)
        .(if (resid_corr == "un") quote(R_mat_resid <- L_resid %*% t(L_resid)) else quote(NULL))
        for (i in 1:num_groups_resid) {
          idx <- which(group_resid == i)
          ni <- length(idx)
          ti <- time_resid[idx]
          mu_i <- eta[idx]
          Vi <- matrix(0, ni, ni)
          for (r in 1:ni) {
            s_r <- if (has_sig_idx) sigma[sigma_idx[idx[r]]] else sigma
            for (c in 1:ni) {
              s_c <- if (has_sig_idx) sigma[sigma_idx[idx[c]]] else sigma
              .(vi_inner_expr)
            }
          }
          diag(Vi) <- diag(Vi) + 1e-3
          log_lik[i] <- multi_normal_lpdf(Y[idx], mu_i, Vi)
        }
      })))
      waic_ast <- as.call(c(list(as.name("{")), waic_exprs))
    } else {
      waic_ll <- switch(family,
        "gaussian" = if (!is.null(sigma_idx)) quote(log_lik <- normal_lpdf(Y, eta, sigma[sigma_idx], sum = FALSE)) else quote(log_lik <- normal_lpdf(Y, eta, sigma, sum = FALSE)),
        "lognormal" = if (!is.null(sigma_idx)) quote(log_lik <- lognormal_lpdf(Y, eta, sigma[sigma_idx], sum = FALSE)) else quote(log_lik <- lognormal_lpdf(Y, eta, sigma, sum = FALSE)),
        "student_t" = if (!is.null(sigma_idx)) quote(log_lik <- student_t_lpdf(Y, nu, eta, sigma[sigma_idx], sum = FALSE)) else quote(log_lik <- student_t_lpdf(Y, nu, eta, sigma, sum = FALSE)),
        "gamma" = quote(log_lik <- gamma_lpdf(Y, shape, shape / exp(eta), sum = FALSE)),
        "bernoulli" = quote(log_lik <- bernoulli_logit_lpmf(Y, eta, sum = FALSE)),
        "binomial" = quote(log_lik <- binomial_logit_lpmf(Y, trials, eta, sum = FALSE)),
        "poisson" = quote(log_lik <- poisson_lpmf(Y, exp(eta), sum = FALSE)),
        "neg_binomial" = quote(log_lik <- neg_binomial_2_lpmf(Y, exp(eta), phi, sum = FALSE)),
        "ordered" = quote(log_lik <- ordered_logistic_lpmf(Y, eta, cutpoints, sum = FALSE)),
        "sequential" = quote(log_lik <- sequential_logistic_lpmf(Y, eta, cutpoints, sum = FALSE))
      )
      waic_ast <- as.call(c(list(as.name("{")), c(transform_exprs, list(waic_ll))))
    }
  }

  code_obj <- list(setup = setup_ast, parameters = param_ast)
  if (!is.null(tran_ast)) code_obj$transform <- tran_ast
  code_obj$model <- model_ast
  generate <- .rtmb_waic_generate_ast(generate, waic_ast)
  if (!is.null(generate)) code_obj$generate <- generate

  par_names_list <- list()
  if (num_sigma_groups > 1) {
    par_names_list$sigma <- sigma_group_names
  }
  if (K > 0) {
    if (family == "sequential") {
      transition_names <- paste0(seq_len(num_categories - 1), "->", 2:num_categories)
      par_names_list$b <- list(fixed_names, transition_names)
    } else {
      par_names_list$b <- fixed_names
    }
    if (regularization == "rhs") {
      if (family == "sequential") {
        par_names_list$z <- list(fixed_names, transition_names)
        par_names_list$lambda <- list(fixed_names, transition_names)
        par_names_list$w_lambda <- list(fixed_names, transition_names)
      } else {
        par_names_list$z <- fixed_names; par_names_list$lambda <- fixed_names; par_names_list$w_lambda <- fixed_names
      }
    } else if (regularization == "ssp") {
      if (family == "sequential") {
        par_names_list$beta_raw <- list(fixed_names, transition_names)
        par_names_list$r <- list(fixed_names, transition_names)
        par_names_list$tau <- list(fixed_names, transition_names)
      } else {
        par_names_list$beta_raw <- fixed_names; par_names_list$r <- fixed_names; par_names_list$tau <- fixed_names
      }
    }
  }
  if (family %in% c("ordered", "sequential")) {
    par_names_list$cutpoints <- paste0("C", seq_len(num_categories - 1))
  }
  if (has_random) {
    for (b in 1:num_bars) {
      r_names <- ranef_names_list[[b]]
      if (num_bars > 1) {
        r_names <- paste0(r_names, ":", group_labels_list[[b]])
      }
      par_names_list[[paste0("sd", suffix(b))]] <- paste0(group_labels_list[[b]], ":", r_names)
      if (num_ranef_list[[b]] > 1) par_names_list[[paste0("corr", suffix(b))]] <- r_names
    }
  }

  view_vars <- c()
  if (has_intercept) {
    view_vars <- c("Intercept")
  }
  if (K > 0) view_vars <- c(view_vars, "b")
  if (family %in% c("ordered", "sequential")) view_vars <- c(view_vars, "cutpoints")
  view_vars <- c(view_vars, "sigma")
  if (has_random) {
    for (b in 1:num_bars) {
      view_vars <- c(view_vars, paste0("sd", suffix(b)))
      if (num_ranef_list[[b]] > 1) view_vars <- c(view_vars, paste0("corr", suffix(b)))
    }
  }
  if (!is.null(resid_corr)) {
    if (resid_corr %in% c("ar1", "cs", "toep")) view_vars <- c(view_vars, "rho_resid")
    if (resid_corr == "un") view_vars <- c(view_vars, "L_resid")
  }

  ordered_data <- env_to_ordered_list(tmp_env, dat, setup_ast)
  ordered_data <- utils::modifyList(ordered_data, dat_weights)
  obj <- rtmb_model(data = ordered_data, code = code_obj, par_names = par_names_list, init = init,
                    fixed = fixed,
                    view = if (!is.null(view)) view else view_vars, silent = FALSE)
  obj$formula <- formula
  obj$raw_data <- data
  obj$family <- family
  # Store preferences
  obj$contrasts <- actual_contrasts
  obj$requested_contrasts <- contrasts


  # --- Metadata for classic inference and other methods ---
  obj$type <- if (has_random || !is.null(resid_corr)) {
    if (family %in% c("gaussian", "lognormal", "student_t")) "lmer" else "glmer"
  } else {
    if (family %in% c("gaussian", "lognormal", "student_t")) "lm" else "glm"
  }
  obj$extra <- list(
    source = "wrapper",
    prior_type = prior$type,
    X_assign = X_assign, 
    X_terms = X_terms, 
    X_colnames = fixed_colnames,
    factors = factors,
    within = within
  )

  fixed_effects <- if (K > 0) "b" else character(0)
  if (has_intercept) {
    fixed_effects <- c(if (use_centering) "Intercept_c" else "Intercept", fixed_effects)
  }
  obj$extra$marginal <- fixed_effects

  # --- mixed-model metadata for robust standard errors ---
  if (obj$type %in% c("lmer", "glmer")) {
    obj$extra$glmer_info <- list(
      family = family,
      num_bars = num_bars,
      group_labels = group_labels_list,
      weight_names = if (num_bars > 0) paste0("robust_re_weight", sapply(1:num_bars, suffix)) else character(0),
      obs_weight_name = "robust_obs_weight",
      group_idx_names = if (num_bars > 0) paste0("group_idx", sapply(1:num_bars, suffix)) else character(0),
      num_groups_list = num_groups_list
    )
  }

  return(obj)
}

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)

## ----eval=FALSE---------------------------------------------------------------
# library(BayesRTMB)
# 
# code <- rtmb_code(
#   setup = {
#     # Data preprocessing and constant definitions
#   },
#   parameters = {
#     # Declaration of parameters to be estimated
#   },
#   transform = {
#     # Calculation of derived variables using parameters and data
#   },
#   model = {
#     # Definition of priors and likelihoods
#   },
#   generate = {
#     # Calculation of posterior predictions and generated quantities
#   }
# )

## ----eval=FALSE---------------------------------------------------------------
# parameters = {
#   # Scalar parameter (with lower bound of 0)
#   sigma = Dim(lower = 0)
# 
#   # Vector parameter (length N)
#   mu = Dim(N)
# 
#   # Matrix parameter (N rows, M columns)
#   X = Dim(N, M)
# }

## ----eval=FALSE---------------------------------------------------------------
# model = {
#   # Priors
#   mu ~ normal(0, 10)
#   sigma ~ exponential(1)
# 
#   # Likelihood
#   Y ~ normal(mu, sigma)
# }

## ----eval=FALSE---------------------------------------------------------------
# model = {
#   lp <- lp + normal_lpdf(mu, 0, 10)
#   lp <- lp + exponential_lpdf(sigma, 1)
#   lp <- lp + normal_lpdf(Y, mu, sigma)
# }

## ----eval=FALSE---------------------------------------------------------------
# setup = {
#   N <- length(Y)  # Get the sample size of the observed data
#   P <- ncol(X)    # Get the number of columns in the design matrix (number of predictors)
# }

  # Simulate two-sample data with a true effect size
  set.seed(123)
  y1 <- rnorm(30, mean = 0.5, sd = 1)
  y2 <- rnorm(30, mean = 0.0, sd = 1)

  # Fit the Bayesian two-sample t-test model
  # r = 0.707 is the standard scale for the Cauchy prior on the effect size
  fit_ttest <- rtmb_ttest(y1, y2, prior = prior_jzs(r = 0.707))

  \donttest{
  # MCMC sampling (chains and iterations reduced for faster execution)
  mcmc_ttest <- fit_ttest$sample(sampling = 500, warmup = 500, chains = 2)
  mcmc_ttest$summary()
  }

#' Plot marginal means with error bars
#'
#' @description
#' Plot method for rtmb_lsmeans objects.
#'
#' @param x An rtmb_lsmeans object.
#' @param y Ignored.
#' @param error_bar Type of error bar ("se" or "ci").
#' @param col Color of the bars.
#' @param main Plot title.
#' @param xlab X axis label.
#' @param ylab Y axis label.
#' @param ... Additional arguments passed to barplot.
#' @export
plot.rtmb_lsmeans <- function(x, y, error_bar = "se", col = NULL, main = NULL, xlab = NULL, ylab = NULL, ...) {
  if (!inherits(x, "rtmb_lsmeans")) {
    if (!all(c("estimate", "Std. Error") %in% names(x))) {
      stop("x must be an rtmb_lsmeans object or a dataframe with estimate and Std. Error.")
    }
  }
  
  if (length(error_bar) > 1) error_bar <- error_bar[1]
  error_bar <- match.arg(error_bar, c("se", "ci"))
  means <- x$estimate
  names(means) <- rownames(x)
  
  if (error_bar == "se") {
    upper <- means + x$`Std. Error`
    lower <- means - x$`Std. Error`
    if (is.null(ylab)) ylab <- "Mean SE"
  } else {
    upper <- x$`Upper 95%`
    lower <- x$`Lower 95%`
    if (is.null(ylab)) ylab <- "Mean CI"
  }
  
  # Styling defaults
  if (is.null(col)) col <- grDevices::hcl.colors(length(means), "Pastel 1")
  if (is.null(main)) main <- "Marginal Means"
  
  # Handle limits
  ylim <- range(c(0, lower, upper), na.rm = TRUE)
  if (all(means >= 0)) ylim[1] <- 0
  if (all(means <= 0)) ylim[2] <- 0
  
  # Add some padding to ylim
  pad <- diff(ylim) * 0.1
  if (ylim[2] > 0) ylim[2] <- ylim[2] + pad
  if (ylim[1] < 0) ylim[1] <- ylim[1] - pad

  # Draw barplot
  old_par <- graphics::par(mar = c(5, 5, 4, 2) + 0.1, mgp = c(3, 0.7, 0), tcl = -0.3)
  on.exit(graphics::par(old_par))
  
  b <- graphics::barplot(means, ylim = ylim, col = col, main = main, xlab = xlab, ylab = ylab, 
               border = "white", las = 1, cex.axis = 0.9, ...)
  
  # Add a light grid
  graphics::abline(h = 0, lwd = 1.5, col = "gray40")
  graphics::grid(nx = NA, ny = NULL, col = "gray90", lty = "solid")
  # Redraw bars over grid
  graphics::barplot(means, add = TRUE, col = col, border = "white", axes = FALSE, ...)

  # Add error bars
  graphics::segments(b, lower, b, upper, lwd = 1.8, col = "gray20")
  # Caps
  eps <- 0.05 * diff(graphics::par("usr")[1:2]) / length(means)
  graphics::segments(b - eps, lower, b + eps, lower, lwd = 1.8, col = "gray20")
  graphics::segments(b - eps, upper, b + eps, upper, lwd = 1.8, col = "gray20")
  
  invisible(b)
}

---
title: "RTMB Internals and Inference Algorithms"
description: "Explains how BayesRTMB turns rtmb_code() into RTMB automatic differentiation objects and routes models to MCMC, MAP, variational inference, and classical estimation."
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{RTMB Internals and Inference Algorithms}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

This article explains how BayesRTMB represents models internally, how those models are converted into RTMB automatic differentiation objects, and how the same model object is routed to MCMC, MAP estimation, variational inference, or classical estimation.

Most users do not need to know these details in order to fit ordinary models. Wrapper functions such as `rtmb_lm()` and `rtmb_glmer()`, together with the basic `rtmb_code()` and `rtmb_model()` workflow, are usually enough.

The internal design becomes useful when you want to:

- write more complex `rtmb_code()` models;
- read the output of `print_code()` from wrapper functions;
- understand `random = TRUE`, Laplace approximation, `marginal`, `fixed`, and `map`;
- compare `sample()`, `optimize()`, `variational()`, and `classic()`;
- understand how standard errors and intervals are calculated.

# 1. Overall Flow

At a high level, BayesRTMB proceeds as follows.

```text
rtmb_code()
  -> rtmb_model()
    -> RTMB_Model
      -> build_ad_obj()
        -> RTMB::MakeADFun()
          -> sample() / optimize() / variational() / classic()
            -> MCMC_Fit / MAP_Fit / VB_Fit / Classic_Fit
```

Users usually write only `rtmb_code()` and call `rtmb_model()`.

`rtmb_model()` checks the model code and data, then creates an `RTMB_Model` R6 object. This object stores the model definition, data, parameter declarations, transformation blocks, generated-quantity blocks, and wrapper metadata.

When an inference method is called, BayesRTMB builds the automatic differentiation object needed by `RTMB::MakeADFun()`. The resulting fit is returned as a method-specific object.

BayesRTMB is organized around the functional priority MCMC, MAP, VB, and then classical estimation. MCMC is the central method when the full posterior distribution matters. MAP provides fast point estimation and approximate checks. VB provides approximate inference for larger models. Classical estimation is a supplementary route for flat-prior wrapper models.

| Method | Main use | Return value |
|---|---|---|
| `sample()` | MCMC with NUTS | `MCMC_Fit` |
| `optimize()` | MAP / ML / marginal MAP | `MAP_Fit` |
| `variational()` | Approximate posterior inference with ADVI | `VB_Fit` |
| `classic()` | Classical estimation for supported flat-prior wrapper models | `Classic_Fit` |

# 2. Blocks in `rtmb_code()`

`rtmb_code()` describes a model as a set of named blocks.

```{r}
code <- rtmb_code(
  data = {
    # Declare data names used by the model
  },
  setup = {
    # Preprocess data
  },
  parameters = {
    # Declare estimable parameters
  },
  transform = {
    # Build linear predictors and derived quantities
  },
  model = {
    # Write likelihood and prior distributions
  },
  generate = {
    # Compute generated quantities after fitting
  }
)
```

Not every block is required. The minimal model usually needs only `parameters` and `model`.

## data

The `data` block declares data names used by the model. `rtmb_model()` checks whether these names exist in the supplied data list.

## setup

The `setup` block is evaluated once before model evaluation. It is the right place to compute data dimensions, design matrices, centered predictors, group indices, and constants used later in the model.

Wrapper functions often place preprocessing in `setup` so that `print_code()` shows enough information to reproduce the model.

## parameters

The `parameters` block declares estimable parameters with `Dim()`.

```{r}
parameters = {
  alpha <- Dim()
  beta  <- Dim(P)
  sigma <- Dim(lower = 0)
  r     <- Dim(G, random = TRUE)
}
```

`Dim()` stores dimension, constraints, initial values, and whether a parameter should be treated as a random effect.

## transform

The `transform` block creates derived quantities from parameters and data.

```{r}
transform = {
  mu <- alpha + X %*% beta
}
```

Quantities created in `transform` can be used in the `model` block and can also appear in summaries. For MAP and classical estimation, standard errors and intervals can be calculated by the delta method or simulation-based propagation.

Only primary parameters declared in `parameters` can be used as Laplace-random targets. Derived quantities created in `transform` cannot be specified directly as `marginal` variables.

## model

The `model` block contains likelihood and prior distributions.

```{r}
model = {
  beta ~ normal(0, 10)
  sigma ~ exponential(1)
  Y ~ normal(mu, sigma)
}
```

BayesRTMB encourages Stan-like sampling syntax. Internally, this is converted into additions to the log probability.

Conceptually, the following two lines describe the same contribution.

```{r}
Y ~ normal(mu, sigma)
```

```{r}
lp <- lp + normal_lpdf(Y, mu, sigma)
```

In ordinary `rtmb_code(model = { ... })`, you do not need to initialize `lp`. BayesRTMB prepares it internally when constructing the executable log-probability function.

## generate

The `generate` block computes quantities after fitting.

```{r}
generate = {
  y_rep <- rnorm(length(Y), mu, sigma)
}
```

Generated quantities are not part of likelihood evaluation or optimization. They are useful for posterior predictive checks, predictions, simulations, residuals, item information, factor scores, and other post-estimation quantities.

# 3. Parameter Representation

Users usually think about parameters on their natural constrained scale:

- `sigma` is positive;
- `theta` is between 0 and 1;
- ordered cutpoints must be increasing;
- simplex vectors must sum to 1.

Optimization and HMC are easier on unconstrained real-valued spaces. BayesRTMB therefore maintains two representations.

| Representation | Meaning |
|---|---|
| constrained scale | Natural user-facing scale |
| unconstrained scale | Scale used by optimizers and samplers |

For example, `sigma <- Dim(lower = 0)` is displayed as a positive `sigma`, but internally it is transformed from an unconstrained value.

Important helper functions include:

- `to_unconstrained()`;
- `to_constrained()`;
- `constrained_vector_to_list()`;
- `unconstrained_vector_to_list()`;
- `generate_flat_names()`.

These transformations are central to `fixed`, `map`, random effects, Laplace approximation, and printed summaries.

# 4. Jacobian Correction

When constrained parameters are transformed to an unconstrained space, density calculations can require a Jacobian adjustment.

If `y` is the natural constrained parameter and `x` is the unconstrained parameter, the relation is:

$$
\log p_X(x) = \log p_Y(y) + \log \left| \frac{dy}{dx} \right|
$$

BayesRTMB users write models on the natural scale. Internally, functions such as `to_unconstrained()`, `to_constrained()`, and `calc_log_jacobian()` add the required correction depending on the inference route.

The target is controlled by `jacobian_target`.

| `jacobian_target` | Meaning |
|---|---|
| `"all"` | Apply correction to all transformed parameters |
| `"random"` | Apply correction to the random side integrated by Laplace approximation |
| `"none"` | Do not add the correction |

For MCMC, BayesRTMB samples in the unconstrained space, so the posterior density generally uses `jacobian_target = "all"`.

For `optimize()` with Laplace approximation, the density transformation needed for the integrated random side uses `jacobian_target = "random"`.

For ordinary optimization without Laplace approximation, internal objective construction usually uses `jacobian_target = "none"`.

Most users do not need to set this option directly. The distinction matters when understanding profile likelihoods, Laplace approximation, or low-level internals.

# 5. RTMB Automatic Differentiation Objects

RTMB provides an R interface to the TMB automatic differentiation engine.

BayesRTMB calls `RTMB::MakeADFun()` through `RTMB_Model$build_ad_obj()`. This creates an object that can evaluate function values, gradients, and Hessian-related quantities.

The model code is not compiled from a separate Stan-like language. Instead, the R model function is evaluated with AD objects. Arithmetic, matrix operations, distribution functions, and transformations are recorded as an automatic differentiation graph.

`rtmb_model()` performs pre-checks before fitting:

- undefined variables in `setup` or `parameters`;
- whether `model` and `transform` can run with initial values;
- whether `log_prob` returns a scalar;
- whether `MakeADFun()` can compute gradients.

These checks aim to catch structural problems before they become harder-to-read RTMB/TMB errors.

# 6. Inference Methods

## sample

`sample()` runs MCMC using NUTS.

```{r}
fit <- mdl$sample()
```

MCMC is the central method when you want the full posterior distribution, not only a point estimate. It supports posterior means, posterior medians, intervals, diagnostics, posterior predictive checks, and bridge sampling.

MCMC samples are drawn on the unconstrained scale and returned on the natural constrained scale.

## optimize

`optimize()` performs MAP, ML, or marginal MAP estimation.

```{r}
fit <- mdl$optimize()
```

With priors, the result is MAP estimation. With flat-prior models, it is interpreted as maximum likelihood or a closely related estimate.

`optimize()` is useful for checking model behavior before MCMC, and for obtaining fast estimates for hierarchical models with Laplace approximation. Confidence intervals and log marginal likelihood approximations should be interpreted as approximations on the unconstrained parameter space.

Common options include:

- `laplace`;
- `marginal`;
- `se_method`;
- `df_method`;
- `map`;
- `fixed`.

## variational

`variational()` approximates the posterior distribution with ADVI.

```{r}
fit <- mdl$variational()
```

VB can be useful for large models or as an initial approximate check before MCMC. It is faster than MCMC, but it is approximate and can underestimate uncertainty. When possible, compare final uncertainty assessments with MCMC.

## classic

`classic()` treats supported wrapper models with `prior_flat()` as classical frequentist analyses.

```{r}
fit <- mdl$classic()
```

It is available only for wrapper-derived models that can be interpreted as classical analyses under flat priors.

`Classic_Fit` supports frequentist post-processing where appropriate, such as degrees of freedom, p values, AIC, BIC, ANOVA, likelihood-ratio tests, correlation tests, and contingency-table tests.

Functionally, `classic()` is lower priority than MCMC, MAP, and VB. It is a supplementary path for checking standard analyses and comparing Bayesian and frequentist workflows.

# 7. `optimize()` vs `classic()`

`optimize()` and `classic()` share low-level RTMB machinery, but their responsibilities differ.

`optimize()` is the API for MAP / ML / marginal MAP. It can be used with informative priors.

`classic()` is the API for frequentist interpretation of supported wrapper models with flat priors.

Internally, `.fit_rtmb()` builds the `MakeADFun()` object, runs optimization, and returns raw components such as `sdreport()` output. It does not itself construct user-facing fit objects.

| Layer | Role |
|---|---|
| `.fit_rtmb()` | Returns raw RTMB optimization components |
| `.inference_pipeline()` | Constructs `MAP_Fit` for `optimize()` |
| `.classic_pipeline()` | Constructs `Classic_Fit` for `classic()` |

This separation lets the same optimization engine support different summaries, degrees of freedom, prior correction, and post-processing behavior.

# 8. Laplace Approximation and `random`

In hierarchical, latent-variable, and mixed models, it is often better to integrate over some parameters rather than optimize all parameters as fixed effects.

Parameters declared with `random = TRUE` are eligible for Laplace approximation.

```{r}
parameters = {
  alpha <- Dim()
  tau   <- Dim(lower = 0)
  r     <- Dim(G, random = TRUE)
}
```

With `optimize(laplace = TRUE)`, BayesRTMB passes random-effect parameters to `RTMB::MakeADFun(random = ...)`. In `optimize()`, `laplace = TRUE` is the default.

`optimize(marginal = ...)` can temporarily treat primary parameters as random so that marginal MAP / empirical Bayes style estimation can be performed. In flat-prior models, marginalizing fixed effects can correspond to restricted maximum likelihood (REML)-like estimation.

```{r}
fit <- mdl$optimize(marginal = c("mean", "delta"))
```

Two internal sets are important:

| Name | Meaning |
|---|---|
| `marginal_vars` | Primary parameter names requested by the user or wrapper through `optimize(marginal = ...)` |
| `laplace_random_vars` | All variables actually passed to `MakeADFun(random = ...)` |

In mixed models, both original random effects and primary parameters selected by `marginal` may be integrated by Laplace approximation. Therefore, `laplace_random_vars` can be broader than `marginal_vars`.

# 9. Standard Errors and Intervals

`optimize()` and `classic()` can report standard errors and intervals, not only point estimates.

Important `se_method` options include:

| `se_method` | Meaning |
|---|---|
| `"wald"` | Wald standard errors based on `sdreport()` and the delta method |
| `"sampling"` | Simulation-based error propagation from approximate normal samples |
| `"none"` | Do not compute standard errors or intervals |

`se_method = "none"` is a lightweight mode. It does not mean that standard errors are zero; it means they are not calculated.

Derived quantities created in `transform` can be handled by the delta method or by simulation-based propagation. Generated quantities are naturally handled from MCMC draws or by post-estimation generation.

If `sdreport()` cannot provide reliable standard errors, BayesRTMB emits diagnostic messages. Examples include non-positive-definite Hessians, unavailable covariance matrices, Hessian fallbacks, and generalized-inverse fallbacks.

# 10. Writing AD-Friendly Code

RTMB model code looks like ordinary R code, but internally it is evaluated with automatic differentiation objects. Some ordinary R patterns can therefore be problematic.

## Avoid parameter-dependent `if` statements

Avoid branches where the computation path changes based on a parameter value.

```{r}
# Avoid this pattern
if (sigma > 1) {
  lp <- lp + normal_lpdf(y, 0, sigma)
} else {
  lp <- lp + normal_lpdf(y, 0, 1)
}
```

Prefer smooth functions, constrained parameterizations, or explicit AD-compatible alternatives.

## Be careful with apply-style functions

Functions such as `apply()`, `lapply()`, `sapply()`, and `ifelse()` may not always behave as expected with AD objects. BayesRTMB catches some cases early, but explicit loops or vectorized arithmetic are often safer.

## Use blocks for their intended purposes

Data-only preprocessing belongs in `setup`.

Derived quantities based on parameters belong in `transform`.

Likelihood and priors belong in `model`.

Generated quantities belong in `generate`.

Keeping this separation makes `print_code()` readable and improves the stability of pre-checks and automatic differentiation.

# 11. Relationship to Stan, TMB, and RTMB

BayesRTMB uses RTMB as its computational backend. RTMB is an R interface to TMB's automatic differentiation engine.

BayesRTMB is similar to Stan in that users write probabilistic models with constrained parameters and priors. It differs in that model code is written as R expressions through `rtmb_code()`, rather than in a separate Stan language file.

| Topic | BayesRTMB | Stan |
|---|---|---|
| Model language | R through `rtmb_code()` | Stan language |
| Automatic differentiation | RTMB / TMB | Stan Math |
| Constraints | `Dim()` and internal transformations | Stan parameter constraints |
| Inference | MAP / classic / NUTS / ADVI | MAP / NUTS / ADVI and others |
| Wrappers | `rtmb_lm()`, `rtmb_glmer()`, and others | Usually hand-written models |
| Generated model inspection | `print_code()` | Stan code itself |

# 12. Summary

BayesRTMB's internal structure can be summarized as follows.

- `rtmb_code()` describes a model as a block-structured object.
- `rtmb_model()` checks the data and code, then creates an `RTMB_Model`.
- `RTMB_Model` stores model code, data, parameter declarations, transformations, generated quantities, and wrapper metadata.
- `build_ad_obj()` creates the automatic differentiation object used by `RTMB::MakeADFun()`.
- `sample()`, `optimize()`, `variational()`, and `classic()` run different inference methods from the same model representation.
- `optimize()` and `classic()` share a low-level engine, but their responsibilities and return objects are distinct.
- For Laplace approximation, the distinction between `random = TRUE`, `marginal_vars`, and `laplace_random_vars` is important.

For practical model-writing examples, see [Writing Model Codes](writing_models.html). To understand wrapper-generated models, see [Wrapper Functions](wrapper_functions.html) and `print_code()`. For a detailed mixed-model and GLMM workflow, see [Hierarchical Models and GLMMs](rtmb_glmer.html).

---
title: "BayesRTMB の内部構造"
description: "rtmb_code() が RTMB の自動微分オブジェクトに変換され、MCMC、MAP 推定、変分推論、古典的推定へ渡される流れを解説します。"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{BayesRTMB の内部構造}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

このページでは、BayesRTMB が内部でどのようにモデルを表現し、RTMB の自動微分オブジェクトへ変換し、推定結果を作っているかを説明します。

通常の分析では、この内部構造を意識する必要はありません。`rtmb_lm()` や `rtmb_glmer()` などのラッパー関数、または `rtmb_code()` と `rtmb_model()` の基本的な使い方を知っていれば十分です。

ただし、次のような場合には内部構造を知っておくと役に立ちます。

- 自分で複雑な `rtmb_code()` を書きたい。
- `print_code()` の出力を読んで、ラッパー関数が作ったモデルを理解したい。
- `random = TRUE`、Laplace 近似、`marginal`、`fixed`、`map` を組み合わせたい。
- `sample()`、`optimize()`、`variational()`、`classic()` の違いを整理したい。
- 標準誤差や信頼区間がどのように計算されているかを知りたい。

# 1. 全体の流れ

BayesRTMB の内部処理は、おおまかに次の流れで進みます。

```text
rtmb_code()
  -> rtmb_model()
    -> RTMB_Model
      -> build_ad_obj()
        -> RTMB::MakeADFun()
          -> sample() / optimize() / variational() / classic()
            -> MCMC_Fit / MAP_Fit / VB_Fit / Classic_Fit
```

ユーザーが書くのは、主に `rtmb_code()` と `rtmb_model()` までです。

`rtmb_model()` は、モデルコードとデータを検査し、`RTMB_Model` という R6 オブジェクトを作ります。この `RTMB_Model` が、モデル定義、データ、パラメータ定義、変換ブロック、生成量ブロック、ラッパー関数由来のメタデータを保持します。

その後、推定メソッドに応じて `RTMB::MakeADFun()` のための自動微分オブジェクトが作られます。得られた結果は、推定方法ごとに次の fit object として返されます。

BayesRTMB は、機能的には MCMC、MAP、VB、classic の順に重心を置いています。MCMC は事後分布を直接扱う中心的な推定法、MAP は高速な点推定と近似的な確認、VB は大きなモデルの近似推論、classic は wrapper 由来モデルを頻度主義的に確認するための補助的な入口です。

| メソッド | 主な用途 | 返り値 |
|---|---|---|
| `sample()` | NUTS による MCMC | `MCMC_Fit` |
| `optimize()` | MAP / ML / marginal MAP | `MAP_Fit` |
| `variational()` | ADVI による近似事後分布 | `VB_Fit` |
| `classic()` | 頻度主義的推定 | `Classic_Fit` |

# 2. rtmb_code の各ブロック

`rtmb_code()` は、モデルをいくつかのブロックに分けて書くための関数です。

```{r}
code <- rtmb_code(
  data = {
    # 入力データの宣言
  },
  setup = {
    # データ前処理
  },
  parameters = {
    # 推定パラメータの宣言
  },
  transform = {
    # パラメータやデータから作る派生量
  },
  model = {
    # 尤度と事前分布
  },
  generate = {
    # 推定後に計算したい生成量
  }
)
```

すべてのブロックが必須ではありません。最小限必要なのは `parameters` と `model` です。

## data

`data` ブロックは、モデルで使うデータ名を宣言するためのブロックです。`rtmb_model()` は、ここに書かれた変数が実際に `data` list に含まれているかを確認します。

## setup

`setup` ブロックは、データ前処理のためのブロックです。

`rtmb_model()` は、まず `data` list を環境に展開し、その環境内で `setup` を評価します。そこで作られた変数や更新された変数は、再び `data` list に戻されます。

たとえば、ラッパー関数が作るコードでは、ユーザーが `print_code()` を見て同じモデルを再現できるように、必要な前処理を `setup` に明示する方針をとっています。

## parameters

`parameters` ブロックでは、推定するパラメータを `Dim()` で宣言します。

```{r}
parameters = {
  alpha <- Dim()
  beta  <- Dim(P)
  sigma <- Dim(lower = 0)
  r     <- Dim(G, random = TRUE)
}
```

`Dim()` は、次元、制約、初期値、ランダム効果かどうかといった情報を持つパラメータ定義を作ります。

`rtmb_model()` はこのブロックを評価し、内部で `par_list` を作ります。`par_list` は、制約変換、初期値生成、flat vector との対応、`MakeADFun()` への受け渡しに使われる重要な情報です。

## transform

`transform` ブロックは、パラメータやデータから派生量を作る場所です。

```{r}
transform = {
  mu <- alpha + X %*% beta
}
```

`transform` で作った量は、`model` ブロックの中で使えます。また、推定後の summary に含めることもできます。MAP 推定や古典的推定では、必要に応じてデルタ法やシミュレーションにより標準誤差や区間推定が計算されます。

ただし、`transform` で作った量は primary parameter ではありません。たとえば `optimize(marginal = ...)` で Laplace 周辺化の対象にできるのは、`parameters` ブロックで宣言されたパラメータだけです。

## model

`model` ブロックは、尤度と事前分布を書く場所です。

```{r}
model = {
  beta ~ normal(0, 10)
  sigma ~ exponential(1)
  Y ~ normal(mu, sigma)
}
```

BayesRTMB では、Stan に近い `~` の sampling syntax を推奨しています。内部では `model_code()` がこの表記を、対数確率 `lp` への加算に変換します。

概念的には、次の2つは同じモデルを表します。

```{r}
Y ~ normal(mu, sigma)
```

```{r}
lp <- lp + normal_lpdf(Y, mu, sigma)
```

ただし、現在の BayesRTMB では、`model_code()` が実行用の `log_prob` 関数を作るときに `lp <- 0` を内部で用意します。そのため、通常の `rtmb_code(model = { ... })` には `lp <- 0` を書きません。

`lp` は内部で使う予約語です。`parameters` ブロックで `lp <- Dim()` のように宣言したり、一般の作業用変数として使ったりしないでください。

## generate

`generate` ブロックは、推定後に計算したい量を書く場所です。

```{r}
generate = {
  y_rep <- rnorm(length(Y), mu, sigma)
}
```

`generate` は、尤度評価や最適化の目的関数には入りません。事後予測、予測値、シミュレーション、因子得点、回転後の因子負荷量など、推定後に見たい量を置くためのブロックです。

モデル定義時に書かなかった生成量も、推定後に `generated_quantities()` で追加できます。

# 3. パラメータ表現

ユーザーは、通常、パラメータを自然な尺度で考えます。

- 標準偏差 `sigma` は正の値。
- 確率 `theta` は 0 から 1 の範囲。
- ordered cutpoints は昇順。
- simplex は合計が 1。

一方で、勾配ベースの最適化や HMC は、制約のない実数空間で動かすほうが扱いやすくなります。

そのため BayesRTMB は、内部で次の2つの表現を使い分けます。

| 表現 | 意味 |
|---|---|
| constrained scale | ユーザーが考える自然な尺度 |
| unconstrained scale | 最適化やサンプリングに使う制約なし尺度 |

たとえば、`sigma <- Dim(lower = 0)` と宣言した場合、ユーザー向けには正の `sigma` として表示されますが、内部では実数全体を動く値に変換されます。

この変換は、主に次の helper によって扱われます。

- `to_unconstrained()`
- `to_constrained()`
- `constrained_vector_to_list()`
- `unconstrained_vector_to_list()`
- `generate_flat_names()`

この層は、`fixed`、`map`、ランダム効果、Laplace 近似、summary 表示の整合性に深く関わります。

# 4. ヤコビアン補正

制約付きのパラメータを非制約空間へ変換すると、確率密度の尺度も変わります。そのため、事後分布を正しく扱うにはヤコビアン補正が必要になります。

概念的には、自然尺度のパラメータを `y`、非制約尺度のパラメータを `x` とすると、次の関係を考えます。

$$
\log p_X(x) = \log p_Y(y) + \log \left| \frac{dy}{dx} \right|
$$

BayesRTMB では、ユーザーは自然尺度でモデルを書きます。内部では `to_unconstrained()`、`to_constrained()`、`calc_log_jacobian()` を使い、推定方法に応じて必要なヤコビアン補正を目的関数に反映します。

補正の対象は `jacobian_target` によって制御されます。

| `jacobian_target` | 意味 |
|---|---|
| `"all"` | すべての変換パラメータに補正を入れる |
| `"random"` | Laplace で積分する random 側に補正を入れる |
| `"none"` | 補正を入れない |

用途としては、MCMC では非制約空間で事後分布全体をサンプリングするため、基本的に `jacobian_target = "all"` を使います。

一方、`optimize()` で Laplace 近似を使う場合は、Laplace で積分する random 側の密度変換が必要になるため、基本的に `jacobian_target = "random"` を使います。Laplace 近似を使わない通常の最適化では、内部目的関数上では `jacobian_target = "none"` が使われます。

通常のユーザーがこの設定を直接操作する必要はほとんどありません。ただし、profile likelihood や Laplace 近似を含む内部処理では、この区別が重要になります。

# 5. RTMB の自動微分オブジェクト

RTMB は、TMB の自動微分エンジンを R から使うためのパッケージです。

BayesRTMB では、`RTMB_Model$build_ad_obj()` が `RTMB::MakeADFun()` を呼び出し、関数値、勾配、ヘッセ行列などを評価できるオブジェクトを作ります。

このとき、モデル関数は通常の数値ではなく AD 型の値を使って評価されます。四則演算、`log()`、`exp()`、行列演算、分布関数などの計算過程が、自動微分のための計算グラフとして記録されます。

Stan のようにモデルコードを C++ に書き換えて明示的にコンパイルするのではなく、R で書いたモデル関数を AD 型で評価し、`MakeADFun()` object として高速に関数値や導関数を評価できるようにする、というイメージです。

`rtmb_model()` はモデル作成時に pre-check を行います。

- `setup` や `parameters` に未定義変数がないか。
- `model` や `transform` が初期値で実行できるか。
- `log_prob` がスカラーの数値を返すか。
- `MakeADFun()` で勾配が計算できるか。

この段階で構造的なエラーを早めに検出することで、RTMB/TMB 側の分かりにくいエラーをなるべく避ける設計になっています。

# 6. 推定メソッドごとの違い

## sample

`sample()` は、NUTS による MCMC サンプリングを行います。

```{r}
fit <- mdl$sample()
```

BayesRTMB では、MCMC が事後分布を直接調べるための中心的な推定法です。点推定だけでなく、事後平均、事後中央値、信用区間、収束診断、事後予測、bridge sampling による周辺尤度推定などを扱えます。

MCMC では、非制約尺度でサンプリングを行い、結果を自然尺度に戻して表示します。`generate` ブロックがあれば、サンプルごとの生成量も計算できます。

## optimize

`optimize()` は、MAP 推定、最尤推定、marginal MAP を担当する高速な推定 API です。

```{r}
fit <- mdl$optimize()
```

事前分布を含むモデルでは MAP 推定になります。追加 prior がない flat-prior 型のモデルでは、最尤推定と同じものとして解釈できます。

`optimize()` は、MCMC の前にモデルの挙動を素早く確認したい場合や、階層モデルで Laplace 近似を使って点推定を得たい場合に便利です。ただし、信頼区間やlog_mlは無制約パラメータ空間において多変量正規分布への近似が成り立つときに正確になります。返り値は `MAP_Fit` です。


主なオプションには、次のものがあります。

- `laplace`
- `marginal`
- `se_method`
- `df_method`
- `map`
- `fixed`

## variational

`variational()` は、ADVI による近似事後分布を計算します。

```{r}
fit <- mdl$variational()
```

大きなモデルや、MCMC の前におおまかな事後分布を確認したい場合に便利です。平均場近似、full-rank 近似、hybrid 型の近似を使い分ける設計になっています。

VB は MCMC より速く動くことがありますが、近似推論です。デフォルトの`meanfield`では制約なしパラメータ空間において独立な多変量正規分布を仮定した推定になるので、信頼区間は小さめに推定されます。`fullrank`を使えば独立でない多変量正規分布を仮定しますが、計算速度が遅くなります。どちらにせよ、最終的な不確実性評価では、可能であれば MCMC の結果と照合するのが安全です。

## classic

`classic()` は、BayesRTMB のラッパー関数を頻度主義的分析として使うための API です。

```{r}
fit <- mdl$classic()
```

`classic()` は、原則として次の条件を満たすモデルだけを対象にします。

- wrapper 由来のモデルである。
- `prior_flat()` のモデルである。

これは、informative prior を含むモデルに対して、あたかも通常の頻度主義的推定であるかのように表示しないための設計です。

`classic()` は `Classic_Fit` を返します。`Classic_Fit` では、モデルに応じて自由度、p 値、AIC、BIC、ANOVA、尤度比検定、相関検定、分割表検定など、頻度主義的な後処理を扱います。

このパッケージの機能的な優先度としては、`classic()` は MCMC、MAP、VB の後に位置づけられます。標準的な検定や頻度主義的な分析とベイズ推定の比較のための補助的な推定経路です。

# 7. optimize と classic の推定結果の違い

BayesRTMB の設計では、`optimize()` と `classic()` の責務を明確に分けています。

`optimize()` は、MAP / ML / marginal MAP のための API です。informative prior を含むモデルも扱います。

`classic()` は、wrapper 由来かつ flat-prior のモデルだけを対象にする頻度主義的 API です。内部的には固定効果をLaplace近似で周辺化した制約付き最尤法（REML）を行うことで、モーメント法と一致する推定量を得ています。

低水準では、どちらも RTMB による最適化エンジンを共有します。内部の `.fit_rtmb()` は、`MakeADFun()` object を作り、最適化を行い、`sdreport()` や共分散などの raw components を返します。

ただし、`.fit_rtmb()` 自体は fit object を作りません。

| 層 | 役割 |
|---|---|
| `.fit_rtmb()` | RTMB 最適化の raw components を返す |
| `.inference_pipeline()` | `optimize()` 用に `MAP_Fit` を作る |
| `.classic_pipeline()` | `classic()` 用に `Classic_Fit` を作る |

この分離により、同じ最適化エンジンを共有しながら、MAP 推定と頻度主義的分析の表示、自由度、prior correction、後処理を分けています。

# 8. Laplace 近似と random

階層モデルや潜在変数モデルでは、すべてのパラメータを同時に固定効果のように扱うよりも、一部を積分消去したいことがあります。

BayesRTMB では、`Dim(..., random = TRUE)` と宣言したパラメータは、Laplace 近似の対象になりえます。

```{r}
parameters = {
  alpha <- Dim()
  tau   <- Dim(lower = 0)
  r     <- Dim(G, random = TRUE)
}
```

`optimize(laplace = TRUE)` では、`random = TRUE` のパラメータを `RTMB::MakeADFun(random = ...)` に渡し、Laplace 近似で周辺化します。`optimize`では`laplace = TRUE`がデフォルトです。

また、`optimize(marginal = ...)` を使うと、`parameters` ブロックで宣言された primary parameter を一時的に random として扱い、marginal MAP / empirical Bayes 型の推定を行えます。flat priorにおいて固定効果を`marginal`に指定すれば、制約付き最尤法(REML)と一致します。

```{r}
fit <- mdl$optimize(marginal = c("mean", "delta"))
```

ここで重要なのが、`marginal_vars` と `laplace_random_vars` の区別です。

| 名前 | 意味 |
|---|---|
| `marginal_vars` | `optimize(marginal = ...)` でユーザーまたは wrapper が指定した primary parameter 名 |
| `laplace_random_vars` | 実際に `MakeADFun(random = ...)` に渡されたすべての変数名 |

mixed model では、もともとの random effects と、`marginal` で指定された primary parameter の両方が Laplace 対象になることがあります(REML)
。そのため、`laplace_random_vars` は `marginal_vars` より広い集合になることがあります。

`marginal = "auto"` は、wrapper が `extra$marginal` に保存した primary parameter 名を使います。ここに入れられるのは、`parameters` ブロックで宣言されたパラメータだけです。`transform` や `generate` で作られた派生量は指定できません。`optimize`でREML推定をしたいときはラッパー関数を使って`marginal = "auto"`とすれば簡単に実行できます。また、そのときは自動的に自由度推定が`df_method = "satterthwaite"`となり、t分布で信頼区間を計算します。`df_method = Inf`とすれば正規分布を仮定します。

# 9. 標準誤差と区間推定

`optimize()` と `classic()` では、点推定だけでなく標準誤差や区間推定も扱います。

`optimize()` の `se_method` には、主に次の選択肢があります。

| `se_method` | 意味 |
|---|---|
| `"wald"` | `sdreport()` やデルタ法にもとづく Wald 型の標準誤差 |
| `"sampling"` | 近似多変量正規サンプルによる誤差伝播 |
| `"none"` | 標準誤差と区間推定を計算しない |

`se_method = "none"` は軽量モードです。標準誤差を 0 とみなすのではなく、標準誤差や信頼区間を計算しない、という意味です。

`transform` で作られた派生量は、Wald 型ではデルタ法、`se_method = "sampling"` ではシミュレーションにもとづいて不確実性を伝播します。

`generate` は基本的に推定後の生成量です。生成量の標準誤差や区間を必要とする場合は、MCMC サンプルや `generated_quantities()` の結果を使って確認するのが自然です。

`sdreport()` が信頼できる標準誤差を返せない場合、BayesRTMB は診断メッセージを出します。たとえば、`pdHess = FALSE`、`cov.fixed` が返らない、標準誤差が非有限、Hessian fallback、`MASS::ginv()` fallback などが該当します。

# 10. AD 型のコードを書くときの注意点

RTMB では、モデルコードは通常の R のように見えますが、内部では自動微分の計算グラフとして記録されます。そのため、普通の R では問題なくても、AD 型の計算では避けたほうがよい書き方があります。

## パラメータに依存する if 文

パラメータの値によって計算経路が変わる `if` 文は避けてください。

```{r}
# 避けたい例
if (sigma > 1) {
  lp <- lp + normal_lpdf(y, 0, sigma)
} else {
  lp <- lp + normal_lpdf(y, 0, 1)
}
```

このような分岐は、計算グラフの記録と相性がよくありません。滑らかな関数、制約つきパラメータ化、または RTMB が提供する AD 対応の条件分岐を検討します。

## apply 系の関数

`apply()`、`lapply()`、`sapply()`、`ifelse()` などは、AD 型のモデルコードでは期待通りに動かないことがあります。BayesRTMB は一部を事前に検出してエラーにします。

必要な場合は、明示的な `for` ループやベクトル化された演算に書き換えるほうが安全です。

## setup と transform を使い分ける

データだけから決まる前処理は `setup` に置きます。こちらはAD型の計算を気にする必要はありません。自由なRの処理が可能です。

パラメータから決まる派生量は `transform` に置きます。こちらはAD型計算が行われます。

尤度や事前分布は `model` に置きます。

生成量は`generate`に置きます。こちらのブロックではAD型計算がされないので、自由にRコードで計算ができます。複雑な計算やAD計算でエラーが出る場合はこちらで計算しておくほうがいいかも知れません。

この分担を守ると、`print_code()` で見たときにモデルの意味が読みやすくなり、内部の pre-check や自動微分も安定しやすくなります。

# 11. Stan、TMB、RTMB との関係

BayesRTMB は、RTMB をバックエンドとして使います。RTMB は、TMB の自動微分エンジンを R から使えるようにしたパッケージです。

Stan と似ている点は、ユーザーが確率モデルを書き、制約変換や勾配ベースの推定を使えることです。

一方で、BayesRTMB では R の構文でモデルを書き、`rtmb_code()` のブロックを通じて RTMB の目的関数を作ります。ラッパー関数が生成したモデルも `print_code()` で確認できるため、標準的な分析から独自モデルへ進みやすい設計になっています。また、テープの記録ではC++へコンパイルをする必要がないので、スピーディーに分析が可能です。

| 観点 | BayesRTMB | Stan |
|---|---|---|
| モデル記述 | R の `rtmb_code()` | Stan 専用言語 |
| 自動微分 | RTMB / TMB | Stan Math |
| 制約変換 | `Dim()` と内部変換 | Stan の parameter 制約 |
| 推定 | MAP / classic / NUTS / ADVI | MAP / NUTS / ADVI など |
| ラッパー関数 | `rtmb_lm()`、`rtmb_glmer()` など | 基本的には手書き |
| 生成コード確認 | `print_code()` | Stan コードそのもの |

# 12. まとめ

BayesRTMB の内部構造は、次のように整理できます。

- `rtmb_code()` は、モデルをブロック構造として記述する入口です。
- `rtmb_model()` は、データとコードを検査し、`RTMB_Model` を作ります。
- `RTMB_Model` は、モデル、パラメータ、変換、生成量、wrapper metadata を保持します。
- `build_ad_obj()` は、`RTMB::MakeADFun()` のための自動微分オブジェクトを作ります。
- `sample()`、`optimize()`、`variational()`、`classic()` は、同じモデル表現から異なる推定を実行します。
- `optimize()` と `classic()` は低水準エンジンを共有しますが、責務と返り値は明確に分かれています。
- Laplace 近似では、`random = TRUE` と `marginal` の扱い、特に `marginal_vars` と `laplace_random_vars` の区別が重要です。

より実践的なモデルの書き方は、[モデルコードの書き方](ja-writing_models.html) を参照してください。ラッパー関数がどのような `rtmb_code()` を作るかは、[ラッパー関数の使い方](ja-wrapper_functions.html) と `print_code()` が参考になります。階層モデルや GLMM の具体的な使い方は、[階層モデル・GLMM](ja-rtmb_glmer.html) で確認できます。

---
title: "Wrapper Functions"
description: "A guide on how to use the convenient wrapper functions available in BayesRTMB (e.g., rtmb_lm, rtmb_glmer, rtmb_ttest)."
output: 
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Wrapper Functions}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

In BayesRTMB, you can freely define models using the `rtmb_code` block, but we also provide **wrapper functions** to quickly execute common statistical analyses (like t-tests, regressions, and factor analysis).

These functions allow you to specify models with a syntax similar to standard R functions like `lm()` and `lme4::glmer()` (though `lme4` itself is not required to run `BayesRTMB`), while fully utilizing the powerful estimation features of BayesRTMB (MCMC, MAP, ADVI).


This page introduces the usage of the main wrapper functions.

---

# 1. rtmb_ttest (Bayesian t-test)

This function is for examining the difference in means between two groups. `rtmb_ttest` is designed to easily perform **effect size (delta) estimation** and **Bayes factor calculation**, in addition to standard t-tests.

First, let's generate some sample data and analyze it.

```{r eval=FALSE}
library(BayesRTMB)
set.seed(123)

# Data generation (2 groups)
y1 <- rnorm(30, mean = 0, sd = 1)
y2 <- rnorm(30, mean = 1, sd = 1)

# Method passing vectors directly
mdl_ttest <- rtmb_ttest(y1, y2)

# Method using formula format (when you have a dataframe)
# df <- data.frame(Y = c(y1, y2), group = rep(c("A", "B"), each = 30))
# mdl_ttest <- rtmb_ttest(Y ~ group, data = df)

# MCMC estimation
fit_ttest <- mdl_ttest$sample()
fit_ttest$summary()
```

```text
## variable    mean    sd     map    q2.5   q97.5  ess_bulk  ess_tail  rhat 
## lp        -88.05  1.23  -87.08  -91.16  -86.63      1543      2467  1.00 
## delta       1.24  0.30    1.25    0.65    1.84      1830      1654  1.01 
## mean        0.57  0.12    0.58    0.34    0.81      3058      2436  1.00 
## sd          0.93  0.09    0.92    0.78    1.13      2371      2755  1.00 
## diff        1.15  0.25    1.11    0.64    1.63      2309      1695  1.01 
## mean0      -0.00  0.17   -0.01   -0.34    0.34      2645      2468  1.00 
## mean1       1.14  0.17    1.14    0.80    1.49      2507      2364  1.00 
```

### Checking the Generated Code

You can check what `rtmb_code` the wrapper function generated internally by using the `print_code()` method on the model object. This can be used as a reference if you want to extend the wrapper's model into your own custom model.

```{r eval=FALSE}
# Display the internally generated RTMB code
mdl_ttest$print_code()
```

```text
## === RTMB Model Code ===
## 
## rtmb_code(
##   parameters = {
##     mean = Dim(1)
##     sd = Dim(1, lower = 0)
##     delta = Dim(1)
##   }, 
##   transform = {
##     diff <- delta * sd
##     mean0 <- mean - diff/2
##     mean1 <- mean + diff/2
##   }, 
##   model = {
##     Y1 ~ normal(mean0, sd)
##     Y2 ~ normal(mean1, sd)
##     delta ~ cauchy(0, r)
##     mean ~ normal(0, 10)
##     sd ~ exponential(0.1)
##   }, 
##   <environment>
## )
```

### Calculating the Bayes Factor

By using the `bayes_factor()` method, you can compute a comparison (Bayes factor) against a null model where the effect size is $\delta = 0$.

```{r eval=FALSE}
# Compare with the model where effect size delta = 0
bf <- fit_ttest$bayes_factor(fixed = list(delta = 0))
bf
```

```text
## Bayes Factor (BF12) : 4777.563 
## Log Bayes Factor    : 8.4717 (Approx. Error = 0.0039)
## Interpretation      : Decisive evidence for Model 1 
```

---

# 2. rtmb_lm (Linear Regression Analysis)

You can perform linear regression with a format identical to standard `lm()`.
Here, we use the `debate` data included in the package (simulated data regarding debate sat).

```{r eval=FALSE}
data(debate)

# Model predicting sat using talk and skill
mdl_lm <- rtmb_lm(sat ~ talk + skill, data = debate)

# Quick check via MAP estimation
fit_lm <- mdl_lm$optimize()
fit_lm$summary()
```

```text
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 416.94
## Approx. Log Marginal Likelihood (Laplace): -425.12
## 
## Point Estimates and 95% Wald CI:
##    variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept     2.14693     0.20761    1.74001    2.55384 
## b[talk]       0.28612     0.05434    0.17961    0.39264 
## b[skill]      0.20106     0.06604    0.07162    0.33050 
## sigma         0.92284     0.03725    0.85265    0.99880 
## Intercept_c   3.43324     0.05333    3.32871    3.53777 
```

### Recommended Settings for Weakly Informative Priors

When calculating Bayes factors (using `Bridge Sampling` or the `bayes_factor` method), the choice of priors strongly influences the results. BayesRTMB recommends automatically configuring appropriate **Weakly Informative Priors** by setting `use_weak_info = TRUE` and specifying the theoretical minimum and maximum values of the objective variable in `y_range`.

```{r eval=FALSE}
# Create a model using weakly informative priors (assuming sat ranges from 1 to 5)
mdl_lm_weak <- rtmb_lm(sat ~ talk + skill,
  data = debate,
  use_weak_info = TRUE,
  y_range = c(1, 5)
)
mdl_lm_weak$print_code()
```

```text
## === RTMB Model Code ===
## 
## rtmb_code(
##   setup = {
##     N <- length(Y)
##     K <- ncol(X)
##     half_d_y <- diff(y_range)/2
##     base_scale <- half_d_y * weak_info_prior$sd_ratio
##     alpha_prior_sd <- half_d_y
##     mid_y <- mean(y_range)
##     sigma_rate <- 1/base_scale
##     tau_rate <- 1/base_scale
##     X_mean <- apply(X, 2, mean)
##     X_sd <- apply(X, 2, sd)
##     beta_prior_sd <- weak_info_prior$max_beta * base_scale/X_sd
##     X_c <- X - rep(1, N) %*% t(X_mean)
##   }, 
##   parameters = {
##     Intercept_c <- Dim(1)
##     b <- Dim(K)
##     sigma <- Dim(1, lower = 0)
##   }, 
##   transform = {
##     Intercept <- Intercept_c - sum(X_mean * b)
##   }, 
##   model = {
##     # Transform
##     eta <- as.vector(Intercept_c + X_c %*% b)
##     # Likelihood (Data)
##     Y ~ normal(eta, sigma)
##     # Priors
##     sigma ~ exponential(sigma_rate)
##     Intercept_c ~ normal(mid_y, alpha_prior_sd)
##     b ~ normal(0, beta_prior_sd)
##   }
## )
```

---

# 3. rtmb_glm (Generalized Linear Models)

By specifying the `family` argument, you can perform logistic regression, Poisson regression, etc.
As an example, we will execute a logistic regression predicting the experimental cond (cond: 0 or 1).

```{r eval=FALSE}
# Logistic regression (family = "bernoulli")
mdl_glm <- rtmb_glm(cond ~ sat + skill,
  data = debate,
  family = "bernoulli"
)

fit_glm <- mdl_glm$sample()
fit_glm$summary()
```

```text
       variable     mean    sd      map     q2.5    q97.5  ess_bulk  ess_tail  rhat 
## lp               -213.72  1.24  -212.80  -216.90  -212.31      1567      2449  1.00 
## Intercept          -1.35  0.50    -1.32    -2.31    -0.38      3740      2794  1.00 
## b[sat]     0.40  0.13     0.38     0.16     0.64      3127      2838  1.00 
## b[skill]           -0.01  0.15     0.01    -0.30     0.28      3445      2925  1.00 
## Intercept_c        -0.00  0.12    -0.03    -0.23     0.23      4129      2544  1.00 
```

### Available Distributions (`family`)

In `rtmb_glm` and `rtmb_glmer`, the following distributions can be specified. Standard link functions are configured internally for each distribution.

| `family` | Link Function | Description |
| :--- | :--- | :--- |
| `gaussian` | identity | Normal distribution. For standard regression analysis. |
| `lognormal` | log | Log-normal distribution. For data that takes positive values. |
| `student_t` | identity | t-distribution. Useful when you want to suppress the impact of outliers. |
| `gamma` | log | Gamma distribution. For positive, skewed data. |
| `bernoulli` | logit | Bernoulli distribution. For binary 0/1 data. |
| `binomial` | logit | Binomial distribution. For data representing counts of success/trials. |
| `poisson` | log | Poisson distribution. For count data. |
| `neg_binomial`| log | Negative binomial distribution. For overdispersed count data. |
| `ordered` | logit | Ordered logistic regression. For ordinal categorical data. |


---

# 4. rtmb_glmer (Generalized Linear Mixed Models)

You can handle models including random effects (varying effects) using the `(1 | group)` notation, similar to the `lme4` package. Let's build a model taking into account that individual sat varies depending on the group they belong to.

```{r eval=FALSE}
# Random intercept model
mdl_glmer <- rtmb_glmer(sat ~ talk + (1 | group),
  data = debate
)

# When including random effects, optimize() with Laplace approximation is fast
opt_glmer <- mdl_glmer$optimize(laplace = TRUE)
opt_glmer$summary()
```

```text
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 404.65
## Approx. Log Marginal Likelihood (Laplace): -412.59
## Note: Random effects are stored in $random_effects
## 
## Point Estimates and 95% Wald CI:
##    variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept     2.60450     0.17902    2.25363    2.95537 
## b[talk]       0.27441     0.05477    0.16706    0.38176 
## sigma         0.77389     0.03867    0.70170    0.85351 
## sd[Int]       0.51833     0.06587    0.40406    0.66492 
## Intercept_c   3.43322     0.06846    3.29904    3.56740 
```

## Regularization

In `rtmb_lm`, `rtmb_glm`, and `rtmb_glmer`, you can use regularization. Regularization is a method that shrinks the coefficients of excessive parameters towards 0, allowing for the construction of more parsimonious models with high predictive power.

In the `BayesRTMB` package, you can select the horseshoe prior (`rhs`) or the spike-and-slab prior (`ssp`) using the `penalty` option. `rhs` is relatively easier to estimate, but coefficients with small effects do not shrink exactly to 0. `ssp` is somewhat heavier to estimate, but it can shrink coefficients completely to 0.

When using regularization, you must also use weakly informative priors, which means you need to input the range of the objective variable. Since our data uses a 5-point scale, we input `c(1, 5)`. MAP estimation often struggles with regularized models, so it's better to use MCMC.

```{r eval=FALSE}
mdl_glmer <-
  rtmb_glmer(
    sat ~ talk + perf + skill + cond + (1 | group),
    data = debate,
    penalty = "ssp",
    y_range = c(1, 5)
  )

mcmc_glmer <- mdl_glmer$sample(parallel = TRUE)
mcmc_glmer
```

```{r eval=FALSE}
mcmc_glmer$draws("b[cond]") |> plot_dens()
```

![Shrunk Posterior Distribution](ssp_posterior_dist.png)

---

# 5. Post-Estimation Analysis (Interaction & Visualization)

After fitting a regression model (LM, GLM, GLMER), you can use `conditional_effects()` and `simple_effects()` to analyze and visualize the results. These methods are currently available for models fitted using MCMC (`sample()`).

### Visualization with `conditional_effects()`

The `conditional_effects()` function is used to visualize the predicted values of a model. It is particularly powerful for understanding interaction effects.

```{r eval=FALSE}
# Linear regression with interaction between talk and cond
mdl_int <- rtmb_lm(sat ~ talk * perf, data = debate)
fit_int <- mdl_int$sample()

# Visualize the interaction effect
# For continuous moderators, it automatically shows Mean ± 1SD
ce <- conditional_effects(fit_int, effect = "talk:perf")
plot(ce)
```

### Simple Effects Analysis with `simple_effects()`

While `conditional_effects()` provides a visual overview, `simple_effects()` allows you to statistically examine the effect of a focal variable at specific levels of a moderator.

- **Simple Slopes**: When the focal variable is continuous.
- **Pairwise Contrasts**: When the focal variable is categorical.

```{r eval=FALSE}
# Calculate simple slopes of 'talk' for each level of 'cond'
se <- simple_effects(fit_int, effect = "talk:perf")
print(se)
```

```text
## --- Simple Effects Analysis ---
##    moderator perf          term estimate  lower upper
##  perf       2.930 Slope of talk    0.038 -0.118 0.191
##  perf       4.690 Slope of talk    0.266  0.161 0.369
##  perf       6.450 Slope of talk    0.494  0.354 0.631
```

By default, for continuous moderators, it evaluates the effect at the Mean and Mean ± 1SD. You can change this behavior by specifying the `sd_multiplier` argument.

---

# 6. rtmb_corr (Correlation Matrix Estimation)

This function is for estimating the correlation relationships between variables.
Here, we will introduce both a 2-variable case and a full correlation matrix case using a portion of the Big Five personality data.

### 2-Variable Correlation and Bayes Factor

We estimate the correlation between two variables (e.g., extraversion items BF1 and BF6), and calculate a Bayes factor testing the null hypothesis of zero correlation ($r = 0$).

```{r eval=FALSE}
data(BigFive)

# Correlation between BF1 and BF6
mdl_corr2 <- rtmb_corr(BigFive[, c("BF1", "BF6")])
mcmc_corr2 <- mdl_corr2$sample()
mcmc_corr2$summary()
```

```text
##  variable     mean    sd      map     q2.5    q97.5  ess_bulk  ess_tail  rhat 
## lp         -493.27  1.55  -492.22  -497.08  -491.16      1799      2658  1.00 
## corr[rho]    -0.60  0.05    -0.60    -0.69    -0.50      2365      2539  1.00 
## mean[BF1]     3.56  0.08     3.56     3.39     3.72      3031      2914  1.00 
## mean[BF6]     2.71  0.09     2.73     2.54     2.89      2978      2872  1.00 
## sd[BF1]       1.10  0.06     1.07     0.99     1.22      3151      3313  1.00 
## sd[BF6]       1.15  0.06     1.16     1.04     1.28      3072      2477  1.00 
```

```{r eval=FALSE}
# Compare against the zero-correlation model (fixed = list(corr = 0))
bf_corr <- mcmc_corr2$bayes_factor(fixed = list(corr = 0))
bf_corr
```

```text
Bayes Factor (BF12) : 6.839475e+15 
Log Bayes Factor    : 36.4615 (Approx. Error = 0.0043)
Interpretation      : Decisive evidence for Model 1 
```

### Correlation Matrix Estimation

If you specify multiple variables at once, the entire correlation matrix is estimated.

```{r eval=FALSE}
# Estimate the correlation matrix of 5 variables
mdl_corr_mat <- rtmb_corr(BigFive[, 1:5])
opt_corr_mat <- mdl_corr_mat$optimize()
opt_corr_mat$summary()
```

```text
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 1254.77
## Approx. Log Marginal Likelihood (Laplace): -1289.58
## 
## Point Estimates and 95% Wald CI:
##      variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## corr[BF1,BF1]   1.00000     0.00000    1.00000    1.00000 
## corr[BF2,BF1]   0.09243     0.07555   -0.05565    0.24050 
## corr[BF3,BF1]   0.09382     0.07552   -0.05420    0.24184 
## corr[BF4,BF1]   0.08807     0.07562   -0.06014    0.23629 
## corr[BF5,BF1]  -0.02158     0.07619   -0.17090    0.12775 
## corr[BF1,BF2]   0.09243     0.07555   -0.05565    0.24050 
## corr[BF2,BF2]   1.00000     0.00000    1.00000    1.00000 
## corr[BF3,BF2]  -0.03374     0.07610   -0.18289    0.11542 
## corr[BF4,BF2]  -0.13588     0.07481   -0.28250    0.01074 
## corr[BF5,BF2]  -0.08526     0.07567   -0.23357    0.06306 
```

---

# 7. rtmb_fa (Exploratory Factor Analysis)

Estimates the common latent factors behind observed variables.

## Factor Rotation

Factor analysis can be slightly time-consuming with MCMC, making MAP estimation faster and easier to use.
Factor rotation can be performed via the `rotate` option.
The rotated factor loadings will have the rotation name appended, like `_promax`.
Because the rotated factor loadings are placed in the `generate` block, MAP does not calculate standard errors for them by default. By specifying `se_sampling = TRUE`, you can compute their standard errors as well.

```{r eval=FALSE}
# 3-factor model, specifying Promax rotation
mdl_fa <- rtmb_fa(BigFive, nfactors = 5, rotate = "promax")

opt_fa <- mdl_fa$optimize(se_sampling = TRUE)

# Check the factor loadings (L), etc.
opt_fa$summary()
```

```text
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 4791.18
## Approx. Log Marginal Likelihood (Laplace): -5016.21
## 
## Point Estimates and 95% Wald CI:
##         variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## L_promax[BF1,1]    0.86657     0.05905    0.72847    0.95902 
## L_promax[BF2,1]    0.02202     0.05325   -0.07631    0.12786 
## L_promax[BF3,1]   -0.09276     0.06886   -0.20404    0.06562 
## L_promax[BF4,1]    0.04927     0.06391   -0.07333    0.17433 
## L_promax[BF5,1]   -0.02101     0.04856   -0.11742    0.07748 
## L_promax[BF6,1]   -0.80459     0.05957   -0.89651   -0.66314 
## L_promax[BF7,1]   -0.01111     0.05307   -0.11843    0.08566 
## L_promax[BF8,1]   -0.07429     0.05917   -0.16758    0.06246 
## L_promax[BF9,1]   -0.08225     0.07363   -0.23533    0.05242 
## L_promax[BF10,1]   0.03086     0.05210   -0.07256    0.13540 
```

The `sort_loadings()` function is useful to sort and display the rotated factor loadings nicely.

```{r eval=FALSE}
opt_fa$generate$L_promax |> sort_loadings()
```

```text
##        V1    V2    V3    V4    V5
## V1   .867 -.117 -.179  .089  .057
## V6  -.805  .000  .048  .200 -.065
## V11  .584  .087  .126  .169 -.197
## V16  .516  .087  .276 -.009  .040
## V2   .022 -.806 -.076 -.013  .043
## V7  -.011 -.800 -.175 -.054  .002
## V12  .065 -.797 -.045 -.083 -.088
## V17  .007 -.763  .070  .009 -.012
## V8  -.074  .172  .830 -.044 -.061
## V3  -.093  .044  .533  .101  .060
## V18 -.036 -.196  .512 -.156 -.066
## V13 -.005 -.078  .488 -.121  .154
## V4   .049  .118 -.080  .607  .058
## V14 -.096 -.037 -.031  .586 -.022
## V19  .094  .054  .001  .506 -.085
## V9  -.082  .064 -.059  .463  .019
## V5  -.021 -.132  .021  .206 -.795
## V20 -.022 -.126  .161  .321 -.762
## V15  .000 -.138  .075  .228  .730
## V10  .031 -.120  .047  .317  .712
```

## Factor Scores

Factor scores can also be output by using the `score = TRUE` option. These scores are additionally pushed to the `generate` output.

```{r eval=FALSE}
mdl_fa <- rtmb_fa(BigFive, nfactors = 5, rotate = "promax", score = TRUE)

opt_fa <- mdl_fa$optimize()

opt_fa$generate$score |> head()
```

```text
##             [,1]        [,2]        [,3]       [,4]       [,5]
## [1,]  0.49722956 -1.18644757 -1.29011492 -0.1941201 -0.4712234
## [2,] -1.21954058 -0.46063346  1.17665101 -0.8095066  1.0632201
## [3,] -0.05617309  0.09471994 -1.47004442  0.5897946 -0.1808377
## [4,]  0.48324361 -1.40302694  0.04676041 -0.8297736 -0.4993311
## [5,] -0.23744265  1.18767815  0.51330882 -0.3034767 -1.6688545
## [6,]  0.20824541 -0.38166057 -1.12921376  0.8736080 -0.5483280
```

## Regularized Factor Analysis

It also supports regularized factor analysis using a Spike-and-Slab Prior (SSP) for estimating sparse factor loadings.

Regularization shrinks the effects of unnecessary parameters toward zero, facilitating the construction of more parsimonious and predictive models. When applied to factor analysis, it can remove rotational indeterminacy, yielding a unique solution.

Since regularized factor analysis can sometimes be difficult to estimate via MAP, it is usually better to use MCMC or ADVI. Here, we present the results from ADVI (Automatic Differentiation Variational Inference).

```{r eval=FALSE}
mdl_fa <- rtmb_fa(BigFive, nfactors = 5, rotate = "ssp")

vb_fa <- mdl_fa$variational(iter = 5000, parallel = TRUE)
vb_fa
```

```text
## variable      mean     sd       map      q2.5     q97.5 
## lp        -5362.15  33.02  -5353.45  -5449.35  -5322.61 
## L[BF1,1]     -0.02   0.07     -0.00     -0.22      0.04 
## L[BF2,1]     -0.82   0.04     -0.83     -0.88     -0.74 
## L[BF3,1]      0.00   0.04     -0.00     -0.05      0.05 
## L[BF4,1]      0.02   0.07      0.00     -0.07      0.25 
## L[BF5,1]     -0.02   0.06     -0.00     -0.19      0.05 
## L[BF6,1]      0.00   0.03      0.00     -0.02      0.05 
## L[BF7,1]     -0.82   0.04     -0.82     -0.88     -0.73 
## L[BF8,1]      0.01   0.05      0.00     -0.05      0.11 
## L[BF9,1]      0.01   0.04      0.00     -0.03      0.08 
```

The output summary of MCMC or ADVI allows access via `EAP` or `MAP`. With regularization, the `MAP` summary makes the output easier to read as the shrunk parameters will rigorously collapse to 0.

```{r eval=FALSE}
vb_fa$MAP("L") |> sort_loadings()
```

```text
##        V1    V2    V3    V4    V5
## V7  -.823  .000 -.001  .001  .010
## V2  -.820  .001  .000  .000  .001
## V12 -.801 -.001  .000  .001  .000
## V17 -.738  .000  .000  .000  .001
## V1  -.001 -.795 -.001  .000  .001
## V6   .000  .773  .001 -.045  .002
## V11  .004 -.610  .096 -.099 -.021
## V16 -.001 -.554 -.001  .000 -.212
## V15 -.001 -.001 -.822 -.001 -.001
## V10  .000 -.001 -.809  .000  .000
## V5  -.004 -.002  .711 -.339 -.001
## V20  .000 -.001  .668 -.448 -.007
## V4   .002 -.001  .000 -.637  .002
## V19  .001  .000 -.001 -.569  .000
## V14 -.001 -.001 -.001 -.540  .002
## V9   .000  .002  .001 -.429  .002
## V8   .001 -.001  .000  .001 -.854
## V3  -.001 -.001 -.001  .000 -.487
## V13 -.027 -.001 -.003  .000 -.428
## V18 -.220 -.001  .001  .001 -.425
```

---

# 8. rtmb_irt (Item Response Theory)

`rtmb_irt()` constructs Item Response Theory (IRT) models suitable for analyzing test data or questionnaire surveys.
Currently, the following models are supported:

- **1PL / 2PL / 3PL Models**: Analysis of binary data (correct/incorrect).
- **Graded Response Model (GRM)**: Analysis of polytomous data (e.g., Likert scales).

Because IRT models are computationally demanding, it is highly effective to utilize the fast approximation via `variational()` or marginalize the ability parameters using `optimize(laplace = TRUE)` when handling large datasets.

### Analysis Example (Graded Response Model)

Using the `BigFive` personality data (polytomous), we will fit a Graded Response Model (GRM). Here, we analyze four items related to "Agreeableness".

```{r eval=FALSE}
data(BigFive)
# Select items 2, 7, 12, 17 (Agreeableness)
dat_irt <- BigFive[, c("BF2", "BF7", "BF12", "BF17")]

# Fit a 2PL Graded Response Model (Ordered Logistic)
mdl_irt <- rtmb_irt(dat_irt, model = "2PL", type = "ordered")

# Fast estimation using MAP with Laplace approximation for latent traits
opt_irt <- mdl_irt$optimize(se_sampling = TRUE)
opt_irt
```

```text
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 855.97
## Approx. Log Marginal Likelihood (Laplace): -862.01
## Note: Random effects are stored in $random_effects
## 
## Point Estimates and 95% Wald CI:
##           variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## a[BF2]               2.59656     0.39595    1.90122    3.46560 
## a[BF7]               2.54272     0.35182    1.95367    3.32212 
## a[BF12]              2.53327     0.39717    1.93398    3.53325 
## a[BF17]              1.92822     0.27020    1.46972    2.50115 
## b[BF2,Threshold1]   -5.52251     0.72815   -6.91884   -4.06858 
## b[BF7,Threshold1]   -5.34943     0.64569   -6.63549   -4.19742 
## b[BF12,Threshold1]  -5.29023     0.65049   -6.54949   -4.01624 
## b[BF17,Threshold1]  -4.31129     0.50114   -5.32272   -3.36251 
## b[BF2,Threshold2]   -2.68286     0.42440   -3.41160   -1.76330 
## b[BF7,Threshold2]   -2.72371     0.39514   -3.49773   -1.93475 
```

### Visualizing Item Response Curves

You can visualize the category response curves (CRCs) for each item using the `item_curve()` function. This helps in understanding how the probability of choosing each category changes with the latent trait ($\theta$).

```{r eval=FALSE}
# Calculate item curves
ic <- item_curve(opt_irt)

# Plot curves (shows category probabilities for each item)
plot(ic)
```

### Item and Test Information

Information functions help evaluate the precision of measurement across different levels of the latent trait.

```{r eval=FALSE}
# Item Information Function: shows which items provide the most information
ii <- item_info(opt_irt)
plot(ii)

# Test Information Function: total information of the test/scale
ti <- test_info(opt_irt)
plot(ti)
```

---

# 9. Missing Data Handling

BayesRTMB wrapper functions feature a standardized `missing` argument to handle missing values (`NA`) gracefully. You can choose the most appropriate method depending on your analysis type.

The default for most wrappers is `"listwise"`, but advanced multivariate wrappers support **Full Information Maximum Likelihood (FIML)** and **Pairwise** methods.

| Wrapper Function | Supported `missing` methods | Default | Description |
| :--- | :--- | :--- | :--- |
| `rtmb_lm`, `rtmb_glm`, `rtmb_lmer`, `rtmb_glmer`, `rtmb_ttest` | `"listwise"` | `"listwise"` | Removes rows containing `NA` in any of the variables present in the model formula. |
| `rtmb_fa`, `rtmb_corr` | `"listwise"`, `"fiml"` | `"listwise"` | `"listwise"` calculates fast sufficient statistics on complete cases. `"fiml"` performs row-wise likelihood evaluation using partial observations. |
| `rtmb_corr` (extra method) | `"pairwise"` | | Available when `method = "pearson"` or `"spearman"`. Produces identical results to base R's `cor(..., use = "pairwise.complete.obs")`. |
| `rtmb_mdu` | `"listwise"`, `"fiml"` | `"listwise"` | `"fiml"` skips missing ratings in the internal likelihood loop, whereas `"listwise"` deletes rows containing any `NA`. |
| `rtmb_irt` | `"listwise"`, `"fiml"` | `"fiml"` | `"fiml"` natively handles missingness by omitting individual-item missing observations (long-format), while `"listwise"` drops students/respondents with any missing items. |

### Example: FIML in Factor Analysis (`rtmb_fa`)

When your data contains missing responses, you can easily enable FIML to keep all respondents in the model and estimate factor loadings using all available data:

```r
data(BigFive)
# Introduce some random missingness
set.seed(42)
BigFive_miss <- BigFive
for (col in 1:ncol(BigFive_miss)) {
  BigFive_miss[sample(1:nrow(BigFive_miss), 10), col] <- NA
}

# Run Factor Analysis with FIML
mdl_fa <- rtmb_fa(BigFive_miss, nfactors = 5, missing = "fiml")
fit_fa <- mdl_fa$optimize()
```

### Example: Pairwise Deletion in Correlation (`rtmb_corr`)

If you want to match R's default correlation tests using pairwise deletion, you can specify `missing = "pairwise"`:

```r
# Classic Pearson correlation with pairwise deletion
mdl_corr <- rtmb_corr(BigFive_miss[, 1:5], method = "pearson", missing = "pairwise")
fit_corr <- mdl_corr$classic()
fit_corr$summary()
```

---

# Summary

By leveraging the wrapper functions in BayesRTMB, you gain the following benefits:
- Perform sophisticated analyses using familiar syntax without writing complex model codes from scratch.
- Seamlessly switch between MAP, MCMC, and VB estimations using the same model object.
- Calculate Bayes factors smoothly.

For a more detailed guide to mixed models and GLMMs, see [Hierarchical Models and GLMMs](rtmb_glmer.html). If you require a more customized model, check out [Writing Model Codes](writing_models.html) and try building your own using `rtmb_code()`. For the model-building pipeline behind the wrappers, see [RTMB Internals and Inference Algorithms](rtmb_internals.html).

---
title: "BayesRTMB Quick Start"
description: "A short introduction to the minimal BayesRTMB workflow, MCMC checks, wrapper functions, visualization, and t tests."
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{BayesRTMB Quick Start}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

# Purpose of This Page

This page walks through the minimal workflow needed to start using BayesRTMB.

It covers the following topics.

1. Write a small model with `rtmb_code()`
2. Create a model object with `rtmb_model()`
3. Estimate the model with `optimize()` and `sample()`
4. Visualize the posterior distribution from MCMC
5. Use wrapper functions for multiple regression and interactions
6. Run a frequentist t test with `classic()`
7. Compute a Bayes factor using a JZS prior and MCMC

Detailed model writing, mixed models, GLMMs, and model comparison are covered in other vignettes.

# 0. Installation and Environment Check

BayesRTMB can be installed from CRAN.

```{r eval=FALSE}
install.packages("BayesRTMB")
```

The development version can be installed from GitHub.
If you use `pak`, install it as follows.

```{r eval=FALSE}
install.packages("pak")
pak::pak("norimune/BayesRTMB")
```

If you use `remotes`, install it as follows.

```{r eval=FALSE}
install.packages("remotes")
remotes::install_github("norimune/BayesRTMB")
```

## Windows Users

BayesRTMB uses RTMB / TMB internally.
For ordinary use, Windows users can install the CRAN binary package without Rtools.
Rtools is only needed for source installation, development, or compiling custom TMB C++ templates.

If you install BayesRTMB from source, you can check whether Rtools is available with the following code.

```{r eval=FALSE}
pkgbuild::check_build_tools(debug = TRUE)
```

If this returns `TRUE`, or reports that build tools are available, your source-install setup is ready.
If Rtools is not found, install the Rtools version that matches your R version, then restart R or RStudio.

# 1. Write a Minimal Model

We start with a small binomial model.
Suppose that 6 successes were observed out of 10 trials.

```{r eval=FALSE}
library(BayesRTMB)

Trial <- 10
Y <- 6

dat <- list(Trial = Trial, Y = Y)

code <- rtmb_code(
  parameters = {
    theta <- Dim(lower = 0, upper = 1)
  },
  model = {
    Y ~ binomial(Trial, theta)
    theta ~ beta(1, 1)
  }
)
```

The `parameters` block declares the parameters to estimate.
Here, `theta` is defined as a parameter constrained between 0 and 1.

The `model` block specifies the sampling distribution and prior distribution.
`Y ~ binomial(Trial, theta)` says that the number of successes `Y` follows a binomial distribution.

# 2. Create a Model Object

Pass the data and model code to `rtmb_model()` to create a model object for estimation.

```{r eval=FALSE}
mdl <- rtmb_model(dat, code)
```

```text
## Pre-checking model code...
## Checking RTMB setup...
```

At this stage, estimation has not yet been run.
`mdl` is an `RTMB_Model` object that stores the model definition and data.

# 3. Run MAP Estimation

Use `optimize()` when you want a fast point estimate.
In BayesRTMB, this is treated as MAP estimation when priors are present, and as something close to maximum likelihood estimation when flat priors are used.

```{r eval=FALSE}
fit_map <- mdl$optimize()
fit_map
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 1.38
## Approx. Log Marginal Likelihood (Laplace): -2.33
## 
## Point Estimates and 95% Wald CI:
## variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## theta      0.60000     0.15492    0.29740    0.84166 
```

In this example, the estimated success probability `theta` is 0.60.

# 4. Inspect the Posterior with MCMC

Use `sample()` when you want to inspect the full posterior distribution.
The settings below are intentionally short for a quick start. For real analyses, use more iterations.

```{r eval=FALSE}
set.seed(1)

fit_mcmc <- mdl$sample(
  sampling = 200,
  warmup = 200,
  chains = 2
)

fit_mcmc$summary()
```

```text
## Starting sequential sampling (chains = 2)...
## chain 1 started... 
## chain 1: iter 100 warmup 
## chain 1: iter 200 warmup 
## chain 1: iter 300 sampling 
## chain 1: iter 400 sampling 
## chain 2 started... 
## chain 2: iter 100 warmup 
## chain 2: iter 200 warmup 
## chain 2: iter 300 sampling 
## chain 2: iter 400 sampling 
## variable   mean    sd    map   q2.5  q97.5  ess_bulk  ess_tail  rhat 
## lp        -3.30  0.69  -2.88  -5.25  -2.80       132       188  1.00 
## theta      0.58  0.13   0.60   0.32   0.83       165       177  1.01 
```

The MCMC summary reports the mean, standard deviation, posterior quantiles, ESS, and R-hat.
As a rough diagnostic, R-hat should be close to 1 and ESS should be sufficiently large.

## Parallel MCMC

Ordinary MCMC can be run without additional setup.
However, if you use parallel MCMC with `sample(parallel = TRUE)`, the suggested packages `future`, `future.apply`, and `progressr` are required.
The `progressr` package is also used internally through `progressr::progressor()` for progress reporting.

```{r eval=FALSE}
install.packages(c("future", "future.apply", "progressr"))
```

These packages are listed in Suggests, so they are not required unless you use parallel execution.

```{r eval=FALSE}
fit_mcmc <- mdl$sample(
  sampling = 1000,
  warmup = 1000,
  chains = 4,
  parallel = TRUE
)
```

# 5. Visualize MCMC Results

MCMC results should be checked visually as well as numerically.
Use `draws()` to extract samples for a target parameter, then inspect densities, traces, autocorrelation, and intervals.

```{r eval=FALSE}
theta_draws <- fit_mcmc$draws("theta")

plot_dens(theta_draws)
plot_trace(theta_draws)
plot_acf(theta_draws)
```

Each plot has a different role.

- `plot_dens()` shows the shape of the posterior distribution.
- `plot_trace()` checks whether the chains mix well.
- `plot_acf()` checks whether autocorrelation is too strong.

For example, these functions produce plots like the following.

![Posterior density plot](plot_dens.png)

![Trace plot](plot_trace.png)

![Autocorrelation plot](plot_acf.png)

# 6. Multiple Regression with a Wrapper Function

For standard analyses, you can start from a wrapper function instead of writing `rtmb_code()` yourself.

Here we use the `debate` data and fit a multiple regression model in which satisfaction `sat` is explained by `talk`, `perf`, and their interaction.

```{r eval=FALSE}
data(debate)

mdl_lm <- rtmb_lm(
  sat ~ talk * perf,
  data = debate,
  gmc = "all",
  prior = prior_normal()
)

fit_lm <- mdl_lm$optimize(
  se_method = "sampling",
  num_samples = 1000,
  seed = 1
)
fit_lm
```

```text
## Starting RTMB optimization...
## 
## Using simulation-based error propagation (1000 samples)...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 402.24
## Approx. Log Marginal Likelihood (Laplace): -414.02
## 
## Point Estimates and 95% Sampling-based CI:
##     variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept_c    3.43325     0.05232    3.33253    3.53604 
## b[talk]        0.26572     0.05305    0.15585    0.36839 
## b[perf]        0.14054     0.02924    0.08274    0.19857 
## b[talk:perf]   0.13022     0.03063    0.06780    0.19111 
## sigma          0.87135     0.03629    0.80181    0.94375 
## Intercept      3.41638     0.05254    3.31562    3.51660 
```

Use `plot_forest()` when you want to view the coefficients at a glance.

```{r eval=FALSE}
fit_lm$draws(c("b[talk]", "b[perf]", "b[talk:perf]")) |>
  plot_forest(point_estimate = "MAP")
```

![Regression coefficient forest plot](lm_forest.png)

# 7. Plot an Interaction

Interactions are often difficult to interpret from a coefficient table alone.
Use `conditional_effects()` to visualize predicted values.

```{r eval=FALSE}
ce <- conditional_effects(fit_lm, effect = "talk:perf")
plot(ce)
```

![Conditional effect plot](conditional_effect.png)

With `effect = "talk:perf"`, you can see how the effect of `talk` changes depending on the value of `perf`.

For a more detailed look, use `simple_effects()` to calculate simple slopes.

```{r eval=FALSE}
simple_effects(fit_lm, effect = "talk:perf")
```

# 8. Run a Frequentist t Test

BayesRTMB wrapper functions can also be used for frequentist analyses.
Calling `classic()` on a t test with `prior_flat()` displays results in a form close to an ordinary t test.

```{r eval=FALSE}
mdl_t <- rtmb_ttest(
  sat ~ cond,
  data = debate,
  prior = prior_flat()
)

fit_t_classic <- mdl_t$classic()
fit_t_classic
```

```text
## Pre-checking model code...
## Checking RTMB setup...
## Starting RTMB optimization...
## 
## 
## Call:
## Classical estimation via ttest 
## 
## Log-Likelihood: -421.320, AIC: 848.640, BIC: 842.640
## 
## Point Estimates and Confidence Intervals:
##            Estimate Std. Error Lower 95% Upper 95%  df  t value     Pr
## diff       -0.37333    0.11297  -0.59564  -0.15102 298 -3.30484 .00107  **
## delta      -0.38161    0.11652  -0.61092  -0.15230 298       NA
## total_mean  3.43333    0.05648   3.32218   3.54449 298       NA
## sd          0.97831    0.04007   0.90254   1.06044 298       NA
## mean0       3.24667    0.07988   3.08947   3.40386 298       NA
## mean1       3.62000    0.07988   3.46280   3.77720 298       NA
```

Here, `diff` is the difference between the two group means, and `delta` is the standardized effect size.
Use `classic()` when you want to inspect a BayesRTMB model as a frequentist analysis.

# 9. Compute a Bayes Factor with a JZS Prior

The same t test can also be used to compute a Bayes factor by placing a Cauchy prior on the effect size `delta` through the JZS prior.

```{r eval=FALSE}
mdl_t_jzs <- rtmb_ttest(
  sat ~ cond,
  data = debate,
  prior = prior_jzs()
)

set.seed(2)

fit_t_jzs <- mdl_t_jzs$sample()

bf <- fit_t_jzs$bayes_factor(fixed = list(delta = 0))
bf
```

```text
## --- Bayes Factor Analysis (Bridge Sampling) ---
## Bayes Factor (BF12) : 21.4323
## Log Bayes Factor    : 3.0649 (Approx. Error = 0.0022)
## Evidence            : Strong evidence for Model 1 
## Comparison model    : Parameters fixed at list(delta = 0) 
```

`fixed = list(delta = 0)` specifies a comparison against the null model where the effect size is fixed at 0.
In this example, `Model 1`, the model that estimates the effect size, is supported.

For real analyses, use more MCMC samples than shown here and check that the Bayes factor is stable.

# Next Steps

This page covered only the entry points to BayesRTMB.
Continue with the page that matches your purpose.

1. **[Wrapper Functions](wrapper_functions.html)**  
   Learn how to run standard analyses such as regression, GLM, mixed models, t tests, correlations, factor analysis, and IRT with wrapper functions.

2. **[Hierarchical Models and GLMMs](rtmb_glmer.html)**  
   Learn how to use `rtmb_glmer()` for hierarchical models, GLMMs, residual correlation, conditional effects, priors, and ANOVA-style workflows.

3. **[Writing Model Codes](writing_models.html)**  
   Learn how to write custom models with the `setup`, `parameters`, `transform`, `model`, and `generate` blocks.

4. **[RTMB Internals and Inference Algorithms](rtmb_internals.html)**  
   Learn about internal processing such as MAP estimation, Laplace approximation, MCMC, and variational inference.

---
title: "BayesRTMB の概要"
description: "BayesRTMB の考え方、モデルの作り方、推定方法の全体像を紹介します。"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{BayesRTMB の概要}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

# BayesRTMB とは

**BayesRTMB** は、RTMB を計算エンジンとして用いながら、R の中で統計モデルを記述し、推定するためのパッケージです。

BayesRTMB では、標準的な分析は `rtmb_lm()` や `rtmb_glmer()` などのラッパー関数から始められます。一方で、必要に応じて `rtmb_code()` を使い、Stan に近い感覚で独自のモデルを書くこともできます。

1つのモデルオブジェクトから、MCMC、MAP 推定、変分推論、そして頻度主義的分析を呼び出せる点が BayesRTMB の基本的な考え方です。

```{r eval=FALSE}
fit_mcmc <- mdl$sample()
fit_map <- mdl$optimize()
fit_vb <- mdl$variational()
fit_freq <- mdl$classic()
```

このページでは、BayesRTMB の位置づけと、パッケージを使ううえで最初に知っておくとよい全体像を紹介します。詳しいコードの書き方や個別の分析例は、他の vignette で扱います。

## 記事へのリンク

以下の記事があります。

- **[日本語版の概要](ja-introduction.html)**
- **[クイックスタート](ja-quick_start.html)**
- **[ラッパー関数の使い方](ja-wrapper_functions.html)**
- **[階層モデル・GLMM・分散分析](ja-rtmb_glmer.html)**
- **[モデルコードの書き方](ja-writing_models.html)**
- **[RTMB の仕組みと推定アルゴリズム](ja-rtmb_internals.html)**

# BayesRTMB でできること

BayesRTMB の主な特徴は次の通りです。

- **R の中でモデルを書ける**  
  `rtmb_code()` を使い、`parameters`, `transform`, `model`, `generate` などのブロックに分けてモデルを記述できます。

- **ラッパー関数からすぐに分析できる**  
  回帰、一般化線形モデル、混合モデル、t 検定、相関、因子分析、IRT、潜在ランク理論、多次元展開法などを、専用のラッパー関数から実行できます。

- **同じモデルから複数の推定方法を使える**  
  MCMC、MAP 推定、変分推論、頻度主義的分析を、同じ `RTMB_Model` オブジェクトから呼び出せます。

- **ランダム効果を効率的に扱える**  
  `random = TRUE` と宣言したパラメータは、MAP 推定では Laplace 近似によって周辺化できます。

- **wrapper で作られたモデルを確認できる**  
  `print_code()` を使うと、ラッパー関数が内部で作っている `rtmb_code()` を確認できます。

- **モデル比較にも対応している**  
  MCMC の結果から bridge sampling による周辺尤度や Bayes factor を計算できます。

# 2つの入口

BayesRTMB には、大きく分けて2つの入口があります。

1つ目は、標準的な分析を行うための **ラッパー関数** です。通常はこちらから始めるのが簡単です。

```{r eval=FALSE}
data(debate)

mdl <- rtmb_lm(sat ~ talk + perf, data = debate)
fit <- mdl$optimize()
fit
```

2つ目は、モデルを自分で書くための **`rtmb_code()`** です。ラッパー関数では表現しにくいモデルや、新しいモデルを試したい場合に使います。

```{r eval=FALSE}
dat <- list(
  Y = debate$sat,
  X = data.matrix(debate[c("talk", "perf")])
)

code <- rtmb_code(
  setup = {
    N <- length(Y)
    P <- ncol(X)
  },
  parameters = {
    alpha = Dim()
    beta = Dim(P)
    sigma = Dim(lower = 0)
  },
  transform = {
    mu <- alpha + X %*% beta
  },
  model = {
    Y ~ normal(mu, sigma)
    alpha ~ normal(0, 10)
    beta ~ normal(0, 10)
    sigma ~ exponential(1)
  }
)

mdl <- rtmb_model(dat, code)
```

`model` ブロックでは、次のような sampling syntax を使います。

```{r eval=FALSE}
Y ~ normal(mu, sigma)
beta ~ normal(0, 10)
sigma ~ exponential(1)
```

これは内部的には対数密度を足し合わせるコードに変換されます。

# ラッパー関数から始める

まずはラッパー関数を使った例を見てみます。ここでは `debate` データを使い、満足度 `sat` を `talk` と `perf` で説明する線形回帰モデルを考えます。

```{r eval=FALSE}
data(debate)

mdl_lm <- rtmb_lm(
  sat ~ talk + perf,
  data = debate
)
```

この時点では、まだ推定は実行されていません。`mdl_lm` は、モデル定義とデータを保持した `RTMB_Model` オブジェクトです。

MAP 推定を行うには、`optimize()` を使います。

```{r eval=FALSE}
fit_map <- mdl_lm$optimize()
fit_map
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 395.58
## Approx. Log Marginal Likelihood (Laplace): -404.61
## 
## Point Estimates and 95% Wald CI:
##  variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept   1.83366     0.21105    1.42000    2.24732 
## b[talk]     0.28694     0.05291    0.18323    0.39064 
## b[perf]     0.15632     0.02987    0.09777    0.21486 
## sigma       0.90453     0.03693    0.83497    0.97988 
```

頻度主義的な線形回帰として推定したい場合は、`classic()` を使います。`classic()` は、ラッパー関数で作られた flat prior のモデルに対して利用できます。

```{r eval=FALSE}
fit_freq <- mdl_lm$classic()
fit_freq
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## Classical estimation via lm 
## 
## Log-Likelihood: -402.220, AIC: 812.440, BIC: 827.255
## 
## Point Estimates and Confidence Intervals:
##           Estimate Std. Error Lower 95% Upper 95%  df  t value     Pr    
## Intercept  1.83366    0.21212   1.41621   2.25110 297  8.64454 <.0001 ***
## b[talk]    0.28694    0.05318   0.18229   0.39159 297  5.39580 <.0001 ***
## b[perf]    0.15632    0.03002   0.09724   0.21539 297  5.20703 <.0001 ***
## sigma      0.90909    0.03730   0.83856   0.98554 297     NA
```

ラッパー関数がどのようなモデルコードを作っているかは、`print_code()` で確認できます。

```{r eval=FALSE}
mdl_lm$print_code()
```

```text
## === RTMB Model Code ===
## 
## rtmb_code(
##   setup = {
##     mf <- model.frame(formula, df)
##     Y <- model.response(mf)
##     X_full <- model.matrix(formula, mf)
##     X <- X_full[, colnames(X_full) != "(Intercept)", drop = FALSE]
##     N <- length(Y)
##     K <- ncol(X)
##   }, 
##   parameters = {
##     Intercept <- Dim(1)
##     b <- Dim(K)
##     sigma <- Dim(num_sigma_groups, lower = 0)
##   }, 
##   model = {
##     # Transform
##     eta <- Intercept + X %*% b
##     # Likelihood (Data)
##     Y ~ normal(eta, sigma)
##     # Priors
##   }
## )
```

`print_code()` は、ユーザーが `rtmb_model()` で同じモデルを再現できるように、`setup`, `parameters`, `transform`, `model` などのブロックを表示します。

# 自分でモデルを書く

より柔軟なモデルを書きたい場合は、`rtmb_code()` を使います。

`rtmb_code()` の主なブロックは次の通りです。

- `setup`: データから必要な量を前処理する
- `parameters`: 推定するパラメータを宣言する
- `transform`: パラメータから派生量や線形予測子を作る
- `model`: 尤度と事前分布を書く
- `generate`: 予測値や生成量を計算する

たとえば、`prior_normal()` に近い素直な normal / exponential prior を持つ回帰モデルは次のように書けます。

```{r eval=FALSE}
dat <- list(
  Y = as.numeric(debate$sat),
  X = data.matrix(debate[c("talk", "perf", "skill")])
)

code_reg <- rtmb_code(
  setup = {
    N <- length(Y)
    P <- ncol(X)
  },
  parameters = {
    alpha = Dim()
    beta = Dim(P)
    sigma = Dim(lower = 0)
  },
  transform = {
    mu <- alpha + X %*% beta
  },
  model = {
    Y ~ normal(mu, sigma)
    alpha ~ normal(0, 10)
    beta ~ normal(0, 2.5)
    sigma ~ exponential(1)
  },
  generate = {
    Y_rep <- rnorm(length(Y), mu, sigma)
  }
)

mdl_reg <- rtmb_model(dat, code_reg)
```

`setup` に前処理を書くことで、`parameters` や `model` ブロックを読みやすくできます。また、`print_code()` で表示したときにも、モデルの再現に必要な処理が見えるようになります。

# 推定方法の使い分け

BayesRTMB は Bayes 推定を基本にしています。そのため、ここでは MCMC、MAP 推定、変分推論、そして頻度主義的分析の順に整理します。

| メソッド | 主な目的 |
|---|---|
| `$sample()` | NUTS による MCMC サンプリングを行う |
| `$optimize()` | MAP 推定または最尤推定を高速に行う |
| `$variational()` | ADVI による近似事後分布を求める |
| `$classic()` | flat prior のラッパーモデルを頻度主義的分析として扱う |

## MCMC

`sample()` は、NUTS による MCMC サンプリングを行います。事後分布全体を見たい場合や、MAP 推定だけでは不確実性を十分に表現しにくい場合に使います。

```{r eval=FALSE}
set.seed(1)

fit_mcmc <- mdl_lm$sample()

fit_mcmc$summary()
```

```text
##  variable     mean    sd      map     q2.5    q97.5  ess_bulk  ess_tail  rhat 
## lp         -397.79  1.54  -396.75  -401.63  -395.94       859      1236  1.01
## Intercept     1.82  0.22     1.78     1.38     2.26       629      1023  1.01
## b[talk]       0.29  0.05     0.28     0.18     0.40       665      1205  1.00
## b[perf]       0.16  0.03     0.15     0.10     0.22       671       990  1.01
## sigma         0.91  0.04     0.91     0.84     1.00       934      1254  1.01
```

## MAP 推定

`optimize()` は、事後分布または尤度を最大化する点推定を行います。モデルの確認や、複雑なモデルの初期検討に向いています。

```{r eval=FALSE}
fit_map <- mdl_lm$optimize()
fit_map$summary()
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 395.58
## Approx. Log Marginal Likelihood (Laplace): -404.61
## 
## Point Estimates and 95% Wald CI:
##  variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept   1.83366     0.21105    1.42000    2.24732 
## b[talk]     0.28694     0.05291    0.18323    0.39064 
## b[perf]     0.15632     0.02987    0.09777    0.21486 
## sigma       0.90453     0.03693    0.83497    0.97988 
```

`se_method = "sampling"` を使うと、近似正規サンプルによる不確実性の伝播を使って、派生量の標準誤差や区間を計算できます。

```{r eval=FALSE}
fit_map <- mdl_lm$optimize(se_method = "sampling")
```

## 変分推論

`variational()` は、ADVI によって事後分布を近似します。MCMC より高速におおまかな結果を得たい場合に使います。ただし、事後分布の不確実性を過小評価することがあるため、最終的な不確実性評価には注意が必要です。

```{r eval=FALSE}
fit_vb <- mdl_lm$variational(
  method = "meanfield",
  iter = 3000,
  num_estimate = 4
)
```

## 頻度主義的分析

`classic()` は、ラッパー関数で作られた flat prior のモデルに対して、頻度主義的な推定結果を返します。

```{r eval=FALSE}
fit_freq <- mdl_lm$classic()
fit_freq$summary()
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## Classical estimation via lm 
## 
## Log-Likelihood: -402.220, AIC: 812.440, BIC: 827.255
## 
## Point Estimates and Confidence Intervals:
##           Estimate Std. Error Lower 95% Upper 95%  df  t value     Pr    
## Intercept  1.83366    0.21212   1.41621   2.25110 297  8.64454 <.0001 ***
## b[talk]    0.28694    0.05318   0.18229   0.39159 297  5.39580 <.0001 ***
## b[perf]    0.15632    0.03002   0.09724   0.21539 297  5.20703 <.0001 ***
## sigma      0.90909    0.03730   0.83856   0.98554 297     NA
```

`lm` や `glm` 型のモデルでは、検定統計量は通常の表記に近い形で表示されます。混合モデルでは、必要に応じて自由度補正や Laplace 近似が使われます。

# ランダム効果と Laplace 近似

階層モデルや混合モデルでは、パラメータを `random = TRUE` として宣言できます。

```{r eval=FALSE}
Y <- debate$sat
group <- as.integer(factor(debate$group))
G <- length(unique(group))

dat_icc <- list(Y = Y, group = group, G = G)

code_icc <- rtmb_code(
  parameters = {
    mu = Dim()
    sigma = Dim(lower = 0)
    tau = Dim(lower = 0)
    r = Dim(G, random = TRUE)
  },
  model = {
    Y ~ normal(mu + r[group] * tau, sigma)
    r ~ normal(0, 1)
    tau ~ exponential(1)
    sigma ~ exponential(1)
  },
  generate = {
    icc <- tau^2 / (tau^2 + sigma^2)
  }
)

mdl_icc <- rtmb_model(dat_icc, code_icc)
fit_icc <- mdl_icc$optimize(laplace = TRUE)
```

MAP 推定では、`laplace = TRUE` が既定値です。`random = TRUE` とされたパラメータは、Laplace 近似によって周辺化されます。

ラッパー関数を使う場合は、次のように混合モデルを書けます。

```{r eval=FALSE}
mdl_hlm <- rtmb_glmer(
  sat ~ talk + perf + (1 | group),
  data = debate,
  family = "gaussian"
)

fit_hlm <- mdl_hlm$optimize()
```

# モデル比較（bridge sampling / WAIC）

## bridge sampling による周辺尤度と Bayes factor

MCMC の結果に対して、bridge sampling による周辺尤度を計算できます。

```{r eval=FALSE}
set.seed(1)

fit_mcmc$bridgesampling()
```

表示は、たとえば次のように出力されます。

```text
## Bridge Sampling Converged: LogML = -404.591 (Error = 0.0032, ESS = 604.0)
```

戻り値は数値の log marginal likelihood で、`error` と `ess` の属性を持ちます。

Bayes factor を計算したい場合は、`bayes_factor()` を使います。

```{r eval=FALSE}
bf <- fit_mcmc$bayes_factor(fixed = list("b[talk]" = 0))
bf
```

```text
## --- Bayes Factor Analysis (Bridge Sampling) ---
## Bayes Factor (BF12) : 142963.4
## Log Bayes Factor    : 11.8703 (Approx. Error = 0.0040)
## Evidence            : Decisive evidence for Model 1 
## Comparison model    : Parameters fixed at list("b[talk]" = 0) 
```

## WAIC によるモデル比較

BayesRTMB では、`rtmb_code()` の `generate` ブロックで、観測ごとの対数尤度を `log_lik` という変数に格納しておくと、推定後に `$WAIC()` メソッドを使って WAIC（広く使える情報量基準）を計算できます。

たとえば、手書きモデルの `generate` ブロックに以下のように `log_lik` を定義します。

```{r eval=FALSE}
  generate = {
    # 観測ごとの対数尤度を計算して log_lik に代入する（sum = FALSE を指定）
    log_lik <- normal_lpdf(Y, mu, sigma, sum = FALSE)
  }
```

※ ラッパー関数（`rtmb_lm()` など）を使う場合は、`WAIC = TRUE` を引数に指定することで、自動的に内部で `log_lik` が生成されます。

```{r eval=FALSE}
mdl_lm <- rtmb_lm(sat ~ talk + perf, data = debate, WAIC = TRUE)
fit_mcmc <- mdl_lm$sample()

# WAIC の計算
fit_mcmc$WAIC()
```

計算結果は以下のように出力されます。

```text
## WAIC
##   WAIC (elpd scale): -400.07398
##   -2 WAIC: 800.14796
##   p_waic: 4.70921
##   lppd: -395.36477
##   nobs: 300, draws: 4000
```

`WAIC()` メソッドは、MCMC (`MCMC_Fit`) や変分推論 (`VB_Fit`)、および `se_method = "sampling"` を指定した MAP 推定 (`MAP_Fit`) の結果に対して利用できます。


# 次のステップ

BayesRTMB の全体像がつかめたら、次は目的に応じて以下の記事を読むのがおすすめです。

1. **[クイックスタート](ja-quick_start.html)**  
   基本的なモデルを実際に動かしながら、`rtmb_code()` と `rtmb_model()` の流れを確認できます。

2. **[ラッパー関数の使い方](ja-wrapper_functions.html)**  
   回帰、一般化線形モデル、混合モデル、t 検定など、標準的な分析をラッパー関数で実行する方法を確認できます。

3. **[階層モデル・GLMM・分散分析](ja-rtmb_glmer.html)**  
   `rtmb_glmer()` を使った階層モデル、GLMM、順序モデル、残差相関、可視化の流れを確認できます。

4. **[モデルコードの書き方](ja-writing_models.html)**  
   `setup`, `parameters`, `transform`, `model`, `generate` の各ブロックを使って、自分でモデルを書く方法を詳しく説明します。

5. **[RTMB の仕組みと推定アルゴリズム](ja-rtmb_internals.html)**  
   Laplace 近似、制約付きパラメータ、MCMC、変分推論などの内部処理を確認できます。

---
title: "Introduction to BayesRTMB"
description: "Introduces the ideas behind BayesRTMB, how to write models, and the overall inference workflow."
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Introduction to BayesRTMB}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

# What is BayesRTMB?

**BayesRTMB** is a package for specifying and estimating statistical models in R, using RTMB as the computational engine.

In BayesRTMB, standard analyses can start from wrapper functions such as `rtmb_lm()` and `rtmb_glmer()`. When needed, you can also use `rtmb_code()` to write custom models with a Stan-like feel.

The basic idea of BayesRTMB is that MCMC, MAP estimation, variational inference, and frequentist analysis can all be called from a single model object.

```{r eval=FALSE}
fit_mcmc <- mdl$sample()
fit_map <- mdl$optimize()
fit_vb <- mdl$variational()
fit_freq <- mdl$classic()
```

This page introduces where BayesRTMB fits and the overall picture that is useful to know when first using the package. More detailed model-writing rules and individual analysis examples are covered in the other vignettes.

## Links to Articles

The following articles are available.

- **[English Introduction](introduction.html)**
- **[Quick Start](quick_start.html)**
- **[Wrapper Functions](wrapper_functions.html)**
- **[Hierarchical Models and GLMMs](rtmb_glmer.html)**
- **[Writing Model Codes](writing_models.html)**
- **[RTMB Internals and Inference Algorithms](rtmb_internals.html)**

# What BayesRTMB Can Do

The main features of BayesRTMB are as follows.

- **Write models directly in R**  
  With `rtmb_code()`, you can specify models by separating them into blocks such as `parameters`, `transform`, `model`, and `generate`.

- **Start analysis immediately from wrapper functions**  
  Regression, generalized linear models, mixed models, t tests, correlations, factor analysis, IRT, latent rank theory, multidimensional unfolding, and other analyses can be run from specialized wrapper functions.

- **Use multiple inference methods from the same model**  
  MCMC, MAP estimation, variational inference, and frequentist analysis can all be called from the same `RTMB_Model` object.

- **Handle random effects efficiently**  
  Parameters declared with `random = TRUE` can be marginalized by Laplace approximation in MAP estimation.

- **Inspect the model created by wrappers**  
  `print_code()` lets you inspect the `rtmb_code()` object internally generated by a wrapper function.

- **Support model comparison**  
  MCMC results can be used to compute marginal likelihoods and Bayes factors by bridge sampling.

# Two Entry Points

BayesRTMB has two broad entry points.

The first is a set of **wrapper functions** for standard analyses. Usually, this is the easiest place to start.

```{r eval=FALSE}
data(debate)

mdl <- rtmb_lm(sat ~ talk + perf, data = debate)
fit <- mdl$optimize()
fit
```

The second is **`rtmb_code()`**, which is used when you want to write the model yourself. This is useful when a model is difficult to express with wrappers or when you want to try a new model.

```{r eval=FALSE}
dat <- list(
  Y = debate$sat,
  X = data.matrix(debate[c("talk", "perf")])
)

code <- rtmb_code(
  setup = {
    N <- length(Y)
    P <- ncol(X)
  },
  parameters = {
    alpha = Dim()
    beta = Dim(P)
    sigma = Dim(lower = 0)
  },
  transform = {
    mu <- alpha + X %*% beta
  },
  model = {
    Y ~ normal(mu, sigma)
    alpha ~ normal(0, 10)
    beta ~ normal(0, 10)
    sigma ~ exponential(1)
  }
)

mdl <- rtmb_model(dat, code)
```

In the `model` block, you can use sampling syntax like this.

```{r eval=FALSE}
Y ~ normal(mu, sigma)
beta ~ normal(0, 10)
sigma ~ exponential(1)
```

Internally, this is converted into code that adds log densities.

# Starting from Wrapper Functions

First, let us look at an example using a wrapper function. Here we use the `debate` data and consider a linear regression model in which satisfaction `sat` is explained by `talk` and `perf`.

```{r eval=FALSE}
data(debate)

mdl_lm <- rtmb_lm(
  sat ~ talk + perf,
  data = debate
)
```

At this point, estimation has not yet been run. `mdl_lm` is an `RTMB_Model` object that stores the model definition and data.

To run MAP estimation, use `optimize()`.

```{r eval=FALSE}
fit_map <- mdl_lm$optimize()
fit_map
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 395.58
## Approx. Log Marginal Likelihood (Laplace): -404.61
## 
## Point Estimates and 95% Wald CI:
##  variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept   1.83366     0.21105    1.42000    2.24732 
## b[talk]     0.28694     0.05291    0.18323    0.39064 
## b[perf]     0.15632     0.02987    0.09777    0.21486 
## sigma       0.90453     0.03693    0.83497    0.97988 
```

If you want to estimate the model as a frequentist linear regression, use `classic()`. `classic()` is available for wrapper models constructed with a flat prior.

```{r eval=FALSE}
fit_freq <- mdl_lm$classic()
fit_freq
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## Classical estimation via lm 
## 
## Log-Likelihood: -402.220, AIC: 812.440, BIC: 827.255
## 
## Point Estimates and Confidence Intervals:
##           Estimate Std. Error Lower 95% Upper 95%  df  t value     Pr    
## Intercept  1.83366    0.21212   1.41621   2.25110 297  8.64454 <.0001 ***
## b[talk]    0.28694    0.05318   0.18229   0.39159 297  5.39580 <.0001 ***
## b[perf]    0.15632    0.03002   0.09724   0.21539 297  5.20703 <.0001 ***
## sigma      0.90909    0.03730   0.83856   0.98554 297     NA
```

You can inspect the model code generated by a wrapper function with `print_code()`.

```{r eval=FALSE}
mdl_lm$print_code()
```

```text
## === RTMB Model Code ===
## 
## rtmb_code(
##   setup = {
##     mf <- model.frame(formula, df)
##     Y <- model.response(mf)
##     X_full <- model.matrix(formula, mf)
##     X <- X_full[, colnames(X_full) != "(Intercept)", drop = FALSE]
##     N <- length(Y)
##     K <- ncol(X)
##   }, 
##   parameters = {
##     Intercept <- Dim(1)
##     b <- Dim(K)
##     sigma <- Dim(num_sigma_groups, lower = 0)
##   }, 
##   model = {
##     # Transform
##     eta <- Intercept + X %*% b
##     # Likelihood (Data)
##     Y ~ normal(eta, sigma)
##     # Priors
##   }
## )
```

`print_code()` displays blocks such as `setup`, `parameters`, `transform`, and `model` so that users can reproduce the same model with `rtmb_model()`.

# Writing Your Own Model

When you want to write a more flexible model, use `rtmb_code()`.

The main blocks of `rtmb_code()` are as follows.

- `setup`: preprocess quantities needed from the data
- `parameters`: declare parameters to be estimated
- `transform`: create derived quantities and linear predictors from parameters
- `model`: write the likelihood and prior distributions
- `generate`: compute predictions and generated quantities

For example, a regression model with simple normal and exponential priors close to `prior_normal()` can be written as follows.

```{r eval=FALSE}
dat <- list(
  Y = as.numeric(debate$sat),
  X = data.matrix(debate[c("talk", "perf", "skill")])
)

code_reg <- rtmb_code(
  setup = {
    N <- length(Y)
    P <- ncol(X)
  },
  parameters = {
    alpha = Dim()
    beta = Dim(P)
    sigma = Dim(lower = 0)
  },
  transform = {
    mu <- alpha + X %*% beta
  },
  model = {
    Y ~ normal(mu, sigma)
    alpha ~ normal(0, 10)
    beta ~ normal(0, 2.5)
    sigma ~ exponential(1)
  },
  generate = {
    Y_rep <- rnorm(length(Y), mu, sigma)
  }
)

mdl_reg <- rtmb_model(dat, code_reg)
```

Writing preprocessing in `setup` makes the `parameters` and `model` blocks easier to read. It also makes the processing needed to reproduce the model visible when `print_code()` is displayed.

# Choosing an Inference Method

BayesRTMB is built primarily around Bayesian estimation. For that reason, this section presents the methods in the order MCMC, MAP estimation, variational inference, and frequentist analysis.

| Method | Main purpose |
|---|---|
| `$sample()` | Run MCMC sampling with NUTS |
| `$optimize()` | Run fast MAP estimation or maximum likelihood estimation |
| `$variational()` | Approximate the posterior distribution with ADVI |
| `$classic()` | Treat a flat-prior wrapper model as a frequentist analysis |

## MCMC

`sample()` runs MCMC sampling with NUTS. Use it when you want to inspect the full posterior distribution or when MAP estimation alone is not enough to represent uncertainty.

```{r eval=FALSE}
set.seed(1)

fit_mcmc <- mdl_lm$sample()

fit_mcmc$summary()
```

```text
##  variable     mean    sd      map     q2.5    q97.5  ess_bulk  ess_tail  rhat 
## lp         -397.79  1.54  -396.75  -401.63  -395.94       859      1236  1.01
## Intercept     1.82  0.22     1.78     1.38     2.26       629      1023  1.01
## b[talk]       0.29  0.05     0.28     0.18     0.40       665      1205  1.00
## b[perf]       0.16  0.03     0.15     0.10     0.22       671       990  1.01
## sigma         0.91  0.04     0.91     0.84     1.00       934      1254  1.01
```

## MAP Estimation

`optimize()` performs point estimation by maximizing the posterior distribution or likelihood. It is useful for checking a model and for initial exploration of complex models.

```{r eval=FALSE}
fit_map <- mdl_lm$optimize()
fit_map$summary()
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 395.58
## Approx. Log Marginal Likelihood (Laplace): -404.61
## 
## Point Estimates and 95% Wald CI:
##  variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept   1.83366     0.21105    1.42000    2.24732 
## b[talk]     0.28694     0.05291    0.18323    0.39064 
## b[perf]     0.15632     0.02987    0.09777    0.21486 
## sigma       0.90453     0.03693    0.83497    0.97988 
```

With `se_method = "sampling"`, standard errors and intervals for generated quantities can be calculated by propagating uncertainty through approximate normal samples.

```{r eval=FALSE}
fit_map <- mdl_lm$optimize(se_method = "sampling")
```

## Variational Inference

`variational()` approximates the posterior distribution using ADVI. It is useful when you want rough results faster than MCMC. However, it can underestimate posterior uncertainty, so caution is needed when using it for final uncertainty assessment.

```{r eval=FALSE}
fit_vb <- mdl_lm$variational(
  method = "meanfield",
  iter = 3000,
  num_estimate = 4
)
```

## Frequentist Analysis

`classic()` returns frequentist estimation results for flat-prior models constructed by wrapper functions.

```{r eval=FALSE}
fit_freq <- mdl_lm$classic()
fit_freq$summary()
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## Classical estimation via lm 
## 
## Log-Likelihood: -402.220, AIC: 812.440, BIC: 827.255
## 
## Point Estimates and Confidence Intervals:
##           Estimate Std. Error Lower 95% Upper 95%  df  t value     Pr    
## Intercept  1.83366    0.21212   1.41621   2.25110 297  8.64454 <.0001 ***
## b[talk]    0.28694    0.05318   0.18229   0.39159 297  5.39580 <.0001 ***
## b[perf]    0.15632    0.03002   0.09724   0.21539 297  5.20703 <.0001 ***
## sigma      0.90909    0.03730   0.83856   0.98554 297     NA
```

For `lm`- and `glm`-type models, test statistics are displayed in a form close to the usual notation. For mixed models, degrees-of-freedom corrections and Laplace approximation are used when needed.

# Random Effects and Laplace Approximation

In hierarchical and mixed models, parameters can be declared with `random = TRUE`.

```{r eval=FALSE}
Y <- debate$sat
group <- as.integer(factor(debate$group))
G <- length(unique(group))

dat_icc <- list(Y = Y, group = group, G = G)

code_icc <- rtmb_code(
  parameters = {
    mu = Dim()
    sigma = Dim(lower = 0)
    tau = Dim(lower = 0)
    r = Dim(G, random = TRUE)
  },
  model = {
    Y ~ normal(mu + r[group] * tau, sigma)
    r ~ normal(0, 1)
    tau ~ exponential(1)
    sigma ~ exponential(1)
  },
  generate = {
    icc <- tau^2 / (tau^2 + sigma^2)
  }
)

mdl_icc <- rtmb_model(dat_icc, code_icc)
fit_icc <- mdl_icc$optimize(laplace = TRUE)
```

In MAP estimation, `laplace = TRUE` is the default. Parameters declared with `random = TRUE` are marginalized by Laplace approximation.

With wrapper functions, a mixed model can be written as follows.

```{r eval=FALSE}
mdl_hlm <- rtmb_glmer(
  sat ~ talk + perf + (1 | group),
  data = debate,
  family = "gaussian"
)

fit_hlm <- mdl_hlm$optimize()
```

# Model Comparison (Bridge Sampling / WAIC)

## Marginal Likelihood and Bayes Factor by Bridge Sampling

For MCMC results, bridge sampling can be used to compute the marginal likelihood.

```{r eval=FALSE}
set.seed(1)

fit_mcmc$bridgesampling()
```

For example, the output is displayed as follows.

```text
## Bridge Sampling Converged: LogML = -404.591 (Error = 0.0032, ESS = 604.0)
```

The returned value is a numeric log marginal likelihood with `error` and `ess` attributes.

Use `bayes_factor()` when you want to compute a Bayes factor.

```{r eval=FALSE}
bf <- fit_mcmc$bayes_factor(fixed = list("b[talk]" = 0))
bf
```

```text
## --- Bayes Factor Analysis (Bridge Sampling) ---
## Bayes Factor (BF12) : 142963.4
## Log Bayes Factor    : 11.8703 (Approx. Error = 0.0040)
## Evidence            : Decisive evidence for Model 1 
## Comparison model    : Parameters fixed at list("b[talk]" = 0) 
```

## Model Comparison by WAIC

In BayesRTMB, if you store the pointwise log likelihood for each observation in a variable named `log_lik` within the `generate` block of `rtmb_code()`, you can calculate the WAIC (Widely Applicable Information Criterion) using the `$WAIC()` method after estimation.

For example, define `log_lik` in the `generate` block of a custom model as follows:

```{r eval=FALSE}
  generate = {
    # Compute the pointwise log likelihood and assign it to log_lik (specify sum = FALSE)
    log_lik <- normal_lpdf(Y, mu, sigma, sum = FALSE)
  }
```

*Note: When using wrapper functions (e.g., `rtmb_lm()`), you can specify `WAIC = TRUE` in the arguments to automatically generate `log_lik` internally.*

```{r eval=FALSE}
mdl_lm <- rtmb_lm(sat ~ talk + perf, data = debate, WAIC = TRUE)
fit_mcmc <- mdl_lm$sample()

# Compute WAIC
fit_mcmc$WAIC()
```

The output will be displayed as follows:

```text
## WAIC
##   WAIC (elpd scale): -400.07398
##   -2 WAIC: 800.14796
##   p_waic: 4.70921
##   lppd: -395.36477
##   nobs: 300, draws: 4000
```

The `$WAIC()` method is available for results from MCMC (`MCMC_Fit`), variational inference (`VB_Fit`), and MAP estimation (`MAP_Fit`) with `se_method = "sampling"`.


# Next Steps

Once you have a general picture of BayesRTMB, the next step is to read the articles that match your purpose.

1. **[Quick Start](quick_start.html)**  
   Walk through a basic model and learn the flow of `rtmb_code()` and `rtmb_model()`.

2. **[Wrapper Functions](wrapper_functions.html)**  
   Learn how to run standard analyses such as regression, generalized linear models, mixed models, and t tests with wrapper functions.

3. **[Hierarchical Models and GLMMs](rtmb_glmer.html)**  
   Learn how to use `rtmb_glmer()` for hierarchical models, GLMMs, residual correlation, ordinal models, priors, and visualization.

4. **[Writing Model Codes](writing_models.html)**  
   Learn how to write your own models using the `setup`, `parameters`, `transform`, `model`, and `generate` blocks.

5. **[RTMB Internals and Inference Algorithms](rtmb_internals.html)**  
   Learn about internal processing such as Laplace approximation, constrained parameters, MCMC, and variational inference.

---
title: "Writing Model Codes"
description: "A guide on how to write model codes in BayesRTMB."
output: 
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Writing Model Codes}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

This page explains how to describe models using the `rtmb_code` block, which is the core of BayesRTMB, and details the available probability distributions and mathematical functions.

## 1. Overview of the Structure and Role of rtmb_code

`rtmb_code()` is a function that allows you to define Bayesian models using an intuitive syntax similar to Stan. A model is described by dividing it into several "blocks" according to their purposes.

```{r eval=FALSE}
library(BayesRTMB)

code <- rtmb_code(
  setup = {
    # Data preprocessing and constant definitions
  },
  parameters = {
    # Declaration of parameters to be estimated
  },
  transform = {
    # Calculation of derived variables using parameters and data
  },
  model = {
    # Definition of priors and likelihoods
  },
  generate = {
    # Calculation of posterior predictions and generated quantities
  }
)
```

### 1-1. parameters block
This block is for declaring the parameters you want to estimate in the model. You use the `Dim()` function to specify the dimension and constraints (e.g., lower and upper bounds) of the variables.

**Example Code:**
```{r eval=FALSE}
parameters = {
  # Scalar parameter (with lower bound of 0)
  sigma = Dim(lower = 0)
  
  # Vector parameter (length N)
  mu = Dim(N)
  
  # Matrix parameter (N rows, M columns)
  X = Dim(N, M)
}
```
Here we are defining generic continuous parameters, but by using the `type` argument you can define parameters (types) with more complex structures, such as "correlation matrices" or "sum-to-zero vectors" (see "3. About Parameter Types (Dim)" below for details).

### 1-2. model block
This is the heart of the model where you define priors and the likelihood (the data generating process). Similar to Stan, using the `~` (tilde) sampling syntax is recommended.

```{r eval=FALSE}
model = {
  # Priors
  mu ~ normal(0, 10)
  sigma ~ exponential(1)

  # Likelihood
  Y ~ normal(mu, sigma)
}
```

The same model can also be written by explicitly adding log densities:

```{r eval=FALSE}
model = {
  lp <- lp + normal_lpdf(mu, 0, 10)
  lp <- lp + exponential_lpdf(sigma, 1)
  lp <- lp + normal_lpdf(Y, mu, sigma)
}
```

In other words, `Y ~ normal(mu, sigma)` and `lp <- lp + normal_lpdf(Y, mu, sigma)` are two ways to write the same model. The `~` syntax is usually easier to read, while `lp <- lp + ..._lpdf(...)` is useful when you need to build a custom likelihood. `lp` is reserved for accumulating log probability, so do not declare it in the `parameters` block or use it as an ordinary working variable.

### 1-3. setup block
This block is executed exactly once before the model computations begin. It is primarily used to extract data sizes (number of rows/columns) as variables from the `data` list passed from R, and to create constants required for computations.

**Example Code:**
```{r eval=FALSE}
setup = {
  N <- length(Y)  # Get the sample size of the observed data
  P <- ncol(X)    # Get the number of columns in the design matrix (number of predictors)
}
```

### 1-4. transform block
This block is for creating derived variables (such as the mean `mu` or linear predictors) necessary for likelihood calculations from parameters and data. The variables created here can be used directly within the `model` block.

> [!IMPORTANT]
> The derived variables calculated here are automatically saved inside the model object. Not only can you obtain samples of their posterior distributions after estimation, but when you run MAP estimation (`optimize`), standard errors and 95% confidence intervals for these derived variables are also automatically calculated and output using the delta method.

### 1-5. generate block
This block is used for creating predictive distributions for Posterior Predictive Checks (PPC) and for generating new quantities of interest calculated from estimated parameters.

> [!IMPORTANT]
> Computations written in the `generate` block are not included in the likelihood evaluation (Automatic Differentiation calculations) and can be executed post-hoc after the estimation is fully complete. Therefore, no matter how computationally heavy the operations written here are, they will not negatively affect the estimation speed (MAP or MCMC).
> Also, if you forgot to write something during model definition or later decide you want to compute another quantity, you can calculate these retroactively by calling the `mdl$generated_quantities(new_generate_code)` method on the estimated model object.

## 2. Available Probability Distributions

In BayesRTMB, you can use many probability distributions within the `model` block. Most univariate distributions are vectorized, meaning if `Y` and `mu` are vectors, a single statement efficiently calculates the sum of log-densities for all elements.

### 2-1. Common Probability Density and Mass Functions

**Continuous Probability Distributions (LPDF)**

| Function | Description |
| :--- | :--- |
| `normal(mean, sd)` | Normal distribution. |
| `lognormal(meanlog, sdlog)` | Log-normal distribution. |
| `exponential(rate)` | Exponential distribution. |
| `cauchy(location, scale)` | Cauchy distribution. |
| `student_t(df, mu, sigma)` | Student's t-distribution. |
| `gamma(shape, rate)` | Gamma distribution. |
| `inverse_gamma(shape, scale)` | Inverse-gamma distribution. |
| `beta(a, b)` | Beta distribution. |

**Discrete Probability Distributions (LPMF)**

| Function | Description |
| :--- | :--- |
| `bernoulli(prob)` | Bernoulli distribution (binary data). |
| `binomial(size, prob)` | Binomial distribution. |
| `poisson(mean)` | Poisson distribution (count data). |
| `neg_binomial_2(mu, size)` | Negative binomial distribution (mean-variance parameterization). |

### 2-2. Advanced Probability Distributions

Distributions are also provided to improve computational stability or for specific modeling purposes.

| Function | Description |
| :--- | :--- |
| `bernoulli_logit(eta)` / `binomial_logit(size, eta)` | Functions that take the linear predictor (`eta`) on the logit scale directly instead of probabilities. Numerically stable. |
| `ordered_logistic(eta, cutpoints)` | Used for ordinal categorical data, such as in ordered logistic regression. |
| `lkj_corr(eta)` | LKJ prior distribution for correlation matrices. |

### 2-3. Probability Distributions for Special Types

Distributions tailored for matrices, multivariate data, and specific model structures.

| Function | Description |
| :--- | :--- |
| `multi_normal(mean, Sigma)` | Standard multivariate normal distribution. |
| `multi_normal_CF(mean, sd, CF_Omega)` | Multivariate normal distribution using a Cholesky factor parameterization. |
| `dirichlet(alpha)` | Dirichlet distribution for a simplex (a vector summing to 1). |
| `lower_tri_normal(mean, sd)` | Normal distribution over the elements of a lower triangular matrix. |
| `centered_tri_multi_normal(sigma)` | Multivariate normal distribution for a centered triangular matrix (used for identifiability constraints). |
| `sufficient_multi_normal_fa(S_mat, N, y_bar, mu, psi, Lambda)` | Likelihood for factor analysis using sufficient statistics. Extremely computationally efficient for large samples. |
| `normal_mixture(pi_w, mean, sd)` | Normal mixture distribution combining multiple normal distributions. |
| `wishart(n, V)` | Wishart distribution. |

## 3. About Parameter Types (Dim)

In the `parameters` block, you define the types and dimensions of parameters using `Dim()`.

* **Scalar**: `Dim()` or `Dim(1)`
* **Vector**: `Dim(N)`
* **Matrix**: `Dim(N, M)`

You can also enforce various constraints using arguments:
* `lower = 0, upper = 1`: Specifies the range of values (e.g., for variances or probabilities).
* `random = TRUE`: Designates the parameter as a random effect subject to Laplace approximation in hierarchical models.

Furthermore, by specifying a specific keyword in the `type` argument, you can define complex types with specific structures.

| `type` Specification | Description |
| :--- | :--- |
| `type = "simplex"` | A vector where all elements are positive and sum to 1 (used for probability assignments, etc.). |
| `type = "centered"` | A vector where the sum of all elements is zero (used for main effects in ANOVA-type models, etc.). |
| `type = "corr_matrix"` | A correlation matrix with 1s on the diagonal satisfying positive definiteness. |
| `type = "CF_corr"` | The Cholesky factor of a correlation matrix (sometimes more computationally efficient than the correlation matrix itself). |
| `type = "lower_tri"` | A lower triangular matrix. |
| `type = "centered_tri"` | A lower triangular matrix constrained so the column sums are zero (useful for identification constraints in factor analysis). |

## 4. About Mathematical Functions

Numerous numerically stable utility functions are available to prevent overflow and underflow during Automatic Differentiation (AD) calculations.

**Link and Inverse Link Functions**

| Function | Description |
| :--- | :--- |
| `logit(x)` | Computes the logit transformation `log(x/(1-x))`. |
| `inv_logit(x)` | Computes the inverse logit (logistic) transformation `1/(1+exp(-x))`. |

**Functions for Numerical Stabilization**

| Function | Description |
| :--- | :--- |
| `log_sum_exp(x)` | Safely computes `log(sum(exp(x)))` using the log-sum-exp trick. |
| `log1p_exp(x)` | Stably computes `log(1 + exp(x))`. |
| `log1m_exp(x)` | Stably computes `log(1 - exp(x))` for `x < 0`. |
| `log_softmax(x)` | Computes the logarithm of the softmax function. |
| `fabs(x)` | A smoothed version of `abs(x)` that guarantees differentiability near zero. |

**Matrix and Vector Transformations**

These are useful when transforming an unconstrained vector into a matrix with a specific structure.

| Function | Description |
| :--- | :--- |
| `centered(x)` | Transforms a vector of length K-1 into a vector of length K that sums to zero. |
| `to_lower_tri(x, M, D)` | Creates an M x D lower triangular matrix from the vector `x`. |
| `to_centered_matrix(x, R, C)` | Creates an R x C matrix where the sum of each column is zero. |
| `to_centered_tri(x, R, C)` | Creates a matrix with lower triangular elements whose column sums are zero, used for identification constraints in factor analysis. |

**Linear Algebra for AD**

| Function | Description |
| :--- | :--- |
| `log_det_chol(L)` | Computes the log-determinant of a covariance matrix from its Cholesky factor `L`. |
| `quad_form_chol(x, L)` | Computes a quadratic form using the Cholesky factor `L`. |
| `distance(x, y)` | Computes the Euclidean distance between two vectors, adding a small epsilon for numerical stability. |

## Conclusion
Rather than writing a complex model from the beginning, it's better to start estimating with simple models first. 
The error messages are still under development, and we plan to make them more user-friendly in the future.

For standard wrapper models, see [Wrapper Functions](wrapper_functions.html). For mixed models and GLMMs in particular, see [Hierarchical Models and GLMMs](rtmb_glmer.html). For the internal inference pipeline, see [RTMB Internals and Inference Algorithms](rtmb_internals.html).

---
title: "BayesRTMB クイックスタート"
description: "BayesRTMB の最小ワークフロー、MCMC の確認、ラッパー関数、可視化、t 検定の入口を短く紹介します。"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{BayesRTMB 日本語クイックスタート}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

# このページの目的

このページでは、BayesRTMB を使い始めるための最小限の流れを確認します。

ここで扱うのは、次の内容です。

1. `rtmb_code()` で小さなモデルを書く
2. `rtmb_model()` でモデルオブジェクトを作る
3. `optimize()` と `sample()` で推定する
4. MCMC の事後分布を可視化する
5. ラッパー関数で重回帰と交互作用を扱う
6. `classic()` による頻度主義的な t 検定を行う
7. JZS prior と MCMC によって Bayes factor を計算する

詳しいモデルコードの書き方や、混合モデル、GLMM、モデル比較の詳細は、別の vignette で扱います。

# 0. インストールと環境確認

BayesRTMB は CRAN からインストールできます。

```{r eval=FALSE}
install.packages("BayesRTMB")
```

開発版は GitHub からインストールできます。
`pak` を使う場合は、次のようにします。

```{r eval=FALSE}
install.packages("pak")
pak::pak("norimune/BayesRTMB")
```

`remotes` を使う場合は、次のようにします。

```{r eval=FALSE}
install.packages("remotes")
remotes::install_github("norimune/BayesRTMB")
```

## Windows ユーザー向け

BayesRTMB は内部で RTMB / TMB を利用します。
通常利用では、Windows ユーザーも CRAN のバイナリ版をインストールすれば Rtools は不要です。
Rtools が必要になるのは、ソースからインストールする場合、パッケージ開発を行う場合、または通常の TMB のように独自の C++ テンプレートをコンパイルする場合です。

BayesRTMB をソースからインストールする場合は、Rtools が利用できるかどうかを次のコードで確認できます。

```{r eval=FALSE}
pkgbuild::check_build_tools(debug = TRUE)
```

`TRUE` が返る、またはビルドツールが利用可能であることが表示されれば、ソースインストールの準備はできています。
Rtools が見つからない場合は、利用している R のバージョンに対応する Rtools をインストールし、R または RStudio を再起動してください。

# 1. 最小モデルを書く

まず、もっとも小さな例として二項モデルを書きます。
ここでは、10 回の試行のうち成功が 6 回観測された状況を考えます。

```{r eval=FALSE}
library(BayesRTMB)

Trial <- 10
Y <- 6

dat <- list(Trial = Trial, Y = Y)

code <- rtmb_code(
  parameters = {
    theta <- Dim(lower = 0, upper = 1)
  },
  model = {
    Y ~ binomial(Trial, theta)
    theta ~ beta(1, 1)
  }
)
```

`parameters` ブロックでは推定するパラメータを宣言します。
ここでは成功確率 `theta` を、0 から 1 の範囲を持つパラメータとして定義しています。

`model` ブロックでは、観測データの分布と事前分布を書きます。
`Y ~ binomial(Trial, theta)` は、成功数 `Y` が二項分布に従うことを表しています。

# 2. モデルオブジェクトを作る

`rtmb_model()` にデータとモデルコードを渡すと、推定用のモデルオブジェクトが作られます。

```{r eval=FALSE}
mdl <- rtmb_model(dat, code)
```

```text
## Pre-checking model code...
## Checking RTMB setup...
```

この段階では、まだ推定は行われていません。
`mdl` は、モデル定義とデータを保持した `RTMB_Model` オブジェクトです。

# 3. MAP 推定を行う

点推定をすばやく得たいときは、`optimize()` を使います。
BayesRTMB では、事前分布がある場合は MAP 推定、flat prior の場合は最尤推定に近い推定として扱えます。

```{r eval=FALSE}
fit_map <- mdl$optimize()
fit_map
```

```text
## Starting RTMB optimization...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 1.38
## Approx. Log Marginal Likelihood (Laplace): -2.33
## 
## Point Estimates and 95% Wald CI:
## variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## theta      0.60000     0.15492    0.29740    0.84166 
```

この例では、成功確率 `theta` の推定値はちょうど 0.60 です。

# 4. MCMC で事後分布を見る

事後分布全体を見たいときは、`sample()` を使います。
クイックスタートでは短い設定にしていますが、実際の分析ではより多くの反復数を使ってください。

```{r eval=FALSE}
set.seed(1)

fit_mcmc <- mdl$sample(
  sampling = 200,
  warmup = 200,
  chains = 2
)

fit_mcmc$summary()
```

```text
## Starting sequential sampling (chains = 2)...
## chain 1 started... 
## chain 1: iter 100 warmup 
## chain 1: iter 200 warmup 
## chain 1: iter 300 sampling 
## chain 1: iter 400 sampling 
## chain 2 started... 
## chain 2: iter 100 warmup 
## chain 2: iter 200 warmup 
## chain 2: iter 300 sampling 
## chain 2: iter 400 sampling 
## variable   mean    sd    map   q2.5  q97.5  ess_bulk  ess_tail  rhat 
## lp        -3.30  0.69  -2.88  -5.25  -2.80       132       188  1.00 
## theta      0.58  0.13   0.60   0.32   0.83       165       177  1.01 
```

MCMC の結果では、平均、標準偏差、事後分位点、ESS、R-hat などを確認できます。
R-hat が 1 に近く、ESS が十分に大きいほど、MCMC の診断としては安心しやすくなります。

## MCMC を並列化する場合

通常の MCMC は追加設定なしで実行できます。
ただし、`sample(parallel = TRUE)` のように MCMC を並列実行する場合は、追加で `future`, `future.apply`, `progressr` が必要です。
`progressr` は、内部で `progressr::progressor()` による進捗表示にも使われます。

```{r eval=FALSE}
install.packages(c("future", "future.apply", "progressr"))
```

これらは BayesRTMB の Suggests パッケージなので、並列化を使わない場合は必須ではありません。

```{r eval=FALSE}
fit_mcmc <- mdl$sample(
  sampling = 1000,
  warmup = 1000,
  chains = 4,
  parallel = TRUE
)
```

# 5. MCMC の結果を可視化する

MCMC の結果は、数値だけでなく図でも確認します。
`draws()` で対象パラメータのサンプルを取り出し、密度、トレース、自己相関、区間推定を確認できます。

```{r eval=FALSE}
theta_draws <- fit_mcmc$draws("theta")

plot_dens(theta_draws)
plot_trace(theta_draws)
plot_acf(theta_draws)
```

それぞれの図は、次の目的で使います。

- `plot_dens()` は、事後分布の形を確認します。
- `plot_trace()` は、チェインがよく混ざっているかを確認します。
- `plot_acf()` は、自己相関が強すぎないかを確認します。

実際には、たとえば次のような図として確認できます。

![Posterior density plot](plot_dens.png)

![Trace plot](plot_trace.png)

![Autocorrelation plot](plot_acf.png)

# 6. ラッパー関数で重回帰を行う

標準的な分析では、`rtmb_code()` を自分で書かずにラッパー関数から始められます。

ここでは、`debate` データを使い、満足度 `sat` を `talk`、`perf`、およびその交互作用で説明する重回帰モデルを推定します。

```{r eval=FALSE}
data(debate)

mdl_lm <- rtmb_lm(
  sat ~ talk * perf,
  data = debate,
  gmc = "all",
  prior = prior_normal()
)

fit_lm <- mdl_lm$optimize(
  se_method = "sampling",
  num_samples = 1000,
  seed = 1
)
fit_lm
```

```text
## Starting RTMB optimization...
## 
## Using simulation-based error propagation (1000 samples)...
## 
## 
## Call:
## MAP Estimation via RTMB
## 
## Negative Log-Posterior: 402.24
## Approx. Log Marginal Likelihood (Laplace): -414.02
## 
## Point Estimates and 95% Sampling-based CI:
##     variable  Estimate  Std. Error  Lower 95%  Upper 95% 
## Intercept_c    3.43325     0.05232    3.33253    3.53604 
## b[talk]        0.26572     0.05305    0.15585    0.36839 
## b[perf]        0.14054     0.02924    0.08274    0.19857 
## b[talk:perf]   0.13022     0.03063    0.06780    0.19111 
## sigma          0.87135     0.03629    0.80181    0.94375 
## Intercept      3.41638     0.05254    3.31562    3.51660 
```

係数を一覧したいときは、`plot_forest()` が便利です。

```{r eval=FALSE}
fit_lm$draws(c("b[talk]", "b[perf]", "b[talk:perf]")) |>
  plot_forest(point_estimate = "MAP")
```

![Regression coefficient forest plot](lm_forest.png)

# 7. 交互作用を図で確認する

交互作用は、係数表だけでは解釈しにくいことがあります。
その場合は、`conditional_effects()` を使って予測値を図示します。

```{r eval=FALSE}
ce <- conditional_effects(fit_lm, effect = "talk:perf")
plot(ce)
```

![Conditional effect plot](conditional_effect.png)

`effect = "talk:perf"` と書くと、`talk` の効果が `perf` の値によってどのように変わるかを確認できます。

より詳しく調べたい場合は、単純傾斜を計算する `simple_effects()` も使えます。

```{r eval=FALSE}
simple_effects(fit_lm, effect = "talk:perf")
```

# 8. t 検定を頻度主義的に行う

BayesRTMB のラッパー関数は、頻度主義的な分析にも使えます。
`prior_flat()` を使った t 検定に対して `classic()` を呼ぶと、通常の t 検定に近い形で結果を表示できます。

```{r eval=FALSE}
mdl_t <- rtmb_ttest(
  sat ~ cond,
  data = debate,
  prior = prior_flat()
)

fit_t_classic <- mdl_t$classic()
fit_t_classic
```

```text
## Pre-checking model code...
## Checking RTMB setup...
## Starting RTMB optimization...
## 
## 
## Call:
## Classical estimation via ttest 
## 
## Log-Likelihood: -421.320, AIC: 848.640, BIC: 842.640
## 
## Point Estimates and Confidence Intervals:
##            Estimate Std. Error Lower 95% Upper 95%  df  t value     Pr
## diff       -0.37333    0.11297  -0.59564  -0.15102 298 -3.30484 .00107  **
## delta      -0.38161    0.11652  -0.61092  -0.15230 298       NA
## total_mean  3.43333    0.05648   3.32218   3.54449 298       NA
## sd          0.97831    0.04007   0.90254   1.06044 298       NA
## mean0       3.24667    0.07988   3.08947   3.40386 298       NA
## mean1       3.62000    0.07988   3.46280   3.77720 298       NA
```

ここでは、`diff` が2群の平均差、`delta` が標準化効果量を表します。
`classic()` は、BayesRTMB のモデルを頻度主義的な推定として確認したいときに使えます。

# 9. JZS prior で Bayes factor を計算する

同じ t 検定でも、JZS prior を使うと、効果量 `delta` に Cauchy 事前分布を置いた Bayes factor を計算できます。

```{r eval=FALSE}
mdl_t_jzs <- rtmb_ttest(
  sat ~ cond,
  data = debate,
  prior = prior_jzs()
)

set.seed(2)

fit_t_jzs <- mdl_t_jzs$sample()

bf <- fit_t_jzs$bayes_factor(fixed = list(delta = 0))
bf
```

```text
## --- Bayes Factor Analysis (Bridge Sampling) ---
## Bayes Factor (BF12) : 21.4323
## Log Bayes Factor    : 3.0649 (Approx. Error = 0.0022)
## Evidence            : Strong evidence for Model 1 
## Comparison model    : Parameters fixed at list(delta = 0) 
```

`fixed = list(delta = 0)` は、効果量を 0 に固定した帰無モデルと比較する指定です。
この例では、`Model 1`、つまり効果量を推定するモデルのほうが支持されています。

実際の分析では、Bayes factor の安定性を確認するために、ここで示した例よりも多い MCMC サンプル数を使うことをおすすめします。

# 次に読むページ

このページでは、BayesRTMB の入口だけを扱いました。
目的に応じて、次のページに進んでください。

1. **[ラッパー関数の使い方](ja-wrapper_functions.html)**  
   回帰、GLM、混合モデル、t 検定、相関、因子分析、IRT など、標準的な分析をラッパー関数で行う方法を確認できます。

2. **[階層モデル・GLMM・分散分析](ja-rtmb_glmer.html)**  
   `rtmb_glmer()` を使った階層モデル、GLMM、残差相関、条件付き効果の可視化を詳しく確認できます。

3. **[モデルコードの書き方](ja-writing_models.html)**  
   `rtmb_code()` の `setup`, `parameters`, `transform`, `model`, `generate` を使って、独自モデルを書く方法を学べます。

4. **[RTMB の仕組みと推定アルゴリズム](ja-rtmb_internals.html)**  
   MAP 推定、Laplace 近似、MCMC、変分推論などの内部処理を確認できます。

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{plot.ce_rtmb}
\alias{plot.ce_rtmb}
\title{Plot method for ce_rtmb class (Base R)}
\usage{
\method{plot}{ce_rtmb}(x, ...)
}
\arguments{
\item{x}{An object of class ce_rtmb}

\item{...}{Additional arguments.}
}
\description{
Plot method for ce_rtmb class (Base R)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_prior.R
\name{prior_rhs}
\alias{prior_rhs}
\title{Specify a Regularized Horseshoe prior for continuous shrinkage}
\usage{
prior_rhs(expected_vars = 3, slab_scale = 2, slab_df = 4, ...)
}
\arguments{
\item{expected_vars}{Expected number of non-zero variables. Default is 3.}

\item{slab_scale}{Scale parameter for the slab distribution. Default is 2.0.}

\item{slab_df}{Degrees of freedom for the slab distribution. Default is 4.0.}

\item{...}{Optional hyperparameters for other distributions.}
}
\value{
A list with class "rtmb_prior"
}
\description{
Specify a Regularized Horseshoe prior for continuous shrinkage
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{training}
\alias{training}
\title{Social Skills Training Data}
\format{
A data frame with 12 rows and 8 columns:
\describe{
  \item{ID}{Participant identifier.}
  \item{time1}{Outcome score at the first measurement occasion.}
  \item{time2}{Outcome score at the second measurement occasion.}
  \item{time3}{Outcome score at the third measurement occasion.}
  \item{time4}{Outcome score at the fourth measurement occasion.}
  \item{a}{Training condition indicator: 0 = control group, 1 = intervention group.}
  \item{b}{Qualitative baseline skill classification.}
  \item{age}{Participant age.}
}
}
\source{
Simulated dummy data created by the package author.
}
\usage{
training
}
\description{
A small dataset containing repeated outcome measurements from a social
skills training study. The outcome was measured at four time points for
participants assigned to a control or intervention condition.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{conditional_effects.mcmc_fit}
\alias{conditional_effects.mcmc_fit}
\title{Calculate conditional effects for MCMC fit objects}
\usage{
\method{conditional_effects}{mcmc_fit}(
  fit,
  effect,
  prob = 0.95,
  sd_multiplier = 1,
  resolution = 100,
  ...
)
}
\arguments{
\item{fit}{An object of class `MCMC_Fit`.}

\item{effect}{Name of the explanatory variable to visualize (e.g., "X1" or "X1:X2").}

\item{prob}{Probability for the credible/confidence interval (default is 0.95).}

\item{sd_multiplier}{Numeric. Multiplier for standard deviation when splitting continuous moderators (default is 1).}

\item{resolution}{Grid resolution to calculate for continuous variables (default is 100).}

\item{...}{Additional arguments.}
}
\value{
A `ce_rtmb` object.
}
\description{
Calculate conditional effects for MCMC fit objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{summary.ce_rtmb}
\alias{summary.ce_rtmb}
\title{Summary method for ce_rtmb class}
\usage{
\method{summary}{ce_rtmb}(object, ...)
}
\arguments{
\item{object}{An object of class ce_rtmb}

\item{...}{Additional arguments.}
}
\description{
Summary method for ce_rtmb class
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{log_mix}
\alias{log_mix}
\title{Log mixture of two probabilities}
\usage{
log_mix(theta, lp1, lp2)
}
\arguments{
\item{theta}{Mixing proportion (between 0 and 1).}

\item{lp1}{Log-probability of the first component.}

\item{lp2}{Log-probability of the second component.}
}
\value{
The log-probability of the mixture.
}
\description{
Log mixture of two probabilities
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_item_info}
\alias{plot_item_info}
\title{Plot item information functions}
\usage{
plot_item_info(fit, ...)
}
\arguments{
\item{fit}{A fitted IRT model object.}

\item{...}{Additional arguments passed to \code{\link{item_info}} or the plot method.}
}
\value{
Invisibly returns the plotted object of class \code{rtmb_item_info}.
}
\description{
Calculate and plot item information functions for a fitted IRT model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_mixture.R
\name{rtmb_mixture}
\alias{rtmb_mixture}
\title{Mixture Model Wrapper for RTMB}
\usage{
rtmb_mixture(
  formula,
  k = 2,
  data = NULL,
  covariance = c("diagonal", "diagonal_equal", "full", "full_equal", "full_equal_corr"),
  prior = prior_flat(),
  y_range = NULL,
  fixed = NULL,
  WAIC = FALSE,
  ...
)
}
\arguments{
\item{formula}{A formula specifying the response variable(s). For multivariate, use \code{cbind(y1, y2) ~ 1}.}

\item{k}{Number of mixture components.}

\item{data}{A data frame containing the variables in the model.}

\item{covariance}{Covariance structure: "diagonal" (default), "diagonal_equal", "full", "full_equal", or "full_equal_corr".}

\item{prior}{Prior configuration: `prior_flat()`, `prior_normal()`,
`prior_weak()`, `prior_rhs()`, or `prior_ssp()`. Default is
`prior_flat()`. If `y_range` is supplied with the default flat prior,
the wrapper automatically switches to `prior_weak()`.}

\item{y_range}{Optional numeric vector or matrix defining the theoretical range (min, max) of response variables.
Specifying this automatically enables weakly informative priors if `prior` is `prior_flat()`.}

\item{fixed}{Optional named list of fixed values for specific parameters.}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}

\item{...}{Additional arguments passed to `rtmb_model`.}
}
\value{
A \code{RTMB_Model} object.
}
\description{
Provides a user-friendly interface for fitting Gaussian mixture models
with optional covariates on class membership probabilities and various
covariance structures.
}
\examples{

  # Simulate 1D mixture data (2 components)
  set.seed(123)
  N <- 100
  group <- rbinom(N, 1, 0.4)
  y <- ifelse(group == 1, rnorm(N, 5, 1), rnorm(N, 0, 1))
  dat <- data.frame(y)

  # Fit a 1D Gaussian mixture model with 2 components
  fit_mix <- rtmb_mixture(y ~ 1, k = 2, data = dat)
  
  # MAP estimation
  map_mix <- fit_mix$optimize()
  map_mix$summary()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{half_normal_lpdf}
\alias{half_normal_lpdf}
\title{Half-Normal log-probability density function}
\usage{
half_normal_lpdf(x, sd, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles (must be non-negative).}

\item{sd}{Vector of standard deviations (scale parameter).}
}
\value{
The sum of the log-density.
}
\description{
Half-Normal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_ttest.R
\name{rtmb_ttest}
\alias{rtmb_ttest}
\title{RTMB-based Bayesian two-sample t-test wrapper function}
\usage{
rtmb_ttest(
  x,
  y = NULL,
  data = NULL,
  r = 0.707,
  paired = FALSE,
  ID = NULL,
  y_range = NULL,
  prior = prior_flat(),
  init = NULL,
  fixed = NULL,
  var.equal = TRUE,
  missing = c("listwise", "fiml"),
  WAIC = FALSE,
  ...
)
}
\arguments{
\item{x}{Numeric vector of responses for group 1, a formula (e.g., `y ~ group`), or a column name (unquoted) if `data` is provided.}

\item{y}{Numeric vector of responses for group 2, or a column name (unquoted) if `data` is provided. Required if `x` is not a formula.}

\item{data}{Data frame containing the variables.}

\item{r}{Numeric; Cauchy prior scale for the effect size (delta). Default is 0.707.}

\item{paired}{Logical; whether to perform a paired t-test.}

\item{ID}{Character; name of the ID variable for paired t-tests (required for formula input with paired = TRUE).}

\item{y_range}{Theoretical minimum and maximum values of the response variable as a vector c(min, max). Required when using weakly informative priors.}

\item{prior}{An object of class `"rtmb_prior"`.
Use `prior_flat()` for no prior, `prior_normal()` for default normal/exponential priors,
or `prior_weak()` for weakly informative Bayesian inference.
Default is `prior_flat()`.}

\item{init}{List of initial values.}

\item{fixed}{Optional named list of fixed values for specific parameters.}

\item{var.equal}{Logical; whether to assume equal variances. Default is TRUE.}

\item{missing}{Missing value handling strategy: "listwise".}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}

\item{...}{Additional arguments.}
}
\value{
An `RTMB_Model` object.
}
\description{
Performs a Bayesian or Frequentist two-sample t-test using RTMB.
}
\details{
For classic inference, heteroscedastic two-sample t-tests use the same
RTMB Satterthwaite machinery as `optimize(marginal = ..., df_method = "satterthwaite")`.
The result is model-based and reproducible from the printed model code.
This corresponds to the Welch-type unequal-variance t-test, but the
degrees of freedom are computed by the package's internal Satterthwaite
procedure rather than by a separate closed-form formula.
When `prior_jzs()` is combined with `var.equal = FALSE`, BayesRTMB uses
a Welch-style JZS extension: the effect size `delta` is an explicit
parameter with a Cauchy prior, and the group mean difference is scaled by
the root-mean-square of the two group standard deviations.
}
\examples{

  # Simulate two-sample data with a true effect size
  set.seed(123)
  y1 <- rnorm(30, mean = 0.5, sd = 1)
  y2 <- rnorm(30, mean = 0.0, sd = 1)

  # Fit the Bayesian two-sample t-test model
  # r = 0.707 is the standard scale for the Cauchy prior on the effect size
  fit_ttest <- rtmb_ttest(y1, y2, prior = prior_jzs(r = 0.707))

  \donttest{
  # MCMC sampling (chains and iterations reduced for faster execution)
  mcmc_ttest <- fit_ttest$sample(sampling = 500, warmup = 500, chains = 2)
  mcmc_ttest$summary()
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{fiml_multi_normal_fa_lpdf}
\alias{fiml_multi_normal_fa_lpdf}
\title{Factor analysis multivariate normal log-probability density function (FIML)}
\usage{
fiml_multi_normal_fa_lpdf(x, mu, Lambda, psi)
}
\arguments{
\item{x}{Matrix of quantiles containing NAs.}

\item{mu}{Vector of means.}

\item{Lambda}{Factor loading matrix (P x K).}

\item{psi}{Vector of unique variances (P).}
}
\value{
The sum of the log-density.
}
\description{
Factor analysis multivariate normal log-probability density function (FIML)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{poisson_lpmf}
\alias{poisson_lpmf}
\title{Poisson log-probability mass function}
\usage{
poisson_lpmf(x, mean, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{mean}{Expected value.}
}
\value{
The sum of the log-probability.
}
\description{
Poisson log-probability mass function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{lkj_corr_lpdf}
\alias{lkj_corr_lpdf}
\title{LKJ correlation log-probability density function}
\usage{
lkj_corr_lpdf(Omega, eta = 1)
}
\arguments{
\item{Omega}{Correlation matrix.}

\item{eta}{Shape parameter.}
}
\value{
The log-density.
}
\description{
LKJ correlation log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{multi_student_t_lpdf}
\alias{multi_student_t_lpdf}
\title{Multivariate Student-t log-probability density function}
\usage{
multi_student_t_lpdf(x, df, mean, Sigma, sum = TRUE)
}
\arguments{
\item{x}{Vector or matrix of quantiles.}

\item{df}{Degrees of freedom.}

\item{mean}{Vector or matrix of means.}

\item{Sigma}{Scale matrix.}
}
\value{
The sum of the log-density.
}
\description{
Multivariate Student-t log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_prior.R
\name{prior_flat}
\alias{prior_flat}
\title{Specify a flat prior}
\usage{
prior_flat()
}
\value{
A list with class `"rtmb_prior"` and `type = "flat"`.
}
\description{
`prior_flat()` specifies that no additional prior density is added by the
wrapper. This is the only prior type allowed for `classic()`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmc_index.R
\name{quantile95}
\alias{quantile95}
\title{Calculate 95\% Quantiles}
\usage{
quantile95(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
A numeric vector of length 2.
}
\description{
Calculate 95\% Quantiles
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_utils.R
\name{to_wide}
\alias{to_wide}
\title{Convert Long Data to Wide Format}
\usage{
to_wide(data, within = "Condition", value = "Value", id = NULL)
}
\arguments{
\item{data}{A data frame in long format.}

\item{within}{The name of the column containing within-subjects factor levels (e.g., "Condition").}

\item{value}{The name of the column containing measurement values (e.g., "Value").}

\item{id}{The identifier columns (e.g., "Subject", "Group"). If NULL, 
inferred as all columns except \code{within} and \code{value}.}
}
\value{
A data frame in wide format.
}
\description{
A user-friendly wrapper around \code{stats::reshape} to convert data from
long format back to wide format.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmc_index.R
\name{r_hat}
\alias{r_hat}
\title{Calculate Rank-normalized Split-R-hat}
\usage{
r_hat(sims)
}
\arguments{
\item{sims}{A matrix of samples (iterations x chains).}
}
\value{
A numeric value.
}
\description{
Calculate Rank-normalized Split-R-hat
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_trace}
\alias{plot_trace}
\title{Plot MCMC trace plots}
\usage{
plot_trace(x, mono = FALSE)
}
\arguments{
\item{x}{A 3D array of posterior samples with dimensions
`(iterations, chains, variables)`.
A 2D matrix `(iterations, chains)` is also allowed and is treated
as a single variable.}

\item{mono}{Logical. If `TRUE`, plot in monochrome. If `FALSE`,
use blue shades.}
}
\value{
No return value. This function is called for its side effect
  of plotting.
}
\description{
Draw trace plots for each parameter across chains.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{item_curve.RTMB_Fit_Base}
\alias{item_curve.RTMB_Fit_Base}
\title{Item Response Curve for RTMB_Fit_Base}
\usage{
\method{item_curve}{RTMB_Fit_Base}(x, theta_seq = seq(-4, 4, length.out = 100), items = NULL, ...)
}
\arguments{
\item{x}{An object of class RTMB_Fit_Base.}

\item{theta_seq}{Sequence of trait values (ability) to evaluate.}

\item{items}{Index or item names to restrict the calculation to specific items (optional).}

\item{...}{Additional arguments.}
}
\description{
Item Response Curve for RTMB_Fit_Base
}
\examples{
\donttest{
  fit <- rtmb_irt(data = BigFive[, 1:5], model = "2PL", type = "ordered")
  map_fit <- fit$optimize()
  ic <- item_curve(map_fit)
  plot(ic)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_mcmc.R
\name{print.summary_BayesRTMB}
\alias{print.summary_BayesRTMB}
\title{print for summary_BayesRTMB class}
\usage{
\method{print}{summary_BayesRTMB}(x, digits = NULL, ...)
}
\arguments{
\item{x}{An object of class summary_BayesRTMB}

\item{digits}{integer}

\item{...}{Additional arguments.}
}
\description{
print for summary_BayesRTMB class
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{log_sum_exp}
\alias{log_sum_exp}
\title{Log-sum-exp function}
\usage{
log_sum_exp(x, y = NULL)
}
\arguments{
\item{x}{A numeric vector or scalar.}

\item{y}{An optional numeric scalar.}
}
\value{
The log of the sum of the exponentials.
}
\description{
Log-sum-exp function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{stz_basis}
\alias{stz_basis}
\title{stz basis function}
\usage{
stz_basis(K)
}
\arguments{
\item{K}{A numeric value}
}
\value{
A transformation matrix
}
\description{
stz basis function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{rtmb_code}
\alias{rtmb_code}
\title{Define an RTMB Model with Stan-like Syntax}
\usage{
rtmb_code(...)
}
\arguments{
\item{...}{Model definition blocks.}
}
\value{
A list of unevaluated code blocks.
}
\description{
\code{rtmb_code} is the primary interface for defining Bayesian or Frequentist models
within the RTMB framework. It allows you to organize model logic into functional
blocks while utilizing a Stan-like \code{~} operator for specifying priors and likelihoods.
}
\details{
\strong{Key Blocks and Logic:}
\itemize{
  \item \code{setup}: Pre-compilation data processing.
  \item \code{parameters}: Declaration of estimated parameters. See \code{\link{parameter_types}} for available constraints.
  \item \code{transform}: Definition of intermediate variables. Use \code{\link{math_functions}} for stable AD calculations.
  \item \code{model}: Likelihood and Priors. Refer to \code{\link{distributions}} for available sampling statements.
  \item \code{generate}: Post-hoc calculation of predictions or diagnostics.
}

\strong{Automatic Differentiation (AD) Note:}
All code within \code{parameters}, \code{transform}, and \code{model} is
automatically differentiated by RTMB. To ensure numerical stability, use
provided utility functions like \code{\link{log_sum_exp}} or \code{\link{inv_logit}}
instead of raw algebraic implementations.
}
\examples{
\donttest{
# Simulate data for a linear regression
set.seed(123)
N <- 100
x <- rnorm(N)
y <- 2.0 + 1.5 * x + rnorm(N, mean = 0, sd = 1)
data_list <- list(N = N, x = x, y = y)

# Define the model using rtmb_code
code <- rtmb_code(
  setup = {
    # Center the predictor variable (executed once)
    x_centered <- x - mean(x)
  },
  parameters = {
    # Define parameters and their constraints
    alpha = Dim(1)
    beta  = Dim(1)
    sigma = Dim(1, lower = 0)
  },
  transform = {
    # Calculate the linear predictor
    mu <- alpha + beta * x_centered
  },
  model = {
    # Priors
    alpha ~ normal(0, 10)
    beta  ~ normal(0, 10)
    sigma ~ exponential(1)

    # Likelihood (Vectorized)
    y ~ normal(mu, sigma)
  },
  generate = {
    # Calculate generated quantities
    y_pred <- mu

    # Must return a named list
    list(y_pred = y_pred)
  }
)

# Create the model object
mod <- rtmb_model(data = data_list, code = code)

# Fit the model using MAP estimation
map_res <- mod$optimize()
map_res$summary(pars = c("alpha", "beta", "sigma"))

# The generated quantity 'y_pred' can also be summarized
map_res$summary("y_pred", max_rows = 5)
}
}
\seealso{
\code{\link{rtmb_model}} for building the model object.
\code{\link{parameter_types}} for constraining parameters.
\code{\link{distributions}} for likelihood functions.
\code{\link{math_functions}} for numerical utilities.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Base_Fit.R
\name{RTMB_Fit_Base}
\alias{RTMB_Fit_Base}
\title{Base class for RTMB Fit objects}
\description{
Base class for RTMB Fit objects

Base class for RTMB Fit objects
}
\details{
An R6 base class providing common methods for Bayesian and MAP inference results.
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{model}}{The `RTMB_Model` object used for estimation.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-RTMB_Fit_Base-get_point_estimate}{\code{RTMB_Fit_Base$get_point_estimate()}}
\item \href{#method-RTMB_Fit_Base-estimate}{\code{RTMB_Fit_Base$estimate()}}
\item \href{#method-RTMB_Fit_Base-EAP}{\code{RTMB_Fit_Base$EAP()}}
\item \href{#method-RTMB_Fit_Base-MAP}{\code{RTMB_Fit_Base$MAP()}}
\item \href{#method-RTMB_Fit_Base-rotate}{\code{RTMB_Fit_Base$rotate()}}
\item \href{#method-RTMB_Fit_Base-fa_rotate}{\code{RTMB_Fit_Base$fa_rotate()}}
\item \href{#method-RTMB_Fit_Base-clone}{\code{RTMB_Fit_Base$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Fit_Base-get_point_estimate"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Fit_Base-get_point_estimate}{}}}
\subsection{Method \code{get_point_estimate()}}{
Abstract method to get a point estimate for a target parameter.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Fit_Base$get_point_estimate(target, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Character string specifying the target parameter name.}

\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A numeric array or matrix of the point estimate.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Fit_Base-estimate"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Fit_Base-estimate}{}}}
\subsection{Method \code{estimate()}}{
Get point estimates for parameters, transformed parameters, and generated quantities.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Fit_Base$estimate(
  pars = NULL,
  type = c("mean", "EAP", "marginal_map", "joint_map", "MAP"),
  component = c("all", "parameters", "transform", "generate"),
  chains = NULL,
  best_chains = NULL,
  drop = TRUE,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Optional character or numeric vector of parameter names or indices to extract.
Supports special keywords: "parameters", "transform", "generate", and "all".}

\item{\code{type}}{Character string specifying the estimation type.}

\item{\code{component}}{Character string specifying the component to filter by.}

\item{\code{chains}}{Numeric vector of chains to include.}

\item{\code{best_chains}}{Number of best chains to include.}

\item{\code{drop}}{Logical; if TRUE and only one parameter is selected, return the value directly instead of a list.}

\item{\code{...}}{Additional arguments passed to draws().}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of point estimates, or a single value if `drop = TRUE`.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Fit_Base-EAP"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Fit_Base-EAP}{}}}
\subsection{Method \code{EAP()}}{
Calculate Expected A Posteriori (EAP) estimates from posterior samples.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Fit_Base$EAP(
  pars = "parameters",
  chains = NULL,
  best_chains = NULL,
  drop = FALSE,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Optional character vector of parameter names to extract.}

\item{\code{chains}}{Numeric vector of chains to include.}

\item{\code{best_chains}}{Number of best chains to include.}

\item{\code{drop}}{Logical; whether to drop the list if only one parameter is selected.}

\item{\code{...}}{Additional arguments passed to `estimate()`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of EAP estimates.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Fit_Base-MAP"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Fit_Base-MAP}{}}}
\subsection{Method \code{MAP()}}{
Calculate Maximum A Posteriori (MAP) estimates.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Fit_Base$MAP(
  pars = "parameters",
  chains = NULL,
  best_chains = NULL,
  type = c("marginal", "joint"),
  drop = FALSE,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Optional character vector of parameter names to extract.}

\item{\code{chains}}{Numeric vector of chains to include.}

\item{\code{best_chains}}{Number of best chains to include.}

\item{\code{type}}{Character string; "marginal" or "joint" MAP.}

\item{\code{drop}}{Logical; whether to drop the list if only one parameter is selected.}

\item{\code{...}}{Additional arguments passed to `estimate()`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of MAP estimates.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Fit_Base-rotate"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Fit_Base-rotate}{}}}
\subsection{Method \code{rotate()}}{
Rotate sampled parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Fit_Base$rotate(target, reference = NULL, linked = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Character string specifying the target variable to base the rotation on.}

\item{\code{reference}}{Matrix to rotate towards. If NULL, the target's point estimate is used.}

\item{\code{linked}}{Character vector of variable names to be rotated in the same direction.}

\item{\code{overwrite}}{Logical; whether to overwrite the stored draws. If FALSE, adds to generated quantities. Default is FALSE.}

\item{\code{suffix}}{Character string to append to the rotated variable names when overwrite is FALSE. Default is "rot".}

\item{\code{...}}{Additional arguments passed to the rotation function.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The updated object invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Fit_Base-fa_rotate"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Fit_Base-fa_rotate}{}}}
\subsection{Method \code{fa_rotate()}}{
Rotate factor loadings and optional factor scores.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Fit_Base$fa_rotate(
  target = "L",
  linked = NULL,
  scores = NULL,
  rotate = "promax",
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Character string specifying the target variable to base the rotation on.}

\item{\code{linked}}{Character vector of variable names to be rotated in the same direction.}

\item{\code{scores}}{Character vector of variable names to be rotated as factor scores (inverse direction).}

\item{\code{rotate}}{Character string specifying the rotation method.}

\item{\code{...}}{Additional arguments passed to the rotation function.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The updated object invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Fit_Base-clone"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Fit_Base-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Fit_Base$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{distributions}
\alias{distributions}
\title{Probability Distributions for RTMB Models}
\description{
This page summarizes the probability distributions available within the `rtmb_code` block.
These functions are designed to be used with the Stan-like sampling syntax (`~`),
which internally adds the log-density to the model's total log-posterior.
}
\details{
\strong{Syntax Styles:}
In `rtmb_code`, you can specify distributions in two ways:
\itemize{
  \item \strong{Sampling Syntax (Recommended):} \code{y ~ normal(mu, sigma)}
  \item \strong{Explicit Function Call:} \code{lp <- lp + normal_lpdf(y, mu, sigma)}
}

\strong{Continuous Distributions (LPDF):}
\itemize{
  \item \code{normal(mean, sd)}: Normal distribution.
  \item \code{lognormal(meanlog, sdlog)}: Lognormal distribution.
  \item \code{exponential(rate)}: Exponential distribution.
  \item \code{cauchy(location, scale)}: Cauchy distribution.
  \item \code{student_t(df, mu, sigma)}: Student's t-distribution.
  \item \code{gamma(shape, rate)}: Gamma distribution.
  \item \code{inverse_gamma(shape, scale)}: Inverse-gamma distribution.
  \item \code{beta(a, b)}: Beta distribution.
}

\strong{Discrete Distributions (LPMF):}
\itemize{
  \item \code{bernoulli(prob)} / \code{bernoulli_logit(eta)}: Binary outcomes.
  \item \code{binomial(size, prob)} / \code{binomial_logit(size, eta)}: Binomial outcomes.
  \item \code{poisson(mean)}: Poisson count data.
  \item \code{neg_binomial_2(mu, size)}: Negative binomial (mean/dispersion parameterization).
  \item \code{ordered_logistic(eta, cutpoints)}: Ordered categorical outcomes.
  \item \code{sequential_logistic(eta, cutpoints)}: Sequential ordered categorical outcomes.
}

\strong{Multivariate and Matrix Distributions:}
\itemize{
  \item \code{multi_normal(mean, Sigma)}: Standard multivariate normal distribution.
  \item \code{multi_student_t(df, mean, Sigma)}: Multivariate Student's t-distribution.
  \item \code{multi_cauchy(mean, Sigma)}: Multivariate Cauchy distribution (Student-t with df=1).
  \item \code{lkj_corr(eta)}: LKJ prior for correlation matrices.
  \item \code{dirichlet(alpha)}: Dirichlet distribution for simplexes.
  \item \code{lower_tri_normal(mean, sd)}: Normal distribution for elements of a lower-triangular matrix.
  \item \code{centered_tri_multi_normal(sigma)}: Multivariate normal for centered triangular matrices (used in identification constraints).
  \item \code{sufficient_multi_normal_fa(S_mat, N, y_bar, mu, psi, Lambda)}: Factor analysis likelihood using sufficient statistics (highly efficient for large sample sizes).
}

\strong{Vectorization:}
Most univariate distributions are vectorized. If \code{y} and \code{mu} are vectors,
\code{y ~ normal(mu, sigma)} will calculate the sum of log-densities for all elements
efficiently.
}
\concept{distributions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{quad_form_diag}
\alias{quad_form_diag}
\title{Quadratic form with a diagonal matrix}
\usage{
quad_form_diag(m, v)
}
\arguments{
\item{m}{A numeric matrix.}

\item{v}{A numeric vector representing the diagonal elements.}
}
\value{
The value of the quadratic form.
}
\description{
Quadratic form with a diagonal matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtmb_syntax.R
\name{rtmb_syntax}
\alias{rtmb_syntax}
\title{Guidelines for Writing RTMB-Compatible Code}
\description{
Models defined in \code{rtmb_code} rely on Automatic Differentiation (AD) via the
\code{RTMB} package. To ensure the model is differentiable and numerically stable,
specific coding practices must be followed.
}
\details{
\strong{1. Automatic Differentiation and \code{advector}:}
Parameters and intermediate calculations involving them are treated as \code{advector}
objects. RTMB "records" all mathematical operations to compute derivatives
automatically. If a function strips these attributes or is not differentiable, the
gradient calculation will fail.

\strong{2. Avoid Discrete Branching:}
Standard R conditional statements like \code{if (x > 0)} or \code{ifelse()} based
on parameter values do not provide derivatives.
\itemize{
  \item \strong{Problem:} They create "jumps" in the likelihood surface.
  \item \strong{Solution:} Use smooth approximations. For example, use \code{\link{fabs}}
  instead of \code{abs}, or \code{\link{log_mix}} for mixture logic.
}

\strong{3. Numerical Stability:}
Computations in log-space are preferred to prevent overflow or underflow.
Use the following stable utilities instead of raw algebraic expressions:
\itemize{
  \item \code{\link{log_sum_exp}}: For summing probabilities in log-space.
  \item \code{\link{log1p_exp}}: For \code{log(1 + exp(x))}.
  \item \code{\link{inv_logit}}: For mapping real numbers to probabilities.
}

\strong{4. Vectorization vs. Loops:}
\itemize{
  \item \strong{Vectorization:} Highly recommended for performance. Standard R
  vectorized arithmetic (\code{+}, \code{-}, \code{*}, \code{/}, \code{log}, \code{exp})
  works seamlessly with AD.
  \item \strong{Loops:} Standard \code{for} loops are safe as long as the operations
  inside are differentiable.
  \item \strong{Avoid \code{apply}:} Functions like \code{apply}, \code{sapply},
  or \code{lapply} may sometimes strip AD attributes and should be replaced with
  vectorized operations or explicit loops.
}

\strong{5. Matrix Operations:}
Use specialized functions for matrix algebra to maintain efficiency:
\itemize{
  \item \code{\link{quad_form_chol}}: For efficient quadratic forms.
  \item \code{\link{log_det_chol}}: For log-determinants via Cholesky factors.
}
}
\seealso{
\code{\link{math_functions}}, \code{\link{distributions}}, \code{\link{rtmb_code}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{safe_rtmb_model}
\alias{safe_rtmb_model}
\title{Safe RTMB model construction (with error message translation)}
\usage{
safe_rtmb_model(data, code, par_names = list(), init = NULL, view = NULL)
}
\arguments{
\item{data}{A list of data that has passed `validate_data`.}

\item{code}{Code blocks for likelihood and priors defined with `model_code({})`.}

\item{par_names}{A list of variable names corresponding to the dimensions of each parameter (optional).}

\item{init}{A list or numeric vector of initial values (optional).}

\item{view}{Character vector of parameter names to be displayed preferentially in summary outputs (optional).}
}
\value{
Returns an `rtmb_model` object (R6 class instance) upon successful compilation.
}
\description{
Wraps `rtmb_model` to translate cryptic error messages generated during `MakeADFun`
execution (originating from C++/RTMB) into user-friendly hints.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_mdu.R
\name{make_bw_from_ydif}
\alias{make_bw_from_ydif}
\alias{restore_bw_from_ydif}
\title{Make Best and Worst Responses from Best-Worst Pair Indices}
\usage{
make_bw_from_ydif(Y_dif, sets)

restore_bw_from_ydif(Y_dif, sets)
}
\arguments{
\item{Y_dif}{Matrix or data frame of pair indices (N persons x P tasks).
Each value must be an integer from 1 to `C * (C - 1)`, where `C` is the
number of items per task.}

\item{sets}{Matrix or data frame of presented item sets (P tasks x C items).}
}
\value{
A list with two data frames: `Best` and `Worst`.
}
\description{
Converts a matrix of Best-Worst pair indices (`Y_dif`) into separate `Best`
and `Worst` data frames. The returned values are positions within each row of
`sets`, not the item labels stored in `sets`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_means.R
\name{plot.rtmb_lsmeans}
\alias{plot.rtmb_lsmeans}
\title{Plot marginal means with error bars}
\usage{
\method{plot}{rtmb_lsmeans}(
  x,
  y,
  error_bar = "se",
  col = NULL,
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{x}{An rtmb_lsmeans object.}

\item{y}{Ignored.}

\item{error_bar}{Type of error bar ("se" or "ci").}

\item{col}{Color of the bars.}

\item{main}{Plot title.}

\item{xlab}{X axis label.}

\item{ylab}{Y axis label.}

\item{...}{Additional arguments passed to barplot.}
}
\description{
Plot method for rtmb_lsmeans objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{bernoulli_logit_lpmf}
\alias{bernoulli_logit_lpmf}
\title{Bernoulli log-probability mass function with logit parameterization}
\usage{
bernoulli_logit_lpmf(x, eta, sum = TRUE)
}
\arguments{
\item{x}{Vector of binary outcomes (0 or 1).}

\item{eta}{Linear predictor (logit probability).}
}
\value{
The sum of the log-probability.
}
\description{
Bernoulli log-probability mass function with logit parameterization
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_mcmc.R
\name{read_mcmc_csv}
\alias{read_mcmc_csv}
\title{Restore MCMC Fit from CSV}
\usage{
read_mcmc_csv(model, name, dir = "BayesRTMB_mcmc", chains = 4, laplace = FALSE)
}
\arguments{
\item{model}{An RTMB_Model object.}

\item{name}{Base name of the saved CSVs.}

\item{dir}{Directory where CSVs are saved. Default is "BayesRTMB_mcmc".}

\item{chains}{Number of chains. Default is 4.}

\item{laplace}{Logical; whether Laplace approximation was used. Default is FALSE.}
}
\value{
An MCMC_Fit object.
}
\description{
Restore MCMC Fit from CSV
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_prior.R
\name{prior_uniform}
\alias{prior_uniform}
\title{Specify a flat prior}
\usage{
prior_uniform()
}
\value{
A list with class `"rtmb_prior"` and `type = "flat"`.
}
\description{
Deprecated alias of `prior_flat()`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{BigFive}
\alias{BigFive}
\title{Big Five Personality Traits Data}
\format{
A data frame with 10 rows and 20 columns:
\describe{
  \item{BF1}{Extraversion item 1}
  \item{BF2}{Neuroticism item 1}
  \item{BF3}{Openness item 1}
  \item{BF4}{Agreeableness item 1}
  \item{BF5}{Conscientiousness item 1}
  \item{BF6}{Extraversion item 2}
  \item{BF7}{Neuroticism item 2}
  \item{BF8}{Openness item 2}
  \item{BF9}{Agreeableness item 2}
  \item{BF10}{Conscientiousness item 2}
  \item{BF11}{Extraversion item 3}
  \item{BF12}{Neuroticism item 3}
  \item{BF13}{Openness item 3}
  \item{BF14}{Agreeableness item 3}
  \item{BF15}{Conscientiousness item 3}
  \item{BF16}{Extraversion item 4}
  \item{BF17}{Neuroticism item 4}
  \item{BF18}{Openness item 4}
  \item{BF19}{Agreeableness item 4}
  \item{BF20}{Conscientiousness item 4}
}
}
\source{
Originally collected by the package author.
}
\usage{
BigFive
}
\description{
A dataset containing responses to 20 items measuring the Big Five personality traits.
This dataset was originally collected by the package author.
The items are arranged in a cyclical order representing the five factors:
Extraversion, Neuroticism, Openness, Agreeableness, and Conscientiousness.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_fa.R
\name{rtmb_fa}
\alias{rtmb_fa}
\title{RTMB-based Factor Analysis Wrapper}
\usage{
rtmb_fa(
  data,
  nfactors = 1,
  rotate = NULL,
  score = FALSE,
  prior = prior_flat(),
  y_range = NULL,
  init = NULL,
  fixed = NULL,
  missing = c("listwise", "fiml"),
  WAIC = FALSE
)
}
\arguments{
\item{data}{Observation data frame or matrix (N x J).}

\item{nfactors}{Number of factors (K).}

\item{rotate}{String specifying the rotation method (e.g., "varimax", "promax", "ssp"). If NULL, no rotation is applied. Specifying "ssp" performs regularized factor analysis.}

\item{score}{Logical; if TRUE, factor scores are calculated in the generate block (default is FALSE).}

\item{prior}{Prior configuration: `prior_flat()`, `prior_normal()`, or `prior_weak()`.
Hyperparameters can be specified within these functions (e.g., `prior_normal(mean_sd = 10, sd_rate = 10)`).
Available parameters for FA: `mean_sd`, `sd_rate`, `loadings_sd`, and `ssp_ratio` (if `rotate = "ssp"`).}

\item{y_range}{A numeric vector of length 2 specifying the theoretical min and max values of the items.
Used to construct weakly informative priors when `prior = prior_weak()`.}

\item{init}{List of initial values.}

\item{fixed}{A named list of parameter values to fix (optional).}

\item{missing}{Missing value handling strategy: "listwise" (default) or "fiml" (Full Information Maximum Likelihood).}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}
}
\description{
Performs exploratory factor analysis (EFA) using RTMB. It supports standard
rotation methods (e.g., varimax, promax) as well as regularized factor analysis
using a Spike-and-Slab Prior (SSP) for estimating sparse loading matrices.
}
\examples{

  # Prepare a subset of the BigFive dataset for factor analysis
  data(BigFive, package = "BayesRTMB")
  fa_data <- BigFive[, 1:10]

  # --- 1. Standard Exploratory Factor Analysis (1 Factor) ---
  fit_fa1 <- rtmb_fa(data = fa_data, nfactors = 1)

  # Maximum A Posteriori (MAP) estimation
  map_fa1 <- fit_fa1$optimize()
  map_fa1$summary()



  # --- 2. Factor Analysis with Rotation and Factor Scores (2 Factors) ---
  # Extract 2 factors, apply Promax rotation during model fitting, and calculate factor scores
  fit_fa2 <- rtmb_fa(data = fa_data, nfactors = 2, rotate = "promax", score = TRUE)

  # Setting se_method = "sampling" enables standard errors and 95\% CIs
  # for transformed and generated quantities, such as factor scores and post-hoc rotations.
  # (It uses multivariate normal sampling from the unconstrained parameter space)
  map_fa2 <- fit_fa2$optimize(se_method = "sampling")
  map_fa2$summary()

  # Post-hoc rotation using the fa_rotate() method
  # You can also apply other rotation methods (e.g., "varimax" from the stats package)
  # to the unrotated loading matrix ("L") after estimation.
  map_fa2$fa_rotate(target = "L", rotate = "varimax")

  # The post-hoc rotated loadings are automatically stored with the method's
  # suffix (e.g., "L_varimax")
  map_fa2$summary("L_varimax")



  # --- 3. Regularized Factor Analysis using Spike-and-Slab Prior (SSP) ---
  # Specifying 'rotate = "ssp"' enables sparse loading matrix estimation
  fit_ssp <- rtmb_fa(data = fa_data, nfactors = 2, rotate = "ssp")

  map_ssp <- fit_ssp$optimize()
  map_ssp$summary()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_prior.R
\name{prior_jzs}
\alias{prior_jzs}
\title{Specify a JZS (Jeffrey-Zellner-Siow) prior for t-tests}
\usage{
prior_jzs(r = 0.707)
}
\arguments{
\item{r}{Numeric; scale parameter for the Cauchy prior on effect size delta. Default is 0.707.}
}
\value{
A list with class "rtmb_prior"
}
\description{
Specify a JZS (Jeffrey-Zellner-Siow) prior for t-tests
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_prior.R
\name{prior_weak}
\alias{prior_weak}
\title{Specify a weakly informative prior}
\usage{
prior_weak(sd_ratio = 0.5, max_beta = 1, ...)
}
\arguments{
\item{sd_ratio}{Ratio of the prior standard deviation to the half-range of the response variable. Default is 0.5.}

\item{max_beta}{Maximum expected effect size. Default is 1.0.}

\item{...}{Optional hyperparameters for other distributions.}
}
\value{
A list with class "rtmb_prior"
}
\description{
Specify a weakly informative prior
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{bw_categorical_logit_lpmf}
\alias{bw_categorical_logit_lpmf}
\title{Best-Worst Categorical Logit Log-PMF (RTMB optimized)}
\usage{
bw_categorical_logit_lpmf(x, U, lambda = 1)
}
\arguments{
\item{x}{Vector of indices of the selected pairs.}

\item{U}{Vector of utilities for each item.}

\item{lambda}{Scaling parameter.}
}
\value{
Log-probability (scalar advector).
}
\description{
Best-Worst Categorical Logit Log-PMF (RTMB optimized)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_prior.R
\name{prior_normal}
\alias{prior_normal}
\title{Specify normal/exponential priors for MAP and Bayesian inference}
\usage{
prior_normal(
  Intercept_sd = 10,
  mu_sd = 10,
  b_sd = 10,
  sigma_rate = 5,
  tau_rate = 5,
  ...
)
}
\arguments{
\item{Intercept_sd}{Standard deviation for the intercept prior. If `NULL`, no intercept prior is added.}

\item{mu_sd}{Standard deviation for mean/intercept priors. If `NULL`, no mean prior is added.}

\item{b_sd}{Standard deviation for coefficient priors. If `NULL`, no coefficient prior is added.}

\item{sigma_rate}{Rate for residual standard deviation priors. If `NULL`, no sigma prior is added.}

\item{tau_rate}{Rate for random-effect standard deviation priors. If `NULL`, no tau prior is added.}

\item{...}{Optional wrapper-specific hyperparameters.}
}
\value{
A list with class `"rtmb_prior"` and `type = "normal"`.
}
\description{
`prior_normal()` specifies normal priors for location parameters and
exponential priors for scale parameters. It is intended for MAP and Bayesian
inference, not for `classic()`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{normal_mixture_lpdf}
\alias{normal_mixture_lpdf}
\title{Normal Mixture log-probability density function}
\usage{
normal_mixture_lpdf(x, pi_w, mean, sd, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles (observations).}

\item{pi_w}{Vector of mixing proportions for each cluster.}

\item{mean}{Vector of means for each cluster.}

\item{sd}{Vector of standard deviations for each cluster.}
}
\value{
The sum of the log-density.
}
\description{
Normal Mixture log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{fa_multi_normal_lpdf}
\alias{fa_multi_normal_lpdf}
\title{Factor analysis multivariate normal log-probability density function}
\usage{
fa_multi_normal_lpdf(x, mu, Lambda, psi, sum = TRUE)
}
\arguments{
\item{x}{Vector or matrix of quantiles.}

\item{mu}{Vector of means.}

\item{Lambda}{Factor loading matrix (P x K).}

\item{psi}{Vector of unique variances (P).}
}
\value{
The sum of the log-density.
}
\description{
Woodbury matrix identity is used for efficient computation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{logit}
\alias{logit}
\title{Logit function}
\usage{
logit(x)
}
\arguments{
\item{x}{A numeric vector of probabilities.}
}
\value{
The logit of x.
}
\description{
Logit function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameters.R
\name{parameter_types}
\alias{parameter_types}
\title{Parameter Types and Constraints in RTMB Models}
\description{
When declaring parameters in the \code{parameters} block using \code{Dim()},
you can specify a \code{type} to automatically apply structural constraints.
These constraints ensure that parameters remain within their valid mathematical
space during estimation.
}
\details{
\strong{Standard Types:}
\itemize{
  \item \code{"vector"}, \code{"matrix"}, \code{"array"}: Standard unconstrained containers.
  Use \code{lower} and \code{upper} for element-wise bounds.
}

\strong{Ordering Constraints:}
\itemize{
  \item \code{"ordered"}: A vector where $x_1 < x_2 < \dots < x_K$.
  \item \code{"positive_ordered"}: A vector where $0 < x_1 < x_2 < \dots < x_K$.
}

\strong{Constrained Vectors:}
\itemize{
  \item \code{"simplex"}: A vector where all elements are >= 0 and sum(x) = 1.
  \item \code{"centered"}: A vector of length K where sum(x) = 0. (Estimated using K-1 degrees of freedom).
}

\strong{Correlation and Covariance Matrices:}
\itemize{
  \item \code{"corr_matrix"}: A symmetric, positive-definite correlation matrix (diagonal elements are 1).
  \item \code{"cov_matrix"}: A symmetric, positive-definite covariance matrix.
  \item \code{"CF_corr"}: The Cholesky factor of a correlation matrix.
  \item \code{"CF_cov"}: The Cholesky factor of a covariance matrix (diagonal elements are positive).
}

\strong{Special Structural Matrices:}
\itemize{
  \item \code{"lower_tri"}: A lower-triangular matrix.
  \item \code{"positive_lower_tri"}: A lower-triangular matrix with positive diagonal elements.
  \item \code{"centered_matrix"}: A matrix where each column sums to zero.
  \item \code{"centered_tri"}: A triangular matrix with column-wise sum-to-zero constraints (often used for identification in factor analysis).
}
}
\seealso{
\code{\link{rtmb_code}}, \code{\link{math_functions}}, \code{\link{distributions}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{mixture_lpdf}
\alias{mixture_lpdf}
\title{Generic Mixture log-probability density function}
\usage{
mixture_lpdf(x, pi_w, lpdf_list, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{pi_w}{Vector or matrix of mixing proportions.}

\item{lpdf_list}{A list of log-density vectors (one for each cluster).}
}
\value{
The sum of the log-density.
}
\description{
Generic Mixture log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameters.R
\name{select_parameters}
\alias{select_parameters}
\title{Select parameters from a list by name or index}
\usage{
select_parameters(all_list, pars)
}
\arguments{
\item{all_list}{A named list of parameters.}

\item{pars}{A character or numeric vector.}
}
\value{
A subset of the list.
}
\description{
Select parameters from a list by name or index
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MCMC_Fit.R
\name{MCMC_Fit}
\alias{MCMC_Fit}
\title{MCMC fit object}
\description{
MCMC fit object

MCMC fit object
}
\details{
An R6 class storing posterior samples and related information
from MCMC estimation.


Bayes factors are computed as ratios of marginal likelihoods estimated
by bridge sampling. The comparison model can be constructed by fixing
parameters with `fixed = list(...)`, or supplied as an already fitted
`MCMC_Fit` object via `comparison_fit`.
Bayes factors are computed only by marginal-likelihood model comparison.
}
\section{Super class}{
\code{\link[BayesRTMB:RTMB_Fit_Base]{BayesRTMB::RTMB_Fit_Base}} -> \code{mcmc_fit}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{model}}{An `RTMB_Model` object used for estimation.}

\item{\code{fit}}{Posterior draws for model parameters.}

\item{\code{random_fit}}{Posterior draws for random effects.}

\item{\code{transform_fit}}{Posterior draws for transformed parameters.}

\item{\code{transform_dims}}{Dimension information for transformed parameters.}

\item{\code{generate_fit}}{Posterior draws for generated quantities.}

\item{\code{generate_dims}}{Dimension information for generated quantities.}

\item{\code{eps}}{Step size used by the sampler.}

\item{\code{accept}}{Acceptance statistics from sampling.}

\item{\code{treedepth}}{Tree depth used in HMC/NUTS sampling.}

\item{\code{laplace}}{Logical; whether Laplace approximation was used.}

\item{\code{posterior_mean}}{Posterior mean estimates.}

\item{\code{log_ml}}{Numeric value storing the calculated log marginal likelihood from bridge sampling.}

\item{\code{comparison_fit}}{An \code{MCMC_Fit} object containing the fitted comparison model.}

\item{\code{max_treedepth}}{Maximum tree depth requested for NUTS/HMC.}

\item{\code{pd_error_count}}{Positive-definite/singularity errors treated as lp = -Inf by chain.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-mcmc_fit-get_point_estimate}{\code{MCMC_Fit$get_point_estimate()}}
\item \href{#method-mcmc_fit-new}{\code{MCMC_Fit$new()}}
\item \href{#method-mcmc_fit-print}{\code{MCMC_Fit$print()}}
\item \href{#method-mcmc_fit-draws}{\code{MCMC_Fit$draws()}}
\item \href{#method-mcmc_fit-summary}{\code{MCMC_Fit$summary()}}
\item \href{#method-mcmc_fit-unconstrain_draws}{\code{MCMC_Fit$unconstrain_draws()}}
\item \href{#method-mcmc_fit-log_prob}{\code{MCMC_Fit$log_prob()}}
\item \href{#method-mcmc_fit-bridgesampling}{\code{MCMC_Fit$bridgesampling()}}
\item \href{#method-mcmc_fit-WAIC}{\code{MCMC_Fit$WAIC()}}
\item \href{#method-mcmc_fit-diagnose}{\code{MCMC_Fit$diagnose()}}
\item \href{#method-mcmc_fit-bayes_factor}{\code{MCMC_Fit$bayes_factor()}}
\item \href{#method-mcmc_fit-transformed_draws}{\code{MCMC_Fit$transformed_draws()}}
\item \href{#method-mcmc_fit-generated_quantities}{\code{MCMC_Fit$generated_quantities()}}
\item \href{#method-mcmc_fit-resolve_switching}{\code{MCMC_Fit$resolve_switching()}}
\item \href{#method-mcmc_fit-clone}{\code{MCMC_Fit$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="EAP"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-EAP'><code>BayesRTMB::RTMB_Fit_Base$EAP()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="MAP"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-MAP'><code>BayesRTMB::RTMB_Fit_Base$MAP()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="estimate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-estimate'><code>BayesRTMB::RTMB_Fit_Base$estimate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="fa_rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-fa_rotate'><code>BayesRTMB::RTMB_Fit_Base$fa_rotate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-rotate'><code>BayesRTMB::RTMB_Fit_Base$rotate()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-get_point_estimate"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-get_point_estimate}{}}}
\subsection{Method \code{get_point_estimate()}}{
Get point estimate for a target parameter.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$get_point_estimate(target, chains = NULL, best_chains = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Target parameter name.}

\item{\code{chains}}{Numeric vector of chains to include. If NULL, all chains are used.}

\item{\code{best_chains}}{Integer; number of best chains to retain based on mean log-posterior.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Matrix, array, vector, or scalar point estimate.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-new"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-new}{}}}
\subsection{Method \code{new()}}{
Create a new `MCMC_Fit` object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$new(
  model,
  fit,
  random_fit,
  eps,
  accept,
  treedepth,
  laplace,
  posterior_mean,
  max_treedepth = NULL,
  pd_error_count = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{model}}{An `RTMB_Model` object used for estimation.}

\item{\code{fit}}{Posterior draws for model parameters.}

\item{\code{random_fit}}{Posterior draws for random effects, if available.}

\item{\code{eps}}{Step size used by the sampler.}

\item{\code{accept}}{Acceptance statistics from sampling.}

\item{\code{treedepth}}{Tree depth used in HMC/NUTS sampling.}

\item{\code{laplace}}{Logical; whether Laplace approximation was used.}

\item{\code{posterior_mean}}{Posterior mean estimates.}

\item{\code{max_treedepth}}{Maximum tree depth requested for NUTS/HMC.}

\item{\code{pd_error_count}}{Positive-definite/singularity errors treated as `lp = -Inf` by chain.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-print"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-print}{}}}
\subsection{Method \code{print()}}{
Print a brief summary of the fitted object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-draws"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-draws}{}}}
\subsection{Method \code{draws()}}{
Extract posterior draws for selected parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$draws(
  pars = NULL,
  chains = NULL,
  best_chains = NULL,
  inc_random = FALSE,
  inc_transform = TRUE,
  inc_generate = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character or numeric vector specifying the names or indices of parameters to extract. If NULL, all available parameters are extracted.}

\item{\code{chains}}{Numeric vector specifying the chains to extract. If NULL, draws from all chains are returned.}

\item{\code{best_chains}}{Integer; number of best chains to retain based on mean log-posterior (lp).}

\item{\code{inc_random}}{Logical; whether to include random effects in the output. Default is FALSE.}

\item{\code{inc_transform}}{Logical; whether to include transformed parameters in the output. Default is TRUE.}

\item{\code{inc_generate}}{Logical; whether to include generated quantities in the output. Default is TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Posterior draws.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-summary"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-summary}{}}}
\subsection{Method \code{summary()}}{
Summarize posterior draws.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$summary(
  pars = NULL,
  chains = NULL,
  best_chains = NULL,
  max_rows = 10,
  digits = 2,
  inc_random = FALSE,
  inc_transform = TRUE,
  inc_generate = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character or numeric vector specifying the names or indices of parameters to summarize. If NULL, all available parameters are summarized.}

\item{\code{chains}}{Numeric vector specifying the chains to extract. If NULL, draws from all chains are used.}

\item{\code{best_chains}}{Integer; number of best chains to retain based on mean log-posterior (lp).}

\item{\code{max_rows}}{Integer; maximum number of rows to print in the summary table. Default is 10.}

\item{\code{digits}}{Integer; number of decimal places to print. Default is 2.}

\item{\code{inc_random}}{Logical; whether to include random effects in the summary. Default is FALSE.}

\item{\code{inc_transform}}{Logical; whether to include transformed parameters in the summary. Default is TRUE.}

\item{\code{inc_generate}}{Logical; whether to include generated quantities in the summary. Default is TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A summary object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-unconstrain_draws"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-unconstrain_draws}{}}}
\subsection{Method \code{unconstrain_draws()}}{
Transform posterior draws to the unconstrained scale.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$unconstrain_draws()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Posterior draws on the unconstrained scale.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-log_prob"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-log_prob}{}}}
\subsection{Method \code{log_prob()}}{
Evaluate log-probability values.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$log_prob(safe = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{safe}}{Logical; whether to wrap the evaluation in a tryCatch block. Default is FALSE for speed.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Numeric vector of log-probability values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-bridgesampling"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-bridgesampling}{}}}
\subsection{Method \code{bridgesampling()}}{
Estimate the marginal likelihood by bridge sampling.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$bridgesampling(
  method = "normal",
  use_neff = TRUE,
  seed = NULL,
  max_iter = 100
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{method}}{Character; the method to use for bridge sampling (e.g., "warp3", "normal"). Default is "warp3".}

\item{\code{use_neff}}{Logical; whether to use the effective sample size (ESS) to adjust for autocorrelation. Default is TRUE.}

\item{\code{seed}}{Integer; random seed for reproducibility. Default is NULL.}

\item{\code{max_iter}}{Integer; maximum number of iterations for the estimation algorithm. Default is 100.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Bridge sampling result.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-WAIC"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-WAIC}{}}}
\subsection{Method \code{WAIC()}}{
Compute WAIC from pointwise generated log likelihood.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$WAIC(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to `draws()`, such as `chains` or `best_chains`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A `waic_BayesRTMB` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-diagnose"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-diagnose}{}}}
\subsection{Method \code{diagnose()}}{
Run basic diagnostics for the MCMC fit.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$diagnose(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to `diagnose_mcmc_fit()`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A `diagnose_BayesRTMB` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-bayes_factor"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-bayes_factor}{}}}
\subsection{Method \code{bayes_factor()}}{
Calculate the Bayes factor by marginal-likelihood model comparison.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$bayes_factor(
  fixed = NULL,
  comparison_fit = NULL,
  bs_method = "normal",
  error_threshold = 0.2,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{fixed}}{Named list of parameter values used to construct the comparison model.
For example, \code{fixed = list(delta = 0)} or
\code{fixed = list("b[x]" = 0)}.}

\item{\code{comparison_fit}}{Optional `MCMC_Fit` object for an already fitted comparison model.}

\item{\code{bs_method}}{Character; the method to use for bridge sampling ("normal" or "warp3").}

\item{\code{error_threshold}}{Numeric; threshold for the approximate error warning.}

\item{\code{...}}{Additional arguments passed to `sample()` when fitting the comparison model (e.g., \code{chains = 4}, \code{sampling = 4000}).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list of class \code{bayes_factor_rtmb} containing Bayes factor results.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-transformed_draws"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-transformed_draws}{}}}
\subsection{Method \code{transformed_draws()}}{
Compute transformed parameters from posterior draws.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$transformed_draws(tran_fn = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{tran_fn}}{A function for transformed parameters.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Transformed parameter draws.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-generated_quantities"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-generated_quantities}{}}}
\subsection{Method \code{generated_quantities()}}{
Compute generated quantities from posterior draws.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$generated_quantities(code)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{code}}{An `rtmb_code(\{ ... \})` or `\{ ... \}` block containing the logic
to be calculated using posterior samples.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The `MCMC_Fit` object itself (invisibly).
Results are appended to the `generate_fit` field.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-resolve_switching"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-resolve_switching}{}}}
\subsection{Method \code{resolve_switching()}}{
Resolve label switching in posterior draws.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$resolve_switching(
  target,
  linked = NULL,
  overwrite = TRUE,
  scalar_fns = list()
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Character string specifying the target variable to base the relabeling on.}

\item{\code{linked}}{Character vector of variable names to be relabeled in the same order as the target. Default is NULL.}

\item{\code{overwrite}}{Logical; whether to overwrite the stored draws in the current object. Default is TRUE.}

\item{\code{scalar_fns}}{A named list of functions to apply to scalar variables for relabeling. Default is an empty list.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Relabeled draws or updated object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-mcmc_fit-clone"></a>}}
\if{latex}{\out{\hypertarget{method-mcmc_fit-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MCMC_Fit$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{multinomial_lpmf}
\alias{multinomial_lpmf}
\title{Multinomial log-probability mass function}
\usage{
multinomial_lpmf(x, size, prob)
}
\arguments{
\item{x}{Vector of counts.}

\item{size}{Total number of trials.}

\item{prob}{Vector of probabilities for each category.}
}
\value{
The sum of the log-probability.
}
\description{
Multinomial log-probability mass function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{sort_loadings}
\alias{sort_loadings}
\title{Sort and display factor loadings neatly}
\usage{
sort_loadings(loadings, cutoff = 0, round_digits = 3)
}
\arguments{
\item{loadings}{Matrix, data frame, or list of factor loadings (if list, the first element is used)}

\item{cutoff}{Absolute loadings below this value will be displayed as blank (default is 0.0)}

\item{round_digits}{Number of decimal places to display (default is 3)}
}
\value{
Sorted loading matrix (returned invisibly, allowing assignment to a variable)
}
\description{
Sort and display factor loadings neatly
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{to_centered_matrix}
\alias{to_centered_matrix}
\title{Vector to centered matrix (RTMB compatible)}
\usage{
to_centered_matrix(x, R, C)
}
\arguments{
\item{x}{A numeric vector of length (R-1) * C.}

\item{R}{Number of rows.}

\item{C}{Number of columns.}
}
\value{
An R x C matrix where each column sums to zero.
}
\description{
Vector to centered matrix (RTMB compatible)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{simple_effects.mcmc_fit}
\alias{simple_effects.mcmc_fit}
\title{Simple effects for MCMC fit objects}
\usage{
\method{simple_effects}{mcmc_fit}(fit, effect, prob = 0.95, sd_multiplier = 1, ...)
}
\arguments{
\item{fit}{An object of class `MCMC_Fit`.}

\item{effect}{Interaction term (e.g., "A:B").}

\item{prob}{Probability for credible intervals.}

\item{sd_multiplier}{Multiplier for SD for continuous moderators.}

\item{...}{Additional arguments.}
}
\description{
Simple effects for MCMC fit objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{logit_normal_lpdf}
\alias{logit_normal_lpdf}
\title{Logit-Normal log-probability density function}
\usage{
logit_normal_lpdf(x, mu, sd, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles (must be strictly between 0 and 1).}

\item{mu}{Vector of means on the logit scale.}

\item{sd}{Vector of standard deviations on the logit scale.}
}
\value{
The sum of the log-density.
}
\description{
Logit-Normal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{log_det_chol}
\alias{log_det_chol}
\title{Log determinant of a Cholesky factor}
\usage{
log_det_chol(L)
}
\arguments{
\item{L}{A lower triangular Cholesky factor matrix.}
}
\value{
The log determinant of the corresponding covariance matrix.
}
\description{
Log determinant of a Cholesky factor
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{validate_data}
\alias{validate_data}
\title{Pre-validation of data and parameters}
\usage{
validate_data(dat_list)
}
\arguments{
\item{dat_list}{A list of data to be passed to the model (usually containing matrices or numeric vectors).}
}
\value{
Returns invisible `NULL` on success. Interrupts execution with `stop()` or issues `warning()` if issues are found.
}
\description{
Before passing data to `rtmb_model`, it checks for the presence of R-specific data types (such as data.frame)
and missing values, and outputs errors or warnings accordingly.
}
\details{
RTMB's automatic differentiation engine requires pure `matrix` or `numeric` types.
If a user mistakenly passes a `data.frame`, incomprehensible errors occur during the construction of the computation graph.
This function catches such issues in advance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{gamma_lpdf}
\alias{gamma_lpdf}
\title{Gamma log-probability density function}
\usage{
gamma_lpdf(x, shape, rate, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{shape}{Shape parameter.}

\item{rate}{Rate parameter.}
}
\value{
The sum of the log-density.
}
\description{
Gamma log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{centered_multi_normal_lpdf}
\alias{centered_multi_normal_lpdf}
\title{Centered / Centered matrix multivariate normal log-probability density function}
\usage{
centered_multi_normal_lpdf(x, sigma, K = NULL)
}
\arguments{
\item{x}{Vector or matrix of quantiles. If a matrix, it assumes each column sums to zero.}

\item{sigma}{Standard deviation parameter. Can be a scalar, or a vector of length ncol(x) if x is a matrix.}

\item{K}{Dimension. If NULL, inferred from the length or nrow of x.}
}
\value{
The log-density.
}
\description{
Centered / Centered matrix multivariate normal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ADVI.R
\name{ADVI_method}
\alias{ADVI_method}
\title{Automatic Differentiation Variational Inference (ADVI)}
\usage{
ADVI_method(
  model,
  par_list,
  pl_full,
  iter = 3000,
  tol_rel_obj = 0.001,
  window_size = 100,
  num_samples = 1000,
  alpha = 0.01,
  laplace = FALSE,
  print_freq = 500,
  method = c("meanfield", "fullrank", "hybrid"),
  update_progress = NULL,
  update_interval = 100
)
}
\arguments{
\item{model}{An RTMB objective function object (`ad_obj`).}

\item{par_list}{A list defining the structure of parameters to be estimated.}

\item{pl_full}{A list defining the full structure of parameters including random effects.}

\item{iter}{Integer; fixed number of iterations for the optimization. Default is 3000.}

\item{tol_rel_obj}{Numeric; relative tolerance for the ELBO to check convergence. Default is 0.001.}

\item{window_size}{Integer; size of the moving window to calculate the median ELBO. Default is 100.}

\item{num_samples}{Integer; number of posterior draws to generate after optimization. Default is 1000.}

\item{alpha}{Numeric; learning rate (step size) for the Adam optimizer. Default is 0.01.}

\item{laplace}{Logical; whether Laplace approximation is used. Default is FALSE.}

\item{print_freq}{Integer; frequency of printing progress to the console. Set to 0 to disable. Default is 500.}

\item{method}{Character; type of variational distribution. One of "meanfield", "fullrank", or "hybrid". Default is "meanfield".}

\item{update_progress}{Optional function to update a progress bar.}

\item{update_interval}{Integer; interval for updating the progress bar. Default is 100.}
}
\value{
A list containing `fit`, `random_fit`, `elbo_history`, `elbo_final`, `rel_obj_final`, and `converged`.
}
\description{
Automatic Differentiation Variational Inference (ADVI)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{fiml_multi_normal_CF_lpdf}
\alias{fiml_multi_normal_CF_lpdf}
\title{Multivariate normal log-probability density function parameterized by Cholesky factor of correlation matrix (FIML)}
\usage{
fiml_multi_normal_CF_lpdf(x, mean, sd, CF_Omega)
}
\arguments{
\item{x}{Vector or matrix of quantiles containing NAs.}

\item{mean}{Vector of means.}

\item{sd}{Vector of standard deviations.}

\item{CF_Omega}{Cholesky factor of the correlation matrix.}
}
\value{
The sum of the log-density using Full Information Maximum Likelihood.
}
\description{
Multivariate normal log-probability density function parameterized by Cholesky factor of correlation matrix (FIML)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{neg_binomial_2_lpmf}
\alias{neg_binomial_2_lpmf}
\title{Negative binomial log-probability mass function (alternative parameterization)}
\usage{
neg_binomial_2_lpmf(x, mu, size, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{mu}{Mean parameter.}

\item{size}{Dispersion parameter.}
}
\value{
The sum of the log-probability.
}
\description{
Negative binomial log-probability mass function (alternative parameterization)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{fabs}
\alias{fabs}
\title{Smooth absolute value function}
\usage{
fabs(x, eps = 1e-08)
}
\arguments{
\item{x}{A numeric vector.}

\item{eps}{A small value added for numerical stability (default is 1e-8).}
}
\value{
The smooth absolute value of x.
}
\description{
Smooth absolute value function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{sufficient_multi_normal_lpdf}
\alias{sufficient_multi_normal_lpdf}
\title{Sufficient statistics multivariate normal log-probability density function (covariance parameterization)}
\usage{
sufficient_multi_normal_lpdf(S_mat, N, y_bar, mean, Sigma)
}
\arguments{
\item{S_mat}{Deviation sum of squares matrix.}

\item{N}{Sample size.}

\item{y_bar}{Sample mean vector.}

\item{mean}{Mean parameter vector.}

\item{Sigma}{Covariance matrix.}
}
\value{
The exact log-likelihood of the N raw observations.
}
\description{
Sufficient statistics multivariate normal log-probability density function (covariance parameterization)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{rtmb_model}
\alias{rtmb_model}
\title{Create an RTMB_Model Object}
\usage{
rtmb_model(
  data,
  code,
  par_names = list(),
  init = NULL,
  view = NULL,
  fixed = NULL,
  silent = FALSE
)
}
\arguments{
\item{data}{A named list containing observation data and constants (e.g., sample size, matrices) used in the model.}

\item{code}{A model definition block generated by \code{rtmb_code(...)} (must include \code{parameters} and \code{model} blocks).}

\item{par_names}{A named list of character vectors corresponding to the dimensions of specific parameters (optional).}

\item{init}{A list or numeric vector of initial values for parameters (optional). If not specified, initialized randomly.}

\item{view}{Character vector of parameter names to be displayed preferentially at the top when outputting results like \code{summary()} (optional).}

\item{fixed}{A named list of parameter values to fix (optional). Useful for scoring or plug-in estimation where some parameters (e.g., item parameters) are fixed to known values.}

\item{silent}{Logical; if TRUE, suppresses diagnostic messages during model creation. Default is FALSE.}
}
\value{
An \code{RTMB_Model} class instance with a compiled and pre-tested automatic differentiation function.

An \code{RTMB_Model} class instance with a compiled and pre-tested automatic differentiation function.
}
\description{
The `rtmb_model` function acts as the core constructor for compiling and combining
user-defined data with the model code (defined via `rtmb_code`). It generates an
`RTMB_Model` (R6 class) instance, which serves as the foundation for performing
Bayesian inference, including Maximum A Posteriori (MAP) estimation, Variational
Inference (ADVI), and Markov Chain Monte Carlo (MCMC) sampling.
}
\details{
\strong{Model Compilation and Pre-checking:}
When this function is called, it evaluates the provided data and model blocks.
It performs a "sandbox execution" (pre-check) using dummy initial values to dynamically
detect common structural errors—such as undefined variables, out-of-bounds indices,
or incompatible matrix operations—before proceeding to the computationally expensive
Automatic Differentiation (MakeADFun) phase. Cryptic backend errors are caught and
translated into user-friendly hints.

\strong{Writing AD-Compatible Code (Important):}
To ensure the model is differentiable, you must follow specific syntax rules
when writing code within \code{rtmb_code}. Avoid discrete branching (\code{if}, \code{ifelse})
based on parameters and use numerically stable functions.
See \code{\link{rtmb_syntax}} for a detailed guide on Automatic Differentiation requirements.

\strong{Initial Values (\code{init}):}
Initial parameter values can be specified as a flat numeric vector or a named list.
If a partial list is provided, the unspecified parameters are automatically initialized
with random values drawn from a uniform distribution on the unconstrained scale.
If \code{init = NULL}, all parameters are initialized randomly.

\strong{Parameter Labeling (\code{par_names}):}
By default, vector or matrix parameters are displayed with numeric indices (e.g., \code{mu[1]}).
You can pass a named list of character vectors to \code{par_names} to assign meaningful
labels to specific dimensions (e.g., \code{mu[Control]}), vastly improving the readability
of summary outputs and trace plots.

\strong{Available Inference Methods on the Returned Object:}
The returned \code{RTMB_Model} instance provides the following core methods:
\itemize{
  \item \code{$optimize(...)}: Performs Maximum A Posteriori (MAP) or Maximum Likelihood estimation. Returns a \code{MAP_Fit} object.
  \item \code{$sample(...)}: Draws posterior samples using the NUTS (No-U-Turn Sampler) algorithm. Returns an \code{MCMC_Fit} object.
  \item \code{$variational(...)}: Performs Mean-field or Full-rank Automatic Differentiation Variational Inference (ADVI). Returns a \code{VB_Fit} object.
}
}
\examples{
\donttest{
# Simulate data for 3 groups
set.seed(123)
N <- 60
group_idx <- sample(1:3, N, replace = TRUE)
group_names <- c("Control", "Treatment_A", "Treatment_B")

# True group means: Control = 0, Treatment_A = 2, Treatment_B = -1
true_means <- c(0, 2, -1)
y <- true_means[group_idx] + rnorm(N, mean = 0, sd = 0.5)

data_list <- list(N = N, K = 3, group_idx = group_idx, y = y)

# Define the model using rtmb_code
model_code <- rtmb_code(
  parameters = {
    mu    = Dim(K)             # Vector of length K (group means)
    sigma = Dim(1, lower = 0)  # Scalar (residual standard deviation)
  },
  model = {
    # Priors
    for (k in 1:K) mu[k] ~ normal(0, 10)
    sigma ~ exponential(1)

    # Likelihood
    for (i in 1:N) {
      y[i] ~ normal(mu[group_idx[i]], sigma)
    }
  }
)

# --- 1. Basic Model Creation ---
# Create the RTMB_Model object
mod_basic <- rtmb_model(
  data = data_list,
  code = model_code
)

# Perform Maximum A Posteriori (MAP) estimation
map_basic <- mod_basic$optimize()
# The summary displays default parameter names: mu[1], mu[2], mu[3]
map_basic$summary()

# --- 2. Optional: Adding Custom Parameter Names and initial values ---
# You can optionally use 'par_names' to assign meaningful labels
# to vector or matrix elements for easier interpretation.
mod_named <- rtmb_model(
  data = data_list,
  code = model_code,
  init = list(mu = rep(0, 3), sigma = 1),
  par_names = list(mu = group_names)
)

map_named <- mod_named$optimize()
# The summary now displays: mu[Control], mu[Treatment_A], mu[Treatment_B]
map_named$summary()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{student_t_lpdf}
\alias{student_t_lpdf}
\title{Student-t log-probability density function}
\usage{
student_t_lpdf(x, df, mu = 0, sigma = 1, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{df}{Degrees of freedom.}

\item{mu}{Location parameter.}

\item{sigma}{Scale parameter.}
}
\value{
The sum of the log-density.
}
\description{
Student-t log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{to_lower_tri}
\alias{to_lower_tri}
\title{Vector to lower triangular matrix (RTMB compatible)}
\usage{
to_lower_tri(x, M, D)
}
\arguments{
\item{x}{A numeric or advector.}

\item{M}{Number of rows.}

\item{D}{Number of columns.}
}
\value{
An M x D lower triangular matrix.
}
\description{
Vector to lower triangular matrix (RTMB compatible)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{inverse_gamma_lpdf}
\alias{inverse_gamma_lpdf}
\title{Inverse-gamma log-probability density function}
\usage{
inverse_gamma_lpdf(x, shape, scale, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{shape}{Shape parameter.}

\item{scale}{Scale parameter.}
}
\value{
The sum of the log-density.
}
\description{
Inverse-gamma log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{item_info.RTMB_Fit_Base}
\alias{item_info.RTMB_Fit_Base}
\title{Item Information Function for RTMB_Fit_Base}
\usage{
\method{item_info}{RTMB_Fit_Base}(x, theta_seq = seq(-4, 4, length.out = 100), items = NULL, ...)
}
\arguments{
\item{x}{An object of class RTMB_Fit_Base.}

\item{theta_seq}{Sequence of trait values (ability) to evaluate.}

\item{items}{Index or item names to restrict the calculation to specific items (optional).}

\item{...}{Additional arguments.}
}
\description{
Item Information Function for RTMB_Fit_Base
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{model_code}
\alias{model_code}
\title{Model Code Wrapper for RTMB}
\usage{
model_code(expr, env = parent.frame())
}
\arguments{
\item{expr}{A block of code containing model description.}

\item{env}{Environment to assign to the generated function.}
}
\value{
A standard R function object taking (dat, par).
}
\description{
Model Code Wrapper for RTMB
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_lm_glm_lmer.R
\name{rtmb_lm}
\alias{rtmb_lm}
\title{RTMB-based Linear Regression wrapper function}
\usage{
rtmb_lm(
  formula,
  data,
  prior = prior_flat(),
  y_range = NULL,
  init = NULL,
  fixed = NULL,
  gmc = NULL,
  centering = NULL,
  view = NULL,
  factors = NULL,
  contrasts = "treatment",
  missing = c("listwise", "fiml"),
  WAIC = FALSE,
  ...
)
}
\arguments{
\item{formula}{Formula (e.g., Y ~ X1 + X2)}

\item{data}{Data frame}

\item{prior}{An object of class `"rtmb_prior"`.
Use `prior_flat()` for no prior, `prior_normal()` for default normal/exponential priors,
or `prior_weak()`, `prior_rhs()`, `prior_ssp()` for weakly informative or regularized Bayesian inference.
Default is `prior_flat()`.}

\item{y_range}{Theoretical minimum and maximum values of the response variable as a vector c(min, max). Specifying this automatically enables weakly informative priors.}

\item{init}{List of initial values.}

\item{fixed}{Optional named list of fixed values for specific parameters.}

\item{gmc}{Character vector of variable names for GMC}

\item{centering}{Alias for `gmc`.}

\item{view}{Optional character vector of parameter names to show first in summary.}

\item{factors}{Character vector of variable names to be treated as factors.}

\item{contrasts}{Character string specifying the contrast type ("treatment" or "sum").}

\item{missing}{Missing value handling strategy: "listwise".}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}

\item{...}{Additional arguments passed to \code{rtmb_glmer()}.}
}
\description{
RTMB-based Linear Regression wrapper function
}
\examples{
  # --- 1. Linear Regression (rtmb_lm) ---
  # Fit a linear regression model using the debate dataset
  data(debate, package = "BayesRTMB")
  fit_lm <- rtmb_lm(sat ~ talk + perf, data = debate)
  map_lm <- fit_lm$optimize()
  map_lm$summary()

  # --- 2. Generalized Linear Model (rtmb_glm) ---
  # Fit a logistic regression model using the debate dataset
  data(debate, package = "BayesRTMB")
  fit_glm <- rtmb_glm(cond ~ talk + sat, data = debate, family = "bernoulli")
  map_glm <- fit_glm$optimize()
  map_glm$summary()

  # --- 3. Generalized Linear Mixed Model (rtmb_glmer) ---
  # Fit a linear mixed-effects model using the debate dataset
  data(debate, package = "BayesRTMB")
  fit_glmer <- rtmb_glmer(talk ~ cond + (1 | group), data = debate, family = "gaussian")

  # MAP estimation using Laplace approximation for random effects
  map_glmer <- fit_glmer$optimize(laplace = TRUE)
  map_glmer$summary()

  # MCMC sampling (chains and iterations reduced for faster execution)
  \donttest{
  mcmc_glmer <- fit_glmer$sample(sampling = 500, warmup = 500, chains = 2)
  mcmc_glmer$summary()
  }

  # --- 4. Linear Mixed Model (rtmb_lmer) ---
  # A convenient wrapper for Gaussian mixed models (identical to rtmb_glmer with family="gaussian")
  fit_lmer <- rtmb_lmer(sat ~ talk + (1 | group), data = debate)
  map_lmer <- fit_lmer$optimize()
  map_lmer$summary()

  # --- 5. Regularized Regression (Variable Selection) ---
  # You can apply regularization to the fixed effects to shrink noise variables towards zero.
  # Use prior = prior_rhs() for the Regularized Horseshoe prior,
  # or prior_ssp() for the Spike-and-Slab prior.
  # Note: When using regularization, you must specify 'y_range' (the theoretical minimum and maximum
  # values of the response variable) to automatically set up the required weakly informative priors.

  # Fit a linear regression using debate predictors with the Horseshoe prior
  fit_rhs <- rtmb_lm(
    sat ~ talk + perf + skill,
    data = debate,
    prior = prior_rhs(),
    y_range = c(1, 5)
  )
  map_rhs <- fit_rhs$optimize()
  # Summarize only the fixed effects (slopes)
  map_rhs$summary("b")

  # Fit a linear regression with the Spike-and-Slab prior
  fit_ssp <- rtmb_lm(
    sat ~ talk + perf + skill,
    data = debate,
    prior = prior_ssp(),
    y_range = c(1, 5)
  )
  map_ssp <- fit_ssp$optimize()
  map_ssp$summary("b")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{cauchy_lpdf}
\alias{cauchy_lpdf}
\title{Cauchy log-probability density function}
\usage{
cauchy_lpdf(x, location, scale, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{location}{Location parameter.}

\item{scale}{Scale parameter.}
}
\value{
The sum of the log-density.
}
\description{
Cauchy log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{bernoulli_lpmf}
\alias{bernoulli_lpmf}
\title{Bernoulli log-probability mass function}
\usage{
bernoulli_lpmf(x, prob, sum = TRUE)
}
\arguments{
\item{x}{Vector of binary outcomes (0 or 1).}

\item{prob}{Probability of success.}
}
\value{
The sum of the log-probability.
}
\description{
Bernoulli log-probability mass function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{math_functions}
\alias{math_functions}
\title{Mathematical and Matrix Utility Functions for RTMB Models}
\description{
This page summarizes the utility functions provided to assist in writing
efficient and numerically stable model code within `rtmb_code`. These functions
are specifically designed to be compatible with RTMB's Automatic Differentiation (AD).
}
\details{
\strong{1. Link and Inverse Functions:}
\itemize{
  \item \code{logit(x)}: Computes the logit transformation \code{log(x/(1-x))}.
  \item \code{inv_logit(x)}: Computes the inverse logit (logistic) transformation \code{1/(1+exp(-x))}.
}

\strong{2. Numerical Stability Utilities:}
These functions use specialized algorithms to prevent overflow/underflow in AD calculations.
\itemize{
  \item \code{log_sum_exp(x)}: Safely computes \code{log(sum(exp(x)))} using the log-sum-exp trick.
  \item \code{log1p_exp(x)}: Computes \code{log(1 + exp(x))} stably.
  \item \code{log1m_exp(x)}: Computes \code{log(1 - exp(x))} stably for \code{x < 0}.
  \item \code{log_softmax(x)}: Computes the log of the softmax function.
  \item \code{fabs(x)}: A smooth version of the absolute value function \code{abs(x)} to ensure differentiability at zero.
}

\strong{3. Matrix and Vector Transformations:}
Used to convert unconstrained vectors into structured matrices (e.g., for factor analysis or identification constraints).
\itemize{
  \item \code{sum_to_zero(x)}: Transforms a vector of length K-1 into a vector of length K that sums to zero.
  \item \code{to_lower_tri(x, M, D)}: Fills a matrix of size M x D with elements from vector \code{x} in a lower-triangular fashion.
  \item \code{to_centered_matrix(x, R, C)}: Creates an R x C matrix where each column sums to zero.
  \item \code{to_centered_tri(x, R, C)}: Creates an R x C matrix with column-wise sum-to-zero constraints on the lower elements (useful for identification in factor analysis).
}

\strong{4. Linear Algebra for AD:}
\itemize{
  \item \code{log_det_chol(L)}: Calculates the log-determinant of a covariance matrix from its Cholesky factor \code{L}.
  \item \code{quad_form_chol(x, L)}: Computes the quadratic form x^T Sigma^(-1) x using the Cholesky factor \code{L}.
  \item \code{distance(x, y)}: Computes the Euclidean distance between two vectors with a small epsilon for stability.
}
}
\concept{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{multi_normal_CF_lpdf}
\alias{multi_normal_CF_lpdf}
\title{Multivariate normal log-probability density function parameterized by Cholesky factor of correlation matrix}
\usage{
multi_normal_CF_lpdf(x, mean, sd, CF_Omega, sum = TRUE)
}
\arguments{
\item{x}{Vector or matrix of quantiles.}

\item{mean}{Vector or matrix of means.}

\item{sd}{Vector of standard deviations.}

\item{CF_Omega}{Cholesky factor of the correlation matrix.}
}
\value{
The sum of the log-density.
}
\description{
Multivariate normal log-probability density function parameterized by Cholesky factor of correlation matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_utils.R
\name{to_long}
\alias{to_long}
\title{Convert Wide Data to Long Format}
\usage{
to_long(data, within = NULL, label = NULL, value = "Value", id = NULL)
}
\arguments{
\item{data}{A data frame in wide format.}

\item{within}{The columns to gather into long format. Can be:
\itemize{
  \item A character vector of column names.
  \item A range string like \code{"time1:time4"}.
  \item A prefix string like \code{"time"} (matches all columns starting with "time").
}}

\item{label}{The name for the new column that will contain the level names (e.g., "Time"). 
If NULL, it defaults to the prefix used in \code{within} or "Condition".}

\item{value}{The name for the new column that will contain the measurement values. 
Default is "Value".}

\item{id}{The identifier columns that should be repeated for each row. 
If NULL (default), all columns NOT specified in \code{within} are treated as IDs.}
}
\value{
A data frame in long format.
}
\description{
A highly intuitive wrapper around \code{stats::reshape} designed for 
psychological research. It converts data from wide format to long format
by identifying within-subjects factors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{wishart_lpdf}
\alias{wishart_lpdf}
\title{Wishart log-probability density function}
\usage{
wishart_lpdf(X, n, V)
}
\arguments{
\item{X}{Symmetric positive-definite matrix (scatter matrix).}

\item{n}{Degrees of freedom (sample size N).}

\item{V}{Scale matrix (covariance matrix Omega).}
}
\value{
The log-density with all constant terms.
}
\description{
Wishart log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{gaussian_process_lpdf}
\alias{gaussian_process_lpdf}
\title{Gaussian Process Log-Density (Squared Exponential Kernel)}
\usage{
gaussian_process_lpdf(
  y,
  x,
  mean = 0,
  magnitude = 1,
  smoothing = 1,
  noise = 0.01,
  sum = TRUE
)
}
\arguments{
\item{y}{Observation vector (N).}

\item{x}{Coordinate vector or matrix (N x D).}

\item{mean}{Mean vector (scalar or length N).}

\item{magnitude}{Signal standard deviation (alpha).}

\item{smoothing}{Length-scale (rho).}

\item{noise}{Measurement noise standard deviation (sigma).}

\item{sum}{Logical; whether to return the sum of log-densities.}
}
\value{
Log-density value.
}
\description{
Calculates the log-density of a Gaussian Process with a Squared Exponential (RBF) kernel.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{binomial_lpmf}
\alias{binomial_lpmf}
\title{Binomial log-probability mass function}
\usage{
binomial_lpmf(x, size, prob, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles (number of successes).}

\item{size}{Number of trials.}

\item{prob}{Probability of success.}
}
\value{
The sum of the log-probability.
}
\description{
Binomial log-probability mass function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{centered_tri_multi_normal_lpdf}
\alias{centered_tri_multi_normal_lpdf}
\title{Centered triangular multivariate normal log-probability density function}
\usage{
centered_tri_multi_normal_lpdf(x, sigma)
}
\arguments{
\item{x}{Matrix of quantiles.}

\item{sigma}{Standard deviation parameter(s).}
}
\value{
The log-density.
}
\description{
Centered triangular multivariate normal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_lrt.R
\name{rtmb_lrt}
\alias{rtmb_lrt}
\title{Fit a Latent Rank Theory (LRT) Model}
\usage{
rtmb_lrt(
  formula,
  k = 3,
  data = NULL,
  rank_coords = NULL,
  covariance = c("diagonal", "diagonal_equal", "full", "full_equal", "full_equal_corr"),
  magnitude = NULL,
  smoothing = NULL,
  noise = 0.01,
  prob_smoothing = FALSE,
  link = c("ordered", "sequential"),
  prior = prior_flat(),
  y_range = NULL,
  fixed = NULL,
  two_stage = FALSE,
  WAIC = FALSE,
  ...
)
}
\arguments{
\item{formula}{A formula specifying the response variable(s).}

\item{k}{Number of ranks (mixture components).}

\item{data}{A data frame containing the variables.}

\item{rank_coords}{Optional numeric vector of coordinates for each rank. Default is 1:k.}

\item{covariance}{Covariance structure: "diagonal", "diagonal_equal", "full", "full_equal", or "full_equal_corr".}

\item{magnitude}{Signal standard deviation for the GP prior. If NULL, it is estimated.}

\item{smoothing}{Length-scale for the GP prior. If NULL, it is estimated.}

\item{noise}{Measurement noise for the GP prior (default is 0.01).}

\item{prob_smoothing}{Logical; whether to apply smoothing to the class membership probabilities.}

\item{link}{Link function for class probabilities: "ordered" or "sequential".}

\item{prior}{Prior configuration: `prior_flat()`, `prior_normal()`,
`prior_weak()`, `prior_rhs()`, or `prior_ssp()`. Default is
`prior_flat()`. If `y_range` is supplied with the default flat prior,
the wrapper automatically switches to `prior_weak()`.}

\item{y_range}{Optional numeric vector or matrix defining the theoretical range (min, max) of response variables.
Specifying this automatically enables weakly informative priors if `prior` is `prior_flat()`.}

\item{fixed}{Optional named list of fixed values for specific parameters.}

\item{two_stage}{Logical; if TRUE, estimate the latent-rank measurement model
first and then estimate the rank regression with delta-method uncertainty
propagation. Currently supported for `$optimize()` only.}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}

\item{...}{Additional arguments passed to `rtmb_model`.}
}
\value{
A \code{RTMB_Model} object.
}
\description{
Fits a Latent Rank Theory model, which is a mixture model with ordered ranks
and Gaussian Process smoothing on the mean profiles.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{normal_lpdf}
\alias{normal_lpdf}
\title{Normal log-probability density function}
\usage{
normal_lpdf(x, mean, sd, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{mean}{Vector of means.}

\item{sd}{Vector of standard deviations.}

\item{sum}{Logical; if TRUE (default), returns the sum of log-densities. If FALSE, returns element-wise log-densities.}
}
\value{
The sum of the log-density (scalar) or a vector of log-densities.
}
\description{
Normal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmc_index.R
\name{ess_bulk}
\alias{ess_bulk}
\title{Calculate Bulk Effective Sample Size}
\usage{
ess_bulk(sims)
}
\arguments{
\item{sims}{A matrix of samples (iterations x chains).}
}
\value{
A numeric value.
}
\description{
Calculate Bulk Effective Sample Size
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{laplace_lpdf}
\alias{laplace_lpdf}
\title{Laplace log-probability density function}
\usage{
laplace_lpdf(x, location = 0, scale = 1, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{location}{Location parameter.}

\item{scale}{Scale parameter.}
}
\value{
The sum of the log-density.
}
\description{
Laplace log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{lower_tri_normal_lpdf}
\alias{lower_tri_normal_lpdf}
\title{Lower-triangular normal log-probability density function}
\usage{
lower_tri_normal_lpdf(x, mean = 0, sd = 1)
}
\arguments{
\item{x}{A matrix of lower-triangular parameters.}

\item{mean}{Mean of the normal distribution.}

\item{sd}{Standard deviation of the normal distribution.}
}
\value{
The log-density calculated only for the non-zero lower triangular elements.
}
\description{
Lower-triangular normal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{transform_code}
\alias{transform_code}
\title{Transformed Code Wrapper for RTMB}
\usage{
transform_code(expr, env = parent.frame())
}
\arguments{
\item{expr}{A block of code containing calculations for transformed parameters.}

\item{env}{Environment to assign to the generated function.}
}
\value{
A function taking (dat, par) that returns a named list.
}
\description{
Transformed Code Wrapper for RTMB
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{inv_logit}
\alias{inv_logit}
\title{Inverse logit function}
\usage{
inv_logit(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
The inverse logit of x.
}
\description{
Inverse logit function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{neg_binomial_lpmf}
\alias{neg_binomial_lpmf}
\title{Negative binomial log-probability mass function}
\usage{
neg_binomial_lpmf(x, size, prob, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{size}{Target for number of successful trials.}

\item{prob}{Probability of success in each trial.}
}
\value{
The sum of the log-probability.
}
\description{
Negative binomial log-probability mass function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{item_info}
\alias{item_info}
\title{Calculate Item Information Function}
\usage{
item_info(x, ...)
}
\arguments{
\item{x}{An object of class RTMB_Fit_Base}

\item{...}{Additional arguments.}
}
\description{
Calculate Item Information Function
}
\examples{
\donttest{
  fit <- rtmb_irt(data = BigFive[, 1:5], model = "2PL", type = "ordered")
  map_fit <- fit$optimize()
  ii <- item_info(map_fit)
  plot(ii)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{item_curve}
\alias{item_curve}
\title{Calculate Item Response Curve / Category Response Curve}
\usage{
item_curve(x, ...)
}
\arguments{
\item{x}{An object of class}

\item{...}{Additional arguments.}
}
\description{
Calculate Item Response Curve / Category Response Curve
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_lsmeans}
\alias{plot_lsmeans}
\title{Plot least-squares marginal means}
\usage{
plot_lsmeans(fit, specs, ...)
}
\arguments{
\item{fit}{A fitted model object (e.g. `Classic_Fit`).}

\item{specs}{Character vector specifying the variables for which marginal means are calculated.}

\item{...}{Additional arguments passed to \code{\link{lsmeans}} or the plot method.}
}
\value{
Invisibly returns the plotted object.
}
\description{
Calculate and plot marginal means for a fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_forest}
\alias{plot_forest}
\title{Plot parameter estimates and credible intervals (Forest Plot)}
\usage{
plot_forest(
  x,
  prob = 0.95,
  point_estimate = c("median", "EAP", "mean", "MAP", "marginal_map", "joint_map")
)
}
\arguments{
\item{x}{A 3D array of posterior samples with dimensions `(iterations, chains, variables)`.}

\item{prob}{Numeric. Probability mass for the credible interval (default is 0.95).}

\item{point_estimate}{Character. Point estimate shown as dots. One of
`"median"` (default), `"EAP"`/`"mean"`, `"MAP"`/`"marginal_map"`, or
`"joint_map"`. `"joint_map"` uses the draw with the largest `lp` variable.}
}
\value{
No return value.
}
\description{
Plot parameter estimates and credible intervals (Forest Plot)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{env_to_ordered_list}
\alias{env_to_ordered_list}
\title{Internal function to convert an environment to a list while maintaining original order}
\usage{
env_to_ordered_list(env, orig_list, code_ast = NULL)
}
\description{
Internal function to convert an environment to a list while maintaining original order
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_corr.R
\name{rtmb_corr}
\alias{rtmb_corr}
\title{Fit a Correlation Model using RTMB}
\usage{
rtmb_corr(
  x = NULL,
  data = NULL,
  ID = NULL,
  covariates = NULL,
  method = c("pearson", "spearman", "reml"),
  prior = prior_flat(),
  y_range = NULL,
  init = NULL,
  fixed = NULL,
  missing = c("listwise", "fiml", "pairwise"),
  WAIC = FALSE
)
}
\arguments{
\item{x}{A matrix, data frame, formula, or expression (e.g., \code{cbind(V1, V2)}) of response variables.}

\item{data}{An optional data frame containing the variables.}

\item{ID}{A character string or expression specifying the group ID variable for multilevel models.}

\item{covariates}{Optional numeric matrix or data frame of covariates to be included in the joint MVN model.}

\item{method}{Correlation method for \code{classic()}: \code{"pearson"}, \code{"spearman"}, or \code{"reml"}.}

\item{prior}{Prior configuration object: `prior_flat()`, `prior_normal()`, or `prior_weak()`. Default is `prior_flat()`.}

\item{y_range}{Optional numeric vector or matrix defining the theoretical range (min, max) of response variables.
Required when using \code{prior_weak()}. Can be a vector of length 2 (applies to all variables) or a matrix/list of length P.}

\item{init}{Optional list of initial values.}

\item{fixed}{Optional named list of fixed values for specific parameters.}

\item{missing}{Missing value handling strategy: "listwise" (default), "pairwise", or "fiml" (Full Information Maximum Likelihood).}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}
}
\description{
`rtmb_corr` fits a correlation model to estimate means, standard deviations, and correlation structures.
It supports simple correlation, multilevel correlation, and classical frequentist estimation.
}
\examples{

  # Estimate the correlation between two variables in the debate dataset
  data(debate, package = "BayesRTMB")

  fit_corr <- rtmb_corr(cbind(sat, perf), data = debate)

  \donttest{
  mcmc_corr <- fit_corr$sample(sampling = 500, warmup = 500, chains = 2)
  mcmc_corr$summary()

  bf_corr <- mcmc_corr$bayes_factor(fixed = list(corr = 0))
  print(bf_corr)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_acf}
\alias{plot_acf}
\title{Plot autocorrelation for one variable across chains}
\usage{
plot_acf(x, var_idx = 1)
}
\arguments{
\item{x}{A 3D array of posterior samples with dimensions
`(iterations, chains, variables)`.}

\item{var_idx}{Integer. Index of the variable to plot.}
}
\value{
No return value. This function is called for its side effect
  of plotting.
}
\description{
Draw autocorrelation plots for a selected parameter across all chains.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_conditional_effects}
\alias{plot_conditional_effects}
\title{Plot conditional effects}
\usage{
plot_conditional_effects(fit, effect, ...)
}
\arguments{
\item{fit}{A fitted model object (e.g., `MCMC_Fit`, `MAP_Fit`, `VB_Fit`).}

\item{effect}{Name of the variable to visualize (e.g., "X1" or "X1:X2").}

\item{...}{Additional arguments passed to \code{\link{conditional_effects}} or the plot method.}
}
\value{
Invisibly returns the plotted object of class \code{ce_rtmb}.
}
\description{
Calculate and plot conditional effects for a fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MAP_Fit.R
\name{MAP_Fit}
\alias{MAP_Fit}
\title{MAP fit object}
\description{
MAP fit object

MAP fit object
}
\details{
An R6 class storing optimization results from maximum a posteriori
(MAP) estimation.
}
\section{Super class}{
\code{\link[BayesRTMB:RTMB_Fit_Base]{BayesRTMB::RTMB_Fit_Base}} -> \code{map_fit}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{model}}{The `RTMB_Model` object used for estimation.}

\item{\code{par_vec}}{Parameter vector on the unconstrained scale (constrained values unlisted).}

\item{\code{par}}{Parameter list on the constrained scale.}

\item{\code{par_unc}}{Parameter vector on the unconstrained scale (raw unconstrained values).}

\item{\code{ci_method}}{Method used for CI estimation ("wald", "sampling", or "none").}

\item{\code{objective}}{RTMB objective function object.}

\item{\code{log_ml}}{Log marginal likelihood or related model criterion.}

\item{\code{convergence}}{Optimizer convergence code.}

\item{\code{sd_rep}}{Standard deviation report object.}

\item{\code{df_fixed}}{Summary table for fixed-effect parameters.}

\item{\code{random_effects}}{Random effect estimates.}

\item{\code{df_transform}}{Summary table for transformed parameter estimates.}

\item{\code{df_generate}}{Summary table for generated quantity estimates.}

\item{\code{opt_history}}{A vector of optimize objective history.}

\item{\code{transform}}{List of transformed parameters maintaining their original dimensions.}

\item{\code{generate}}{List of generated quantities maintaining their original dimensions.}

\item{\code{se_samples}}{List of simulated samples for standard error estimation.}

\item{\code{laplace}}{Logical; whether Laplace approximation was used.}

\item{\code{vcov_unc}}{Variance-covariance matrix of parameters in unconstrained space.}

\item{\code{map}}{List; the parameter mapping used.}

\item{\code{marginal_vars}}{Character vector of parameter names requested through `optimize(marginal = ...)`.}

\item{\code{laplace_random_vars}}{Character vector of all parameter names passed to `MakeADFun(random = ...)` during Laplace approximation.}

\item{\code{idx_fix_active}}{Numeric vector; mapping between active parameters and full unconstrained vector.}

\item{\code{show_df}}{Logical; whether to display degrees of freedom in the summary output.}

\item{\code{view}}{Character vector of parameter names to prioritize in summary.}

\item{\code{fallback_needed}}{Logical; whether Hessian/SE fallback was used during optimization.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-map_fit-get_point_estimate}{\code{MAP_Fit$get_point_estimate()}}
\item \href{#method-map_fit-new}{\code{MAP_Fit$new()}}
\item \href{#method-map_fit-ranef}{\code{MAP_Fit$ranef()}}
\item \href{#method-map_fit-draws}{\code{MAP_Fit$draws()}}
\item \href{#method-map_fit-summary}{\code{MAP_Fit$summary()}}
\item \href{#method-map_fit-print}{\code{MAP_Fit$print()}}
\item \href{#method-map_fit-generated_quantities}{\code{MAP_Fit$generated_quantities()}}
\item \href{#method-map_fit-WAIC}{\code{MAP_Fit$WAIC()}}
\item \href{#method-map_fit-diagnose}{\code{MAP_Fit$diagnose()}}
\item \href{#method-map_fit-profile}{\code{MAP_Fit$profile()}}
\item \href{#method-map_fit-clone}{\code{MAP_Fit$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="EAP"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-EAP'><code>BayesRTMB::RTMB_Fit_Base$EAP()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="MAP"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-MAP'><code>BayesRTMB::RTMB_Fit_Base$MAP()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="estimate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-estimate'><code>BayesRTMB::RTMB_Fit_Base$estimate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="fa_rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-fa_rotate'><code>BayesRTMB::RTMB_Fit_Base$fa_rotate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-rotate'><code>BayesRTMB::RTMB_Fit_Base$rotate()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-get_point_estimate"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-get_point_estimate}{}}}
\subsection{Method \code{get_point_estimate()}}{
Get point estimate for a target parameter.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$get_point_estimate(target, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Target parameter name.}

\item{\code{...}}{Additional arguments, ignored for MAP fits.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Matrix, array, vector, or scalar point estimate.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-new"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-new}{}}}
\subsection{Method \code{new()}}{
Create a new `MAP_Fit` object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$new(
  model,
  par_vec = NULL,
  par = NULL,
  objective = NULL,
  log_ml = NULL,
  convergence = NULL,
  sd_rep = NULL,
  df_fixed = NULL,
  random_effects = NULL,
  df_transform = NULL,
  df_generate = NULL,
  opt_history = NULL,
  transform = NULL,
  generate = NULL,
  se_samples = NULL,
  par_unc = NULL,
  ci_method = "wald",
  laplace = TRUE,
  map = NULL,
  vcov_unc = NULL,
  marginal_vars = NULL,
  laplace_random_vars = NULL,
  idx_fix_active = NULL,
  show_df = TRUE,
  view = NULL,
  fallback_needed = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{model}}{The `RTMB_Model` object used for estimation.}

\item{\code{par_vec}}{Parameter vector on the unconstrained scale (constrained values unlisted).}

\item{\code{par}}{Parameter list on the constrained scale.}

\item{\code{objective}}{The objective function value at the optimum.}

\item{\code{log_ml}}{Log marginal likelihood.}

\item{\code{convergence}}{Optimizer convergence code.}

\item{\code{sd_rep}}{The `sdreport` object from TMB.}

\item{\code{df_fixed}}{Data frame of fixed effects estimates and CIs.}

\item{\code{random_effects}}{Data frame of random effects estimates and CIs.}

\item{\code{df_transform}}{Data frame of transformed parameters.}

\item{\code{df_generate}}{Data frame of generated quantities.}

\item{\code{opt_history}}{Data frame of optimization history.}

\item{\code{transform}}{List of transformed parameters maintaining their original dimensions.}

\item{\code{generate}}{List of generated quantities maintaining their original dimensions.}

\item{\code{se_samples}}{List of simulated samples for standard error estimation.}

\item{\code{par_unc}}{Parameter vector on the unconstrained scale (raw values).}

\item{\code{ci_method}}{Method used for CI estimation ("wald", "sampling", or "none").}

\item{\code{laplace}}{Logical; whether Laplace approximation was used.}

\item{\code{map}}{List; the parameter mapping used.}

\item{\code{vcov_unc}}{Variance-covariance matrix of parameters in unconstrained space.}

\item{\code{marginal_vars}}{Character vector of parameter names requested through `optimize(marginal = ...)`.}

\item{\code{laplace_random_vars}}{Character vector of all parameter names passed to `MakeADFun(random = ...)` during Laplace approximation.}

\item{\code{idx_fix_active}}{Numeric vector; mapping between active parameters and full unconstrained vector.}

\item{\code{show_df}}{Logical; whether to display degrees of freedom in the summary output. Default is TRUE.}

\item{\code{view}}{Character vector of parameter names to prioritize in summary.}

\item{\code{fallback_needed}}{Logical; whether Hessian/SE fallback was used during optimization.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-ranef"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-ranef}{}}}
\subsection{Method \code{ranef()}}{
Return random effect estimates as a named list.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$ranef()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A named list of random effect estimates.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-draws"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-draws}{}}}
\subsection{Method \code{draws()}}{
Extract samples from the asymptotic posterior distribution.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$draws(
  pars = NULL,
  inc_random = FALSE,
  inc_transform = TRUE,
  inc_generate = TRUE,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character or numeric vector specifying the names or indices of parameters to extract.}

\item{\code{inc_random}}{Logical; whether to include random effects.}

\item{\code{inc_transform}}{Logical; whether to include transformed parameters.}

\item{\code{inc_generate}}{Logical; whether to include generated quantities.}

\item{\code{...}}{Ignored.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
An array of samples [iterations, 1, parameters].
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-summary"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-summary}{}}}
\subsection{Method \code{summary()}}{
Summarize MAP estimates.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$summary(
  pars = NULL,
  max_rows = 10,
  digits = 5,
  ranef = FALSE,
  view = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character vector specifying the names of parameters to summarize. If NULL, all available parameters are summarized.}

\item{\code{max_rows}}{Maximum number of rows to print in summaries. Default is 10.}

\item{\code{digits}}{Number of digits to print.}

\item{\code{ranef}}{Logical; whether to also display random effect estimates. Default is FALSE.}

\item{\code{view}}{Character vector of parameter names to prioritize or filter by.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A summary object, typically a data frame.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-print"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-print}{}}}
\subsection{Method \code{print()}}{
Print a brief summary of the fitted object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$print(pars = NULL, max_rows = 10, digits = 5, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character vector specifying the names of parameters to summarize.}

\item{\code{max_rows}}{Maximum number of rows to print in summaries.}

\item{\code{digits}}{Number of digits to print.}

\item{\code{...}}{Additional arguments passed to the `summary` method.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-generated_quantities"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-generated_quantities}{}}}
\subsection{Method \code{generated_quantities()}}{
Compute generated quantities from the MAP estimate.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$generated_quantities(code)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{code}}{An `rtmb_code(\{ ... \})` or `\{ ... \}` block containing the logic to be calculated using the MAP estimate.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The `MAP_Fit` object itself (invisibly). Results are added or updated in the `generate` list and `df_generate`.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-WAIC"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-WAIC}{}}}
\subsection{Method \code{WAIC()}}{
Compute an approximate WAIC from sampling-based uncertainty propagation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$WAIC()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
A `waic_BayesRTMB` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-diagnose"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-diagnose}{}}}
\subsection{Method \code{diagnose()}}{
Run basic diagnostics for the MAP fit.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$diagnose(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to `diagnose_map_fit()`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A `diagnose_BayesRTMB` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-profile"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-profile}{}}}
\subsection{Method \code{profile()}}{
Calculate Profile Likelihood confidence intervals for specific parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$profile(
  pars = NULL,
  level = 0.95,
  trace = FALSE,
  digits = 5,
  show_plot = FALSE,
  quiet = FALSE,
  jacobian = "none",
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character vector of parameter names to profile. If NULL, all fixed parameters are profiled.}

\item{\code{level}}{Confidence level (default is 0.95).}

\item{\code{trace}}{Logical; whether to print profiling progress. Default is FALSE.}

\item{\code{digits}}{Integer; number of decimal places to print. Default is 5.}

\item{\code{show_plot}}{Logical; whether to plot the profile likelihood curves. Default is FALSE.}

\item{\code{quiet}}{Logical; whether to suppress text output. Default is FALSE.}

\item{\code{jacobian}}{Character; "none" (default), "random", or "all". Whether to include Jacobian adjustments for transformations.}

\item{\code{...}}{Additional arguments passed to TMB::tmbprofile (e.g., ytol).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A data frame containing the profile-based confidence intervals, with the raw profile objects stored in the "profiles" attribute.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-map_fit-clone"></a>}}
\if{latex}{\out{\hypertarget{method-map_fit-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{MAP_Fit$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{sufficient_multi_normal_fa_lpdf}
\alias{sufficient_multi_normal_fa_lpdf}
\title{Sufficient statistics factor analysis multivariate normal log-probability density function}
\usage{
sufficient_multi_normal_fa_lpdf(S_mat, N, y_bar, mu, psi, Lambda)
}
\arguments{
\item{S_mat}{Deviation sum of squares matrix.}

\item{N}{Sample size.}

\item{y_bar}{Sample mean vector.}

\item{mu}{Mean parameter vector.}

\item{psi}{Vector of unique variances (P).}

\item{Lambda}{Factor loading matrix (P x K).}
}
\value{
The exact log-likelihood of the N raw observations.
}
\description{
Sufficient statistics factor analysis multivariate normal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameters.R
\name{Dim}
\alias{Dim}
\title{Define parameter dimensions and types}
\usage{
Dim(dim = 1, type = NULL, lower = NULL, upper = NULL, random = FALSE)
}
\arguments{
\item{dim}{Dimensions of the parameter.}

\item{type}{Type of the parameter.}

\item{lower}{Lower bound.}

\item{upper}{Upper bound.}

\item{random}{Logical; whether it is a random effect.}
}
\description{
Define parameter dimensions and types
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{lognormal_lpdf}
\alias{lognormal_lpdf}
\title{Lognormal log-probability density function}
\usage{
lognormal_lpdf(x, meanlog, sdlog, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{meanlog}{Mean of the distribution on the log scale.}

\item{sdlog}{Standard deviation of the distribution on the log scale.}
}
\value{
The sum of the log-density.
}
\description{
Lognormal log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{quad_form_chol}
\alias{quad_form_chol}
\title{Quadratic form using a Cholesky factor}
\usage{
quad_form_chol(x, L)
}
\arguments{
\item{x}{A numeric vector.}

\item{L}{A lower triangular Cholesky factor matrix.}
}
\value{
The value of the quadratic form.
}
\description{
Quadratic form using a Cholesky factor
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{categorical_logit_lpmf}
\alias{categorical_logit_lpmf}
\title{Categorical log-probability mass function with logit parameterization}
\usage{
categorical_logit_lpmf(x, eta, sum = TRUE)
}
\arguments{
\item{x}{Integer or integer vector. The observed category index.}

\item{eta}{A numeric vector of linear predictors (unnormalized log-probabilities).}
}
\value{
The sum of the log-probability.
}
\description{
Categorical log-probability mass function with logit parameterization
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{categorical_lpmf}
\alias{categorical_lpmf}
\title{Categorical log-probability mass function}
\usage{
categorical_lpmf(x, prob, sum = TRUE)
}
\arguments{
\item{x}{Vector of categorical outcomes.}

\item{prob}{Vector or matrix of probabilities.}
}
\value{
The sum of the log-probability.
}
\description{
Categorical log-probability mass function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{bayes_factor}
\alias{bayes_factor}
\title{Calculate Bayes Factor}
\usage{
bayes_factor(logml1, logml2, error_threshold = 0.2)
}
\arguments{
\item{logml1}{The first model fit (e.g., `mcmc_fit`) or its log-marginal likelihood.}

\item{logml2}{The second model fit (e.g., `mcmc_fit`) or its log-marginal likelihood.}

\item{error_threshold}{Threshold for warning about high bridge-sampling error (default 0.2).}
}
\value{
An object of class `bayes_factor` containing the Bayes factor, log Bayes factor, 
  approximate estimation error, and interpretation.
}
\description{
Compare two models by calculating the Bayes Factor based on their
marginal likelihoods.
}
\examples{
\donttest{
  # Compare two models using Bayes Factor
  data(debate, package = "BayesRTMB")
  fit1 <- rtmb_lm(sat ~ talk, data = debate)
  fit2 <- rtmb_lm(sat ~ talk + perf, data = debate)
  map1 <- fit1$optimize()
  map2 <- fit2$optimize()
  bf <- bayes_factor(map1, map2)
  print(bf)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{print.ce_simple}
\alias{print.ce_simple}
\title{Print simple effects}
\usage{
\method{print}{ce_simple}(x, digits = 3, ...)
}
\arguments{
\item{x}{A `ce_simple` object.}

\item{digits}{Number of digits to print.}

\item{...}{Additional arguments.}
}
\description{
Print simple effects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_glmer.R
\name{make_glmer_re_terms}
\alias{make_glmer_re_terms}
\title{Prepare GLMM Formula Components}
\usage{
make_glmer_re_terms(
  formula,
  data,
  family = "gaussian",
  resid_group = NULL,
  resid_time = NULL,
  within = NULL,
  factors = NULL,
  missing = "listwise"
)
}
\arguments{
\item{formula}{lme4-style formula.}

\item{data}{Data frame.}

\item{family}{Character string of the distribution family.}

\item{resid_group}{Optional residual-correlation grouping variable.}

\item{resid_time}{Optional residual-correlation time variable.}

\item{within}{Optional list for wide-to-long conversion when the response is
written as `cbind(...)`.}

\item{factors}{Optional character vector of variables to treat as factors.}

\item{missing}{Missing value handling strategy: "listwise".}
}
\value{
A list containing `Y`, `X`, `trials`, `offset`, `N`, fixed-effect
metadata, and random-effect terms.
}
\description{
Builds the response, fixed-effect model matrix, and lme4-style random-effect
design components used by `rtmb_glmer()`. This is the user-facing helper that
reproduces what the wrapper places in the generated `setup` block.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_test_info}
\alias{plot_test_info}
\title{Plot test information function}
\usage{
plot_test_info(fit, ...)
}
\arguments{
\item{fit}{A fitted IRT model object.}

\item{...}{Additional arguments passed to \code{\link{test_info}} or the plot method.}
}
\value{
Invisibly returns the plotted object of class \code{rtmb_test_info}.
}
\description{
Calculate and plot test information function for a fitted IRT model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RTMB_Model.R
\name{RTMB_Model-class}
\alias{RTMB_Model-class}
\alias{RTMB_Model}
\title{RTMB model object}
\description{
An R6 class representing a Bayesian model built with RTMB.
This class stores model components and provides methods for
building the automatic differentiation object, optimizing the
posterior, and drawing posterior samples.
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{data}}{Named list containing observation data.}

\item{\code{par_list}}{Evaluated list of parameter definitions.}

\item{\code{log_prob}}{Executable function for likelihood and prior calculations.}

\item{\code{transform}}{Function for calculating transformed parameters.}

\item{\code{generate}}{Function for calculating generated quantities.}

\item{\code{par_names}}{List of variable names corresponding to parameter dimensions.}

\item{\code{pl_full}}{Complete parameter list with expanded constraints and dimension information used internally.}

\item{\code{formula}}{Formula used for model generation (if created via a wrapper function).}

\item{\code{requested_contrasts}}{Character; user-requested contrast setting (e.g. "treatment").}

\item{\code{contrasts}}{Character; internal contrast setting used for fitting (e.g. "sum").}

\item{\code{raw_data}}{Data frame; the original data used for the model.}

\item{\code{family}}{Character string of the distribution family (if created via a wrapper function).}

\item{\code{init}}{List or numeric vector of initial values.}

\item{\code{view}}{Character vector of variable names to prioritize in summary.}

\item{\code{code}}{Original model AST generated by `rtmb_code()`.}

\item{\code{map}}{A list specifying parameter mappings for fixing values (e.g., via \code{fixed_model()}).}

\item{\code{prior_correction}}{Numeric value for legacy or low-level marginal-likelihood correction.
Fixed-parameter prior removal is handled during AD objective construction.}

\item{\code{fixed_prior_specs}}{Internal specification for removing fixed-parameter prior contributions during AD evaluation.}

\item{\code{type}}{Character; the type of the model (e.g. 'lmer', 'glm', 'table', 'ttest', 'corr', 'mediation').}

\item{\code{extra}}{List; used to store auxiliary data (like term assignments, factors, test results) needed by methods such as classic().}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-RTMB_Model-new}{\code{RTMB_Model$new()}}
\item \href{#method-RTMB_Model-prepare_init}{\code{RTMB_Model$prepare_init()}}
\item \href{#method-RTMB_Model-get_par_list}{\code{RTMB_Model$get_par_list()}}
\item \href{#method-RTMB_Model-get_ad_obj}{\code{RTMB_Model$get_ad_obj()}}
\item \href{#method-RTMB_Model-to_unconstrained}{\code{RTMB_Model$to_unconstrained()}}
\item \href{#method-RTMB_Model-to_constrained}{\code{RTMB_Model$to_constrained()}}
\item \href{#method-RTMB_Model-unconstrained_vector_to_list}{\code{RTMB_Model$unconstrained_vector_to_list()}}
\item \href{#method-RTMB_Model-constrained_vector_to_list}{\code{RTMB_Model$constrained_vector_to_list()}}
\item \href{#method-RTMB_Model-calculate_satterthwaite_df}{\code{RTMB_Model$calculate_satterthwaite_df()}}
\item \href{#method-RTMB_Model-calculate_reml_satterthwaite_df}{\code{RTMB_Model$calculate_reml_satterthwaite_df()}}
\item \href{#method-RTMB_Model-build_ad_obj}{\code{RTMB_Model$build_ad_obj()}}
\item \href{#method-RTMB_Model-optimize}{\code{RTMB_Model$optimize()}}
\item \href{#method-RTMB_Model-classic}{\code{RTMB_Model$classic()}}
\item \href{#method-RTMB_Model-sample}{\code{RTMB_Model$sample()}}
\item \href{#method-RTMB_Model-variational}{\code{RTMB_Model$variational()}}
\item \href{#method-RTMB_Model-print_code}{\code{RTMB_Model$print_code()}}
\item \href{#method-RTMB_Model-print_log_prob}{\code{RTMB_Model$print_log_prob()}}
\item \href{#method-RTMB_Model-print_transform}{\code{RTMB_Model$print_transform()}}
\item \href{#method-RTMB_Model-print_generate}{\code{RTMB_Model$print_generate()}}
\item \href{#method-RTMB_Model-get_n_obs}{\code{RTMB_Model$get_n_obs()}}
\item \href{#method-RTMB_Model-fixed_model}{\code{RTMB_Model$fixed_model()}}
\item \href{#method-RTMB_Model-clone}{\code{RTMB_Model$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-new"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-new}{}}}
\subsection{Method \code{new()}}{
Create a new `RTMB_Model` object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$new(
  data,
  par_list,
  log_prob,
  transform = NULL,
  generate = NULL,
  par_names = NULL,
  init = NULL,
  view = NULL,
  code = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{data}}{Named list containing observation data.}

\item{\code{par_list}}{Evaluated list of parameter definitions.}

\item{\code{log_prob}}{Executable function for likelihood and prior calculations.}

\item{\code{transform}}{Function for calculating transformed parameters (optional).}

\item{\code{generate}}{Function for calculating generated quantities (optional).}

\item{\code{par_names}}{List of variable names corresponding to parameter dimensions.}

\item{\code{init}}{List or numeric vector of initial values.}

\item{\code{view}}{Character vector of variable names to prioritize in summary.}

\item{\code{code}}{Original model AST generated by `rtmb_code()`.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-prepare_init"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-prepare_init}{}}}
\subsection{Method \code{prepare_init()}}{
Prepare and format initial values for the model parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$prepare_init(init_arg)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{init_arg}}{Optional list or numeric vector of initial values. If NULL, defaults to `self$init` or random generation.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A flat numeric vector of constrained initial values.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-get_par_list"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-get_par_list}{}}}
\subsection{Method \code{get_par_list()}}{
Get the current parameters as a named list.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$get_par_list(init = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{init}}{Optional numeric vector or list of initial values. If NULL, uses the model's current initial values.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of constrained parameters.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-get_ad_obj"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-get_ad_obj}{}}}
\subsection{Method \code{get_ad_obj()}}{
Get the RTMB automatic differentiation object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$get_ad_obj(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Arguments passed to \code{build_ad_obj}.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A TMB objective object (ad_obj).
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-to_unconstrained"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-to_unconstrained}{}}}
\subsection{Method \code{to_unconstrained()}}{
Convert a list of constrained parameters to unconstrained space.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$to_unconstrained(par_list_constrained)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{par_list_constrained}}{A named list of parameters in their natural space.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of parameters in unconstrained space.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-to_constrained"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-to_constrained}{}}}
\subsection{Method \code{to_constrained()}}{
Convert a list of unconstrained parameters to constrained (natural) space.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$to_constrained(par_list_unconstrained)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{par_list_unconstrained}}{A named list of parameters in unconstrained space.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of parameters in constrained space.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-unconstrained_vector_to_list"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-unconstrained_vector_to_list}{}}}
\subsection{Method \code{unconstrained_vector_to_list()}}{
Convert a flat unconstrained vector to a named list.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$unconstrained_vector_to_list(vec)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vec}}{A numeric vector in unconstrained space.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of unconstrained parameters.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-constrained_vector_to_list"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-constrained_vector_to_list}{}}}
\subsection{Method \code{constrained_vector_to_list()}}{
Convert a flat constrained vector to a named list.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$constrained_vector_to_list(vec)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vec}}{A numeric vector in constrained space.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of constrained parameters.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-calculate_satterthwaite_df"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-calculate_satterthwaite_df}{}}}
\subsection{Method \code{calculate_satterthwaite_df()}}{
Calculate Satterthwaite degrees of freedom for parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$calculate_satterthwaite_df(
  ad_obj,
  idx_fix_active = NULL,
  L_u_total = NULL,
  opt_par = NULL,
  max_df = NULL,
  silent = FALSE,
  return_sensitivities = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ad_obj}}{An RTMB objective object at the optimum.}

\item{\code{idx_fix_active}}{Integer vector; indices of active fixed parameters in the full unconstrained vector.}

\item{\code{L_u_total}}{Integer; total length of the full unconstrained parameter vector.}

\item{\code{opt_par}}{Optional numeric vector of optimized parameters.}

\item{\code{max_df}}{Numeric; maximum allowed degrees of freedom. Default is NULL.}

\item{\code{silent}}{Logical; whether to suppress informational messages. Default is FALSE.}

\item{\code{return_sensitivities}}{Logical; if TRUE, returns Hessian sensitivities and V for delta method.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A numeric vector of estimated degrees of freedom (length = L_u_total). Inf for random effects.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-calculate_reml_satterthwaite_df"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-calculate_reml_satterthwaite_df}{}}}
\subsection{Method \code{calculate_reml_satterthwaite_df()}}{
Calculate Satterthwaite degrees of freedom for integrated fixed effects (REML).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$calculate_reml_satterthwaite_df(
  ad_obj,
  opt_par,
  beta_idx,
  max_df = NULL,
  silent = FALSE,
  return_sensitivities = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ad_obj}}{An RTMB objective object.}

\item{\code{opt_par}}{Numeric vector of optimized variance components.}

\item{\code{beta_idx}}{Integer vector; indices of fixed effects within the random effects vector.}

\item{\code{max_df}}{Numeric; maximum allowed degrees of freedom. Default is NULL.}

\item{\code{silent}}{Logical; whether to suppress informational messages. Default is FALSE.}

\item{\code{return_sensitivities}}{Logical; if TRUE, returns Hessian sensitivities and delta-method components.}

\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A numeric vector of estimated degrees of freedom for the fixed effects.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-build_ad_obj"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-build_ad_obj}{}}}
\subsection{Method \code{build_ad_obj()}}{
Build the RTMB automatic differentiation object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$build_ad_obj(
  init = NULL,
  laplace = FALSE,
  jacobian_target = "none",
  map = NULL,
  .marginal_vars = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{init}}{Optional numeric vector or list of initial values for the parameters. Default is NULL.}

\item{\code{laplace}}{Logical; whether to use Laplace approximation to marginalize random effects. Default is FALSE.}

\item{\code{jacobian_target}}{Character string specifying which parameters to apply Jacobian adjustments to:
\itemize{
  \item \code{"all"}: Apply to all parameters (standard for Bayesian MCMC).
  \item \code{"random"}: Apply only to parameters with \code{random = TRUE} (standard for Laplace approximation/REML).
  \item \code{"none"}: No Jacobian adjustment (standard for Maximum Likelihood estimation).
}
Default is "none".}

\item{\code{map}}{Optional list specifying parameters to fix. Passed directly to MakeADFun. Default is NULL.}

\item{\code{.marginal_vars}}{Internal use for profiling marginalized parameters.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
An RTMB objective object (ad_obj).
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-optimize"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-optimize}{}}}
\subsection{Method \code{optimize()}}{
Perform Maximum Likelihood or Maximum A Posteriori (MAP) estimation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$optimize(
  laplace = TRUE,
  init = NULL,
  num_estimate = 1,
  control = list(),
  optimizer = "nlminb",
  method = "BFGS",
  map = NULL,
  fixed = NULL,
  se_method = c("wald", "sampling", "none"),
  num_samples = 1000,
  seed = 123,
  marginal = "none",
  df_method = "auto",
  view = NULL,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{laplace}}{Logical; whether to use Laplace approximation for random effects. Default is TRUE.}

\item{\code{init}}{Optional initial values for parameters (numeric vector or list).}

\item{\code{num_estimate}}{Integer; Number of multi-start optimization runs. Default is 1.}

\item{\code{control}}{A list of control settings passed to the optimizer.}

\item{\code{optimizer}}{Character; The optimizer to use: "nlminb" (default) or "optim".}

\item{\code{method}}{Character; The method for "optim" (e.g., "BFGS", "L-BFGS-B").}

\item{\code{map}}{Optional list specifying parameters to fix (factors).}

\item{\code{fixed}}{Optional list specifying parameter values to fix.}

\item{\code{se_method}}{Character; method for uncertainty estimation.
\itemize{
  \item \code{"wald"}: Wald standard errors and Wald confidence intervals.
  \item \code{"sampling"}: Simulation-based error propagation.
  \item \code{"none"}: Do not compute standard errors, confidence intervals, or degrees of freedom.
}}

\item{\code{num_samples}}{Integer; number of samples to draw when se_method is "sampling". Default is 1000.}

\item{\code{seed}}{Integer; random seed for sampling.}

\item{\code{marginal}}{Character vector or "auto" or "none"; specifies which parameters to marginalize out via Laplace approximation.
\itemize{
  \item \code{"none"} (default): No additional marginalization.
  \item \code{"auto"}: Uses variables specified by the model wrapper (e.g., fixed effects in lmer).
  \item \code{"all"}: Marginalize all parameters (advanced use).
  \item \code{character vector}: Specific parameter names to marginalize.
}}

\item{\code{df_method}}{Character or numeric; method for finite degrees-of-freedom approximation.
Used only when `marginal` resolves to one or more parameters and `se_method != "none"`.
\itemize{
  \item \code{"auto"}: Uses Satterthwaite approximation when marginalization is active.
  \item \code{"satterthwaite"}: Satterthwaite approximation.
  \item \code{"inf"} or \code{"none"}: Use infinite degrees of freedom.
  \item numeric: Use the supplied fixed degrees of freedom.
}}

\item{\code{view}}{Character vector of parameter names to prioritize in summary display.}

\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A fitted `MAP_Fit` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-classic"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-classic}{}}}
\subsection{Method \code{classic()}}{
Perform frequentist inference (REML/ML) with model-appropriate degrees of freedom.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$classic(
  df_method = "auto",
  se_method = c("wald", "sampling"),
  num_samples = 1000,
  seed = 123,
  view = NULL,
  map = NULL,
  fixed = NULL,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{df_method}}{Character; Method for calculating degrees of freedom. Default is "auto".
\itemize{
  \item \code{"auto"}: Determined based on model type (e.g., Satterthwaite for mixed models).
  \item \code{"residual"}: Residual degrees of freedom.
  \item \code{"satterthwaite"}: Satterthwaite approximation (robust for mixed models).
  \item \code{"inf"}: Use z-distribution (infinite degrees of freedom).
}}

\item{\code{se_method}}{Character; The method for CI estimation: "wald" (default) or "sampling".}

\item{\code{num_samples}}{Integer; number of samples to draw when se_method is "sampling". Default is 1000.}

\item{\code{seed}}{Integer; random seed for sampling.}

\item{\code{view}}{Character vector of parameters to prioritize in the summary output.}

\item{\code{map}}{Optional list specifying parameters to fix.}

\item{\code{fixed}}{Optional list specifying parameter values to fix.}

\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A `Classic_Fit` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-sample"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-sample}{}}}
\subsection{Method \code{sample()}}{
Draw posterior samples from the model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$sample(
  sampling = 1000,
  warmup = 1000,
  chains = 4,
  thin = 1,
  seed = sample.int(1e+06, 1),
  delta = 0.8,
  max_treedepth = 10,
  parallel = FALSE,
  laplace = FALSE,
  init = NULL,
  init_jitter = 0.1,
  save_csv = NULL,
  map = NULL,
  fixed = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{sampling}}{Number of sampling iterations. Default is 1000.}

\item{\code{warmup}}{Number of warmup iterations. Default is 1000.}

\item{\code{chains}}{Number of MCMC chains. Default is 4.}

\item{\code{thin}}{Thinning interval. Default is 1.}

\item{\code{seed}}{Random seed.}

\item{\code{delta}}{Target acceptance rate for HMC/NUTS. Default is 0.8.}

\item{\code{max_treedepth}}{Maximum tree depth for HMC/NUTS. Default is 10.}

\item{\code{parallel}}{Logical; whether to run chains in parallel. Default is FALSE.}

\item{\code{laplace}}{Logical; whether to use Laplace approximation. Default is FALSE.}

\item{\code{init}}{Optional initial values for parameters.}

\item{\code{init_jitter}}{sd of randomize initial values for parameters.}

\item{\code{save_csv}}{Optional list for saving MCMC results. e.g., list(name = "model", dir = "BayesRTMB_mcmc").}

\item{\code{map}}{Optional list specifying parameters to fix (factors).}

\item{\code{fixed}}{Optional list specifying parameter values to fix.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A fitted `MCMC_Fit` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-variational"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-variational}{}}}
\subsection{Method \code{variational()}}{
Run Automatic Differentiation Variational Inference (ADVI).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$variational(
  iter = 3000,
  tol_rel_obj = 0.005,
  window_size = 100,
  num_samples = 1000,
  num_estimate = 4,
  alpha = 0.01,
  laplace = FALSE,
  print_freq = 1000,
  method = c("meanfield", "fullrank", "hybrid"),
  parallel = FALSE,
  seed = sample.int(1e+06, 1),
  init = NULL,
  save_csv = NULL,
  map = NULL,
  fixed = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{iter}}{Integer; fixed number of iterations for the optimization. Default is 3000.}

\item{\code{tol_rel_obj}}{Numeric; relative tolerance for the ELBO change to determine convergence. Default is 0.005.}

\item{\code{window_size}}{Integer; window size for median smoothing in the convergence check. Default is 100.}

\item{\code{num_samples}}{Integer; number of posterior samples to generate from the fitted variational distribution. Default is 1000.}

\item{\code{num_estimate}}{Integer; number of times to run the VB estimation (treated as chains). Default is 4.}

\item{\code{alpha}}{Numeric; learning rate for the Adam optimizer. Default is 0.01.}

\item{\code{laplace}}{Logical; whether to use Laplace approximation to marginalize random effects. Default is FALSE.}

\item{\code{print_freq}}{Integer; iterations interval for progress output. Set to 0 to disable. Default is 1000.}

\item{\code{method}}{Character; method of Variational Inference. Default is "meanfield".}

\item{\code{parallel}}{Logical; whether to run estimations in parallel. Default is FALSE.}

\item{\code{seed}}{Integer; random seed for reproducibility.}

\item{\code{init}}{Optional numeric vector or list for initial parameter values. Default is NULL.}

\item{\code{save_csv}}{Optional list for saving VB results. e.g., list(name = "model", dir = "BayesRTMB_vb").}

\item{\code{map}}{Optional list specifying parameters to fix (factors).}

\item{\code{fixed}}{Optional list specifying parameter values to fix.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A fitted `VB_Fit` object containing posterior samples and diagnostic information.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-print_code"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-print_code}{}}}
\subsection{Method \code{print_code()}}{
Print model code or model structure.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$print_code()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-print_log_prob"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-print_log_prob}{}}}
\subsection{Method \code{print_log_prob()}}{
Print the internal log-probability function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$print_log_prob()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-print_transform"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-print_transform}{}}}
\subsection{Method \code{print_transform()}}{
Print the internal transformation function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$print_transform()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-print_generate"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-print_generate}{}}}
\subsection{Method \code{print_generate()}}{
Print the internal generation function.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$print_generate()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-get_n_obs"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-get_n_obs}{}}}
\subsection{Method \code{get_n_obs()}}{
Automatically detect and count the number of independent data points (observations).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$get_n_obs()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Integer; total number of observations, or NULL if model code is missing.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-fixed_model"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-fixed_model}{}}}
\subsection{Method \code{fixed_model()}}{
Create a model with fixed parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$fixed_model(fixed = list(), silent = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{fixed}}{A named list of parameter values to fix.}

\item{\code{silent}}{Logical; whether to suppress informational messages. Default is FALSE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new RTMB_Model object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-RTMB_Model-clone"></a>}}
\if{latex}{\out{\hypertarget{method-RTMB_Model-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{RTMB_Model$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{parameters_code}
\alias{parameters_code}
\title{Code block for parameter definitions}
\usage{
parameters_code(expr)
}
\arguments{
\item{expr}{A parameter definition expression enclosed in `{}`.}
}
\value{
A lazily evaluated list of parameters.
}
\description{
Defines the list of parameters to be passed to `rtmb_model`.
By writing in the format `variable_name = Dim(...)` within a block enclosed in `{}`,
evaluation is delayed, enabling strict error checking during model construction.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{to_centered_tri}
\alias{to_centered_tri}
\title{Vector to centered triangular matrix (RTMB compatible)}
\usage{
to_centered_tri(x, R, C)
}
\arguments{
\item{x}{A numeric vector of appropriate length.}

\item{R}{Number of rows.}

\item{C}{Number of columns.}
}
\value{
An R x C matrix with column-wise sum-to-zero constraints on lower elements.
}
\description{
Vector to centered triangular matrix (RTMB compatible)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{softmax}
\alias{softmax}
\title{Softmax function}
\usage{
softmax(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
A numeric vector of softmax probabilities.
}
\description{
Softmax function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{inject_namespace}
\alias{inject_namespace}
\title{Internal function to search AST and inject namespace to package functions}
\usage{
inject_namespace(expr, pkg = "BayesRTMB")
}
\arguments{
\item{expr}{Unevaluated code (AST)}

\item{pkg}{Target package name}
}
\description{
Internal function to search AST and inject namespace to package functions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classic.R
\name{lsmeans}
\alias{lsmeans}
\title{Least Squares Means (Marginal Means)}
\usage{
lsmeans(object, specs, ...)
}
\arguments{
\item{object}{A fitted model object (e.g., `Classic_Fit`).}

\item{specs}{A character vector of factor names to calculate means for.}

\item{...}{Additional arguments passed to the method.}
}
\description{
Calculate least squares means (also known as marginal means or predicted means)
for categorical factors in a fitted model. It also supports pairwise comparisons
and simple main effects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{print.bayes_factor_rtmb}
\alias{print.bayes_factor_rtmb}
\title{Print method for bayes_factor_rtmb objects}
\usage{
\method{print}{bayes_factor_rtmb}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class bayes_factor_rtmb.}

\item{digits}{Number of decimal places to display (default is 4).}

\item{...}{Additional arguments.}
}
\description{
Print method for bayes_factor_rtmb objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_mdu.R
\name{make_ydif_from_bw}
\alias{make_ydif_from_bw}
\title{Make Best-Worst Pair Indices from Best and Worst Responses}
\usage{
make_ydif_from_bw(Best, Worst, sets)
}
\arguments{
\item{Best}{Matrix or data frame of best responses (N persons x P tasks).
Values must be positions within the corresponding row of `sets`, from
`1` to `ncol(sets)`.}

\item{Worst}{Matrix or data frame of worst responses (N persons x P tasks).
Values must be positions within the corresponding row of `sets`, from
`1` to `ncol(sets)`.}

\item{sets}{Matrix or data frame of presented item sets (P tasks x C items).}
}
\value{
An integer matrix of pair indices (`Y_dif`).
}
\description{
Converts separate `Best` and `Worst` response matrices into `Y_dif` pair
indices for Best-Worst MDU models. The returned index is the position of the
ordered `(best, worst)` pair among all `C * (C - 1)` position pairs generated
from each row of `sets`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{simple_effects}
\alias{simple_effects}
\title{Calculate Simple Effects}
\usage{
simple_effects(fit, effect, prob = 0.95, sd_multiplier = 1, ...)
}
\arguments{
\item{fit}{Model fit object (e.g., `map_fit`, `mcmc_fit`).}

\item{effect}{Character string of the interaction (e.g., "A:B"). The first variable is the focal variable.}

\item{prob}{Probability for the credible/confidence interval (default is 0.95).}

\item{sd_multiplier}{Multiplier for SD for continuous moderators (default is 1).}

\item{...}{Additional arguments.}
}
\value{
A `ce_simple` object (data frame) containing the estimated effects and their credible intervals.
}
\description{
Calculate the effect of a focal variable at different levels of a moderator.
For categorical focal variables, it calculates pairwise differences (contrasts).
For continuous focal variables, it calculates the slope (simple slopes).
}
\examples{
\donttest{
  data(debate, package = "BayesRTMB")
  fit <- rtmb_lm(sat ~ talk * perf, data = debate)
  map_fit <- fit$optimize()
  # Effect of talk at different levels of performance
  se <- simple_effects(map_fit, effect = "talk:perf")
  print(se)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods_post_estimation.R
\name{conditional_effects}
\alias{conditional_effects}
\title{Calculate Conditional Effects}
\usage{
conditional_effects(fit, effect, prob = 0.95, sd_multiplier = 1, ...)
}
\arguments{
\item{fit}{Model fit object (e.g., `mcmc_fit`, `map_fit`).}

\item{effect}{Name of the variable to visualize (e.g., "X1" or "X1:X2").}

\item{prob}{Probability for the credible/confidence interval (default is 0.95).}

\item{sd_multiplier}{Numeric. Multiplier for standard deviation when splitting continuous moderators (default is 1).}

\item{...}{Additional arguments.}
}
\value{
An object of class `ce_rtmb` containing the predicted values and their credible intervals.
}
\description{
Calculate and visualize the predicted values (marginal effects) of a variable,
potentially conditional on the levels of another variable (interaction).
}
\examples{
\donttest{
  data(debate, package = "BayesRTMB")
  fit <- rtmb_lm(sat ~ talk * perf, data = debate)
  map_fit <- fit$optimize()
  ce <- conditional_effects(map_fit, effect = "talk:perf")
  plot(ce)
  summary(ce)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmc_index.R
\name{map_est}
\alias{map_est}
\title{Maximum A Posteriori (MAP) Estimate}
\usage{
map_est(z)
}
\arguments{
\item{z}{A numeric vector of samples.}
}
\value{
A numeric value.
}
\description{
Maximum A Posteriori (MAP) Estimate
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_pairs}
\alias{plot_pairs}
\title{Plot pairs for posterior samples}
\usage{
plot_pairs(x, pars = NULL)
}
\arguments{
\item{x}{A 3D array of posterior samples with dimensions `(iterations, chains, variables)`.}

\item{pars}{Character vector or integer vector specifying which parameters to plot.
If NULL (default), up to the first 10 parameters are plotted to prevent overplotting.}
}
\value{
No return value.
}
\description{
Draw a scatterplot matrix to examine posterior correlations between parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_mediation.R
\name{rtmb_mediation}
\alias{rtmb_mediation}
\title{RTMB-based Mediation Analysis Wrapper}
\usage{
rtmb_mediation(
  formula,
  data,
  family = "gaussian",
  prior = prior_flat(),
  y_range = NULL,
  fixed = NULL,
  view = NULL,
  WAIC = FALSE,
  ...
)
}
\arguments{
\item{formula}{A list of formulas defining the regression paths (e.g., `list(M ~ X, Y ~ X + M)`).}

\item{data}{A data frame containing the variables.}

\item{family}{A single character string or a list of character strings specifying the error distribution
for each equation (e.g., `family = list("gaussian", "binomial")`). Default is "gaussian".}

\item{prior}{An object of class "rtmb_prior" specifying the prior distribution.}

\item{y_range}{Theoretical minimum and maximum values of the response variable.}

\item{fixed}{A named list of parameter values to fix (optional).}

\item{view}{Character vector of parameter names to prioritize in summary.}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}

\item{...}{Additional arguments passed to the model construction.}
}
\value{
An `RTMB_Model` object.
}
\description{
`rtmb_mediation` performs mediation analysis by simultaneously estimating multiple
GLM regression equations. It automatically identifies mediation paths and calculates
indirect, direct, and total effects.
}
\details{
The function identifies mediation paths by looking for variables that are
responses in one equation and predictors in another. Indirect effects are
calculated as the product of coefficients along these paths (\eqn{a * b}).

\strong{Uncertainty Estimation}:
When using `$optimize(ci_method = "sampling")`, the function provides asymmetric
confidence intervals for indirect effects based on the distribution of products,
which is more accurate than the standard Sobel test (Delta Method).
}
\examples{

  # Simulate mediation data
  set.seed(123)
  N <- 100
  X <- rnorm(N)
  # M is influenced by X
  M <- 0.5 * X + rnorm(N, 0, 0.5)
  # Y is influenced by both X and M
  Y <- 0.3 * X + 0.8 * M + rnorm(N, 0, 0.5)
  dat <- data.frame(X, M, Y)

  # Fit a mediation model
  # The formula list specifies the two regression equations
  fit_med <- rtmb_mediation(list(M ~ X, Y ~ X + M), data = dat)
  
  # Maximum A Posteriori (MAP) estimation
  map_med <- fit_med$optimize()
  # The summary automatically calculates indirect, direct, and total effects
  map_med$summary()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_mdu.R
\name{rtmb_mdu}
\alias{rtmb_mdu}
\title{RTMB-based Multidimensional Unfolding Wrapper}
\usage{
rtmb_mdu(
  data,
  ndim = 2,
  distance = c("squared", "euclidean"),
  alpha = c("random", "fix"),
  method = c("rating", "Best", "Best-Worst", "MDS"),
  sets = NULL,
  prior = prior_flat(),
  y_range = NULL,
  init = NULL,
  fixed = NULL,
  view = NULL,
  distance_eps = 1e-04,
  missing = c("listwise", "fiml"),
  WAIC = FALSE
)
}
\arguments{
\item{data}{Numeric matrix or data frame (N rows x M items) for
`method = "rating"`. For choice methods, a list containing `Best` and,
for `method = "Best-Worst"`, `Worst`. For `method = "Best-Worst"`, a
single matrix/data frame is treated as pre-coded `Y_dif` pair indices.}

\item{ndim}{Number of unfolding dimensions.}

\item{distance}{Character; `"squared"` uses squared Euclidean distance
(default, often easier for optimization), while `"euclidean"` uses Euclidean
distance.}

\item{alpha}{Character; `"random"` estimates item-specific alpha values as
random effects (default), while `"fix"` estimates a single common alpha.}

\item{method}{Character; `"rating"` for continuous ratings, `"Best"` for
best-only choice tasks, `"Best-Worst"` for best-worst choice tasks, or
`"MDS"` for fitting a multidimensional scaling model to a distance matrix.}

\item{sets}{Matrix or data frame of presented item sets (P tasks x C items)
for choice methods.}

\item{prior}{Prior configuration: `prior_flat()`, `prior_normal()`, or
`prior_weak()`. `prior_flat()` creates a maximum-likelihood model suitable
for `classic()`. The latent coordinates `delta` and `theta` are always
treated as random effects with normal scale priors, similarly to IRT ability
parameters.}

\item{y_range}{Optional response range for `method = "rating"`. If supplied
with the default flat prior, `prior_weak()` is used. For choice methods
(`"Best"` and `"Best-Worst"`), `prior_weak()` behaves like
`prior_normal()` because there is no observed rating scale.}

\item{init}{Optional named list of initial values.}

\item{fixed}{Optional named list of parameter values to fix.}

\item{view}{Character vector of parameter names to prioritize in summaries.}

\item{distance_eps}{Small positive constant added to the distance.}

\item{missing}{Missing value handling strategy: "listwise" (default) or "fiml" (Full Information Maximum Likelihood).}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}
}
\value{
An `RTMB_Model` object.
}
\description{
Fits a multidimensional unfolding model for preference/rating data. Rows are
persons or observations and columns are stimuli/items. The model represents
both row scores (`theta`) and item locations (`delta`) in a shared D-dimensional
space.
}
\examples{

  # Simulate rating data for Multidimensional Unfolding (MDU)
  set.seed(123)
  N <- 50  # Number of persons
  M <- 10  # Number of items
  D <- 2   # Number of dimensions
  
  # True person and item coordinates in a 2D space
  theta <- matrix(rnorm(N * D), N, D)
  delta <- matrix(rnorm(M * D), M, D)
  
  # Generate distance-like ratings (smaller means more preferred)
  Y <- matrix(NA, N, M)
  for(i in 1:N) {
    for(j in 1:M) {
      Y[i, j] <- sum((theta[i,] - delta[j,])^2) + rnorm(1, 0, 0.5)
    }
  }

  # Fit a 2-dimensional MDU model
  fit_mdu <- rtmb_mdu(Y, ndim = 2, distance = "squared", method = "rating")
  
  # MAP estimation
  map_mdu <- fit_mdu$optimize()
  map_mdu$summary()
  
  # Note: MDU models have many parameters, so MCMC sampling might take time.
  \donttest{
  # mcmc_mdu <- fit_mdu$sample(sampling = 500, warmup = 500, chains = 2)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesRTMB_package.R
\docType{package}
\name{BayesRTMB-package}
\alias{BayesRTMB}
\alias{BayesRTMB-package}
\title{BayesRTMB: Bayesian Modeling with RTMB}
\description{
The `BayesRTMB` package provides an intuitive, R-based Bayesian modeling
environment using the `RTMB` package as its computational backend.
It allows users to perform Maximum A Posteriori (MAP) estimation,
Markov Chain Monte Carlo (MCMC) sampling, and Automatic Differentiation
Variational Inference (ADVI) for hierarchical and complex non-linear models
through a unified and user-friendly interface.
}
\details{
The standard workflow of the package consists of three main steps:

1. **Model Definition**: Use [rtmb_code()] to define the model structure
   (likelihood, prior distributions, transformations, etc.) using standard R syntax.
2. **Model Instantiation**: Pass the data and the generated model code to
   [rtmb_model()] to create an `RTMB_Model` R6 object.
3. **Estimation**: Call the appropriate method on the model object:
   * `$optimize()`: For MAP estimation and Laplace approximation.
   * `$sample()`: For MCMC sampling using the No-U-Turn Sampler (NUTS).
   * `$variational()`: For approximate Bayesian inference using ADVI.

For more detailed examples and tutorials, please refer to the package vignettes.
}
\seealso{
Useful links:
* [https://github.com/norimune/BayesRTMB](https://github.com/norimune/BayesRTMB)
* Report bugs at [https://github.com/norimune/BayesRTMB/issues](https://github.com/norimune/BayesRTMB/issues)
}
\author{
\strong{Maintainer}: Hiroshi Shimizu \email{simizu706@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{debate}
\alias{debate}
\title{Debate Simulation Data}
\format{
A data frame with 300 rows and 6 columns:
\describe{
  \item{group}{Group identifier. Each group consists of 3 members.}
  \item{sat}{Self-reported satisfaction level of the individual after the debate.}
  \item{talk}{Amount of talk or utterances by the individual during the debate.}
  \item{perf}{Evaluated performance of the group debate (group-level variable).}
  \item{skill}{Evaluated conversation skill of the individual.}
  \item{cond}{Experimental condition indicator (e.g., 0 = control, 1 = treatment).}
}
}
\source{
Simulated dummy data created by the package author.
}
\usage{
debate
}
\description{
A simulated dataset containing individual satisfaction scores and related
variables from a 3-person group debate experiment. This dummy data was
created by the package author to demonstrate hierarchical (multilevel) modeling.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcmc_index.R
\name{ess_basic}
\alias{ess_basic}
\title{Basic Effective Sample Size for a single chain or pooled chains}
\usage{
ess_basic(sims)
}
\arguments{
\item{sims}{A numeric vector or matrix of samples.}
}
\value{
A numeric value.
}
\description{
Basic Effective Sample Size for a single chain or pooled chains
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_doc.R
\name{rtmb_wrappers}
\alias{rtmb_wrappers}
\title{Common Features and Arguments of RTMB Wrapper Functions}
\description{
The RTMB wrapper functions (`rtmb_lm`, `rtmb_glm`, `rtmb_glmer`, `rtmb_fa`, etc.)
share a unified interface designed to make Bayesian and Frequentist inference
accessible through familiar R formulas and standard model specifications.
}
\details{
All wrapper functions in this package are built upon the same core engine.
This ensures that regardless of the model type, the workflow for estimation,
summary, and expansion remains consistent.

\strong{1. Unified Inference Methods:}
Every model object returned by a wrapper function provides the following methods:
\itemize{
  \item \code{$classic()}: Performs standard frequentist estimation (MLE/REML) using `prior_flat()`. Supports robust standard errors and Satterthwaite/Between-Within degrees of freedom approximations.
  \item \code{$optimize()}: Performs Maximum A Posteriori (MAP) estimation (Empirical Bayes).
  \item \code{$sample()}: Performs MCMC sampling (NUTS via tmbstan) for full Bayesian inference.
  \item \code{$variational()}: Performs Variational Inference (ADVI) for fast posterior approximation.
}

\strong{2. Prior API and Regularization:}
You can specify the prior distribution using the `prior` argument:
\itemize{
  \item \code{prior_flat()}: No additional prior (default). Required for \code{classic()} inference.
  \item \code{prior_normal()}: Manual normal/exponential priors for parameters.
  \item \code{prior_weak()}: Weakly informative priors based on \code{y_range}.
  \item \code{prior_rhs()}: Regularized Horseshoe prior for continuous shrinkage.
  \item \code{prior_ssp()}: Spike-and-Slab prior for sparse variable selection.
}
\emph{Note: When using `prior_weak()`, `prior_rhs()`, or `prior_ssp()`, you must specify \code{y_range = c(min, max)}
to let the model set appropriate global scales for the priors.}

\strong{3. Model Comparison and Bayes Factors:}
Model comparison is performed exclusively via Bridge Sampling. To compare a full model 
against a nested null model, use the \code{$bayes_factor(fixed = list(...))} method on an 
MCMC fit object. For example, \code{fit$bayes_factor(fixed = list(b = 0))} tests the 
hypothesis that the fixed effect \code{b} is exactly zero.

\strong{4. Standard Errors and Degrees of Freedom:}
The \code{$classic()} and \code{$optimize()} methods provide advanced summary options:
\itemize{
  \item \code{se_method}: Calculate `"robust"` (Huber-White) or `"cluster-robust"` standard errors by providing a cluster variable.
  \item \code{df_method}: Use `"satterthwaite"` for mixed models or `"bw"` (Between-Within) for multilevel correlation models to obtain accurate p-values and confidence intervals.
}

\strong{5. Fixed parameters:}
Use \code{fixed = list(...)} during model construction to fix model parameters to specified values.
For example, \code{fixed = list(delta = 0)} fixes the parameter \code{delta} to zero.
This same syntax is used in \code{$bayes_factor()} to define the restricted model.
}
\concept{wrappers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{log_sum_exp_matrix}
\alias{log_sum_exp_matrix}
\title{Log-sum-exp function for matrices (row-wise)}
\usage{
log_sum_exp_matrix(M)
}
\arguments{
\item{M}{A numeric matrix or advector matrix.}
}
\value{
A numeric vector of row-wise log-sum-exp.
}
\description{
Log-sum-exp function for matrices (row-wise)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VB_Fit.R
\name{VB_Fit}
\alias{VB_Fit}
\title{VB fit object}
\description{
VB fit object

VB fit object
}
\details{
An R6 class storing posterior samples and related information
from Automatic Differentiation Variational Inference (ADVI).
}
\section{Super class}{
\code{\link[BayesRTMB:RTMB_Fit_Base]{BayesRTMB::RTMB_Fit_Base}} -> \code{advi_fit}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{model}}{An `RTMB_Model` object used for estimation.}

\item{\code{fit}}{A 3D array of posterior draws for fixed model parameters.}

\item{\code{random_fit}}{A 3D array of posterior draws for random effects, if available.}

\item{\code{transform_fit}}{A 3D array of posterior draws for transformed parameters, if available.}

\item{\code{generate_fit}}{A 3D array of posterior draws for generated quantities, if available.}

\item{\code{transform_dims}}{A list storing dimension information for transformed parameters.}

\item{\code{generate_dims}}{A list storing dimension information for generated quantities.}

\item{\code{elbo_history}}{A list of numeric vectors storing the Evidence Lower Bound (ELBO) history during optimization for each chain.}

\item{\code{laplace}}{Logical; whether Laplace approximation was used to marginalize random effects.}

\item{\code{posterior_mean}}{A named numeric vector of posterior mean estimates.}

\item{\code{ELBO}}{A numeric vector of final ELBO values for each chain.}

\item{\code{rel_obj_vals}}{A numeric vector of final relative objective tolerance values for each chain.}

\item{\code{best_chain}}{Integer; the index of the chain with the maximum ELBO.}

\item{\code{mu_history}}{Matrix of the parameter trajectory from the final window.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-advi_fit-get_point_estimate}{\code{VB_Fit$get_point_estimate()}}
\item \href{#method-advi_fit-new}{\code{VB_Fit$new()}}
\item \href{#method-advi_fit-print}{\code{VB_Fit$print()}}
\item \href{#method-advi_fit-draws}{\code{VB_Fit$draws()}}
\item \href{#method-advi_fit-summary}{\code{VB_Fit$summary()}}
\item \href{#method-advi_fit-plot_elbo}{\code{VB_Fit$plot_elbo()}}
\item \href{#method-advi_fit-plot_trajectory}{\code{VB_Fit$plot_trajectory()}}
\item \href{#method-advi_fit-transformed_draws}{\code{VB_Fit$transformed_draws()}}
\item \href{#method-advi_fit-generated_quantities}{\code{VB_Fit$generated_quantities()}}
\item \href{#method-advi_fit-WAIC}{\code{VB_Fit$WAIC()}}
\item \href{#method-advi_fit-diagnose}{\code{VB_Fit$diagnose()}}
\item \href{#method-advi_fit-clone}{\code{VB_Fit$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="EAP"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-EAP'><code>BayesRTMB::RTMB_Fit_Base$EAP()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="MAP"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-MAP'><code>BayesRTMB::RTMB_Fit_Base$MAP()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="estimate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-estimate'><code>BayesRTMB::RTMB_Fit_Base$estimate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="fa_rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-fa_rotate'><code>BayesRTMB::RTMB_Fit_Base$fa_rotate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-rotate'><code>BayesRTMB::RTMB_Fit_Base$rotate()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-get_point_estimate"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-get_point_estimate}{}}}
\subsection{Method \code{get_point_estimate()}}{
Get point estimate for a target parameter.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$get_point_estimate(target, chains = NULL, best_chains = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Target parameter name.}

\item{\code{chains}}{Numeric vector of chains to include. If NULL, all chains are used.}

\item{\code{best_chains}}{Integer; number of best chains to retain based on ELBO.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Matrix, array, vector, or scalar point estimate.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-new"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-new}{}}}
\subsection{Method \code{new()}}{
Create a new `VB_Fit` object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$new(
  model,
  fit,
  random_fit,
  elbo_history,
  laplace,
  posterior_mean,
  ELBO,
  rel_obj_vals,
  best_chain,
  mu_history
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{model}}{An `RTMB_Model` object.}

\item{\code{fit}}{A 3D array of parameter draws.}

\item{\code{random_fit}}{A 3D array of random effect draws.}

\item{\code{elbo_history}}{A list of numeric vectors of ELBO values for each chain.}

\item{\code{laplace}}{Logical; indicates if Laplace approximation was used.}

\item{\code{posterior_mean}}{A named numeric vector of posterior means.}

\item{\code{ELBO}}{A numeric vector of final ELBO values for each chain.}

\item{\code{rel_obj_vals}}{A numeric vector of final relative objective tolerance values for each chain.}

\item{\code{best_chain}}{Integer; the index of the chain with the maximum ELBO.}

\item{\code{mu_history}}{Matrix of the parameter trajectory from the final window.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-print"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-print}{}}}
\subsection{Method \code{print()}}{
Print a brief summary of the fitted object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to the `summary` method.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-draws"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-draws}{}}}
\subsection{Method \code{draws()}}{
Extract posterior draws for selected parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$draws(
  pars = NULL,
  chains = NULL,
  best_chains = NULL,
  inc_random = FALSE,
  inc_transform = TRUE,
  inc_generate = TRUE,
  best_only = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character or numeric vector specifying the names or indices of parameters to extract. If NULL, all available parameters are extracted.}

\item{\code{chains}}{Numeric vector specifying the chains to extract. If NULL, draws from all chains are returned.}

\item{\code{best_chains}}{Integer; number of best chains to retain based on ELBO. If supplied, chains with the highest ELBO are retained.}

\item{\code{inc_random}}{Logical; whether to include random effects in the output. Default is FALSE.}

\item{\code{inc_transform}}{Logical; whether to include transformed parameters in the output. Default is TRUE.}

\item{\code{inc_generate}}{Logical; whether to include generated quantities in the output. Default is TRUE.}

\item{\code{best_only}}{Logical; whether to extract only from the chain with the maximum ELBO. Default is FALSE unless explicitly requested.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A 3D array of posterior draws `[iterations, chains, parameters]`.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-summary"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-summary}{}}}
\subsection{Method \code{summary()}}{
Summarize posterior draws.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$summary(
  pars = NULL,
  max_rows = 10,
  digits = 2,
  inc_random = FALSE,
  inc_transform = TRUE,
  inc_generate = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character or numeric vector specifying the names or indices of parameters to summarize. If NULL, all available parameters are summarized.}

\item{\code{max_rows}}{Integer; maximum number of rows to print in the summary table. Default is 10.}

\item{\code{digits}}{Integer; number of decimal places to print. Default is 2.}

\item{\code{inc_random}}{Logical; whether to include random effects in the summary. Default is FALSE.}

\item{\code{inc_transform}}{Logical; whether to include transformed parameters in the summary. Default is TRUE.}

\item{\code{inc_generate}}{Logical; whether to include generated quantities in the summary. Default is TRUE.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A data frame containing the summarized posterior statistics.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-plot_elbo"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-plot_elbo}{}}}
\subsection{Method \code{plot_elbo()}}{
Plot the ELBO history to diagnose convergence.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$plot_elbo(tail_n = 1000, ests = NULL, type = "l", ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{tail_n}}{Integer; the number of recent iterations to plot. If NULL, plots the entire history. Default is 2000.}

\item{\code{ests}}{Character string `"best"`, numeric vector of estimate indices (e.g., `c(1, 3)`), or `NULL` to plot all. Default is `NULL`.}

\item{\code{type}}{Character string; the type of plot. Default is "l" (lines).}

\item{\code{...}}{Additional arguments passed to the `plot` function.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-plot_trajectory"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-plot_trajectory}{}}}
\subsection{Method \code{plot_trajectory()}}{
Plot the parameter trajectory from the final optimization window.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$plot_trajectory(pars = NULL, type = "l", ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pars}}{Character vector specifying the names of parameters to plot. If NULL, plots all available parameters.}

\item{\code{type}}{Character string; the type of plot. Default is "l" (lines).}

\item{\code{...}}{Additional arguments passed to the `matplot` function.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-transformed_draws"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-transformed_draws}{}}}
\subsection{Method \code{transformed_draws()}}{
Compute transformed parameters from posterior draws.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$transformed_draws(tran_fn = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{tran_fn}}{An optional user-supplied function that takes data and parameter lists to return transformed quantities.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The `VB_Fit` object itself, invisibly.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-generated_quantities"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-generated_quantities}{}}}
\subsection{Method \code{generated_quantities()}}{
Compute generated quantities from posterior draws.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$generated_quantities(code)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{code}}{An `rtmb_code(\{ ... \})` or `\{ ... \}` block containing the logic
to be calculated using posterior samples.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The `VB_Fit` object itself (invisibly).
Results are appended to the `generate_fit` field.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-WAIC"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-WAIC}{}}}
\subsection{Method \code{WAIC()}}{
Compute WAIC from pointwise generated log likelihood.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$WAIC(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to `draws()`, such as `chains` or `best_chains`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A `waic_BayesRTMB` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-diagnose"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-diagnose}{}}}
\subsection{Method \code{diagnose()}}{
Run basic diagnostics for the variational fit.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$diagnose(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to `diagnose_vb_fit()`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A `diagnose_BayesRTMB` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-advi_fit-clone"></a>}}
\if{latex}{\out{\hypertarget{method-advi_fit-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{VB_Fit$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_loglinear.R
\name{rtmb_loglinear}
\alias{rtmb_loglinear}
\title{RTMB-based Log-linear analysis (Poisson regression)}
\usage{
rtmb_loglinear(
  formula,
  data,
  prior = prior_flat(),
  fixed = NULL,
  WAIC = FALSE,
  ...
)
}
\arguments{
\item{formula}{A formula (e.g., `~ A + B + A:B`) or a contingency table.}

\item{data}{A data frame (required if `formula` is used).}

\item{prior}{An object of class "rtmb_prior" specifying the prior distribution.}

\item{fixed}{Optional named list of fixed values for specific parameters.}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}

\item{...}{Additional arguments passed to `rtmb_glm()`.}
}
\value{
An `RTMB_Model` object.
}
\description{
Performs Bayesian or Frequentist log-linear analysis (Poisson regression)
on a contingency table or raw data.
}
\examples{

  # Create a contingency table
  tab <- matrix(c(10, 20, 30, 40), nrow = 2)
  dimnames(tab) <- list(A = c("A1", "A2"), B = c("B1", "B2"))
  
  # Fit a log-linear model (independence model: ~ A + B)
  fit_log <- rtmb_loglinear(~ A + B, data = tab)
  
  # MAP estimation
  map_log <- fit_log$optimize()
  map_log$summary()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_glmer.R
\name{make_glmer_Z_matrix}
\alias{make_glmer_Z_matrix}
\title{Reconstruct an Observation-Level Random-Effect Design Matrix}
\usage{
make_glmer_Z_matrix(Zt, group_idx, N = length(group_idx))
}
\arguments{
\item{Zt}{Transposed block random-effect design matrix.}

\item{group_idx}{Integer group index for each observation.}

\item{N}{Number of observations. Defaults to `length(group_idx)`.}
}
\value{
A matrix with `N` rows and one column per random-effect coefficient.
}
\description{
Converts the transposed block random-effect design matrix used internally by
`rtmb_glmer()` into an observation-level matrix. This is kept as a small
utility for users who want to work from the block matrix returned by
`make_glmer_re_terms()`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classic.R
\name{Classic_Fit}
\alias{Classic_Fit}
\title{Classic fit object}
\description{
An R6 class representing the results of a classical (frequentist) estimation.
}
\section{Super class}{
\code{\link[BayesRTMB:RTMB_Fit_Base]{BayesRTMB::RTMB_Fit_Base}} -> \code{Classic_Fit}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{model}}{The `RTMB_Model` object used for estimation.}

\item{\code{fit}}{The result of the estimation (dataframe or lm object).}

\item{\code{par}}{a named list of parameter estimates.}

\item{\code{vcov}}{Variance-covariance matrix of fixed effects.}

\item{\code{se_method}}{Character string specifying the method used for standard errors.}

\item{\code{cluster}}{Character string specifying the cluster variable name, if any.}

\item{\code{robust_type}}{Character string specifying the robust standard error correction type, if used.}

\item{\code{bootstrap_results}}{A matrix containing bootstrap samples, if applicable.}

\item{\code{test_results}}{List of additional test results (e.g., chisq.test).}

\item{\code{view}}{Character vector of parameter names to prioritize in summary.}

\item{\code{par_vec}}{Numeric vector of parameter estimates.}

\item{\code{objective}}{Final objective value.}

\item{\code{log_lik}}{Log-likelihood.}

\item{\code{restricted_log_lik}}{Restricted log-likelihood, if classical estimation used REML-style marginalization.}

\item{\code{rss}}{Residual sum of squares for classical linear models, if available.}

\item{\code{df_residual}}{Residual degrees of freedom for classical linear models, if available.}

\item{\code{convergence}}{Convergence code.}

\item{\code{sd_rep}}{TMB sdreport object.}

\item{\code{df_fixed}}{Dataframe of fixed effects results.}

\item{\code{random_effects}}{Dataframe of random effects results.}

\item{\code{df_transform}}{Dataframe of transformed parameters.}

\item{\code{df_generate}}{Dataframe of generated quantities.}

\item{\code{opt_history}}{Dataframe of optimization history.}

\item{\code{transform}}{List of transformed parameters.}

\item{\code{generate}}{List of generated quantities.}

\item{\code{se_samples}}{List of simulated samples for SE estimation.}

\item{\code{par_unc}}{Numeric vector of unconstrained parameter estimates.}

\item{\code{vcov_unc}}{Variance-covariance matrix of parameters in unconstrained space.}

\item{\code{ci_method}}{Method used for CI estimation.}

\item{\code{laplace}}{Whether Laplace approximation was used.}

\item{\code{map}}{Parameter mapping used.}

\item{\code{df_method}}{Character string specifying the degrees of freedom calculation method.}

\item{\code{idx_fix_active}}{Numeric vector; mapping between active parameters and full unconstrained vector.}

\item{\code{show_df}}{Logical; whether to display degrees of freedom in the summary output.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Classic_Fit-get_point_estimate}{\code{Classic_Fit$get_point_estimate()}}
\item \href{#method-Classic_Fit-new}{\code{Classic_Fit$new()}}
\item \href{#method-Classic_Fit-robust_se}{\code{Classic_Fit$robust_se()}}
\item \href{#method-Classic_Fit-compute_robust}{\code{Classic_Fit$compute_robust()}}
\item \href{#method-Classic_Fit-bootstrap}{\code{Classic_Fit$bootstrap()}}
\item \href{#method-Classic_Fit-compute_bootstrap}{\code{Classic_Fit$compute_bootstrap()}}
\item \href{#method-Classic_Fit-AIC}{\code{Classic_Fit$AIC()}}
\item \href{#method-Classic_Fit-BIC}{\code{Classic_Fit$BIC()}}
\item \href{#method-Classic_Fit-WAIC}{\code{Classic_Fit$WAIC()}}
\item \href{#method-Classic_Fit-diagnose}{\code{Classic_Fit$diagnose()}}
\item \href{#method-Classic_Fit-print}{\code{Classic_Fit$print()}}
\item \href{#method-Classic_Fit-logLik}{\code{Classic_Fit$logLik()}}
\item \href{#method-Classic_Fit-EAP}{\code{Classic_Fit$EAP()}}
\item \href{#method-Classic_Fit-MAP}{\code{Classic_Fit$MAP()}}
\item \href{#method-Classic_Fit-summary}{\code{Classic_Fit$summary()}}
\item \href{#method-Classic_Fit-anova}{\code{Classic_Fit$anova()}}
\item \href{#method-Classic_Fit-lsmeans}{\code{Classic_Fit$lsmeans()}}
\item \href{#method-Classic_Fit-.calc_contrast}{\code{Classic_Fit$.calc_contrast()}}
\item \href{#method-Classic_Fit-.get_lsmeans_df}{\code{Classic_Fit$.get_lsmeans_df()}}
\item \href{#method-Classic_Fit-.construct_par_list}{\code{Classic_Fit$.construct_par_list()}}
\item \href{#method-Classic_Fit-clone}{\code{Classic_Fit$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="estimate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-estimate'><code>BayesRTMB::RTMB_Fit_Base$estimate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="fa_rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-fa_rotate'><code>BayesRTMB::RTMB_Fit_Base$fa_rotate()</code></a></span></li>
<li><span class="pkg-link" data-pkg="BayesRTMB" data-topic="RTMB_Fit_Base" data-id="rotate"><a href='../../BayesRTMB/html/RTMB_Fit_Base.html#method-RTMB_Fit_Base-rotate'><code>BayesRTMB::RTMB_Fit_Base$rotate()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-get_point_estimate"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-get_point_estimate}{}}}
\subsection{Method \code{get_point_estimate()}}{
Get point estimate for a target parameter.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$get_point_estimate(target, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{target}}{Target parameter name.}

\item{\code{...}}{Additional arguments, ignored for classic fits.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Matrix, array, vector, or scalar point estimate.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-new"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-new}{}}}
\subsection{Method \code{new()}}{
Create a new `Classic_Fit` object.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$new(
  model,
  par_vec = NULL,
  par = NULL,
  objective = NULL,
  log_lik = NULL,
  restricted_log_lik = NULL,
  convergence = NULL,
  sd_rep = NULL,
  df_fixed = NULL,
  random_effects = NULL,
  df_transform = NULL,
  df_generate = NULL,
  opt_history = NULL,
  transform = NULL,
  generate = NULL,
  se_samples = NULL,
  par_unc = NULL,
  vcov_unc = NULL,
  ci_method = "wald",
  laplace = TRUE,
  map = NULL,
  test_results = list(),
  view = NULL,
  fit = NULL,
  vcov = NULL,
  df_method = "auto",
  idx_fix_active = NULL,
  show_df = TRUE,
  rss = NULL,
  df_residual = NULL,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{model}}{The `RTMB_Model` object.}

\item{\code{par_vec}}{Numeric vector of parameter estimates.}

\item{\code{par}}{List of parameter estimates.}

\item{\code{objective}}{Final objective value.}

\item{\code{log_lik}}{Full log-likelihood used for information criteria.}

\item{\code{restricted_log_lik}}{Restricted log-likelihood, if classical estimation used REML-style marginalization.}

\item{\code{convergence}}{Convergence code.}

\item{\code{sd_rep}}{TMB sdreport object.}

\item{\code{df_fixed}}{Dataframe of fixed effects results.}

\item{\code{random_effects}}{Dataframe of random effects results.}

\item{\code{df_transform}}{Dataframe of transformed parameters.}

\item{\code{df_generate}}{Dataframe of generated quantities.}

\item{\code{opt_history}}{Dataframe of optimization history.}

\item{\code{transform}}{List of transformed parameters.}

\item{\code{generate}}{List of generated quantities.}

\item{\code{se_samples}}{List of simulated samples for SE estimation.}

\item{\code{par_unc}}{Parameter vector on unconstrained scale.}

\item{\code{vcov_unc}}{Covariance matrix on unconstrained scale.}

\item{\code{ci_method}}{Method used for CI estimation.}

\item{\code{laplace}}{Whether Laplace approximation was used.}

\item{\code{map}}{Parameter mapping used.}

\item{\code{test_results}}{List of additional test results (e.g., chisq.test).}

\item{\code{view}}{Character vector of parameter names to prioritize in summary.}

\item{\code{fit}}{Legacy argument for backward compatibility (maps to df_fixed).}

\item{\code{vcov}}{Variance-covariance matrix of parameters.}

\item{\code{df_method}}{Method for degrees of freedom calculation.}

\item{\code{idx_fix_active}}{Numeric vector; mapping between active parameters and full unconstrained vector.}

\item{\code{show_df}}{Logical; whether to display degrees of freedom in the summary output.}

\item{\code{rss}}{Residual sum of squares for classical linear models, if available.}

\item{\code{df_residual}}{Residual degrees of freedom for classical linear models, if available.}

\item{\code{...}}{Additional arguments passed to the constructor.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-robust_se"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-robust_se}{}}}
\subsection{Method \code{robust_se()}}{
Compute robust standard errors (sandwich estimator).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$robust_se(
  cluster = NULL,
  type = c("HC3", "HC0", "HC1", "CR1", "CR0"),
  inplace = FALSE,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{cluster}}{Character; variable name for clustering.}

\item{\code{type}}{Character; "HC3" (default), "HC0", or "HC1" for non-clustered
robust SE; "CR1" or "CR0" for clustered robust SE.}

\item{\code{inplace}}{Logical; if FALSE, return a robust-SE copy without modifying the original fit.
If TRUE, update the object in place.}

\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Self.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-compute_robust"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-compute_robust}{}}}
\subsection{Method \code{compute_robust()}}{
(Deprecated) Use robust_se() instead.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$compute_robust(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Arguments passed to robust_se(), including inplace.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Self.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-bootstrap"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-bootstrap}{}}}
\subsection{Method \code{bootstrap()}}{
Compute nonparametric bootstrap standard errors and confidence intervals.
Currently implemented only for mediation models.
Bootstrap refits mediation equations with base R lm.fit()/glm.fit() when possible,
and falls back to RTMB refits for unsupported families.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$bootstrap(n_boot = 1000, seed = NULL, inplace = FALSE, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n_boot}}{Integer; number of bootstrap samples.}

\item{\code{seed}}{Optional integer random seed.}

\item{\code{inplace}}{Logical; if FALSE, return a bootstrap-SE copy without modifying the original fit.
If TRUE, update the object in place.}

\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A Classic_Fit object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-compute_bootstrap"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-compute_bootstrap}{}}}
\subsection{Method \code{compute_bootstrap()}}{
Compute nonparametric bootstrap standard errors and confidence intervals.
Currently implemented only for mediation models.
Bootstrap refits mediation equations with base R lm.fit()/glm.fit() when possible,
and falls back to RTMB refits for unsupported families.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$compute_bootstrap(n_boot = 1000, seed = NULL, inplace = FALSE, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n_boot}}{Integer; number of bootstrap samples.}

\item{\code{seed}}{Optional integer random seed.}

\item{\code{inplace}}{Logical; if FALSE, return a bootstrap-SE copy without modifying the original fit.
If TRUE, update the object in place.}

\item{\code{...}}{Additional arguments.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A Classic_Fit object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-AIC"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-AIC}{}}}
\subsection{Method \code{AIC()}}{
Get the AIC of the fitted model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$AIC()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-BIC"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-BIC}{}}}
\subsection{Method \code{BIC()}}{
Get the BIC of the fitted model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$BIC()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-WAIC"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-WAIC}{}}}
\subsection{Method \code{WAIC()}}{
WAIC is not defined for classical fits.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$WAIC()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-diagnose"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-diagnose}{}}}
\subsection{Method \code{diagnose()}}{
Run basic diagnostics for the classical fit.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$diagnose(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to `diagnose_classic_fit()`.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A `diagnose_BayesRTMB` object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-print"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-print}{}}}
\subsection{Method \code{print()}}{
Print the fit results.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Additional arguments passed to `summary()`.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-logLik"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-logLik}{}}}
\subsection{Method \code{logLik()}}{
Get the Log-Likelihood of the fitted model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$logLik()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-EAP"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-EAP}{}}}
\subsection{Method \code{EAP()}}{
EAP estimates are not available for Classic_Fit.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$EAP(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Ignored.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-MAP"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-MAP}{}}}
\subsection{Method \code{MAP()}}{
MAP estimates are not available for Classic_Fit.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$MAP(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{Ignored.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-summary"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-summary}{}}}
\subsection{Method \code{summary()}}{
Display a summary of the estimation results.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$summary(view = NULL, digits = 5, max_rows = 10, ranef = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{view}}{Character vector of parameter names to prioritize or filter by.}

\item{\code{digits}}{Number of digits to print for estimates.}

\item{\code{max_rows}}{Maximum number of rows to display in the coefficient table.}

\item{\code{ranef}}{Logical; whether to show random effects in the summary.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-anova"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-anova}{}}}
\subsection{Method \code{anova()}}{
Perform ANOVA (Wald F-tests / Chisq-tests) on the fitted model.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$anova(method = c("reml", "ls"), type = 3)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{method}}{Character; "reml" (standard) or "ls" (experimental).}

\item{\code{type}}{Integer; Type of Sum of Squares (only Type III supported currently).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A data frame containing the ANOVA table.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-lsmeans"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-lsmeans}{}}}
\subsection{Method \code{lsmeans()}}{
Calculate Least Squares Means (Marginal Means) and contrasts.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$lsmeans(
  specs = NULL,
  pairwise = FALSE,
  simple = NULL,
  adjust = "holm",
  protect = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{specs}}{Character vector of factors to calculate means for.}

\item{\code{pairwise}}{Logical; whether to perform pairwise comparisons.}

\item{\code{simple}}{Character vector of factors to hold constant for simple main effects.}

\item{\code{adjust}}{Character; p-value adjustment method (e.g., "bonferroni", "holm", "none").}

\item{\code{protect}}{Logical; whether to use hierarchical (protected) testing.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A data frame containing the marginal means or contrasts.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-.calc_contrast"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-.calc_contrast}{}}}
\subsection{Method \code{.calc_contrast()}}{
(Internal) Calculate metrics for a contrast.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$.calc_contrast(L, specs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{L}}{Contrast matrix.}

\item{\code{specs}}{Variable names for lsmeans.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A data frame with estimate, SE, df, t-value, and p-value.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-.get_lsmeans_df"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-.get_lsmeans_df}{}}}
\subsection{Method \code{.get_lsmeans_df()}}{
(Internal) Get representative DF for lsmeans.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$.get_lsmeans_df(specs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{specs}}{Variable names for lsmeans.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Degrees of freedom.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-.construct_par_list"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-.construct_par_list}{}}}
\subsection{Method \code{.construct_par_list()}}{
(Internal) Construct a list of parameters from the fit.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$.construct_par_list(fit)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{fit}}{The fit result (dataframe or lm object).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A named list of parameters.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Classic_Fit-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Classic_Fit-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Classic_Fit$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{sum_to_zero}
\alias{sum_to_zero}
\title{Sum-to-zero transformation}
\usage{
sum_to_zero(x)
}
\arguments{
\item{x}{A numeric vector of length K-1.}
}
\value{
A numeric vector of length K whose elements sum to zero.
}
\description{
Transforms a vector of length K-1 to a vector of length K that sums to zero
using an orthogonal basis matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{squared_distance}
\alias{squared_distance}
\title{Squared Euclidean distance}
\usage{
squared_distance(x, y, eps = 1e-08)
}
\arguments{
\item{x}{A numeric vector.}

\item{y}{A numeric vector.}

\item{eps}{A small value added for numerical stability (default is 1e-8).}
}
\value{
The squared Euclidean distance between x and y.
}
\description{
Squared Euclidean distance
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameters.R
\name{generate_random_init}
\alias{generate_random_init}
\title{Generate Random Initial Values}
\usage{
generate_random_init(pl_full, par_list, range = 2)
}
\arguments{
\item{pl_full}{A list containing the full parameter structure, including the `init` field which serves as a template.}

\item{par_list}{A list defining the structure and constraints of the parameters, including `unc_length`.}

\item{range}{Numeric; the range for the uniform distribution `[-range, range]` used for generating unconstrained values. Default is 2.}
}
\value{
A numeric vector of initial values where `NA` elements are replaced with randomly generated constrained values.
}
\description{
Generates random initial values for model parameters by drawing from a uniform distribution
on the unconstrained scale and then transforming them to the constrained scale.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_mdu.R
\name{make_init_mdu}
\alias{make_init_mdu}
\title{Create Initial Values for Multidimensional Unfolding}
\usage{
make_init_mdu(
  Y,
  D,
  distance = c("squared", "euclidean"),
  alpha = c("random", "fix"),
  distance_eps = 1e-04,
  min_sigma = 0.1
)
}
\arguments{
\item{Y}{Numeric matrix or data frame (N rows x M items).}

\item{D}{Number of unfolding dimensions.}

\item{distance}{Character; `"squared"` or `"euclidean"`.}

\item{alpha}{Character; `"random"` for item-specific alpha initial values or
`"fix"` for a common alpha initial value.}

\item{distance_eps}{Small positive constant added to the distance.}

\item{min_sigma}{Minimum initial residual standard deviation.}
}
\value{
A named list of initial values.
}
\description{
Create Initial Values for Multidimensional Unfolding
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers_prior.R
\name{prior_ssp}
\alias{prior_ssp}
\title{Specify a Spike-and-Slab prior for variable selection}
\usage{
prior_ssp(ssp_ratio = 0.25, max_beta = 1, ...)
}
\arguments{
\item{ssp_ratio}{Prior probability of inclusion for each variable. Default is 0.25.}

\item{max_beta}{Maximum expected effect size. Default is 1.0.}

\item{...}{Optional hyperparameters for other distributions.}
}
\value{
A list with class "rtmb_prior"
}
\description{
Specify a Spike-and-Slab prior for variable selection
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{beverage}
\alias{beverage}
\title{Beverage Preference Data}
\format{
A data frame (or matrix) with [INSERT NUMBER OF ROWS] rows and [INSERT NUMBER OF COLUMNS] columns.
\describe{
  \item{AJ}{Apple Juice preference rating}
  \item{BT}{Black Tea preference rating}
  \item{CF}{Coffee preference rating}
  \item{CL}{Cola preference rating}
  \item{GT}{Green Tea preference rating}
  \item{OJ}{Orange Juice preference rating}
  \item{OT}{Oolong Tea preference rating}
}
}
\source{
Adachi, K. (2000). A Random Effect Model in Metric Multidimensional Unfolding.
\emph{The Japanese Journal of Behaviormetrics}, 27(1), 12-23.
\doi{10.2333/jbhmk.27.12}
}
\usage{
beverage
}
\description{
A dataset containing preference ratings for various beverages.
This data was used to demonstrate a random effect model in metric
multidimensional unfolding.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{log_softmax}
\alias{log_softmax}
\title{Log-softmax function}
\usage{
log_softmax(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
A numeric vector of log-softmax probabilities.
}
\description{
Log-softmax function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{beta_lpdf}
\alias{beta_lpdf}
\title{Beta log-probability density function}
\usage{
beta_lpdf(x, a, b, sum = TRUE)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{a}{Shape parameter alpha.}

\item{b}{Shape parameter beta.}
}
\value{
The sum of the log-density.
}
\description{
Beta log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_mdu}
\alias{plot_mdu}
\title{Plot Multidimensional Unfolding Configuration}
\usage{
plot_mdu(
  delta,
  theta = NULL,
  item_alpha = NULL,
  phi = NULL,
  dims = c(1, 2),
  radius = NULL,
  signs = c(1, 1),
  item_labels = NULL,
  show_phi = TRUE,
  show_density = TRUE,
  circle_scale = 1,
  alpha = 0.2,
  contour_n = 60,
  distance = c("auto", "squared", "euclidean"),
  point_estimate = c("EAP", "MAP", "mean", "marginal_map", "joint_map"),
  main = "MDU Configuration",
  xlab = NULL,
  ylab = NULL,
  ...
)
}
\arguments{
\item{delta}{Item coordinate matrix (M items x D dimensions), or a fitted
object with an `EAP()` method.}

\item{theta}{Person coordinate matrix (N persons x D dimensions). If `delta`
is a fitted object and `theta` is `NULL`, it is extracted from the fit.}

\item{item_alpha}{Optional item alpha vector used for item radii. If `delta`
is a fitted object and `item_alpha` is `NULL`, `alpha` is extracted when
available.}

\item{phi}{Deprecated alias for `item_alpha`.}

\item{dims}{Integer vector of length 2 specifying dimensions to plot.}

\item{radius}{Numeric plot radius. If `NULL`, a radius is chosen from the
coordinates.}

\item{signs}{Numeric vector of length 2. Use `-1` to flip an axis.}

\item{item_labels}{Optional item labels. Defaults to row names or item
numbers.}

\item{show_phi}{Logical; whether to draw circles based on item alpha.}

\item{show_density}{Logical; whether to draw density contours for `theta`.}

\item{circle_scale}{Numeric multiplier for item radii.}

\item{alpha}{Circle transparency.}

\item{contour_n}{Grid size passed to `MASS::kde2d()`.}

\item{distance}{Character; `"auto"`, `"squared"`, or `"euclidean"`. Used
to transform item alpha values into plotted radii. `"auto"` uses the fit's
stored distance when available.}

\item{point_estimate}{Character; point estimate used when `delta` is a fit
object. Passed to `estimate()`.}

\item{main, xlab, ylab}{Plot title and axis labels.}

\item{...}{Additional arguments passed to `plot()`.}
}
\value{
Invisibly returns a list with plotted `delta`, `theta`, and
  `item_alpha`.
}
\description{
Draws an item-person map for multidimensional unfolding models. Item
locations are shown as labels, optional blue circles show item alpha/radius
values, and the respondent locations are summarized by a two-
dimensional kernel density contour.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{wishart_CF_lpdf}
\alias{wishart_CF_lpdf}
\title{Wishart log-probability density function parameterized by Cholesky factor of correlation matrix}
\usage{
wishart_CF_lpdf(X, n, sd, CF_Omega)
}
\arguments{
\item{X}{Symmetric positive-definite matrix (scatter matrix).}

\item{n}{Degrees of freedom (sample size N).}

\item{sd}{Vector of standard deviations.}

\item{CF_Omega}{Cholesky factor of the correlation matrix.}
}
\value{
The log-density with all constant terms.
}
\description{
Wishart log-probability density function parameterized by Cholesky factor of correlation matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrapper_table.R
\name{rtmb_table}
\alias{rtmb_table}
\title{RTMB-based Contingency Table Analysis (Chi-squared Test)}
\usage{
rtmb_table(
  x,
  y = NULL,
  data = NULL,
  correct = TRUE,
  prior = prior_flat(),
  fixed = NULL,
  WAIC = FALSE,
  ...
)
}
\arguments{
\item{x}{Variable name, formula, table, or matrix.}

\item{y}{Variable name (optional if x is a formula).}

\item{data}{A data frame.}

\item{correct}{Logical; if TRUE, apply Yates' continuity correction for 2x2 classic analyses.}

\item{prior}{Prior specification (Bayesian mode). Default is `prior_flat()`.}

\item{fixed}{Optional named list of fixed values for specific parameters.}

\item{WAIC}{Logical; if TRUE, add pointwise `log_lik` to the generate block for WAIC.}

\item{...}{Additional arguments.}
}
\value{
An `RTMB_Model` object.
}
\description{
`rtmb_table` performs a chi-squared test of independence between two categorical variables.
It provides both classic (frequentist) Pearson chi-squared tests and Bayesian multinomial-style models.
}
\examples{
\donttest{
# Classic chi-squared test
rtmb_table(skill, cond, data = debate)$classic()
rtmb_table(table(debate$skill, debate$cond))$classic()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{dirichlet_lpdf}
\alias{dirichlet_lpdf}
\title{Dirichlet log-probability density function}
\usage{
dirichlet_lpdf(x, alpha)
}
\arguments{
\item{x}{Vector or matrix of simplexes.}

\item{alpha}{Vector of concentration parameters.}
}
\value{
The sum of the log-density.
}
\description{
Dirichlet log-probability density function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributions.R
\name{sequential_logistic_lpmf}
\alias{sequential_logistic_lpmf}
\title{Sequential logistic log-probability mass function}
\usage{
sequential_logistic_lpmf(x, eta, cutpoints, sum = TRUE)
}
\arguments{
\item{x}{Vector of ordered categorical outcomes.}

\item{eta}{Linear predictor.}

\item{cutpoints}{Vector of sequential cutpoints.}
}
\value{
The sum of the log-probability.
}
\description{
Sequential logistic log-probability mass function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/math_functions.R
\name{log1m}
\alias{log1m}
\title{Log of one minus x}
\usage{
log1m(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
The natural logarithm of 1 - x.
}
\description{
Log of one minus x
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{with_rtmb_error_handling}
\alias{with_rtmb_error_handling}
\title{Catch and Translate RTMB Errors into User-Friendly Messages}
\usage{
with_rtmb_error_handling(expr, block_name)
}
\arguments{
\item{expr}{An R expression to be evaluated, typically representing the logic within
a specific model block.}

\item{block_name}{A character string identifying the context of the error
(e.g., "parameters", "model", "transform", or "generate").}
}
\value{
The result of evaluating the \code{expr}. If an error occurs, it throws
a formatted [stop()] message.
}
\description{
This internal helper function wraps an R expression in a [tryCatch()] block to
capture cryptic error messages generated by the RTMB automatic differentiation
engine or C++ backend, translating them into descriptive hints.
}
\details{
It uses regular expressions to categorize common RTMB failure modes, such as:
\itemize{
  \item **Missing Objects/Functions**: Detects if a variable or a specific distribution
  function (e.g., `normal_lpdf`) is missing from the environment.
  \item **Index Errors**: Identifies subscript out of bounds errors.
  \item **AD Type Destruction**: Catches the "lost class attribute" error, often caused
  by incorrect initialization of vectors (e.g., using `numeric(N)` and partial assignment
  inside loops).
  \item **Matrix Dimension Mismatch**: Identifies non-conformable arguments in
  matrix multiplication.
}
}
\keyword{internal}
